#ifndef Blb_esc2tr.c
#ifdef Doc_esc2tr.c
/*
   A skeleton of calling ESC from the transport C-codes:
   */
void main_of_the_transport_code()
{
  ...;
  int	Npv=8;
  double Pressure[21],
  Jparallel[21],
  Rpv[12],
  Zpv[12];

  double RBtor,Rext,gFtor;
  double t;

  double Rmaxis,
  Zmaxis,
  gFtor,
  Fpol[21],
  gM[21],
  G22[21],
  G33[21],
  D2[21],
  D3[21],
  Sside[21],
  Sside1[21];
  ...;
  /* e.g., initiation of the transport code */
  ...;
   
  esc0_("NSTX");  /* ESC initiation routine. One time call. */

  ...;
  /* e.g., setting the initial data for the transport code;
  ...;
   

  /* ESC Input initiation routine. Specific to a transport code. */
  astra2e_(Pressure
	   ,Jparallel
	   ,Rpv
	   ,Zpv
	   ,&Npv
	   ,&RBtor
	   ,&Rext
	   ,&gFtor);  /* ASTRA/ESC input initiation routine. One time call. */

  ...;
  /* time advancing loop */
  for(t=t0; t < t1; t+=dt){
    ...;
    calculations_of_the_plasma_profiles_and_boundary_points();
    ...;
    {
      int Ffail,Fjob,Mpol;
      
      Fjob	=0;
      Mpol	=6;
      
      esc_(&Ffail,&Fjob,&Mpol,&t);  /* ESC call */
      
      if((Ffail&0x01)){
	/* Repeat with more iterations */
	while(Ffail == 1){
	  esc_(&Ffail,&Fjob,&Mpol,&t);
	}
      }
      if((Ffail&0x02)){
	t	-=dt;
	reducing_step_in_the_plasma_boundary_advancing();
      }
      if((Ffail&0x04)){
	t	-=dt;
	reducing_step_in_the_plasma_pressure_profile();
      }
      if((Ffail&0x08)){
	t	-=dt;
	reducing_step_in_the_plasma_current_profile();
      }
    }      
    ...;
    
    /* ESC Output initiation routine. Specific to a transport code. */
    e2astra_(&Rmaxis,
	     &Zmaxis,
	     &gFtor,
	     Fpol,
	     gM,
	     G22,
	     G33,
	     D2,
	     D3,
	     Sside,
	     Sside1);  /* ESC/ASTRA output routine. */ 
    ...;
  }
  ...;
  esc1_();  /* ESC deinitiation routine. One time call. */
  ...;
}

/*
   A skeleton of calling ESC from the transport FORTRAN-codes:
   */

        program main
	implicit none
	...
	character*12 Machine
	integer	Npv
	double precision Pressure(21),Jparallel(21),Rpv(12),Zpv(12)

	double precision RBtor,Rext,gFtor
	double precision t;

	double precision Rmaxis,Zmaxis,gFtor,Fpol(21),gM(21),G22(21)
	1    ,G33(21),D2(21),D3(21),Sside(21),Sside1(21);
	
	integer Ffail,Fjob,Mpol
	...

c       e.g., initiation of the transport code
	...
c#1
	Machine	='NSTX'
	call esc0(Machine)	! ESC initiation routine. One time call
c#
	...
c	e.g., setting the initial data for the transport code
	...
c#2
c       ESC Input initiation routine. Specific to a transport code.
        NPv=8
	call astra2e(Pressure,Jparallel,Rpv,Zpv,Npv,RBtor,Rext,gFtor)
c	ASTRA/ESC input initiation routine. One time call.
c#
	...
c       time advancing loop
	t	=t0
	do while(t < t1)
	   ...
	   ... calculations_of_the_plasma_profiles_and_boundary_points
	   ...
	   
	   Fjob	=0
	   Mpol	=6
c#3
	   call esc(Ffail,Fjob,Mpol,t) ! ESC call
c#
	   if(Ffail/2.ne.0)then
	      do while(Ffail.eq.1) ! Repeat with more iterations
		 call esc(Ffail,Fjob,Mpol,t)
	      enddo
	   endif
	   Ffail	=Ffail/2;
	   if(Ffail/2.ne.0)then
	      t=t-dt
	      ... reducing_step_in_the_plasma_boundary_advancing
	   endif
	   Ffail	=Ffail/2;
	   if(Ffail/2.ne.0)then
	      t	=t-dt
	      ... reducing_step_in_the_plasma_pressure_profile
	   endif
	   Ffail	=Ffail/2;
	   if(Ffail/2.ne.0)then
	      t	=t-dt
	      ... reducing_step_in_the_plasma_current_profile
	   endif
	   ...
c#4
c       ESC Output initiation routine. Specific to a transport code.
	   call  e2astra(Rmaxis,Zmaxis,gFtor,Fpol,gM,G22,G33,D2,D3
	&	,Sside,Sside1)	!      ESC/ASTRA output routine.
	   ...
c#
	enddo
	...
c#5
	call esc1()		! ESC deinitiation routine. One time call.
c#
	...
	end
#endif
#ifndef mzl_System_esc2tr.c
#ifdef Tbl_UnusedID
  extern double cr3;		/*                         :esc2tr.c */
  extern double cr4;		/*                         :esc2tr.c */
  extern double cr6;		/*                         :esc2tr.c */
  extern double cgp;		/*                         :esc2tr.c */
  extern double cgp_4;		/*                         :esc2tr.c */
  extern double cr2gp;		/*                         :esc2tr.c */
  extern double cgm0;		/*                         :esc2tr.c */
  extern double *ESgF1a;	/*                         :esc2tr.c */
  extern double *ESgF2a;	/*                         :esc2tr.c */
  extern double *ESgm1a;	/*                         :esc2tr.c */
  extern double *ESgm2a;	/*                         :esc2tr.c */
  extern double *ESFF;		/*                         :esc2tr.c */
  extern double *ESFF1a;	/*                         :esc2tr.c */
  extern double *ESFF2a;	/*                         :esc2tr.c */
  extern double *ESg22c1;	/*                         :esc2tr.c */
  extern double *ESg22c2;	/*                         :esc2tr.c */
  extern double *ESDPc1;	/*                         :esc2tr.c */
  extern double *ESDPc2;	/*                         :esc2tr.c */
  extern double *ESLc1;		/*                         :esc2tr.c */
  extern double *ESLc2;		/*                         :esc2tr.c */
  extern double *ESVc1;		/*                         :esc2tr.c */
  extern double *ESVc2;		/*                         :esc2tr.c */
  void esc0_();			/*         .blb_esc.mzl_ESC:esc2tr.c */
  char *MachineNm;		/* .esc0_().blb_esc.mzl_ESC:esc2tr.c */
  void esc_();			/*         .blb_esc.mzl_ESC:esc2tr.c */
  int *Ffail;			/*  .esc_().blb_esc.mzl_ESC:esc2tr.c */
  int *Fjob;			/*  .esc_().blb_esc.mzl_ESC:esc2tr.c */
  void esc1_();			/*         .blb_esc.mzl_ESC:esc2tr.c */
  void tsc2e();			/*       .blb_TSC.mzl_ESCTR:esc2tr.c */
  void astra2e_();		/*     .blb_ASTRA.mzl_ESCTR:esc2tr.c */
  void e2astra_();		/*     .blb_ASTRA.mzl_ESCTR:esc2tr.c */
#endif
  Systm sqrt();
  Undef InitConstants();	/* NO prototype */
  Undef ESInitPlGeomInt();	/* NO prototype */
  Undef ESReInit1DPlV();	/* NO prototype */
  Undef ESReInit2DTPlV();	/* NO prototype */
  Undef ESReInitPlProf();	/* NO prototype */
  Undef EcInitArrays();		/* NO prototype */
  Undef ESReInitMetrTnsrCS();	/* NO prototype */
  Undef ECReInitFgY();		/* NO prototype */
  Undef ESGetPlVDAddr();	/* NO prototype */
  Undef ESInit1DPlV();		/* NO prototype */
  Undef ESInit2DTPlV();		/* NO prototype */
  Undef ESReal2RefData();	/* NO prototype */
  Undef ESReal2RefGeom();	/* NO prototype */
  Undef ESPlVdata2PlV();	/* NO prototype */
  Undef ESRef2RealGeom();	/* NO prototype */
  Undef ESRealGeom();		/* NO prototype */
  Undef ECRcs3D2Rcs2D();	/* NO prototype */
  Undef ES3DMetrTnsrCS();	/* NO prototype */
  Undef ESInpPlPrCr();		/* NO prototype */
  Undef EC1DEqSolv();		/* NO prototype */
  Undef ESPlProf2Data();	/* NO prototype */
  Undef EC2DEqSolv();		/* NO prototype */
  Undef ECgYcs2dRZcsBound();	/* NO prototype */
  Undef ECVirtualPlV2vdrPlV();	/* NO prototype */
  Undef ECgYrc2dRZcs();		/* NO prototype */
  Undef ECvdrz2vrz();		/* NO prototype */
  Undef ECCheckJacob();		/* NO prototype */
  Undef ECRcs2D2Rcs3D();	/* NO prototype */
  Undef ECIfEqFound();		/* NO prototype */
  Undef ESDeInitEsc();		/* NO prototype */
  Undef ECDeInitFgY();		/* NO prototype */
  Undef EcDeInitArrays();	/* NO prototype */
  Undef DeInitSplines();	/* NO prototype */
  Undef TSCLim2DPlV();		/* NO prototype */
  Undef ESInit2DPlGeom();	/* NO prototype */
  Undef EcInitPlVacData();	/* NO prototype */
  Undef ECvr2r();		/* NO prototype */
  Undef ECvz2z();		/* NO prototype */
  Undef ESSetPlVInt();		/* NO prototype */
  Undef ESGetPlPrfAddr();	/* NO prototype */
  Undef ESSwitchInpRadC();	/* NO prototype */
  Undef ESFirstSetSplDPr();	/* NO prototype */
  Undef ESFirstSetSplDCr();	/* NO prototype */
  Undef splP();			/* NO prototype */
  extern int ESNa;		/* External */
  extern int ESNp;		/* External */
  extern int ESMp;		/* External */
  extern int ESFp;		/* External */
  extern int TFCNt;		/* External */
  extern int TFCNp;		/* External */
  extern int ESLt;		/* External */
  extern int ESNt;		/* External */
  extern int ESMp1;		/* External */
  extern int ES2Mp1;		/* External */
  extern double ESRBt;		/* External */
  extern int ESEqSolvRadC;	/* External */
  extern double cr12;		/* External */
  extern double *ESgt;		/* External */
  extern double ESRext;		/* External */
  extern double crgm0;		/* External */
  extern int ESEqSolvFl;	/* External */
  extern int ESEqSolvIt;	/* External */
  extern int ESEqSolvInPr;	/* External */
  extern int ESEqSolvInCr;	/* External */
  extern int ESnDtPr;		/* External */
  extern int ESnDtCr;		/* External */
  extern double ESxDtPr[];	/* External */
  extern double ESxDtCr[];	/* External */
  extern int ESNa1;		/* External */
  extern double *ESaF;		/* External */
  extern double *ESaF1a;	/* External */
  extern double *ESaF2a;	/* External */
  extern double *ESaR0;		/* External */
  extern double cr2;		/* External */
  extern double *ESaZ0;		/* External */
  extern double *ESgF;		/* External */
  extern double c2gp;		/* External */
  extern double *ESgm;		/* External */
  extern double *ESsa;		/* External */
  extern double *ESg22c;	/* External */
  extern double *ESLc;		/* External */
  extern double *ESDPc;		/* External */
  extern double *ESVc;		/* External */
  extern double *ESsb;		/* External */
  extern double *ESsb1a;	/* External */
  extern double *rcT1a;		/* External */
  extern double *rsT1a;		/* External */
  extern int ESFp1;		/* External */
  extern double *rcT;		/* External */
  extern double *rsT;		/* External */
  extern int ESNp1;		/* External */
  extern double *cs1;		/* External */
  extern double *sn1;		/* External */
  extern double *ESsr;		/* External */
#ifdef Tbl_ESC
#endif
#ifdef Tbl_ESCTR
#endif
#endif
#ifndef mzl_ESC
extern double cr2;	/* undefined */
extern double cr3;	/* undefined */
extern double cr4;	/* undefined */
extern double cr6;	/* undefined */
extern double cr12;	/* undefined */
extern double cgp;	/* undefined */
extern double c2gp;	/* undefined */
extern double cgp_4;	/* undefined */
extern double cr2gp;	/* undefined */
extern double cgm0;	/* undefined */
extern double crgm0;	/* undefined */
extern int ESNa;	/* undefined */
extern int ESNa1;	/* undefined */
extern int ESNp;	/* undefined */
extern int ESNp1;	/* undefined */
extern int ESFp;	/* undefined */
extern int ESFp1;	/* undefined */
extern int TFCNt;	/* undefined */
extern int TFCNp;	/* undefined */
extern int ESLt;	/* undefined */
extern int ESNt;	/* undefined */
extern int ESMp;	/* undefined */
extern int ESMp1;	/* undefined */
extern int ES2Mp1;	/* undefined */
extern int ESEqSolvFl;	/* undefined */
extern int ESEqSolvIt;	/* undefined */
extern int ESEqSolvRadC;/* undefined */
extern int ESEqSolvInPr;/* undefined */
extern int ESEqSolvInCr;/* undefined */
extern double ESxDtPr[];/* undefined */
extern double ESxDtCr[];/* undefined */
extern double *ESaR0;	/* undefined */
extern double *ESaZ0;	/* undefined */
extern double *ESsa;	/* undefined */
extern int ESnDtPr;	/* undefined */
extern int ESnDtCr;	/* undefined */
extern double ESRBt;	/* undefined */
extern double ESRext;	/* undefined */
extern double *ESaF;	/* undefined */
extern double *ESaF1a;	/* undefined */
extern double *ESaF2a;	/* undefined */
extern double *ESgF;	/* undefined */
extern double *ESgF1a;	/* undefined */
extern double *ESgF2a;	/* undefined */
extern double *ESgm;	/* undefined */
extern double *ESgm1a;	/* undefined */
extern double *ESgm2a;	/* undefined */
extern double *ESFF;	/* undefined */
extern double *ESFF1a;	/* undefined */
extern double *ESFF2a;	/* undefined */
extern double *ESgt;	/* undefined */
extern double *ESsr;	/* undefined */
extern double *ESsb;	/* undefined */
extern double *ESsb1a;	/* undefined */
extern double *rcT;	/* undefined */
extern double *rsT;	/* undefined */
extern double *rcT1a;	/* undefined */
extern double *rsT1a;	/* undefined */
extern double *cs1;	/* undefined */
extern double *sn1;	/* undefined */
extern double *ESg22c;	/* undefined */
extern double *ESg22c1;	/* undefined */
extern double *ESg22c2;	/* undefined */
extern double *ESDPc;	/* undefined */
extern double *ESDPc1;	/* undefined */
extern double *ESDPc2;	/* undefined */
extern double *ESLc;	/* undefined */
extern double *ESLc1;	/* undefined */
extern double *ESLc2;	/* undefined */
extern double *ESVc;	/* undefined */
extern double *ESVc1;	/* undefined */
extern double *ESVc2;	/* undefined */
#endif
#ifndef mzl_ESCTR
static double *rdPV;	/*  */
static double *zdPV;	/*  */
static double *spp;	/*  */
static double *spj;	/*  */
static double *srpv;	/*  */
static double *szpv;	/*  */
static int np;		/*  */
static int nj;		/*  */
static int npv;		/*  */
static int mpv;		/*  */
static double cssp;	/*  */
static double cssj;	/*  */
static double cssr;	/*  */
static double *ydPr;	/*  */
static double *ydCr;	/*  */
static double *xdPr;	/*  */
static double *xdCr;	/*  */
static double *rxBt;	/*  */
static double *gFt;	/*  */
static double stesc;	/*  */
#endif
#ifndef mzl_ESC
#ifndef blb_esc
void esc0_(MachineNm)
  char *MachineNm;	/* Name of the machine */
{
  Undef InitConstants();
  Undef ESInitPlGeomInt();
  Undef ESReInit1DPlV();
  Undef ESReInit2DTPlV();
  Undef ESReInitPlProf();
  Undef EcInitArrays();
  Undef ESReInitMetrTnsrCS();
  Undef ECReInitFgY();
  Undef ESGetPlVDAddr();
  Undef ESInit1DPlV();
  Undef ESInit2DTPlV();
  extern int ESNa;	/* undefined */
  extern int ESNp;	/* undefined */
  extern int ESMp;	/* undefined */
  extern int ESFp;	/* undefined */
  extern int TFCNt;	/* undefined */
  extern int TFCNp;	/* undefined */
  extern int ESLt;	/* undefined */
  extern int ESNt;	/* undefined */
  static double *rdPV;	/*  .mzl_ESCTR:esc2tr.c */
  static double *zdPV;	/*  .mzl_ESCTR:esc2tr.c */
/*
     ESC initiation routine
     */
#ifdef Doc_esc0_

#endif
}
void esc_(Ffail,Fjob,Mpol,sTime)
  int *Ffail;		/*Flag of failure:
			  0 - normal;
			  1 - lack of iterations (suspicious results);
			  2 - big jump in boundary shape;
			  4 - big jump in input profile associated
			  with pressure;
			  8 - big jump in input profile associated   
			  with curent density;
			  ....;
			  */
  int *Fjob;		/* Flag for job assignment:
			   0 - nothing special was requested;
			   ...;
			   */
  int *Mpol;		/* Number of Fourier harmonics in $\Psi$ */
  double *sTime;	/* time */
{
  Undef ESReal2RefData();
  Undef ESReal2RefGeom();
  Undef ESPlVdata2PlV();
  Undef ESRef2RealGeom();
  Undef ESRealGeom();
  Undef ECRcs3D2Rcs2D();
  Undef ES3DMetrTnsrCS();
  Undef ESInpPlPrCr();
  Undef EC1DEqSolv();
  Undef ESPlProf2Data();
  Undef EC2DEqSolv();
  Undef ECgYcs2dRZcsBound();
  Undef ECVirtualPlV2vdrPlV();
  Undef ECgYrc2dRZcs();
  Undef ECvdrz2vrz();
  Undef ECCheckJacob();
  Undef ECRcs2D2Rcs3D();
  Undef ECIfEqFound();
  extern int ESMp1;	/* undefined */
  extern int ES2Mp1;	/* undefined */
  extern double ESRBt;	/* undefined */
  int *Mpol;		/*  .esc_().blb_esc.mzl_ESC:esc2tr.c */
  double *sTime;	/*  .esc_().blb_esc.mzl_ESC:esc2tr.c */
  static double stesc;	/*               .mzl_ESCTR:esc2tr.c */
  static double *rxBt;	/*               .mzl_ESCTR:esc2tr.c */
  static int npv;	/*               .mzl_ESCTR:esc2tr.c */
  static double *rdPV;	/*               .mzl_ESCTR:esc2tr.c */
  static double *srpv;	/*               .mzl_ESCTR:esc2tr.c */
  static double cssr;	/*               .mzl_ESCTR:esc2tr.c */
  static double *zdPV;	/*               .mzl_ESCTR:esc2tr.c */
  static double *szpv;	/*               .mzl_ESCTR:esc2tr.c */
  static int np;	/*               .mzl_ESCTR:esc2tr.c */
  static double *ydPr;	/*               .mzl_ESCTR:esc2tr.c */
  static double *spp;	/*               .mzl_ESCTR:esc2tr.c */
  static double cssp;	/*               .mzl_ESCTR:esc2tr.c */
  static int nj;	/*               .mzl_ESCTR:esc2tr.c */
  static double *ydCr;	/*               .mzl_ESCTR:esc2tr.c */
  static double *spj;	/*               .mzl_ESCTR:esc2tr.c */
  static double cssj;	/*               .mzl_ESCTR:esc2tr.c */
/*
   */
#ifdef Doc_esc_

#endif
}
void esc1_()
{
  Undef ESDeInitEsc();
  Undef ECDeInitFgY();
  Undef EcDeInitArrays();
  Undef DeInitSplines();
/*
   */
#ifdef Doc_esc1_

#endif
}
#endif
#endif
#ifndef mzl_ESCTR
#ifndef blb_TSC
void tsc2e()
{
  Undef TSCLim2DPlV();
  Undef ESInit2DPlGeom();
  Undef EcInitPlVacData();
  Undef ECRcs3D2Rcs2D();
  Undef ECvr2r();
  Undef ECvz2z();
  extern int ESEqSolvRadC;	/* undefined */
/*
   */
#ifdef Doc_tsc2e

#endif
}
#endif
#ifndef blb_ASTRA
static double rc[];	/*  */
static double rca[];	/*  */
static double rs[];	/*  */
static double rsa[];	/*  */
static double as[];	/*  */
static double as2p[];	/*  */
static double as1[];	/*  */
static double as12p[];	/*  */
static double ch2hp;	/*  */
static double crHp;	/*  */
void astra2e_(Pressure,Jparallel,Rpv,Zpv,Npv,RBtor,Rext,gFtor)
  double *Pressure;	/* Plasma pressure [MPa] */
  double *Jparallel;	/* Parallel current density [MA/m^2] */
  double *Rpv;		/* R [m] of top, bottom, left, right and
			   other points at the plasma-vacuum inteface */
  double *Zpv;		/* Z [m] of top, bottom, left, right and 
			   other points at the plasma-vacuum inteface */
  int *Npv;		/* Number of plasma-vacuum points */
  double *RBtor;	/* product $RB_{tor}$ [m T] outside the plasma */
  double *Rext;		/* characteristic R [m] */
  double *gFtor;	/* total toroidal flux [Vsec] */
{
  Undef ESSetPlVInt();
  Undef ESGetPlPrfAddr();
  Undef ESInit2DPlGeom();
  Undef EcInitPlVacData();
  Undef ECRcs3D2Rcs2D();
  Undef ECvr2r();
  Undef ECvz2z();
  Undef ESSwitchInpRadC();
  Undef ESFirstSetSplDPr();
  Undef ESFirstSetSplDCr();
  extern double cr12;		/* undefined */
  extern double *ESgt;		/* undefined */
  extern double ESRext;		/* undefined */
  extern double crgm0;		/* undefined */
  extern int ESEqSolvFl;	/* undefined */
  extern int ESEqSolvIt;	/* undefined */
  extern int ESEqSolvInPr;	/* undefined */
  extern int ESEqSolvInCr;	/* undefined */
  extern int ESnDtPr;		/* undefined */
  extern int ESnDtCr;		/* undefined */
  extern double ESxDtPr[];	/* undefined */
  extern double ESxDtCr[];	/* undefined */
  extern int ESNa1;		/* undefined */
  extern double *ESaF;		/* undefined */
  extern double *ESaF1a;	/* undefined */
  extern double *ESaF2a;	/* undefined */
  extern double *ESaR0;		/* undefined */
  extern double cr2;		/* undefined */
  extern double *ESaZ0;		/* undefined */
  double *Pressure;	/*  .astra2e_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  double *Jparallel;	/*  .astra2e_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  double *Rpv;		/*  .astra2e_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  double *Zpv;		/*  .astra2e_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  int *Npv;		/*  .astra2e_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  double *RBtor;	/*  .astra2e_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  double *gFtor;	/*  .astra2e_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  double *Rext;		/*  .astra2e_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  static double ch2hp;	/*             .blb_ASTRA.mzl_ESCTR:esc2tr.c */
  static double crHp;	/*             .blb_ASTRA.mzl_ESCTR:esc2tr.c */
  static double *spp;	/*                       .mzl_ESCTR:esc2tr.c */
  static double *spj;	/*                       .mzl_ESCTR:esc2tr.c */
  static double *srpv;	/*                       .mzl_ESCTR:esc2tr.c */
  static double *szpv;	/*                       .mzl_ESCTR:esc2tr.c */
  static int npv;	/*                       .mzl_ESCTR:esc2tr.c */
  static int mpv;	/*                       .mzl_ESCTR:esc2tr.c */
  static double *rxBt;	/*                       .mzl_ESCTR:esc2tr.c */
  static double *gFt;	/*                       .mzl_ESCTR:esc2tr.c */
  static double cssp;	/*                       .mzl_ESCTR:esc2tr.c */
  static double cssj;	/*                       .mzl_ESCTR:esc2tr.c */
  static double cssr;	/*                       .mzl_ESCTR:esc2tr.c */
  static int np;	/*                       .mzl_ESCTR:esc2tr.c */
  static int nj;	/*                       .mzl_ESCTR:esc2tr.c */
  static double *ydPr;	/*                       .mzl_ESCTR:esc2tr.c */
  static double *ydCr;	/*                       .mzl_ESCTR:esc2tr.c */
  static double *xdPr;	/*                       .mzl_ESCTR:esc2tr.c */
  static double *xdCr;	/*                       .mzl_ESCTR:esc2tr.c */
/*
   */
#ifdef Doc_astra2e_
  \section{Skeleton of \Cline{astra2e()} }

  \begin{verbatim}
  void astra2e_(
    double *Pressure,
    double *Jparallel,
    double *Rpv,
    double *Zpv,
    int *Npv,
    double *RBtor,
    double *Rext,
    double *gFtor
    )
   {
   }
  \end{verbatim}

\section{Input from ASTRA}

\begin{tabular}{|lrp{1.7in}p{2.in}|}\hline
\multicolumn{4}{|c|}{\large\bf Subroutine \Cline{e2astra\_()}}\\\hline
\bf ID	&Units &Notation&\bf Meaning\\\hline
{\tt Pressure[21]}&$[MPa]$	&$p_i$
&total pressure at the equidistant radial grid points
$a_i=\q{\R{\gF_i}{\gF_{int}}}$ \\\hline 
{\tt Jparallel[21]}	&$[MA/m^2]$&$\j_{||,i}$
&parallel component of the plasma current density at the grid points\\\hline  
{\tt Rpv[]}	&$[m]$
&$r_{top},r_{bottom},r_{inner},r_{outer}$, $r_5,r_6,\ldots$
&$r_i$ of characteristic points at the plasma boundary\\\hline
{\tt Zpv[]}	&$[m]$
&$z_{top},z_{bottom},z_{inner},z_{outer}$, $z_5,z_6,\ldots$
&$z_i$ of characteristic points at the plasma boundary\\\hline
{\tt *Npv}	&$[]$	&$N_{plasma-vacuum}<13$
&Total number of characteristic points at the plasma boundary\\\hline
{\tt *RBtor}	&$[m\cdot T]$	&$rB_{tor}$
&product $rB_{tor}$ outside the plasma\\\hline
{\tt *Rext}	&$[m]$	&$R_ext$&Reference major radius (e.g., geometric
center of the chamber)\\\hline
{\tt *gFtor}	&$[Vsec]$&$\gF_{int}$&Total toroidal flux inside the
plasma (not used at the moment)\\\hline
\end{tabular}
\vskip12pt
Radial grid is assumed to be equidistant with respect to normalized
square root of toroidal flux
\bqbl
&&
a
\equiv
\q{\R{\gF}{\gF_{int}}}
.\eqbl
The parallel current density is defined as
\bqbl
&&
-{\pa\gY\over\pa t}
+{\gY'\over\gF'}{\pa\gF\over\pa t}
=
{2\gp R_0\over\gs_{||}}j_{||}
,\quad
j_{||}
=
\R{<\wsj\cdot\waB>}{<B_{tor}\cdot\na\gf>R_0}
=
\R{(\wsj\cdot\waB\q{g})_0}{rB_{tor}(\R{R_0\q{g}}{r^2})_0}
,\eqbl
where $R_0$ is the radius of magnetic axis, $\q{g}$ is Jacobian, and
$r$ is the current cylindrical (major) radius. 
#endif
}
void e2astra_(Rmaxis,Zmaxis,gFtor,Fpol,gM,G22,G33,D2,D3,Sside,Sside1)
  double *Rmaxis;	/* R [m] of magnetic axis */
  double *Zmaxis;	/* Z [m] of magnetic axis */
  double *gFtor;	/* total toroidal flux [Vsec] */
  double *Fpol;		/* profile of $RB_{tor}$ [mT] */
  double *gM;		/* profile of inverse q */
  double *G22;		/* profile of ${g_{\gt\gt}\over rD}$ */
  double *G33;		/* profile of ${D\over r}$ */
  double *D2;		/* profile of $D$ */
  double *D3;		/* profile of $rD$ */
  double *Sside;	/* profile of $S$  */
  double *Sside1;	/* profile of $S_1$  */
{
  Undef splP();
  extern double *ESgF;	/* undefined */
  extern double c2gp;	/* undefined */
  extern double *ESgm;	/* undefined */
  extern double *ESsa;	/* undefined */
  extern double *ESg22c;/* undefined */
  extern double *ESLc;	/* undefined */
  extern double *ESDPc;	/* undefined */
  extern double *ESVc;	/* undefined */
  extern double *ESsb;	/* undefined */
  extern double *ESsb1a;/* undefined */
  extern double *rcT1a;	/* undefined */
  extern double *rsT1a;	/* undefined */
  extern int ESFp1;	/* undefined */
  extern double *rcT;	/* undefined */
  extern double *rsT;	/* undefined */
  extern int ESNp1;	/* undefined */
  extern double *cs1;	/* undefined */
  extern double *sn1;	/* undefined */
  extern double *ESsr;	/* undefined */
  double *Rmaxis;	/*  .e2astra_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  double *Zmaxis;	/*  .e2astra_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  double *gFtor;	/*  .e2astra_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  double *Fpol;		/*  .e2astra_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  double *gM;		/*  .e2astra_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  double *G22;		/*  .e2astra_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  double *G33;		/*  .e2astra_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  double *D2;		/*  .e2astra_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  double *D3;		/*  .e2astra_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  double *Sside;	/*  .e2astra_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  double *Sside1;	/*  .e2astra_().blb_ASTRA.mzl_ESCTR:esc2tr.c */
  static double crHp;	/*             .blb_ASTRA.mzl_ESCTR:esc2tr.c */
  static double rca[];	/*             .blb_ASTRA.mzl_ESCTR:esc2tr.c */
  static double rsa[];	/*             .blb_ASTRA.mzl_ESCTR:esc2tr.c */
  static double rc[];	/*             .blb_ASTRA.mzl_ESCTR:esc2tr.c */
  static double rs[];	/*             .blb_ASTRA.mzl_ESCTR:esc2tr.c */
  static double as[];	/*             .blb_ASTRA.mzl_ESCTR:esc2tr.c */
  static double as1[];	/*             .blb_ASTRA.mzl_ESCTR:esc2tr.c */
  static double as2p[];	/*             .blb_ASTRA.mzl_ESCTR:esc2tr.c */
  static double as12p[];/*             .blb_ASTRA.mzl_ESCTR:esc2tr.c */
  static double ch2hp;	/*             .blb_ASTRA.mzl_ESCTR:esc2tr.c */
/*
   */
#ifdef Doc_e2astra_
  \section{Skeleton of \Cline{e2astra()} }

  \begin{verbatim}
  void e2astra_(
    double *Rmaxis,
    double *Zmaxis,
    double *gFtor,
    double *Fpol,
    double *gM,
    double *G22,
    double *G33,
    double *D2,
    double *D3,
    double *Sside,
    double *Sside1
    )
  {
  }
  \end{verbatim}
\section{Output for ASTRA}

\begin{tabular}{|lrp{1.7in}p{2.in}|}\hline
\multicolumn{4}{|c|}{\large\bf Subroutine \Cline{e2astra\_()}}\\\hline
\bf ID	&Units &Notation&\bf Meaning\\\hline
{\tt *Rmaxis}	&$[m]$&$R_0$&$r$ of the magnetic axis\\\hline
{\tt *Zmaxis}	&$[m]$&$Z_0$&$z$ of the magnetic axis\\\hline
{\tt *gFtor}	&$[V\cdot sec]$&$\gF_{int}\equiv \int_{plasma}B_{tor}dS$
&Total toroidal flux inside the plasma cross-section\\\hline 
{\tt Fpol[21]}	&$[m\cdot T]$&$F_{pol}=rB_{tor}$
&Poroidal current at the grid points\\\hline 
{\tt gM[21]}	&$[]$&${1\over q}$
&${1\over q}$ values at the grid points\\\hline 
{\tt G22[21]}	&$[m^{-1}]$&$G_{22}$
&$G_{22}$ values at the grid points\\\hline 
{\tt G33[21]}	&$[m^{-1}]$&$G_{33}$
&$G_{33}$ values at the grid points\\\hline 
{\tt D2[21]}	&$[m^{-1}]$&$D_2$
&2D Jacobian $D_2$  at the grid points\\\hline 
{\tt D3[21]}	&$[m^{-1}]$&$D_3$
&3D Jacobian $D_3$  at the grid points\\\hline 
{\tt Sside[21]}	&$[m]$&$\left({g_{\gt\gt}r^2\over\sqrt{g}}\right)_0$
&$S_{side}$  at the grid points\\\hline 
{\tt Sside1[21]}&$[m^2]$&$\left(\sqrt{g_{\gt\gt}}r\right)_0$
&$S_{side,1}$  at the grid points\\\hline 
\end{tabular}
\vskip12pt
Flux coordinates $a,\gt$ in ESC 
\bqbl
&&
r(a,\gt)
=
r^c_0(a)
+2\sum^{k<K}_{k=1}[r^c_k(a)\cos k\gt+r^s_k(a)\sin k\gt]
,\\&&
z(a,\gt)
=
r^s_0(a)
+b(a)\sin\gt
,\eqbl
where radial flux coordinate for use in ASTRA was set as
\bqbl
&&
a
\equiv
{\q{\gF}\over\q{\gF_{int}}}
,\quad
\gF_{int}\equiv\int_{plasma}B_{tor}dS
.\eqbl
Metric coefficients
\bqbl
&&
g_{aa}
=
r'^2_a
+z'^2_a
,\quad
g_{a\gt}
=
r'_ar'_\gt
+z'_az'_\gt
,\quad
g_{\gt\gt}
=
r'^2_\gt
+z'^2_\gt
,\quad
D
=
r'_az'_\gt
-r'_\gt z'_a
.\eqbl
Output poloidal current (determining the toroidal magnetic field at the
magnetic surface) is defined as
\bqbl
&&
F_{pol}=RB_{tor}\ [m\cdot Tesla]
.\eqbl
Averaged output metric coefficients
\bqbl
&&
D_2
=
(D)_0
,\quad
D_3
=
(rD)_0
,\quad
S_p
=
2\gp\int D_2da
,\quad
V
=
4\gp^2\int D_3da
,\\&&
G_{22}
=
\left({g_{\gt\gt}\over rD}\right)_0
,\quad
G_{33}
=
\left({D\over r}\right)_0
,\\&&
S_{side}
=
\left({g_{\gt\gt}r^2\over rD}\right)_0
,\quad
S_{side,1}
=
\left(\sqrt{g_{\gt\gt}}r\right)_0
.\eqbl

#endif
}
#endif
#endif
#endif

#undef Doc_Cb_Structure
#ifdef Blb_esc2tr.c
Blb_esc2tr.c{
  mzl_ESC{
  }
  mzl_ESCTR{
  }
  mzl_ESC{
    blb_esc{
    }
  }
  mzl_ESCTR{
    blb_TSC{
    }
    blb_ASTRA{
    }
  }
}
#endif

#undef Doc_Table_Of_Content
#ifndef Blb_esc2tr.c
			PS p= -1 (  0);
#ifndef mzl_ESC
			PS p=  1 (  1);
#endif
#ifndef mzl_ESCTR
			PS p=  1 (  1);
#endif
#ifndef mzl_ESC
			PS p=  1 (  1);
#ifndef blb_esc
			PS p=  1 (  1);
void esc0_();		PS p=  1 (  1);
void esc_();		PS p=  2 (  1);
void esc1_();		PS p=  3 (  2);
#endif
#endif
#ifndef mzl_ESCTR
			PS p=  3 (  2);
#ifndef blb_TSC
			PS p=  3 (  2);
void tsc2e();		PS p=  3 (  2);
#endif
#ifndef blb_ASTRA
			PS p=  3 (  2);
void astra2e_();	PS p=  3 (  2);
void e2astra_();	PS p=  4 (  2);
#endif
#endif
#endif
