#define LDBL sizeof(double)
#define LINT sizeof(int)
#define NGRP 0x20
#define NTYPEG 0x10
typedef struct{
  char Nm[0x20];/* MDSplus tree name */
  char P[0x20];	/* Prefix */
  char S[0x20];	/* Suffix */
  char NmT[0x20];/* Time */
  char NmA[0x20];/* Radial */
  int k;	/* Number of waveform */
  int k0;	/* Starting Index */
  int k1;	/* End Index+1 */
  int n;	/* dimension of t */
  int m;	/* reduced dimension of t */
  int i0;	/* Starting Index of time frame */
  int i;	/* Current Index of the time sample */
  int i1;	/* End Index of time frame+1 */
  int id;	/* ID */
  int kt;	/* Types of groups:
		   0 - Plasma current 
		   1 - OH coil 
		   2 - PFCoils 
		   3 - TFCoil 
		   4 - Voltage
		   5 - Flux loop 
		   6 - Saddle coil
		   7 - B-probe 
		   8 - F-Loop (diamagnetic loop) 
		   9 - V-loop 
		*/
  int ka;	/* Type of the radial coordinate:
		   0 - R
		   1 - a
		*/
  int na1;    	/* number of radial points */
  double *a;	/* pointer to the radial coordinate */
  double *T;	/* pointer to a full t */
  double *t;	/* pointer to a reduced t */
}GROUPDT;

typedef struct{
  int n1;		/*number of mesh points elements*/
  double r[0x100];	/* r coordinate */
  double z[0x100];	/* z coordinate */
  double t[0x101];	/* parametrizing variable */
  double dt;		/* element of parametrizing variable */
  double nr[0x100];	/* r-component of external normal vector */
  double nz[0x100];	/* z-component of external normal vector */
  double dL[0x100];	/* z-component of external normal vector */
  double L[0x101];	/* Length */
  double LW;		/* Total length */
  char Nm[16];		/* name of the Wall */
}WALL;

typedef struct PFBLOCK{
  double r[2];	/* R of the vertices of the block */
  double z[2];	/* Z of the center of the block */
  double R;	/* R of the center of the block */
  double Z;	/* Z of the center of the block */
  double dR;	/* dR half the radial size of the block */
  double dZ;	/* dZ half the vertical size of the block */
  double tn;	/* number of real turns in the block */
  double gr;	/* $\rh$ - (averaged) resistivity of the material */
  double Res;   /* electrical resistance of material */
  double gt;	/* characteristic L/R time for this coil */
  double AI;	/* Ampere-Turn Current in the PFBlock */
  double I;	/* OneTurn Current in the PFBlock */
  double gY;	/* Vsec on the block */
  double V;	/* Voltage on the PFBlock */
  double gq;	/* angle of slicing */
  int nr;	/* number of working divisions in R direction */
  int nz;	/* number of working divisions in Z direction */
  int kS;	/* 0 - rectangular block;
		   1 - horizontally sliced parallelogram;
		   2 - vertically sliced parallelogram; 
		   */
  int kc;	/* color */
  int kC;	/* Coil # the PFB belongs to in the code */
  int kA;	/* Optimization index for current:
		   0 - desactivated block;
		   1, -1 - active PFBlock;
		   */
  int kD;	/* Number of data column in the data file */
  int kG;	/* group number corresponding to time sampling */
  char Nm[16];	/* name of the Block */
  char NmC[32];	/* name of the Coil (Circuit) */
}PFBLOCK;

typedef struct PSBLOCK{
  double r[4];	/* R of the vertices of the block */
  double z[4];	/* Z of the center of the block */
  double R;	/* R average */
  double Z;	/* Z average */
  double dS;	/* Area of cross-section */
  double ff;	/* form-factor of the cross-section */
  double gs;	/* $\rh$ - (averaged) conductivity of the material */
  double gS;    /* electrical conductivity of the PSBlock */
  double tn;	/* Fraction of the Total Current in the PSCoil */
  double I;	/* Total Current in the PFBlock */
  double gY;	/* Vsec on the block */
  double V;	/* Voltage on the PFBlock */
  int n1;	/* number of working divisions between 0-1 vertices */
  int n2;	/* number of working divisions between 0-3 vertices */
  int kC;	/* Coil # the PSB belongs to inside the code */
  int kA;	/* Optimization index for current:
		   0 - permanently desactivated block;
		   1, -1 - active PFBlock;
		   */
  int kD;	/* Number of data column in the data file */
  int kG;	/* group number corresponding to time sampling */
  int c;	/* if 0 - temporarily desactivated block, if 1-16 then color */
  char Nm[16];	/* name of the Block */
  char NmC[16];	/* name of the Coil (Circuit) */
}PSBLOCK;

typedef struct PFCOIL{
  double tn;	/* number of signed turns in the Coil */
  double Tn;	/* number of turns in the Coil */
  double Res;   /* electrical resistance of material */
  double gt;	/* PFCoil characteristic L/R time */
  double I;	/* Current in the turn of the PFCoil*/
  double V;	/* Voltage on the PFCoil */
  double gY;	/* Vsec on the PFCoil */
  double I0;	/* Reference current in the coil */
  double Ie;	/* EFIT resulting current in the coil */
  double dI;	/* Error in the current in the coil */
  double V0;	/* Reference voltage in the coil */
  double gY0;	/* Reference Vsec in the coil */
  double Emf;	/* electromotive force attached to the block */
  double wW;	/* regularization coefficient in gY_kI_i matrix */
  double wgY;	/* regularization coefficient in gY_kgY_k matrix */
  double wI;	/* regularization coefficient in I_kI_i matrix */
  double wR;	/* regularization coefficient in I_iI_i matrix */
  double wV;	/* regularization coefficient in V_kI_i matrix */
  int nB;	/* Number of PFBlocks in the PFCoil */
  int *kB;	/* Pointer on  the number series of PFBlocks */
  int kA;	/* Optimization index for current:
		   0 - desactivated PFCoil;
		   1 - preprogrammed PFCoil;
		   2 - feedback PFCoil;
		   3 - optimization PFCoil;
		   */
  int kD;	/* Number of data column in the data file */
  int kG;	/* group number corresponding to time sampling */
  int c;	/* if 0 - temporarily desactivated Coil, if 1-16 then color */
  char Nm[32];	/* name of the Coil (circuit)*/
  char NmS[32];	/* name of the Signal in the DataBase for the Coil*/
}PFCOIL;

typedef struct MFPROBE{
  double R;	/* R of the center of the probe */
  double Z;	/* Z of the center of the probe */
  double R1;	/* R of the center of the probe */
  double Z1;	/* Z of the center of the probe */
  double ga;	/* angle in degrees between probe axis and axis of symmetry */
  double Tr;	/* Radial component of the normal vector */
  double Tz;	/* Axial component of the normal vector */
  double s;	/* sensitivity of the probe, T/(V sec), see, pfc.h.d */
  double aErr;	/* absolute Error */
  double rErr;	/* relative Error */
  double S;	/* calculated signal on the probe */
  double S0;	/* signal data on the probe */
  double V0;	/* signal of tan(gg) */
  double V;	/* signal of tan(gg) */
  double Se;	/* EFIT calculated signal at the probe */
  double dS;	/* error in the signal on the probe */
  double gY;	/*  calculated poloidal flux on the probe */
  double Br;	/*  calculated Br at the probe */
  double Bz;	/*  calculated Bz at the probe */
  double w;	/* dimensionless regularization coefficient */
  int kA;	/* type of the probe:
		   1 - \gY loop,
		   2 - saddle coil,
		   3 - magnetic local probe,
		   4 - diamagnetic loop,
		   5 - Voltage loop,
		*/
  int kG;	/* group number corresponding to time sampling */
  int c;	/* color */
  char Nm[32];	/* name of the probe */
  char NmS[32];	/* name of the signal in the Data Base */
}MFPROBE;

typedef struct DATAMSE{
  int kA;	/* bitwise activity index.
		   bits meaning:
		   0 - existing but ignored data.
		   1 - reliable data.
		   2 - data intended for spline approximation of pitch angle.
		   8 - mapped data.
		   */
  int m;	/* Beam number */
  double x;	/* R of the pick up point */
  double y;	/* Z of the pick up point */
  double r;	/* R of the pick up point */
  double z;	/* Z of the pick up point */
  double gf;	/* gf of the pick up point */

  double vx;	/* vx of beam vector */
  double vy;	/* vy of beam vector */
  double vz;	/* vz of beam vector */

  double tr;	/* tr of beam vector */
  double tz;	/* tz of beam vector */
  double tf;	/* tgf of beam vector */

  double a;	/* a of the pick up point */
  double gt;	/* gt of the pick up point */
  double ra;	/* dra in the pick up point */
  double rq;	/* drq in the pick up point */
  double za;	/* dza in the pick up point */
  double zq;	/* dzq in the pick up point */
  double D;	/* D in the pick up point */

  double gg;	/* the measured polarization angle */
  double dgg;	/* the error in measured polarization angle */
  double ga;	/* angle in MSE geometry */
  double gd;	/* angle in MSE geometry */
  double gq;	/* angle in MSE geometry */
  double gW;	/* angle in MSE geometry */
  double P[6];	/* coefficients for conversion of Br,Bz,Bt into 
		   polarization angle */
  double S;	/* calculated tan(gg), gg - polarization angle */
  double S1;	/* calculated tan(gg), gg - polarization angle */
  double S0;	/* signal of tan(gg) */
  double Se;	/* EFIT signal of tan(gg) */
  double dS;	/* error in the signal */
  double errA;	/* absolute error in measurements */
  double errR;	/* relative error in measurements */
  double w;	/* dimensionless weight coefficient */
  double Bz;	/* z - component of the magnetic field */
  double dBz;	/* error bar in z - component of the magnetic field */
  double dgY;	/* derivative of the poloidal flux */
  double gi;	/* pitch angle */
}DATAMSE;

typedef struct SPL{
  double f;
  double x;
  double y;
  double xx;
  double yy;
  double xy;
  double xxy;
  double yyx;
}SPL;

typedef struct{
  char A[0x100];	/* Name of the a-axis */
  char T[0x100];	/* Name of the t-axis */
  int k;
  int k0;
  int k1;
  int m;	/* dimension of a */
  int n;	/* dimension of t */
  int i0;
  int i;
  int i1;
  int id;	/* ID */
  int *kt;	/* activity of the time slice */
  int j;	/* index of selected time */
  double *t;
  double *a;
}WFGROUP;

typedef struct RAWDATA{
  double C;	/* Common factor */
  double G;	/* Common Amplification factor */
  double A;	/* Amplification factor of raw data */
  double B;	/* Bias of raw data */
  double dC;	/* Error in Common factor */
  double dG;	/* Error in Common Amplification factor */
  double dA;	/* Error in Amplification factor of raw data */
  double dB;	/* Error in Bias of raw data */
  double S;	/* current raw signal */
  double dS;	/* estimated error in the current raw signal */
  double ga;	/* smoothing factor for splining */
  double *D;	/* pointer to the Waveform data */
  double *T;	/* pointer to the Waveform times */
  double *a;	/* pointer to the Waveform coordinate */
  double *V0;	/* pointer to splined signal */
  double *V2;	/* pointer to second derivative fo splined signal */
  double *ti;	/* pointer to the time rule for splined signal */
  double *V;	/* signal of the virtual current */
  double *dV;	/* signal of the virtual current */
  double *errR;	/* relative error in profiles for each channel*/
  double *errA;	/* absolute error in profiles for each channel*/
  int I1;	/* number of spline data points */
  int kA;	/* Activity factor */
  int kG;	/* Group number corresponding to time sampling */
  int kS;	/* Reference to the index of signal */
  int c;	/* color */
  int m;	/* number of radial points for profile data */
  int n;	/* number of Wafeform data */
  int i1;	/* index of start of Wafeform interval of interest */
  int i;	/* current index of the Wafeform interval of interest */
  int i2;	/* index of end of Wafeform interval of interest */
  char Nm[32];	/* Name of the signal */
}RAWDATA;

typedef struct{
  int iA;
  int K;
  int K1;
  double err;
  double dT;
  double tk[0x80];
  double r0[0x80];
  double r2[0x80];
  char Nm[32];
}RESPFUNC;

typedef struct{
  double x[2];
  double y[2];
  double z[2];
  double rP[2];
  double tx; /* x[1]-x[0] */
  double ty; /* y[1]-y[0] */
  double tz; /* z[1]-z[0] */
  double tE[2];
  double aE[2];  /*  a of boundary punch points */
  double qE[2]; /* gq of boundary punch points */
  double L[2];

  double S;  /* Signal */
  double dS;  /* Error in the signal */
  double *a; /*  a of integration points */
  double *q; /* gq of integration points */
  double *f;
  double *t;
  double *r; /* r of integration points */
  double *ra; /* ra of integration points */
  double *rq; /* rq of integration points */
  double *za; /* za of integration points */
  double *zq; /* zq of integration points */
  double *D; /* D of integration points */
  double *V;
  int n1;     /*  n1 of integration points */
}SIGHTLINE;
