#ifndef Blb_escD.c
/*
   ESC documentation. Subject to change by L.E.Zakharov without notice.
   
   */
#ifdef Doc_escD.c
%------------------------------------------------------------------------
\title{Theory of perturbed equilibria for solving the Grad-Shafranov
equation} 
\author{L.~E.~Zakharov, A.~Pletzer}
\date{June 21, 1999}
\maketitle
\begin{abstract}
The theory of perturbed magnetohydrodynamic equilibria is presented for
different formulations of the tokamak equilibrium problem. For numerical
codes, it gives an explicit Newton scheme for solving the Grad-Shafranov
equation subject to different constraints. The problem of stability of
axisymmetric modes is shown to be a particular case of the equilibrium
perturbation theory.   
\end{abstract}

\Section{Introduction}
Plasma equilibria lay at the fundamental level of magnetic confinement
studies. All plasma processes including linear and early nonlinear stage
of magnetohydrodynamic (MHD) instabilities, transport and plasma 
flows, waves, micro-instabilities and turbulence represent different
kinds of deviations from MHD equilibrium, and, thus, require accurate
calculations of equilibrium configurations
\cite{Shafranov62,Grad70,Grad75,Zakharov81,Khairutdinov93,Ernst98}. 

This paper describes a rigorous method for solving the nonlinear
Grad-Shafranov equation given different sets of input profiles, e.g.,
$dp/d\gY$ and $FdF/d\gY$, $p$ and $q$, $p$ and $j_\parallel$, etc, where
$p$ is the plasma pressure, $F$ the poloidal current, $q$ the safety
factor, $j_\parallel$ the averaged parallel component of the
current density, and $\gY$ the poloidal flux.

Because of the nonlinearity of the Grad-Shafranov (GSh) equation, all
numerical methods for equilibrium calculations are iterative by nature
\cite{Lackner76} and can be subdivided into two classes: 1) Eulerian,
x1which use a prescribed (e.g., rectangular or conformal to the plasma
boundary) mesh calculation 
\cite{Callen72,Feneberg73,Helton78,Johnson79,Goedbloed81,Blum81,Lutjens92} and 
2) Lagrangian, which use curvilinear flux coordinates for equilibrium
calculations
\cite{DeLucia80,Ling85,Gruber87,Huysmans91,Turkington93,Ludwig97}.  

Eulerian methods have the advantage of being able to easily reproduce
the 2-D geometry of 
complicated configurations \cite{Helton78}. They are widely used for
simulations of equilibrium control in tokamaks \cite{Albanese98} and for
the interpretation of experimental magnetic measurements
\cite{Lao85,Blum90,Ferron97}. The disadvantage of those methods is in
their limiting range of formulations of equilibrium problems, restricted
essentially to a prescribed right hand side of the Grad-Shafranov (GSh)
equation. Formulation requiring different input profiles can only be
implemented with difficulty, due to the resulting frequent need of
averaging over magnetic surfaces \cite{LoDestro94}.

The use of flux coordinates allows to substantially extend the range of
soluble problems. Essentially, any set of two 1-D functions of
the radial coordinate, which uniquely determines the current density and
the pressure profile \cite{Grad70,Grad75,Clark77,Hirshman79,Ling85}, can
be used in flux coordinate based Lagrangian codes. The disadvantage of
flux coordinate codes lies in their difficulty to treat free boundary
plasmas with a separatrix. 

Methods, which use flux coordinates, in their turn, can be subdivided
into two categories: a) variational
\cite{Khait80,Lao81,Lao84,Ling85,Turkington93} 
and inverse coordinate methods \cite{Vabishevich78,Degtyarev85,Gruber87}
and  b) adaptive grid methods
\cite{DeLucia80,Degtyarev91,Huysmans91}. Variational codes solve
equations where the unknowns are the flux coordinates themselves. Adaptive
grid codes solve equilibrium equations in given curvilinear coordinates
and then use the solution for advancing the coordinate system. 

Practically all existing codes use a simple iteration method for solving
the nonlinear GSh equation. Although this approach is sufficient in most
of situations, there is still a demand for fast methods which can be
used for extensive calculations of magnetic configurations in transport
codes, for scrutinized studies of plasma stability (especially
non-ideal), for equilibrium reconstruction from experimental
measurements and for simulations of equilibrium control in real
machines.  

This paper describes a perturbative approach for solving the GSh
equation using constraints that are specific to each formulation. The
discrepancy between the nonlinear equation and its linear version then
becomes a driving contribution. Such an approach is particularly
adequate for the adaptive grid codes \cite{DeLucia80}. It is, however,
more complicated to implement in variational or inverse coordinate codes
due to the difficulty to linearize the equations in these codes
(although some attempts have been made, \cite{Ling85}).

%The theory of perturbed equilibrium, which is presented in this paper,
%suggests use of perturbed GSh equation (with constraints, specific to
%each particular formulation of the equilibrium problem), when
%discrepancy of nonlinear equation is considered as a driving
%perturbation in the right hand side of linearized equation. This
%approach is rather simple for implementation into adaptive grid codes 
%\cite{DeLucia80}, which solve classical Grad-Shafranov equation with its
%linear differential operator in a given coordinate system. It is more 
%complicated to linearize the highly nonlinear equations of
%the variational or inverse coordinate formulations (although such attempts
%have been performed, \cite{Ling85}). 

Together with an explicit algorithm which advances the coordinates, the
theory of perturbed equilibria can be cast in a Newton scheme for solving 
non-linear tokamak equilibrium problems. 

The use of perturbed equilibrium theory, however, finds applications
beyond equilibrium calculations. In Sec.~5 we describe how the
combination of flux conservation and adiabatic pressure constraints
yields a formulation for axisymmetric stability problems
\cite{Chu78,Haney97}, thus, allowing equilibrium and axisymmetric
stability calculations to be covered by the same codes.

%It is possible to notice, that the theory of ideal marginal stability is
%a particular case (with constraints of flux conservation and of adiabatic
%pressure) of perturbed equilibria.  This fact makes a generic link
%between equilibrium and stability problems which allows to cover both
%equilibrium and stability (of axisymmetric modes \cite{Chu78,Haney97})
%by the same codes.  

The paper is organized in the following manner. Sect.~2 introduces the
set of physical variables describing the equilibrium problems. Sect.~3
lists the relationships between the basic plasma profiles in equilibrium
configurations. Sect.~4 describes the equations of perturbed
equilibrium. Sect.~5 describes the coordinate advancing method, which is
consistent with the choice of poloidal angle. Sect.~6 shows the
connection between the Euler equation of ideal MHD stability and the
theory of perturbed equilibrium. In the Summary, some results from the 
equilibrium code, based on the perturbation theory, are presented.

In this text, we use $m$, $sec$, $T$, $MA$, $MPa$ as units for length, time,
magnetic field, current and pressure, respectively, as well as 
$V$-$sec$, $MA/m^2$, etc. We will also use rationalized variables,
typically present inside numerical codes and will distinguish them from
the real physics variables by the bar sign.

\Section{2-D magnetic fields and plasma currents in curvilinear coordinates}
Let $a,\gt,\gf$ be curvilinear coordinates in toroidal topology, where $a$
represents a radial coordinate, $\gt$ a poloidal angle and $\gf$ a
toroidal angle (identical to the azimuthal angle in cylindrical 
coordinates). The cylindrical coordinates, $r,\gf,z$, can be expressed as
\bqbl
&&
r
=
r(a,\gt,\gf)
,\quad
z
=
z(a,\gt,\gf)
\lbq{TR}
.\eqbl
We do not assume that the coordinates $a,\gt,\gf$ are flux
coordinates. It is the goal of the adaptive method to adjust ($a,\gt,\gf$) to
magnetic flux tubes. 

As an example, the transformation (\rfq{TR}) can be expressed in the
form of Fourier expansion in poloidal angle
\bqbl
&&
r
=
r^c_0(a)
+2\sum^{m\leq M}_{m=1}\left[
  r^c_m(a)\cos m\gt
 +r^s_m(a)\sin m\gt
 \right]
,\\&&
z
=
z^c_0(a)
+2\sum^{m\leq M}_{m=1}\left[
  z^c_m(a)\cos m\gt
 +z^s_m(a)\sin m\gt
 \right]
\lbq{FS}
.\eqbl
This form is not unique and allows for different choices of poloidal
angles. This issue has been intensively discussed in the literature,
especially for stellarator equilibria (see, e.g., \cite{Hirshman98}
and the references there). For tokamaks, there is a simple form of
Fourier representation \cite{Zakharov81}
\bqbl
&&
r
=
r^c_0
+2\sum^{m\leq M}_{m=1}(r^c\cos m\gt+r^s\sin m\gt)
,\\&&
z
=
z_0(a)
+b(a)\sin\gt
\lbq{polangl}
\eqbl
with only two harmonics in the $z$-coordinate. This form implies that
any horizontal line $z=const$ intersects cross-section of the magnetic 
surface not more than two times. Despite of this limitation, this
representation is suitable for practically all tokamak configurations. 

In the radial direction, cubic splines or, e.g., Hermite polynomials can
be used to represent the Fourier coefficients in the differentiable form,
which is necessary for the calculation of the
%and as cubic splines of radial coordinates, e.g., 
%\bqbl
%&&
%r^c_m(a) 
%=
%X_ir^c_m(a_i)+x_ir^c_m(a_{i+1})
%+[(X_i^3-X_i)r''^c_m(a_i)+(x_i^3-x_i)r''^c_m(a_{i+1})]{h_i^2\over6}
%,\\&&
%x_i\equiv{a-a_i\over h_i}
%,\quad
%h_i\equiv a_{i+1}-a_i
%,\quad
%X_i\equiv1-x_i
%,\quad
%a_i\leq a\leq a_{i+1}
%\lbq{SP}
%,\eqbl
%when the functions of radial coordinate $a$ are determined by their values
%and values of their second derivatives on a given radial grid. In some
%cases (e.g., for calculations of particle trajectories), spline
%representation with respect to all variables could be more preferable
%for faster calculations. 
metric tensor $g_{ik}$
\bqbl
&&
dl^2
\equiv
g_{aa}da^2
+2g_{a\gt}dad\gt
+g_{\gt\gt}d\gt^2
+r^2d\gf^2
,\eqbl
specifying the element of length, which can easily be obtained from
(\rfq{TR})
\bqbl
&&
g_{aa}
=
 r'^2_a
+z'^2_a
,\quad
g_{a\gt}
=
 r'_ar'_\gt
+z'_az'_\gt
,\quad
g_{\gt\gt}
=
 r'^2_\gt
+z'^2_\gt
,\eqbl
where, e.g., $r'_a\equiv\pa r/\pa a$. The poloidal Jacobian $D$ and the
total Jacobian $\sqrt{g}$ are
\bqbl
&&
D
\equiv
{D(r,z)\over D(a,\gt)}
=
 r'_az'_\gt
-r'_\gt z'_a
,\quad
\sqrt{g}
=
rD
,\eqbl
respectively.

The magnetic vector potential $\waA$ is represented
through its covariant components as 
\bqbl
&&
\waA
\equiv
 A_a\na a
+A_\gt\na \gt
+A_\gf\na \gf
=
 A_a\na a
+{\gF\over2\gp}\na\gt 
+{\gY\over2\gp}\na\gf 
\equiv
 A_a\na a
+\bgF\na\gt 
+\bgY\na\gf 
\lbq{BbA}
,\eqbl
where $\gF$ is the toroidal flux trough the cross-section of coordinates
$a=const$, $\gf=const$, while $\gY$ is the poloidal flux between the
cylindrical axis $r=0$ and the coordinate surface $a=const$,
$\gt=const$. Note, that while the poloidal component $A_\gt=\bgF$ in
axisymmetric system can be chosen as a function of $a$ 
only, $\bgF=\bgF(a)$, $\bgY$ may be non-constant, $\bgY=\bgY(a,\gt)$, on
the  toroidal coordinate surface $a=const$. We also introduce a periodic 
function $\gh$, related to the radial component of the vector potential
\bqbl
&&
A_a
\equiv
-{d\bgF\over da}\gh
\lbq{gh}
.\eqbl 
In the case of flux coordinates, when $\gY=\gY(a)$, the function $\gh$
determines the transition from $\gt$ to the ``straight field line''
poloidal angle $\gq$ 
\bqbl
&&
\gq
\equiv
\gt+\gh
.\eqbl

Eqs.~(\rfq{BbA}, \rfq{gh}) give a contravariant form for the magnetic field
\bqbl
&&
\waB
=
\na\times\waA
=
 \bgY'_\gt(\na\gt\times\na\gf)
+\bgF'(1+\gh'_\gt)(\na a\times\na\gt) 
+\bgY'_a(\na a\times\na\gf)
,\\&&
\waB
\equiv
B^a\Bse_a
+B^\gt\Bse_\gt
+B^\gf\Bse_\gf
,\quad
\Bse_a
\equiv
 r'_a\Bse_r
+z'_a\Bse_z
,\quad
\Bse_\gt
\equiv
 r'_\gt\Bse_r
+z'_\gt\Bse_z
,\quad
B^a
=
\waB\cdot\na a
,\quad
\ldots
,\\&&
rDB^a
=
\bgY'_\gt
,\quad
rDB^\gt
=
-\bgY'_a
,\quad
rDB^\gf
=
\bgF'(1+\gh'_\gt)
.\eqbl
The toroidal field, $B_{tor}$, is related to the total poloidal current
$F=F(a,\gt)$ in a simple way
\bqbl
&&
B_{tor}
=
rB^\gf
=
{\baF\over r}
,\quad
\baF
\equiv
0.2F
=
rB_{tor}
,\eqbl
which gives the expression of the toroidal flux in terms of poloidal current
\bqbl
&&
\bgF'(1+\gh'_\gt)
=
{\baF D\over r}
,\quad
\bgF'
=
\left({\baF D\over r}\right)_0
.\eqbl
Here and throughout the paper we use $(\ldots)_0$ to denote the zero
harmonic in poloidal angle $\gt$, reserving the notation
$<\ldots>$ for conventional averaging between magnetic surfaces 
\bqbl
&&
<f>
\equiv
{\oint\q{g}fd\gt\over\oint\q{g}d\gt}
=
{\oint{fd\gt\over B_{pol}}\over\oint{d\gt\over B_{pol}}}
=
{(\q{g}f)_0\over(\q{g})_0}
\lbq{sav}
.\eqbl

The plasma current density $\wsj$ can be expressed in contravariant form,
\bqbl
&&
\gm_0\wsj
\equiv
\bsj^a\Bse_a
+\bsj^\gt\Bse_\gt
+\bsj^\gf\Bse_\gf
,\quad
\gm_0
\equiv
0.4\gp
,\\&&
rD\bsj^a
=
\baF'_\gt
,\quad
rD\bsj^\gt
=
-\baF'_a
,\quad
rD\bsj^\gf
=
\baJ'_a+\gn'_\gt
\eqbl
in terms of the poloidal $\baF(a,\gt)$ and toroidal $\baJ=0.2J$ currents, and
a periodic function $\gn'_\gt$. The toroidal current $J$ is a
function of $a$ only  $J=J(a)$, while the function $\gn'_\gt$ represents the
oscillating part of the toroidal current density (related to the
Pfirsch-Shl{\"u}ter current).

Equilibrium equation
\bqbl
&&
\na p
=
\wsj\times\waB 
\eqbl
gives the Grad-Shafranov expression for the toroidal component of the
current density in terms of poloidal flux derivatives of the plasma
pressure $p'(\gY)$ and $FF'(\gY)$ 
\bqbl
&&
\bsj_{tor}
=
r{d\bsp\over d\bgY}
+{1\over r}\baF{d\baF\over d\bgY}
,\quad
\bsp
\equiv
\gm_0 p
,\\&&
\baJ'+\gn'_\gt
=
rD{d\bsp\over d\bgY}
+{D\over r}\baF{d\baF\over d\bgY}
,\eqbl
which, after substitution into toroidal component of Amp{\`e}re's law
\bqbl
&&
\na\times\waB
=
\gm_0\wsj
,\eqbl
leads to the Grad-Shafranov equation
\bqbl
&&
\gD^*\bgY
\equiv
{\pa^2\bgY\over\pa r^2}
-{1\over r}{\pa\bgY\over\pa r}
+{\pa^2\bgY\over\pa z^2}
=
-r^2{d\bsp\over d\bgY}
-\baF{d\baF\over d\bgY}
\lbq{GSh2D}
.\eqbl
We introduce the notation
\bqbl
&&
P(\gY)
\equiv
{d\bsp\over d\bgY}
,\quad
T(\gY)
\equiv
\baF{d\baF\over d\bgY}
\eqbl
for the two 1-D functions in the RHS of this equation.  In the
curvilinear coordinates, the GSh equation can be written in the form 
\bqbl
&&
{\cal L}\bgY
=
-rDP
-{D\over r}T
\lbq{GrSh0}
,\eqbl
where ${\cal L}$ is a partial differential operator
\bqbl
&&
{\cal L}
\equiv
 {\pa\over\pa a}{g_{\gt\gt}\over rD}{\pa\over\pa a}
-{\pa\over\pa a}{g_{a\gt}\over rD}{\pa\over\pa\gt}
-{\pa\over\pa \gt}{g_{a\gt}\over rD}{\pa\over\pa a}
+{\pa\over\pa \gt}{g_{aa}\over rD}{\pa\over\pa\gt}
=
{D\over r}\gD^*
.\eqbl
We designate the geometric coefficients present in the differential GSh
operator by
\bqbl
&&
K
\equiv
{g_{\gt\gt}\over rD}
,\quad
G_{a\gt}
\equiv
{g_{a\gt}\over rD}
,\quad
G_{aa}
\equiv
{g_{aa}\over rD}
,\eqbl
and in the RHS by
\bqbl
&&
L
\equiv
{D\over r}
,\quad
V
\equiv
Dr
.\eqbl

\Section{Basic plasma profiles in equilibrium configurations}
In this section, assuming that $a=a^*,\gt,\gf$ are already equilibrium
flux coordinates
\bqbl
&&
\bgY
=
\bgY(a^*)
,\quad
\bsp
=
\bsp(a^*)
,\quad
\baF
=
\baF(a^*)
,\eqbl
we give expressions for typical 1-D plasma profiles and their relationships
with the RHS of the GSh equation. They can be used as input equilibrium
profiles in the context of transport and stability studies.   

Denoting the magnetic axis radius by $R_0$, we can express
the toroidal current density
\bqbl
&&
\bsj_{tor}
\equiv
\bsj_s{R_0\over r}
+\bsj_p\left({r\over R_0}-{R_0\over r}\right)
,\eqbl
in terms of two 1-D functions $\bsj_s(a^*)$ and $\bsj_p(a^*)$, which have
the dimensions of a current density:
\bqbl
&&
\bsj_p
\equiv
PR_0
,\quad
\bsj_s
\equiv
{T\over R_0}+PR_0
.\eqbl
At the magnetic axis we have $\bsj_{tor}=\bsj_s$.

The total current $\baJ$ through the cross-section of the magnetic surface
is
\bqbl
&&
\baJ
=
\int^{a^*}_0(TL_0+PV_0)da
,\quad
L_0
\equiv
(L)_0
,\quad
V_0
\equiv
(V)_0
.\eqbl
The GSh equation, written in flux coordinates, then reduces to
\bqbl
&&
{\pa(K\bgY')\over\pa a^*}
-{\pa(G_{a^*\gt}\bgY')\over\pa \gt}
=
-TL
-PV
,\eqbl
which, after averaging
\bqbl
&&
(K_0\bgY')'
=
-TL_0
-PV_0
,\quad
K_0
\equiv
(K)_0
\lbq{AvGSh}
\eqbl
gives an expression of the toroidal current $\baJ$
\bqbl
&&
\baJ
=
-K_0\bgY'
\eqbl
in terms of $\bgY'$.
The toroidal flux $\bgF$ through the cross-section of the magnetic surface
is 
\bqbl
&&
\bgF
=
\int^{a^*}_0 \baF L_0da
,\eqbl
whereas the safety factor $q$ has the form
\bqbl
&&
q
=
-{\bgF'\over\bgY'}
=
-{\baF L_0\over\bgY'}
\lbq{q}
.\eqbl
The slow time evolution is described by the parallel component of Ohm's
law (taken here in its simplest form \cite{Shafranov62,Grad70})
\bqbl
&&
\waB\cdot\wsj
=
\bgs\waB\cdot\waE
,\quad
\waE
=
-\na\gf_E
-{\pa\waA\over\pa t}
,\quad
\bgs
\equiv
10^{-6}\gs
\lbq{Ohm}
,\eqbl
where $\waE$ is the electric field, $\gf_E$ is the scalar potential of
electric field, $\gs$ is electrical conductivity (the factor $10^{-6}$
appeared in the rationalized $\bgs$ because of our units of $MA/m^2$ for
the current density). After averaging over the layer between magnetic
surfaces (\rfq{sav}) we get
\bqbl
&&
<\waB\cdot\wsj>
=
-\bgs\left<\waB\cdot{\pa\waA\over\pa t}\right>
,\eqbl
Eq.~(\rfq{Ohm}) gives the equilibrium evolution equation in the form of
\bqbl
&&
{\pa\bgY'\over\pa t}
+q{\pa\bgF\over\pa t}
=
-{<\waB\cdot\wsj>\over\bgs<\waB\cdot\na\gf>}
.\eqbl
We introduce a profile $j_\parallel$ associated with Ohm's law
\bqbl
&&
j_\parallel
\equiv
{<\waB\cdot\wsj>\over R_0<\waB\cdot\na\gf>}
,\eqbl
which has the dimension of a current density and, at the magnetic axis, is
equal to the value of the physical current density. In flux coordinates 
\bqbl
&&
j_\parallel
=
{1\over R_0L_0}\left(\baJ'+{K_0\bgY'\baF'\over\baF}\right)
=
{1\over R_0L_0}\left(L_0T+V_0P+{K_0\bgY'\baF'\over\baF}\right)
,\eqbl
In terms of this profile, the time evolution equation then becomes
\bqbl
&&
{\pa\bgY'\over\pa t}
+q{\pa\bgF\over\pa t}
=
-{R_0j_\parallel\over\bgs}
.\eqbl
It can also be written in the form of a diffusion equation for $\bgY$
\bqbl
&&
{\pa\bgY'\over\pa t}
+q{\pa\bgF\over\pa t}
=
{1\over L_0\bgs}\left[(K_0\bgY')'-{K_0\bgY'\baF'\over\baF}\right]
.\eqbl
The 1-D profiles mentioned here essentially exhaust the list of
typical input profiles for equilibrium calculations. Nevertheless,
there are other possible non-conventional profiles,
e.g., the pitch angle $\gi$ of poloidal magnetic field, related to the MSE
measurements in tokamaks \cite{Hirshman94,Batha96a},
\bqbl
&&
\gi
\equiv
{B_{pol}\over B_{tor}}
\eqbl
as a function of the major radius $r$, or bootstrap fraction of the
current density.

For different input profiles, it is necessary to use different forms of
averaged GSh equation (\rfq{AvGSh}) in order to generate the zeroth order
approximation for the solution. We list here some of them for a typical
set of input profiles: 
\begin{enumerate}
\item For given $P$ and $T$, the first order differential equations:
\bqbl
&&
J'=L_0T+V_0P
,\quad
\bgY'=-\R{J}{K_0}
,\quad
\baF\baF'=T\bgY'
,\mbox{and}\quad
\bsp'=P\bgY'
\eqbl
determine all necessary equilibrium profiles. 
\item For given $\bsj_\parallel$ and $P$, we use 
\bqbl
&&
\left(\R{\baJ}{\baF}\right)'
=
\R{L_0R_0\bsj_\parallel}{\baF}
,\quad
\bgY'=-\R{J}{K_0}
,\quad
\left(1+\R{\baJ^2}{K_0L_0\baF^2}\right)\baF\baF'
=
\R{V_0\baJ}{K_0L_0}P
-\R{\baJ}{K_0}R_0\bsj_\parallel
,\quad
\bsp'=P\bgY'
.\eqbl
\item For given $\bsj_\parallel$ and $\bsp$ (an appropriate
choice for transport codes), the equations are similar
to the previous set
\bqbl
&&
\left(\R{\baJ}{\baF}\right)'
=
\R{L_0R_0\bsj_\parallel}{\baF}
\quad
\bgY'=-\R{J}{K_0}
,\quad
\left(1+\R{\baJ^2}{K_0L_0\baF^22}\right)\baF\baF'
=
-\R{V_0}{L_0}\bsp'
-\R{\baJ}{K_0}R_0\bsj_\parallel
.\eqbl
\item If the rotational transform $\gm=1/q$ is specified, then either
\bqbl
&&
\left(1+K_0L_0\gm^2\right)\baF'
+\gm(K_0L_0\gm)'\baF
=
\gm\R{V_0}{L_0}P
\eqbl
should be used if $P$ is given, or
\bqbl
&&
\left(1+K_0L_0\gm^2\right)\baF\baF'
+\gm(K_0L_0\gm)'\baF^2
=
-\R{V_0}{L_0}\bsp'
,\eqbl
if $p$ is given. Other profiles can be found from the algebraic
relationships of this section.
\item Given $\bgY$ and either $\bsp$ or $P$ (another appropriate
choice for interfacing with transport codes) leads simply to
\bqbl
&&
L_0\baF\baF'
=
-(K_0\bgY')'
-V_0P
=
-(K_0\bgY')'
-V_0\R{\bsp'}{\bgY'}
.\eqbl
\end{enumerate}

\Section{Perturbed Grad-Shafranov equation}
The Grad-Shafranov equation (\rfq{GrSh0}) implies that there is a degree
of freedom in the choice of the two 1-D profiles (e.g., $P$, $T$ or $p$,
$j_\parallel$) uniquely determining the pressure and current
distributions. In fact, there is also a degree of freedom in the choice
of the radial variable $a$ which labels the magnetic surfaces (e.g.,
$a=\q{\bgF/\bgF_b}$ , where $\bgF_b$ is the total toroidal flux through
the plasma cross-section). Thus, besides the boundary conditions, there 
are in total three 1-D constraints for solving the GSh equation.  

Assuming that the coordinate system $a,\gt$ is close to the real (target)
flux coordinates, $a^*,\gt$,
\bqbl
&&
a
=
a^*
+\gx(a,\gt)
\lbq{a2bA}
,\eqbl
we can write the transition from the current $r=r(a,\gt)$, $z=z(a,\gt)$
coordinates to the target flux coordinates $r=r^*(a^*,\gt)$,
$z=z^*(a^*,\gt)$ in explicit form as
\bqbl
&&
r^*
\equiv
r(a^*+\gx,\gt)
\simeq
r(a^*,\gt)
+r'_a\gx
,\quad
z^*
\equiv
z(a^*+\gx,\gt)
\simeq
z(a^*,\gt)
+z'_a\gx
\lbq{rz2rzA}
\eqbl
in linear approximation. The displacement $\gx$ of the coordinates can
be found from the flux coordinate equation 
\bqbl
&&
\bgY(a,\gt)
\equiv
\bgY^0(a)
+\gy(a,\gt)
=
const
%=
%\bgY(a^*)
\lbq{bgY0}
,\eqbl
where $\bgY^0(a)$ is the main order approximation for the solution,
which can be obtained by solving the averaged equations of the previous
section, and $\gy$ the (unknown) linear perturbation of the
poloidal flux function. In linear approximation, Eq.~(\rfq{bgY0})
gives a relationship between the displacement $\gx$ and the perturbation 
of the poloidal flux 
\bqbl
&&
\bgY'^0\gx
+\gy
=
const
%=
%\bgY(a^*)
%-\bgY^0
\lbq{gx}
.\eqbl
In the displacement $\gx$ it is necessary to distinguish between its
average, $\gx_0$, and oscillatory, $\gx_\til$, parts
\bqbl
&&
\gx
=
\gx_0(a)
+\gx_\til(a,\gt)
.\eqbl
Eq.~(\rfq{gx}), in fact, determines only the oscillatory part of the
displacement
\bqbl
&&
\gx_\til
=
-{\gy_\til\over\bgY'^0} 
\lbq{gY_til}
,\eqbl
%since $\bgY(a^*)$ in the RHS of Eq.~(\rfq{gx}) is unknown.
since the constant in the RHS of Eq.~(\rfq{gx}) is unknown.
Equation (\rfq{gY_til}) is equivalent to the condition of absence of
radial component of the magnetic field $B\cdot\na a^*=0$, written in
linearized form.

The average part of displacement $\gx_0$ %(and, thus, $\bgY(a^*)$)
is determined by the labeling of the magnetic surfaces. For example, if
the poloidal flux $\bgY$ is chosen as the radial coordinate $a$, then  
\bqbl
&&
\gx_0
=
{a-\bgY^0-\gy_0\over\bgY'^0}
\lbq{gx0a}
.\eqbl
Note, that because $\bgY^0$ is calculated by solving the 1-D equations of
the previous section, it is easy to adjust coordinates in a way that
$a=\bgY^0$. In this case 
\bqbl
&&
\gx_0
=
-{\gy_0\over\bgY'^0}
,\quad
\gx
=
-{\gy\over\bgY'^0}
\lbq{gy}
\eqbl
and the displacement $\gx$ is ``frozen'' into poloidal flux as in
ideal stability theory. 
 
If the normalized toroidal flux serves as the radial coordinate,
$a\equiv\bgF/\bgF_b$, then 
\bqbl
&&
\gx_0
=
{a\bgF_b-\bgF\over\bgF'}
\lbq{gx0b}
.\eqbl
If square root $\q{\bgF/\bgF_b}$ of the normalized toroidal flux is
chosen as $a$, then 
\bqbl 
&&
\gx_0
=
2{a\q{\bgF\bgF_b}-\bgF\over\bgF'}
\lbq{gx0c}
.\eqbl
Other choices can be covered in analogous way.
%Given $\gx_0$, the
%zeroth harmonic of the perturbation of the poloidal flux $\gy_0$ can be
%defined as 
%\bqbl 
%&&
%\gy_0
%\equiv
%-\bgY'^0\gx_0
%\lbq{gy0}
%.\eqbl
%Then, the perturbation $\gy$ of $\bgY$ can be represented in terms of 
%displacement $\gx$ as
%\bqbl 
%&&
%\gy
%=
%-\bgY'^0\gx
%\lbq{gy}
%,\eqbl
%which simulates ``frozenness'' of coordinate surfaces into the poloidal
%flux like in the ideal stability theory.

Now, it is possible to linearize the Grad-Shafranov equation
\bqbl
&&
\gD^*\bgY
=
-r^2P(a^*)
-T(a^*)
.\eqbl
Its differential operator is already linear and only the RHS has to be
linearized
\bqbl
&&
P(a^*)
=
P(a)
-P'\gx
+\gd P
,\quad
T(a^*)
=
T(a)
-T'\gx
+\gd T
\lbq{gdPT}
,\eqbl
where in both equations the first term in the RHS represents the main
approximation, while the second term is the convective perturbation. The
third term represents the non-convective part of perturbations of
$\gd P$ or $\gd T$ and is determined by linearization of the
constraints on the two inputs profiles, as shown below. Using
Eqs.~(\rfq{gdPT}), the perturbed GSh equation can be written as
\bqbl
&&
\gD^*\bgY
=
-r^2P(a)
-T(a)
+r^2P'\gx
+T'\gx
-r^2\gd P
-\gd T
\lbq{GrSh}
,\eqbl
or in the curvilinear coordinates as
\bqbl
&&
{\cal L}\bgY
=
-VP
-LT
+VP'\gx
+LT'\gx
-V\gd P
-L\gd T
\lbq{GrShc}
.\eqbl
The conventional simple iteration scheme \cite{Lackner76} for solving GSh
equation used in most equilibrium codes, takes into account only
the first two terms in the RHS of Eq.~(\rfq{GrSh}). The addition of
linear terms into the RHS accelerates convergence of solving the
nonlinear GSh equaton. 

Now, let us consider the perturbations $\gd P$ and $\gd T$ in
Eq.(\rfq{GrSh}), which depend on the input profiles. If the input
profiles are $P(a)=d\bsp/d\bgY$ and $T(a)=\baF d\baF/d\bgY$,
then  $\gd P=0$ and $\gd T=0$. 

Typically, e.g., in interfacing with the transport codes, pressure
profile $\bsp(a)$, rather than $P$ is given as a function of $a$. In
this case, 
\bqbl
&&
\gd P
\equiv
\gd\R{\bsp'}{\bgY'^0}
=
-\R{\bsp'}{\bgY'^0}\R{\gd\gy'_0}{\bgY'^0}
=
-P\R{\gd\gy'_0}{\bgY'^0}
,\eqbl
where $\gd\gy_0$ is the non-convective part of the perturbation of $\gy_0$:
\bqbl
&&
\gd\gy_0
\equiv
\gy_0
+\bgY'^0\gx_0
.\eqbl
In more complicated cases, the relationship between $\gd P$ and $\gd T$
and the input profiles contain certain averaged metric coefficients, such
as $L_0$, $V_0$, $K_0$.  Their perturbations can be obtained by using
the coordinate transformation (\rfq{rz2rzA}). It is straightforward to
see that 
\bqbl
&&
\gd D
=
(D\gx)'_a
.\eqbl
Consequently,
\bqbl
&&
\gd V
=
\gd (rD)
=
(V\gx)'_a
,\quad
\gd L
=
\gd\left({D\over r}\right)
=
(L\gx)'_a
.\eqbl
The perturbation of the metric coefficient $K$
\bqbl
&&
\gd K
=
\R{g_{\gt\gt}}{rD}
=
(K\gx)'_a
-2K\gx'_a
+2G_{a\gt}\gx'_\gt
\lbq{K}
\eqbl
can be expressed in terms of the displacement $\gx$.

Now, let us consider typical input profiles, related to the current
density, taken as exemples in the previous section. If $q(a)$ (or
$\gm=1/q$) is given as an input profile, the perturbation of $\baF$ can
be obtained from formula (\rfq{q}) 
\bqbl
&&
\gd \baF L_0
=
-\baF\gd L_0
-q\gd\gy'_0
=
-\baF(L\gx)'_{a0}
-q\gd\gy'_0
.\eqbl
This yields $\gd T$ in the form of the second order integro-differential
operator 
\bqbl
&&
\gd T
=
\R{(\baF\gd \baF)'}{\bgY'^0}
-T\R{\gd\gy'_0}{\bgY'^0}
=
\r{\bgY'^0}\left(
 -\R{\baF^2(L\gx)'_{a0}}{L_0}
 +\baF^2\R{\gd\gy'_0}{\bgY'^0}
\right)'_a
-T\R{\gd\gy'_0}{\bgY'^0}
\lbq{gdTq}
.\eqbl
If the radial coordinate is chosen as a function of either the poloidal
$\bgY$ or toroidal $\bgF$ flux (typically the case in the
transport codes), then $\gd\gy_0=0$ and there is only one term left in
the RHS of this equation.  

When expression (\rfq{gdTq}) for $\gd T$ is substituted into perturbed
GSh equation (\rfq{GrSh}), it can be noticed that because of the 
strong toroidal magnetic field of the tokamak, the differential operator
coming from $\gd T$ and containing a big factor $\baF^2$ will be
dominant. By taking this into account, it is possible to use a
simplified approximate condition 
\bqbl
&&
(L\gx)_0
=
0
\lbq{frzn}
\eqbl
instead of the rigorous expression (\rfq{gdTq}) for $\gd T$. In order to
do this in an effective way, let us distinguish between the averaged and
oscillatory parts of the GSh equation. Then, $\gd T$ can be kept only in
the averaged component of equation (\rfq{GrSh}) 
\bqbl
&&
({\cal L}\bgY)_0
=
-V_0P
-L_0T
-V_0\gd P
-L_0\gd T
+P'(V\gx)_0
+T'(L\gx)_0
\lbq{avGSh}
,\eqbl
and excluded from the oscillating part of this equation
\bqbl
&&
{\cal L}\bgY
-{L\over L_0}({\cal L}\bgY)_0
=
\left(V_0{L\over L_0}-V\right)(P+\gd P)
+P'\left[(V\gx)_0{L\over L_0}-V\gx\right]
+T'\left[(L\gx)_0{L\over L_0}-L\gx\right]
\lbq{oscGSh}
.\eqbl
This separation gives more flexibility in the treatment of
constraints. On the one hand, the 1-D equation (\rfq{avGSh}) and 2-D
equation (\rfq{oscGSh}) can be 
considered as a system of equations if both $\gd P$ and $\gd T$ are
provided by other considerations. On the other hand, the entire averaged
equation (\rfq{avGSh}), which contains the term $\gd T$, can be replaced
with simpler constraints, such as, e.g., the algebraic expression
(\rfq{frzn}). The simplest case of similar constraints corresponds to
formulation 5 in the previous section, where $\bgY$ is specified as an
input profile and there is no needs to derive $\gd T$. 

The rigorous expressions for $\gd T$ can be obtained in a similar way
when $\bsj_\parallel$ is the input profile. Because $\gd T$ affects
only the convergence rate, it is possible to use
some reasonable simplifications in $\gd T$, and to drop, for example, terms
containing the ratio between poloidal and toroidal magnetic fields in
tokamaks. For given $\bsj_\parallel$ (formulations 2, 3 in the previous
section), we get a simple expression for $\gd T$ 
\bqbl
&&
\gd T
=
\gd R_0 j_\parallel
-{V_0\over L_0}\gd P
-{(V\gx)'_{a0}\over L_0}P
+{V_0(L\gx)'_{a0}\over L^2_0}P
.\eqbl

The boundary conditions for the GSh equation reduces to the requirement
that the plasma-vacuum interface is a magnetic surface. When this
condition is translated into a specific form, many formulations are
possible. E.g., for a fixed (or prescribed) plasma boundary, the
geometry of the plasma-vacuum interface can be given through the Fourier
coefficients
\bqbl
&&
r^c|_{a=a_b}
=r^c_b
,\quad
r^s|_{a=a_b}
=r^s_b
,\quad
z^c|_{a=a_b}
=z^c_b
,\quad
z^s|_{a=a_b}
=z^s_b
.\eqbl
Here, $a_b$ is the boundary value of $a$. Another way to describe the
plasma edge is to specify a number of characteristic points
\bqbl
&&
r(a_b,\gt_i)
=
r_i
,\quad
z(a_b,\gt_i)
=
z_i
,\quad
0\leq i\leq N
,\eqbl
which gives equations for $\gt_i$ and Fourier coefficients at the plasma
boundary. 

Free boundary problems have wide variety of forms for the boundary
conditions. We mention here only one, which corresponds to a given
external field from the poloidal equilibrium system
\bqbl
&&
\left[\gY_{ext}(r,z)
+\gY_{pl}(r,z)\right]_{plasma-vacuum}
=
const
\lbq{BC}
,\eqbl
where $\gY_{ext}$ is the field of the external coils (calculated in
straightforward manner) and $\gY_{pl}$ is the proper field of the plasma
current. Typicaly, $\gY_{ext}$ is known through in terms of Green function
(as is described in Ref.\cite{Lackner76}). On the perturbed plasma
boundary 
\bqbl
&&
a
=
a_b+\gx_b
,\quad
r=r_b+r'_a\gx_b
,\quad
z=z_b+z'_a\gx_b
,\eqbl
in linear approximation, the external flux is equal to
\bqbl
&&
\gY_{ext}
=
\gY_{ext}(r(a_b,\gt),z(a_b,\gt))
+\gY'_{r,ext}r'_a\gx_b
+\gY'_{z,ext}z'_a\gx_b
.\eqbl
The poloidal flux of the plasma current can be represented in terms of
the contour integral as it was suggested in Ref.\cite{Zakharov72}
\bqbl
&&
\gY_{pl}
=
\r{\gm_0}\int^{2\gp}_0\gY_0K_0\gY'd\gt
,\eqbl
where $\gY_0(r,z;r_1,z_1)$ is the Green function for the GSh equation,
$r,z$ is the point where $\gY_{pl}$ is calculated, while $r_1,z_1$ are
coordinates of intergration point. All quantities here should be
calculated at the perturbed boundary. In linear approximation,
\bqbl
\gY_{pl}
=&&
\R{\gY'^0}{\gm_0}\int^{2\gp}_0\gY_0(r_b,z_b;r_{1b},z_{1b})K_0d\gt
+\R{\gY'^0}{\gm_0}\int^{2\gp}_0\gY'_{0,r}r'_a\gx_bK_0d\gt
\\&&
+\R{\gY'^0}{\gm_0}\int^{2\gp}_0\gY'_{0,z}z'_a\gx_bK_0d\gt
+\R{\gY'^0}{\gm_0}\int^{2\gp}_0\gY'_{0,r1}r'_a\gx_bK_0d\gt
+\R{\gY'^0}{\gm_0}\int^{2\gp}_0\gY'_{0,z1}z'_a\gx_bK_0d\gt
\\&&
+\R{\gY'^0}{\gm_0}\int^{2\gp}_0\gY_0\gd K_0d\gt
-\R{\gY'^0}{\gm_0}\int^{2\gp}_0\gY_0K_0\gx'_{b,a}d\gt
,\eqbl
where $\gd K_0$ is given by Eq.~\rfq{K}.

Boundary condition \rfq{BC} with this expression for the plasma proper
field gives an intergral equation for the displacement of the boundary
$\gx_b$ (which in Fourier space reduces to the matrix equation for
Fourier harmonics of $gx_b$).

The perturbed Grad-Shafranov equation (\rfq{GrSh}) or (\rfq{GrShc})
together with Eq.(\rfq{gY_til}) 
%and one of (\rfq{gx0a})-(\rfq{gx0c})
determine the flux function $\bgY$ and the displacement of coordinates
$\gx$, given boundary conditions and three additional constraints
specifying $\gx_0$, $\gd P$, and $\gd T$. The coordinate system can
then be advanced explicitly. This process can be repeated until
sufficient accuracy is reached. In this form, it constitutes the Newton
method for solving nonlinear equilibrium equations. 

\Section{Advancing the coordinate system}
The perturbed equilibrium equation is written in terms of the unknown poloidal
flux $\bgY=\bgY^0+\gy$ and the radial plasma displacement $\gx$. The
relation between $\gy$ and $\gx$ is algebraic, given by (\rfq{gY_til})
and either one of (\rfq{gx0a})-(\rfq{gx0c}). Once the solution is
obtained, it should be 
used for advancing the coordinate system toward the flux
coordinates. A straigtforward algorithm (\rfq{rz2rzA}) for advancing the
coordinates does not preserve the specific choice of the poloidal angle,
e.g. (\rfq{polangl}). A more flexible scheme containing both the radial
$\gx$ and tangential $\gs$ displacements 
\bqbl
&&
r
\to
r+r'_a\gx+r'_\gt\gs
,\quad
z
\to
z+z'_a\gx+z'_\gt\gs
\eqbl
must be applied.
In linear approximation the tangential displacements is arbitrary and,
thus, can be used to satisfy constraints on the poloidal angle in the
coordinate representation.

For the poloidal angle determined by Eq.~(\rfq{polangl}), the tangential
displacement should satisfy equation 
\bqbl
&&
\gd z
=
\gd z_0
+\gd b\sin\gt
=
z'_a\gx+b\gs\cos\gt
,\eqbl
in order to preserve the specific form of Fourier expansion of the
$z$-coordinate. Here $\gd z_0$, $\gd b$ are correction to its Fourier
coefficients, which can be determined from
\bqbl
&&
\gd z_0
+\gd b
=
(z'_0+b')\gx|_{\gt=\gp/2}
,\quad
\gd z_0
-\gd b
=
(z'_0-b')\gx|_{\gt=-\gp/2}
.\eqbl
This leads to the following equation for the perturbation $\gd r$
\bqbl
&&
b\cos\gt\gd r
=
D\gx
+r'_\gt(\gd z_0+\gd b\sin\gt)
,\eqbl
which can be easily solved in Fourier space for the Fourier coefficients
of $\gd r$. 

Note, that for the coordinate system, which is defined by
Eqs.~(\rfq{polangl}), the vertical semiaxis $b$ can be chosen as a
radial coordinate $a$ with a simple geometrical interpretation. In this
case, $\gx_0$ is determined from the condition $\gd b=0$
\bqbl
&&
\gx_0
=
-\R{z'_0}{2b'}\left(\gx_\til|_{\gt=\gp/2}-\gx_\til|_{\gt=-\gp/2}\right)
-\r{2}\left(\gx_\til|_{\gt=\gp/2}+\gx_\til|_{\gt=-\gp/2}\right)
.\eqbl

Explicit coordinate advancing algorithm is consistent with the Newton
approach for solving the entire equilibrium problem. 

\Section{Perturbed equilibria and ideal energy principle}
The perturbed equilibrium equations, written in the previous section, cover
arbitrary (axisymmetric) perturbations. As a particular case, they
should reproduce the marginally stable perturbations of the ideal
stability theory if the condition of frozen magnetic field is imposed as
the constraint. Indeed, it is easy to see how this happens.

The local adiabatic equation of state, used in the ideal stability
theory, can be expressed in our notation as 
\bqbl
&&
p(rD)^{\gg_0}
=
pV^{\gg_0}_0
=
const
,\eqbl
where $\gg_0$ is the adiabatic constant. Based on this expression,
the non-convective change in the pressure $\gd\bsp$ can be obtained as 
\bqbl
&&
\gd\bsp
=
-\gg_0\bsp Y_0
,\quad
Y_0
\equiv
{(V\gx)'_0\over V_0}
,\eqbl
which leads to the following expression for $\gd P$
\bqbl
&&
\gd P
=
{\gd\bsp'\over\bgY'}
=
-{(\gg_0\bsp Y_0)'\over\bgY'}
.\eqbl
The non-convective perturbation of $\baF$ is determined by the frozen
magnetic field condition. Assuming, that the radial coordinate is frozen
into the toroidal flux $\gd\bgF=0$
\bqbl
&&
\gd\bgF'
=
\gd(L_0\baF)
=
0
,\eqbl
the perturbation $\gd\baF$ can be found in the form
\bqbl
&&
L_0\gd\baF
=
-\baF(L\gx)'_0
,\quad
\gd\baF^2
=
2\baF\gd\baF
=
-2\baF^2X_0
,\quad
X_0
\equiv
{(L\gx)'_0\over L_0}
,\eqbl
which leads to the following expression for $\gd T$
\bqbl
&&
\gd T
\equiv
{(\gd\baF^2)'\over2\bgY'}
=
-{(\baF^2X_0)'\over\bgY'}
.\eqbl
With these $\gd P$ and $\gd T$ in the RHS, the perturbed equilibrium
equation (\rfq{GrSh}), when applied to an equilibrium configuration,
becomes identical to the Euler equation, which can be obtained from the
energy principle for (marginal, $\gg=0$) stability of the ideal
axisymmetric modes \cite{Chu78,Haney97}, i.e., 
\bqbl
&&
\gD^*\gy
=
r^2{(\gg_0\bsp Y_0)'\over\bgY'}
+{(\baF^2X_0)'\over\bgY'}
+r^2P'\gx
+T'\gx
.\eqbl
For ideal plasmas, the displacement of coordinates $\gx$ coincides
with the radial (contravariant) component of the plasma displacement in
stability theory. 

The frozen condition of the magnetic flux is a natural constraint for the fast
evolution of equilibrium, e.g., when the equilibrium control is lost and
the plasma motion is determined by the eddy currents in the resistive
surroundings. Adiabaticity of the pressure is not well justified for
those problems. In fact, in experiments, pressure losses may occur
on a much faster time scale. For this reason, a more simple, prescribed
pressure model as described in the previous section seems to be more
adequate. While conventional ideal stability theory is not
applicable to this case, the perturbed equilibrium theory can describe
such a situation. 

The fact that the ideal marginal stability condition can be reproduced
by the perturbation theory, allows to combine in a natural way both
equilibrium and axisymmetric stability problems in the same numerical
code. In the conventional practice, these two problems are, typically,
treated as separate. 

\Section{Summary}
The theory of perturbed equilibrium gives a rigorous algorithm for
solving different equilibrium problems in tokamaks.

Its use leads to fast numerical codes, which find application in
transport simulations where the magnetic configuration evolves
gradually. In these circumstances, the Newton scheme is most effective
and requires, typically, only two iterations. In high beta cases, the
Newton scheme eliminates oscillations, which the simple iterations 
scheme may exhibit.

One of such a code, ESC (standing for Equilibrium and Stability Code),
has been developed in the Princeton Plasma Physics Laboratory in 1994
and has been intensively used for treatment of the Tokamak Fusion Test
Reactor (TFTR) data (see, for example,
Ref.\cite{Meade91,Zakharov96}). It is also 
applicable for low aspect ratio equilibria. Fig.~1 shows the
effectiveness of the coordinate system (\rfq{polangl}). The analytical
Soloviev solution for the National Spherical Torus Experiment
(NSTX) aspect ratio is reproduced accurately with only
5 harmonics (intensive equilibrium and stability studies of NSTX with
other codes are presented in Ref.\cite{Menard97}). Flexibility of the ESC
code in accepting different input profiles facilitates its interface and
comparison with other codes. Figs.~2,3 shows the comparison of ESC with
VMEC \cite{Lao81} and JSOLVER \cite{DeLucia80} for NSTX equilibrium with
$\gb=8\ \%$ and $\gb=16\ \%$. While VMEC shows small deviations form
ESC, the accuracy of JSOLVER (with a fixed number of nodes 65x65,
suitable for use in the transport codes) gradually deteriorates with
increasing pressure. High-$\gb$ equilibrium with $\gb=45\ \%$ on Fig.~4
was calculated by ESC. More details of ESC numerical technique will be
published separately. 

Although the present theory has been developed for axisymmetric
configuration, its ideas are also applicable to 3-D MHD equilibrium
codes, where the issue of numerical efficiency becomes critical. 

\Section*{Acknowledgments}

This work was supported by the U.S.\ Department of Energy
Contract No.\ DE-AC02-76-CHO-3073.

\bibliography{stab}
\bibliographystyle{prsty}

\newpage
\begin{figure}
\begin{tabular}{ccc}
\psfig{figure=/u/wrk/Esc/Doc/f1a.ps,%
bbllx=1in,bblly=1in,bburx=7.5in,bbury=10in,width=1.8in}
&\psfig{figure=/u/wrk/Esc/Doc/f1b.ps,%
bbllx=1in,bblly=1in,bburx=7.5in,bbury=10in,width=1.8in}
&\psfig{figure=/u/wrk/Esc/Doc/f1c.ps,%
bbllx=1in,bblly=1in,bburx=7.5in,bbury=10in,width=1.8in}
\end{tabular}
\caption{Comparison of a small aspect ratio Soloviev equilibrium
with an ESC, 5 harmonics calculation. a) Magnetic surfaces for
equidistant $a=\q{\bgF/\bgF_b}$, b) $p$ - pressure profile. Numerical
and analytical solutions are indistinguishable. c) $q$ - profile as
functions of $a$.} 
\end{figure}

\begin{figure}
\begin{tabular}{ccc}
\psfig{figure=/u/wrk/Esc/Doc/f2a.ps,%
bbllx=1in,bblly=1in,bburx=7.5in,bbury=10in,width=1.8in}
&\psfig{figure=/u/wrk/Esc/Doc/f2b.ps,%
bbllx=1in,bblly=1in,bburx=7.5in,bbury=10in,width=1.8in}
&\psfig{figure=/u/wrk/Esc/Doc/f2c.ps,%
bbllx=1in,bblly=1in,bburx=7.5in,bbury=10in,width=1.8in}
\end{tabular}
\caption{Comparison of ESC calculations with the VMEC and JSOLVER
codes. 
a) NSTX equilibrium configuration with
$\gb=2\gm_0\left<p\right>/B^2_{ext}= 8\ \%$ ($\left<p\right>$ is the
volume-averaged pressure, $B_{ext}=1\ T$ is the vacuum toroidal magnetic
field at the middle of the cross-section). Magnetic surfaces are
shown for equidistant $a=\q{\bgF/\bgF_b}$ b) ESC and VMEC magnetic
surfaces are shown for equidistant $\bgF/\bgF_b$; c) ESC and JSOLVER
magnetic surfaces are shown for equidistant $\bgY/\bgY_b$.
}
\end{figure}

\begin{figure}
\begin{tabular}{ccc}
\psfig{figure=/u/wrk/Esc/Doc/f3a.ps,%
bbllx=1in,bblly=1in,bburx=7.5in,bbury=10in,width=1.8in}
&\psfig{figure=/u/wrk/Esc/Doc/f3b.ps,%
bbllx=1in,bblly=1in,bburx=7.5in,bbury=10in,width=1.8in}
&\psfig{figure=/u/wrk/Esc/Doc/f3c.ps,%
bbllx=1in,bblly=1in,bburx=7.5in,bbury=10in,width=1.8in}
\end{tabular}
\caption{Same comparison as in Fig.~2 but for $\gb=16\ \%$.
}
\end{figure}

\begin{figure}
\vskip12pt
\begin{tabular}{ccc}
\psfig{figure=/u/wrk/Esc/Doc/f4a.ps,%
bbllx=1in,bblly=1in,bburx=7.5in,bbury=10in,width=1.8in}
&\psfig{figure=/u/wrk/Esc/Doc/f4b.ps,%
bbllx=1in,bblly=1in,bburx=7.5in,bbury=10in,width=1.8in}
&\psfig{figure=/u/wrk/Esc/Doc/f4c.ps,%
bbllx=1in,bblly=1in,bburx=7.5in,bbury=10in,width=1.8in}
\end{tabular}
\caption{a) Extreme NSTX magnetic configuration with $\gb=45 \%$; b)
pressure and c) $q$-profiles as functions of $\q{\bgF/\bgF_b}$ for
Fig.~2,3,4a. 
}
\end{figure}
%-----------------------------------------------------------------------------
\newpage
Figure captions:

Fig.~1. Comparison of a small aspect ratio Soloviev equilibrium with
ESC (5 harmonics) calculations. a) Magnetic surfaces for equidistant
$a=\q{\bgF/\bgF_b}$, b) $p$ - pressure profile. Numerical and analytical
solutions are indistinguishable. c) $q$ - profile as
functions of $a$. 

\vskip12pt

Fig.~2. Comparison of ESC calculation with VMEC and JSOLVER codes.
a) NSTX equilibrium configuration with
$\gb=2\gm_0\left<p\right>/B^2_{ext}= 8\ \%$ ($\left<p\right>$ is the
volume-averaged pressure, $B_{ext}=1 T$ is the vacuum toroidal magnetic
field at the middle of the cross-section). Magnetic surfaces are
shown for equidistant $a=\q{\bgF/\bgF_b}$ b) ESC and VMEC magnetic
surfaces are shown for equidistant $\bgF/\bgF_b$; c) ESC and JSOLVER
magnetic surfaces are shown for equidistant $\bgY/\bgY_b$; 

Fig.~3. Same comparison for $\gb=16\ \%$.

Fig.~4. a) Extreme NSTX magnetic configuration for $\gb=45 \%$; b)
pressure and c) $q$-profiles as functions of $\q{\bgF/\bgF_b}$ for
Fig.~2,3,4a. 

\Section{Energy principle of axisymmetric ideal MHD}
Functional of potential energy.
\bqbl
&&
W={1\over8\gp}\int\left\{
\tilde{\waB}^2-\vgx\times(\na\times\waB)\cdot\tilde{\waB}
+0.4\gp(\vgx\cdot\na p)(\na\cdot\vgx)
+0.4\gp\gg_0p(\na\cdot\vgx)^2
\right\}dV
.\eqbl
Displacement vector and the perturbed magnetic field.
\bqbl
&&
\gx^i\equiv
\left\{\gx^a,\gx^\gt,\gx^\phi\right\}=
\left\{\gx,\gx^\gt,\gx^\gf\right\}
,\quad
\tilde{\bf B}=\na\times\left(\vec{\gx}\times{\bf B}\right)
,\quad
X\equiv\bgF'(1+\gh'_\gt)\gx^\gt+\bgY'\gx^\gf
,\\&&
\left(\vec{\gx}\times{\bf B}\right)_i=
 rD\left\{
B^3\gx^\gt-B^2\gx^\gf
, -B^3\gx
,B^2\gx
\right\}=
\left\{
X
,-\bgF'(1+\gh'_\gt)\gx
,-\bgY'\gx
\right\}
,\\&&
\tilde{B}^i=
{1\over rD}\left\{
-\bgY'\gx'_\gt
,(\bgY'\gx)'_a
,-X'_\gt-\left({D\over r}\baF\gx\right)'_a
\right\}
.\eqbl
Contribution of $\tilde{\waB}^2$
\bqbl
{1\over8\gp}\int
\tilde{\waB}^2dV&=&
{1\over8\gp}\int\left\{
{g_{aa}\over rD}\gy'^2_\gt
-2{g_{a\gt}\over rD}\gy'_a\gy'_\gt
+{g_{\gt\gt}\over rD}\gy'^2_a
+{r\over D}\left[X'_\gt+\left({D\over r}\baF\gx\right)'_a\right]^2
\right\}da d\gt d\gf
.\eqbl
Contribution of $\vgx\times(\na\times{\waB})\cdot\tilde{\waB}$
\bqbl
&&
0.4\gp\left(\vgx\times{\wsj}\right)_i=
\left\{
rDP\gx^\gt+{D\over r}T\gx^\gt+\baF'\gx^\gf
,-rDP\gx-{D\over r}T\gx
,-\baF'\gx
\right\}
,\\&&
0.4\gp\left(\vgx\times{\wsj}\right)_i=
\left\{
rDP\gx^\gt
+{\baF'\over\bgY'}X
,-rDP\gx-{D\over r}T\gx
,-\baF'\gx
\right\}
,\\&&
{1\over8\gp}\int\left\{
-0.4\gp\vgx\times{\wsj}\cdot\tilde{\waB}
\right\}dV=
{1\over8\gp}\int\left\{
rDP\gx^\gt\bgY'\gx'_\gt
\right.\\&&\left.
+\baF'X\gx'_\gt
+\left(rDP+{D\over r}T\right)\gx(\bgY'\gx)'_a
-\baF'\gx X'_\gt
-\baF'\gx\left({D\over r}\baF\gx\right)'_a
\right\}da d\gt d\gf
.\eqbl
Contribution of $-0.4\gp(\vgx\times\wsj)\cdot\tilde{\waB}+0.4\gp
p'\gx(\na\cdot\vgx)$ 
\bqbl
&&
{1\over8\gp}\int\left\{
-0.4\gp\vgx\times{\wsj}\cdot\tilde{\waB}
+0.4\gp p'\gx(\na\cdot\vgx) 
\right\}dV
=
\\&&
{1\over4}\int\left\{
\bsp'_a(rD\gx)'_a\gx
+\left(rDP+{D\over r}T\right)\gx(\bgY'\gx)'_a
-2\baF'\gx X'_\gt
-\baF'\gx\left({D\over r}\baF\gx\right)'_a
\right\}da d\gt
.\eqbl
Integral of potential energy
\bqbl
W
=&&
{1\over4}\int\left\{
{g_{aa}\over rD}\gy'^2_\gt
-2{g_{a\gt}\over rD}\gy'_a\gy'_\gt
+{g_{\gt\gt}\over rD}\gy'^2_a
+{r\over D}\left[X'_\gt
 +\baF\left({D\over r}\gx\right)'_a
 \right]^2
+{\gg_0p\over rD}\left[(rD\gx^\gt)'_\gt+(rD\gx)'_a\right]^2
\right.\\&&\left.
-rDP'\bgY'\gx^2
-{D\over r}T'\bgY'\gx^2
\right\}da d\gt
.\eqbl
Expression for $X'_\gt$
\bqbl
&&
X'_\gt
+\baF\left({D\over r}\gx\right)'_a
=
{D\over r}\baF X_0
,\quad
\left({D\over r}\right)_0X_0
=
\left({D\over r}\gx\right)'_{0a}
.\eqbl
Expression for $Y'_\gt$
\bqbl
&&
Y'_\gt
+(rD\gx)'_a
=
rDY_0
,\quad
(rD)_0Y_0
=
(rD\gx)'_{0a}
.\eqbl
Euler equation
\bqbl
&&
\left(
  {g_{\gt\gt}\over rD}\gy'_a
 -{g_{a\gt}\over rD}\gy'_\gt
 \right)'_a
+\left(
  {g_{aa}\over rD}\gy'_\gt
 -{g_{a\gt}\over rD}\gy'_a
 \right)'_\gt
=
\\&&
-rD{P'\over\bgY'}\gy
-{D\over r}{T'\over\bgY'}\gy
+{D\over r\bgY'}\left(\baF{r\over D}\left[X'_\gt
 -\baF\left({D\over r\bgY'}\gy\right)'_a
 \right] \right)'_a
+{rD\over\bgY'}\left({\gg_0p\over rD}\left[Y'_\gt
 -\left({rD\over\bgY'}\gy\right)'_a\right]\right)'_a
.\eqbl
It can be rewritten as a perturbed equilibrium equation
\bqbl
&&
\left(
  {g_{aa}\over rD}\gy'_\gt
 -{g_{a\gt}\over rD}\gy'_a
 \right)'_\gt
+\left(
  {g_{\gt\gt}\over rD}\gy'_a
 -{g_{a\gt}\over rD}\gy'_\gt
 \right)'_a
=
-rD{P'\over\bgY'}\gy
-{D\over r}{T'\over\bgY'}\gy
-{D\over r}\gd T
-rD\gd P
,\\&&
\gd P
=
-{(\gg_0pY_0)'_a\over\bgY'}
,\quad
\gd T
=
-{(\baF^2X_0)'_a\over\bgY'}
.\eqbl

\Section{Grad-Shafranov equation in terms of $\gx$}

GSh equation in flux coordinates
\bqbl
&&
 \left(\R{g_{tt}}{rD}\bgY'\right)'_a
-\left(\R{g_{at}}{rD}\bgY'\right)'_t
=
D\R{R_0}{r}j_s
+D\left(\R{r}{R_0}-\R{R_0}{r}\right)j_p
.\eqbl
Perturbaton of coordinate system
\bqbl
&&
r
=
r+r'_a\gx
,\quad
z
=
z+z'_a\gx
,\quad
\gd r
=
r'_a\gx
.\eqbl
Perturbaton of $D$
\bqbl
&&
D
\equiv
r'_az'_t
-r'_tz'_a
=
(r+r'_a\gx)'_a(z+z'_a\gx)'_t
-(r+r'_a\gx)'_t(z+z'_a\gx)'_a
,\\&&
\gd D
=
(D\gx)'_a
,\quad
\gd (rD)
=
(rD\gx)'_a
.\eqbl
Perturbaton of $g_{tt}$
\bqbl
&&
g_{tt}
\equiv
r'^2_t+z'^2_t
=
(r+r'_a\gx)'^2_t+(z+z'_a\gx)'^2_t
,\quad
\gd g_{tt}
=
2r'_t(r'_a\gx'_t+r''_{at}\gx)
+2z'_t(z'_a\gx'_t+z''_{at}\gx)
=\\&&
(g_{tt})'_a\gx
+2g_{at}\gx'_t
.\eqbl

Perturbaton of $g_{at}$
\bqbl
&&
g_{at}
\equiv
r'_ar'_t+z'_az'_t
=
(r+r'_a\gx)'_a(r+r'_a\gx)'_t
+(z+z'_a\gx)'_a(z+z'_a\gx)'_t
,\\&&
\gd g_{at}
=
(g_{at})'_a\gx
+g_{aa}\gx'_t
+g_{at}\gx'_a
.\eqbl
Perturbed part of the Left Hand Side of the equilibrium equation 
\bqbl
&&
 \left(\R{g_{tt}}{rD}\bgY'\right)'_a
-\left(\R{g_{at}}{rD}\bgY'\right)'_t
=\\&&
 \left[
  -\R{g_{tt}}{rD}\bgY'\gx'_a
  +\left(\R{g_{tt}}{rD}\bgY'\right)'_a\gx
  +\R{2g_{at}}{rD}\bgY'\gx'_t
 \right]'_a
-\left[
   \R{g_{aa}}{rD}\bgY'\gx'_t
  +\left(\R{g_{at}}{rD}\bgY'\right)'_a\gx
 \right]'_t
,\\&&
LHS
=
 \left[
  -\R{g_{tt}}{rD}\bgY'\gx'_a
  +\left(\R{g_{tt}}{rD}\bgY'\right)'_a\gx
  +\R{2g_{at}}{rD}\bgY'\gx'_t
 \right]'_a
-\left[
   \R{g_{aa}}{rD}\bgY'\gx'_t
  +\left(\R{g_{at}}{rD}\bgY'\right)'_a\gx
 \right]'_t
.\eqbl
#endif

#ifndef mzl_System_escD.c /* --------------- Use of Variables --------{ */
#ifdef Tbl_UnusedID/* ------------- List of Unused variables ------{ */
  int ESShMem;			/*          <Cntr:esc<Extrn:escD.c */
  void *ESlShMem;		/*          <Cntr:esc<Extrn:escD.c */
  int *ESShMemI;		/*          <Cntr:esc<Extrn:escD.c */
  double *ESShMemT;		/*          <Cntr:esc<Extrn:escD.c */
  double *ESShMemB;		/*          <Cntr:esc<Extrn:escD.c */
  double *ESShMemP;		/*          <Cntr:esc<Extrn:escD.c */
  double *ESShMemE;		/*          <Cntr:esc<Extrn:escD.c */
  unsigned int ESmemlev;	/*          <Cntr:esc<Extrn:escD.c */
  unsigned int ESmemlevP;	/*          <Cntr:esc<Extrn:escD.c */
  int ESEqSolvFl;		/*          <Cntr:esc<Extrn:escD.c */
  int ESEqSolvRadC;		/*          <Cntr:esc<Extrn:escD.c */
  int ESEqSolvInPr;		/*          <Cntr:esc<Extrn:escD.c */
  int ESEqSolvInCr;		/*          <Cntr:esc<Extrn:escD.c */
  int ESEqSolvInRadC;		/*          <Cntr:esc<Extrn:escD.c */
  int FlagPoints;		/*          <Cntr:esc<Extrn:escD.c */
  double ESEqSolvTol;		/*          <Cntr:esc<Extrn:escD.c */
  double ESEqTol;		/*          <Cntr:esc<Extrn:escD.c */
  int ESFail;			/*          <Cntr:esc<Extrn:escD.c */
  int ESEqSolvIt;		/*          <Cntr:esc<Extrn:escD.c */
  int ESEqIt;			/*          <Cntr:esc<Extrn:escD.c */
  int ESRecoveryFlag;		/*          <Cntr:esc<Extrn:escD.c */
  int ESkNormJ;			/*          <Cntr:esc<Extrn:escD.c */
  int ESkNormP;			/*          <Cntr:esc<Extrn:escD.c */
  int VMFlag;			/*          <Cntr:esc<Extrn:escD.c */
  int ESNMSE;			/*          <Cntr:esc<Extrn:escD.c */
  double ESrMSE[];		/*          <Cntr:esc<Extrn:escD.c */
  double ESgiMSE[];		/*          <Cntr:esc<Extrn:escD.c */
  double ESgaG[];		/*          <Cntr:esc<Extrn:escD.c */
  double ESgaP[];		/*          <Cntr:esc<Extrn:escD.c */
  double ESgaC[];		/*          <Cntr:esc<Extrn:escD.c */
  char ESMachineNm[];		/*          <Cntr:esc<Extrn:escD.c */
  char ESRunID[];		/*          <Cntr:esc<Extrn:escD.c */
  char ESMessage[];		/*          <Cntr:esc<Extrn:escD.c */
  double ESTime;		/*          <Cntr:esc<Extrn:escD.c */
  int ESLt;			/*        <3Dcore:esc<Extrn:escD.c */
  int ESNt;			/*        <3Dcore:esc<Extrn:escD.c */
  int ESNt1;			/*        <3Dcore:esc<Extrn:escD.c */
  int ESnAT;			/*        <3Dcore:esc<Extrn:escD.c */
  int ESnAPT;			/*        <3Dcore:esc<Extrn:escD.c */
  int ESLt0;			/*        <3Dcore:esc<Extrn:escD.c */
  int ESNLt;			/*        <3Dcore:esc<Extrn:escD.c */
  int ESNLt1;			/*        <3Dcore:esc<Extrn:escD.c */
  double *ESaR0;		/*        <3Dcore:esc<Extrn:escD.c */
  double *ESaZ0;		/*        <3Dcore:esc<Extrn:escD.c */
  double *ESsb;			/*        <3Dcore:esc<Extrn:escD.c */
  double *ESsb1a;		/*        <3Dcore:esc<Extrn:escD.c */
  double *ESsb2a;		/*        <3Dcore:esc<Extrn:escD.c */
  double *rT1a;			/*        <3Dcore:esc<Extrn:escD.c */
  double *zT1a;			/*        <3Dcore:esc<Extrn:escD.c */
  double *rT1gt;		/*        <3Dcore:esc<Extrn:escD.c */
  double *zT1gt;		/*        <3Dcore:esc<Extrn:escD.c */
  double *ESsr;			/*        <3Dcore:esc<Extrn:escD.c */
  double *ESsra;		/*        <3Dcore:esc<Extrn:escD.c */
  double *ESsrt;		/*        <3Dcore:esc<Extrn:escD.c */
  double *ESsrat;		/*        <3Dcore:esc<Extrn:escD.c */
  double *ESsz;			/*        <3Dcore:esc<Extrn:escD.c */
  double *ESsza;		/*        <3Dcore:esc<Extrn:escD.c */
  double *ESszt;		/*        <3Dcore:esc<Extrn:escD.c */
  double *ESszat;		/*        <3Dcore:esc<Extrn:escD.c */
  double *ESaB;			/*        <3Dcore:esc<Extrn:escD.c */
  double *ESaBa;		/*        <3Dcore:esc<Extrn:escD.c */
  double *ESaBt;		/*        <3Dcore:esc<Extrn:escD.c */
  double *ESaBat;		/*        <3Dcore:esc<Extrn:escD.c */
  double *ESgH;			/*        <3Dcore:esc<Extrn:escD.c */
  double *ESgHa;		/*        <3Dcore:esc<Extrn:escD.c */
  double *ESgHt;		/*        <3Dcore:esc<Extrn:escD.c */
  double *ESgHat;		/*        <3Dcore:esc<Extrn:escD.c */
  double *gf1T;			/*        <3Dcore:esc<Extrn:escD.c */
  double *csLT;			/*        <3Dcore:esc<Extrn:escD.c */
  double *snLT;			/*        <3Dcore:esc<Extrn:escD.c */
  double *cs1T;			/*        <3Dcore:esc<Extrn:escD.c */
  double *sn1T;			/*        <3Dcore:esc<Extrn:escD.c */
  double *rcT;			/*        <3Dcore:esc<Extrn:escD.c */
  double *rcT1a;		/*        <3Dcore:esc<Extrn:escD.c */
  double *rcT2a;		/*        <3Dcore:esc<Extrn:escD.c */
  double *rcT1t;		/*        <3Dcore:esc<Extrn:escD.c */
  double *rsT;			/*        <3Dcore:esc<Extrn:escD.c */
  double *rsT1a;		/*        <3Dcore:esc<Extrn:escD.c */
  double *rsT2a;		/*        <3Dcore:esc<Extrn:escD.c */
  double *rsT1t;		/*        <3Dcore:esc<Extrn:escD.c */
  double *dR0T;			/*        <3Dcore:esc<Extrn:escD.c */
  double *dZ0T;			/*        <3Dcore:esc<Extrn:escD.c */
  double *dbT;			/*        <3Dcore:esc<Extrn:escD.c */
  double *dbT1a;		/*        <3Dcore:esc<Extrn:escD.c */
  double *dbT2a;		/*        <3Dcore:esc<Extrn:escD.c */
  double *drcT;			/*        <3Dcore:esc<Extrn:escD.c */
  double *drcT1a;		/*        <3Dcore:esc<Extrn:escD.c */
  double *drcT2a;		/*        <3Dcore:esc<Extrn:escD.c */
  double *drsT;			/*        <3Dcore:esc<Extrn:escD.c */
  double *drsT1a;		/*        <3Dcore:esc<Extrn:escD.c */
  double *drsT2a;		/*        <3Dcore:esc<Extrn:escD.c */
  int ESNp;			/*        <2Dcore:esc<Extrn:escD.c */
  int ESNp1;			/*        <2Dcore:esc<Extrn:escD.c */
  int ESnAP;			/*        <2Dcore:esc<Extrn:escD.c */
  int ESnAF;			/*        <2Dcore:esc<Extrn:escD.c */
  int ESFp;			/*        <2Dcore:esc<Extrn:escD.c */
  int ESFp1;			/*        <2Dcore:esc<Extrn:escD.c */
  int *ESnF;			/*        <2Dcore:esc<Extrn:escD.c */
  int *ESmF;			/*        <2Dcore:esc<Extrn:escD.c */
  int *ESkF;			/*        <2Dcore:esc<Extrn:escD.c */
  int ESNf;			/*        <2Dcore:esc<Extrn:escD.c */
  int ESNf1;			/*        <2Dcore:esc<Extrn:escD.c */
  int ESMp;			/*        <2Dcore:esc<Extrn:escD.c */
  int ESMp1;			/*        <2Dcore:esc<Extrn:escD.c */
  int ES2Mp;			/*        <2Dcore:esc<Extrn:escD.c */
  int ES2Mp1;			/*        <2Dcore:esc<Extrn:escD.c */
  int ESnMp;			/*        <2Dcore:esc<Extrn:escD.c */
  double R0;			/*        <2Dcore:esc<Extrn:escD.c */
  double Z0;			/*        <2Dcore:esc<Extrn:escD.c */
  double *ECr;			/*        <2Dcore:esc<Extrn:escD.c */
  double *ECz;			/*        <2Dcore:esc<Extrn:escD.c */
  double *vr;			/*        <2Dcore:esc<Extrn:escD.c */
  double *vz;			/*        <2Dcore:esc<Extrn:escD.c */
  double *dra;			/*        <2Dcore:esc<Extrn:escD.c */
  double *drgt;			/*        <2Dcore:esc<Extrn:escD.c */
  double *dza;			/*        <2Dcore:esc<Extrn:escD.c */
  double *dzgt;			/*        <2Dcore:esc<Extrn:escD.c */
  double *ESgh;			/*        <2Dcore:esc<Extrn:escD.c */
  double *ESgt;			/*        <2Dcore:esc<Extrn:escD.c */
  double *EScs1;		/*        <2Dcore:esc<Extrn:escD.c */
  double *ESsn1;		/*        <2Dcore:esc<Extrn:escD.c */
  double *cs2;			/*        <2Dcore:esc<Extrn:escD.c */
  double *sn2;			/*        <2Dcore:esc<Extrn:escD.c */
  double ESaRx;			/*        <2Dcore:esc<Extrn:escD.c */
  double ESaZx;			/*        <2Dcore:esc<Extrn:escD.c */
  int ESNsep;			/*        <2Dcore:esc<Extrn:escD.c */
  double ESRsep[];		/*        <2Dcore:esc<Extrn:escD.c */
  double ESZsep[];		/*        <2Dcore:esc<Extrn:escD.c */
  int ESiAx;			/*        <2Dcore:esc<Extrn:escD.c */
  int ESnBL;			/*        <2Dcore:esc<Extrn:escD.c */
  int ESnBL1;			/*        <2Dcore:esc<Extrn:escD.c */
  int ESNax;			/*        <2Dcore:esc<Extrn:escD.c */
  int ESNax1;			/*        <2Dcore:esc<Extrn:escD.c */
  double ESLx[];		/*        <2Dcore:esc<Extrn:escD.c */
  double ESVx[];		/*        <2Dcore:esc<Extrn:escD.c */
  double ESDx[];		/*        <2Dcore:esc<Extrn:escD.c */
  double ESgFPlVx;		/*        <2Dcore:esc<Extrn:escD.c */
  double EcRjet[];		/*        <2Dcore:esc<Extrn:escD.c */
  double EcZjet[];		/*        <2Dcore:esc<Extrn:escD.c */
  int EcNjet;			/*        <2Dcore:esc<Extrn:escD.c */
  int ESNa;			/*        <1Dcore:esc<Extrn:escD.c */
  int ESNa1;			/*        <1Dcore:esc<Extrn:escD.c */
  double *ESsa;			/*        <1Dcore:esc<Extrn:escD.c */
  double *ESqa;			/*        <1Dcore:esc<Extrn:escD.c */
  double *ESpa;			/*        <1Dcore:esc<Extrn:escD.c */
  double *ESp3a;		/*        <1Dcore:esc<Extrn:escD.c */
  double *ECb;			/*        <1Dcore:esc<Extrn:escD.c */
  double *ECb1a;		/*        <1Dcore:esc<Extrn:escD.c */
  double *ECb2a;		/*        <1Dcore:esc<Extrn:escD.c */
  double ESa0;			/*        <1Dcore:esc<Extrn:escD.c */
  double ESav;			/*        <1Dcore:esc<Extrn:escD.c */
  int ESFlHole;			/*        <1Dcore:esc<Extrn:escD.c */
  int ESFlVirtualB;		/*        <1Dcore:esc<Extrn:escD.c */
  int ESiIv;			/*        <1Dcore:esc<Extrn:escD.c */
  int ESBNa;			/*        <1Dcore:esc<Extrn:escD.c */
  int ESBNa1;			/*        <1Dcore:esc<Extrn:escD.c */
  double *ESBsa;		/*        <1Dcore:esc<Extrn:escD.c */
  double *ESBpa;		/*        <1Dcore:esc<Extrn:escD.c */
  double *ESBsb;		/*        <1Dcore:esc<Extrn:escD.c */
  double *ESBsb1a;		/*        <1Dcore:esc<Extrn:escD.c */
  double *ESBsb2a;		/*        <1Dcore:esc<Extrn:escD.c */
  int TFCLt;			/*           <TFC:esc<Extrn:escD.c */
  int TFCNt;			/*           <TFC:esc<Extrn:escD.c */
  int TFCNp;			/*           <TFC:esc<Extrn:escD.c */
  int TFCNp1;			/*           <TFC:esc<Extrn:escD.c */
  double *xTFC;			/*           <TFC:esc<Extrn:escD.c */
  double *yTFC;			/*           <TFC:esc<Extrn:escD.c */
  double *zTFC;			/*           <TFC:esc<Extrn:escD.c */
  double *gfTFC;		/*           <TFC:esc<Extrn:escD.c */
  double *rTFC;			/*           <TFC:esc<Extrn:escD.c */
  double ITFC;			/*           <TFC:esc<Extrn:escD.c */
  double *EStgF;		/*        <2DPlPr:esc<Extrn:escD.c */
  double *EStgF1a;		/*        <2DPlPr:esc<Extrn:escD.c */
  double *EStgF3a;		/*        <2DPlPr:esc<Extrn:escD.c */
  double *EStgY;		/*        <2DPlPr:esc<Extrn:escD.c */
  double *EStgY1a;		/*        <2DPlPr:esc<Extrn:escD.c */
  double *EStgY3a;		/*        <2DPlPr:esc<Extrn:escD.c */
  double *EStgY1a2t;		/*        <2DPlPr:esc<Extrn:escD.c */
  PlProf PRdC;			/*        <1DPlPr:esc<Extrn:escD.c */
  PlProf PlPrP[];		/*        <1DPlPr:esc<Extrn:escD.c */
  PlProf PlCrP[];		/*        <1DPlPr:esc<Extrn:escD.c */
  RadCoord RdC[];		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgF0;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgF01a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgY0;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgY01a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgY02a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgi;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgi1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgi2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESaY;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESaY1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESaY2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESRC;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESRC1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESRC2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESqgF;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESqgF1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESqgF2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESqgY;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESqgY1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESqgY2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESsh;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESsh1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESsh2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESqV;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESqV1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESqV2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESjp;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESjp1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESjp2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESsp;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESsp1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESsp2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESPs;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESPs1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESPs2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgb;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgb1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgb2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgB;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgB1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgB2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESjs;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESjs1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESjs2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESjb;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESjb1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESjb2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESaJ;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESaJ1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESaJ2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESaT;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESaT1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESaT2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESjB;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESjB1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESjB2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESsq;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESsq1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESsq2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgm;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgm1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgm2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgY;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgY1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgY2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgF;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgF1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESgF2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESaF;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESaF1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESaF2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESFF;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESFF1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESFF2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESdgY;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESdgY1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESdgY2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESjR;			/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESjR1a;		/*        <1DPlPr:esc<Extrn:escD.c */
  double *ESjR2a;		/*        <1DPlPr:esc<Extrn:escD.c */
  int ESnDtPr;			/*        <1DPlPr:esc<Extrn:escD.c */
  int ESnDtCr;			/*        <1DPlPr:esc<Extrn:escD.c */
  double ESxDtPr[];		/*        <1DPlPr:esc<Extrn:escD.c */
  double ESxDtCr[];		/*        <1DPlPr:esc<Extrn:escD.c */
  double ESgFPlV;		/*        <PlCnst:esc<Extrn:escD.c */
  double ESgFPlV1a;		/*        <PlCnst:esc<Extrn:escD.c */
  double ESgY2a1a;		/*        <PlCnst:esc<Extrn:escD.c */
  double ESRBt;			/*        <PlCnst:esc<Extrn:escD.c */
  double ESRext;		/*        <PlCnst:esc<Extrn:escD.c */
  double ESBt;			/*        <PlCnst:esc<Extrn:escD.c */
  double ESIpl;			/*        <PlCnst:esc<Extrn:escD.c */
  double ESgimn;		/*        <PlCnst:esc<Extrn:escD.c */
  double ESgimx;		/*        <PlCnst:esc<Extrn:escD.c */
  double ESgbN;			/*        <PlCnst:esc<Extrn:escD.c */
  double ESgbext;		/*        <PlCnst:esc<Extrn:escD.c */
  double *ESemK;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESemL;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESemV;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESemW;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESemU1;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESemU2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESemK2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESemL2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESemV2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESemW2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESemU12;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESemU22;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg22c;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg22s;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg22c1;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg22s1;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg22c2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg22s2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESr22;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESr222;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESr22c;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESr22s;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESr22c2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESr22s2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg12c;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg12s;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg12c1;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg12s1;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg12c2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg12s2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg11c;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg11s;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg11c1;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg11s1;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg11c2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESg11s2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESLc;			/*         <3Dgik:esc<Extrn:escD.c */
  double *ESLs;			/*         <3Dgik:esc<Extrn:escD.c */
  double *ESLc1;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESLs1;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESLc2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESLs2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESVc;			/*         <3Dgik:esc<Extrn:escD.c */
  double *ESVs;			/*         <3Dgik:esc<Extrn:escD.c */
  double *ESVc1;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESVs1;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESVc2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESVs2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESDPc;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESDPs;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESDPc1;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESDPs1;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESDPc2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESDPs2;		/*         <3Dgik:esc<Extrn:escD.c */
  double *ESFaux;		/*           <SPL:esc<Extrn:escD.c */
  double *ESd2Faux;		/*           <SPL:esc<Extrn:escD.c */
  double *ESGaac;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGaas;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGaac2;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGaas2;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGaqc;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGaqs;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGaqc2;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGaqs2;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGqqc;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGqqs;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGqqc2;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGqqs2;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGazc;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGazs;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGazc2;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGazs2;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGqzc;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGqzs;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGqzc2;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGqzs2;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGzzc;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGzzs;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGzzc2;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESGzzs2;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESsVc;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESsVs;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESsVc2;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  double *ESsVs2;		/* <stab:MHD<stab:esc<Extrn:escD.c */
  int ESStN;			/* <stab:MHD<stab:esc<Extrn:escD.c */
  int ESStM1;			/* <stab:MHD<stab:esc<Extrn:escD.c */
  int ESStM2;			/* <stab:MHD<stab:esc<Extrn:escD.c */
  int ESStNRec;			/* <stab:MHD<stab:esc<Extrn:escD.c */
  int ESStMRec;			/* <stab:MHD<stab:esc<Extrn:escD.c */
  int ESStKRec;			/* <stab:MHD<stab:esc<Extrn:escD.c */
#endif /* }=== End of List of Unused variables ======*/
#ifndef H/* ---------- Used External Variables ------{ */
#ifdef Tbl_Extrn
#endif
#ifdef Tbl_Cntr
#endif
#ifdef Tbl_3Dcore
#endif
#ifdef Tbl_2Dcore
#endif
#ifdef Tbl_1Dcore
#endif
#ifdef Tbl_TFC
#endif
#ifdef Tbl_2DPlPr
#endif
#ifdef Tbl_1DPlPr
#endif
#ifdef Tbl_PlCnst
#endif
#ifdef Tbl_3Dgik
#endif
#ifdef Tbl_SPL
#endif
#ifdef Tbl_Tsc2
#endif
#ifdef Tbl_stab
#endif
#endif /* }======== End of Used External Variables ======= */

#endif /* }======== End of Use of Variables ======== */


#ifndef mzl_Extrn

#ifndef blb_esc
#ifdef Doc_esc
\Section{Notations in the code and documentation}
Bar is used for physical variables in rationalized units
\bqbl
\arl
\baF&\equiv&rB_{tor},&\baI&\equiv&0.2I,&\bsj&\equiv&\gm_0j\\
\bgF&\equiv&\R{\gF}{2\gp},&\bgY&\equiv&\R{\gY}{2\gp},&\bsp&\equiv&\gm_0p\\
\ear
\eqbl
and geometrical variables
\bqbl
\arl
D&\equiv&\R{D(r,z)}{D(a,\gt)},&K&\equiv&\R{g_{\gt\gt}}{arD}
,&N&\equiv&\R{g_{a\gt}}{arD},&M&\equiv&\R{g_{aa}}{arD}\\
\baD&\equiv&\R{D}{a},&\baK&\equiv&\R{K}{a}
,&\baN&\equiv&N,&\baM&\equiv&aM\\
L&\equiv&\R{DR_0}{r},&V&\equiv&D\left(\R{r}{R_0}-\R{R_0}{r}\right)
,&\baL&\equiv&\R{L}{a},&\baV&\equiv&\R{V}{a}\\
\ear
.\eqbl

Boundary values of physics variables have the subscript 'b':
\bqbl
\bgY_b
,\quad
\gm_b
.\eqbl
Virtual boundary have the subscript 'v':
\bqbl
\bgY_v\equiv\bgY(1)
,\quad
\gm_v\equiv\gm(1)
.\eqbl
#endif

#ifndef mzl_Cntr
int ESShMem;		/*  */
void *ESlShMem;		/*  */
int *ESShMemI;		/*  */
double *ESShMemT;	/*  */
double *ESShMemB;	/*  */
double *ESShMemP;	/*  */
double *ESShMemE;	/*  */
unsigned int ESmemlev;	/* Level of memeory allocation */
unsigned int ESmemlevP;	/*  */
int ESEqSolvFl;		/*  Flag for 2D equilibrium solver:
			    0x30    - variational;
			    0x00 - LSODE;
			    0x01 - Runge-Kutta;
			    */
int ESEqSolvRadC;	/*  Choice of radial coordinate:
			    0 - sb;
			    1 - qsv;
			    2 - qgF;
			    3 - qgY;
			    */
int ESEqSolvInPr;	/*  Choice of input pressure associated profile:
			    0 - bsjp;
			    1 - Ps;
			    2 - bsp;
			    */
int ESEqSolvInCr;	/*  Choice of input current associated profile:
			    0 - bsjs;
			    1 - bsjb;
			    2 - not used;
			    3 - FF';
			    4 - bsjB;
			    5 - not used;
			    6 - sq;
			    7 - gm;
			    */
int ESEqSolvInRadC;	/*  */
int FlagPoints;		/*  */
double ESEqSolvTol;	/*  Relative tolarance of 2D solver */
double ESEqTol;		/*  */
int ESFail;		/*  */
int ESEqSolvIt;		/*  Number of iterations in 2D solver */
int ESEqIt;		/*  */
int ESRecoveryFlag;	/*  */
int ESkNormJ;		/*  */
int ESkNormP;		/*  */
int VMFlag;		/*  Flag for interfacing with others EqCodes:
			    1 - VMEC
			    2 - Soloviev solution
			    */
int ESNMSE;		/*  */
double ESrMSE[];	/*  */
double ESgiMSE[];	/*  */
double ESgaG[];		/*  */
double ESgaP[];		/*  */
double ESgaC[];		/*  */
char ESMachineNm[];	/*  */
char ESRunID[];		/*  */
char ESMessage[];	/*  */
double ESTime;		/*  */
#endif

#ifndef mzl_3Dcore
int ESLt;	/*
  L- main period of the system:
  ESLt < 0 - Left-rotated;
  ESLt = 0 - axisymmetric
  ESLt < 0 - Right-rotated;
  */
int ESNt;	/* $N_t$ - Number of toroidal intervals */
int ESNt1;	/*  */
int ESnAT;	/*  */
int ESnAPT;	/*  */
int ESLt0;	/*  */
int ESNLt;	/*  */
int ESNLt1;	/*  */
double *ESaR0;	/* R0(gf) - of RMC */
double *ESaZ0;	/* Z0(gf) - of RMC */
double *ESsb;	/* sb(a,gf) - of RMC */
double *ESsb1a;	/*  */
double *ESsb2a;	/*  */
double *rT1a;	/*  */
double *zT1a;	/*  */
double *rT1gt;	/*  */
double *zT1gt;	/*  */
double *ESsr;	/* sr(a,gt,gf) - of RMC */
double *ESsra;	/* a - derivative of r for Bicubic splines */
double *ESsrt;	/* \gq - derivative of r for Bicubic splines */
double *ESsrat;	/* a,\gq - mixed derivative of r for Bicubic splines */
double *ESsz;	/* sz(a,gt,gf) - of RMC */
double *ESsza;	/* a - derivative of z for Bicubic splines */
double *ESszt;	/* \gq - derivative of z for Bicubic splines */
double *ESszat;	/* a,\gq - mixed derivative of z for Bicubic splines */
double *ESaB;	/* B - amplitude of the magnetic field for Bicubic splines */
double *ESaBa;	/* a - derivative of B for Bicubic splines */
double *ESaBt;	/* \gq - derivative of B for Bicubic splines */
double *ESaBat;	/* a,\gq - mixed derivative of B for Bicubic splines */
double *ESgH;	/* \gh - basic function for Bicubic splines */
double *ESgHa;	/* a - derivative of \gh for Bicubic splines */
double *ESgHt;	/* \gq - derivative of \gh for Bicubic splines */
double *ESgHat;	/* a,\gq - mixed derivative of \gh for Bicubic splines */
double *gf1T;	/*  */
double *csLT;	/* $\cos L\ph$ */
double *snLT;	/* $\sin L\ph$ */
double *cs1T;	/* $\cos\ph$  */
double *sn1T;	/* $\sin\ph$  */
double *rcT;	/* $r^c_k$ - cos-Fourier component of $r$ */
double *rcT1a;	/* $r'^c_k$ - cos-Fourier component of $r$ */
double *rcT2a;	/* $r''^c_k$ - cos-Fourier component of $r$ */
double *rcT1t;	/*  */
double *rsT;	/* $r^s_k$ - sin-Fourier component of $r$ */
double *rsT1a;	/* $r'^s_k$ - sin-Fourier component of $r$ */
double *rsT2a;	/* $r''^s_k$ - sin-Fourier component of $r$ */
double *rsT1t;	/*  */
double *dR0T;	/*  */
double *dZ0T;	/*  */
double *dbT;	/*  */
double *dbT1a;	/*  */
double *dbT2a;	/*  */
double *drcT;	/*  */
double *drcT1a;	/*  */
double *drcT2a;	/*  */
double *drsT;	/*  */
double *drsT1a;	/*  */
double *drsT2a;	/*  */
#endif

#ifndef mzl_2Dcore
int ESNp;	/* Number of poloidal intervals */
int ESNp1;	/*  */
int ESnAP;	/*  */
int ESnAF;	/*  */
int ESFp;	/* Number of Fourier components in RMC */
int ESFp1;	/*  */
int *ESnF;	/* Set of toriodal wave numbers */
int *ESmF;	/* Set of poliodal wave numbers */
int *ESkF;	/* Activity of wave numbers */
int ESNf;	/*  */
int ESNf1;	/*  */
int ESMp;	/* Highest poloidal wave number in gY */
int ESMp1;	/*  */
int ES2Mp;	/*  */
int ES2Mp1;	/*  */
int ESnMp;	/*  */
double R0;	/*  */
double Z0;	/*  */
double *ECr;	/*  */
double *ECz;	/*  */
double *vr;	/*  */
double *vz;	/*  */
double *dra;	/*  */
double *drgt;	/*  */
double *dza;	/*  */
double *dzgt;	/*  */
double *ESgh;	/* gh - straight filed line angle */
double *ESgt;	/* gt - poloidal angle */
double *EScs1;	/* $\cos(\gt)$ */
double *ESsn1;	/* $\sin(\gt)$ */
double *cs2;	/* cos(2gt) */
double *sn2;	/* sin(2gt) */
double ESaRx;	/* R of X-point */
double ESaZx;	/* Z of X-point */
int ESNsep;	/*  */
double ESRsep[];/*  */
double ESZsep[];/*  */
int ESiAx;	/* 0/1 - nonactive/active X-point */
int ESnBL;	/* number of radial intervals in the Separatrix layer */
int ESnBL1;	/* ESnBL+1 */
int ESNax;	/*  */
int ESNax1;	/*  */
double ESLx[];	/*  */
double ESVx[];	/*  */
double ESDx[];	/*  */
double ESgFPlVx;/*  */
double EcRjet[];/*  */
double EcZjet[];/*  */
int EcNjet;	/*  */
#endif

#ifndef mzl_1Dcore
int ESNa;		/* Number of radial intervals */
int ESNa1;		/*  */
double *ESsa;		/*  */
double *ESqa;		/*  */
double *ESpa;		/*  */
double *ESp3a;		/*  */
double *ECb;		/*  */
double *ECb1a;		/*  */
double *ECb2a;		/*  */
double ESa0;		/*  */
double ESav;		/*  */
int ESFlHole;		/* Bitwise Flag of modications related to change in 
		   the hole size:  
		   1 - interpolate or extrapolate Fourier harmonics 
		   of coordinates;
		   2 - change the radial coordinate;
		   4 - change the F-profile;
		   8 - interpolate or extrapolate js,jp-profiles;
		   16 - calculate the derivatives of coordinates at the
		   hole boundary;
		   32 - recalculate the plasma profiles;
		   */
int ESFlVirtualB;	/*  */
int ESiIv;		/*  */
int ESBNa;		/* Number of radial intervals in the boundary layer zone */
int ESBNa1;		/*  */
double *ESBsa;		/* $a$ in the boundary layer zone */
double *ESBpa;		/* $a^2$ in the boundary layer zone */
double *ESBsb;		/* $b  $ in the boundary layer zone */
double *ESBsb1a;	/* $b' $ in the boundary layer zone */
double *ESBsb2a;	/* $b''$ in the boundary layer zone */
#endif

#ifndef mzl_TFC
int TFCLt;	/*  */
int TFCNt;	/*  */
int TFCNp;	/*  */
int TFCNp1;	/*  */
double *xTFC;	/*  */
double *yTFC;	/*  */
double *zTFC;	/*  */
double *gfTFC;	/*  */
double *rTFC;	/*  */
double ITFC;	/*  */
#endif

#ifndef mzl_2DPlPr
double *EStgF;		/* tbgF(a,gf) */
double *EStgF1a;	/*  */
double *EStgF3a;	/*  */
double *EStgY;		/*  tbgY(a,gf) */
double *EStgY1a;	/*  */
double *EStgY3a;	/*  */
double *EStgY1a2t;	/*  */
#endif

#ifndef mzl_1DPlPr
PlProf PRdC;		/*  */
PlProf PlPrP[];		/*  */
PlProf PlCrP[];		/*  */
RadCoord RdC[];		/*  */
double *ESgF0;		/* bgF(a) */
double *ESgF01a;	/*  */
double *ESgY0;		/* bgY(a) */
double *ESgY01a;	/*  */
double *ESgY02a;	/*  */
double *ESgi;		/* gi(a) */
double *ESgi1a;		/*  */
double *ESgi2a;		/*  */
double *ESaY;		/* aY(a) */
double *ESaY1a;		/*  */
double *ESaY2a;		/*  */
double *ESRC;		/*  */
double *ESRC1a;		/*  */
double *ESRC2a;		/*  */
double *ESqgF;		/* normalized qgF */
double *ESqgF1a;	/*  */
double *ESqgF2a;	/*  */
double *ESqgY;		/* normalized qgY */
double *ESqgY1a;	/*  */
double *ESqgY2a;	/*  */
double *ESsh;		/* normalized vertical semiaxis */
double *ESsh1a;		/*  */
double *ESsh2a;		/*  */
double *ESqV;		/* normalized qaV */
double *ESqV1a;		/*  */
double *ESqV2a;		/*  */
double *ESjp;		/* bsjp(a)-profile */
double *ESjp1a;		/*  */
double *ESjp2a;		/*  */
double *ESsp;		/* bsp(a)-profile */
double *ESsp1a;		/*  */
double *ESsp2a;		/*  */
double *ESPs;		/* aP(a)-profile */
double *ESPs1a;		/*  */
double *ESPs2a;		/*  */
double *ESgb;		/* gbj(a)-profile */
double *ESgb1a;		/*  */
double *ESgb2a;		/*  */
double *ESgB;		/* gb(a)-profile */
double *ESgB1a;		/*  */
double *ESgB2a;		/*  */
double *ESjs;		/* bsjs(a)-profile */
double *ESjs1a;		/*  */
double *ESjs2a;		/*  */
double *ESjb;		/* bsj||(a)-profile */
double *ESjb1a;		/*  */
double *ESjb2a;		/*  */
double *ESaJ;		/* baJ(a)-profile */
double *ESaJ1a;		/*  */
double *ESaJ2a;		/*  */
double *ESaT;		/* aT(a)-profile */
double *ESaT1a;		/*  */
double *ESaT2a;		/*  */
double *ESjB;		/* bsjB(a)-profile */
double *ESjB1a;		/*  */
double *ESjB2a;		/*  */
double *ESsq;		/* sq(a)-profile */
double *ESsq1a;		/*  */
double *ESsq2a;		/*  */
double *ESgm;		/* gm(a)-profile */
double *ESgm1a;		/*  */
double *ESgm2a;		/*  */
double *ESgY;		/* bgY(a)-profile */
double *ESgY1a;		/*  */
double *ESgY2a;		/*  */
double *ESgF;		/* bgF(a)-profile */
double *ESgF1a;		/*  */
double *ESgF2a;		/*  */
double *ESaF;		/* baF(a)-profile */
double *ESaF1a;		/*  */
double *ESaF2a;		/*  */
double *ESFF;		/* baF^2(a)-profile */
double *ESFF1a;		/*  */
double *ESFF2a;		/*  */
double *ESdgY;		/*  */
double *ESdgY1a;	/*  */
double *ESdgY2a;	/*  */
double *ESjR;		/*  */
double *ESjR1a;		/*  */
double *ESjR2a;		/*  */
int ESnDtPr;		/*  */
int ESnDtCr;		/*  */
double ESxDtPr[];	/*  */
double ESxDtCr[];	/*  */
#endif

#ifndef mzl_PlCnst
double ESgFPlV;		/*  */
double ESgFPlV1a;	/*  */
double ESgY2a1a;	/*  */
double ESRBt;		/*  */
double ESRext;		/*  */
double ESBt;		/*  */
double ESIpl;		/*  */
double ESgimn;		/* gmmin */
double ESgimx;		/* gmmax */
double ESgbN;		/*  */
double ESgbext;		/*  */
#endif

#ifndef mzl_3Dgik
double *ESemK;	/*  */
double *ESemL;	/*  */
double *ESemV;	/*  */
double *ESemW;	/*  */
double *ESemU1;	/*  */
double *ESemU2;	/*  */
double *ESemK2;	/*  */
double *ESemL2;	/*  */
double *ESemV2;	/*  */
double *ESemW2;	/*  */
double *ESemU12;/*  */
double *ESemU22;/*  */
double *ESg22c;	/* \C{k}{\R{g_{\gt\gt}}{arD}} */
double *ESg22s;	/* \S{k}{\R{g_{\gt\gt}}{arD}} */
double *ESg22c1;/* \D1{\C{k}{\R{g_{\gt\gt}}{arD}}} */
double *ESg22s1;/* \D1{\S{k}{\R{g_{\gt\gt}}{arD}}} */
double *ESg22c2;/* \D2{\C{k}{\R{g_{\gt\gt}}{arD}}} */
double *ESg22s2;/* \D2{\S{k}{\R{g_{\gt\gt}}{arD}}} */
double *ESr22;	/*  */
double *ESr222;	/*  */
double *ESr22c;	/* \C{k}{\R{arD}{g_{\gt\gt}}} */
double *ESr22s;	/*  */
double *ESr22c2;/*  */
double *ESr22s2;/*  */
double *ESg12c;	/* \C{k}{\R{g_{a\gt}}{rD}} */
double *ESg12s;	/*  */
double *ESg12c1;/*  */
double *ESg12s1;/*  */
double *ESg12c2;/*  */
double *ESg12s2;/*  */
double *ESg11c;	/* \C{k}{\R{ag_{aa}}{rD}} */
double *ESg11s;	/*  */
double *ESg11c1;/*  */
double *ESg11s1;/*  */
double *ESg11c2;/*  */
double *ESg11s2;/*  */
double *ESLc;	/* \C{k}{\R{R_0D}{ar}} */
double *ESLs;	/*  */
double *ESLc1;	/*  */
double *ESLs1;	/*  */
double *ESLc2;	/*  */
double *ESLs2;	/*  */
double *ESVc;	/*  */
double *ESVs;	/*  */
double *ESVc1;	/*  */
double *ESVs1;	/*  */
double *ESVc2;	/*  */
double *ESVs2;	/*  */
double *ESDPc;	/* \C{k}{\R{D}{a}} */
double *ESDPs;	/*  */
double *ESDPc1;	/*  */
double *ESDPs1;	/*  */
double *ESDPc2;	/*  */
double *ESDPs2;	/*  */
#endif

#ifndef mzl_SPL
double *ESFaux;		/*  */
double *ESd2Faux;	/*  */
#ifdef Doc_ESd2Faux
#endif

#ifndef blb_Machine

#ifndef mzl_Tsc2

#ifndef blb_Year

#ifndef mzl_Tsc2

#ifndef blb_Shot

#ifndef mzl_Tsc2

#ifndef blb_Equil

#ifndef mzl_Tsc2
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif

#ifndef mzl_stab

#ifndef blb_MHD

#ifndef mzl_stab
double *ESGaac;	/*  */
double *ESGaas;	/*  */
double *ESGaac2;/*  */
double *ESGaas2;/*  */
double *ESGaqc;	/*  */
double *ESGaqs;	/*  */
double *ESGaqc2;/*  */
double *ESGaqs2;/*  */
double *ESGqqc;	/*  */
double *ESGqqs;	/*  */
double *ESGqqc2;/*  */
double *ESGqqs2;/*  */
double *ESGazc;	/*  */
double *ESGazs;	/*  */
double *ESGazc2;/*  */
double *ESGazs2;/*  */
double *ESGqzc;	/*  */
double *ESGqzs;	/*  */
double *ESGqzc2;/*  */
double *ESGqzs2;/*  */
double *ESGzzc;	/* \C{k}{\R{ar}{D}} */
double *ESGzzs;	/*  */
double *ESGzzc2;/*  */
double *ESGzzs2;/*  */
double *ESsVc;	/*  */
double *ESsVs;	/*  */
double *ESsVc2;	/*  */
double *ESsVs2;	/*  */
int ESStN;	/*  */
int ESStM1;	/*  */
int ESStM2;	/*  */
int ESStNRec;	/*  */
int ESStMRec;	/*  */
int ESStKRec;	/*  */
#ifdef Doc_ESStKRec
#endif
#endif
#endif
#endif
#endif
#endif
#endif

#undef Doc_Unused_Doc_Information /* ------------{ */
#ifdef Doc_Unused_Doc_Information
#ifndef mzl_1DPlPr
double ESqgFe;
/* $\q{\bgF_e} - boundary value */
#endif
#endif  /* }== Remove this section ======== */

#undef Doc_Cb_Structure
#ifdef Blb_escD.c
Blb_escD.c{
  mzl_Extrn{
    blb_esc{
      mzl_Cntr{
      }
      mzl_3Dcore{
      }
      mzl_2Dcore{
      }
      mzl_1Dcore{
      }
      mzl_TFC{
      }
      mzl_2DPlPr{
      }
      mzl_1DPlPr{
      }
      mzl_PlCnst{
      }
      mzl_3Dgik{
      }
      mzl_SPL{
        blb_Machine{
          mzl_Tsc2{
            blb_Year{
              mzl_Tsc2{
                blb_Shot{
                  mzl_Tsc2{
                    blb_Equil{
                      mzl_Tsc2{
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
      mzl_stab{
        blb_MHD{
          mzl_stab{
          }
        }
      }
    }
  }
}
#endif

#undef Doc_Table_Of_Content
#ifndef Blb_escD.c
	PS p= -1 (  0);
#ifndef mzl_Extrn
	PS p=  1 (  1);
#ifndef blb_esc
	PS p=  1 (  1);
#ifndef mzl_Cntr
	PS p=  1 (  1);
#endif
#ifndef mzl_3Dcore
	PS p=  1 (  1);
#endif
#ifndef mzl_2Dcore
	PS p=  1 (  1);
#endif
#ifndef mzl_1Dcore
	PS p=  2 (  1);
#endif
#ifndef mzl_TFC
	PS p=  2 (  1);
#endif
#ifndef mzl_2DPlPr
	PS p=  2 (  1);
#endif
#ifndef mzl_1DPlPr
	PS p=  2 (  1);
#endif
#ifndef mzl_PlCnst
	PS p=  3 (  2);
#endif
#ifndef mzl_3Dgik
	PS p=  3 (  2);
#endif
#ifndef mzl_SPL
	PS p=  3 (  2);
#ifndef blb_Machine
	PS p=  3 (  2);
#ifndef mzl_Tsc2
	PS p=  3 (  2);
#ifndef blb_Year
	PS p=  3 (  2);
#ifndef mzl_Tsc2
	PS p=  3 (  2);
#ifndef blb_Shot
	PS p=  3 (  2);
#ifndef mzl_Tsc2
	PS p=  3 (  2);
#ifndef blb_Equil
	PS p=  3 (  2);
#ifndef mzl_Tsc2
	PS p=  4 (  2);
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#ifndef mzl_stab
	PS p=  4 (  2);
#ifndef blb_MHD
	PS p=  4 (  2);
#ifndef mzl_stab
	PS p=  4 (  2);
#endif
#endif
#endif
#endif
#endif
#endif
