#ifndef Blb_escZ.c
/*  */
#ifdef Doc_escZ.c
\Section{Equilibrium Spline Interface \c{ESI}}
The inteface is designed for fast and unambigous calculations at any point 
inside the plasma of quantaties, which are necessary for transport, 
stability and quiding center orbit codes. (In future, \c{ESI} will be extended 
to its outside the plasma.)

\c{ESI} generates the values of an irreducible set of basic equilibrium 
functions, which is sufficient for calculations of other quantities without 
numerical differentiation or interpolation.

The set of basic functions (all \c{double}) includes
\vskip1ex
\begin{tabular}{|l|l|p{3in}|}\hline
\multicolumn{3}{|c|}{\bf Set of 2D output Basic functions}\\\hline
\c{C-} or \c{FORTRAN} & Math &Comment\\\hline
\c{r[n],ra[n],rq[n]}	&$r,r'_a,r'_\gq$&Arrays for major radius and its 
derivatives in [m]\\
\c{r(n),ra(n),rq(n)}	&&same for FORTRAN\\\hline
\c{z[n],za[n],zq[n]}	&$z,z'_a,z'_\gq$&Arrays for vertical coordinate and 
its derivatives in [m]\\ 
\c{z(n),za(n),zq(n)}	&&same for FORTRAN\\\hline
\c{B[n],Ba[n],Bq[n]}	&$B,B'_a,B'_\gq$&Arrays for module of $|\vaB|$ and 
its derivatives\\
\c{B(n),Ba(n),Bq(n)}	&&same for FORTRAN\\\hline
\c{gh[n],gha[n],ghq[n]}	&$\gh,\gh'_a,\gh'_\gq$&Arrays of $\gh$ function for 
transformation into straight field line coordinates\\
\c{gh(n),gha(n),ghq(n)}	&&same for FORTRAN\\\hline
\multicolumn{3}{|c|}{\bf Set of output Basic radial profiles}\\\hline
&&\\
F[n],Fa[n]&$\baF,\baF'$&$\baF\equiv rB_{tor}$ in [m T]\\
F(n),Fa(n)&$\baF,\baF'$&same for FORTRAN \\\hline
&&\\
gFa[n],gFaa[n]&$\bgF',\bgF''$& Toroidal flux $\bgF(a)\equiv\gF/(2\gp)$ in 
[Vsec]\\
gFa(n),gFaa(n)&$\bgF',\bgF''$&same for FORTRAN\\\hline
&&\\
gYa[n],gYaa[n]&$\bgY',\bgY''$&Poloidal flux $\bgY(a)\equiv\gY/(2\gp)$ in 
[Vsec], $\bgY=0$ at the plasma boundary\\
gYa(n),gYaa(n)&$$&same for FORTRAN\\\hline
&&\\
T[n],Ta[n]&$\baF\baF'/\bgY'$&Grad-Shafranov function in the RHS\\
T(n),Ta(n)&$\baF\baF'/\bgY'$&same for FORTRAN\\\hline
&&\\
P[n],Pa[n]&$\bsp'/\bgY'$&Grad-Shafranov function in the RHS, where 
$\bsp\equiv\gm_0 p$ is the "rationalized" plasma pressure. If $p$ is in [MPa], 
(or [MJ/m$^3$], or [J/cm$^3$]), then $\gm_0\equiv0.4\gp$\\
P(n),Pa(n)&$\bsp'/\bgY'$&same for FORTRAN\\
\hline\end{tabular}
\vskip1em
{\em The calling routine should provide the storage for basic functions.}

\Section{Compilation}

\c{cc -c -o escZ.o escZ.c}
 
\Section{Logic and use of ESI}

All basic functions are calculated for an array of points $a_i,\gq_i$, which 
are passed to \c{ESI} functions as parameters (together with number \c{n} of "
points). In this regard, points are considered like positions of independent 
"particles" (with no assumptions about regular grids along $a$ and $\gq$).

In order to simplify the calls of \c{ESI} functions, the first call provides 
the addresses of arrays allocated for basic function to \c{ESI} routines:

\begin{verbatim}
       call esilink2c(F,Fa,gFa,gFaa,gYa,gYaa,T,Ta,P,Pa
     \&     ,r,ra,rq
     \&     ,z,za,zq
     \&     ,B,Ba,Bq
     \&     ,gh,gha,ghq
     \&     ,isw)
\end{verbatim} 

Then, all calls of other routines put the data into appropriate arrays from 
this list. 

In the call of \c{esilink2c} a special integer array \c{isw(n)} is present 
as an argument. It is reserved in \c{ESI} for indexing "particles", which 
depends on the particular routines. E.g., in the guiding center motion, 
this array is used for distinguishing particles with "potato" kind of orbits.

For transport and stability purposes this array is not used and any interger 
can be used instead of \c{isw} in FORTRAN- or \c{NULL} in C- calls.  

%\Section{List of available \c{ESI} routines}


\Section{Example of using ESI from \c{FORTRAN}}
\begin{enumerate}
\item Linking FORTRAN arrays and \c{ESI}
\begin{verbatim}
       call esilink2c(F,Fa,gFa,gFaa,gYa,gYaa,T,Ta,P,Pa
     \&     ,r,ra,rq
     \&     ,z,za,zq
     \&     ,B,Ba,Bq
     \&     ,gh,gha,ghq
     \&     ,isw)
\end{verbatim} 
Should be called once after allocating output arrays.

Same call from C
\begin{verbatim}
 esilink2c_(F,Fa,gFa,gFaa,gYa,gYaa,T,Ta,P,Pa
           ,r,ra,rq,z,za,zq,B,Ba,Bq,gh,gha,ghq,isw)
\end{verbatim} 

\item  Generating all basic functions in $n$ points $a_i=$\c{a[i]}, 
$\gq_i=$\c{gq[i]} in FORTRAN
\begin{verbatim}
      k=esiget2dfunctions(a,gq,n)
\end{verbatim} 
and in C-code
\begin{verbatim}
      k=esiget2dfunctions_(a,gq,&n);
\end{verbatim} 
Here, \c{k} is an integer which is returned 0 if all $0 \leq a_i \leq1$.
The results are placed in the output arrays mentioned in \c{esilink2c()}.
\item Advancing guiding center motion from FORTRAN
\begin{verbatim}
        k=gcmotion(dXgr,dXa,dXgq,dXgf,Xgr,Xa,Xgq,Xgm,n)
\end{verbatim} 
or from C
\begin{verbatim}
        k=gcmotion(dXgr,dXa,dXgq,dXgf,Xgr,Xa,Xgq,Xgm,&n);
\end{verbatim} 
where \c{n} is the number of particles, \c{Xgr,Xa,Xgq,Xgm} are arrays
for $\gr_\parallel\equiv v_\parallel mc/eB$, 
$\gr_\perp\equiv v_\perp mc/eB$

This two routines 

\item Getting profiles at a specific point \c{a[i]} from FORTRAN 
\begin{verbatim}
         k=esigetprofiles(F(i),Fa(i), gFa(i),gFaa(i), gYa(i),gYaa(i)
     1        ,T(i),Ta(i), P(i),Pa(i), a(i))
\end{verbatim} 
and C
\begin{verbatim}
  k=esigetprofiles_(F+i,Fa+i, gFa+i,gFaa+i, gYa+i,gYaa+i,T+i,Ta+i
                    , P+i,Pa+i,a+i);
\end{verbatim} 
\end{enumerate}

\end{document}
#endif

#ifndef mzl_System_escZ.c /* --------------- Use of Variables --------{ */
#ifdef Tbl_UnusedID/* ------------- List of Unused variables ------{ */
  int ESISetMem();		/*                      :escZ.c */
  int ESICopy();		/*                      :escZ.c */
  int esiread_();		/*                      :escZ.c */
  int esireadrz_();		/*                      :escZ.c */
  static REAL *ap;		/*                      :escZ.c */
  static REAL *qp;		/*                      :escZ.c */
  void esilink2c_();		/*                      :escZ.c */
  void esilink2c4ne_();		/*                      :escZ.c */
  void settemperatures_();	/*                      :escZ.c */
  int esigetprofiles_();	/*                      :escZ.c */
  int esiget2dfunctions0();	/*                      :escZ.c */
  REAL q;			/* .esiget2dfunctions0():escZ.c */
  int esiget2dfunctions_();	/*                      :escZ.c */
  int esigetmagfluxes_();	/*                      :escZ.c */
  int esigetrzb_();		/*                      :escZ.c */
  int gcmotion_();		/*                      :escZ.c */
  int MFieldLines();		/*                      :escZ.c */
  void esigetpressure_();	/*                      :escZ.c */
  static double cc;		/*                      :escZ.c */
  static double cEeV;		/*                      :escZ.c */
  void esigetdensity_();	/*                      :escZ.c */
  void GetEPCollisions();	/*                      :escZ.c */
  double *Ti;			/*    .GetEPCollisions():escZ.c */
  void getcollisions_();	/*                      :escZ.c */
  int setparticletype_();	/*                      :escZ.c */
  int rzmotion_();		/*                      :escZ.c */
  REAL *gf;			/*          .rzmotion_():escZ.c */
#endif /* }=== End of List of Unused variables ======*/
#ifndef H/* ---------- Used External Variables ------{ */
  Ignor double;
  Ignor REAL;
  Ignor int;
  Systm NULL;
  Systm atan();
  Systm sqrt();
  Systm free();
  Systm exp();
  Systm malloc();
  Systm printf();
  Systm fopen();
  Systm fread();
  Systm fclose();
  Systm feof();
  Systm fgetc();
  Systm strlen();
  Systm strncmp();
  Systm fscanf();
  Systm isspace();
  Systm cos();
  Systm sin();
  Systm log();
  Undef calloc();	/* NO prototype */
#endif /* }======== End of Used External Variables ======= */

#endif /* }======== End of Use of Variables ======== */

static double cr2;	/*  */
static double cr3;	/*  */
static double cr4;	/*  */
static double cr6;	/*  */
static double cr12;	/*  */
static double cgp;	/*  */
static double c2gp;	/*  */
static double cgp_4;	/*  */
static double cr2gp;	/*  */
static double cgm0;	/*  */
static double crgm0;	/*  */
static double c2rqgp;	/*  */
static int Na;		/*  */
static int Na1;		/*  */
static int Np;		/*  */
static int Np1;		/*  */
static int MemFl;	/*  */
static REAL *sa;	/*  */
static REAL *gq;	/*  */
static REAL *aF;	/*  */
static REAL *daF;	/*  */
static REAL *dgF;	/*  */
static REAL *d2gF;	/*  */
static REAL *dgY;	/*  */
static REAL *d2gY;	/*  */
static REAL *aT;	/*  */
static REAL *daT;	/*  */
static REAL *aP;	/*  */
static REAL *daP;	/*  */
static REAL *gF;	/*  */
static REAL *gY;	/*  */
static REAL *sne;	/*  */
static REAL *dsne;	/*  */
static REAL *sr;	/*  */
static REAL *sra;	/*  */
static REAL *srq;	/*  */
static REAL *sraq;	/*  */
static REAL *sz;	/*  */
static REAL *sza;	/*  */
static REAL *szq;	/*  */
static REAL *szaq;	/*  */
static REAL *aB;	/*  */
static REAL *aBa;	/*  */
static REAL *aBq;	/*  */
static REAL *aBaq;	/*  */
static REAL *gH;	/*  */
static REAL *gHa;	/*  */
static REAL *gHq;	/*  */
static REAL *gHaq;	/*  */
static REAL *gHqq;	/*  */
static REAL *gHaqq;	/*  */
static int i0;		/*  */
static int i1;		/*  */
static int j00;		/*  */
static int j10;		/*  */
static int j01;		/*  */
static int j11;		/*  */
static REAL A;		/*  */
static REAL ha;		/*  */
static REAL rha;	/*  */
static REAL hq;		/*  */
static REAL rhq;	/*  */
static REAL cgq0;	/*  */
static REAL crgq0;	/*  */
static REAL xF;		/*  */
static REAL XF;		/*  */
static REAL xD;		/*  */
static REAL XD;		/*  */
static REAL dX;		/*  */
static REAL dxD;	/*  */
static REAL dXD;	/*  */
static REAL yF;		/*  */
static REAL YF;		/*  */
static REAL yD;		/*  */
static REAL YD;		/*  */
static REAL dY;		/*  */
static REAL dyD;	/*  */
static REAL dYD;	/*  */
static REAL y;		/*  */
static REAL Y;		/*  */
static REAL yy;		/*  */
static REAL YY;		/*  */
static int nF;		/*  */
static double F0[];	/*  */
static double F1[];	/*  */
static double F2[];	/*  */
static double F3[];	/*  */
static double tF1;	/*  */
static double hF;	/*  */
static double rhF;	/*  */

int InitLocalConstants()
{
  static double cr2;	/*  :escZ.c */
  static double cr3;	/*  :escZ.c */
  static double cr4;	/*  :escZ.c */
  static double cr6;	/*  :escZ.c */
  static double cr12;	/*  :escZ.c */
  static double cgp_4;	/*  :escZ.c */
  static double cgp;	/*  :escZ.c */
  static double c2gp;	/*  :escZ.c */
  static double cr2gp;	/*  :escZ.c */
  static double cgm0;	/*  :escZ.c */
  static double crgm0;	/*  :escZ.c */
  static double c2rqgp;	/*  :escZ.c */
/*
   */
#ifdef Doc_InitLocalConstants
\section{Skeleton of InitLocalConstants()}
  \begin{verbatim}
    InitLocalConstants(){
    }
  \end{verbatim}
\section{InitLocalConstants()}
#endif
}

int esifree_()
{
  static int MemFl;	/*  :escZ.c */
  static REAL *gF;	/*  :escZ.c */
  static REAL *gH;	/*  :escZ.c */
  static REAL *aB;	/*  :escZ.c */
  static REAL *sz;	/*  :escZ.c */
  static REAL *sr;	/*  :escZ.c */
  static REAL *gq;	/*  :escZ.c */
/*
   */
#ifdef Doc_esifree_
\section{Skeleton of esifree_()}
  \begin{verbatim}
    esifree_(){
    }
  \end{verbatim}
\section{esifree_()}
#endif
}

int SetFunctionErf()
{
  static double hF;	/*  :escZ.c */
  static double tF1;	/*  :escZ.c */
  static int nF;	/*  :escZ.c */
  static double rhF;	/*  :escZ.c */
  static double cr3;	/*  :escZ.c */
  static double c2rqgp;	/*  :escZ.c */
  static double F0[];	/*  :escZ.c */
  static double F1[];	/*  :escZ.c */
  static double F2[];	/*  :escZ.c */
  static double F3[];	/*  :escZ.c */
/*
   */
#ifdef Doc_SetFunctionErf
\section{Skeleton of SetFunctionErf()}
  \begin{verbatim}
    SetFunctionErf(){
    }
  \end{verbatim}
\section{SetFunctionErf()}
Integral in the collision operator\cite{NRL}
\bqbl
&&
\gy(x)
=
\R{2}{\q{\gp}}\int^x_0\q{t}e^{-t}dt
=
\R{4}{\q{\gp}}\int^{\q{x}}_0t^2e^{-t^2}dt
,\\&&
\gy(x)
=
\R{4}{\q{\gp}}\left\{\parbox{4in}{$\db
\R{x\q{x}}{3}e^{-x}\left(1
+\R{2x}{5}
+\R{e^{x}}{x\q{x}}\int^{\q{x}}_0\R{4t^6}{5}e^{-t^2}dt
\right)
,\quad x \leq25,
$
\\
$\db
\gy_1
-\R{\q{x}}{2}e^{-x}\left(
1
+\R{1}{2x}
-\R{1}{4x^2}
-\q{x}e^{x}\int^{\q{x}}_{5}\R{3}{4t^4}e^{-t^2}dt
\right)
,\quad x >25,
$}
\right.
\\&&
\gy_1
=
\int^{5}_0t^2e^{-t^2}dt
+ \R{\q{x}}{2}e^{-x}\left(
 1
 +\R{1}{2x}
 -\R{1}{4x^2}
\right)_{x=25}
.\eqbl
Spline calculations
\bqbl
&&
\gy(x)
=
\left\{\parbox{2.3in}{$\db
xF_1(v)
,\quad
v=\q{x}
,\quad x \leq25,
$
\\
$\db
\R{2}{\q{\gp}}\gy_1-\q{x}e^{-x}F_2(x)
=
1
,\quad x >25,
$}
\right.
\\&&
F_1(v)=
\R{4v}{3\q{\gp}}\left(e^{-v^2}+\R{2v^2e^{-v^2}}{5}
+\R{4}{5v^3}\int^v_0t^6e^{-t^2}dt
\right)
,\\&&
F'_1(v)=
\R{4v}{3\q{\gp}}\left(-\R{6ve^{-v^2}}{5}
-\R{12}{5v^4}\int^v_0t^6e^{-t^2}dt
\right)
+\R{F_1}{v}
.\eqbl
\Subsection{Polynomial representation}
Hermitian representation
\bqbl
&&
T
=
{x_1-x\over h}
,\quad
t
=
{x-x_0\over h}
,\quad
h
\equiv
x_1-x_0
,\\&&
f(t)=
(3T^2-2T^3)f_0
+(3t^2-2t^3)f_1
+T^2thf'_0
-t^2Thf'_1
,\\&&
hf'(t)=
6tT(f_1-f_0)
+(T^2-2Tt)hf'_0
+(t^2-2tT)hf'_1
,\\&&
h^2f''(t)=
6(T-t)(f_1-f_0)
+(2t-4T)hf'_0
+(4t-2T)hf'_1
,\\&&
h^3f'''(t)=
-12(f_1-f_0)
+6hf'_0
+6hf'_1
.\eqbl
Polynomial representation
\bqbl
&&
f_0
=
f_0
,\quad
f'_0
=
f'_0
,\quad
\R{1}{2}h^2f''_0
=
3(f_1-f_0)
-2hf'_0
-hf'_1
,\quad
\R{1}{6}h^3f'''_0
=
-2(f_1-f_0)
+hf'_0
+hf'_1
,\\&&
f(t)=
f_0+f'_0t+\R{1}{2}f''_0t^2+\R{1}{6}f'''_0t^3
.\eqbl

\begin{thebibliography}{10}
\bibitem{NRL}
NRP Plasma Formulary. NRL Publication 177-4405, p.31 (1990). 
\end{thebibliography}
#endif
}

int ESISetMem(gq0,na1,np1)
  double *gq0;	/*  */
  int na1;	/*  */
  int np1;	/*  */
{
  int np1;			/*  .ESISetMem():escZ.c */
  int na1;			/*  .ESISetMem():escZ.c */
  double *gq0;			/*  .ESISetMem():escZ.c */
  static int Np1;		/*              :escZ.c */
  static int Na1;		/*              :escZ.c */
  static int Na;		/*              :escZ.c */
  static int Np;		/*              :escZ.c */
  static REAL *gq;		/*              :escZ.c */
  static REAL *sa;		/*              :escZ.c */
  static REAL *aF;		/*              :escZ.c */
  static REAL *daF;		/*              :escZ.c */
  static REAL *dgF;		/*              :escZ.c */
  static REAL *d2gF;		/*              :escZ.c */
  static REAL *dgY;		/*              :escZ.c */
  static REAL *d2gY;		/*              :escZ.c */
  static REAL *aT;		/*              :escZ.c */
  static REAL *daT;		/*              :escZ.c */
  static REAL *aP;		/*              :escZ.c */
  static REAL *daP;		/*              :escZ.c */
  static REAL *sr;		/*              :escZ.c */
  static REAL *sra;		/*              :escZ.c */
  static REAL *srq;		/*              :escZ.c */
  static REAL *sraq;		/*              :escZ.c */
  static REAL *sz;		/*              :escZ.c */
  static REAL *sza;		/*              :escZ.c */
  static REAL *szq;		/*              :escZ.c */
  static REAL *szaq;		/*              :escZ.c */
  static REAL *aB;		/*              :escZ.c */
  static REAL *aBa;		/*              :escZ.c */
  static REAL *aBq;		/*              :escZ.c */
  static REAL *aBaq;		/*              :escZ.c */
  static REAL *gH;		/*              :escZ.c */
  static REAL *gHa;		/*              :escZ.c */
  static REAL *gHq;		/*              :escZ.c */
  static REAL *gHaq;		/*              :escZ.c */
  static REAL *gHqq;		/*              :escZ.c */
  static REAL *gHaqq;		/*              :escZ.c */
  static int MemFl;		/*              :escZ.c */
  static REAL *gF;		/*              :escZ.c */
  static REAL *gY;		/*              :escZ.c */
  int InitLocalConstants();	/*              :escZ.c */
  int SetFunctionErf();		/*              :escZ.c */
/*
   */
#ifdef Doc_ESISetMem
\Section{Skeleton of ESISetMem()}
  \begin{verbatim}
    ESISetMem(){
    }
  \end{verbatim}
\Section{ESISetMem()}
Routine links spline coefficients with the shared memory.
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int ESIMemAlloc(na1,np1)
  int na1;	/*  */
  int np1;	/*  */
{
  int np1;			/*  .ESIMemAlloc():escZ.c */
  int na1;			/*  .ESIMemAlloc():escZ.c */
  static int Np1;		/*                :escZ.c */
  static int Na1;		/*                :escZ.c */
  static int Na;		/*                :escZ.c */
  static int Np;		/*                :escZ.c */
  static REAL *gq;		/*                :escZ.c */
  static REAL *sr;		/*                :escZ.c */
  static REAL *sz;		/*                :escZ.c */
  static REAL *aB;		/*                :escZ.c */
  static REAL *gH;		/*                :escZ.c */
  static int MemFl;		/*                :escZ.c */
  static REAL *sa;		/*                :escZ.c */
  static REAL *aF;		/*                :escZ.c */
  static REAL *daF;		/*                :escZ.c */
  static REAL *dgF;		/*                :escZ.c */
  static REAL *d2gF;		/*                :escZ.c */
  static REAL *dgY;		/*                :escZ.c */
  static REAL *d2gY;		/*                :escZ.c */
  static REAL *aT;		/*                :escZ.c */
  static REAL *daT;		/*                :escZ.c */
  static REAL *aP;		/*                :escZ.c */
  static REAL *daP;		/*                :escZ.c */
  static REAL *sne;		/*                :escZ.c */
  static REAL *dsne;		/*                :escZ.c */
  static REAL *sra;		/*                :escZ.c */
  static REAL *srq;		/*                :escZ.c */
  static REAL *sraq;		/*                :escZ.c */
  static REAL *sza;		/*                :escZ.c */
  static REAL *szq;		/*                :escZ.c */
  static REAL *szaq;		/*                :escZ.c */
  static REAL *aBa;		/*                :escZ.c */
  static REAL *aBq;		/*                :escZ.c */
  static REAL *aBaq;		/*                :escZ.c */
  static REAL *gHa;		/*                :escZ.c */
  static REAL *gHq;		/*                :escZ.c */
  static REAL *gHaq;		/*                :escZ.c */
  static REAL *gHqq;		/*                :escZ.c */
  static REAL *gHaqq;		/*                :escZ.c */
  static REAL *gF;		/*                :escZ.c */
  static REAL *gY;		/*                :escZ.c */
  int InitLocalConstants();	/*                :escZ.c */
  int SetFunctionErf();		/*                :escZ.c */
/*
   */
#ifdef Doc_ESIMemAlloc
\section{Skeleton of ESIMemAlloc()}
  \begin{verbatim}
    ESIMemAlloc(){
    }
  \end{verbatim}
\section{ESIMemAlloc()}
#endif
}

int ESICopy(esr,esz,eaB,egH,na1,np1)
  double *esr;	/*  */
  double *esz;	/*  */
  double *eaB;	/*  */
  double *egH;	/*  */
  int na1;	/*  */
  int np1;	/*  */
{
  int np1;		/*  .ESICopy():escZ.c */
  int na1;		/*  .ESICopy():escZ.c */
  double *esr;		/*  .ESICopy():escZ.c */
  double *esz;		/*  .ESICopy():escZ.c */
  double *eaB;		/*  .ESICopy():escZ.c */
  double *egH;		/*  .ESICopy():escZ.c */
  static REAL *sr;	/*            :escZ.c */
  static REAL *sra;	/*            :escZ.c */
  static REAL *srq;	/*            :escZ.c */
  static REAL *sraq;	/*            :escZ.c */
  static REAL *sz;	/*            :escZ.c */
  static REAL *sza;	/*            :escZ.c */
  static REAL *szq;	/*            :escZ.c */
  static REAL *szaq;	/*            :escZ.c */
  static REAL *aB;	/*            :escZ.c */
  static REAL *aBa;	/*            :escZ.c */
  static REAL *aBq;	/*            :escZ.c */
  static REAL *aBaq;	/*            :escZ.c */
  static REAL *gH;	/*            :escZ.c */
  static REAL *gHa;	/*            :escZ.c */
  static REAL *gHq;	/*            :escZ.c */
  static REAL *gHaq;	/*            :escZ.c */
  static REAL *gHqq;	/*            :escZ.c */
  static REAL *gHaqq;	/*            :escZ.c */
/*
   */
#ifdef Doc_ESICopy
\section{Skeleton of ESICopy()}
  \begin{verbatim}
    ESICopy(){
    }
  \end{verbatim}
\section{ESICopy()}
#endif
}

int ESIReadBD(FNm)
  char *FNm;	/*  */
{
  char *FNm;		/*  .ESIReadBD():escZ.c */
  static int MemFl;	/*              :escZ.c */
  static int Np1;	/*              :escZ.c */
  static int Na1;	/*              :escZ.c */
  static REAL *gq;	/*              :escZ.c */
  static REAL *sa;	/*              :escZ.c */
  static REAL *aF;	/*              :escZ.c */
  static REAL *daF;	/*              :escZ.c */
  static REAL *dgF;	/*              :escZ.c */
  static REAL *d2gF;	/*              :escZ.c */
  static REAL *dgY;	/*              :escZ.c */
  static REAL *d2gY;	/*              :escZ.c */
  static REAL *aT;	/*              :escZ.c */
  static REAL *daT;	/*              :escZ.c */
  static REAL *aP;	/*              :escZ.c */
  static REAL *daP;	/*              :escZ.c */
  static REAL *sr;	/*              :escZ.c */
  static REAL *sra;	/*              :escZ.c */
  static REAL *srq;	/*              :escZ.c */
  static REAL *sraq;	/*              :escZ.c */
  static REAL *sz;	/*              :escZ.c */
  static REAL *sza;	/*              :escZ.c */
  static REAL *szq;	/*              :escZ.c */
  static REAL *szaq;	/*              :escZ.c */
  static REAL *aB;	/*              :escZ.c */
  static REAL *aBa;	/*              :escZ.c */
  static REAL *aBq;	/*              :escZ.c */
  static REAL *aBaq;	/*              :escZ.c */
  static REAL *gHq;	/*              :escZ.c */
  static REAL *gHaq;	/*              :escZ.c */
  static REAL *gHqq;	/*              :escZ.c */
  static REAL *gHaqq;	/*              :escZ.c */
  int esifree_();	/*              :escZ.c */
  int ESIMemAlloc();	/*              :escZ.c */
/*
   */
#ifdef Doc_ESIReadBD
\section{Skeleton of ESIReadBD()}
  \begin{verbatim}
    ESIReadBD(){
    }
  \end{verbatim}
\section{ESIReadBD()}
#endif
}

int ESIReadAD(FNm)
  char *FNm;	/*  */
{
  char *FNm;		/*  .ESIReadAD():escZ.c */
  static int MemFl;	/*              :escZ.c */
  static int Np1;	/*              :escZ.c */
  static int Na1;	/*              :escZ.c */
  static REAL *gq;	/*              :escZ.c */
  static REAL *sa;	/*              :escZ.c */
  static REAL *aF;	/*              :escZ.c */
  static REAL *daF;	/*              :escZ.c */
  static REAL *dgF;	/*              :escZ.c */
  static REAL *d2gF;	/*              :escZ.c */
  static REAL *dgY;	/*              :escZ.c */
  static REAL *d2gY;	/*              :escZ.c */
  static REAL *aT;	/*              :escZ.c */
  static REAL *daT;	/*              :escZ.c */
  static REAL *aP;	/*              :escZ.c */
  static REAL *daP;	/*              :escZ.c */
  static REAL *sr;	/*              :escZ.c */
  static REAL *sra;	/*              :escZ.c */
  static REAL *srq;	/*              :escZ.c */
  static REAL *sraq;	/*              :escZ.c */
  static REAL *sz;	/*              :escZ.c */
  static REAL *sza;	/*              :escZ.c */
  static REAL *szq;	/*              :escZ.c */
  static REAL *szaq;	/*              :escZ.c */
  static REAL *aB;	/*              :escZ.c */
  static REAL *aBa;	/*              :escZ.c */
  static REAL *aBq;	/*              :escZ.c */
  static REAL *aBaq;	/*              :escZ.c */
  static REAL *gHq;	/*              :escZ.c */
  static REAL *gHaq;	/*              :escZ.c */
  static REAL *gHqq;	/*              :escZ.c */
  static REAL *gHaqq;	/*              :escZ.c */
  static REAL *sne;	/*              :escZ.c */
  static REAL *dsne;	/*              :escZ.c */
  int esifree_();	/*              :escZ.c */
  int ESIMemAlloc();	/*              :escZ.c */
/*
   */
#ifdef Doc_ESIReadAD
\section{Skeleton of ESIReadAD()}
  \begin{verbatim}
    ESIReadAD(){
    }
  \end{verbatim}
\section{ESIReadAD()}
#endif
}

int ESIInit()
{
  static REAL cgq0;	/*  :escZ.c */
  static REAL *gq;	/*  :escZ.c */
  static int Np;	/*  :escZ.c */
  static REAL crgq0;	/*  :escZ.c */
  static REAL hq;	/*  :escZ.c */
  static REAL rhq;	/*  :escZ.c */
  static REAL ha;	/*  :escZ.c */
  static REAL *sa;	/*  :escZ.c */
  static REAL rha;	/*  :escZ.c */
  static int Na1;	/*  :escZ.c */
  static REAL *gH;	/*  :escZ.c */
  static REAL *gHa;	/*  :escZ.c */
  static int Np1;	/*  :escZ.c */
  static REAL *gHq;	/*  :escZ.c */
  static double cr6;	/*  :escZ.c */
  static REAL *gHqq;	/*  :escZ.c */
  static REAL *gHaq;	/*  :escZ.c */
  static REAL *gHaqq;	/*  :escZ.c */
  static REAL *gF;	/*  :escZ.c */
  static REAL *dgF;	/*  :escZ.c */
  static REAL *d2gF;	/*  :escZ.c */
  static double cr3;	/*  :escZ.c */
  static REAL *gY;	/*  :escZ.c */
  static REAL *dgY;	/*  :escZ.c */
  static REAL *d2gY;	/*  :escZ.c */
  static int Na;	/*  :escZ.c */
/*
   */
#ifdef Doc_ESIInit
\section{Skeleton of ESIInit()}
  \begin{verbatim}
    ESIInit(){
    }
  \end{verbatim}
\section{ESIInit()}
Hermitain representation
\bqbl
&&
X
=
{a_1-a\over h}
,\quad
x
=
{a-a_0\over h}
,\quad
h
\equiv
a_1-a_0
,\\&&
f(x)=
(3X^2-2X^3)f_0
+(3x^2-2x^3)f_1
+X^2xhf'_0
-x^2Xhf'_1
,\\&&
hf'(x)=
6xX(f_1-f_0)
+(X^2-2Xx)hf'_0
+(x^2-2xX)hf'_1
,\\&&
h^2f''(x)=
6(X-x)(f_1-f_0)
+(2x-4X)hf'_0
+(4x-2X)hf'_1
,\\&&
h^3f'''(x)=
-12(f_1-f_0)
+6hf'_0
+6hf'_1
.\eqbl
Polynomial representation
\bqbl
&&
f_0
=
f_0
,\quad
f'_0
=
f'_0
,\quad
\R{1}{2}h^2f''_0
=
3(f_1-f_0)
-2hf'_0
-hf'_1
,\quad
\R{1}{6}h^3f'''_0
=
-2(f_1-f_0)
+hf'_0
+hf'_1
,\\&&
f(t)=
f_0+hf'_0x+\R{h^2}{2}f''_0x^2+\R{h^3}{6}f'''_0x^3
.\eqbl
Integration of Hermit polynomials
\bqbl
&&
\int^{\gq_1}_{\gq_0}fd\gq
=
{h_\gq\over2}(f_0+f_1)
+{h^2_\gq\over12}(f'_0-f'_1)
,\quad
y=\R{\gq-\gq_0}{h_\gq}
,\quad
Y=\R{\gq_1-\gq}{h_\gq}
,\\&&
\int^\gq_{\gq_0}fd\gq
=
\R{h_\gq}{2}(1-2Y^3+Y^4)f_0
+\R{h_\gq}{2}(2y^3-y^4)f_1
+\R{h^2_\gq}{12}(1-Y^4-4Y^3 y)f'_{\gq0}
\\&&\qquad
-\R{h^2_\gq}{12}(4y^3 Y+y^4)f'_{\gq1}
.\eqbl
Integration of Hermit polynomials with a weight
\bqbl
&&
\int^{a_1}_{a_0}afda
=
a_0\R{h_a}{2}\left[f_0+f_1+\R{h_a}{6}(f'_0-f'_1)\right]
+h^2_a\int^1_0xfdx
,\\&&
h^2_a\int^1_0xfdx
=
h^2_a\int^1_0\left(f_0x+hf'_0x^2+\R{h^2}{2}f''_0x^3
 +\R{h^3}{6}f'''_0x^4\right)dx
=
\R{h^2_a}{2}f_0+\R{h^3_a}{3}f'_0+\R{h^4_a}{8}f''_0+\R{h^5_a}{30}f'''_0
,\\&&
h^2f''_0=
6(f_1-f_0)-4hf'_0-2hf'_1
,\\&&
h^3f'''_0=
-12(f_1-f_0)+6hf'_0+6hf'_1
,\\&&
h^2_a\int^1_0xfdx
=
\R{h^2_a}{2}f_0
+\R{h^3_a}{3}f'_0
+\R{h^2_a}{4}(3(f_1-f_0)-2hf'_0-hf'_1)
+\R{h^2_a}{5}(-2(f_1-f_0)+hf'_0+hf'_1)
,\\&&
h^2_a\int^1_0xfdx
=
\R{h^2_a}{20}(3f_0+7f_1)
+\R{h^3_a}{60}(2f'_0-3f'_1)
 .\eqbl
Finally,
\bqbl
&&
\int^{a_1}_{a_0}afda
=
\R{a_0h_a}{2}\left[f_0+f_1+\R{h_a}{6}(f'_0-f'_1)\right]
+\R{h^2_a}{20}(3f_0+7f_1)+\R{h^3_a}{60}(2f'_0-3f'_1)
,\\&&
x=\R{a-a_0}{h_a}
,\quad
X=\R{a_1-a}{h_a}
,\\&&
\int^a_{a_0}fda
=
\R{h_a}{2}(1-2X^3+X^4)f_0
+\R{h_a}{2}(2x^3-x^4)f_1
+\R{h^2_a}{12}(1-X^4-4X^3 x)f'_{a0}
\\&&\qquad
-\R{h^2_a}{12}(4x^3 X+x^4)f'_{a1}
.\eqbl
#endif
}

int esiread_(FName)
  char *FName;	/*  */
{
  char *FName;		/*  .esiread_():escZ.c */
  int ESIReadAD();	/*             :escZ.c */
  int ESIReadBD();	/*             :escZ.c */
  int ESIInit();	/*             :escZ.c */
/*
   */
#ifdef Doc_esiread_
\section{Skeleton of esiread_()}
  \begin{verbatim}
    esiread_(){
    }
  \end{verbatim}
\section{esiread_()}
#endif
}
static int MemFlRZ;	/*  */
static int nrMap;	/*  */
static int nzMap;	/*  */
static int nrMap1;	/*  */
static int nzMap1;	/*  */
static int nMap;	/*  */
static int nMap1;	/*  */
static double rBox[];	/*  */
static double zBox[];	/*  */
static double drMap;	/*  */
static double dzMap;	/*  */
static double rdrMap;	/*  */
static double rdzMap;	/*  */
static int *iAMap;	/*  */
static int *k00Map;	/*  */
static int irMap;	/*  */
static int izMap;	/*  */
static int k00;		/*  */
static int k01;		/*  */
static int k10;		/*  */
static int k11;		/*  */
static double *f0Map;	/*  */
static double *fxMap;	/*  */
static double *fyMap;	/*  */
static double *f2Map;	/*  */

int ESIMemAllocRZ(n,n1)
  int n;	/*  */
  int n1;	/*  */
{
  Undef calloc();
  int n;		/*  .ESIMemAllocRZ():escZ.c */
  int n1;		/*  .ESIMemAllocRZ():escZ.c */
  static int *iAMap;	/*                  :escZ.c */
  static int *k00Map;	/*                  :escZ.c */
  static double *f0Map;	/*                  :escZ.c */
  static double *fxMap;	/*                  :escZ.c */
  static double *fyMap;	/*                  :escZ.c */
  static double *f2Map;	/*                  :escZ.c */
  static int MemFlRZ;	/*                  :escZ.c */
  /*
   */
#ifdef Doc_ESIMemAllocRZ
\Section{Skeleton of \c{ESIMemAllocRZ()}}
  \begin{verbatim}
    ESIMemAllocRZ(){
    }
  \end{verbatim}
\Section{\c{ESIMemAllocRZ()}}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int esifreerz_()
{
  static double *f0Map;	/*  :escZ.c */
  static int *iAMap;	/*  :escZ.c */
  static int MemFlRZ;	/*  :escZ.c */
  /*
   */
#ifdef Doc_esifreerz_
\Section{Skeleton of \c{esifreerz_()}}
  \begin{verbatim}
    esifreerz_(){
    }
  \end{verbatim}
\Section{\c{esifreerz_()}}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int ESIReadRZ(FNm)
  char *FNm;	/*  */
{
  char *FNm;		/*  .ESIReadRZ():escZ.c */
  static int MemFlRZ;	/*              :escZ.c */
  static int nrMap;	/*              :escZ.c */
  static int nzMap;	/*              :escZ.c */
  static int nMap1;	/*              :escZ.c */
  static int nMap;	/*              :escZ.c */
  static int nrMap1;	/*              :escZ.c */
  static int nzMap1;	/*              :escZ.c */
  static double drMap;	/*              :escZ.c */
  static double rBox[];	/*              :escZ.c */
  static double dzMap;	/*              :escZ.c */
  static double zBox[];	/*              :escZ.c */
  static double rdrMap;	/*              :escZ.c */
  static double rdzMap;	/*              :escZ.c */
  static int *k00Map;	/*              :escZ.c */
  static int *iAMap;	/*              :escZ.c */
  static double *f0Map;	/*              :escZ.c */
  static double *fxMap;	/*              :escZ.c */
  static double *fyMap;	/*              :escZ.c */
  static double *f2Map;	/*              :escZ.c */
  int esifreerz_();	/*              :escZ.c */
  int ESIMemAllocRZ();	/*              :escZ.c */
  /*
   */
#ifdef Doc_ESIReadRZ
\Section{Skeleton of \c{ESIReadRZ()}}
  \begin{verbatim}
    ESIReadRZ(){
    }
  \end{verbatim}
\Section{\c{ESIReadRZ()}}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int esireadrz_(FName)
  char *FName;	/*  */
{
  char *FName;		/*  .esireadrz_():escZ.c */
  int ESIReadRZ();	/*               :escZ.c */
  /*
   */
#ifdef Doc_esireadrz_
\Section{Skeleton of \c{esireadrz_()}}
  \begin{verbatim}
    esireadrz_(){
    }
  \end{verbatim}
\Section{\c{esireadrz_()}}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}
static int *sw;		/*  */
static REAL *r;		/*  */
static REAL *ra;	/*  */
static REAL *rq;	/*  */
static REAL *z;		/*  */
static REAL *za;	/*  */
static REAL *zq;	/*  */
static REAL *B;		/*  */
static REAL *Ba;	/*  */
static REAL *Bq;	/*  */
static REAL *gh;	/*  */
static REAL *gha;	/*  */
static REAL *ghq;	/*  */
static REAL *ap;	/*  */
static REAL *qp;	/*  */
static REAL *F;		/*  */
static REAL *Fa;	/*  */
static REAL *gFa;	/*  */
static REAL *gFaa;	/*  */
static REAL *gYa;	/*  */
static REAL *gYaa;	/*  */
static REAL *T;		/*  */
static REAL *Ta;	/*  */
static REAL *P;		/*  */
static REAL *Pa;	/*  */
static REAL *Ne;	/*  */
static REAL *dNe;	/*  */
static REAL aTe;	/*  */
static REAL aTi;	/*  */
static REAL raT;	/*  */

void esilink2c_(XaF,XaFa,XgFa,XgFaa,XgYa,XgYaa,XT,XTa,XP,XPa,Xr,Xra,Xrq,Xz,Xza,Xzq,XB,XBa,XBq,Xgh,Xgha,Xghq,Xsw)
  REAL *XaF;	/*  */
  REAL *XaFa;	/*  */
  REAL *XgFa;	/*  */
  REAL *XgFaa;	/*  */
  REAL *XgYa;	/*  */
  REAL *XgYaa;	/*  */
  REAL *XT;	/*  */
  REAL *XTa;	/*  */
  REAL *XP;	/*  */
  REAL *XPa;	/*  */
  REAL *Xr;	/*  */
  REAL *Xra;	/*  */
  REAL *Xrq;	/*  */
  REAL *Xz;	/*  */
  REAL *Xza;	/*  */
  REAL *Xzq;	/*  */
  REAL *XB;	/*  */
  REAL *XBa;	/*  */
  REAL *XBq;	/*  */
  REAL *Xgh;	/*  */
  REAL *Xgha;	/*  */
  REAL *Xghq;	/*  */
  int *Xsw;	/*  */
{
  REAL *XaF;		/*  .esilink2c_():escZ.c */
  REAL *XaFa;		/*  .esilink2c_():escZ.c */
  REAL *XgFa;		/*  .esilink2c_():escZ.c */
  REAL *XgFaa;		/*  .esilink2c_():escZ.c */
  REAL *XgYa;		/*  .esilink2c_():escZ.c */
  REAL *XgYaa;		/*  .esilink2c_():escZ.c */
  REAL *XT;		/*  .esilink2c_():escZ.c */
  REAL *XTa;		/*  .esilink2c_():escZ.c */
  REAL *XP;		/*  .esilink2c_():escZ.c */
  REAL *XPa;		/*  .esilink2c_():escZ.c */
  REAL *Xr;		/*  .esilink2c_():escZ.c */
  REAL *Xra;		/*  .esilink2c_():escZ.c */
  REAL *Xrq;		/*  .esilink2c_():escZ.c */
  REAL *Xz;		/*  .esilink2c_():escZ.c */
  REAL *Xza;		/*  .esilink2c_():escZ.c */
  REAL *Xzq;		/*  .esilink2c_():escZ.c */
  REAL *XB;		/*  .esilink2c_():escZ.c */
  REAL *XBa;		/*  .esilink2c_():escZ.c */
  REAL *XBq;		/*  .esilink2c_():escZ.c */
  REAL *Xgh;		/*  .esilink2c_():escZ.c */
  REAL *Xgha;		/*  .esilink2c_():escZ.c */
  REAL *Xghq;		/*  .esilink2c_():escZ.c */
  int *Xsw;		/*  .esilink2c_():escZ.c */
  static REAL *F;	/*               :escZ.c */
  static REAL *Fa;	/*               :escZ.c */
  static REAL *gFa;	/*               :escZ.c */
  static REAL *gFaa;	/*               :escZ.c */
  static REAL *gYa;	/*               :escZ.c */
  static REAL *gYaa;	/*               :escZ.c */
  static REAL *T;	/*               :escZ.c */
  static REAL *Ta;	/*               :escZ.c */
  static REAL *P;	/*               :escZ.c */
  static REAL *Pa;	/*               :escZ.c */
  static REAL *r;	/*               :escZ.c */
  static REAL *ra;	/*               :escZ.c */
  static REAL *rq;	/*               :escZ.c */
  static REAL *z;	/*               :escZ.c */
  static REAL *za;	/*               :escZ.c */
  static REAL *zq;	/*               :escZ.c */
  static REAL *B;	/*               :escZ.c */
  static REAL *Ba;	/*               :escZ.c */
  static REAL *Bq;	/*               :escZ.c */
  static REAL *gh;	/*               :escZ.c */
  static REAL *gha;	/*               :escZ.c */
  static REAL *ghq;	/*               :escZ.c */
  static int *sw;	/*               :escZ.c */
/*
   */
#ifdef Doc_esilink2c_
\section{Skeleton of esilink2c_()}
  \begin{verbatim}
    esilink2c_(){
    }
  \end{verbatim}
\section{esilink2c_()}
#endif
}

void esilink2c4ne_(Xne,Xdne)
  double *Xne;	/*  */
  double *Xdne;	/*  */
{
  double *Xne;		/*  .esilink2c4ne_():escZ.c */
  double *Xdne;		/*  .esilink2c4ne_():escZ.c */
  static REAL *Ne;	/*                  :escZ.c */
  static REAL *dNe;	/*                  :escZ.c */
/*
   */
#ifdef Doc_esilink2c4ne_
\section{Skeleton of esilink2c4ne_()}
  \begin{verbatim}
    esilink2c4ne_(){
    }
  \end{verbatim}
\section{esilink2c4ne_()}
#endif
}

void settemperatures_(Te,Ti)
  double *Te;	/*  */
  double *Ti;	/*  */
{
  double *Te;		/*  .settemperatures_():escZ.c */
  double *Ti;		/*  .settemperatures_():escZ.c */
  static REAL aTe;	/*                     :escZ.c */
  static REAL aTi;	/*                     :escZ.c */
  static REAL raT;	/*                     :escZ.c */
/*
   */
#ifdef Doc_settemperatures_
\section{Skeleton of settemperatures_()}
  \begin{verbatim}
    settemperatures_(){
    }
  \end{verbatim}
\section{settemperatures_()}
#endif
}

int esigetprofiles_(sF,sFa,sgFa,sgFaa,sgYa,sgYaa,sT,sTa,sP,sPa,a)
  REAL *sF;	/*  */
  REAL *sFa;	/*  */
  REAL *sgFa;	/*  */
  REAL *sgFaa;	/*  */
  REAL *sgYa;	/*  */
  REAL *sgYaa;	/*  */
  REAL *sT;	/*  */
  REAL *sTa;	/*  */
  REAL *sP;	/*  */
  REAL *sPa;	/*  */
  REAL *a;	/*  */
{
  REAL *a;		/*  .esigetprofiles_():escZ.c */
  REAL *sF;		/*  .esigetprofiles_():escZ.c */
  REAL *sFa;		/*  .esigetprofiles_():escZ.c */
  REAL *sgFa;		/*  .esigetprofiles_():escZ.c */
  REAL *sgFaa;		/*  .esigetprofiles_():escZ.c */
  REAL *sgYa;		/*  .esigetprofiles_():escZ.c */
  REAL *sgYaa;		/*  .esigetprofiles_():escZ.c */
  REAL *sT;		/*  .esigetprofiles_():escZ.c */
  REAL *sTa;		/*  .esigetprofiles_():escZ.c */
  REAL *sP;		/*  .esigetprofiles_():escZ.c */
  REAL *sPa;		/*  .esigetprofiles_():escZ.c */
  static REAL A;	/*                    :escZ.c */
  static int i0;	/*                    :escZ.c */
  static REAL *sa;	/*                    :escZ.c */
  static REAL rha;	/*                    :escZ.c */
  static int Na;	/*                    :escZ.c */
  static int i1;	/*                    :escZ.c */
  static REAL XF;	/*                    :escZ.c */
  static REAL XD;	/*                    :escZ.c */
  static REAL ha;	/*                    :escZ.c */
  static REAL xF;	/*                    :escZ.c */
  static REAL xD;	/*                    :escZ.c */
  static REAL dX;	/*                    :escZ.c */
  static REAL dXD;	/*                    :escZ.c */
  static REAL dxD;	/*                    :escZ.c */
  static REAL *aF;	/*                    :escZ.c */
  static REAL *daF;	/*                    :escZ.c */
  static REAL *dgF;	/*                    :escZ.c */
  static REAL *d2gF;	/*                    :escZ.c */
  static REAL *gFaa;	/*                    :escZ.c */
  static REAL *gFa;	/*                    :escZ.c */
  static REAL *dgY;	/*                    :escZ.c */
  static REAL *d2gY;	/*                    :escZ.c */
  static REAL *gYaa;	/*                    :escZ.c */
  static REAL *gYa;	/*                    :escZ.c */
  static REAL *aT;	/*                    :escZ.c */
  static REAL *daT;	/*                    :escZ.c */
  static REAL *aP;	/*                    :escZ.c */
  static REAL *daP;	/*                    :escZ.c */
  static REAL *Ne;	/*                    :escZ.c */
  static REAL *sne;	/*                    :escZ.c */
  static REAL *dsne;	/*                    :escZ.c */
  static REAL raT;	/*                    :escZ.c */
  static REAL *dNe;	/*                    :escZ.c */
/*
   */
#ifdef Doc_esigetprofiles_
\section{Skeleton of esigetprofiles_()}
  \begin{verbatim}
    esigetprofiles_(){
    }
  \end{verbatim}
\section{esigetprofiles_()}
#endif
}

int esiget2dfunctions0(a,q,k)
  REAL a;	/*  */
  REAL q;	/*  */
  int k;	/*  */
{
  REAL a;		/*  .esiget2dfunctions0():escZ.c */
  int k;		/*  .esiget2dfunctions0():escZ.c */
  static int j01;	/*                       :escZ.c */
  static int j00;	/*                       :escZ.c */
  static int j10;	/*                       :escZ.c */
  static int Np1;	/*                       :escZ.c */
  static int j11;	/*                       :escZ.c */
  static REAL rha;	/*                       :escZ.c */
  static REAL XF;	/*                       :escZ.c */
  static REAL XD;	/*                       :escZ.c */
  static REAL xF;	/*                       :escZ.c */
  static REAL xD;	/*                       :escZ.c */
  static REAL dX;	/*                       :escZ.c */
  static REAL dXD;	/*                       :escZ.c */
  static REAL dxD;	/*                       :escZ.c */
  static REAL *sr;	/*                       :escZ.c */
  static REAL *sra;	/*                       :escZ.c */
  static REAL *srq;	/*                       :escZ.c */
  static REAL *sraq;	/*                       :escZ.c */
  static REAL *r;	/*                       :escZ.c */
  static REAL YF;	/*                       :escZ.c */
  static REAL yF;	/*                       :escZ.c */
  static REAL YD;	/*                       :escZ.c */
  static REAL yD;	/*                       :escZ.c */
  static REAL *rq;	/*                       :escZ.c */
  static REAL dY;	/*                       :escZ.c */
  static REAL dYD;	/*                       :escZ.c */
  static REAL dyD;	/*                       :escZ.c */
  static REAL *ra;	/*                       :escZ.c */
  static REAL *sz;	/*                       :escZ.c */
  static REAL *sza;	/*                       :escZ.c */
  static REAL *szq;	/*                       :escZ.c */
  static REAL *szaq;	/*                       :escZ.c */
  static REAL *zq;	/*                       :escZ.c */
  static REAL *z;	/*                       :escZ.c */
  static REAL *za;	/*                       :escZ.c */
  static REAL *aB;	/*                       :escZ.c */
  static REAL *aBa;	/*                       :escZ.c */
  static REAL *aBq;	/*                       :escZ.c */
  static REAL *aBaq;	/*                       :escZ.c */
  static REAL *B;	/*                       :escZ.c */
  static REAL *Bq;	/*                       :escZ.c */
  static REAL *Ba;	/*                       :escZ.c */
  static REAL *F;	/*                       :escZ.c */
  static REAL *aF;	/*                       :escZ.c */
  static int i0;	/*                       :escZ.c */
  static int i1;	/*                       :escZ.c */
  static REAL *daF;	/*                       :escZ.c */
  static REAL *Fa;	/*                       :escZ.c */
  static REAL *gFa;	/*                       :escZ.c */
  static REAL *dgF;	/*                       :escZ.c */
  static REAL *d2gF;	/*                       :escZ.c */
  static REAL *gFaa;	/*                       :escZ.c */
  static REAL *gYa;	/*                       :escZ.c */
  static REAL *dgY;	/*                       :escZ.c */
  static REAL *d2gY;	/*                       :escZ.c */
  static REAL *gYaa;	/*                       :escZ.c */
  static REAL *T;	/*                       :escZ.c */
  static REAL *aT;	/*                       :escZ.c */
  static REAL *daT;	/*                       :escZ.c */
  static REAL *Ta;	/*                       :escZ.c */
  static REAL *P;	/*                       :escZ.c */
  static REAL *aP;	/*                       :escZ.c */
  static REAL *daP;	/*                       :escZ.c */
  static REAL *Pa;	/*                       :escZ.c */
  static REAL *Ne;	/*                       :escZ.c */
  static REAL *sne;	/*                       :escZ.c */
  static REAL *dsne;	/*                       :escZ.c */
  static REAL raT;	/*                       :escZ.c */
  static REAL *dNe;	/*                       :escZ.c */
/*
   */
#ifdef Doc_esiget2dfunctions0
\Section{Skeleton of esiget2dfunctions0()}
  \begin{verbatim}
    esiget2dfunctions0(){
    }
  \end{verbatim}
\Section{esiget2dfunctions0()}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int esiget2dfunctions_(a0,gq0,n)
  REAL *a0;	/*  */
  REAL *gq0;	/*  */
  int *n;	/*  */
{
  int *n;		/*  .esiget2dfunctions_():escZ.c */
  REAL *a0;		/*  .esiget2dfunctions_():escZ.c */
  REAL *gq0;		/*  .esiget2dfunctions_():escZ.c */
  static REAL crgq0;	/*                       :escZ.c */
  static REAL cgq0;	/*                       :escZ.c */
  static REAL rhq;	/*                       :escZ.c */
  static int Np;	/*                       :escZ.c */
  static REAL y;	/*                       :escZ.c */
  static REAL *gq;	/*                       :escZ.c */
  static REAL Y;	/*                       :escZ.c */
  static REAL YY;	/*                       :escZ.c */
  static REAL yy;	/*                       :escZ.c */
  static REAL YF;	/*                       :escZ.c */
  static REAL YD;	/*                       :escZ.c */
  static REAL hq;	/*                       :escZ.c */
  static REAL yF;	/*                       :escZ.c */
  static REAL yD;	/*                       :escZ.c */
  static REAL dY;	/*                       :escZ.c */
  static REAL dYD;	/*                       :escZ.c */
  static REAL dyD;	/*                       :escZ.c */
  static int i0;	/*                       :escZ.c */
  static REAL *sa;	/*                       :escZ.c */
  static REAL rha;	/*                       :escZ.c */
  static int Na;	/*                       :escZ.c */
  static int i1;	/*                       :escZ.c */
  static REAL XF;	/*                       :escZ.c */
  static REAL XD;	/*                       :escZ.c */
  static REAL ha;	/*                       :escZ.c */
  static REAL xF;	/*                       :escZ.c */
  static REAL xD;	/*                       :escZ.c */
  static REAL dX;	/*                       :escZ.c */
  static REAL dXD;	/*                       :escZ.c */
  static REAL dxD;	/*                       :escZ.c */
  static int j00;	/*                       :escZ.c */
  static int Np1;	/*                       :escZ.c */
  static int j01;	/*                       :escZ.c */
  static int j10;	/*                       :escZ.c */
  static int j11;	/*                       :escZ.c */
  static REAL *sr;	/*                       :escZ.c */
  static REAL *sra;	/*                       :escZ.c */
  static REAL *srq;	/*                       :escZ.c */
  static REAL *sraq;	/*                       :escZ.c */
  static REAL *r;	/*                       :escZ.c */
  static REAL *rq;	/*                       :escZ.c */
  static REAL *ra;	/*                       :escZ.c */
  static REAL *sz;	/*                       :escZ.c */
  static REAL *sza;	/*                       :escZ.c */
  static REAL *szq;	/*                       :escZ.c */
  static REAL *szaq;	/*                       :escZ.c */
  static REAL *z;	/*                       :escZ.c */
  static REAL *zq;	/*                       :escZ.c */
  static REAL *za;	/*                       :escZ.c */
  static REAL *aB;	/*                       :escZ.c */
  static REAL *aBa;	/*                       :escZ.c */
  static REAL *aBq;	/*                       :escZ.c */
  static REAL *aBaq;	/*                       :escZ.c */
  static REAL *B;	/*                       :escZ.c */
  static REAL *Bq;	/*                       :escZ.c */
  static REAL *Ba;	/*                       :escZ.c */
  static REAL *gHq;	/*                       :escZ.c */
  static REAL *gHaq;	/*                       :escZ.c */
  static REAL *gHqq;	/*                       :escZ.c */
  static REAL *gHaqq;	/*                       :escZ.c */
  static REAL *ghq;	/*                       :escZ.c */
  static REAL *gh;	/*                       :escZ.c */
  static REAL *gH;	/*                       :escZ.c */
  static REAL *gHa;	/*                       :escZ.c */
  static double cr6;	/*                       :escZ.c */
  static REAL *gha;	/*                       :escZ.c */
  static REAL *F;	/*                       :escZ.c */
  static REAL *aF;	/*                       :escZ.c */
  static REAL *daF;	/*                       :escZ.c */
  static REAL *Fa;	/*                       :escZ.c */
  static REAL *gFa;	/*                       :escZ.c */
  static REAL *dgF;	/*                       :escZ.c */
  static REAL *d2gF;	/*                       :escZ.c */
  static REAL *gFaa;	/*                       :escZ.c */
  static REAL *gYa;	/*                       :escZ.c */
  static REAL *dgY;	/*                       :escZ.c */
  static REAL *d2gY;	/*                       :escZ.c */
  static REAL *gYaa;	/*                       :escZ.c */
  static REAL *T;	/*                       :escZ.c */
  static REAL *aT;	/*                       :escZ.c */
  static REAL *daT;	/*                       :escZ.c */
  static REAL *Ta;	/*                       :escZ.c */
  static REAL *P;	/*                       :escZ.c */
  static REAL *aP;	/*                       :escZ.c */
  static REAL *daP;	/*                       :escZ.c */
  static REAL *Pa;	/*                       :escZ.c */
  static REAL *Ne;	/*                       :escZ.c */
  static REAL *sne;	/*                       :escZ.c */
  static REAL *dsne;	/*                       :escZ.c */
  static REAL raT;	/*                       :escZ.c */
  static REAL *dNe;	/*                       :escZ.c */
/*
   */
#ifdef Doc_esiget2dfunctions_
\section{Skeleton of esiget2dfunctions\_()}
  \begin{verbatim}
    esiget2dfunctions_(){
    }
  \end{verbatim}
\section{esiget2dfunctions\_()}
1-D radial function
\bqbl
&&
X
=
\R{a_1-a}{h_0}
,\quad
x
=
\R{a-a_0}{h_0}
,\quad
h
\equiv
h_0
,\\&&
f(a)=(3X^2-2X^3)f_0+(3x^2-2x^3)f_1+X^2xhf'_0-x^2Xhf'_1
,\\&&
f'=-\R{6Xx}{h}f_0+\R{6xX}{h}f_1+X(1-3x)f'_0+x(1-3X)f'_1
,\\&&
\int^{a_1}_{a_0}fda=\R{h}{2}(f_0+f_1)+\R{h^2}{12}(f'_0-f'_1)
.\eqbl

2-D function
\bqbl
&&
X_a
=
{a_1-a\over h_a}
,\quad
x_a
=
{a-a_0\over h_a}
,\quad
h_a
\equiv
a_1-a_0
,\quad
Y
=
{\gq_1-\gq\over h_\gq}
,\quad
y
=
{\gq-\gq_0\over h_\gq}
,\quad
h_\gq
\equiv
\gq_1-\gq_0
,\\&&
f(a,\gq)=
(3Y^2-2Y^3)f_0
+(3y^2-2y^3)f_1
+Y^2 y h_\gq f'_{\gq0}
-y^2 Y h_\gq f'_{\gq1}
,\\&&
f_0
=
(3X^2_a-2X^3_a)f_{00}
+(3x^2_a-2x^3_a)f_{10}
+X^2_a x_a h_a f'_{a00}
-x^2_a X_a h_a f'_{a10}
,\\&&
f_1
=
(3X^2_a-2X^3_a)f_{01}
+(3x^2_a-2x^3_a)f_{11}
+X^2_a x_a h_a f'_{a01}
-x^2_a X_a h_a f'_{a11}
,\\&&
f'_{a0}
=
(3X^2_a-2X^3_a)f'_{a00}
+(3x^2_a-2x^3_a)f'_{a10}
+X^2_a x_a h_a f''_{aa00}
-x^2_a X_a h_a f''_{aa10}
,\\&&
f'_{a1}
=
(3X^2_a-2X^3_a)f'_{a01}
+(3x^2_a-2x^3_a)f'_{a11}
+X^2_a x_a h_a f''_{aa01}
-x^2_a X_a h_a f''_{aa11}
,\\&&
\int^{\gq_1}_{\gq_0}fd\gq
=
{h_\gq\over2}(f_0+f_1)
+{h^2_\gq\over12}(f'_0-f'_1)
,\\&&
\int^\gq_{\gq_0}fd\gq
=
\R{h_\gq}{2}(1-2Y^3+Y^4)f_0
+\R{h_\gq}{2}(2y^3-y^4)f_1
+\R{h^2_\gq}{12}(1-Y^4-4Y^3 y)f'_{\gq0}
-\R{h^2_\gq}{12}(4y^3 Y+y^4)f'_{\gq1}
.\eqbl
Calculations of $\gh$
\bqbl
&&
\gh
=
\int^\gq_0 \gh'_\gq d\gq
=
\int^{\gq_0}_0 f d\gq
+\int^\gq_{\gq_0}f d\gq
,\quad
f
\equiv
\gh'_\gq
,\\&&
f(a,\gq)=
(3X^2_a-2X^3_a)f_{00}(\gq)
+(3x^2_a-2x^3_a)f_{10}(\gq)
+X^2_a x_a h_a f'_{a00}(\gq)
-x^2_a X_a h_a f'_{a10}(\gq)
,\\&&
\int^{\gq_0}_0 f d\gq
=
(3X^2_a-2X^3_a)\int^{\gq_0}_0 f(a_0)d\gq
+(3x^2_a-2x^3_a)\int^{\gq_0}_0 f(a_1)d\gq
\\&&
+X^2_a x_a h_a\int^{\gq_0}_0 f'_a(a_0)d\gq
-x^2_a X_a h_a\int^{\gq_0}_0 f'_a(a_1)d\gq
\\&&
\int^{\gq_0}_0 f'_z d\gq
=
(3X^2_a-2X^3_a)'_a\int^{\gq_0}_0 f(a_0)d\gq
+(3x^2_a-2x^3_a)'_a\int^{\gq_0}_0 f(a_1)d\gq
\\&&
+(X^2_a x_a)'_a h_a\int^{\gq_0}_0 f'_a(a_0)d\gq
-(x^2_a X_a)'_a h_a\int^{\gq_0}_0 f'_a(a_1)d\gq
,\\&&
\int^\gq_{\gq_0}f d\gq
=
(3X^2_a-2X^3_a)\left[\R{h_\gq}{2}(1-2Y^3+Y^4)f_{00}
+\R{h_\gq}{2}(2y^3-y^4)f_{01}
+\R{h^2_\gq}{12}(1-Y^4-4Y^3 y)f'_{\gq00}
-\R{h^2_\gq}{12}(4y^3 Y+y^4)f'_{\gq01}\right]
\\&&
+(3x^2_a-2x^3_a)\left[\R{h_\gq}{2}(1-2Y^3+Y^4)f_{10}
+\R{h_\gq}{2}(2y^3-y^4)f_{11}
+\R{h^2_\gq}{12}(1-Y^4-4Y^3 y)f'_{\gq10}
-\R{h^2_\gq}{12}(4y^3 Y+y^4)f'_{\gq11}\right]
\\&&
+X^2_a x_a h_a\left[\R{h_\gq}{2}(1-2Y^3+Y^4)f'_{a00}
+\R{h_\gq}{2}(2y^3-y^4)f'_{a01}
+\R{h^2_\gq}{12}(1-Y^4-4Y^3 y)f''_{a\gq00}
-\R{h^2_\gq}{12}(4y^3 Y+y^4)f''_{a\gq01}\right]
\\&&
-x^2_a X_a h_a\left[\R{h_\gq}{2}(1-2Y^3+Y^4)f'_{a10}
+\R{h_\gq}{2}(2y^3-y^4)f'_{a11}
+\R{h^2_\gq}{12}(1-Y^4-4Y^3 y)f''_{a\gq10}
-\R{h^2_\gq}{12}(4y^3 Y+y^4)f''_{a\gq11}\right]
.\eqbl
#endif
}

int esigetmagfluxes_(gf,gy,a0)
  REAL *gf;	/*  */
  REAL *gy;	/*  */
  REAL *a0;	/*  */
{
  REAL *a0;		/*  .esigetmagfluxes_():escZ.c */
  REAL *gf;		/*  .esigetmagfluxes_():escZ.c */
  REAL *gy;		/*  .esigetmagfluxes_():escZ.c */
  static int i0;	/*                     :escZ.c */
  static REAL *sa;	/*                     :escZ.c */
  static REAL rha;	/*                     :escZ.c */
  static int Na;	/*                     :escZ.c */
  static int i1;	/*                     :escZ.c */
  static REAL *dgF;	/*                     :escZ.c */
  static REAL ha;	/*                     :escZ.c */
  static REAL *d2gF;	/*                     :escZ.c */
  static REAL *gF;	/*                     :escZ.c */
  static double cr3;	/*                     :escZ.c */
  static REAL *dgY;	/*                     :escZ.c */
  static REAL *d2gY;	/*                     :escZ.c */
  static REAL *gY;	/*                     :escZ.c */
  static double c2gp;	/*                     :escZ.c */
/*
   */
#ifdef Doc_esigetmagfluxes_
\Section{Skeleton of esigetmagfluxes\_()}
  \begin{verbatim}
    esigetmagfluxes_(){
    }
  \end{verbatim}
\Section{esigetmagfluxes\_()}
Routine calculates toroidal and poloidal fluxes (in [Vsec]) assuming 
$\gY(1)=0$ at the plasma boundary.
\bqbl
&&
\gF=2\gp\int^a_0a\R{\bgF'}{a}da
,\quad
\gY=2\gp\int^a_{1}a\R{\bgY'}{a}da
.\eqbl
\Subsection{Description}
Hermitain representation
\bqbl
&&
f=\left\{
 \arr
 \R{\bgF'}{a}\\
 \R{\bgY'}{a}\\
\ear
\right.
,\quad
x
=
{a-a_0\over h}
,\quad
X
=
{a_1-a\over h}
,\quad
h
\equiv
a_1-a_0
,\\&&
f(x)=
(3X^2-2X^3)f_0
+(3x^2-2x^3)f_1
+X^2xhf'_0
-x^2Xhf'_1
,\\&&
hf'(x)=
6xX(f_1-f_0)
+(X^2-2Xx)hf'_0
+(x^2-2xX)hf'_1
,\\&&
h^2f''(x)=
6(X-x)(f_1-f_0)
+(2x-4X)hf'_0
+(4x-2X)hf'_1
,\\&&
h^3f'''(x)=
-12(f_1-f_0)
+6hf'_0
+6hf'_1
.\eqbl
Polynomial representation
\bqbl
&&
f_0
=
f_0
,\quad
f'_0
=
f'_0
,\quad
\R{1}{2}h^2f''_0
=
3(f_1-f_0)
-2hf'_0
-hf'_1
,\quad
\R{1}{6}h^3f'''_0
=
-2(f_1-f_0)
+hf'_0
+hf'_1
,\\&&
f(x)=
f_0+hf'_0x+\R{h^2}{2}f''_0x^2+\R{h^3}{6}f'''_0x^3
.\eqbl
Integration of Hermit polynomials with a weight $a$
\bqbl
&&
\int^a_{a_0}afda
=
a_0h\int^x_0 fdx
+h^2\int^x_0xfdx
,\\&&
a_0h\int^x_0 fdx
=
a_0hx\left(f_0+\R{h}{2}f'_0x+\R{h^2}{6}f''_0x^2+\R{h^3}{24}f'''_0x^3\right)
,\\&&
h^2\int^x_0xfdx
=
h^2x^2\left[\R{1}{2}f_0+\R{h}{3}f'_0x+\R{h^2}{8}f''_0x^2
 +\R{h^3}{30}f'''_0x^3\right]
,\\&&
h^2f''_0=
6(f_1-f_0)-4hf'_0-2hf'_1
,\\&&
h^3f'''_0=
-12(f_1-f_0)+6hf'_0+6hf'_1
 .\eqbl
Finally
\bqbl
&&
\int^a_{a_0}afda
=
a_0hx\left(f_0+\R{h}{2}f'_0x+\R{h^2}{6}f''_0x^2+\R{h^3}{24}f'''_0x^3\right)
+h^2x^2\left[\R{1}{2}f_0+\R{h}{3}f'_0x+\R{h^2}{8}f''_0x^2
 +\R{h^3}{30}f'''_0x^3\right]
,\\&&
h^2f''_0=
6(f_1-f_0)-4hf'_0-2hf'_1
,\\&&
h^3f'''_0=
-12(f_1-f_0)+6hf'_0+6hf'_1
 .\eqbl

\Subsection{Algorithm}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int esigetrzb_(R,Z,Bm,a0,q0,n)
  REAL *R;	/*  */
  REAL *Z;	/*  */
  REAL *Bm;	/*  */
  REAL *a0;	/*  */
  REAL *q0;	/*  */
  int *n;	/*  */
{
  int *n;		/*  .esigetrzb_():escZ.c */
  REAL *a0;		/*  .esigetrzb_():escZ.c */
  REAL *q0;		/*  .esigetrzb_():escZ.c */
  REAL *R;		/*  .esigetrzb_():escZ.c */
  REAL *Z;		/*  .esigetrzb_():escZ.c */
  REAL *Bm;		/*  .esigetrzb_():escZ.c */
  static int i0;	/*               :escZ.c */
  static REAL *sa;	/*               :escZ.c */
  static REAL rha;	/*               :escZ.c */
  static int Na;	/*               :escZ.c */
  static int i1;	/*               :escZ.c */
  static REAL XF;	/*               :escZ.c */
  static REAL XD;	/*               :escZ.c */
  static REAL ha;	/*               :escZ.c */
  static REAL xF;	/*               :escZ.c */
  static REAL xD;	/*               :escZ.c */
  static REAL dX;	/*               :escZ.c */
  static REAL dXD;	/*               :escZ.c */
  static REAL dxD;	/*               :escZ.c */
  static REAL crgq0;	/*               :escZ.c */
  static REAL cgq0;	/*               :escZ.c */
  static REAL rhq;	/*               :escZ.c */
  static int Np;	/*               :escZ.c */
  static REAL *gq;	/*               :escZ.c */
  static REAL YF;	/*               :escZ.c */
  static REAL YD;	/*               :escZ.c */
  static REAL hq;	/*               :escZ.c */
  static REAL yF;	/*               :escZ.c */
  static REAL yD;	/*               :escZ.c */
  static REAL dY;	/*               :escZ.c */
  static REAL dYD;	/*               :escZ.c */
  static REAL dyD;	/*               :escZ.c */
  static int j00;	/*               :escZ.c */
  static int Np1;	/*               :escZ.c */
  static int j01;	/*               :escZ.c */
  static int j10;	/*               :escZ.c */
  static int j11;	/*               :escZ.c */
  static REAL *sr;	/*               :escZ.c */
  static REAL *sra;	/*               :escZ.c */
  static REAL *srq;	/*               :escZ.c */
  static REAL *sraq;	/*               :escZ.c */
  static REAL *sz;	/*               :escZ.c */
  static REAL *sza;	/*               :escZ.c */
  static REAL *szq;	/*               :escZ.c */
  static REAL *szaq;	/*               :escZ.c */
  static REAL *aB;	/*               :escZ.c */
  static REAL *aBa;	/*               :escZ.c */
  static REAL *aBq;	/*               :escZ.c */
  static REAL *aBaq;	/*               :escZ.c */
/*
   */
#ifdef Doc_esigetrzb_
\section{Skeleton of esigetrzb_()}
  \begin{verbatim}
    esigetrzb_(){
    }
  \end{verbatim}
\section{esigetrzb_()}
#endif
}

int gcmotion_(dXgr,dXa,dXgq,dXgf,Xgr,Xa,Xgq,Xgm,n)
  REAL *dXgr;	/*  */
  REAL *dXa;	/*  */
  REAL *dXgq;	/*  */
  REAL *dXgf;	/*  */
  REAL *Xgr;	/*  */
  REAL *Xa;	/*  */
  REAL *Xgq;	/*  */
  REAL *Xgm;	/*  */
  int *n;	/*  */
{
  int *n;		/*  .gcmotion_():escZ.c */
  REAL *Xa;		/*  .gcmotion_():escZ.c */
  REAL *Xgq;		/*  .gcmotion_():escZ.c */
  REAL *Xgr;		/*  .gcmotion_():escZ.c */
  REAL *Xgm;		/*  .gcmotion_():escZ.c */
  REAL *dXgr;		/*  .gcmotion_():escZ.c */
  REAL *dXgf;		/*  .gcmotion_():escZ.c */
  REAL *dXa;		/*  .gcmotion_():escZ.c */
  REAL *dXgq;		/*  .gcmotion_():escZ.c */
  static REAL crgq0;	/*              :escZ.c */
  static REAL cgq0;	/*              :escZ.c */
  static REAL rhq;	/*              :escZ.c */
  static int Np;	/*              :escZ.c */
  static REAL *gq;	/*              :escZ.c */
  static REAL YF;	/*              :escZ.c */
  static REAL YD;	/*              :escZ.c */
  static REAL hq;	/*              :escZ.c */
  static REAL yF;	/*              :escZ.c */
  static REAL yD;	/*              :escZ.c */
  static REAL dY;	/*              :escZ.c */
  static REAL dYD;	/*              :escZ.c */
  static REAL dyD;	/*              :escZ.c */
  static int i0;	/*              :escZ.c */
  static REAL *sa;	/*              :escZ.c */
  static REAL rha;	/*              :escZ.c */
  static int Na;	/*              :escZ.c */
  static int i1;	/*              :escZ.c */
  static REAL XF;	/*              :escZ.c */
  static REAL XD;	/*              :escZ.c */
  static REAL ha;	/*              :escZ.c */
  static REAL xF;	/*              :escZ.c */
  static REAL xD;	/*              :escZ.c */
  static REAL dX;	/*              :escZ.c */
  static REAL dXD;	/*              :escZ.c */
  static REAL dxD;	/*              :escZ.c */
  static int j00;	/*              :escZ.c */
  static int Np1;	/*              :escZ.c */
  static int j01;	/*              :escZ.c */
  static int j10;	/*              :escZ.c */
  static int j11;	/*              :escZ.c */
  static REAL *sr;	/*              :escZ.c */
  static REAL *sra;	/*              :escZ.c */
  static REAL *srq;	/*              :escZ.c */
  static REAL *sraq;	/*              :escZ.c */
  static REAL *sz;	/*              :escZ.c */
  static REAL *sza;	/*              :escZ.c */
  static REAL *szq;	/*              :escZ.c */
  static REAL *szaq;	/*              :escZ.c */
  static REAL *aB;	/*              :escZ.c */
  static REAL *aBa;	/*              :escZ.c */
  static REAL *aBq;	/*              :escZ.c */
  static REAL *aBaq;	/*              :escZ.c */
  static REAL *aF;	/*              :escZ.c */
  static REAL *daF;	/*              :escZ.c */
  static REAL *dgY;	/*              :escZ.c */
  static REAL *d2gY;	/*              :escZ.c */
  static REAL *aT;	/*              :escZ.c */
  static REAL *daT;	/*              :escZ.c */
  static REAL *aP;	/*              :escZ.c */
  static REAL *daP;	/*              :escZ.c */
  static int *sw;	/*              :escZ.c */
/*
   */
#ifdef Doc_gcmotion_
\section{Skeleton of gcmotion\_()}
  \begin{verbatim}
    gcmotion\_(){
    }
  \end{verbatim}
\section{gcmotion\_()}
Using the matrix
\begin{eqnarray}
&&
\maA
\equiv
\left|
\begin{array}{rr}
\db a_{11}
\da a_{12}
\dd a_{21}
\da a_{22}
\end{array}
\right|
,\quad
|\maA|
\equiv
a_{11}a_{22}-a_{12}a_{21}
=
J[|B|^2+\gr_\parallel(\waB\cdot(\na\times\waB))]
,\end{eqnarray}
where
\bqbl
&&
a_{11}\equiv B_\gq
,\quad a_{12}\equiv J(B^\gf+\gr_\parallel j^\gf)
,\quad a_{21}\equiv B_\gf
,\quad a_{22}\equiv-J(B^\gq+\gr_\parallel j^\gq)
,\eqbl
the equation of drift motion can be written in the explicit form
\begin{eqnarray}
&&
\left|
\begin{array}{l}
\db\d{\gr}_\parallel
\dd\dd
\dd\d{a}
\dd\dd
\dd\d{\gq}
\dd\dd
\dd\d{\gf}
\end{array}
\right|
=
\left|
\begin{array}{rrrr}
\db0
\da0
\da \R{a_{22}}{|\maA|}
\da -\R{a_{12}}{|\maA|}
\dd
\dd0
\da0
\da -\R{a_{21}}{|\maA|}
\da \R{a_{11}}{|\maA|}
\dd
\dd \R{a_{22}}{|\maA|}
\da -\R{a_{21}}{|\maA|}
\da0
\da-\R{B_a}{|\maA|}
\dd
\dd -\R{a_{12}}{|\maA|}
\da \R{a_{11}}{|\maA|}
\da\R{B_a}{|\maA|}
\da0
\end{array}
\right|
\cdot
\left|
\begin{array}{r}
\db\R{\pa H}{\pa\gr_\parallel}
\dd
\dd\R{\pa H}{\pa a}
\dd
\dd-\R{\pa H}{\pa\gq}
\dd
\dd-\R{\pa H}{\pa\gf}
\end{array}
\right|
.\end{eqnarray}
Near the axis
\bqbl
&&
a_{11}
=
B_\gq
=
a^2K\R{\bgY'}{a}
,\quad
a_{12}=arD(B^\gf+\gr_\parallel j^\gf)
,\quad a_{21}\equiv B_\gf
,\quad a_{22}\equiv-arD(B^\gq+\gr_\parallel j^\gq)
.\eqbl
Transition to local $x,y$
\bqbl
&&
x
\equiv a\cos\gq
,\quad
y
\equiv a\sin\gq
,\\&&
\d{x}
=
\d{a}\cos\gq-a\sin\gq\d{\gq}
,\quad
\d{y}
=
\d{a}\sin\gq+a\cos\gq\d{\gq}
,\\&&
\d{a}
=
\R{a_{21}}{|\maA|}\R{\pa H}{\pa\gq}
,\quad
\d{\gq}
=
\R{a_{22}}{|\maA|}\R{\pa H}{\pa\gr_\parallel}
-\R{a_{21}}{|\maA|}\R{\pa H}{\pa a}
.\eqbl
#endif
}

int MFieldLines(dXgq,dXgf,Xa,Xgq,n)
  REAL *dXgq;	/*  */
  REAL *dXgf;	/*  */
  REAL *Xa;	/*  */
  REAL *Xgq;	/*  */
  int n;	/*  */
{
  int n;		/*  .MFieldLines():escZ.c */
  REAL *Xa;		/*  .MFieldLines():escZ.c */
  REAL *Xgq;		/*  .MFieldLines():escZ.c */
  REAL *dXgq;		/*  .MFieldLines():escZ.c */
  REAL *dXgf;		/*  .MFieldLines():escZ.c */
  static REAL crgq0;	/*                :escZ.c */
  static REAL cgq0;	/*                :escZ.c */
  static REAL rhq;	/*                :escZ.c */
  static int Np;	/*                :escZ.c */
  static REAL *gq;	/*                :escZ.c */
  static REAL YF;	/*                :escZ.c */
  static REAL YD;	/*                :escZ.c */
  static REAL hq;	/*                :escZ.c */
  static REAL yF;	/*                :escZ.c */
  static REAL yD;	/*                :escZ.c */
  static REAL dY;	/*                :escZ.c */
  static REAL dYD;	/*                :escZ.c */
  static REAL dyD;	/*                :escZ.c */
  static int i0;	/*                :escZ.c */
  static REAL *sa;	/*                :escZ.c */
  static REAL rha;	/*                :escZ.c */
  static int Na;	/*                :escZ.c */
  static int i1;	/*                :escZ.c */
  static REAL XF;	/*                :escZ.c */
  static REAL XD;	/*                :escZ.c */
  static REAL ha;	/*                :escZ.c */
  static REAL xF;	/*                :escZ.c */
  static REAL xD;	/*                :escZ.c */
  static REAL dX;	/*                :escZ.c */
  static REAL dXD;	/*                :escZ.c */
  static REAL dxD;	/*                :escZ.c */
  static int j00;	/*                :escZ.c */
  static int Np1;	/*                :escZ.c */
  static int j01;	/*                :escZ.c */
  static int j10;	/*                :escZ.c */
  static int j11;	/*                :escZ.c */
  static REAL *sr;	/*                :escZ.c */
  static REAL *sra;	/*                :escZ.c */
  static REAL *srq;	/*                :escZ.c */
  static REAL *sraq;	/*                :escZ.c */
  static REAL *sz;	/*                :escZ.c */
  static REAL *sza;	/*                :escZ.c */
  static REAL *szq;	/*                :escZ.c */
  static REAL *szaq;	/*                :escZ.c */
  static REAL *aB;	/*                :escZ.c */
  static REAL *aBa;	/*                :escZ.c */
  static REAL *aBq;	/*                :escZ.c */
  static REAL *aBaq;	/*                :escZ.c */
  static REAL *aF;	/*                :escZ.c */
  static REAL *daF;	/*                :escZ.c */
  static REAL *dgY;	/*                :escZ.c */
  static REAL *d2gY;	/*                :escZ.c */
/*
   */
#ifdef Doc_MFieldLines
\section{Skeleton of MFieldLines()}
  \begin{verbatim}
    MFieldLines(){
    }
  \end{verbatim}
\section{MFieldLines()}
Magnetic field line equations
\bqbl
&&
\R{d\gq}{B^\gq}
=
\R{d\gf}{B^\gf}
,\quad
B^\gf
=\R{\baF}{r^2}
,\quad
B^\gq
=
-\R{\bgY'}{rD}
,\\&&
d\gq=\ga B^\gq
,\quad
d\gf=\ga B^\gf
,\quad
dl
=
\ga \q{g_{\gq\gq}B^\gq B^\gq+r^2B^\gf B^\gf}
,\\&&
d\gq=dl\R{B^\gq}{\q{g_{\gq\gq}B^\gq B^\gq+r^2B^\gf B^\gf}}
,\quad
d\gf=dl\R{B^\gf}{\q{g_{\gq\gq}B^\gq B^\gq+r^2B^\gf B^\gf}}
.\eqbl
#endif
}

void esigetpressure_(p,dp,a,n)
  double *p;	/*  */
  double *dp;	/*  */
  double *a;	/*  */
  int *n;	/*  */
{
  int *n;		/*  .esigetpressure_():escZ.c */
  double *a;		/*  .esigetpressure_():escZ.c */
  double *p;		/*  .esigetpressure_():escZ.c */
  double *dp;		/*  .esigetpressure_():escZ.c */
  static REAL *aP;	/*                    :escZ.c */
  static REAL *dgY;	/*                    :escZ.c */
  static int Na1;	/*                    :escZ.c */
  static REAL *sa;	/*                    :escZ.c */
  static REAL *daP;	/*                    :escZ.c */
  static REAL *d2gY;	/*                    :escZ.c */
  static REAL ha;	/*                    :escZ.c */
  static int Na;	/*                    :escZ.c */
  static REAL rha;	/*                    :escZ.c */
/*
   */
#ifdef Doc_esigetpressure_
\section{Skeleton of esigetpressure_()}
  \begin{verbatim}
    esigetpressure_(){
    }
  \end{verbatim}
\section{esigetpressure_()}
\section{Routine calls for getting collision parmeters}
  FORTRAN calls:
  \begin{verbatim}
    call setparticletype('e','P')  /* First character argument: test particle;
                                     Second character argument: backgroud part;

                                       Type  of particles:
                                       'e' -electrons;
                                       'H' -protons;
                                       'P' -protons;
                                       'D' -deutrons;
                                       'T' -tritons;
                                       'A' -alphas.
                                       */
    call esigetdensity(ne,dne,Te,Ti,a) /* Arguments:
                                         ne - density [10^20/m^3] (output);
                                         dne- derivative with respect to a;
                                         Te - electron temperature [keV];
                                         Ti - ion temperature [keV];
                                         a  - normalized radial coordinate;
                                         */
    call getcollisions(gn,E,T,n)      /* Arguments:
                                         gn -Collision grequency;
                                         E  -Test particle Energy [keV];
                                         T  -background temperature [keV];
                                         n  -background density [10^{20}/m^3]
                                         */
  \end{verbatim}

#endif
}
static double cme;	/*  */
static double cmi;	/*  */
static double cc;	/*  */
static double ce;	/*  */
static double cEeV;	/*  */
static double cEkeV;	/*  */
static char tga;	/*  */
static char tgb;	/*  */
static double sega;	/*  */
static double smga;	/*  */
static double gmga;	/*  */
static double Zga;	/*  */
static double pZei;	/*  */
static double segb;	/*  */
static double smgb;	/*  */
static double gmgb;	/*  */
static double Zgb;	/*  */

void esigetdensity_(ne,dne,Te,Ti,a)
  double *ne;	/*  */
  double *dne;	/*  */
  double *Te;	/*  */
  double *Ti;	/*  */
  double *a;	/*  */
{
  double *a;		/*  .esigetdensity_():escZ.c */
  double *Te;		/*  .esigetdensity_():escZ.c */
  double *Ti;		/*  .esigetdensity_():escZ.c */
  double *ne;		/*  .esigetdensity_():escZ.c */
  double *dne;		/*  .esigetdensity_():escZ.c */
  static REAL *sa;	/*                   :escZ.c */
  static REAL rha;	/*                   :escZ.c */
  static int Na;	/*                   :escZ.c */
  static REAL ha;	/*                   :escZ.c */
  static REAL *sne;	/*                   :escZ.c */
  static REAL *dsne;	/*                   :escZ.c */
/*
   */
#ifdef Doc_esigetdensity_
\section{Skeleton of esigetdensity_()}
  \begin{verbatim}
    esigetdensity_(){
    }
  \end{verbatim}
\section{esigetdensity_()}
#endif
}

void GetEECollisions(gn,E,Te,ne)
  double *gn;	/*  */
  double *E;	/*  */
  double *Te;	/*  */
  double *ne;	/*  */
{
  double *E;		/*  .GetEECollisions():escZ.c */
  double *Te;		/*  .GetEECollisions():escZ.c */
  double *ne;		/*  .GetEECollisions():escZ.c */
  double *gn;		/*  .GetEECollisions():escZ.c */
  static double cEkeV;	/*                    :escZ.c */
  static double smga;	/*                    :escZ.c */
  static double smgb;	/*                    :escZ.c */
  static double tF1;	/*                    :escZ.c */
  static double cr2;	/*                    :escZ.c */
  static double rhF;	/*                    :escZ.c */
  static double F0[];	/*                    :escZ.c */
  static double F1[];	/*                    :escZ.c */
  static double F2[];	/*                    :escZ.c */
  static double F3[];	/*                    :escZ.c */
  static double c2rqgp;	/*                    :escZ.c */
  static double sega;	/*                    :escZ.c */
  static double segb;	/*                    :escZ.c */
  static double cgp;	/*                    :escZ.c */
/*
   */
#ifdef Doc_GetEECollisions
\section{Skeleton of GetEECollisions()}
  \begin{verbatim}
    GetEECollisions(){
    }
  \end{verbatim}
\section{GetEECollisions()}
Transverse diffusion \cite{NRL}
\bqbl
&&
\R{d(\wsv_\ga-\bar{\wsv}_\ga)^2_\perp}{dt}
=
\gn^{\ga/\gb}_\perp v^2_\ga
,\quad
\gn^{\ga/\gb}_\perp
=
2\left[
 \left(1-\R{1}{2x^{\ga/\gb}}\right)\gy(x^{\ga/\gb})+\gy'(x^{\ga/\gb})
\right]\gn^{\ga/\gb}_0
,\\&&
\gn^{\ga/\gb}_0
=
4\gp \R{e^2_\ga e^2_\gb}{m^2_\ga}\gl_{\ga\gb}\R{n_\gb}{v^3_\ga}
,\quad
x^{\ga/\gb}
=
\R{m_\gb v^2_\ga}{2kT_\gb}
,\\&&
\gy(x)
=
\R{2}{\q{\gp}}\int^x_0\q{t}e^{-t}dt
,\quad
\gy'(x)
=
\R{d\gy}{dx}
.\eqbl
Coulumb Logarithm for electron-electron collisions
\bqbl
&&
\gl_{ee}
=
\left\{\parbox{1.9in}{
$\db23-\ln\left(\q{\R{n_e}{T^3_e}}\right),\quad T_e<10\ \mbox{eV}$
\\
$24-\ln\left(\R{\q{n_e}}{T_e}\right),\quad T_e>10\ \mbox{eV}$}
\right.
.\eqbl

\begin{thebibliography}{10}
\bibitem{NRL}
NRP Plasma Formulary. NRL Publication 177-4405, p.31 (1990). 
\end{thebibliography}
#endif
}

void GetIECollisions(gn,E,Te,ne)
  double *gn;	/*  */
  double *E;	/*  */
  double *Te;	/*  */
  double *ne;	/*  */
{
  double *E;		/*  .GetIECollisions():escZ.c */
  double *Te;		/*  .GetIECollisions():escZ.c */
  double *ne;		/*  .GetIECollisions():escZ.c */
  double *gn;		/*  .GetIECollisions():escZ.c */
  static double cEkeV;	/*                    :escZ.c */
  static double smga;	/*                    :escZ.c */
  static double smgb;	/*                    :escZ.c */
  static double tF1;	/*                    :escZ.c */
  static double cr2;	/*                    :escZ.c */
  static double rhF;	/*                    :escZ.c */
  static double F0[];	/*                    :escZ.c */
  static double F1[];	/*                    :escZ.c */
  static double F2[];	/*                    :escZ.c */
  static double F3[];	/*                    :escZ.c */
  static double c2rqgp;	/*                    :escZ.c */
  static double pZei;	/*                    :escZ.c */
  static double sega;	/*                    :escZ.c */
  static double segb;	/*                    :escZ.c */
  static double cgp;	/*                    :escZ.c */
/*
   */
#ifdef Doc_GetIECollisions
\section{Skeleton of GetIECollisions()}
  \begin{verbatim}
    GetIECollisions(){
    }
  \end{verbatim}
\section{GetIECollisions()}
Transverse diffusion \cite{NRL}
\bqbl
&&
\R{d(\wsv_\ga-\bar{\wsv}_\ga)^2_\perp}{dt}
=
\gn^{\ga/\gb}_\perp v^2_\ga
,\quad
\gn^{\ga/\gb}_\perp
=
2\left[
 \left(1-\R{1}{2x^{\ga/\gb}}\right)\gy(x^{\ga/\gb})+\gy'(x^{\ga/\gb})
\right]\gn^{\ga/\gb}_0
,\\&&
\gn^{\ga/\gb}_0
=
4\gp \R{e^2_\ga e^2_\gb}{m^2_\ga}\gl_{\ga\gb}\R{n_\gb}{v^3_\ga}
,\quad
x^{\ga/\gb}
=
\R{m_\gb v^2_\ga}{2kT_\gb}
,\\&&
\gy(x)
=
\R{2}{\q{\gp}}\int^x_0\q{t}e^{-t}dt
,\quad
\gy'(x)
=
\R{d\gy}{dx}
.\eqbl
Electron-ion Collisions
\bqbl
&&
\gl_{ei}
=
\left\{\parbox{2.8in}{
$\db23-\ln\left(Z\q{\R{n_e}{T^3_e}}\right)
,\quad T_i\R{m_e}{m_i} < T_e < 10Z^2\ \mbox{eV}$
\\
$\db24-\ln\left(\R{\q{n_e}}{T_e}\right)
,\quad T_i\R{m_e}{m_i} < 10Z^2\ \mbox{eV} < T_e$
\\
$\db30-\ln\left(\R{Z^2}{\gm}\q{\R{n_i}{T_i^3}}\right)
,\quad T_e < T_iZ\R{m_e}{m_i}$
}
\right.
,\quad
\gm
=
\R{m_i}{m_p}
.\eqbl
\begin{thebibliography}{10}
\bibitem{NRL}
NRP Plasma Formulary. NRL Publication 177-4405, p.31 (1990). 
\end{thebibliography}
#endif
}

void GetEPCollisions(gn,E,Te,Ti,ne)
  double *gn;	/*  */
  double *E;	/*  */
  double *Te;	/*  */
  double *Ti;	/*  */
  double *ne;	/*  */
{
  double *E;		/*  .GetEPCollisions():escZ.c */
  double *Te;		/*  .GetEPCollisions():escZ.c */
  double *ne;		/*  .GetEPCollisions():escZ.c */
  double *gn;		/*  .GetEPCollisions():escZ.c */
  static double cEkeV;	/*                    :escZ.c */
  static double smga;	/*                    :escZ.c */
  static double smgb;	/*                    :escZ.c */
  static double tF1;	/*                    :escZ.c */
  static double cr2;	/*                    :escZ.c */
  static double rhF;	/*                    :escZ.c */
  static double F0[];	/*                    :escZ.c */
  static double F1[];	/*                    :escZ.c */
  static double F2[];	/*                    :escZ.c */
  static double F3[];	/*                    :escZ.c */
  static double c2rqgp;	/*                    :escZ.c */
  static double sega;	/*                    :escZ.c */
  static double segb;	/*                    :escZ.c */
  static double cgp;	/*                    :escZ.c */
/*
   */
#ifdef Doc_GetEPCollisions
\section{Skeleton of GetEPCollisions()}
  \begin{verbatim}
    GetEPCollisions(){
    }
  \end{verbatim}
\section{GetEPCollisions()}
#endif
}

void GetIICollisions(gn,E,Ti,ne)
  double *gn;	/*  */
  double *E;	/*  */
  double *Ti;	/*  */
  double *ne;	/*  */
{
  double *E;		/*  .GetIICollisions():escZ.c */
  double *Ti;		/*  .GetIICollisions():escZ.c */
  double *ne;		/*  .GetIICollisions():escZ.c */
  double *gn;		/*  .GetIICollisions():escZ.c */
  static double cEkeV;	/*                    :escZ.c */
  static double smga;	/*                    :escZ.c */
  static double smgb;	/*                    :escZ.c */
  static double tF1;	/*                    :escZ.c */
  static double cr2;	/*                    :escZ.c */
  static double rhF;	/*                    :escZ.c */
  static double F0[];	/*                    :escZ.c */
  static double F1[];	/*                    :escZ.c */
  static double F2[];	/*                    :escZ.c */
  static double F3[];	/*                    :escZ.c */
  static double c2rqgp;	/*                    :escZ.c */
  static double Zga;	/*                    :escZ.c */
  static double Zgb;	/*                    :escZ.c */
  static double gmga;	/*                    :escZ.c */
  static double gmgb;	/*                    :escZ.c */
  static double sega;	/*                    :escZ.c */
  static double segb;	/*                    :escZ.c */
  static double cgp;	/*                    :escZ.c */
/*
   */
#ifdef Doc_GetIICollisions
\section{Skeleton of GetIICollisions()}
  \begin{verbatim}
    GetIICollisions(){
    }
  \end{verbatim}
\section{GetIICollisions()}
#endif
}

void getcollisions_(gn,E,Tp,n)
  double *gn;	/*  */
  double *E;	/*  */
  double *Tp;	/*  */
  double *n;	/*  */
{
  double *gn;			/*  .getcollisions_():escZ.c */
  double *E;			/*  .getcollisions_():escZ.c */
  double *Tp;			/*  .getcollisions_():escZ.c */
  double *n;			/*  .getcollisions_():escZ.c */
  static char tga;		/*                   :escZ.c */
  static char tgb;		/*                   :escZ.c */
  static double pZei;		/*                   :escZ.c */
  static double Zgb;		/*                   :escZ.c */
  static double Zga;		/*                   :escZ.c */
  void GetEECollisions();	/*                   :escZ.c */
  void GetIECollisions();	/*                   :escZ.c */
  void GetIICollisions();	/*                   :escZ.c */
/*
   */
#ifdef Doc_getcollisions_
\section{Skeleton of getcollisions_()}
  \begin{verbatim}
    getcollisions_(){
    }
  \end{verbatim}
\section{getcollisions_()}
#endif
}

int setparticletype_(ega,egb)
  char *ega;	/*  */
  char *egb;	/*  */
{
  char *ega;		/*  .setparticletype_():escZ.c */
  char *egb;		/*  .setparticletype_():escZ.c */
  static char tga;	/*                     :escZ.c */
  static char tgb;	/*                     :escZ.c */
  static double gmga;	/*                     :escZ.c */
  static double cme;	/*                     :escZ.c */
  static double cmi;	/*                     :escZ.c */
  static double Zga;	/*                     :escZ.c */
  static double sega;	/*                     :escZ.c */
  static double ce;	/*                     :escZ.c */
  static double smga;	/*                     :escZ.c */
  static double gmgb;	/*                     :escZ.c */
  static double Zgb;	/*                     :escZ.c */
  static double segb;	/*                     :escZ.c */
  static double smgb;	/*                     :escZ.c */
/*
   */
#ifdef Doc_setparticletype_
\section{Skeleton of setparticletype\_()}
  \begin{verbatim}
    setparticletype_(){
    }
  \end{verbatim}
\section{setparticletype\_()}
Transverse diffusion
\bqbl
&&
\R{d(\wsv_\ga-\bar{\wsv}_\ga)^2_\perp}{dt}
=
\gn^{\ga/\gb}_\perp v^2_\ga
,\quad
\gn^{\ga/\gb}_\perp
=
2\left[
 \left(1-\R{1}{2x^{\ga/\gb}}\right)\gy(x^{\ga/\gb})+\gy'(x^{\ga/\gb})
\right]\gn^{\ga/\gb}_0
,\\&&
\gn^{\ga/\gb}_0
=
4\gp \R{e^2_\ga e^2_\gb}{m^2_\ga}\gl_{\ga\gb}\R{n_\gb}{v^3_\ga}
,\quad
x^{\ga/\gb}
=
\R{m_\gb v^2_\ga}{2kT_\gb}
,\\&&
\gy(x)
=
\R{2}{\q{\gp}}\int^x_0\q{t}e^{-t}dt
,\quad
\gy'(x)
=
\R{d\gy}{dx}
.\eqbl
Coulumb Logarithm for electron-electron collisions
\bqbl
&&
\gl_{ee}
=
\left\{\parbox{1.9in}{
$\db23-\ln\left(\q{\R{n_e}{T^3_e}}\right),\quad T_e<10\ \mbox{eV}$
\\
$24-\ln\left(\R{\q{n_e}}{T_e}\right),\quad T_e>10\ \mbox{eV}$}
\right.
.\eqbl
Electron-ion Collisions
\bqbl
&&
\gl_{ei}
=
\left\{\parbox{2.8in}{
$\db23-\ln\left(Z\q{\R{n_e}{T^3_e}}\right)
,\quad T_i\R{m_e}{m_i} < T_e < 10Z^2\ \mbox{eV}$
\\
$\db24-\ln\left(\R{\q{n_e}}{T_e}\right)
,\quad T_i\R{m_e}{m_i} < 10Z^2\ \mbox{eV} < T_e$
\\
$\db30-\ln\left(\R{Z^2}{\gm}\q{\R{n_i}{T_i^3}}\right)
,\quad T_e < T_iZ\R{m_e}{m_i}$
}
\right.
,\quad
\gm
=
\R{m_i}{m_p}
.\eqbl
Mixed ion-ion collisions
\bqbl
&&
\gl_{ij}
=
23-\ln\left(\R{Z_iZ_j(\gm_i+\gm_j)}{\gm_iT_j+\gm_jT_i}
 \q{\R{n_iZ^2_i}{T_i}+\R{n_jZ^2_j}{T_j}}\right)
.\eqbl
Counterstreaming ions
\bqbl
&&
v_D
=
\gb_Dc
,\quad
\R{kT_i}{m_i},\R{kT_j}{m_j}
<
v^2_D
<
\R{kT_e}{m_e}
,\\&&
\gl_{ij}
=
35-\ln\left(\R{Z_iZ_j(\gm_i+\gm_j)}{\gm_i\gm_j\gb^2_D}\q{\R{n_e}{T_e}}\right)
.\eqbl
#endif
}

int rzmotion_(dR,dZ,dgf,R,Z,gf,n)
  REAL *dR;	/*  */
  REAL *dZ;	/*  */
  REAL *dgf;	/*  */
  REAL *R;	/*  */
  REAL *Z;	/*  */
  REAL *gf;	/*  */
  int *n;	/*  */
{
  int *n;		/*  .rzmotion_():escZ.c */
  REAL *R;		/*  .rzmotion_():escZ.c */
  REAL *Z;		/*  .rzmotion_():escZ.c */
  REAL *dR;		/*  .rzmotion_():escZ.c */
  REAL *dZ;		/*  .rzmotion_():escZ.c */
  REAL *dgf;		/*  .rzmotion_():escZ.c */
  static double rBox[];	/*              :escZ.c */
  static double zBox[];	/*              :escZ.c */
  static double rdrMap;	/*              :escZ.c */
  static int nrMap;	/*              :escZ.c */
  static int irMap;	/*              :escZ.c */
  static double rdzMap;	/*              :escZ.c */
  static int nzMap;	/*              :escZ.c */
  static int izMap;	/*              :escZ.c */
  static int *iAMap;	/*              :escZ.c */
  static int k00;	/*              :escZ.c */
  static int *k00Map;	/*              :escZ.c */
  static int k01;	/*              :escZ.c */
  static int k10;	/*              :escZ.c */
  static int nrMap1;	/*              :escZ.c */
  static int k11;	/*              :escZ.c */
  static double *f0Map;	/*              :escZ.c */
  static double *fxMap;	/*              :escZ.c */
  static double *fyMap;	/*              :escZ.c */
  static double *f2Map;	/*              :escZ.c */
  /*
   */
#ifdef Doc_rzmotion_
\Section{Skeleton of \c{rzmotion\_()}}
  \begin{verbatim}
    rzmotion\_(){
    }
  \end{verbatim}
\Section{\c{rzmotion\_()}}
\Subsection{Description}
Routine determines time derivatives of cylindrical coordinates of a
particle in a magnetic field
\bqbl
&&
m\R{d\wsv}{dt}=\R{e}{c}\wsv\times\waB
,\quad
\gt\equiv \gW t
,\quad
\gW\equiv \R{eB_0}{mc}
,\quad
\R{d\wsv}{d\gt}=\wsv\times\b\waB
,\quad
\b\waB\equiv\R{\waB}{B_0}
,\\&&
\wsr=r\cos\gf\wse_x+r\sin\gf\wse_y+z\wse_z
,\quad
v_z=\R{dz}{dt}
,\quad
v_r=\R{dr}{dt}
,\quad
v_\gf
=
r\R{d\gf}{dt}
,\\&&
\wsv=v_r\wse_r+v_\gf\wse_\gf+v_z\wse_z
,\quad
\R{d\wsv}{dt}=
\R{dv_r}{dt}\wse_r+\R{dv_\gf}{dt}\wse_\gf+\R{dv_z}{dt}\wse_z
+v_r\R{d\gf}{dt}\wse_\gf-v_\gf\R{d\gf}{dt}\wse_r
.\eqbl
Equations of motion
\bqbl
&&
\R{dv_r}{dt}-\R{v^2_\gf}{r}
=
v_\gf\baB_z-v_z\baB_\gf
,\quad
r\R{d^2\gf}{dt^2}
+2v_r\R{d\gf}{dt}
=
v_z\baB_r-v_r\baB_z
,\quad
\R{dv_z}{dt}
=
v_r\baB_\gf-v_\gf\baB_r
,\\&&
\R{dz}{dt}=v_z
,\quad
\R{dr}{dt}=v_r
,\quad
\R{d\gf}{dt}
=
\R{v_\gf}{r}
.\eqbl

\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}
#endif

#undef Doc_Unused_Doc_Information /* ------------{ */
#ifdef Doc_Unused_Doc_Information
#ifndef blb_escZ.c
#ifdef Doc_escZ.c
\title{ESI: a Bicubic Spline Interface for Gyrokinetic Codes in Toroidal
Geometry.} 
\author{Leonid E. Zakharov, Roscoe White, Alex Pletzer\\
{\em Princeton University, Pinceton Plasma Physics Laboratory}}
\date{October 17, 2002}
\maketitle
\begin{abstract}
A compact and comprehensive computer program interface for exchanging
data between magnetohydrodynamic equilibrium codes with gyrokinetic and
particle orbit code codes is presented. The interface, which is  written
in C and accessible from Fortran, communicates via a file format that
contains the bicubic spline coefficients of a minimal set of flux
coordinate functions. This allows the reconstruction of basic functions
and their derivatives at any position in the plasma. All vector fields
and geometric coefficients required to integrate the guiding center
trajectories can be derived using algebraic operations only. We
anticipate this interface to find widespread applications for stability
and transport calculations as well.
\end{abstract} 
\tableofcontents

\Section{Introduction}
In plasma simulations, which deal with highly anisotropic equations,
the equilibrium codes play a fundamental role in supplying the information
about the background plasma state in flux coordinates related to the 
geometry of the magnetic field. Then, other, e.g., stability, transport, 
particle orbit or gyro-kinetic codes use this information for simulating
plasma processes with more sophisticated plasma models.

While the equilibrium configuration depends essentially only on two
radial profiles and the shape of the plasma boundary, the request of
other codes may contain numerous 2- or 3-dimensional functions
representing, e.g., components of the magnetic fields, current density, 
curvature of the field lines or the magnitude of the magnetic field and
its derivatives. All of these functions are inter-related, and because
of finite accuracy of the equilibrium codes it is always a matter of
decision of what should be taken as input data and what should be
calculated inside the ``client'' codes.

There are two extreme approaches to making such a decision. Both rely 
on the exchange of data files. One of them requires all necessary
functions at all necessary positions to be supplied by the equilibrium
codes. Working only for closely related pairs of codes and making them
highly dependable, this approach requires vast computer resources, and 
although widespread, it became totally irrelevant at present when
the client codes need to calculate all input functions at arbitrary 
point inside the plasma. 

Another extreme approach uses only primitive information from
the equilibrium codes, like coordinates of grid points and a number of
radial profiles, while all necessary functions are reconstructed
internally inside the client codes. The problem here is in calculation of
space derivatives, entering into many physics variables. With no
universal mathematical rules of doing numerical differentiation, this
approach unavoidably introduces its own inaccuracies and causes
potential convergence problems in interaction with equilibrium codes.

The here proposed Equilibrium Spline Interface (ESI) relies on the 
selection of a set of basic functions which consists of five radial
profiles and three 2-dimensional functions (for axisymmetric
configurations like tokamaks) or five 3-dimensional functions for
stellarators). These functions and their first derivatives with respect
to flux coordinates (including mixed derivatives) are stored in the data file
by the equilibrium code. In addition, the interface provides the
computer independent C-source file (e.g., escZ.c for ESC equilibrium
code) with C- or FORTRAN callable routines which perform the bibubic
spline reconstruction of basic functions and their first derivatives at
any point inside the plasma. Then, the client code can calculate the
necessary vector or scalar field variables by algebraic operations as it
is described below.

Sect.~2 introduces notations and gives definitions of basic
functions and their relations with the physical variables and plasma
profiles. Sect.~3 gives the Lagrangian equation of the guiding center
motion. Sect.~4 describes the basic functions of the interface and
specifies the method of calculating fields and profiles. In particular, the
expression for the metric coefficients are given together with basic
vectors of curvilinear coordinates and the curvature of the magnetic
field lines. This section also desribes transition to straight field
line coordinates useful for stability codes. Sect.~5 derives the
Hamiltonian form of guiding center equations and determines an
extension of ESI by an additional 2-dimensional function necessary for
Hamiltonian equations. Finally, Sect.~6 describes a minimum set of ESI
routines and their calls. 

The units [m], [T], [MA], [MPa] are used in ESI for lengths, magnetic
fields, electric currents, and the plasma pressure.

\Section{Basic relationships in equilibrium configurations}
In arbirary curvilinear flux coordinates $a,\gq,\gf$, where the
``radial'' coordinate $a$ labels the magnetic surfaces, the
vector potential $\waA$ has the following covariant representation 
\bqbl
&&
\waA
=
-\bgF'\gh\na a
+\bgF\na\gq
+\bgY\na\gf
,\qquad
\gh
=
\gh(a,\gq,\gf)
,\quad
\bgF
=
\bgF(a)
,\quad
\bgY
=
\bgY(a)
.\eqbl
Here $2\gp\bgF$ and $2\gp\bgY$ are equal to the toroidal $\gF$ and
poloidal $\gY$ fluxes through the contours $a=const,\gf=const$ and
$a=const,\gq=const$ respectively, while $\gh$ is a periodic function of
both $\gq,\gf$
\bqbl
&&
\int\gh d\gq=\int\gh d\gf=0
.\eqbl
The coordinates $a,\gq,\gf$ are assumed to be used by the equilibrium
code, and, thus, considered as the reference coordinates. Typically, the
angle $\gf$ is the azimuth of cylindrical coordinates $r,\gf,z$,
although in this section we do not assume this specific meaning.

The contravariant representation of the magnetic field $\waB$ has the form 
\bqbl
&&
\waB
=
JB^\gq(\na\gf\times\na a)
+JB^\gf(\na a\times\na\gq)
,\\&&
B^\gq
\equiv
(\waB\cdot\na\gq)
=
-\R{\bgY'+\bgF'\gh'_\gf}{J}
,\quad
B^\gf
\equiv
(\waB\cdot\na\gf)
=
\R{\bgF'(1+\gh'_\gq)}{J}
.\eqbl
The Jacobian $J$ gives the differential volume
\bqbl
&&
J
=
J(a,\gq,\gf)
\equiv
\R{D(x,y,z)}{D(a,\gq,\gf)}
=
\R{1}{((\na a\times\na\gq)\cdot\na\gf)}
.\eqbl
Because of $\gh$, the two angle contravariant components of magnetic
field are not necessarily proportional to each other. With a special
choice of one of the angles it is possible to eliminate $\gh$, e.g., 
\bqbl
&&
\gz=\gf-q\gh
,\quad
q
\equiv
-\R{\bgF'}{\bgY'}
=
q(a)
,\qquad
\waA
=
\bgF\na\gq
+\bgY\na\gf
+\na(q\bgY\gh)
,\eqbl
from the essential part of the vector potential. This change of
variables makes the magnetic field line straight in the new coordinates
$a,\gq,\gz$ 
\bqbl
&&
\waB
=
\baJ B^\gq(\na\gz\times\na a)
+\baJ B^\gz(\na a\times\na\gq)
,\\&&
B^\gq
=
-\R{\bgY'}{\baJ}
,\quad
B^\gz
\equiv
(\waB\cdot\na\gz)
=
\R{\bgF'}{\baJ}
,\quad
\baJ
=
\R{J}{1-q\gh'_\gf}
,\quad
\R{B^\gz}{B^\gq}
=
q(a)
,\eqbl
where $\baJ$ is Jacobian in variables $a,\gq,\gz$. Another special choice
are coordinates $a,\gq,\gz$, called canonical, where the guiding center
equation of motion have the Hamiltonian form. The transition to
them will be presented in an explicit form in Sec.~5.

In curvilinear coordinates the Maxwell equation for the current
density $\wsj$ 
\bqbl
&&
\gm_0\wsj
=
(\na\times\waB)
,\\&&
\gm_0\wsj
=
J\bsj^a(\na\gq\times\na\gf)
+J\bsj^\gq(\na\gf\times\na a)
+J\bsj^\gf(\na a\times\na\gq)
,\\&&
J\bsj^a=\R{\pa B_\gf}{\pa\gq}-\R{\pa B_\gq}{\pa\gf}
,\quad
J\bsj^\gq=\R{\pa B_a}{\pa\gf}-\R{\pa B_\gf}{\pa a}
,\quad
J\bsj^\gf=\R{B\pa _\gq}{\pa a}-\R{\pa B_a}{\pa\gq}
\eqbl
uses the covariant components of the magnetic field 
\bqbl
&&
\waB
=
B_a\na a
+B_\gq\na\gq
+B_\gf\na\gf
.\eqbl
They can be expressed in terms of contravariant components and the metric
tensor as
\bqbl
&&
B_a
=
g_{a\gq}B^\gq
+g_{a\gf}B^\gf
,\quad
B_\gq
=
g_{\gq\gq}B^\gq
+g_{\gq\gf}B^\gf
,\quad
B_\gf
=
g_{\gq\gf}B^\gq
+g_{\gf\gf}B^\gf
.\eqbl
In its turn, the metric tensor is defined by the expression for the
element of length
\bqbl
&&
dl^2
=
g_{aa}da^2
+2g_{a\gq}dad\gq
+2g_{a\gf}dad\gf
+g_{\gq\gq}d\gq^2
+2g_{\gq\gf}d\gq d\gf
+g_{\gf\gf}d\gf^2
.\eqbl

In equilibrium configurations 
\bqbl
&&
\na\bsp
=
(\wsj\times\waB)
,\quad
\bsp(a)\equiv\gm_0p(a)
,\qquad
\gm_0
\equiv0.4\gp
,\eqbl
where $p$ is the plasma pressure, $\bsj^a=0$ and $\wsj$ has a simplified form
\bqbl
&&
\gm_0\wsj
=
J\bsj^\gq(\na\gf\times\na a)
+J\bsj^\gf(\na a\times\na\gq)
,\\&&
\bsj^\gq
\equiv
\gm_0(\wsj\cdot\na\gq)
=
-\R{\baF'+\gn'_\gf}{J}
,\quad
\bsj^\gf
\equiv
\gm_0(\wsj\cdot\na\gf)
=
\R{\baI'+\gn'_\gq}{J}
,\quad
\baF
=
\baF(a)
,\quad
\baI
=
\baI(a)
.\eqbl
Here, $5\baI$ and $5\baF$ are the toroidal and poloidal currents (in
[MA]) through the above mentioned contours.

The radial projection of the equilibrium equation gives the radial
pressure balance equation
\bqbl
&&
J\bsp'
=
-\bgF'(1+\gh'_\gq)(\baF'+\gn'_\gf)
+(\baI'+\gn'_\gq)(\bgY'+\bgF'\gh'_\gf)
,\eqbl
which can be considered as an equation for $\gn$ and one of the currents
$\baF$ or $\baI$.

The equation $\gm_0\wsj=(\na\times\waB)$ determines the covariant 
components of the equilibrium magnetic field in the form
\bqbl
&&
\waB
=
B_a\na a
+B_\gq\na\gq
+B_\gf\na\gf
,\qquad
B_a
=
\gs'_a-\gn
,\quad
B_\gq
=
\baI+\gs'_\gq
,\quad
B_\gf
=
\baF+\gs'_\gf
.\eqbl
Such a specific form of $B_\gq,B_\gz$, in terms of two 1-dimensional
profiles, and the gradient of $\gs$ is the result of pressure balance.

Explicit realtionships between co- and contra-variant components of
magnetic field 
\bqbl
&&
\gs'_a-\gn
=
B_a=-\R{g_{a\gq}}{J}\bgY'(1-q\gh'_\gf)+\R{g_{a\gf}}{J}\bgF'(1+\gh'_\gq)
,\\&&
\baI+\gs'_\gq
=
B_\gq=-\R{g_{\gq\gq}}{J}\bgY'(1-q\gh'_\gf)+\R{g_{\gq\gf}}{J}\bgF'(1+\gh'_\gq)
,\\&&
\baF+\gs'_\gf
=
B_\gf=-\R{g_{\gq\gf}}{J}\bgY'(1-q\gh'_\gf)+\R{g_{\gf\gf}}{J}\bgF'(1+\gh'_\gq)
\eqbl
can be considered as equations for the functions $\gs,\gn,\gh$ and two
of the 1-D profiles $\bgF,\bgY,\baI,\baF$. The equilibrium codes solve
the resulting second order partial differential equations obtained
either by elimination of $\gs,\gn$ 
\bqbl
&&
\bgY'(1-q\gh'_\gf)[(B_\gq)'_a-(B_a)'_\gq]
-\bgF'(1+\gh'_\gq)[(B_\gf)'_a-(B_a)'_\gf]
=
J\bsp'
,\eqbl
where $B_a,B_\gq,B_\gf$ are the above written differential operators, or
by using $(\na\cdot\waB)=0$ with the covariant representation of $\waB$
and the radial pressure balance equation
\bqbl
&&
J\na^2\gs
+(J\na a\cdot\waB^*)'_a
+\left(J\na\gq\cdot\waB^*\right)'_\gq
+\left(J\na\gf\cdot\waB^*\right)'_\gf
=
0
,\quad
\waB^*
\equiv
-\gn\na a+\baI\na\gq+\baF\na\gf
,\\&&
J\bsp'
=
-\bgF'(1+\gh'_\gq)\baF'
+(\bgY'+\bgF'\gh'_\gf)\baI'
-J(\waB\cdot\na\gn)
\lbq{3Deq}
.\eqbl
Traditionally, the first form is used for axisymmetric configurations
$\pa/\pa\gf=0$, where it reduces to the Grad-Shafranov equation
\bqbl
&&
-\left(\R{g_{\gq\gq}}{J}\bgY'\right)'_a
+\left(\R{g_{a\gq}}{J}\bgY'\right)'_\gq
=
J\R{\bsp'}{\bgY'}
+\R{J}{r^2}\R{\baF\baF'}{\bgY'}
,\eqbl
while Eqs.(\ref{3Deq}) for the stellarator equilibrium calculations.

\Section{Lagrangian Equations of the guiding center motion} 
The guiding center Lagrangian  obtained by Littlejohn has the following
normalized form
\bqbl
&&
L
=
\left(\waA+\gr_\parallel\waB\right)\cdot\wsv-H
,\quad
\wsv
\equiv
\R{d\wsr}{d\gt}
,\quad
\gt
\equiv
\gW_ct
,\quad
\gW_c
\equiv
\R{eB_0}{mc}
,\\&&
H
\equiv
\gr^2_\parallel\R{B^2}{2}
+\gm B
+\gj_E
,\quad
\gm
\equiv
\R{\gr^2_\perp}{2B}
,\quad
\gj_E
\equiv
\R{mc^2\gF_E}{e}
\equiv
\gr_ER_0B^2_0
,\quad
\gr_E
\equiv
\R{c\gF_E}{R_0B_0\gW_c}
.\eqbl
Here, $\wsv$ is considered as the normalized time derivative of the
particle position, $\gW_c$ is the ion cyclotron frequency in the
reference magnetic field $B_0$ whatever is used for normalization (e.g.,
for ESI Units, $B_0=1$ T and $\gW_c=95.79\cdot10^{+6}$ 1/sec for
protons), $\gr_\parallel,\gr_\perp$ are the ion Larmor radii calculated 
based on particle velocities parallel and perpendicular to the field. In
the Hamiltonian $H$ the last term $\gj_E$ is associated with the electric
field potential $\gF_E$ and can be expressed in terms of the Larmor radius
$\gr_E$ calculated based on drift velocity in the effective electric
field $\gF_E/R_0$ where $R_0$ is some characteristic unit length (e.g.,
1 m).

Written for generalized coordinates of a particle $a,\gq,\gf$, the
Lagrangian has the form
\bqbl
&&
L=
P_a\d{a}
+P_\gq\d{\gq}
+P_\gf\d{\gf}
-H
,\\&&
P_a
\equiv
\gr_\parallel B_a
,\quad
P_\gq
\equiv
\bgF+\int^a_0\bgF'\gh'_\gq da+\gr_\parallel B_\gq
,\quad
P_\gf
\equiv
\bgY+\int^a_0\bgF'\gh'_\gf da+\gr_\parallel B_\gf
.\eqbl
The straightforward variation of $L$ with respect to
$\gr_\parallel,a,\gq,\gf$ leads to the following system of coupled
equations of motion
\begin{eqnarray}
&&
\left|
\begin{array}{rrrr}
\db0
\da B_a
\da B_\gq
\da B_\gf
\dd
\dd-B_a
\da0
\da J(B^\gf+\gr_\parallel j^\gf)
\da -J(B^\gq+\gr_\parallel j^\gq)
\dd
\dd B_\gq
\da J(B^\gf+\gr_\parallel j^\gf)
\da0
\da0
\dd
\dd B_\gf
\da-J(B^\gq+\gr_\parallel j^\gq)
\da0
\da0
\end{array}
\right|
\cdot
\left|
\begin{array}{l}
\db\d{\gr}_\parallel
\dd
\dd\d{a}
\dd
\dd\d{\gq}
\dd
\dd\d{\gf}
\end{array}
\right|
=
\left|
\begin{array}{r}
\db\R{\pa H}{\pa\gr_\parallel}
\dd\R{\pa H}{\pa a}
\dd-\R{\pa H}{\pa\gq}
\dd-\R{\pa H}{\pa\gf}
\end{array}
\right|
.\end{eqnarray}
Using the matrix
\begin{eqnarray}
&&
\maA
\equiv
\left|
\begin{array}{rr}
\db a_{11}
\da a_{12}
\dd a_{21}
\da a_{22}
\end{array}
\right|
,\quad
|\maA|
\equiv
a_{11}a_{22}-a_{12}a_{21}
=
J[|B|^2+\gr_\parallel(\waB\cdot(\na\times\waB))]
,\end{eqnarray}
where
\bqbl
&&
a_{11}\equiv B_\gq
,\quad a_{12}\equiv J(B^\gf+\gr_\parallel j^\gf)
,\quad a_{21}\equiv B_\gf
,\quad a_{22}\equiv-J(B^\gq+\gr_\parallel j^\gq)
,\eqbl
the equation of drift motion can be written in the explicit form
\begin{eqnarray}
&&
\left|
\begin{array}{l}
\db\d{\gr}_\parallel
\dd\dd
\dd\d{a}
\dd\dd
\dd\d{\gq}
\dd\dd
\dd\d{\gf}
\end{array}
\right|
=
\left|
\begin{array}{rrrr}
\db0
\da0
\da \R{a_{22}}{|\maA|}
\da -\R{a_{12}}{|\maA|}
\dd
\dd0
\da0
\da -\R{a_{21}}{|\maA|}
\da \R{a_{11}}{|\maA|}
\dd
\dd \R{a_{22}}{|\maA|}
\da -\R{a_{21}}{|\maA|}
\da0
\da-\R{B_a}{|\maA|}
\dd
\dd -\R{a_{12}}{|\maA|}
\da \R{a_{11}}{|\maA|}
\da\R{B_a}{|\maA|}
\da0
\end{array}
\right|
\cdot
\left|
\begin{array}{r}
\db\R{\pa H}{\pa\gr_\parallel}
\dd
\dd\R{\pa H}{\pa a}
\dd
\dd-\R{\pa H}{\pa\gq}
\dd
\dd-\R{\pa H}{\pa\gf}
\end{array}
\right|
.\end{eqnarray}

\Section{Set of basic functions for Equilibrium Spline Interface}
The coupling matrix of the guiding center equations 
contain explicitely the covariant components of the magnetic field, i.e.,
functions $\gs(a,\gq,\gf),\gn(a,\gq,\gf)$ and profiles
$\baI(a),\baF(a)$. Contravariant components enter into the matrix only  
together with the factor $J$ and are represented by 
$\bgY'(a),\bgF'(a)$ and by angle derivatives of $\gh(a,\gq,\gf)$. The
vector in the right hand side of the equations contains $|B|$ and its
spatial derivatives.

Thus, a comprehensive interface between equilibrium and particle orbit
code would need to contain information on the spline representation of four
functions $\gs,\gn,\gh,|B|$ and of four profiles
$\baI,\baF,\bgY',\bgF'$. Nevertheless, this conclusion is not
correct. In addition, it is necessary to convert the curvilinear
coordinates into the laboratory coordinates. This additional information
concerning about geometry of the curvilinear coordinates can, in fact,
replace some of functions, entering the guiding center equations.

Taking this into account, the proposed ESI contains a specially
selected set of functions which allow to reconstruct the guiding center
coefficients without taking second derivatives. They can also be used
for interfacing with other codes, such as stability or transport simulation
codes. 

Without loosing practicality, we describe ESI for the case when $\gf$ is
the azimuth of the cylindrical coordinates. Essentially all numerical
equilibrium codes use this choice of angle and this interface is
encouraging this choice. In both 2-D and 3-D cases the interface
contains the following radial profiles 
\bqbl
&&
\baF(a)
,\bgF'(a)
,\bgY'(a)
,T(a)
,P(a)
,\qquad
T\equiv\R{\baF\baF'}{\bgY'}
,\quad
P\equiv\R{\bsp'}{\bgY'}
.\eqbl
In 2-D configurations they are supplemented by three periodic functions 
\bqbl
&&
r
=
r(a,\gq,\gf)
,\quad
z
=
z(a,\gq,\gf)
,\quad
|B|
=
B(a,\gq,\gf)
,\quad
\gh
=
\gh(a,\gq,\gf)
\eqbl
and their first (including mixed) derivatives
\bqbl
&&
\baF'
,\bgF''
,\bgY''
,T'
,P'
,\quad
r'_a,r'_\gq,r''_{a\gq}
,\quad
z'_a,z'_\gq,z''_{a\gq}
,\quad
|B|'_a,|B|'_\gq,|B|''_{a\gq}
.\eqbl
Stored on the rectangular grid $a,\gq,\gf$, this set allows 
reconstruction of the basic functions and their first derivatives at any
point in the domain by using bicubic spline representation. The function 
$q(a)$ is not included into ESI because it and its derivative can be
expressed in term of $\bgY',\bgF'$ and their derivatives.

For 3-D configuratons two additional functions $\gh,\gn$ should be
included into interface together with the additional derivatives of
above mentioned functions, giving 
\bqbl
&&
r'_\gf,r''_{a,\gf},r''_{\gq,\gf}
,\quad
z'_\gf,z''_{a,\gf},z''_{\gq,\gf}
,\quad
|B|'_\gf,|B|''_{a,\gf},|B|''_{\gq,\gf}
,\\&&
\gh'_a,\gh'_\gq,\gh''_{a\gq},\gh'_\gf,\gh''_{a,\gf},\gh''_{\gq,\gf}
,\quad
\gn'_a,\gn'_\gq,\gn''_{a\gq},\gn'_\gf,\gn''_{a,\gf},\gn''_{\gq,\gf}
.\eqbl

Note, that $B$ has been included into the set of basic functions despite
the fact that it can be expressed in terms of other basic functions and
their derivatives
\bqbl
&&
B^2
=
\R{g_{\gq\gq}}{g}\bgY'^2(1-q\gh'_\gf)^2
-2\R{g_{\gq\gf}}{g}\bgY'(1-q\gh'_\gf)\bgF'(1+\gh'_\gq)
+\R{g_{\gf\gf}}{g}\bgF'^2(1+\gh'_\gq)^2
.\eqbl
The reason is that the first derivatives of $|B|$ are required for the
particle orbit and other codes. 

The following subsections give a recipe of using the basic functions. 

\Subsection{Geometry and metric coefficients}
For both the 2-D case 
\bqbl
&&
D
\equiv
z'_ar'_\gq-z'_\gq r'_a
,\quad
J
=
rD
,\quad
L
\equiv
\R{D}{r}
,\\&&
K
=
\R{g_{\gq\gq}}{rD}
=
\R{r'^2_\gq+z'^2_\gq}{rD}
,\quad
N
=
\R{g_{a\gq}}{rD}
=
\R{r'_ar'_\gq+z'_az'_\gq}{rD}
,\quad
M
=
\R{g_{aa}}{rD}
=
\R{r'^2_a+z'^2_a}{rD}
\eqbl
and 3-D case
\bqbl
&&
g_{aa}
=
r'^2_a+z'^2_a
,\quad
g_{a\gq}
=
r'_ar'_\gq+z'_az'_\gq
,\quad
g_{\gq\gq}
=
r'^2_\gq+z'^2_\gq
,\\&&
g_{a\gf}
=
r'_ar'_\gf+z'_az'_\gf
,\quad
g_{\gq\gf}
=
r'_\gq r'_\gf+z'_\gq z'_\gf
,\quad
g_{\gf\gf}
=
r^2+r'^2_\gf+z'^2_\gf
,\quad
J
=
rD
\eqbl
the metric coefficients can be calculated using first derivatives of
the spline functions $r(a,\gq,\gf),z(a,\gq,\gf)$ . The functions
$K,L,M,N$ have the same form for 2- and 3-dimensional cases.

Basic vectors of curvilinear coordinates have the following
reprsentation in terms of laboratory unit vectors and first derivatives
of coordinates
\bqbl
&&
\na a=\R{r'_\gq(\wse_z-z'_\gf\na\gf)-z'_\gq(\wse_r-r'_\gf\na\gf)}{D}
,\quad
\na\gq=\R{z'_a(\wse_r-r'_\gf\na\gf)-r'_a(\wse_z-z'_\gf\na\gf)}{D}
,\quad
\na\gf=\R{\wse_\gf}{r}
,\\&&
\na a\times\na\gq
=
\R{r\wse_\gf+r'_\gf\wse_r+z'_\gf\wse_z}{rD}
,\quad
\na\gq\times\na\gf
=
\R{z'_a\wse_z+r'_a\wse_r}{rD}
,\quad
\na\gf\times\na a
=
\R{r'_\gq\wse_r+z'_\gq\wse_z}{rD}
.\eqbl

\Subsection{Magnetic field and the current density}
The necessary combinations $\bgF',\bgY'$ of contravariant components of
the magnetic field entering the guiding center equations are part of
the basic functions 
\bqbl
&&
JB^\gq
=
-\bgY'-\bgF'\gh'_\gf
,\quad
JB^\gf
=
\bgF'(1+\gh'_\gq)
.\eqbl
In the case of symmetry
\bqbl
&&
JB^\gq
=
-\bgY'
,\quad
B^\gf
=
\R{\baF}{r^2}
,\quad
JB^\gf
=
L\baF
\eqbl
and the function $\gh$ is not required for calculation of the
particle orbits. Using the metric tensor, the covariant components of the
magnetic field, which enter into the equation of motion, can be calculated
in a straightforward way in the 2-D case as 
\bqbl
&&
B_a
=
-N\bgY'
,\quad
B_\gq
=
-K\bgY'
,\quad
B_\gf
=
\baF
,\eqbl
and in a little bit more complicated way inthe 3-D case
\bqbl
&&
B_a
=
-N(\bgY'+\bgF'\gh'_\gf)
+\R{g_{a\gf}}{rD}\bgF'(1+\gh'_\gq)
,\\&&
B_\gq
=
-K(\bgY'+\bgF'\gh'_\gf)
+\R{g_{\gq\gf}}{rD}\bgF'(1+\gh'_\gq)
,\\&&
B_\gf
=
-\R{g_{\gq\gf}}{rD}(\bgY'+\bgF'\gh'_\gf)
+\R{g_{\gf\gf}}{rD}\bgF'(1+\gh'_\gq)
,\eqbl
which involves the function $\gh$.

So far, the calcualtion did not use the fact that the magnetic field
represents the equilibrium configuration. This fact should be
explicitely used in calculation of the contravariant components of the 
current density. In 2-D case they are given by 
\bqbl
&&
J\bsj^\gq
=
-\baF'
,\quad
\bsj^\gf
=
P+\R{T}{r^2}
,\quad
J\bsj^\gf
=
rDP+LT
,\eqbl
while in 3-D by 
\bqbl
&&
J\bsj^\gq
=
-(\baF'+\gn'_\gf)
,\quad
J\bsj^\gf
=
\R{rDP}{1-q\gh'_\gf}
+q(1+\gh'_\gq)\R{\baF'+\gn'_\gf}{\baF(1-q\gh'_\gf)}
,\eqbl
where the radial pressure balance equation has been used for calculating
$J\bsj^\gf$. In the 3-D case, the current density cannot be reconstructed
without having the explicit representation of $\gn$, which
should be included in ESA. Note, that the major inaccuracy in the
equilibrium codes is concentrated in the relation between the
magnetic field and the current density. In the approach described, this
inaccuracy enters only in the drift terms $\propto\gr_\parallel$ in the
equation of motion and, thus, is not essential.

ESI allows calculation of the components of the vector potential, rarely
needed  
\bqbl
&&
\waA
=
-\bgF'\gh\na a
+\bgF\na\gq
+\bgY\na\gf
\eqbl
by integrating the cubic representation of $\bgF'(a),\bgY'(a)$.

\Subsection{Straight field line coordinates}
While there are is special reason for using straight field coordinates
for the particle orbits, the stability codes use the Fourier
decomposition of the test functions in straight field line
coordinates. Without transforming the poloidal angle $\gq$, 
transformation to the straight field line coordinates $a,\gq,\gz$ can be
performed by the substitution
\bqbl
&&
\gz=\gf-q\gh
,\quad
\na\gz=\na\gf(1-q\gh'_\gf)-q\gh'_\gq\na\gq-(q\gh)'_a\na a
.\eqbl
This relationship allows to calculation of the metric tensor of the straight
field line coordinates if $\gh$ is included into the interface set of
basic functions
\bqbl
&&
d\gf=
\R{d\gz}{1-q\gh'_\gf}+\R{q\gh'_\gq d\gq}{1-q\gh'_\gf}
+\R{(q\gh)'_ada}{1-q\gh'_\gf}
,\\&&
g_{aa}\to g_{aa}
+2g_{a\gf}\R{(q\gh)'_a}{1-q\gh'_\gf}+g_{\gf\gf}\R{(q\gh)'^2_a}{(1-q\gh'_\gf)^2}
,\\&&
g_{a\gq}\to g_{a\gq}
+g_{a\gf}\R{q\gh'_\gq}{1-q\gh'_\gf}
+g_{\gq\gf}\R{(q\gh)'_a}{1-q\gh'_\gf}
+g_{\gf\gf}\R{q\gh'^2_\gq}{(1-q\gh'_\gf)^2}
,\\&&
g_{\gq\gq}\to g_{\gq\gq}
+2g_{\gq\gf}\R{q\gh'_\gq}{1-q\gh'_\gf}
+g_{\gf\gf}\R{q\gh'^2_\gq}{(1-q\gh'_\gf)^2}
,\\&&
g_{a\gz}=\R{g_{a\gf}}{1-q\gh'_\gf}
,\quad
g_{\gq\gz}=\R{g_{\gq\gf}}{1-q\gh'_\gf}
,\quad
g_{\gz\gz}=\R{g_{\gf\gf}}{(1-q\gh'_\gf)^2}
.\eqbl
Even for axisymmetric configurations the straight field line coordinates
contain $g_{a\gz},g_{\gq\gz}$ components of the metric tensor.

The vector potential, contravariant component of magnetic field and the
current density can be calculated as 
\bqbl
&&
\waA
=
\bgF\na\gq
+\bgY\na\gf
+\na(\bgY q\gh)
,\\&&
B^\gq
=
-\R{\bgY'(1-q\gh'_\gf)}{rD}
,\quad
B^\gz
=
\R{\bgF'(1-q\gh'_\gf)}{rD}
,\\&&
\bsj^\gq
=
-\R{\baF'+\gn'_\gf}{rD}
,\quad
\bsj^\gz
=
\R{(\baI'+\gn'_\gq)(1-q\gh'_\gf)}{rD}
+\R{(\baF'+\gn'_\gf)q\gh'_\gq}{rD}
,\eqbl
while the covariant components of the magnetic field are
\bqbl
&&
\waB
=
\left((q\gh)'_a B_\gz+\gs'_a-\gn\right)\na a
+\left(q\gh'_\gq B_\gz+\baI+\gs'_\gq\right)\na\gq
+B_\gz\na\gz
,\quad
B_\gz
=
\R{\baF+\gs'_\gf}{1-q\gh'_\gf}
.\eqbl
In particular, for axisymmetric configurations
\bqbl
&&
\waB
=
\left(\bgs'_a-\bgn\right)\na a
+\left(\baI+\bgs'_\gq\right)\na\gq
+\baF\na\gz
,\quad
\bgs
\equiv
\gs+q\gh\baF
,\quad
\bgn
\equiv
\gn+q\gh\baF'
.\eqbl

\Subsection{Curvature of the magnetic field lines}
The curvature of the magnetic field lines is determined as
\bqbl
&&
\wsk
\equiv
\left(\R{\waB}{|B|}\cdot\na\right)\R{\waB}{|B|}
=
-\left(\R{\waB}{|B|}\times\left(\na\times\R{\waB}{|B|}\right)\right)
=
\R{\bsp'}{|B|^2}\na a
-\R{((\waB\cdot\na)|B|)}{|B|^3}\waB
+\R{|B|'_a}{|B|}\na a
+\R{|B|'_\gq}{|B|}\na\gq
\eqbl
and can be calculated using basic set of functions in a straightforward way.

\Section{Hamiltonian form of the guiding center equations}
Although Lagrangian written for generalized coordinates $a,\gq,\gf$ of a
particle conserves the particle energy $E=H$ and, in a case of symmetry,
the corresponding canonical momentum, it does not have the Hamiltonian
form because of the presence of $P_a\d{a}$ term. In this section we
desribe the coordinate transformation which allows to exactly eliminate
this term in the case of symmetry, and to do this approximately with a 
rigorous small parameter in the case of 3-D configurations.

\Subsection{Hamiltonian equations for axisymmetric case}
In the case of symmetry, $\pa/\pa\gf=0$, the Lagrangian, written for the 
generalized coordinates $a,\gq,\gf$ of a particle
\bqbl
&&
L=
P_a\d{a}
+P_\gq\d{\gq}
+P_\gf\d{\gf}
-H
,\\&&
P_a
\equiv
\gr_\parallel(\gs'_a-\gn)
,\quad
P_\gq
\equiv
\bgF+\int^a_0\bgF'\gh'_\gq da+\gr_\parallel(\baI+\gs'_\gq)
,\quad
P_\gf
\equiv
\bgY+\int^a_0\bgF'\gh'_\gf da+\gr_\parallel\baF
\eqbl
can be converted into the Hamiltonian form with no $\d{a}$ term by a change
of the ignorable variable, i.e.
\bqbl
&&
\gf
=
\gz-\bgz(a,\gq)
,\quad
\d{\gf}
=
\d{\gz}-\bgz'_a\d{a}-\bgz'_\gq\d{\gq}
,\eqbl
where $\bgz$ is a periodic function of the poloidal angle $\gq$. With
the following choice of $\bgz$
\bqbl
&&
\bgz
\equiv
\int^a_0\R{\gs'_a-\gn}{\baF}da=\R{\gs}{\baF}+\tgz
,\quad
\tgz
\equiv
\int^a_0\R{\baF'\gs-\baF\gn}{\baF^2}da
\eqbl
the covariant representation of the equilibrium magnetic field  has no 
radial component
\bqbl
&&
\waB
=
(\gs'_a-\gn)\na a
+(\baI+\gs'_\gq)\na\gq
+\baF\na\gf
=
(\baI-\baF\tgz'_\gq)\na\gq+\baF\na\gz
.\eqbl
In its turn the vector potential acquires the form
\bqbl
&&
\waA
=
-\bgF'\gh\na a
+\bgF\na\gq
+\bgY\na\gf
=
-\bgh\na a
+\bgF\na\gq
+\bgY\na\gz
-\na(\bgY\bgz)
,\\&&
\bgh
\equiv
\gh+q\bgz
=
\gh+q\R{\gs}{\baF}+q\tgz
.\eqbl
As a result, the Lagrangian takes the Hamiltonian form in variables
$a,\gq,\gz$ 
\bqbl
&&
L=
-\R{d}{dt}(\bgF'\bgh)
+\baP_\gq\d{\gq}
+\baP_\gz\d{\gz}
-H
,\\&&
\baP_\gq
\equiv
\bgF+\int^a_0\bgF'\bgh_\gq da+\gr_\parallel(\baI-\baF\tgz'_\gq)
,\quad
\baP_\gz
=
P_\gf
=
\bgY+\gr_\parallel\baF
\eqbl
with no explicit time derivative $\d{a}$. The equation of motion for
$a,\gq,\gz$ can be now written as 
\begin{eqnarray}
&&
\left|
\begin{array}{rrrr}
\db0
\da0
\da \R{\pa\baP_\gq}{\pa\gr_\parallel}
\da \R{\pa\baP_\gz}{\pa\gr_\parallel}
\dd0
\da0
\da \R{\pa\baP_\gq}{\pa a}
\da \R{\pa\baP_\gz}{\pa a}
\dd \R{\pa\baP_\gq}{\pa\gr_\parallel}
\da \R{\pa\baP_\gq}{\pa a}
\da0
\da0
\dd \R{\pa\baP_\gz}{\pa\gr_\parallel}
\da \R{\pa\baP_\gz}{\pa a}
\da0
\da0
\end{array}
\right|
\cdot
\left|
\begin{array}{l}
\db\d{\gr}_\parallel
\dd
\dd\d{a}
\dd
\dd\d{\gq}
\dd
\dd\d{\gz}
\end{array}
\right|
=
\left|
\begin{array}{r}
\db\R{\pa H}{\pa\gr_\parallel}
\dd\R{\pa H}{\pa a}
\dd-\R{\pa H}{\pa\gq}
\dd-\R{\pa H}{\pa\gz}
\end{array}
\right|
,\end{eqnarray}
where
\bqbl
&&
\R{\pa\baP_\gq}{\pa\gr_\parallel}
=
a_{11}
\equiv
\baI-\baF\tgz'_\gq
,\quad
\R{\pa\baP_\gq}{\pa a}
=
a_{12}
\equiv
\bgF'(1+\bgh'_\gq)+\gr_\parallel[\baI'-(\baF\tgz'_\gq)'_a]
,\\&&
\R{\pa\baP_\gz}{\pa\gr_\parallel}
=
a_{21}
\equiv
\baF
,\quad
\R{\pa\baP_\gz}{\pa a}
=
a_{22}
\equiv
\bgY'(1-q\gh'_\gf)+\gr_\parallel\baF'
,\quad
\R{\pa H}{\pa\gz}
=
0
.\eqbl
By inverting the matrix in a similar way as for the Lagrangian equations
the equations of motion can be written in the explicit form
\begin{eqnarray}
&&
\left|
\begin{array}{l}
\db\d{\gr}_\parallel
\dd\dd
\dd\d{a}
\dd\dd
\dd\d{\gq}
\dd\dd
\dd\d{\gz}
\end{array}
\right|
=
\left|
\begin{array}{rrrr}
\db0
\da0
\da \R{a_{22}}{|\maA|}
\da-\R{a_{12}}{|\maA|}
\dd
\dd0
\da0
\da-\R{a_{21}}{|\maA|}
\da \R{a_{11}}{|\maA|}
\dd
\dd \R{a_{22}}{|\maA|}
\da-\R{a_{21}}{|\maA|}
\da0
\da0
\dd
\dd-\R{a_{12}}{|\maA|}
\da \R{a_{11}}{|\maA|}
\da0
\da0
\end{array}
\right|
\cdot
\left|
\begin{array}{r}
\db\R{\pa H}{\pa\gr_\parallel}
\dd
\dd\R{\pa H}{\pa a}
\dd
\dd-\R{\pa H}{\pa\gq}
\dd
\dd-\R{\pa H}{\pa\gz}
\end{array}
\right|
.\end{eqnarray}
In order to use the Hamiltonian equations, if it appears to be necessary,
in addition to basic interface functions, it is sufficient to add to ESI
the spline representation for the function $\baF\tgz'_\gq$.

The Hamiltonian form of the equations of motion guarantees that the
guiding center motion not only preserves the value of Hamiltonian $H$ and
the momentum $\baP_\gz=P_\gf$, but also preserves the phase volume of an
ensemble of particles in the parameter space $\gq,\gz,\baP_\gq,\baP_\gz$. 

\Subsection{Approximate Hamiltonian equations for 3-D configurations}
In the 3-D case there is no explicit transformation to the canonical
coordinates. Nevertheless, the approximate transformation can be easily 
performed using the fact that 
\bqbl
&&
\R{B_a}{B_\gf}
=
O\left(\R{a^2_0}{R^2}\right)
\ll1
,\eqbl
where $a_0,R$ are minor and major radii of the configuration. Using the
substitution
\bqbl
&&
\gf
=
\gz-\bgz(a,\gq,\gf)
,\quad
\bgz
\equiv
\int^a_0\R{\gs'_a-\gn}{\baF+\gs'_\gf}da
,\quad
\d{\gf}
=
\d{\gz}-\bgz'_a\d{a}-\bgz'_\gq\d{\gq}-\bgz'_\gf\d{\gf}
,\eqbl
the magnetic field and vector potential can be written in the form 
\bqbl
&&
\waB
=
[(\baI+\gs'_\gq)-(\baF+\gs'_\gf)\bgz'_\gq]\na\gq
+(\baF+\gs'_\gf)(\na\gz-\bgz'_\gf\na{\gf})
\simeq
[(\baI+\gs'_\gq)-(\baF+\gs'_\gf)\bgz'_\gq]\na\gq
+(\baF+\gs'_\gf)(1-\bgz'_\gf)\na\gz
,\\&&
\waA
=
-\bgF'\gh\na a
+\bgF\na\gq
+\bgY\na\gf
=
-\bgF'\bgh\na a
+\bgF\na\gq
+\bgY\na\gz
-\na(\bgY\bgz)
,\\&&
\bgF'\bgh
\equiv
\bgF'\gh+q\bgz
.\eqbl
As a result, the Lagrangian acquires the Hamiltonian form
\bqbl
&&
L=
-\R{d}{dt}(\bgF'\bgh)
+\baP_\gq\d{\gq}
+\baP_\gz\d{\gz}
-H
,\\&&
\baP_\gq
\equiv
\bgF+\int^a_0\bgF'\bgh_\gq da
+\gr_\parallel[\baI+\gs'_\gq-(\baF+\gs'_\gf)\bgz'_\gq]
,\quad
\baP_\gz
=
P_\gf
=
\bgY+\int^a_0\bgF'\bgh_\gf da+\gr_\parallel(\baF+\gs'_\gf)(1-\bgz'_\gf)
.\eqbl
Addition of function $\bgz$ into the minimal set of basic function of ESI
allows use of the Hamiltonian form of equation of guiding center motion.

While making variation of $L$ it is necessary to take into account that
in ESI all functions are given as function of $a,\gq,\gf$ rather than of
$a,\gq,\gz$. The difference is small and could be negligible. Otherwise,
it is necessary to use relations 
\bqbl
&&
\left.\R{\pa}{\pa a}\right|_{\gq,\gz}
=
\R{\pa}{\pa a}+\R{\pa\gf}{\pa a}\R{\pa}{\pa\gf}
,\quad
\left.\R{\pa}{\pa\gq}\right|_{a,\gz}
=
\R{\pa}{\pa\gq}+\R{\pa\gf}{\pa\gq}\R{\pa}{\pa\gf}
,\quad
\left.\R{\pa}{\pa\gz}\right|_{a,\gq}
=
\R{\pa\gf}{\pa\gz}\R{\pa}{\pa\gf}
,\\&&
\R{\pa\gf}{\pa a}=-\R{\bgz'_a}{1+\bgz'_\gf}
,\quad
\R{\pa\gf}{\pa\gq}=-\R{\bgz'_\gq}{1+\bgz'_\gf}
,\quad
\R{\pa\gf}{\pa\gz}=\R{1}{1+\bgz'_\gf}
\eqbl
in deriving the equation of motion from this Lagrangian. 

\Section{Minimal set of ESI routines}
At present ESI is functional for Equilibrium and Stability Code (ESC)
for axisymmetric tokamak equilibrium configurations. Here we briefly
describe the call of the minimal set of routines of the C-source file
\c{escZ.c} necessary for the guiding center particle codes. More
complete information is given in the \c{escZ.c.d} document file (which
contains as a part also this text).
\vskip1em
\noindent
\begin{tabular}{|p{1.3in}|p{1.5in}|p{3.2in}|}\hline
C call&FORTRAN call& Comment\\\hline 

\c{esiread\_(FNm);}&\c{call esiread(FNm)}& \c{FNm} - character string
of the name of the file. Reads the ESI spline data file generated by an
equiibrium code.\\\hline

\c{esilink2c\_(aF,aFa ,gFa,gFaa,gYa,gYaa ,T,Ta,P,Pa 
,r,ra,rq,z,za,zq ,B,Ba,Bq ,gh,gha,ghq,sw)}
&\c{\ call esilink(aF,aFa \&,gFa,gFaa,gYa,gYaa \&,T,Ta,P,Pa 
\&,r,ra,rq,z,za,zq \&,B,Ba,Bq \&,gh,gha,ghq,sw)}
& Called once, it sends to ESI the addresses of arrays, which will contain
values of the basic functions. Here,\{\c{aF,aFa}\}=$\baF,\baF'$, 
\{\c{gFa,gFaa}\}=$\bgF',\bgF''$,
\{\c{gYa,gYaa}\}=$\bgY',\bgY''$,
\{\c{T,Ta,P,Pa}\}=$T,T',P,P'$,
\{\c{r,ra,rq,z,za,zq}\}=$r,r'_a,r'_\gq,z,z'_a,z'_\gq$,
\{\c{B,Ba,Bq,gh,gha,ghq}\}=$|B|,|B|'_a,|B|'_\gq,\gh,\gh'_a,\gh'_\gq$,
and \c{sw} is the interger array which could be necessary for marking
the particles. \\ \hline

\c{esiget2dfunctions\_ (a,gq,\&n);}
&\c{\ call \&esiget2dfunctions \&(a,gq,n)}& Designed for call inside
time advancing loops. Calculates the basic functions for $n$ particles in
positions \c{a[i],q[i]}=$a_i,\gq_i$. If \c{sw[i]}$\neq0$, the routine returns
\c{gFa[i]}=$\bgF'_i/a_i$, \c{gYa[i]}=$\bgY'_i/a_i$,
\c{rq[i]}=$r'_{\gq,i}/a_i$, \c{zq[i]}=$z'_{\gq,i}/a_i$,
\c{Bq[i]}=$|B|'_{\gq,i}/a_i$\\\hline 

\c{esifree\_();}&\c{call esifree()}& frees the memory allocated for ESI
by the \c{esiread\_()} call\\\hline
\end{tabular}

\vskip1em
ESI also contains several service routines. Some of them are given in
the Table

\vskip1em
\noindent
\begin{tabular}{|p{1.3in}|p{1.5in}|p{3.2in}|}\hline
C call&FORTRAN call& Comment\\\hline 
\c{ esigetrzb\_(r,z,B, a,q,\&n);}
&\c{call esigetrzb(r,z,B,a,q,n)}
&Calculates $r,z,|B|$ for $n$ particles in positions $a_i,\gq_i$\\\hline

\c{esigetprofiles\_ (\&F,\&Fa,\&gFa,\&gFaa,
 \&gYa,\&gYaa,\&T,\&Ta, \&P,\&Pa,\&a);}
&\c{call esigetprofiles (F,Fa,gFa,gFaa, gYa,gYaa,T,Ta,P,Pa,a)}
&Calculates $\baF,\baF',\bgF',\bgF'',\bgY',\bgY'',T,T',P,P'$ on the
 magnetic surface $a$.
\\\hline
\end{tabular}
\vskip1em
Upon necessity the minimal set of ESI routines can be extended in
accordance with the above discussion of the guiding center equaitons. 

%=====================================
\end{document}

Written for generalized coordinates of a particle $a,\gq,\gf$, the
Lagrangian has the form
\bqbl
&&
L=
P_a\d{a}
+P_\gq\d{\gq}
+P_\gf\d{\gf}
-H
,\\&&
P_a
\equiv
\gr_\parallel B_a
,\quad
P_\gq
\equiv
\bgF+\int^a_0\bgF'\gh'_\gq da+\gr_\parallel B_\gq
,\quad
P_\gf
\equiv
\bgY+\int^a_0\bgF'\gh'_\gf da+\gr_\parallel B_\gf
.\eqbl
The straightforward variation of $L$ with respect to
$\gr_\parallel,a,\gq,\gf$ leads to the following system of coupled
equations of motion
\begin{eqnarray}
&&
\left|
\begin{array}{rrrr}
\db0
\da B_a
\da B_\gq
\da B_\gf
\dd
\dd-B_a
\da0
\da J(B^\gf+\gr_\parallel j^\gf)
\da -J(B^\gq+\gr_\parallel j^\gq)
\dd
\dd B_\gq
\da J(B^\gf+\gr_\parallel j^\gf)
\da0
\da-J\gr_\parallel j^a
\dd
\dd B_\gf
\da-J(B^\gq+\gr_\parallel j^\gq)
\da J\gr_\parallel j^a
\da0
\end{array}
\right|
\cdot
\left|
\begin{array}{l}
\db\d{\gr}_\parallel
\dd
\dd\d{a}
\dd
\dd\d{\gq}
\dd
\dd\d{\gf}
\end{array}
\right|
=
\left|
\begin{array}{r}
\db\R{\pa H}{\pa\gr_\parallel}
\dd\R{\pa H}{\pa a}
\dd-\R{\pa H}{\pa\gq}
\dd-\R{\pa H}{\pa\gf}
\end{array}
\right|
.\end{eqnarray}
Using the matrix
\begin{eqnarray}
&&
\maA
\equiv
\left|
\begin{array}{rr}
\db a_{11}
\da a_{12}
\dd a_{21}
\da a_{22}
\end{array}
\right|
,\quad
|\maA|
\equiv
a_{11}a_{22}-a_{12}a_{21}
=
J[|B|^2+\gr_\parallel(\waB\cdot(\na\times\waB))]
,\end{eqnarray}
where
\bqbl
&&
a_{11}\equiv B_\gq
,\quad a_{12}\equiv J(B^\gf+\gr_\parallel j^\gf)
,\quad a_{21}\equiv B_\gf
,\quad a_{22}\equiv-J(B^\gq+\gr_\parallel j^\gq)
,\eqbl
the equation of drift motion can be written in the explicit form
\begin{eqnarray}
&&
\left|
\begin{array}{l}
\db\d{\gr}_\parallel
\dd\dd
\dd\d{a}
\dd\dd
\dd\d{\gq}
\dd\dd
\dd\d{\gf}
\end{array}
\right|
=
\left|
\begin{array}{rrrr}
\db0
\da0
\da \R{a_{22}}{|\maA|}
\da -\R{a_{12}}{|\maA|}
\dd
\dd0
\da0
\da -\R{a_{21}}{|\maA|}
\da \R{a_{11}}{|\maA|}
\dd
\dd \R{a_{22}}{|\maA|}
\da -\R{a_{21}}{|\maA|}
\da0
\da-\R{B_a}{|\maA|}
\dd
\dd -\R{a_{12}}{|\maA|}
\da \R{a_{11}}{|\maA|}
\da\R{B_a}{|\maA|}
\da0
\end{array}
\right|
\cdot
\left|
\begin{array}{r}
\db\R{\pa H}{\pa\gr_\parallel}
\dd
\dd\R{\pa H}{\pa a}
\dd
\dd-\R{\pa H}{\pa\gq}
\dd
\dd-\R{\pa H}{\pa\gf}
\end{array}
\right|
.\end{eqnarray}

\begin{eqnarray}
&&
\left|
\begin{array}{rrrr}
\db0
\da B_a
\da B_\gq
\da B_\gf
\dd
\dd-B_a
\da0
\da J(B^\gf+\gr_\parallel j^\gf)
\da -J(B^\gq+\gr_\parallel j^\gq)
\dd
\dd B_\gq
\da J(B^\gf+\gr_\parallel j^\gf)
\da0
\da-J\gr_\parallel j^a
\dd
\dd B_\gf
\da-J(B^\gq+\gr_\parallel j^\gq)
\da J\gr_\parallel j^a
\da0
\end{array}
\right|
.\end{eqnarray}

\begin{eqnarray}
&&
A=
-B_a
\left|
\begin{array}{rrr}
\dd-B_a
\da J(B^\gf+\gr_\parallel j^\gf)
\da -J(B^\gq+\gr_\parallel j^\gq)
\dd
\dd B_\gq
\da0
\da-J\gr_\parallel j^a
\dd
\dd B_\gf
\da J\gr_\parallel j^a
\da0
\end{array}
\right|
\\&&
+B_\gq\left|
\begin{array}{rrr}
\dd-B_a
\da0
\da -J(B^\gq+\gr_\parallel j^\gq)
\dd
\dd B_\gq
\da J(B^\gf+\gr_\parallel j^\gf)
\da-J\gr_\parallel j^a
\dd
\dd B_\gf
\da-J(B^\gq+\gr_\parallel j^\gq)
\da0
\end{array}
\right|
\\&&
-B_\gf\left|
\begin{array}{rrr}
\dd-B_a
\da0
\da J(B^\gf+\gr_\parallel j^\gf)
\dd
\dd B_\gq
\da J(B^\gf+\gr_\parallel j^\gf)
\da0
\dd
\dd B_\gf
\da-J(B^\gq+\gr_\parallel j^\gq)
\da J\gr_\parallel j^a
\end{array}
\right|
.\end{eqnarray}

D=|A|


A^{-1}

0		Jj^a	JB^q		JB^f
-Jj^a		0	B_f		-JB^f
JB^q		B_f	0		-B_a
JB^f		-B_q	B_a		0

(
 0	JB^f	-JB^q
 JB^f	0	-Jj^a
-JB^q	Jj^a	 0
)=0
-(
-B_a	JB^f	-JB^q
 B_q	0	-Jj^a
 B_f	Jj^a	 0
)=Jj^a

(
-B_a	 0	-B^q
 B_q	 B^f	-j^a
 B_f	-B^q	 0
)=JB^q
-(
-B_a	 0	B^f
 B_q	 B^f	0
 B_f	-B^q	j^a
)=-J^2
-----------------------------------------
-(
 B_a	B_q	 B_f
 JB^f	0	-Jj^a
-JB^q	Jj^a	 0
)
(
 0	B_q	 B_f
 B_q	0	-Jj^a
 B_f	Jj^a	 0
)
-(
 0	 B_a	 B_f
 B_q	 JB^f	-Jj^a
 B_f	-JB^q	 0
)
(
 0	 B_a	B_q
 B_q	 JB^f	0
 B_f	-JB^q	Jj^a
)
-----------------------------------------
-----------------------------------------
(
 B_a	B_q	 B_f
 0	JB^f	-JB^q
-JB^q	Jj^a	 0
)
-(
 0	B_q	 B_f
-B_a	JB^f	-JB^q
 B_f	Jj^a	 0
)
(
 0	 B_a	 B_f
-B_a	 0	-JB^q
 B_f	-JB^q	 0
)
-(
 0	 B_a	B_q
-B_a	 0	JB^f
 B_f	-JB^q	Jj^a
)
-----------------------------------------
-(
B_a	B_q	 B_f
0	JB^f	-JB^q
JB^f	0	-Jj^a
)
(
 0	 B_q	 B_f
-B_a	 JB^f	-JB^q
 B_q	 0	-Jj^a
)
-(
 0	 B_a	 B_f
-B_a	 0	-JB^q
 B_q	 JB^f	-Jj^a
)
(
 0	 B_a	B_q
-B_a	 0	JB^f
 B_q	 JB^f	0
)
-----------------------------------------
-----------------------------------------

--------------------------------
 0	 B_a	B_q	 B_f
-B_a	 0	JB^f	-JB^q
 B_q	 JB^f	0	-Jj^a
 B_f	-JB^q	Jj^a	 0
---------------------------------

#endif
#endif
#endif  /* }== Remove this section ======== */

#undef Doc_Cb_Structure
#ifdef Blb_escZ.c
Blb_escZ.c{
}
#endif

#undef Doc_Table_Of_Content
#ifndef Blb_escZ.c
				PS p=  1 (  1);
int InitLocalConstants();	PS p=  3 (  2);
int esifree_();			PS p=  4 (  2);
int SetFunctionErf();		PS p=  4 (  2);
int ESISetMem();		PS p=  5 (  3);
int ESIMemAlloc();		PS p=  6 (  3);
int ESICopy();			PS p=  7 (  4);
int ESIReadBD();		PS p=  8 (  4);
int ESIReadAD();		PS p=  9 (  5);
int ESIInit();			PS p= 18 (  9);
int esiread_();			PS p= 19 ( 10);
int ESIMemAllocRZ();		PS p= 20 ( 10);
int esifreerz_();		PS p= 20 ( 10);
int ESIReadRZ();		PS p= 20 ( 10);
int esireadrz_();		PS p= 22 ( 11);
void esilink2c_();		PS p= 22 ( 11);
void esilink2c4ne_();		PS p= 23 ( 12);
void settemperatures_();	PS p= 23 ( 12);
int esigetprofiles_();		PS p= 23 ( 12);
int esiget2dfunctions0();	PS p= 24 ( 12);
int esiget2dfunctions_();	PS p= 26 ( 13);
int esigetmagfluxes_();		PS p= 29 ( 15);
int esigetrzb_();		PS p= 29 ( 15);
int gcmotion_();		PS p= 31 ( 16);
int MFieldLines();		PS p= 34 ( 17);
void esigetpressure_();		PS p= 36 ( 18);
void esigetdensity_();		PS p= 37 ( 19);
void GetEECollisions();		PS p= 37 ( 19);
void GetIECollisions();		PS p= 38 ( 19);
void GetEPCollisions();		PS p= 39 ( 20);
void GetIICollisions();		PS p= 40 ( 20);
void getcollisions_();		PS p= 40 ( 20);
int setparticletype_();		PS p= 41 ( 21);
int rzmotion_();		PS p= 43 ( 22);
#endif
