#ifndef Blb_legcoef.c
#ifdef Doc_Blb_legcoef.c
\title{Time evolution equation and 3-D boundary conditions on a thin
resistive wall}
\author{C.~V.~Atanasiu, L.~E.~Zakharov}
\date{June 21, 1998}
\maketitle
\begin{abstract}
A surface parabolic (time dependent) PDE for the normal component of the
magnetic field $B_n$ is obtained for thin conductive screens outside the
plasma. Two additional surface integral equations for two magnetic
charges $\gs_i$, $\gs_e$ allow to express screen current $K$ as well as
tangential (both external and internal) components of magnetic field
$B_\gt$ in terms of $B_n$ on the screen. By determining everywhere in
the space the reaction magnetic field of external screens on evolving
plasma configuration, these magnetic charges allow to shift boundary
conditions from external screens onto any closed surface, surrounding
the plasma. 
\end{abstract}

\section{Introduction}
Realistic simulations of 3-D plasma time evolution require appropriate
evaluation of external magnetic fields from the edge currents in the
surrounding screens. Because of very complicated geometry of conductive
construction elements outside the plasma in toroidal devices, three
approximations are typically adopted for their modelling  
\begin{enumerate}
\item Slow time evolution, when only a few time constants for basic
components of magnetic field (like dipole, quadrupole, etc.) are
retained in the circuit equations for those external fields.
\item Fast time evolution (e.g., for studing plasma disruptions, plasma
rotation, feedback stabilization), when current density on external
elements can be sumulated by the thin skin currents, while magnetic
field does not penetrate into the body of the construction. In this
case, the normal component of evolving magnetic field is assumed to
vanish on the screens.
\item Special case of the fast time evolution when an external element
 can be considered as a thin with respect to its position to the plasma.
\end{enumerate}
In the first case, instead of numerical calculation, the slow time
constants can be measured in a real device either by simulating the
plasma by a conducting ring current, or by accumulating the data base
for those constants from real plasma discharges. The assumption is that
the slow time scales are not sensitive to the highest moments of the
plasma configuration and, thus, represent, the invariants of the
machine. On  the other hand, simulation of the fast evolution, requires
numerical evaluation of the edge currents, consistent with the plasma
behaviour. 

In this paper, we consider an efficient way of simulation of the back
reaction of external construction elements on a fast plasma
evolution. The method is based on replacement of the grid based
calculations of the magnetic field outside the plasma (in a generically
limitted domain) by expressing magnetic field of elements surrounding
the plasma by their sources (Green function approach). This allows to
formulate the boundary conditions for the plasma MHD on any surface,
surrounding the plasma (irrespective to the position of external
screens). 

In Sect.~2, a basis magnetostatic problem is formulated and resolved,
which allows to express the source of the vacuum magnetic field in terms
of magnetic charges on the screens. In addition to the field itself,
these magnetic charges determine the egde current density distribution
$\Bsi$, which can be expressed in terms of the total current stream
function $K$ 
\bqbl
&&
\Bsi
=
\na K\times\Bsn
,\eqbl
where $\Bsn$ is the normal vector (chosen as external, in the case of
the closed screen).

In Sect.~3, a parabolic PDE on the surface of screens is formulated for
the time evolution of the normal component of the magnetic field.

In Sect.~4, the case of the boundary conditions on a toroidally
symmetric boundary of the simulation domain is considered in terms of
Fourier expansion over toroidal azimuth.

In Sect.~5, the case of the boundary conditions on a toroidally
symmetric boundary of the simulation domain is considered in terms of
Fourier expansion over toroidal azimuth.

\section{}


\section{Time evolution of $B_n$ on the surface of the screen}
\bqbl
&&
{1\over r}\left({r\over\gs d}K'_l\right)'_l
+{1\over r}\left({1\over\gs dr}K'_\gf\right)'_\gf
=
-{\pa B^{ext}_n\over\pa t}
-{\pa B^{scr}_n\over\pa t}
.\eqbl



#endif
#ifndef mzl_System_legcoef.c
  Systm printf();	/* NO prototype */
#endif
void main()
{
  Undef printf();
  /*
   */
#ifdef Doc_main

#endif
}
#endif

#undef Doc_Cb_Structure
#ifdef Blb_legcoef.c
Blb_legcoef.c{
}
#endif

#undef Doc_Table_Of_Content
#ifndef Blb_legcoef.c
		PS p= -1;
void main();	PS p=  1;
#endif
