#ifndef Blb_splines.c
#ifdef Doc_splines.c
\Section{Function representation as a global spline}
Exact splines in the intervals $x_i<x<x_{i+1}$, $i=0$ --- $(N-1)$
\bqbl
g(x)=A_ig_i+B_ig_{i+1}
+[(A_i^3-A_i)g_i''+(B_i^3-B_i)g_{i+1}'']{h_i^2\over6}
,\quad
h_i\equiv x_{i+1}-x_i
,\\
\int_{x_i}^x g(x)dx=
{h_i\over2}
\left\{
(1-A_i^2)g_i
+B_i^2g_{i+1}
-B_i^2{h_i^2\over12}
\left[(1+A_i)^2g_i''+(2-B_i^2)g_{i+1}''\right]
\right\}
,\\
\int_{x_i}^{x_{i+1}} g(x)dx=
{h_i\over2}
\left[
g_i
+g_{i+1}
-{h_i^2\over12}
(g_i''+g_{i+1}'')
\right]
,\quad
A_i\equiv{x_{i+1}-x\over h_i}
,\quad
B_i\equiv{x-x_i\over h_i}
,\\
g'(x)={g_{i+1}-g_i\over h_i}
+[(3B_i^2-1)g_{i+1}''-(3A_i^2-1)g_i'']{h_i\over6}
,\\
g'_i={g_{i+1}-g_i\over h_i}
-[g_{i+1}''+2g_i'']{h_i\over6}
,\quad
g'_i={g_i-g_{i-1}\over h_{i-1}}
+[2g_i''+g_{i-1}'']{h_{i-1}\over6}
,\\
g''(x)=A_ig_i''+B_ig_{i+1}''
.\eqbl
\Section{Smoothing splines for periodic functions}
Notation for coefficients of tridiagonal equations, $i=1$ --- $(N-1)$
\bqbl
&&
h=x_{i+1}-x_i
,\quad
H={h^2\over6}
,\\&&
g'(x)={g_{i+1}-g_i\over h}
+[(3B_i^2-1)g_{i+1}''-(3A_i^2-1)g_i'']{h\over6}
,\\&&
g'_0={g_1-g_0\over h}-(g''_1+2g''_0){h\over6}
,\quad
g'_N={g_N-g_{N-1}\over h}
+(2g''_N+g''_{N-1}){h\over6}
.\eqbl
Tridiagonal equations
\bqbl
&&
-g_{i-1}
+2g_i
-g_{i+1}
+Hg''_{i-1}
+4Hg''_i
+Hg''_{i+1}
=0
,\quad
1\leq i\leq N-1
,\\&&
g_N
=
g_0
,\quad
g''_N
=
g''_0
,\quad
-g_1+2g_0-g_{N-1}+Hg''_1+4Hg''_0+Hg''_{N-1}
=
0
,\quad
i=0
.\eqbl

Least squares functional for a given function $f(x)$
\bqbl
&&
W=
\sum^{i<N}_{i=0}(g_i-f_i)^2
+\ga_0\sum^{N-1}_{i=0}g_i^2
+{\ga_1\over h^2}\sum^{N-1}_{i=0}(g_{i+1}-g_i)^2
+\ga_2L^4\sum^{N-1}_{i=0}g''^2_i
+{\ga_3L^4\over h^2}\sum^{N-1}_{i=0}(g''_{i+1}-g''_i)^2
\\&&
+2\sum_{i=0}^{i<N}\gl_i
(-g_{i-1}
 +2g_i
 -g_{i+1}
 +Hg''_{i-1}
 +4Hg''_i
 +Hg''_{i+1}
 )
,\quad
\bga_1
\equiv
{\ga_1\over h^2}
,\quad
\bga_2
\equiv
\ga_2L^4
,\quad
\bga_3
\equiv
{\ga_3L^4\over h^2}
.\eqbl
Unknown variables are $g_i$, $g''_i$, $\gl_i$, $0\leq i < N$ in 
the intermediate points
\bqbl
&&
-\bga_1g_{i-1}-\gl_{i-1}
+(1+\ga_0+2\bga_1)g_i+2\gl_i
-\bga_1g_{i+1}-\gl_{i+1}
=
f_i
,\\&&
-\bga_3g''_{i-1}+H\gl_{i-1}
+(\bga_2+2\bga_3)g''_i+4H\gl_i
-\bga_3g''_{i+1}+H\gl_{i+1}
=0
,\\&&
-g_{i-1}
+Hg''_{i-1}
+2g_i
+4Hg''_i
-g_{i+1}
+Hg''_{i+1}
=0
,\\&&
g_{-1}
=
g_{N-1}
,\quad
g''_{-1}
=
g''_{N-1}
,\quad
\gl_{-1}
=
\gl_{N-1}
.\eqbl

\Section{Matrix form of equations}
\def\db{\displaystyle}
\def\dd{\\\displaystyle}
\def\da{&\displaystyle}
\def\bqy{\begin{eqnarray}}
\def\eqy{\end{eqnarray}}

Matrix equations $0\leq i <N$ 
\bqy
&&
-TY_{i-1}+WY_i-TY_{i+1}=R_i
,\quad
Y_i=\left|
\begin{array}{c}
\db g_i
\dd g''_i
\dd \gl_i
\end{array}
\right|
,\quad
R_i=\left|
\begin{array}{c}
\db f_i
\dd 0
\dd 0
\end{array}
\right|
,\\&&
T
=
\left|
\begin{array}{ccc}
\db \ga_1	\da 0		\da 1
\dd 0		\da \bga_3	\da -H
\dd 1		\da -H		\da 0
\end{array}
\right|
,\quad
W
=
\left|
\begin{array}{ccc}
\db 1+\ga_0+2\bga_1	\da 0			\da 2
\dd 0			\da \bga_2+2\bga_3	\da 4H
\dd 2			\da 4H			\da 0
\end{array}
\right|
\eqy
Backsubstitution and decomposition loops
\bqbl
&&
Y_i=V_iY_{i+1}+U_i+g_{-1}U^0_i+g''_{-1}U^1_i+\gl_{-1}U^2_i
,\quad
Y_0=W^{-1}TY_1+W^{-1}R_0+W^{-1}TY_{-1}
,\\&&
V_i=(W-TV_{i-1})^{-1}T
,\quad
U_i=(W-TV_{i-1})^{-1}R_i+V_iU_{i-1}
,\quad
U^k_i=V_iU^k_{i-1}
,\\&&
V_0=W^{-1}T
,\quad
U_0=W^{-1}R_i+V_0(g_{-1}U^0_i+g''_{-1}U^1_i+\gl_{-1}U^2_i)
,\quad
Y_{N-1}=V_{N-1}Y_0+U_{N-1}
.\eqbl

\Section{Subroutine {\tt f2splP()}}
{\tt f2splP(g,d2g,f,x,nx,ga0,ga\_1,ga\_2,ga\_3)}
determines spline approximation of $f(x)\simeq g(x)$ with 
regularization parameters $\ga_0,\ \ga_1,\ \ga_2,\ \ga_3$. 

\begin{tabular}{|lll|}\hline
{\tt f[i]}	&$f(x_i)$	&function to be approximated\\ \hline
{\tt x[i]}	&$x_i$&grid of arguments for $f$ and(!) for splines $g$,
 $g''$\\ \hline
{\tt g[i]}&$p(x_i)g(x_i)\simeq f(t_i)$&spline of the function $f$ \\ \hline
{\tt d2g[i]}	&$g''(x_i)=g''(x_i)$&spline coefficients\\ \hline
{\tt nx}	&$N,\ x_0\leq x\leq x_N$&number of spline intervals \\ \hline
{\tt ga0}	&$\ga_0$	&parameter of regularization\\ \hline
{\tt ga1}	&$\ga_1$	&parameter of regularization\\ \hline
{\tt ga2}	&$\ga_2$	&parameter of regularization\\ \hline
{\tt ga3}	&$\ga_3$	&parameter of regularization\\ \hline
\end{tabular}
#endif

#ifndef mzl_System_splines.c /* --------------- Use of Variables --------{ */
#ifdef Tbl_UnusedID/* ------------- List of Unused variables ------{ */
  static double ccr5;	/*         <SPLINES:splines.c */
  static double ccr7;	/*         <SPLINES:splines.c */
  int DeInitSplines();	/*         <SPLINES:splines.c */
  int spl();		/*         <SPLINES:splines.c */
  int spl1();		/*         <SPLINES:splines.c */
  int spl2();		/*         <SPLINES:splines.c */
  int spl_p();		/*         <SPLINES:splines.c */
  int splp1();		/*         <SPLINES:splines.c */
  int splr0();		/*         <SPLINES:splines.c */
  int SetIspl();	/*         <SPLINES:splines.c */
  int splR();		/*         <SPLINES:splines.c */
  int Splr1();		/*         <SPLINES:splines.c */
  int splr1();		/*         <SPLINES:splines.c */
  int *n;		/* .splr1()<SPLINES:splines.c */
  int splr2();		/*         <SPLINES:splines.c */
  int *n;		/* .splr2()<SPLINES:splines.c */
  int spl0();		/*         <SPLINES:splines.c */
  int spl_f0();		/*         <SPLINES:splines.c */
  int f2spl();		/*         <SPLINES:splines.c */
  int f2spl2();		/*         <SPLINES:splines.c */
  int spl_d();		/*         <SPLINES:splines.c */
  int spli();		/*         <SPLINES:splines.c */
  int splI();		/*         <SPLINES:splines.c */
  int splEr();		/*         <SPLINES:splines.c */
  int splIr();		/*         <SPLINES:splines.c */
  int Data2Spline();	/*         <SPLINES:splines.c */
  int CholDcmp3();	/*         <SPLINES:splines.c */
#endif /* }=== End of List of Unused variables ======*/
#ifndef H/* ---------- Used External Variables ------{ */
  Ignor double;
  Systm NULL;
  Systm free();
  Systm malloc();
  Systm fabs();
  Undef printf();	/* NO prototype */
  int inv2x2();		/* No prototype */
#ifdef Tbl_SPLINES
  Undef printf();	/* NO prototype */
  Undef printf();	/* NO prototype */
  int inv2x2();		/* No prototype */
#endif
#endif /* }======== End of Used External Variables ======= */

#endif /* }======== End of Use of Variables ======== */


#ifndef mzl_SPLINES
static double ccr2;	/*  */
static double ccr4;	/*  */
static double ccr5;	/*  */
static double ccr3;	/*  */
static double ccr6;	/*  */
static double ccr12;	/*  */
static double ccr7;	/*  */
static double ccr30;	/*  */
static int Nv;		/*  */
static int Ispl;	/*  */
static int Ispl1;	/*  */
static double rhSpl;	/*  */
static double hSpl;	/*  */
static double hhSpl;	/*  */
static double xSpl;	/*  */
static double xxSpl;	/*  */
static double XSpl;	/*  */
static double XXSpl;	/*  */
static double *v0;	/*  */
static double *v1;	/*  */
static double *v2;	/*  */
static double *v4;	/*  */
static double *vp;	/*  */
static double *d2f;	/*  */
static double *u;	/*  */
static double *gl;	/*  */
static double *v3;	/*  */

int DeInitSplines()
{
/*
   */
#ifdef Doc_DeInitSplines
\Section{Skeleton of \c{DeInitSplines()}}
  \begin{verbatim}
    DeInitSplines(){
    }
  \end{verbatim}
\Section{\c{DeInitSplines()}}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int spl(g,d2g,dg0,dg1,x,n)
  double *g;	/*  */
  double *d2g;	/*  */
  double *dg0;	/*  */
  double *dg1;	/*  */
  double *x;	/*  */
  int *n;	/*  */
{
  Undef printf();
/*
   */
#ifdef Doc_spl
\Section{Skeleton of \c{spl()}}
  \begin{verbatim}
    spl(){
    }
  \end{verbatim}
\Section{\c{spl()}}
\Subsection{Description}
\Subsubsection{Evaluation of spline representation}
Notation for coefficients of tridiagonal equations, $i=1$ --- $(N-1)$
\bqbl
&&
h_i=x_{i+1}-x_i
,\quad
r_i={g_{i+1}-g_i\over h_i}
,\quad
t_i={h_i\over6}
,\quad
w_i={h_i\over3}
.\eqbl
Equations at $i=0$
\bqbl
&&
w_0g_0''+t_0g_1''=r_0-g_0'
,\quad\mbox{or}\quad
(g_0''+g_2'')-2g_1''
=0
.\eqbl
Equations at $i=1$
\bqbl
&&
t_{i-1}g_{i-1}''
+(w_i+w_{i-1})g_i''
+t_ig_{i+1}''
=
r_i-r_{i-1}
,\quad\mbox{or}\\&&
t_{i-1}(g_{i-1}''+g_{i+1}'')
+(w_i+w_{i-1})g_i''
+(t_i-t_{i-1})g_{i+1}''
=
r_i-r_{i-1}
.\eqbl
Equations at $2\leq i\leq N-1$.
\bqbl
&&
t_{i-1}g_{i-1}''
+(w_i+w_{i-1})g_i''
+t_ig_{i+1}''
=r_i-r_{i-1}
.\eqbl
Equations at $i=N$
\bqbl
&&
t_{N-1}g_{N-1}''
+w_{N-1}g_N''
=
g_N'-r_{N-1}
,\quad
\mbox{or}
\quad
g_{N-2}''-2g_{N-1}''+g_N''
=0
.\eqbl
Backsubstitution and decomposition loops
\bqbl
&&
s_ig''_{i-1}+w_ig''_i+t_ig''_{i+1}=R_i
,\quad
g''_i=v_ig''_{i+1}+u_i
,\quad
v_0=-w_0^{-1}t_0
,\quad
u_0=w_0^{-1}R_0
,\\&&
v_i=-(s_iv_{i-1}+w_i)^{-1}t_i
,\quad
u_i=(s_iv_{i-1}+w_i)^{-1}(R_i-s_iu_{i-1})
,\\&&
\bar{s}_N\equiv Q_Nv_{N-2}+s_N
,\quad
\bar{R}_N\equiv R_N-Q_Nu_{N-2}
,\quad
g''_N=[\bar{s}_Nv_{N-1}+w_N]^{-1}[\bar{R}_N-\bar{s}_Nu_{N-1}]
.\eqbl
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int spl1(g,d1g,d2g,dg0,dg1,x,n)
  double *g;	/*  */
  double *d1g;	/*  */
  double *d2g;	/*  */
  double *dg0;	/*  */
  double *dg1;	/*  */
  double *x;	/*  */
  int n;	/*  */
{
  Undef printf();
  /*
   */
#ifdef Doc_spl1
\Section{Skeleton of \c{spl1()}}
  \begin{verbatim}
    spl1(){
    }
  \end{verbatim}
\Section{\c{spl1()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*g}&$$&\\
\c{double }&\c{*d1g}&$$&\\
\c{double }&\c{*d2g}&$$&\\
\c{double }&\c{*dg0}&$$&\\
\c{double }&\c{*dg1}&$$&\\
\c{double }&\c{*x}&$$&\\
\c{int }&\c{n}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{Undef }&\c{printf()}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int spl2(g,d2g,d2g0,d2g1,x,n)
  double *g;	/*  */
  double *d2g;	/*  */
  double *d2g0;	/*  */
  double *d2g1;	/*  */
  double *x;	/*  */
  int *n;	/*  */
{
/*
   */
#ifdef Doc_spl2
\Section{Skeleton of \c{spl2()}}
  \begin{verbatim}
    spl2(){
    }
  \end{verbatim}
\Section{\c{spl2()}}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int spl_p(g,d2g,x,n)
  double *g;	/*  */
  double *d2g;	/*  */
  double *x;	/*  */
  int n;	/*  */
{
/*
   */
#ifdef Doc_spl_p
\Section{Skeleton of \c{spl_p()}}
  \begin{verbatim}
    spl_p(){
    }
  \end{verbatim}
\Section{\c{spl_p()}}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int splp1(g,d1g,d2g,x,n)
  double *g;	/*  */
  double *d1g;	/*  */
  double *d2g;	/*  */
  double *x;	/*  */
  int n;	/*  */
{
  /*
   */
#ifdef Doc_splp1
\Section{Skeleton of \c{splp1()}}
  \begin{verbatim}
    splp1(){
    }
  \end{verbatim}
\Section{\c{splp1()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*g}&$$&\\
\c{double }&\c{*d1g}&$$&\\
\c{double }&\c{*d2g}&$$&\\
\c{double }&\c{*x}&$$&\\
\c{int }&\c{n}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int splr0(f,df,t0,g,d2g,x,n)
  double *f;	/*  */
  double *df;	/*  */
  double *t0;	/*  */
  double *g;	/*  */
  double *d2g;	/*  */
  double *x;	/*  */
  int *n;	/*  */
{
/*
   */
#ifdef Doc_splr0
\Section{Skeleton of \c{splr0()}}
  \begin{verbatim}
    splr0(){
    }
  \end{verbatim}
\Section{\c{splr0()}}
\Subsection{Description}
\Subsubsection{Function reconstruction from its global spline}
\Cline{splr0()} determines $f(t)$ and optionally $f'(t)$ at a random $t$.

\begin{tabular}{|lll|}\hline
{\tt f}	&$f(t0)$	&function to be reconstructed\\ \hline
{\tt df}	&$f'(t0)$	&its first derivative \\ \hline
{\tt t0[i]}	&$t$		&argument \\ \hline
{\tt g[i]}	&$g(t_i)=f(t_i)$&spline of the function $f$ \\ \hline
{\tt d2g[i]}	&$g''(t_i)=f''(t_i)$&spline coefficients of the function $f$ \\ \hline
{\tt *n}	&$N,\ t_0\leq t\leq t_N$&number of spline intervals \\ \hline
\end{tabular}

\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int SetIspl(x,s,n)
  double *x;	/*  */
  double s;	/*  */
  int n;	/*  */
{
/*
   */
#ifdef Doc_SetIspl
\Section{Skeleton of \c{SetIspl()}}
  \begin{verbatim}
    SetIspl(){
    }
  \end{verbatim}
\Section{\c{SetIspl()}}
\Subsection{Description}
\Subsubsection{First step of 2 steps function reconstruction}
\Cline{SetIspl()} prepares spline coefficients at point $s$, given mesh 
points $x_i,\ 0 \leq i \leq n$.

\begin{tabular}{|lll|}\hline
{\tt x[i]}	&$x_i$	&mesh points \\ \hline
{\tt s}	&$x$		&argument \\ \hline
{\tt n}	&$N,\ t_0\leq t\leq t_N$&number of spline intervals \\ \hline
\end{tabular}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int splR(f,df,g,d2g)
  double *f;	/*  */
  double *df;	/*  */
  double *g;	/*  */
  double *d2g;	/*  */
{
/*
   */
#ifdef Doc_splR
\Section{Skeleton of \c{splR()}}
  \begin{verbatim}
    splR(){
    }
  \end{verbatim}
\Section{\c{splR()}}
\Subsection{Description}
\Subsubsection{Function reconstruction from its global spline}
\Cline{splR()} determines $f(x)$ and optionally $f'(x)$ at in the point $s$, 
specified earlier in call of the routine \Cline{SetIspl()}.

\begin{tabular}{|lll|}\hline
{\tt f}	&$f(s)$	&function to be reconstructed\\ \hline
{\tt df}	&$f'(s)$	&its first derivative \\ \hline
{\tt g[i]}	&$g(x_i)=f(x_i)$&spline of the function $f$ \\ \hline
{\tt d2g[i]}	&$g''(x_i)=f''(x_i)$&spline coefficients of the function $f$ \\ \hline
\end{tabular}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int Splr1(f,df,t,g,d2g,x,i)
  double *f;	/*  */
  double *df;	/*  */
  double t;	/*  */
  double *g;	/*  */
  double *d2g;	/*  */
  double *x;	/*  */
  int i;	/*  */
{
  /*
   */
#ifdef Doc_Splr1
\Section{Skeleton of \c{Splr1()}}
  \begin{verbatim}
    Splr1(){
    }
  \end{verbatim}
\Section{\c{Splr1()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*f}&$$&\\
\c{double }&\c{*df}&$$&\\
\c{double }&\c{t}&$$&\\
\c{double }&\c{*g}&$$&\\
\c{double }&\c{*d2g}&$$&\\
\c{double }&\c{*x}&$$&\\
\c{int }&\c{i}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int splr1(f,df,t0,g,d2g,x,n,it)
  double *f;	/*  */
  double *df;	/*  */
  double *t0;	/*  */
  double *g;	/*  */
  double *d2g;	/*  */
  double *x;	/*  */
  int *n;	/*  */
  int *it;	/*  */
{
/*
   */
#ifdef Doc_splr1
\Section{Skeleton of \c{splr1()}}
  \begin{verbatim}
    splr1(){
    }
  \end{verbatim}
\Section{\c{splr1()}}
\Subsection{Description}
\Subsubsection{Function reconstruction from its global spline}
\Cline{splr1()} determines $f(t)$ and optionally $f'(t)$ in the point $t_0$
located in interval $x_{i}\leq t_0\leq x_{i+1}$

\begin{tabular}{|lll|}\hline
{\tt f}	&$f(t0)$	&function to be reconstructed\\ \hline
{\tt df}	&$f'(t0)$	&its first derivative \\ \hline
{\tt t0}	&$t_0$		&argument \\ \hline
{\tt g[i]}	&$g(x_i)=f(x_i)$&spline of the function $f$ \\ \hline
{\tt d2g[i]}	&$g''(x_i)=f''(x_i)$&spline coefficients of the function $f$\\
\hline
{\tt x}		&$x_i$		&mesh points\\ \hline
{\tt *n}	&$N,\ t_0\leq t\leq t_N$&totla number of spline intervals\\
\hline
{\tt *it}	&$i,\ x_i\leq t\leq x_{i+1}$&number of spline interval\\\hline
\end{tabular}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int splr2(F,dF,d2F,t0,g,d2g,x,n,it)
  double *F;	/*  */
  double *dF;	/*  */
  double *d2F;	/*  */
  double *t0;	/*  */
  double *g;	/*  */
  double *d2g;	/*  */
  double *x;	/*  */
  int *n;	/*  */
  int *it;	/*  */
{
/*
   */
#ifdef Doc_splr2
\Section{Skeleton of \c{splr2()}}
  \begin{verbatim}
    splr2(){
    }
  \end{verbatim}
\Section{\c{splr2()}}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int spl0(g,d2g,x,n)
  double g[];	/*  */
  double d2g[];	/*  */
  double x[];	/*  */
  int *n;	/*  */
{
/*
   */
#ifdef Doc_spl0
\Section{Skeleton of \c{spl0()}}
  \begin{verbatim}
    spl0(){
    }
  \end{verbatim}
\Section{\c{spl0()}}
\Subsection{Description}
\Subsubsection{Spline representation with $g''_0=0,\ g''_N=0$}
\Cline{spl0()} determines spline coefficients
for a function $g(x)$ with boundary conditions $g''(x_0)=0$ and $g''(x_N)=0$.

\begin{tabular}{|lll|}\hline
{\tt g[i]}	&$g(x_i)$&spline coefficients of the function $g(x)$ \\ \hline
{\tt d2g[i]}	&$g''(x_i)$&spline coefficients\\ \hline
{\tt x}		&$x_i$		&mesh points\\ \hline
{\tt *n}	&$N,\ x_0\leq t\leq x_N$&number of spline intervals \\ \hline
\end{tabular}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int spl_f0(g,d2g,p,f,x,n,dg0,dg1)
  double *g;	/*  */
  double *d2g;	/*  */
  double *p;	/*  */
  double *f;	/*  */
  double *x;	/*  */
  double *dg0;	/*  */
  double *dg1;	/*  */
  int *n;	/*  */
{
/*
   */
#ifdef Doc_spl_f0
\Section{Skeleton of \c{spl_f0()}}
  \begin{verbatim}
    spl_f0(){
    }
  \end{verbatim}
\Section{\c{spl_f0()}}
\Subsection{Description}
\Subsubsection{Spline representation with a leading factor}
Spline with a factor $p(x)$
\bqbl
&&
p(x)g(x)=f(x)
.\eqbl
Notation for coefficients of tridiagonal equations, $i=1$ --- $(N-1)$
\bqbl
&&
h_i=x_{i+1}-x_i
,\quad
r_i={g_{i+1}-g_i\over h_i}
,\quad
t_i={h_i\over6}
,\quad
w_i={h_i\over3}
.\eqbl
In the case of $p_0\neq0$, equations at $i=0,1$
\bqbl
&&
g_0=f_0/p_0
,\quad
w_0g_0''+t_0g_1''=r_0-g_0'
,\\&&
g_i=f_i/p_i
,\quad
t_{i-1}g_{i-1}''
+(w_i+w_{i-1})g_i''
+t_ig_{i+1}''
=r_i-r_{i-1}
.\eqbl
In the case of $p_0=0$, equations at $i=0,1$
\bqbl
&&
w_0g_0''
+t_0g_1''
=
r_0-g_0'
,\quad
(w_0+t_0-t_1)g_0''
+(w_1+w_0+t_0+w_1)g_1''
=
r_1-g_0'
,\\&&
g_1=f_1/p_1
,\quad
(t_0+w_0)g_0''
+(w_1+w_0+t_0)g_1''
+t_1g_2''
=
r_1-g_0'
.\eqbl
Equations at $2\leq i\leq N-1$.
\bqbl
&&
g_i=f_i/p_i
,\quad
t_{i-1}g_{i-1}''
+(w_i+w_{i-1})g_i''
+t_ig_{i+1}''
=r_i-r_{i-1}
.\eqbl
Equations at $i=N$
\bqbl
&&
g_i=f_i/p_i
,\quad
t_{N-1}g_{N-1}''
+w_{N-1}g_N''
=g_N'-r_{N-1}
,\quad
\mbox{or}
\quad
g_{N-2}''-2g_{N-1}''+g_N''
=0
.\eqbl
Backsubstitution and decomposition loops
\bqbl
&&
s_ig''_{i-1}+w_ig''_i+t_ig''_{i+1}=R_i
,\quad
g''_i=v_ig''_{i+1}+u_i
,\quad
v_0=-w_0^{-1}t_0
,\quad
u_0=w_0^{-1}R_0
,\\&&
v_i=-(s_iv_{i-1}+w_i)^{-1}t_i
,\quad
u_i=(s_iv_{i-1}+w_i)^{-1}(R_i-s_iu_{i-1})
,\\&&
\bar{s}_N\equiv Q_Nv_{N-2}+s_N
,\quad
\bar{R}_N\equiv R_N-Q_Nu_{N-2}
,\quad
g''_N=[\bar{s}_Nv_{N-1}+w_N]^{-1}[\bar{R}_N-\bar{s}_Nu_{N-1}]
.\eqbl
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int inv3x3(w,rw)
  double w[];	/*  */
  double rw[];	/*  */
{
/*
   */
#ifdef Doc_inv3x3
\Section{Skeleton of \c{inv3x3()}}
  \begin{verbatim}
    inv3x3(){
    }
  \end{verbatim}
\Section{\c{inv3x3()}}
\Subsection{Description}
\def\ds#1{$\displaystyle #1$}
\def\db{\displaystyle}
\def\dd{\\\displaystyle}
\def\da{&\displaystyle}
\Section{Inversion of symmetric the 3x3 matrix}
\begin{eqnarray}
W&=&\left|
\begin{array}{lll}
\db w_{11}\da w_{12}\da w_{13}
\dd w_{21}\da w_{22}\da w_{23}
\dd w_{31}\da w_{32}\da w_{33}
\end{array}
\right|
,\quad
D=w_{11}R_{11}+w_{12}R_{21}+w_{13}R_{31}
,\\
W^{-1}&=&\left|
\begin{array}{lll}
\db {R_{11}\over D}\da {R_{12}\over D}\da {R_{13}\over D}
\dd {R_{21}\over D}\da {R_{22}\over D}\da {R_{23}\over D}
\dd {R_{31}\over D}\da {R_{32}\over D}\da {R_{33}\over D}
\end{array}
\right|
=
\left|
\begin{array}{lll}
\db {w_{22}w_{33}-w_{23}w_{32}\over D}\da {w_{13}w_{32}-w_{12}w_{33}\over D}\da
{w_{12}w_{23}-w_{13}w_{22}\over D}

\dd {w_{23}w_{31}-w_{21}w_{33}\over D}\da {w_{11}w_{33}-w_{13}w_{31}\over D}\da
{w_{13}w_{21}-w_{11}w_{23}\over D}

\dd {w_{21}w_{32}-w_{22}w_{31}\over D}\da {w_{12}w_{31}-w_{11}w_{32}\over D}\da
{w_{22}w_{11}-w_{12}w_{21}\over D}
\end{array}
\right|
\end{eqnarray}

\begin{tabular}{|lll|}\hline
{\tt w}	&$W$	&Array representing a symmetric matrix $W$\\ \hline
{\tt rw}	&$W^{-1}$&Array representing the symmetric matrix $W^{-1}$
\\ \hline
\end{tabular}
\bqbl
&&
w_0=W_{11}
,\quad
w_1=W_{12}
,\quad
w_2=W_{13}
,\quad
w_3=W_{22}
,\quad
w_4=W_{23}
,\quad
w_5=W_{33}
,\\&&
rw_0=W_{11}^{-1}
,\quad
rw_1=W_{12}^{-1}
,\quad
rw_2=W_{13}^{-1}
,\quad
rw_3=W_{22}^{-1}
,\quad
rw_4=W_{23}^{-1}
,\quad
rw_5=W_{33}^{-1}
.\eqbl
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int f2spl(g,d2g,dg0,dg1,f,p,x,nx,ga0,ga_1,ga_2,ga_3)
  double *g;	/*  */
  double *d2g;	/*  */
  double *dg0;	/*  */
  double *dg1;	/*  */
  double *f;	/*  */
  double *p;	/*  */
  double *x;	/*  */
  int *nx;	/*  */
  double *ga0;	/*  */
  double *ga_1;	/*  */
  double *ga_2;	/*  */
  double *ga_3;	/*  */
{
/*
   */
#ifdef Doc_f2spl
\Section{Skeleton of \c{f2spl()}}
  \begin{verbatim}
    f2spl(){
    }
  \end{verbatim}
\Section{\c{f2spl()}}
\c{f2spl(g,d2g,f,p,x,nx,ga0,ga\_1,ga\_2,ga\_3,df)}
determines spline approximation of $f(x)\simeq p(x)g(x)$ with a leading
factor $p(x)$ and regularization parameters $\ga_0,\ \ga_1,\ \ga_2,\ \ga_3$.
providing a given derivative at the boundary.

\begin{verbatim}
int f2spl(g,d2g,dg0,dg1,f,p,x,nx,ga0,ga_1,ga_2,ga_3)
  /*
  f2spl() gives smoothing spline solution to equation p(x)g(x)=f; Here,

  double *g     - g(x_i),   0 <= i <= *nx - output array of spline
                            coefficients;
  double *d2g   - g''(x_i), 0 <= i <= *nx - output array of spline
                            coefficients;
  double *dg0   - g'(x_0) - input value of the first derivative at i=0.
                            Can be NULL;
  double *dg1   - g'(x_{*nx}) - input value of the first derivative at i=*nx.
                            Can be NULL;
  double *f     - f(x_i),   0 <= i <= *nx - input value of f(x) in RHS
                            Can be g;
  double *p     - p(x_i),   0 <= i <= *nx - input value of p(x) in RHS
                            Can be NULL, that corrsponds p=1;
  double *x     - x_i,   0 <= i <= *nx - input array of grid points for
                            splines;
  double *ga0    - smoothing parameters in front of \int g^2dx.
                   Typical value is 0;
  double *ga_1   - smoothing parameters in front of \int g'^2dx.
                   Typical value is 1e-9;
  double *ga_2   - smoothing parameters in front of \int g''^2dx.
                   Typical value is 0 or 1e-9;
  double *ga_3   - smoothing parameters in front of \int g''^2dx;
                   Typical value is 0 or 1e-10;
  int *nx        - Number of grid intervals.
                   (Total number of data points is equal *nx+1;
  */
\end{verbatim}

\Subsection{Description}
Notation for coefficients of tridiagonal equations, $i=1$ --- $(N-1)$
\bqbl
&&
h_i=x_{i+1}-x_i
,\quad
q_i={h_{i-1}+h_i\over2}
,\quad
c^1_i={1\over h_i}
,\quad
c^0_i={1\over h_{i-1}}+{1\over h_i}
,\quad
d^1_i={h_i\over6}
,\quad
d^0_i={x_{i+1}-x_{i-1}\over3}
.\eqbl
Tridiagonal equations
\bqbl
&&
g'(x)={g_{i+1}-g_i\over h_i}
+[(3B_i^2-1)g_{i+1}''-(3A_i^2-1)g_i'']{h_i\over6}
,\\&&
g'_0={g_1-g_0\over h_0}-(g_1''+2g_0''){h_0\over6}
,\quad
g'_N={g_N-g_{N-1}\over h_{N-1}}
+(2g''_N+g_{N-1}''){h_{N-1}\over6}
,\\&&
-c_{i-1}^1g_{i-1}
+c_i^0g_i
-c_i^1g_{i+1}
+d_{i-1}^1g_{i-1}''
+d_i^0g_i''
+d_i^1g_{i+1}''
=0
,\quad
0\leq i\leq N
.\eqbl

\Subsubsection{Functional for minimization}
Least squares functional for given function $f(x)$
and the leading factor $p(x)$
\bqbl
&&
W=
\sum_{i=0}^N q_i(p_ig_i-f_i)^2
+\ga_0\sum_{i=0}^{N}q_ig_i^2
+\ga_1\sum_{i=0}^{N-1}{(g_{i+1}-g_i)^2\over h_i}
+\ga_2\sum_{i=0}^{N}q_ig_i''^2
+\ga_3\sum_{i=0}^{N-1}{(g_{i+1}''-g_i'')^2\over h_i}
\\&&
+2\gl_0(C_0^0g_0+D_0^0g_0''+C_0^1g_1+D_0^1g_1''-F_0)+E_0\gl_0^2
\\&&
+2\sum_{i=1}^{N-1}\gl_i
(-c_{i-1}^1g_{i-1}
+c_i^0g_i
-c_i^1g_{i+1}
+d_{i-1}^1g_{i-1}''
+d_i^0g_i''
+d_i^1g_{i+1}'')
\\&&
+2\gl_N(C_N^1g_{N-1}+D_N^1g_{N-1}''+C_N^0g_N+D_N^0g_N''-F_N)+E_N\gl_N^2
+2\gl_{N+1}(C^1_{N+1}g_N-F_{N+1})
.\eqbl
Unknown variables are
$g_i$, $g_i''$, $\gl_i''$, $0\leq i\leq N$.

Equation at $i=0$
\bqbl
&&
(q_i\ga_0+\ga_1c_i^1+q_ip_i^2)g_i+2C_0^0\gl_i
-\ga_1c_i^1g_{i+1}-c_i^1\gl_{i+1}=q_ip_if_i
,\\&&
(q_i\ga_2+\ga_3c_i^1)g_i''+D_0^0\gl_0
-\ga_3c_i^1g_{i+1}''+d_i^1\gl_{i+1}=0
,\\&&
C_0^0g_0+D_0^0g_0''+E_0\gl_0+C_0^1g_{i+1}+D_0^1g_{i+1}''=F_0
,\quad
c^1_0g_0+2d^1_0g_0''-c^1_0g_1+d^1_0g_1''=0
.\eqbl
Equation at $i=1$
\bqbl
&&
-\ga_1c_{i-1}^1g_{i-1}+C_0^1\gl_{i-1}
+(q_i\ga_0+\ga_1c_i^0+q_ip_i^2)g_i+c_i^0\gl_i
-\ga_1c_i^1g_{i+1}-c_i^1\gl_{i+1}=q_ip_if_i
,\\&&
-\ga_3c_{i-1}^1g_{i-1}''+D_0^1\gl_{i-1}
+(q_i\ga_2+\ga_3c_i^0)g_i''+d_i^0\gl_i
-\ga_3c_i^1g_{i+1}''+d_i^1\gl_{i+1}
=0
,\\&&
-c_{i-1}^1g_{i-1}
+d_{i-1}^1g_{i-1}''
+c_i^0g_i
+d_i^0g_i''
-c_i^1g_{i+1}
+d_i^1g_{i+1}''
=0
.\eqbl
Equations at $2\leq i\leq N-2$.
\bqbl
&&
-\ga_1c_{i-1}^1g_{i-1}-c_{i-1}^1\gl_{i-1}
+(q_i\ga_0+\ga_1c_i^0+q_ip_i^2)g_i+c_i^0g_i\gl_i
-\ga_1c_i^1g_{i+1}-c_i^1\gl_{i+1}=q_ip_if_i
,\\&&
-\ga_3c_{i-1}^1g_{i-1}''+d_{i-1}^1\gl_{i-1}
+(q_i\ga_2+\ga_3c_i^0)g_i''+d_i^0\gl_i
-\ga_3c_i^1g_{i+1}''+d_{i-1}^1\gl_{i+1}
=0
,\\&&
-c_{i-1}^1g_{i-1}
+d_{i-1}^1g_{i-1}''
+c_i^0g_i
+d_i^0g_i''
-c_i^1g_{i+1}
+d_i^1g_{i+1}''
=0
.\eqbl
Equations at $i=N-1$
\bqbl
&&
-\ga_1c_{i-1}^1g_{i-1}-c_{i-1}^1\gl_{i-1}
+(q_i\ga_0+\ga_1c_i^0+q_ip_i^2)g_i+c_i^0\gl_i
+C_N^1\gl_{i+1}=q_ip_if_i+\ga_1c_i^1g_{i+1}
,\\&&
-\ga_3c_{i-1}^1g_{i-1}''+d_{i-1}^1\gl_{i-1}
+(q_i\ga_2+\ga_3c_i^0)g_i''+d_i^0\gl_i
-\ga_3c_i^1g_{i+1}''+D_N^1\gl_{i+1}
=0
,\\&&
-c_{i-1}^1g_{i-1}
+d_{i-1}^1g_{i-1}''
+c_i^0g_i
+d_i^0g_i''
+d_i^1g_{i+1}''
=
c_i^1g_{i+1}
.\eqbl
Equations at $i=N$
\bqbl
&&
p_ig_i=f_i
,\\&&
-\ga_3c_i^1g_{i-1}''+d_{i-1}^1\gl_{i-1}
+(q_i\ga_2+\ga_3c_{i-1}^1)g_i''+D_N^0\gl_i
=0
,\\&&
C_N^1g_{i-1}+D_N^1g_{i-1}''+D_N^0g_i''+E_N\gl_i=F_N-C_N^0g_i
,\\&&
-c^1_{N-1}g_{N-1}
+d^1_{N-1}g_{N-1}''
+2d^1_{N-1}g_N''
=g'_N
-c^1_{N-1}g_N
.\eqbl

\Subsubsection{Matrix form of equations}
Boundary conditions at $i=0$:
\def\db{\displaystyle}
\def\dd{\\\displaystyle}
\def\da{&\displaystyle}
\def\bqy{\begin{eqnarray}}
\def\eqy{\end{eqnarray}}

\bqy
\left|
\begin{array}{ccc}
\db q_i\ga_0+\ga_1c_i^1+q_ip_i^2	\da 0	\da C_0^0
\dd 0		\da q_i\ga_2+\ga_3c_i^1		\da D_0^0
\dd C_0^0	\da D_0^0			\da E_0
\end{array}
\right|
\left|
\begin{array}{c}
\db g_i
\dd g_i''
\dd \gl_i
\end{array}
\right|
&+&
\left|
\begin{array}{ccc}
\db -\ga_1c_i^1	\da 0			\da -c_i^1
\dd 0		\da -\ga_3c_i^1		\da d_i^1
\dd C_0^1	\da D_0^1		\da 0
\end{array}
\right|
\left|
\begin{array}{c}
\db g_{i+1}
\dd g_{i+1}''
\dd \gl_{i+1}
\end{array}
\right|
\nonumber\\
&=&
\left|
\begin{array}{c}
\db q_ip_if_i
\dd 0
\dd F_0
\end{array}
\right|
.\eqy
Equation at $i=1$
\bqy
\left|
\begin{array}{ccc}
\db -\ga_1c_{i-1}^1	\da 0		\da C_0^1
\dd 0		\da -\ga_3c_{i-1}^1	\da D_0^1
\dd -c_{i-1}^1	\da d_{i-1}^1		\da 0
\end{array}
\right|
\left|
\begin{array}{c}
\db g_{i-1}
\dd g_{i-1}''
\dd \gl_{i-1}
\end{array}
\right|
&+&
\left|
\begin{array}{ccc}
\db q_i\ga_0+\ga_1c_i^0+q_ip_i^2	\da 0		\da c_i^0
\dd 0		\da q_i\ga_2+\ga_3c_i^0	\da d_i^0
\dd c_i^0	\da d_i^0			\da 0
\end{array}
\right|
\left|
\begin{array}{c}
\db g_i
\dd g_i''
\dd \gl_i
\end{array}
\right|
\nonumber\\
+\left|
\begin{array}{ccc}
\db -\ga_1c_i^1	\da 0			\da -c_i^1
\dd 0		\da -\ga_3c_i^1		\da d_i^1
\dd -c_i^1	\da d_i^1		\da 0
\end{array}
\right|
\left|
\begin{array}{c}
\db g_{i+1}
\dd g_{i+1}''
\dd \gl_{i+1}
\end{array}
\right|
&=&
\left|
\begin{array}{c}
\db q_ip_if_i
\dd 0
\dd 0
\end{array}
\right|
.\eqy
Matrix equations at $i=2$ --- $(N-2)$
\bqy
\left|
\begin{array}{ccc}
\db -\ga_1c_{i-1}^1	\da 0			\da -c_{i-1}^1
\dd 0		\da -\ga_3c_{i-1}^1	\da d_{i-1}^1
\dd -c_{i-1}^1	\da d_{i-1}^1		\da 0
\end{array}
\right|
\left|
\begin{array}{c}
\db g_{i-1}
\dd g_{i-1}''
\dd \gl_{i-1}
\end{array}
\right|
&+&
\left|
\begin{array}{ccc}
\db q_i\ga_0+\ga_1c_i^0+q_ip_i^2	\da 0		\da c_i^0
\dd 0		\da q_i\ga_2+\ga_3c_i^0	\da d_i^0
\dd c_i^0	\da d_i^0				\da 0
\end{array}
\right|
\left|
\begin{array}{c}
\db g_i
\dd g_i''
\dd \gl_i
\end{array}
\right|
\nonumber\\
+\left|
\begin{array}{ccc}
\db -\ga_1c_i^1	\da 0			\da -c_i^1
\dd 0		\da -\ga_3c_i^1	\da d_i^1
\dd -c_i^1	\da d_i^1		\da 0
\end{array}
\right|
\left|
\begin{array}{c}
\db g_{i+1}
\dd g_{i+1}''
\dd \gl_{i+1}
\end{array}
\right|
&=&
\left|
\begin{array}{c}
\db q_ip_if_i
\dd 0
\dd 0
\end{array}
\right|
.\eqy
Equation at $i=(N-1)$
\bqy
\left|
\begin{array}{ccc}
\db -\ga_1c_{i-1}^1	\da 0		\da -c_{i-1}^1
\dd 0		\da -\ga_3c_{i-1}^1	\da d_{i-1}^1
\dd -c_{i-1}^1	\da d_{i-1}^1		\da 0
\end{array}
\right|
\left|
\begin{array}{c}
\db g_{i-1}
\dd g_{i-1}''
\dd \gl_{i-1}
\end{array}
\right|
&+&
\left|
\begin{array}{ccc}
\db q_i\ga_0+\ga_1c_i^0+q_ip_i^2	\da 0	\da c_i^0
\dd 0			\da q_i\ga_2+\ga_3c_i^0	\da d_i^0
\dd c_i^0		\da d_i^0		\da 0
\end{array}
\right|
\left|
\begin{array}{c}
\db g_i
\dd g_i''
\dd \gl_i
\end{array}
\right|
\nonumber\\
+\left|
\begin{array}{ccc}
\db 0	\da 0		\da C_N^1
\dd 0	\da -\ga_3c_i^1	\da D_N^1
\dd 0	\da d_i^1	\da 0
\end{array}
\right|
\left|
\begin{array}{c}
\db g_{i+1}
\dd g_{i+1}''
\dd \gl_{i+1}
\end{array}
\right|
&=&
\left|
\begin{array}{c}
\db q_ip_if_i+\ga_1c_i^1g_N
\dd 0
\dd c_i^1g_N
\end{array}
\right|
.\eqy
Boundary conditions at $i=N$:
\bqy
\left|
\begin{array}{ccc}
\db 0	\da 0	\da 0
\dd 0	\da -\ga_3c_{i-1}^1	\da d_{i-1}^1
\dd C_N^1	\da D_N^1		\da 0
\end{array}
\right|
\left|
\begin{array}{c}
\db g_{i-1}
\dd g_{i-1}''
\dd \gl_{i-1}
\end{array}
\right|
+
\left|
\begin{array}{ccc}
\db p_i	\da 0	\da 0
\dd 0	\da q_i\ga_2+\ga_3c_{i-1}^1	\da D_N^0
\dd 0	\da D_N^0			\da E_N
\end{array}
\right|
\left|
\begin{array}{c}
\db g_i
\dd g_i''
\dd \gl_i
\end{array}
\right|
&=&
\left|
\begin{array}{c}
\db f_i
\dd 0
\dd F_N-C^0_Ng_N
\end{array}
\right|
.\eqy

\Subsubsection{Solution in the matrix form}
Matrix equations $i=0$ --- $N$ 
\bqy
&&
T_{i-1}^\dagger Y_{i-1}+W_iY_i+T_iY_{i+1}=R_i
,\quad T_{-1}=0
,\quad T_N=0
,\quad
Y_i=\left|
\begin{array}{c}
\db g_i
\dd g_i''
\dd \gl_i
\end{array}
\right|
,\quad
R_i=\left|
\begin{array}{c}
\db q_ip_if_i
\dd 0
\dd 0
\end{array}
\right|
\eqy
For $i=0$
\bqy
W_i=
\left|
\begin{array}{ccc}
\db q_i\ga_0+\ga_1c_i^1+q_ip_i^2	\da 0	\da C_0^0
\dd 0		\da q_i\ga_2+\ga_3c_i^1\da D_0^0
\dd C_0^0	\da D_0^0		\da E_0
\end{array}
\right|
,\quad
T_i=\left|
\begin{array}{ccc}
\db -\ga_1c_i^1		\da 0			\da -c_i^1
\dd 0		\da -\ga_3c_i^1	\da d_i^1
\dd C_0^1	\da D_0^1		\da 0
\end{array}
\right|
.\eqy
For $i=1$ --- $N-2$
\bqy
W_i=
\left|
\begin{array}{ccc}
\db q_i\ga_0+\ga_1c_i^0+q_ip_i^2\da 0	\da c_i^0
\dd 0		\da q_i\ga_2+\ga_3c_i^0	\da d_i^0
\dd c_i^0	\da d_i^0		\da 0
\end{array}
\right|
,\quad
T_i=\left|
\begin{array}{ccc}
\db -\ga_1c_i^1	\da 0		\da -c_i^1
\dd 0		\da -\ga_3c_i^1	\da d_i^1
\dd -c_i^1	\da d_i^1	\da 0
\end{array}
\right|
.\eqy
For $i=(N-1)$
\bqy
W_i=
\left|
\begin{array}{ccc}
\db q_i\ga_0+\ga_1c_i^0+q_ip_i^2\da 0	\da c_i^0
\dd 0		\da q_i\ga_2+\ga_3c_i^0	\da d_i^0
\dd c_i^0	\da d_i^0		\da 0
\end{array}
\right|
,\quad
T_i=\left|
\begin{array}{ccc}
\db 0	\da 0		\da C_N^1
\dd 0		\da -\ga_3c_i^1	\da D_N^1
\dd 0	\da d_i^1	\da 0
\end{array}
\right|
.\eqy
For $i=N$
\bqy
W_i=\left|
\begin{array}{ccc}
\db p_i	\da 0	\da 0
\dd 0	\da q_i\ga_2+\ga_3c_{i-1}^1	\da D_N^0
\dd 0	\da D_N^0			\da 0
\end{array}
\right|
.\eqy
Backsubstitution and decomposition loops
\bqbl
&&
Y_i=V_iY_{i+1}+U_i
,\quad
V_0=-W_0^{-1}T_0
,\quad
U_0=W_0^{-1}R_0
,\\&&
V_i=-(T_{i-1}^\dagger V_{i-1}+W_i)^{-1}T_i
,\quad
U_i=(T_{i-1}^\dagger V_{i-1}+W_i)^{-1}(R_i-T_{i-1}^\dagger U_{i-1})
,\\&&
Y_N=(T_{N-1}^\dagger V_{N-1}+W_N)^{-1}(R_N-T_{N-1}^\dagger U_{N-1})
.\eqbl

\Subsubsection{Subroutine {\tt f2spl}}
{\tt f2spl(g,d2g,f,p,x,nx,ga0,ga\_1,ga\_2,ga\_3)}
determines spline approximation of $f(x)\simeq p(x)g(x)$ with a leading
factor $p(x)$ and regularization parameters $\ga_0,\ \ga_1,\ \ga_2,\ \ga_3$.

\begin{tabular}{|lll|}\hline
{\tt f[i]}	&$f(x_i)$	&function to be approximated\\ \hline
{\tt p[i]}	&$p(x_i)$	&given leading factor\\ \hline
{\tt x[i]}	&$x_i$&grid of arguments for $f$ and(!) for splines $g$,
 $g''$\\ \hline
{\tt g[i]}&$p(x_i)g(x_i)\simeq f(t_i)$&spline of the function $f$ \\ \hline
{\tt d2g[i]}	&$g''(x_i)=g''(x_i)$&spline coefficients\\ \hline
{\tt *nx}	&$N,\ x_0\leq x\leq x_N$&number of spline intervals \\ \hline
{\tt ga0}	&$\ga_0$	&parameter of regularization\\ \hline
{\tt ga1}	&$\ga_1$	&parameter of regularization\\ \hline
{\tt ga2}	&$\ga_2$	&parameter of regularization\\ \hline
{\tt ga3}	&$\ga_3$	&parameter of regularization\\ \hline
{\tt *df}	&$g_N'$	&first derivative at the right boundary\\\hline
\end{tabular}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int f2spl2(g,d2g,d2g0,d2g1,f,p,x,nx,ga0,ga_1,ga_2,ga_3)
  double *g;	/*  */
  double *d2g;	/*  */
  double *d2g0;	/*  */
  double *d2g1;	/*  */
  double *f;	/*  */
  double *p;	/*  */
  double *x;	/*  */
  int *nx;	/*  */
  double *ga0;	/*  */
  double *ga_1;	/*  */
  double *ga_2;	/*  */
  double *ga_3;	/*  */
{
/*
   */
#ifdef Doc_f2spl2
\Section{Skeleton of \c{f2spl2()}}
  \begin{verbatim}
    f2spl2(){
    }
  \end{verbatim}
\Section{\c{f2spl2()}}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int spl_d(g,d2g,f,x,nx,dg1)
  double *g;	/*  */
  double *d2g;	/*  */
  double *f;	/*  */
  double *x;	/*  */
  double *dg1;	/*  */
  int *nx;	/*  */
{
  int inv2x2();	/* No prototype */
/*
   */
#ifdef Doc_spl_d
\Section{Skeleton of \c{spl_d()}}
  \begin{verbatim}
    spl_d(){
    }
  \end{verbatim}
\Section{\c{spl_d()}}
\Subsection{Description}
\Subsubsection{Spline differentiation}
Notation for coefficients of tridiagonal equations, $i=1$ --- $(N-1)$
\bqbl
&&
h_i=x_{i+1}-x_i
,\quad
r_i={g_{i+1}-g_i\over h_i}
,\quad
t_i={h_i\over6}
,\quad
w_i={h_i\over3}
.\eqbl
Spline derivative of
\bqbl
&&
g(x)=f(x)'
,\quad
\int_{x_0}^x g(x)dx=f(x)
,\\&&
\int_{x_i}^{x_{i+1}} g(x)dx=
{h_i\over2}
\left[g_i+g_{i+1}-{h_i^2\over12}(g_i''+g_{i+1}'')\right]
=
f_{i+1}-f_i
,\\&&
g_i+g_{i+1}-{h_i^2\over12}(g_i''+g_{i+1}'')
=
2{f_{i+1}-f_i\over h_i}
.\eqbl
Equations at $i=0$
\bqbl
&&
g_0
=
f_0'
,\quad\mbox{or}\quad
g_0+g_1-{h_i^2\over12}(g_0''+g_1'')
=
2{f_1-f_0\over h_0}
,\\&&
w_0g_0''+t_0g_1''=r_0-f_0''
,\quad\mbox{or}\quad
(g_0''+g_2'')-2g_1''
=0
.\eqbl
Equations at $i=1$
\bqbl
&&
t_{i-1}g_{i-1}''
+(w_i+w_{i-1})g_i''
+t_ig_{i+1}''
=
r_i-r_{i-1}
,\quad\mbox{or}\\&&
t_{i-1}(g_{i-1}''+g_{i+1}'')
+(w_i+w_{i-1})g_i''
+(t_i-t_{i-1})g_{i+1}''
=
r_i-r_{i-1}
.\eqbl
Equations at $2\leq i\leq N-1$.
\bqbl
&&
t_{i-1}g_{i-1}''
+(w_i+w_{i-1})g_i''
+t_ig_{i+1}''
=r_i-r_{i-1}
.\eqbl
Equations at $i=N$
\bqbl
&&
t_{N-1}g_{N-1}''
+w_{N-1}g_N''
=
g_N'-r_{N-1}
,\quad
\mbox{or}
\quad
g_{N-2}''-2g_{N-1}''+g_N''
=0
.\eqbl
Backsubstitution and decomposition loops
\bqbl
&&
s_ig''_{i-1}+w_ig''_i+t_ig''_{i+1}=R_i
,\quad
g''_i=v_ig''_{i+1}+u_i
,\quad
v_0=-w_0^{-1}t_0
,\quad
u_0=w_0^{-1}R_0
,\\&&
v_i=-(s_iv_{i-1}+w_i)^{-1}t_i
,\quad
u_i=(s_iv_{i-1}+w_i)^{-1}(R_i-s_iu_{i-1})
,\\&&
\bar{s}_N\equiv Q_Nv_{N-2}+s_N
,\quad
\bar{R}_N\equiv R_N-Q_Nu_{N-2}
,\quad
g''_N=[\bar{s}_Nv_{N-1}+w_N]^{-1}[\bar{R}_N-\bar{s}_Nu_{N-1}]
.\eqbl
Notation for coefficients of tridiagonal equations, $i=1$ --- $(N-1)$
\bqbl
&&
h_i=x_{i+1}-x_i
,\quad
r_i={g_{i+1}-g_i\over h_i}
,\quad
t_i={h_i\over6}
,\quad
w_i={h_i\over3}
.\eqbl
Notation for coefficients of tridiagonal equations, $i=1$ --- $(N-1)$
\bqbl
&&
h_i=x_{i+1}-x_i
,\quad
c_i^1={1\over h_i}
,\quad
c_i^0={1\over h_{i-1}}+{1\over h_i}
,\quad
t_i={h_i\over6}
,\quad
d_i^0={x_{i+1}-x_{i-1}\over3}
.\eqbl
Tridiagonal equations
\bqbl
&&
g'(x)={g_{i+1}-g_i\over h_i}
+[(3B_i^2-1)g_{i+1}''-(3A_i^2-1)g_i'']{h_i\over6}
,\\&&
g'_0={g_1-g_0\over h_0}-(g_1''+2g_0''){h_0\over6}
,\quad
g'_N={g_N-g_{N-1}\over h_{N-1}}
+(2g''_N+g_{N-1}''){h_{N-1}\over6}
,\\&&
-c_{i-1}^1g_{i-1}
+c_i^0g_i
-c_i^1g_{i+1}
+d_{i-1}^1g_{i-1}''
+d_i^0g_i''
+t_ig_{i+1}''
=0
,\quad
0<i<N
.\eqbl
Equation at $i=0$
\bqbl
&&
g_0=2f_0
,\\&&
c^1_0g_0+2d^1_0g_0''-c^1_0g_1+d^1_0g_1''=0
.\eqbl
Equations at $1\leq i\leq N-1$.
\bqbl
&&
{x_i+2x_{i-1}\over6}g_{i-1}
+{2x_i+x_{i-1}\over6}g_i
-h_{i-1}^2{7x_i+8x_{i-1}\over360}g_{i-1}''
-h_{i-1}^2{8x_i+7x_{i-1}\over360}g_i''
={x_i^2f_i-x_{i-1}^2f_{i-1}\over h_{i-1}}
,\\&&
-c_{i-1}^1g_{i-1}
+d_{i-1}^1g_{i-1}''
+c_i^0g_i
+d_i^0g_i''
-c_i^1g_{i+1}
+t_ig_{i+1}''
=0
.\eqbl
Equations at $i=N$
\bqbl
&&
{x_i+2x_{i-1}\over6}g_{i-1}
+{2x_i+x_{i-1}\over6}g_i
-h_{i-1}^2{7x_i+8x_{i-1}\over360}g_{i-1}''
-h_{i-1}^2{8x_i+7x_{i-1}\over360}g_i''
={x_i^2f_i-x_{i-1}^2f_{i-1}\over h_{i-1}}
,\\&&
g_{N-2}''-2g_{N-1}''+g_N''
=0
.\eqbl
Matrix equation at $i=0$:
\def\db{\displaystyle}
\def\dd{\\\displaystyle}
\def\da{&\displaystyle}
\def\bqy{\begin{eqnarray}}
\def\eqy{\end{eqnarray}}
\bqy
\left|
\begin{array}{cc}
\db 1	  	\da 0
\dd c_0^1 	\da 2d_0^1
\end{array}
\right|
\left|
\begin{array}{c}
\db g_i
\dd g_i''
\end{array}
\right|
&+&
\left|
\begin{array}{cc}
\db 0		\da 0
\dd -c_0^1	\da d_0^1
\end{array}
\right|
\left|
\begin{array}{c}
\db g_{i+1}
\dd g_{i+1}''
\end{array}
\right|
=
\left|
\begin{array}{c}
\db 2f_i
\dd 0
\end{array}
\right|
.\eqy
Matrix equations at $i=1$ --- $N-1$
\bqy
\left|
\begin{array}{cc}
\db s_i^{11}	\da s_i^{12}
\dd -c_{i-1}^1	\da d_{i-1}^1
\end{array}
\right|
\left|
\begin{array}{c}
\db g_{i-1}
\dd g_{i-1}''
\end{array}
\right|
&+&
\left|
\begin{array}{cc}
\db w_i^{11}	\da w_i^{12}
\dd c_i^0	\da d_i^0
\end{array}
\right|
\left|
\begin{array}{c}
\db g_i
\dd g_i''
\end{array}
\right|
+\left|
\begin{array}{cc}
\db 0		\da 0
\dd -c_i^1	\da t_i
\end{array}
\right|
\left|
\begin{array}{c}
\db g_{i+1}
\dd g_{i+1}''
\end{array}
\right|
=
\left|
\begin{array}{c}
\db r_i
\dd 0
\end{array}
\right|
.\eqy
Equation at $i=N$:
\bqy
\left|
\begin{array}{cc}
\db 0	\da 0
\dd 0	\da 1
\end{array}
\right|
\left|
\begin{array}{c}
\db g_{i-2}
\dd g_{i-2}''
\end{array}
\right|
&+&
\left|
\begin{array}{cc}
\db s_i^{11}	\da s_i^{12}
\dd 0		\da -2
\end{array}
\right|
\left|
\begin{array}{c}
\db g_{i-1}
\dd g_{i-1}''
\end{array}
\right|
+\left|
\begin{array}{cc}
\db w_i^{11}	\da w_i^{12}
\dd 0		\da 1
\end{array}
\right|
\left|
\begin{array}{c}
\db g_i
\dd g_i''
\end{array}
\right|
=
\left|
\begin{array}{c}
\db r_i
\dd 0
\end{array}
\right|
.\eqy

Matrix equations
\bqbl
&&
W_0Y_0+T_0Y_1=R_0
,\quad
S_iY_{i-1}+W_iY_i+T_iY_{i+1}=R_i
,\quad
Q_NY_{N-2}+S_NY_{N-1}+W_NY_N=R_N
\eqbl
For $i=0$
\bqy
W_0&=&
\left|
\begin{array}{cc}
\db 1	  	\da 0
\dd c_0^1 	\da 2d_0^1
\end{array}
\right|
,\quad
T_0=
\left|
\begin{array}{cc}
\db 0		\da 0
\dd -c_0^1	\da d_0^1
\end{array}
\right|
,\quad
R_0
=
\left|
\begin{array}{c}
\db 2f_0
\dd 0
\end{array}
\right|
.\eqy
For $i=1$ --- $N-1$
\bqy
S_i&=&
\left|
\begin{array}{cc}
\db s_i^{11}	\da s_i^{12}
\dd -c_{i-1}^1	\da d_{i-1}^1
\end{array}
\right|
,\quad
W_i=
\left|
\begin{array}{cc}
\db w_i^{11}	\da w_i^{12}
\dd c_i^0	\da d_i^0
\end{array}
\right|
,\quad
T_i=
\left|
\begin{array}{cc}
\db 0		\da 0
\dd -c_i^1	\da t_i
\end{array}
\right|
,\quad
R_i
=
\left|
\begin{array}{c}
\db r_i
\dd 0
\end{array}
\right|
.\eqy
For $i=N$
\bqy
Q_N&=&
\left|
\begin{array}{cc}
\db 0	\da 0
\dd 0	\da 1
\end{array}
\right|
,\quad
S_N=
\left|
\begin{array}{cc}
\db s_i^{11}	\da s_i^{12}
\dd 0		\da -2
\end{array}
\right|
,\quad
W_N=
\left|
\begin{array}{cc}
\db w_i^{11}	\da w_i^{12}
\dd 0		\da 1
\end{array}
\right|
,\quad
R_N=
\left|
\begin{array}{c}
\db r_i
\dd 0
\end{array}
\right|
.\eqy
Backsubstitution and decomposition loops
\bqbl
&&
Y_i=V_iY_{i+1}+U_i
,\quad
V_0=-W_0^{-1}T_0
,\quad
U_0=W_0^{-1}R_0
,\\&&
V_i=-(S_iV_{i-1}+W_i)^{-1}T_i
,\quad
U_i=(S_iV_{i-1}+W_i)^{-1}(R_i-S_iU_{i-1})
,\\&&
\bar{S}_N\equiv Q_NV_{N-2}+S_N
,\quad
\bar{R}_N\equiv R_N-Q_NU_{N-2}
,\quad
Y_N=[\bar{S}_NV_{N-1}+W_N]^{-1}[\bar{R}_N-\bar{S}_NU_{N-1}]
.\eqbl
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int inv2x2(w,rw)
  double w[];	/*  */
  double rw[];	/*  */
{
/*
   */
#ifdef Doc_inv2x2
\Section{Skeleton of \c{inv2x2()}}
  \begin{verbatim}
    inv2x2(){
    }
  \end{verbatim}
\Section{\c{inv2x2()}}
\Subsection{Description}
\Section{Inversion of symmetric the 2x2 matrix}
\def\db{\displaystyle}
\def\dd{\\\displaystyle}
\def\da{&\displaystyle}
\def\bqy{\begin{eqnarray}}
\def\eqy{\end{eqnarray}}
\bqy
W&=&\left|
\begin{array}{cc}
\db w_0\da w_1
\dd w_2\da w_3
\end{array}
\right|
,\quad
D=w_0w_3-w_1w_2
,\quad
W_{-1}=\left|
\begin{array}{cc}
\db {w_3\over D}\da -{w_1\over D}
\dd -{w_2\over D}\da {w_0\over D}
\end{array}
\right|
\eqy
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int spli(f,g,d2g,x,nx)
  double *f;	/*  */
  double *g;	/*  */
  double *d2g;	/*  */
  double *x;	/*  */
  int *nx;	/*  */
{
/*
   */
#ifdef Doc_spli
\Section{Skeleton of \c{spli()}}
  \begin{verbatim}
    spli(){
    }
  \end{verbatim}
\Section{\c{spli()}}
Routine calculates the array of the solutions of the following equation
\bqbl 
x^2_if(x_i)=\int_0^{x_i} xg(x)dx
\eqbl
on an nonuniform grid.
\Subsection{Description}
\Subsubsection{Spline integration with a leading factor}
\bqbl
[x^2f(x)]'=xg(x)
,\quad
x^2f(x)=\int_0^x xg(x)dx
,\eqbl
where $g(x)$ is the spline function.
Integral over one spline interval $x_i$ --- $x_{i+1}$
\bqbl
&&
g(x)=A_ig_i+B_ig_{i+1}
+[(A_i^3-A_i)g_i''+(B_i^3-B_i)g_{i+1}'']{h_i^2\over6}
,\quad
h_i\equiv x_{i+1}-x_i
,\quad
x=x_i+B_i*h_i
,\\&&
\int_{x_i}^{x_{i+1}} xg(x)dx
=
x_i\int_{x_i}^{x_{i+1}}g(x)dx
+h_i\int_{x_i}^{x_{i+1}}B_ig(x)dx
,\\&&
x_i\int_{x_i}^{x_{i+1}}g(x)dx
=
x_i{h_i\over2}
\left[
g_i
+g_{i+1}
-{h_i^2\over12}
(g_i''+g_{i+1}'')
\right]
,\\&&
h_i\int_{x_i}^{x_{i+1}}B_ig(x)dx
=
h_i^2\int_0^1t\left[(1-t)g_i+tg_{i+1}
  +[((1-t)^3-(1-t))g_i''+(t^3-t)g_{i+1}'']{h_i^2\over6}
  \right]dt
\\&&
=
h_i^2\int_0^1\left[(1-t)tg_i+t^2g_{i+1}
  +[(t(1-t)^3-(1-t)t)g_i''+(t^4-t^2)g_{i+1}'']{h_i^2\over6}
  \right]dt
\\&&
=
h_i^2\left[{1\over6}g_i+{1\over3}g_{i+1}
  +[-{7\over60}g_i''-{2\over15}g_{i+1}'']{h_i^2\over6}
  \right]
\\&&
\int_{x_i}^{x_{i+1}} xg(x)dx
=
x_i{h_i\over2}\left[g_i+g_{i+1}-{h_i^2\over12}(g_i''+g_{i+1}'')\right]
\\&&
+\left[{1\over6}g_i+{1\over3}g_{i+1}\right]h_i^2
-[{7\over60}g_i''+{2\over15}g_{i+1}'']{h_i^4\over6}
.\eqbl
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int splI(ef,g,d2g,x,n)
  double *ef;	/*  */
  double *g;	/*  */
  double *d2g;	/*  */
  double *x;	/*  */
  int n;	/*  */
{
  /*
   */
#ifdef Doc_splI
\Section{Skeleton of \c{splI()}}
  \begin{verbatim}
    splI(){
    }
  \end{verbatim}
\Section{\c{splI()}}
Routine calculates the array $f(x_i)$ of integrals from the splined
function $g(x)$ 
\bqbl 
f(x_i)=\int_0^{x_i} xg(x)dx
\eqbl
on an nonuniform grid. See more at \c{spli()}.
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*ef}&$$&\\
\c{double }&\c{*g}&$$&\\
\c{double }&\c{*d2g}&$$&\\
\c{double }&\c{*x}&$$&\\
\c{int }&\c{n}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int splEr(ef,def,d2ef,r0,Ef,g,d2g,r,i)
  double *ef;	/*  */
  double *def;	/*  */
  double *d2ef;	/*  */
  double r0;	/*  */
  double *Ef;	/*  */
  double *g;	/*  */
  double *d2g;	/*  */
  double *r;	/*  */
  int i;	/*  */
{
  /*
   */
#ifdef Doc_splEr
\Section{Skeleton of \c{splEr()}}
  \begin{verbatim}
    splEr(){
    }
  \end{verbatim}
\Section{\c{splEr()}}
Routine calculates $f(r),f'(r)/(r),$ from the splined function
$g(x)$ 
\bqbl 
f(r)=\int^r_0 rg(r)dr
,\quad
g(r)
,\quad
g'(r)
\eqbl
on an nonuniform grid. See more at \c{spli()}.
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*ef}&$$&\\
\c{double }&\c{*def}&$$&\\
\c{double }&\c{*d2ef}&$$&\\
\c{double }&\c{r0}&$$&\\
\c{double }&\c{*Ef}&$$&\\
\c{double }&\c{*g}&$$&\\
\c{double }&\c{*d2g}&$$&\\
\c{double }&\c{*r}&$$&\\
\c{int }&\c{i}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int splIr(ef,def,d2ef,r0,Ef,g,d2g,r,i)
  double *ef;	/*  */
  double *def;	/*  */
  double *d2ef;	/*  */
  double r0;	/*  */
  double *Ef;	/*  */
  double *g;	/*  */
  double *d2g;	/*  */
  double *r;	/*  */
  int i;	/*  */
{
  /*
   */
#ifdef Doc_splIr
\Section{Skeleton of \c{splIr()}}
  \begin{verbatim}
    splIr(){
    }
  \end{verbatim}
\Section{\c{splIr()}}
Routine calculates $f(r),f',f''$ from the splined function $g(x)$ 
\bqbl 
f=\int^r_0 rg(r)dr
,\quad
f'=rg
,\quad
f''=(rg)'
\eqbl
on an nonuniform grid. See more at \c{spli()}.
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*ef}&$$&\\
\c{double }&\c{*def}&$$&\\
\c{double }&\c{*d2ef}&$$&\\
\c{double }&\c{r0}&$$&\\
\c{double }&\c{*Ef}&$$&\\
\c{double }&\c{*g}&$$&\\
\c{double }&\c{*d2g}&$$&\\
\c{double }&\c{*r}&$$&\\
\c{int }&\c{i}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int Data2Spline(g0,g2,f,x,I1,ga0,ga1,ga2,ga3)
  double *g0;	/*  */
  double *g2;	/*  */
  double *f;	/*  */
  double *x;	/*  */
  int I1;	/*  */
  double ga0;	/*  */
  double ga1;	/*  */
  double ga2;	/*  */
  double ga3;	/*  */
{
/*
   */
#ifdef Doc_Data2Spline
\Section{Skeleton of \c{Data2Spline()}}
  \begin{verbatim}
    Data2Spline(){
      V[0],U[0];
      while(i<I2){
	V[1:I-3],U[1:I-2];
      }
      V[I-2],U[I-2];
    }
  \end{verbatim}
\Section{\c{Data2Spline()}}
\Subsection{Description}
Routine generates a smoothing spline representation $g_i,g''_i$ for a
data array $f_i$. 
\begin{verbatim}
int Data2Spline(g,d2g,f,x,I,ga0,ga_1,ga_2,ga_3)
  /*

  double *g     - g(x_i),   0 <= i < I - output array of spline
                            coefficients;
  double *d2g   - g''(x_i), 0 <= i < I - output array of spline
                            coefficients;
  double *f     - f(x_i),   0 <= i < I - input value of f(x), can be g;
  double *x     - x_i,   0 <= i < I - input array of grid points for
                            splines;
  int I         - Total number of grid points;
  double ga0    - smoothing parameters in front of \sum g^2_i.
                   Typical value is 0;
  double ga_1   - smoothing parameters in front of \int g'^2dx.
                   Typically value 1e-9;
  double ga_2   - smoothing parameters in front of \int g''^2dx.
                   Typically value 0 or 1e-9;
  double ga_3   - smoothing parameters in front of \int g'''^2dx;
                   Typically value 0 or 1e-10;
  */
\end{verbatim}

\Subsection{Description}
Notation for coefficients of tridiagonal equations, $0\leq i< I-1$
\bqbl
&&
A_{i+1}\equiv\R{x_{i+1}-x}{h_{i+1}}
,\quad
B_{i+1}\equiv\R{x-x_i}{h_{i+1}}
,\quad
h_{i+1}=x_{i+1}-x_i
,\\&&
H_0=\R{h_1}{2}
,\quad
H_i=\R{h_i+h_{i+1}}{2}
,\quad
H_{I-1}=\R{h_{I-1}}{2}
,\quad
r_{i+1}=\R{1}{h_{i+1}}
,\quad
p_{i+1}=\R{h_{i+1}}{6}
,\quad
q_{i+1}=\R{h_{i+1}}{3}
.\eqbl
Matching derivatives in internal points
\bqbl
&&
g'(x)=\R{g_{i+1}-g_i}{h_{i+1}}
+[(3B_{i+1}^2-1)g''_{i+1}-(3A_{i+1}^2-1)g''_i]\R{h_{i+1}}{6}
,\\&&
g'_i=\R{g_{i+1}-g_i}{h_{i+1}}-(g''_{i+1}+2g''_i)\R{h_{i+1}}{6}
,\quad
g'_i=\R{g_i-g_{i-1}}{h_i}+(2g''_i+g''_{i-1}]\R{h_i}{6}
,\quad
1\leq i< N-1
.\eqbl
Tridiagonal equations for free boundary splines
\bqbl
g''_0-2g''_1+g''_2=0
,\quad&&
i=0
,\\
-r_ig_{i-1}+(r_i+r_{i+1})g_i-r_{i+1}g_{i+1}
+p_ig''_{i-1}+(q_i+q_{i+1})g''_i+p_{i+1}g''_{i+1}
=
0
,\quad&&
1\leq i\leq N-2
,\\
g''_{N-3}-2g''_{N-2}+g''_{N-1}=0
,\quad&&
i=N-1
.\eqbl

\Subsubsection{Functional for minimization}
Least squares functional of unknown variables $g_i$, $g_i''$, $\gl_i''$,
$0\leq i<I$ for a given function $f(x)$ 
\bqbl
&&
W=
\sum_{i=0}^{i<I} H_i(g_i-f_i)^2
+\ga_0\sum^{i<I}_{i=0}H_ig_i^2
+\bga_1\sum^{i<I}_{i=1}r_i(g_i-g_{i-1})^2
+\bga_2\sum^{i<I}_{i=0}H_ig''^2_i
+\bga_3\sum^{i<I}_{i=1}r_i(g''_i-g''_{i-1})^2
\\&&
+\gl^2_0
+2\sum^{i<I-1}_{i=1}\gl_i
[-r_ig_{i-1}+(r_i+r_{i+1})g_i-r_{i+1}g_{i+1}
+p_ig''_{i-1}+(q_i+q_{i+1})g''_i+p_{i+1}g''_{i+1}]
+\gl^2_{I-1}
,\\&&
\bga_1=d^2\ga_1
,\quad
\bga_2=d^4\ga_2
,\quad
\bga_3=d^6\ga_2
,\quad
d\equiv x_{I-1}-x_0
.\eqbl
Equation at $i=0$
\bqbl
&&
(H_i+\ga_0H_i+\bga_1r_{i+1})g_0-\bga_1r_{i+1}g_{i+1}-r_{i+1}\gl_{i+1}=H_if_i
,\\&&
(\bga_2H_0+\bga_3r_{i+1})g''_0-\bga_3r_{i+1}g''_{i+1}+p_{i+1}\gl_{i+1}=0
,\\&&
\gl_0=0
,\quad
g''_i-2g''_{i+1}+g''_{i+2}=0
.\eqbl
Equations at $0<i<I-1$ (because $\gl_0=0$ and $\gl_{I-1}=0$ equations
for $i=1$ and $i=I-1$ can be taken in the same general form.
\bqbl
&&
-\bga_1r_ig_{i-1}
-r_i\gl_{i-1}
+[H_i+\ga_0H_i+\bga_1(r_i+r_{i+1})]g_i
-\bga_1r_{i+1}g_{i+1}
+(r_i+r_{i+1})\gl_i
-r_{i+1}\gl_{i+1}
\\&&\qquad
=
H_if_i
,\\&&
-\bga_3r_ig''_{i-1}
+p_i\gl_{i-1}
+[\bga_2H_i+\bga_3(r_i+r_{i+1})]g''_i
-\bga_3r_{i+1}g''_{i+1}
+(q_i+q_{i+1})\gl_i
+p_{i+1}\gl_{i+1}
=0
,\\&&
-r_ig_{i-1}+(r_i+r_{i+1})g_i-r_{i+1}g_{i+1}
+p_ig''_{i-1}+(q_i+q_{i+1})g''_i+p_{i+1}g''_{i+1}
=0
.\eqbl
Equations at $i=I-1$
\bqbl
&&
-\bga_1r_ig_{i-1}
-r_i\gl_{i-1}
+(H_i+\ga_0H_i+\bga_1r_i)g_i
=H_if_i
,\\&&
-\bga_3r_ig''_{i-1}
+p_i\gl_{i-1}
+(\bga_2H_i+\bga_3r_i)g''_i
=
0
,\\&&
\gl_i=0
,\quad
-r_ig_{i-1}+p_ig''_{i-1}+\gl_i=0
.\eqbl

\Subsubsection{Matrix form of equations}
\bqbl
&&
\maW_0\vaX_0-\maT_1\vaX_1=\vaF_1
,\quad
-\maT^\dagger_i\vaX_{i-1}+\maW_i\vaX_i-\maT_{i+1}\vaX_{i+1}=\vaF_i
,\quad
-\maT^\dagger_i\vaX_{i-1}+\maW_i\vaX_i=\vaF_i|_{i=I-1}
,\\&&
\vaU_0=\maW^{-1}_0\vaF_0
,\quad
\maV_0=\maW^{-1}_0\maT_1
,\quad
\vaU_i
=
(\maW_i-\maT^\dagger_i\maV_{i-1})^{-1}(\vaF_i+\maT^\dagger_i\vaU_{i-1})
,\quad
\maV_i
=(\maW_i-\maT^\dagger_i\maV_{i-1})^{-1}\maT_{i+1}
,\\&&
\vaX_{I-1}
=
\vaU_{I-1}
,\quad
\vaX_i=\maV_i\vaX_{i+1}+\vaU_i
,\eqbl
where
\bqbl
&&
\maT_1
=
\left|\arr
\bga_1r_i&0&r_i\\
0&\bga_3r_1&-p_1\\
0&0&0\\
\ear
\right|
,\quad
\maT_i=\left|\arr
\bga_1r_i&0&r_i\\
0&\bga_3r_i&-p_i\\
r_i&-p_i&0\\
\ear
\right|
,\quad
\maT_{I-1}=\left|\arr
\bga_1r_i&0&0\\
0&\bga_3r_i&0\\
r_i&-p_i&0\\
\ear
\right|
,\\&&
\maW_0
=\left|\arr
H_i+\ga_0H_i+\bga_1r_{i+1}&0&0\\
0&\bga_2H_i+\bga_3r_{i+1}&0\\
0&0&1\\
\ear
\right|
,\\&&
\maW_i=\left|\arr
H_i+\ga_0H_i+\bga_1(r_i+r_{i+1})&0&r_i+r_{i+1}\\
0&\bga_2H_i+\bga_3(r_i+r_{i+1})&q_i+q_{i+1}\\
r_i+r_{i+1}&q_i+q_{i+1}&0\\
\ear
\right|
,\\&&
\maW_{I-1}
=\left|\arr
H_i+\ga_0H_i+\bga_1r_i&0&0\\
0&\bga_2H_i+\bga_3r_i&0\\
0&0&1\\
\ear
\right|
.\eqbl

\Subsection{Algorithm}
\Subsubsection{Cholesky decomposition}
\bqbl
&&
\maW
=
\msw\cdot\msw^\dagger
,\quad
w_{ij}|_{i<j}=0
,\\&&
W_{ij}|_{j\leq i}
=
\sum^{k\leq j}_{k=0}w_{ik}w^*_{jk}
,\quad
w^2_{jj}
=
W_{jj}
-\sum^{k<j}_{k=0}w_{jk}w^*_{jk}
,\quad
w_{ij}w_{jj}|_{j<i}
=
W_{ij}|_{j<i}
-\sum^{k<j}_{k=0}w_{ik}w^*_{jk}
.\eqbl
\Subsubsection{Cholesky backsubstitution}
\bqbl
&&
\msw\cdot\msw^\dagger\vaX
=
\vaY
,\quad
\msw\vaZ
=
\vaY
,\quad
\msw^\dagger\vaX
=
\vaZ
,\\&&
w_{ii}Z_i
+\sum^{j<i}_{j=0}w_{ij}Z_j
=
Y_i
,\quad
w_{ii}Z_i
=
Y_i
-\sum^{j<i}_{j=0}w_{ij}Z_j
,\\&&
\sum^{j<n}_{j=i}w_{ji}X_j
=
Z_i
,\quad
w_{ii}X_i
=
Z_i
-\sum^{j<n}_{j>i}w_{ji}X_j
.\eqbl
\Subsubsection{$3\times3$ case}
\bqbl
&&
w^2_{00}
=
W_{00}
,\quad
w_{10}w_{00}
=
W_{10}
,\quad
w_{20}w_{00}
=
W_{20}
,\\&&
w^2_{11}
=
W_{11}-w_{10}w^*_{10}
,\quad
w_{21}w_{11}
=
W_{21}-w_{20}w^*_{10}
,\quad
w^2_{22}
=
W_{22}
-w_{20}w^*_{20}
-w_{21}w^*_{21}
,\\&&
Z_0
=
\R{Y_0}{w_{00}}
,\quad
Z_1
=
\R{Y_1-w_{10}Z_0}{w_{11}}
,\quad
Z_2
=
\R{Y_2-w_{20}Z_0-w_{21}Z_1}{w_{22}}
,\\&&
X_2
=
\R{Z_2}{w_{22}}
,\quad
X_1
=
\R{Z_1-w_{21}X_2}{w_{11}}
,\quad
X_0
=
\R{Z_0-w_{10}X_1-w_{20}X_2}{w_{00}}
.\eqbl
New notations
\bqbl
&&
w_{00}
=
\R{1}{W_{00}}
,\quad
w_{10}
=
W_{10}
,\quad
w_{20}
=
W_{20}
,\quad
w_{11}
=
\R{1}{W_{11}-w^2_{10}w_{00}}
,\\&&
w_{21}
=
W_{21}-w_{20}w_{10}w_{00}
,\quad
w^2_{22}
=
\R{1}{W_{22}-w^2_{20}w_{00}-w^2_{21}w_{11}}
,\\&&
Z_0
=
w_{00}Y_0
,\quad
Z_1
=
(Y_1-w_{10}Z_0)w_{11}
,\quad
X_2
=
(Y_2-w_{20}Z_0-w_{21}Z_1)w_{22}
,\\&&
X_1
=
Z_1-w_{21}X_2w_{11}
,\quad
X_0
=
Z_0-(w_{10}X_1+w_{20}X_2)w_{00}
.\eqbl

\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int CholDcmp3()
{
/*
   */
#ifdef Doc_CholDcmp3
\Section{Skeleton of \c{CholDcmp3()}}
  \begin{verbatim}
    CholDcmp3(){
    }
  \end{verbatim}
\Section{\c{CholDcmp3()}}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{Known Bugs}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}
#endif
#endif

#undef Doc_Table_Of_Content
#ifndef Blb_splines.c
			PS p= -1 (  0);
#ifndef mzl_SPLINES
			PS p=  1 (  1);
int DeInitSplines();	PS p=  1 (  1);
int spl();		PS p=  1 (  1);
int spl1();		PS p=  2 (  1);
int spl2();		PS p=  4 (  2);
int spl_p();		PS p=  5 (  3);
int splp1();		PS p=  6 (  3);
int splr0();		PS p=  7 (  4);
int SetIspl();		PS p=  7 (  4);
int splR();		PS p=  7 (  4);
int Splr1();		PS p=  7 (  4);
int splr1();		PS p=  8 (  4);
int splr2();		PS p=  8 (  4);
int spl0();		PS p=  8 (  4);
int spl_f0();		PS p=  9 (  5);
int inv3x3();		PS p= 10 (  5);
int f2spl();		PS p= 11 (  6);
int f2spl2();		PS p= 14 (  7);
int spl_d();		PS p= 17 (  9);
int inv2x2();		PS p= 19 ( 10);
int spli();		PS p= 19 ( 10);
int splI();		PS p= 20 ( 10);
int splEr();		PS p= 20 ( 10);
int splIr();		PS p= 20 ( 10);
int Data2Spline();	PS p= 21 ( 11);
int CholDcmp3();	PS p= 23 ( 12);
#endif
#endif
