#ifndef Blb_2esc.c
/*  */
#ifdef Doc_2esc.c
\Section{Overview}
{\em
In the radial input profiles the first value is assumed at the magnetic axis 
(independent on the first value of the radial grid).

The output is given on the actual radial grid.
}
\Section{Layout of the shared memory for ESC}
\ESC exchanges with other codes through the shared memory which has
the following layout (where \c{Li,Ld} are the lentgh of \c{int,double})
\vskip1ex
\btbl{|llrrp{2.2in}|}\hline
ID 	& Byte		& Length	&Section & Comment\\\hline
\c{ShMemI}&\c{0 Li}	& \c{32Li}	&&Integer Parameters\\
\c{Fl}&\c{ShMemI}	& \c{1 Li}	&
&Return flag\\
\c{Na}&\c{1 Li}	& \c{1 Li}	&\c{Machine} 
&number of radial intervals\\
\c{Np}&\c{2 Li}	& \c{1 Li}	&\c{Machine}
&number of poloidal intervals\\
\c{Fp}&\c{3 Li}	& \c{1 Li}	&\c{Machine}
&number of Fourier coefficients\\
\c{hH}&\c{4 Li}	& \c{1 Li}	&\c{PProf, CProf}
& ID of input profiles and radial coordinate\\
\c{Npv} &\c{5 Li}	& \c{1 Li}	&\c{PlVac}
& \c{Npv < 240} - number of boundary data points\\
\c{nAst}&\c{6 Li}	& \c{1 Li}	&\c{Equil}
&\c{nAst < 0x200} number of profile data\\
\c{Mpol}&\c{7 Li}	& \c{1 Li}	&\c{Equil}
&\c{Mpol < 16} number of harmonics\\
\c{Fjob}&\c{8 Li}	& \c{1 Li}	&\c{Equil}
&Flag specifying the ESC job\\
\c{Ffail}&\c{9 Li}	& \c{1 Li}	&\c{Equil}
&Return failure flag\\
\c{nBal}&\c{10 Li}	& \c{1 Li}	&\c{BalSt}
&\c{nBal < 0x100} number of tested surfaces\\
\hline
\c{nP}&\c{6 Li}	& \c{1 Li}	&\c{RadCoord}
& Number of profile data points\\
\c{Mp}&\c{7 Li}	& \c{1 Li}	&\c{RadCoord}
& Highest poloidal harmonics in $\bgY$\\
\c{Fjob}&\c{8 Li}	& \c{1 Li}	&
& Flag for equilibrium solver\\
\c{Ffail}&\c{9 Li}	& \c{1 Li}	&
& Error Flag of equilibrium solver\\
\hline
\c{ShMemT}&\c{32 Li}& \c{16 Ld}	&\c{}	&Parameters\\
\c{Time}&\c{ShMemT}& \c{1 Ld}	&\c{PlVac}&$t$ reference time\\
\c{ESaR0}&\c{\&Time+Ld}	& \c{1 Ld}	&\c{PlVac}
&$R_0$ r of magnetic axis\\
\c{ESaZ0}&\c{\&Time+2Ld}& \c{1 Ld}	&\c{PlVac}
&$Z_0$ z of magnetic axis\\
\c{RBtor}&\c{\&Time+3Ld}& \c{1 Ld}	&\c{PlVac}&$R_{ext}B_{tor}$ data\\
\c{Rext}&\c{\&Time+4Ld}	& \c{1 Ld}	&\c{PlVac}
&$R_{ext}$ data - reference radius\\
\hline
\c{ShMemB}&\c{ShMemT+16}& \c{480 Ld}	&\c{PlVac}
& boundary data points\\
\c{Rpv}&\c{ShMemB}& \c{Npv Ld}	&\c{PlVac}
& boundary $r_i$ - data points\\
\c{Zpv}&\c{Rpv+Npv Ld}& \c{Npv Ld}	&\c{PlVac}
& boundary $z_i$ - data points\\
\hline
\c{ShMemP} &\c{Rpv+480Ld}	& \c{0x600 Ld}	&\c{}
&$a_i$-radial grid data points (from transport code to ESC)\\
\c{a}	&\c{ShMemP}	& \c{nP Ld}	&\c{RadCoord}
&$a_i$-radial grid data points (from transport code to ESC)\\
\c{p}	&\c{\&a+nP Ld}	& \c{nP Ld}	&\c{PProf}
&$p_i$-pressure related data\\
\c{j}	&\c{\&p+nP Ld}	& \c{nP Ld}	&\c{CProf}
&$j_i$-current related data\\
\hline
\c{ShMemE}&\c{ShMemP+0x600 Ld}	& \c{Nesi Ld}	&\c{Equil}
&Start of \c{esi} interface\\
\c{esi}&\c{ShMemE}	& \c{Np1+(11+18*Np1)*Na1}	&\c{Equil}
&\c{esi} interface output data\\
\hline
\c{ShMBal}&\c{ShMemE+Nesi*Ld}	& \c{0x100*(Ld+Li)}	&\c{BalSt}
&Start of ballooning stability data\\
\c{ShMaBl}&\c{ShMemE+Nesi*Ld}	& \c{0x100*Ld}	&\c{BalSt}
&\c{aBal[]}=0 ESC minor radii of tested surfaces\\
\c{ShMkBl}&\c{ShMaBl+256*Ld}	& \c{0x100*Li}	&\c{BalSt}
&\c{kBal[]}=0 in surface is stable, 1 - ballooning unstable, 2 - Mercier
unstable\\
\hline
\etbl

\Subsection{Extra memory for Fourier coefficients}
\btbl{|llrrp{1.6in}|}\hline
ID 	& Byte		& Length	&Section & Comment\\\hline
\c{ESsb}&\c{ShMaBl+0x100*Ld}	& \c{Na1}
&&$b_i$ spline coefficients\\
\c{ESsb2a}&\c{ESsb+Na1*Ld}	& \c{Na1}
&&$b''_i$ spline coefficients\\
\c{rcT}&\c{ESsb2a+Na1*Ld}	& \c{Na1*Fp1*Ld}
&&$r^c_i$ Fourier coefficients\\
\c{rsT}&\c{rcT+Na1*Fp1*Ld}	& \c{Na1*Fp1*Ld}
&&$r^s_i$ Fourier coefficients\\
\c{rcT2a}&\c{rsT+Na1*Fp1*Ld}	& \c{Na1*Fp1*Ld}
&&$r''^c_i$ Fourier coefficients\\
\c{rsT2a}&\c{rcT2a+Na1*Fp1*Ld}	& \c{Na1*Fp1*Ld}
&&$r''^s_i$ Fourier coefficients\\
\hline
L&\c{}	& \c{Na1*(2+4*Fp1)*Ld}
&&Total length\\
\hline
\etbl

\Subsection{Fourier representation of $r(a,\gq),z(a,\gq)$}
\bqbl
r=R_0+r^c_0(a)+\sum_{k=1}^{k<K_1}\left[r^c_k(a)\cos k\gq+r^s_k(a)\sin\gq\right]
,\quad
z=Z_0+r^s_0(a)+b(a)\sin\gq
.\eqbl
Note, that the position of the magnetic axis is extracted from the $k=0$
Fourier coefficients, $r^c_0(0)=r^s_0(0)=0$.

Spline coefficients are stored in 1-D arrays as
\bqbl
R_0=\c{ESaR0[0]}
,\quad
Z_0=\c{ESaZ0[0]}
,\quad
K_1=\c{ESFp1}
,\quad
N_1=\c{ESNa1}
\quad-\quad{\rm intergers\ for\ dimensions}
,\\
\\
r^c_k(a_i)|_{0\leq k<K_1}^{0\leq i<N_1}=\c{rcT[N_1*k+i]}
,\quad
r''^c_k(a_i)|_{0\leq k<K_1}^{0\leq i<N_1}=\c{rcT2a[N_1*k+i]}
,\\
\\
r^s_k(a_i)|_{0\leq k<K_1}^{0\leq i<N_1}=\c{rcT[N_1*k+i]}
,\quad
r''^s_k(a_i)|_{0\leq k<K_1}^{0\leq i<N_1}=\c{rcT2a[N_1*k+i]}
,\\
\\
b(a_i)|^{0\leq i<N_1}=\c{ESsb[i]}
,\quad
b''(a_i)|^{0\leq i<N_1}=\c{ESsb2a[i]}
.\eqbl

\Subsection{Reconstriction of Fourier coefficients}
The example is of reconstruction of Fourier coefficients and their first
derivatives in a specific point $a=0.44$ is given by
\vskip1em
{\obeylines\obeyspaces\c{
\{
\  extern int ESNa1,ESFp1;
\  extern int *ESaR0,*ESaZ0;
\  extern double *ESsb,*ESsb2a,*rcT,*rcT2a,*rsT,*rsT2a;
\  int k,m;
\  double b,b1a,rc[32],rc1a[32],rs[32],rs1a[32];
\  double a=0.44;
\  ESSetSplA(a);
\  splRA(\&b,\&b1a,ESsb,ESsb2a);
\  for(k=0; k < ESFp1; k++)\{
\    m=ESNa1*k;
\    splRA(rc+k,rc1a+k,rcT+m,rcT2a+m);
\    splRA(rs+k,rs1a+k,rsT+m,rsT2a+m);
\  \}
\  rc[0]+=ESaR0[0];
\  rs[0]+=ESaZ0[0];
\}
}}
\vskip1em
Here, routine \c{ESSetSplA(a)} finds the spline interval, which $a$
belongs to. Then the routine \c{splRA(rc+k, rc1a+k, rcT+m,rcT2a+m)}
calcultes function and its derivative given the spline coefficient for
function and its second derivative.

It is possible to pass \c{NULL} in the place of the first derivative,
i.e., \c{splRA(rc+k,NULL,rcT+m,rcT2a+m)}.


\Section{Skeleton of Shared memory communication}
\noindent\begin{tabular}{|rrlrrlp{2.2in}|}\hline
\multicolumn{3}{|c|}{\c{ASTRA}}&\multicolumn{3}{|c|}{\c{ESC}}&Comment\\\hline
\c{ASTRA}&\c{Command}&\c{ESC}&\c{ESC}&\c{Command}&\c{ASTRA}
&Semaphore positions: \c{L}ocked, 
\c{O}pen\\\hline
&\c{system();}&\c{O}&&&& Launching \ESC\\
\c{L}&&&\c{uL}&\c{shmget();}&\c{O}& Allocating shared memory in \ESC\\
\c{uL}&\c{shmat();}&\c{O}&\c{L}&&& Allocating shared memory in \c{ASTRA}\\
\c{L}&&&\c{u}&\c{PFC-WaveF}&& Hardware and WaveForm processing\\\hline
\c{L}&&&\c{uL}&&\c{O}& Openning a freeway for \c{ASTRA} loop\\
\c{uL}&\c{escin(}&\c{O}&\c{L}&&& Input data for \ESC\\
\c{L}&&&\c{u}&\c{PlVacShM();}&& Reading Plasma boundary data\\
\c{L}&&&\c{u}&\c{PlPrShM();}&& Reading Plasma pressure data\\
\c{L}&&&\c{uL}&\c{PlCrShM();}&\c{O}& Reading Plasma current data\\
\c{uL}&\c{IfFailure();}&\c{O}&\c{L}&&& Confirmation of data processing\\
\c{L}&\c{);}&&\c{u}&&&\\
\c{L}&&&\c{uL}&&\c{O}& Openning a freeway for \c{esc()} in \c{ASTRA}\\
\c{uL}&\c{esc(}&\c{O}&\c{L}&&& Putting data for equilibrium solver\\
\c{L}&&&\c{uL}&\c{2DEqSolv();}&\c{O}& Equilibrium solving\\
\c{uL}&\c{IfFailure();}&\c{O}&\c{L}&&& Confirmation of data processing\\
\c{L}&\c{);}&&\c{u}&&&\\
\c{L}&&&\c{uL}&\c{EqOutShM();}&\c{O}& Output into shared memory\\
\c{uL}&\c{e2astra();}&\c{O}&\c{L}&&&Taking equilibrium data\\
\hline
\end{tabular}
#endif

#ifndef mzl_System_2esc.c /* --------------- Use of Variables --------{ */
#ifndef H/* ---------- Used External Variables ------{ */
  Ignor double;
  Ignor int;
  Ignor key_t;
  Ignor void;
  Ignor char;
  Systm IPC_CREAT;
  Systm SETVAL;
  Systm NULL;
  Systm IPC_RMID;
  Systm getpid();
  Systm semget();
  Systm semctl();
  Systm sprintf();
  Systm semop();
  Systm system();
  Systm shmget();
  Systm shmat();
  Systm atan();
  Systm memcpy();
  Systm shmctl();
  Undef Mysemun;	/* undefined */
  Undef Sem0;		/* undefined */
  Undef Sem1;		/* undefined */
  Undef ESNa;		/* undefined */
  Undef ESNp;		/* undefined */
  Undef ESFp;		/* undefined */
  Undef ESNa1;		/* undefined */
  Undef ESNp1;		/* undefined */
  Undef ESFp1;		/* undefined */
  Undef ESISetMem();	/* NO prototype */
  Undef esifree_();	/* NO prototype */
  Undef ESIInit();	/* NO prototype */
#endif /* }======== End of Used External Variables ======= */

#endif /* }======== End of Use of Variables ======== */

double *ESaR0;			/*  */
double *ESaZ0;			/*  */
double *ESsr;			/*  */
double *ESsz;			/*  */
double *ESaB;			/*  */
double *ESsb;			/*  */
double *ESsb2a;			/*  */
double *rcT;			/*  */
double *rcT2a;			/*  */
double *rsT;			/*  */
double *rsT2a;			/*  */
double ESgt[];			/*  */
static int Pid;			/*  */
static int SemID;		/*  */
static int ShmID;		/*  */
static struct sembuf bufP;	/*  */
static struct sembuf bufV;	/*  */
static char *lShm;		/*  */
static int kD;			/*  */
static int kI;			/*  */
static int *ShMemI;		/*  */
static double *ShMemT;		/*  */
static double *ShMemB;		/*  */
static double *ShMemP;		/*  */
static double *ShMemE;		/*  */
static double *ShMBal;		/*  */
static double *aBal;		/*  */
static int *kBal;		/*  */
static int nBal;		/*  */
static int NBal;		/*  */

int LaunchEsc(Opt)
  char *Opt;	/*  */
{
  Undef Mysemun;
  Undef Sem0;
  Undef Sem1;
  Undef ESNa;
  Undef ESNp;
  Undef ESFp;
  Undef ESNa1;
  Undef ESNp1;
  Undef ESFp1;
  Undef ESISetMem();
  char *Opt;			/*  .LaunchEsc():2esc.c */
  static int kD;		/*              :2esc.c */
  static int kI;		/*              :2esc.c */
  static int Pid;		/*              :2esc.c */
  static int SemID;		/*              :2esc.c */
  static struct sembuf bufP;	/*              :2esc.c */
  static struct sembuf bufV;	/*              :2esc.c */
  static int ShmID;		/*              :2esc.c */
  static char *lShm;		/*              :2esc.c */
  static int *ShMemI;		/*              :2esc.c */
  static double *ShMemT;	/*              :2esc.c */
  static double *ShMemB;	/*              :2esc.c */
  static double *ShMemP;	/*              :2esc.c */
  static double *ShMemE;	/*              :2esc.c */
  double *ESaR0;		/*              :2esc.c */
  double *ESaZ0;		/*              :2esc.c */
  static double *ShMBal;	/*              :2esc.c */
  static double *aBal;		/*              :2esc.c */
  static int *kBal;		/*              :2esc.c */
  static int NBal;		/*              :2esc.c */
  double *ESsr;			/*              :2esc.c */
  double *ESsz;			/*              :2esc.c */
  double *ESaB;			/*              :2esc.c */
  double *ESsb;			/*              :2esc.c */
  double *ESsb2a;		/*              :2esc.c */
  double *rcT;			/*              :2esc.c */
  double *rcT2a;		/*              :2esc.c */
  double *rsT;			/*              :2esc.c */
  double *rsT2a;		/*              :2esc.c */
  double ESgt[];		/*              :2esc.c */
/*  :2esc.c */
#ifdef Doc_LaunchEsc
\section{Skeleton of LaunchEsc()}
  \begin{verbatim}
    LaunchEsc(){
    }
  \end{verbatim}
\section{LaunchEsc()}
\subsection{Description}
\subsection{Algorithm}
\subsection{ToDo}
\subsection{References}
#endif
}

void esc0_()
{
  int LaunchEsc();	/*  :2esc.c */
/*
   */
#ifdef Doc_esc0_
\section{Skeleton of esc0_()}
  \begin{verbatim}
    esc0_(){
    }
  \end{verbatim}
\section{esc0_()}
\subsection{Description}
\subsection{Algorithm}
\subsection{ToDo}
\subsection{References}
#endif
}

void esc0()
{
  void esc0_();	/*  :2esc.c */
/*
   */
#ifdef Doc_esc0
\Section{Skeleton of \c{esc0()}}
  \begin{verbatim}
    esc0(){
    }
  \end{verbatim}
\Section{\c{esc0()}}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Formal parameters}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

void esc1_()
{
  Undef esifree_();
  Undef Mysemun;
  static int SemID;		/*  :2esc.c */
  static struct sembuf bufP;	/*  :2esc.c */
  static char *lShm;		/*  :2esc.c */
  static int kI;		/*  :2esc.c */
  static struct sembuf bufV;	/*  :2esc.c */
  static int ShmID;		/*  :2esc.c */
/*  :2esc.c */
#ifdef Doc_esc1_
\section{Skeleton of esc1_()}
  \begin{verbatim}
    esc1_(){
    }
  \end{verbatim}
\section{esc1_()}
\subsection{Description}
\subsection{Algorithm}
\subsection{ToDo}
\subsection{References}
#endif
}

void escin_(pProf,jProf,aProf,nProf,nrhH,Rpv,Zpv,Npv,RBtor,Rext)
  double *pProf;	/*  */
  double *jProf;	/*  */
  double *aProf;	/*  */
  int *nProf;		/*  */
  int *nrhH;		/*  */
  double *Rpv;		/*  */
  double *Zpv;		/*  */
  int *Npv;		/*  */
  double *RBtor;	/*  */
  double *Rext;		/*  */
{
  int *nrhH;			/*  .escin_():2esc.c */
  int *Npv;			/*  .escin_():2esc.c */
  int *nProf;			/*  .escin_():2esc.c */
  double *RBtor;		/*  .escin_():2esc.c */
  double *Rext;			/*  .escin_():2esc.c */
  double *Rpv;			/*  .escin_():2esc.c */
  double *Zpv;			/*  .escin_():2esc.c */
  double *aProf;		/*  .escin_():2esc.c */
  double *pProf;		/*  .escin_():2esc.c */
  double *jProf;		/*  .escin_():2esc.c */
  static int SemID;		/*           :2esc.c */
  static struct sembuf bufP;	/*           :2esc.c */
  static int *ShMemI;		/*           :2esc.c */
  static double *ShMemT;	/*           :2esc.c */
  static int kD;		/*           :2esc.c */
  static double *ShMemB;	/*           :2esc.c */
  static double *ShMemP;	/*           :2esc.c */
  static struct sembuf bufV;	/*           :2esc.c */
/*
   */
#ifdef Doc_escin_
\Section{Skeleton of \c{escin\_()}}
  \begin{verbatim}
    escin_(){
    }
  \end{verbatim}
\Section{\c{escin\_()}}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{3.5in}|}\hline
\c{ID}&\bf\em Math&\bf\em Units&\bf\em Comment\\\hline
\c{pProf}&\em depend on \c{H}&\em in \c{dDhH}&pressure related profile\\
\c{jProf}&\em depend on \c{h}&\em in \c{dDhH}&current density related profile\\
\c{jProf[nProf]}&$I_{pl}$&[MA]&Total plasma current in\\
\c{aProf}&\em depend on \c{dD}&\em in \c{dDhH}&radial grid for profiles\\
\c{nProf}&$n$&&number of points in the profiles\\
\c{dDhH}&$$&&Control Index composed from 4 decimal digits\\
\c{Rpv}&$$&[m]&r-coordinates of plasma boundary points\\
\c{Zpv}&$$&[m]&r-coordinates of plasma boundary points\\
\c{Npv}&$N_{pl-vacuum}$&&number of plasma boundary points. {\em First
  and last point 
  may coinside.}\\
\c{RBtor}&$rB_{\gf,ext}$&[m$\cdot$T]&Product of external toroidal field 
$B_{\gf,ext}(r)$ and $r$\\
\c{Rext}&$R_{ext}$&[m]&Reference radius used for normalization of
magnetic field on $B_{\gf,ext}(R_{ext})$\\
\hline\end{tabular}\vskip1ex

The meaning of profiles depends on value of Index \c{dDhH}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\multicolumn{4}{|c|}{\bf Meaning of 4 digits in \c{dDhH}}\\\hline
\c{ID}&\bf\em Math&\bf\em Units&\bf\em Comment\\\hline
\c{H=0}&$\bsj_p\equiv R_0\R{d\bsp}{d\bgY}$
&$\left[\R{\mbox{MA}}{\mbox{m}^2}\right]$
&$R_0$ [m] is the radius of magnetic axis, $\bsp\equiv\gm_0p$ [MPa],
$\gm_0=0.4\gp$, $\bgY\equiv\R{\gY}{2\gp}$ [Vsec]\\
\c{H=1}&$P\equiv\R{d\bsp}{d\bgY}$&$\left[\R{\mbox{MA}}{\mbox{m}^3}\right]$
&$\bsp\equiv\gm_0p$ [MPa], $\gm_0=0.4\gp$, $\bgY\equiv\R{\gY}{2\gp}$ [Vsec]\\
\c{H=2}&$\bsp$&[MPa]&$\bsp\equiv\gm_0p$ [MPa], $\gm_0=0.4\gp$\\\hline
\c{h=0}&$\bsj_s\equiv\bsj_p+\R{\baF}{R_0}\R{d\baF}{d\bgY}$
&$\left[\R{\mbox{MA}}{\mbox{m}^2}\right]$&$R_0$ [m] is the radius of
magnetic axis, $\baF\equiv rB_\gf$ [m$\cdot$T], 
$\bgY\equiv\R{\gY}{2\gp}$ [Vsec]\\
\c{h=1}&$\bsj_\parallel\equiv\R{\left<\wsj\cdot\waB\right>}
 {R_0\left<\waB\cdot\na\gf\right>}$
&$\left[\R{\mbox{MA}}{\mbox{m}^2}\right]$&$R_0$ [m] is the radius of
magnetic axis, averaging is performed over poloidal angle $\gq$ with
Jacobian $\q{g}$ as a weight factor\\
\c{h=2}&$\bsj_B\equiv\R{\left<\wsj\cdot\waB\right>}
{\left<\waB\cdot\na\gf\right>}$
&$\left[\R{\mbox{MA}}{\mbox{m}}\right]$&averaging is performed over
poloidal angle $\gq$ with Jacobian $\q{g}$ as a weight factor\\
\c{h=3}&$T\equiv\baF\R{d\baF}{d\bgY}$
&$\left[\R{\mbox{MA}}{\mbox{m}}\right]$&$\baF\equiv rB_\gf$ [m$\cdot$T], 
$\bgY\equiv\R{\gY}{2\gp}$ [Vsec]\\
\c{h=6}&$q$&&safety factor\\
\c{h=7}&$\gm=\d\gi\equiv\R{1}{q}$&&inverse safety factor, or
rotational transform \\
\c{h=8}&$-\gY$&&minus (stupid, because of current ASTRA implementation)
poloidal flux. \em Works so far only with ASTRA code.\\\hline 
\c{d=0}&&&The radial coordinate for plasma profiles is converted by ESC
into a NORMALIZED coordinate. \em The input \c{aProf} array may be not
normalized\\
\c{d=1}&&&Plasma profiles are considered as given on absolute
coordinate. \em ESC extends or shrinks the profiles during iterations.\\\hline
\c{D=0}&$b$&[m]&vertical semiaxis as the radial coordinate\\\hline
\c{D=1}&$a_v\equiv\q{\R{V}{2\gp^2 R_{ext}}}$&$[m]$&volume $V$ based radial 
coordinate\\\hline
\c{D=2}&$a_\gF\equiv\q{\R{\gF}{\gp B_{ext}}}$&$[m]$&toroidal flux $\gF$
based radial coordinate\\\hline
\c{D=3}&$a_\gY\equiv a_0\q{\R{2(\gY_0-\gY)}{\gm_0RI_{pl}}}$&$[m]$&poloidal
flux $\gY$ based radial coordinate. \em For positive current $I_{pl}>0$
poloidal flux $\gY$ decays from the magnetic axis toward the plasma
boundary.\\\hline 
\hline\end{tabular}\vskip1ex
Processing the boundary points by ESC depends on value of \c{Npl}.
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\multicolumn{4}{|c|}{\bf Interpretation of \c{Npl} by ESC}\\\hline
\c{ID}&\bf\em Math&\bf\em Units&\bf\em Comment\\\hline
\c{Npl < 13}&$$&&Radial points \c{Rpl,Zpl} are considered as
characteristic points, 4 first of them enumerted as top, bottom, left,
right points. \em ESC generates Fourier representation of the boundary by
exact matching the characterisitc points.\\
\c{Npl > 12}&$$&&ESC matches \c{Rpl,Zpl} by square root minimization of
Fourier representation of the boundary.\\
\hline\end{tabular}\vskip1ex
\Subsection{Input}
\Subsection{Output}
\vskip1ex\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

void escin(pProf,jProf,aProf,nProf,nrhH,Rpv,Zpv,Npv,RBtor,Rext)
  double *pProf;	/*  */
  double *jProf;	/*  */
  double *aProf;	/*  */
  int *nProf;		/*  */
  int *nrhH;		/*  */
  double *Rpv;		/*  */
  double *Zpv;		/*  */
  int *Npv;		/*  */
  double *RBtor;	/*  */
  double *Rext;		/*  */
{
  double *pProf;/*  .escin():2esc.c */
  double *jProf;/*  .escin():2esc.c */
  double *aProf;/*  .escin():2esc.c */
  int *nProf;	/*  .escin():2esc.c */
  int *nrhH;	/*  .escin():2esc.c */
  double *Rpv;	/*  .escin():2esc.c */
  double *Zpv;	/*  .escin():2esc.c */
  int *Npv;	/*  .escin():2esc.c */
  double *RBtor;/*  .escin():2esc.c */
  double *Rext;	/*  .escin():2esc.c */
  void escin_();/*          :2esc.c */
/*
   */
#ifdef Doc_escin
\Section{Skeleton of \c{escin()}}
  \begin{verbatim}
    escin(){
    }
  \end{verbatim}
\Section{\c{escin()}}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Formal parameters}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

void esc_(Ffail,Fjob,Mpol,sTime)
  int *Ffail;		/*  */
  int *Fjob;		/*  */
  int *Mpol;		/*  */
  double *sTime;	/*  */
#ifdef Doc_sTime
#endif
{
  Undef ESIInit();
  double *sTime;		/*  .esc_():2esc.c */
  int *Fjob;			/*  .esc_():2esc.c */
  int *Mpol;			/*  .esc_():2esc.c */
  int *Ffail;			/*  .esc_():2esc.c */
  static int SemID;		/*         :2esc.c */
  static struct sembuf bufP;	/*         :2esc.c */
  static double *ShMemT;	/*         :2esc.c */
  static int *ShMemI;		/*         :2esc.c */
  static struct sembuf bufV;	/*         :2esc.c */
/*
   */
#ifdef Doc_esc_
\section{Skeleton of esc_()}
  \begin{verbatim}
    esc_(){
    }
  \end{verbatim}
\section{esc_()}
\subsection{Description}
\subsection{Algorithm}
\subsection{ToDo}
\subsection{References}
#endif
}

void esc(Ffail,Fjob,Mpol,sTime)
  int *Ffail;		/*  */
  int *Fjob;		/*  */
  int *Mpol;		/*  */
  double *sTime;	/*  */
#ifdef Doc_sTime
#endif
{
  int *Ffail;	/*  .esc():2esc.c */
  int *Fjob;	/*  .esc():2esc.c */
  int *Mpol;	/*  .esc():2esc.c */
  double *sTime;/*  .esc():2esc.c */
  void esc_();	/*        :2esc.c */
/*
   */
#ifdef Doc_esc
\Section{Skeleton of \c{esc()}}
  \begin{verbatim}
    esc(){
    }
  \end{verbatim}
\Section{\c{esc()}}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Formal parameters}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|llp{4in}|}\hline
\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int getbalst_(a,ind,n)
  double *a;	/*  */
  int *ind;	/*  */
  int *n;	/*  */
{
  int *n;		/*  .getbalst_():2esc.c */
  double *a;		/*  .getbalst_():2esc.c */
  int *ind;		/*  .getbalst_():2esc.c */
  static int *ShMemI;	/*              :2esc.c */
  static double *aBal;	/*              :2esc.c */
  static int *kBal;	/*              :2esc.c */
  /*
   */
#ifdef Doc_getbalst_
\Section{Skeleton of \c{getbalst_()}}
  \begin{verbatim}
    getbalst_(){
    }
  \end{verbatim}
\Section{\c{getbalst_()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*a}&$$&\\
\c{int }&\c{*ind}&$$&\\
\c{int }&\c{*n}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{int }&\c{*n}&$$&\\
\c{static int }&\c{*ShMemI}&$$&\\
\c{double }&\c{*a}&$$&\\
\c{static double }&\c{*aBal}&$$&\\
\c{int }&\c{*ind}&$$&\\
\c{static int }&\c{*kBal}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}
#endif

#undef Doc_Cb_Structure
#ifdef Blb_2esc.c
Blb_2esc.c{
}
#endif

#undef Doc_Table_Of_Content
#ifndef Blb_2esc.c
			PS p= -1 (  0);
int LaunchEsc();	PS p=  1 (  1);
void esc0_();		PS p=  3 (  2);
void esc0();		PS p=  3 (  2);
void esc1_();		PS p=  3 (  2);
void escin_();		PS p=  3 (  2);
void escin();		PS p=  5 (  3);
void esc_();		PS p=  5 (  3);
void esc();		PS p=  5 (  3);
int getbalst_();	PS p=  6 (  3);
#endif
