#include <stdio.h>
#include <string.h>
#include <math.h>

typedef struct HERM{
  double cT1;
  double cT2;
  double kT1;
  double kT2;
  double ct1;
  double ct2;
  double kt1;
  double kt2;
}HERM;

static HERM g[4],u[4],v[4];

static double P[10][10];
/* Calculation of elementary power integrals */
int IntPower()
{
  int m1,m2,m,n;
  double S,f;

  for(m1=0; m1 < 10; m1++){
    for(m2=0; m2 < 10-m1; m2++){
      if(m1 < m2){
	m	=m1;
	n	=m2;
      }
      else{
	m	=m2;
	n	=m1;
      }
      n++;
      S	=1./n;
      while(m > 0){
	n++;
	S	*=(double)m/n;
	m--;
      }
      P[m1][m2]	=S;
    }
    while(m2 < 10){
      P[m1][m2]	=0.;
      m2++;
    }
  }

#ifdef H
  for(m1=0; m1 < 10; m1++){
    printf("P[%1d][0-4]=%13.7e %13.7e %13.7e %13.7e %13.7e\n",m1
	   ,P[m1][0],P[m1][1],P[m1][2],P[m1][3],P[m1][4]);
  }
    printf("------------------------------------------\n");
  for(m1=0; m1 < 10; m1++){
    printf("P[%1d][5-9]=%13.7e %13.7e %13.7e %13.7e %13.7e\n",m1
	   ,P[m1][5],P[m1][6],P[m1][7],P[m1][8],P[m1][9]);
  }
#endif
  return(0);
}

double uuu[4][4][4],uuv[4][4][4],uvv[4][4][4];

/* Calculation of Hermittian elementary integrals */
void main()
{
  int k,k1,k2,k3,i1,i2,i3,kT1,kT2,kT3,kt1,kt2,kt3;
  double S,f1,f2,f3;
  HERM *pu2,*pu3,*pU2,*pU3;

  g[0].cT1	=1.;
  g[0].cT2	=0.;
  g[0].kT1	=1;
  g[0].kT2	=0;
  g[0].ct1	=0.;
  g[0].ct2	=0.;
  g[0].kt1	=0;
  g[0].kt2	=0;

  g[1].cT1	=0.;
  g[1].cT2	=0.;
  g[1].kT1	=0;
  g[1].kT2	=0;
  g[1].ct1	=1.;
  g[1].ct2	=0.;
  g[1].kt1	=1;
  g[1].kt2	=0;

  g[2].cT1	=1.;
  g[2].cT2	=-1.;
  g[2].kT1	=3;
  g[2].kT2	=1;
  g[2].ct1	=0.;
  g[2].ct2	=0.;
  g[2].kt1	=0;
  g[2].kt2	=0;

  g[3].cT1	=0.;
  g[3].cT2	=0.;
  g[3].kT1	=0;
  g[3].kT2	=0;
  g[3].ct1	=1.;
  g[3].ct2	=-1.;
  g[3].kt1	=3;
  g[3].kt2	=1;

  u[0].cT1	=3.;
  u[0].cT2	=-2.;
  u[0].kT1	=2;
  u[0].kT2	=3;
  u[0].ct1	=0.;
  u[0].ct2	=0.;
  u[0].kt1	=0;
  u[0].kt2	=0;

  u[1].cT1	=0.;
  u[1].cT2	=0.;
  u[1].kT1	=0;
  u[1].kT2	=0;
  u[1].ct1	=3.;
  u[1].ct2	=-2.;
  u[1].kt1	=2;
  u[1].kt2	=3;

  u[2].cT1	=1.;
  u[2].cT2	=-1.;
  u[2].kT1	=2;
  u[2].kT2	=3;
  u[2].ct1	=0.;
  u[2].ct2	=0.;
  u[2].kt1	=0;
  u[2].kt2	=0;

  u[3].cT1	=0.;
  u[3].cT2	=0.;
  u[3].kT1	=0;
  u[3].kT2	=0;
  u[3].ct1	=-1.;
  u[3].ct2	=1.;
  u[3].kt1	=2;
  u[3].kt2	=3;

  v[0].cT1	=6.;
  v[0].cT2	=-6.;
  v[0].kT1	=2;
  v[0].kT2	=1;
  v[0].ct1	=0.;
  v[0].ct2	=0.;
  v[0].kt1	=0;
  v[0].kt2	=0;

  v[1].cT1	=0.;
  v[1].cT2	=0.;
  v[1].kT1	=0;
  v[1].kT2	=0;
  v[1].ct1	=6.;
  v[1].ct2	=-6.;
  v[1].kt1	=1;
  v[1].kt2	=2;

  v[2].cT1	=3.;
  v[2].cT2	=-2.;
  v[2].kT1	=2;
  v[2].kT2	=1;
  v[2].ct1	=0.;
  v[2].ct2	=0.;
  v[2].kt1	=0;
  v[2].kt2	=0;

  v[3].cT1	=0.;
  v[3].cT2	=0.;
  v[3].kT1	=0;
  v[3].kT2	=0;
  v[3].ct1	=3.;
  v[3].ct2	=-2.;
  v[3].kt1	=2;
  v[3].kt2	=1;

  IntPower();
  for(k=0; k < 3; k++){
    switch(k){
    case 0:
      pU2	=u;
      pU3	=u;
      break;
    case 1:
      pU2	=u;
      pU3	=v;
      break;
    case 2:
      pU2	=v;
      pU3	=v;
      break;
    }
    for(k1=0; k1 < 4; k1++){
      for(k2=0; k2 < 4; k2++){
	pu2	=pU2+k2;
	for(k3=0; k3 < 4; k3++){
	  pu3	=pU3+k3;
	  S	=0.;
	  for(i1=0; i1 < 4; i1++){
	    kT1	=0;
	    kt1	=0;
	    switch(i1){
	    case 0:
	      f1	=g[k1].cT1;
	      kT1	=g[k1].kT1;
	      break;
	    case 1:
	      f1	=g[k1].cT2;
	      kT1	=g[k1].kT2;
	      break;
	    case 2:
	      f1	=g[k1].ct1;
	      kt1	=g[k1].kt1;
	      break;
	    case 3:
	      f1	=g[k1].ct2;
	      kt1	=g[k1].kt2;
	      break;
	    }
	    for(i2=0; i2 < 4; i2++){
	      kT2	=kT1;
	      kt2	=kt1;
	      switch(i2){
	      case 0:
		f2	=f1*pu2->cT1;
		kT2	+=pu2->kT1;
		break;
	      case 1:
		f2	=f1*pu2->cT2;
		kT2	+=pu2->kT2;
		break;
	      case 2:
		f2	=f1*pu2->ct1;
		kt2	+=pu2->kt1;
		break;
	      case 3:
		f2	=f1*pu2->ct2;
		kt2	+=pu2->kt2;
		break;
	      }
	      for(i3=0; i3 < 4; i3++){
		kT3	=kT2;
		kt3	=kt2;
		switch(i3){
		case 0:
		  f3	=f2*pu3->cT1;
		  kT3	+=pu3->kT1;
		  break;
		case 1:
		  f3	=f2*pu3->cT2;
		  kT3	+=pu3->kT2;
		  break;
		case 2:
		  f3	=f2*pu3->ct1;
		  kt3	+=pu3->kt1;
		  break;
		case 3:
		  f3	=f2*pu3->ct2;
		  kt3	+=pu3->kt2;
		  break;
		}
		if(f3 != 0.){
		  S	+=f3*P[kT3][kt3];
		}
	      }
	    }
	  } /* i1 */
	  switch(k){
	  case 0:
	    uuu[k1][k2][k3]	=S;
	    break;
	  case 1:
	    uuv[k1][k2][k3]	=S;
	    break;
	  case 2:
	    uvv[k1][k2][k3]	=S;
	    break;
	  }
	}
      }
    }
  }
  for(k1=0; k1 < 4; k1++){
    printf("------------------------------------------\n");
    for(k2=0; k2 < 4; k2++){
      printf("$u_%1d$ &%16.9e&%16.9e&%16.9e&%16.9e\\\\\n",k1+1
	     ,uuu[k1][k2][0],uuu[k1][k2][1],uuu[k1][k2][2],uuu[k1][k2][3]);
    }
  }
  for(k1=0; k1 < 4; k1++){
    printf("------------------------------------------\n");
    for(k2=0; k2 < 4; k2++){
      printf("$u_%1d$ &%16.9e&%16.9e&%16.9e&%16.9e\\\\\n",k1+1
	     ,uuv[k1][k2][0],uuv[k1][k2][1],uuv[k1][k2][2],uuv[k1][k2][3]);
    }
  }
  for(k1=0; k1 < 4; k1++){
    printf("------------------------------------------\n");
    for(k2=0; k2 < 4; k2++){
      printf("$v_%1d$ &%16.9e&%16.9e&%16.9e&%16.9e\\\\\n",k1+1
	     ,uvv[k1][k2][0],uvv[k1][k2][1],uvv[k1][k2][2],uvv[k1][k2][3]);
    }
  }
#ifndef H
#endif
}
