#include <math.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "esc.h"

#ifndef mzl_ESmain
extern double cr2,cr3,cr4,cr6,cr12,cgp,c2gp,cgp_4,cr2gp,cgm0,crgm0;
#endif

extern char ShotNm[32];

#ifndef mzl_1Dcore
extern int ESNa,ESNa1;
extern double ESa0,*ESsa,*ESgt;
#endif

#ifndef mzl_2Dcore
extern int ESNp,ESNp1,ESnAP;
extern int *ESnF,*ESmF,*ESkF,ESNf,ESNf1;
extern int ESMp,ESMp1,ES2Mp,ES2Mp1,ESnMp;
extern int ESFp,ESFp1,ESnAF;

extern double *ESemK,*ESemL,*ESemV,*ESemW,*ESemU1,*ESemU2;
extern double *ESemK2,*ESemL2,*ESemV2,*ESemW2,*ESemU12,*ESemU22;

extern double *ESr22,*ESr222,*ESr22c,*ESr22s,*ESr22c2,*ESr22s2;
extern double *ESg22c,*ESg22s,*ESg22c1,*ESg22s1,*ESg22c2,*ESg22s2;
extern double *ESg12c,*ESg12s,*ESg11c,*ESg11s;
extern double *ESg12c2,*ESg12s2,*ESg11c2,*ESg11s2;
extern double *ESLc,*ESLs,*ESVc,*ESVs,*ESDPc,*ESDPs;
extern double *ESLc1,*ESLs1,*ESVc1,*ESVs1,*ESDPc1,*ESDPs1;
extern double *ESLc2,*ESLs2,*ESVc2,*ESVs2,*ESDPc2,*ESDPs2;
extern double *ESjp,*ESjp1a,*ESjp2a,*ESjs,*ESjs1a,*ESjs2a
,*ESdgY,*ESdgY1a,*ESdgY2a;

extern double *cs1,*sn1,*cs2,*sn2;
extern double *dra,*drgt,*dza,*dzgt;
#endif

#ifndef mzl_3Dcore
extern int ESLt,ESNt,ESNt1,ESnAT,ESnAPT,ESLt0,ESNLt,ESNLt1;
extern double *ESsr,*ESsz,*rT1a,*zT1a,*rT1gt,*zT1gt;
extern double *ESaR0,*ESaZ0,*ESsb,*ESsb1a,*ESsb2a
,*rcT,*rsT,*rcT1a,*rsT1a,*rsT2a,*rcT2a;
#endif
extern double *ESFaux,*ESd2Faux;

static double Rmx=0.,Rmn=0.,Zmx=0.,Zmn=0.;
static double gtrmx=0.,gtrmn=0.,Zrmx=0.,Zrmn=0.;

int ES3DMetrTnsrCSHole()
{
  extern double ESa0;
  int i,j,ji,n1,im,im0,m,m1,mm;
  int k,ki,kj;
  double *rc,*rca,*rs,*rsa,*df1;

  double r0,rr0,b;
  double s,ba,rA;
  double raa,ragt,zaa,zagt,dg0,dg1;
  double *D,*L,*V,*G22,*G12,*G11;

  D	=dra+ESNp1;
  L	=D+ESNp1;
  rc	=L+ESNp1;

  V	=dza+ESNp1;
  G22	=V+ESNp1;
  rca	=G22+ESNp1;

  G12	=drgt+ESNp1;
  rs	=G12+ESNp1;

  G11	=dzgt+ESNp1;
  rsa	=G11+ESNp1;

  r0	=ESaR0[0];
  rr0	=1./r0;
  im0	=0;

  i	=0;
  ji	=0;
  rA	=1./ESsa[i];
  b	=ESsb[i];
  ba	=ESsb1a[i];
  ki	=i;
  rca[0]	=rcT1a[ki];
  rsa[0]	=rsT1a[ki];
  for(k=1; k < ESFp1; k++){
    ki	+=ESNa1;
    rca[k]	=2.*rcT1a[ki];
    rsa[k]	=2.*rsT1a[ki];
    rc[k]	=2.*k*rcT[ki];
    rs[k]	=2.*k*rsT[ki];
  }
  for(j=0; j < ESNp; j++){
    dra[j]	=rca[0];
    drgt[j]	=0.;
    kj	=0;
    for(k=1; k < ESFp1; k++){
      kj	+=j;
      if(kj >= ESNp){
	kj	-=ESNp;
      }
      dra[j]	+=rca[k]*cs1[kj]+rsa[k]*sn1[kj];
      drgt[j]	+=-rc[k]*sn1[kj]+rs[k]*cs1[kj];
    }
    dzgt[j]	=b*cs1[j];
    dza[j]	=rsa[0]+ba*sn1[j];
    D[j]	=dra[j]*dzgt[j]-drgt[j]*dza[j];
    if(D[j] <= 0.){
      printf("D[i=%2d][j=%2d]=%10.3e <= 0%c\n",i,j,D[j],7);
      return(1);
    }
    L[j]	=D[j]*r0/ESsr[ji]*rA;
    V[j]	=D[j]*ESsr[ji]*rr0*rA-L[j];
    dg0		=1./(D[j]*ESsr[ji]);
    D[j]	*=rA;
    G11[j]	=ESsa[i]*(dra[j]*dra[j]+dza[j]*dza[j])*dg0;
    G12[j]	=(dra[j]*drgt[j]+dza[j]*dzgt[j])*dg0;
    G22[j]	=(drgt[j]*drgt[j]+dzgt[j]*dzgt[j])*dg0*rA;
    rT1a[ji]	=dra[j];
    rT1gt[ji]	=drgt[j];
    zT1a[ji]	=dza[j];
    zT1gt[ji]	=dzgt[j];
    ji++;
  }
  D[j]	=D[0];
  L[j]	=L[0];
  V[j]	=V[0];
  G11[j]	=G11[0];
  G12[j]	=G12[0];
  G22[j]	=G22[0];
  rT1a[ji]	=dra[0];
  rT1gt[ji]	=drgt[0];
  zT1a[ji]	=dza[0];
  zT1gt[ji]	=dzgt[0];
  ji++;
  im		=0;
  ESgP2gF(ESDPc,ESDPs,D,ES2Mp);
  ESgP2gF(ESLc,ESLs,L,ES2Mp);
  ESgP2gF(ESVc,ESVs,V,ES2Mp);
  ESgP2gF(ESg22c,ESg22s,G22,ES2Mp);
  ESgP2gF(ESg12c,ESg12s,G12,ES2Mp);
  ESgP2gF(ESg11c,ESg11s,G11,ES2Mp);
  
  ji		-=ESNp1;
  ki		=i;
  rca[0]	=rcT2a[ki];
  rsa[0]	=rsT2a[ki];
  for(k=1; k < ESFp1; k++){
    ki	+=ESNa1;
    rca[k]	=2.*rcT2a[ki];
    rsa[k]	=2.*rsT2a[ki];
    b		=2.*k;
    rc[k]	=b*rcT1a[ki];
    rs[k]	=b*rsT1a[ki];
  }
  b		=ESsb1a[i];
  ba		=ESsb2a[i];
  for(j=0; j < ESNp; j++){
    raa		=rca[0];
    ragt	=0.;
    kj		=0;
    for(k=1; k < ESFp1; k++){
      kj	+=j;
      if(kj >= ESNp){
	kj	-=ESNp;
      }
      raa	+=rca[k]*cs1[kj]+rsa[k]*sn1[kj];
      ragt	+=-rc[k]*sn1[kj]+rs[k]*cs1[kj];
    }
    zaa		=rsa[0]+ba*sn1[j];
    zagt	=b*cs1[j];
    dg0		=-dra[j]/ESsr[ji];
    dg1		=raa*dzgt[j]+dra[j]*zagt-ragt*dza[j]-drgt[j]*zaa;
    L[j]	=(dg0-rA)*L[j]+dg1*r0/ESsr[ji];
    V[j]	=(D[j]*dra[j]+dg1*ESsr[ji]-D[j]*ESsr[ji]*rA)*rA*rr0-L[j];
    dg0		-=dg1/D[j];

    G12[j]	*=dg0;
    G22[j]	*=(dg0-rA);
    dg0		+=rA;
    G11[j]	*=dg0;
    dg0		=1./(D[j]*ESsr[ji]);
    G11[j]	+=2.*(dra[j]*raa+dza[j]*zaa)*dg0*ESsa[i];
    G12[j]	+=(raa*drgt[j]+dra[j]*ragt+zaa*dzgt[j]+dza[j]*zagt)*dg0;
    G22[j]	+=2.*(drgt[j]*ragt+dzgt[j]*zagt)*dg0*rA;
    D[j]	=(dg1-D[j]*rA)*rA;
    ji++;
  }
  D[j]	=D[0];
  L[j]	=L[0];
  V[j]	=V[0];
  G11[j]	=G11[0];
  G12[j]	=G12[0];
  G22[j]	=G22[0];
  ji++;
  im	=0;
  ESgP2gF(ESDPc2,ESDPs2,D,ES2Mp);
  ESgP2gF(ESLc2,ESLs2,L,ES2Mp);
  ESgP2gF(ESVc2,ESVs2,V,ES2Mp);
  ESgP2gF(ESg22c2,ESg22s2,G22,ES2Mp);
  ESgP2gF(ESg12c2,ESg12s2,G12,ES2Mp);
  ESgP2gF(ESg11c2,ESg11s2,G11,ES2Mp);

  for(i=1; i < ESNa1; i++){
    rA	=1./ESsa[i];
    b	=ESsb[i];
    ba	=ESsb1a[i];
    ki	=i;
    rca[0]	=rcT1a[ki];
    rsa[0]	=rsT1a[ki];
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      rca[k]	=2.*rcT1a[ki];
      rsa[k]	=2.*rsT1a[ki];
      rc[k]	=2.*k*rcT[ki];
      rs[k]	=2.*k*rsT[ki];
    }
    for(j=0; j < ESNp; j++){
      dra[j]	=rca[0];
      drgt[j]	=0.;
      kj	=0;
      for(k=1; k < ESFp1; k++){
	kj		+=j;
	if(kj >= ESNp){
	  kj	-=ESNp;
	}
	dra[j]	+=rca[k]*cs1[kj]+rsa[k]*sn1[kj];
	drgt[j]	+=-rc[k]*sn1[kj]+rs[k]*cs1[kj];
      }
      dzgt[j]	=b*cs1[j];
      dza[j]	=rsa[0]+ba*sn1[j];
      D[j]	=dra[j]*dzgt[j]-drgt[j]*dza[j];
      if(D[j] <= 0.){
	printf("D[i=%2d][j=%2d]=%10.3e <= 0%c\n",i,j,D[j],7);
	return(1);
      }
      L[j]	=D[j]*r0/ESsr[ji]*rA;
      V[j]	=D[j]*ESsr[ji]*rr0*rA-L[j];
      dg0	=1./(D[j]*ESsr[ji]);
      D[j]	*=rA;
      G11[j]	=ESsa[i]*(dra[j]*dra[j]+dza[j]*dza[j])*dg0;
      G12[j]	=(dra[j]*drgt[j]+dza[j]*dzgt[j])*dg0;
      G22[j]	=(drgt[j]*drgt[j]+dzgt[j]*dzgt[j])*dg0*rA;
      rT1a[ji]	=dra[j];
      rT1gt[ji]	=drgt[j];
      zT1a[ji]	=dza[j];
      zT1gt[ji]	=dzgt[j];
      ji++;
    }
    D[j]	=D[0];
    L[j]	=L[0];
    V[j]	=V[0];
    G11[j]	=G11[0];
    G12[j]	=G12[0];
    G22[j]	=G22[0];
    rT1a[ji]	=dra[0];
    rT1gt[ji]	=drgt[0];
    zT1a[ji]	=dza[0];
    zT1gt[ji]	=dzgt[0];
    ji++;
    ESgP2gF(ESDPc+i,ESDPs+i,D,ES2Mp);
    ESgP2gF(ESLc+i,ESLs+i,L,ES2Mp);
    ESgP2gF(ESVc+i,ESVs+i,V,ES2Mp);
    ESgP2gF(ESg22c+i,ESg22s+i,G22,ES2Mp);
    ESgP2gF(ESg12c+i,ESg12s+i,G12,ES2Mp);
    ESgP2gF(ESg11c+i,ESg11s+i,G11,ES2Mp);
  }
  i		=ESNa;
  ji		-=ESNp1;
  ki		=i;
  rca[0]	=rcT2a[ki];
  rsa[0]	=rsT2a[ki];
  for(k=1; k < ESFp1; k++){
    ki	+=ESNa1;
    rca[k]	=2.*rcT2a[ki];
    rsa[k]	=2.*rsT2a[ki];
    b		=2.*k;
    rc[k]	=b*rcT1a[ki];
    rs[k]	=b*rsT1a[ki];
  }
  b		=ESsb1a[i];
  ba		=ESsb2a[i];
  for(j=0; j < ESNp; j++){
    raa	=rca[0];
    ragt=0.;
    kj	=0;
    for(k=1; k < ESFp1; k++){
      kj	+=j;
      if(kj >= ESNp){
	kj	-=ESNp;
      }
      raa	+=rca[k]*cs1[kj]+rsa[k]*sn1[kj];
      ragt	+=-rc[k]*sn1[kj]+rs[k]*cs1[kj];
    }
    zaa	=rsa[0]+ba*sn1[j];
    zagt=b*cs1[j];
    dg0	=-dra[j]/ESsr[ji];
    dg1	=raa*dzgt[j]+dra[j]*zagt-ragt*dza[j]-drgt[j]*zaa;
    L[j]=(dg0-1.)*L[j]+dg1*r0/ESsr[ji];
    V[j]=(D[j]*dra[j]+dg1*ESsr[ji]-D[j]*ESsr[ji])*rr0-L[j];
    dg0	-=dg1/D[j];
    G12[j]	*=dg0;
    G22[j]	*=(dg0-1.);
    dg0	+=1.;
    G11[j]	*=dg0;
    dg0	=1./(D[j]*ESsr[ji]);
    G11[j]	+=2.*(dra[j]*raa+dza[j]*zaa)*dg0*ESsa[i];
    G12[j]	+=(raa*drgt[j]+dra[j]*ragt+zaa*dzgt[j]+dza[j]*zagt)*dg0;
    G22[j]	+=2.*(drgt[j]*ragt+dzgt[j]*zagt)*dg0;
    D[j]	=dg1-D[j];
    ji++;
  }
  D[j]	=D[0];
  L[j]	=L[0];
  V[j]	=V[0];
  G11[j]=G11[0];
  G12[j]=G12[0];
  G22[j]=G22[0];
  ji++;
  ESgP2gF(ESDPc2+i,ESDPs2+i,D,ES2Mp);
  ESgP2gF(ESLc2+i,ESLs2+i,L,ES2Mp);
  ESgP2gF(ESVc2+i,ESVs2+i,V,ES2Mp);
  ESgP2gF(ESg22c2+i,ESg22s2+i,G22,ES2Mp);
  ESgP2gF(ESg12c2+i,ESg12s2+i,G12,ES2Mp);
  ESgP2gF(ESg11c2+i,ESg11s2+i,G11,ES2Mp);

  i	=0;
  j	=ESNa;
  dg0	=ESDPc2[i];
  dg1	=ESDPc2[j];
  splAA(ESDPc+i,ESDPc1+i,ESDPc2+i,&dg0,&dg1);
  dg0	=ESLc2[i];
  dg1	=ESLc2[j];
  splAA(ESLc+i,ESLc1+i,ESLc2+i,&dg0,&dg1);
  dg0	=ESVc2[i];
  dg1	=ESVc2[j];
  splAA(ESVc+i,ESVc1+i,ESVc2+i,&dg0,&dg1);
  dg0	=ESg22c2[i];
  dg1	=ESg22c2[j];
  splAA(ESg22c+i,ESg22c1+i,ESg22c2+i,&dg0,&dg1);
  dg0	=ESg12c2[i];
  dg1	=ESg12c2[j];
  splA(ESg12c+i,ESg12c2+i,&dg0,&dg1);
  dg0	=ESg11c2[i];
  dg1	=ESg11c2[j];
  splA(ESg11c+i,ESg11c2+i,&dg0,&dg1);
  for(m=1; m < ES2Mp1; m++){
    i 	+=ESNa1;
    j	=i+ESNa;
    dg0	=ESDPc2[i];
    dg1	=ESDPc2[j];
    splAA(ESDPc+i,ESDPc1+i,ESDPc2+i,&dg0,&dg1);
    dg0	=ESDPs2[i];
    dg1	=ESDPs2[j];
    splAA(ESDPs+i,ESDPs1+i,ESDPs2+i,&dg0,&dg1);
    dg0	=ESLc2[i];
    dg1	=ESLc2[j];
    splAA(ESLc+i,ESLc1+i,ESLc2+i,&dg0,&dg1);
    dg0	=ESLs2[i];
    dg1	=ESLs2[j];
    splAA(ESLs+i,ESLs1+i,ESLs2+i,&dg0,&dg1);
    dg0	=ESVc2[i];
    dg1	=ESVc2[j];
    splAA(ESVc+i,ESVc1+i,ESVc2+i,&dg0,&dg1);
    dg0	=ESVs2[i];
    dg1	=ESVs2[j];
    splAA(ESVs+i,ESVs1+i,ESVs2+i,&dg0,&dg1);
    dg0	=ESg22c2[i];
    dg1	=ESg22c2[j];
    splAA(ESg22c+i,ESg22c1+i,ESg22c2+i,&dg0,&dg1);
    dg0	=ESg22s2[i];
    dg1	=ESg22s2[j];
    splAA(ESg22s+i,ESg22s1+i,ESg22s2+i,&dg0,&dg1);
    dg0	=ESg12c2[i];
    dg1	=ESg12c2[j];
    splA(ESg12c+i,ESg12c2+i,&dg0,&dg1);
    dg0	=ESg12s2[i];
    dg1	=ESg12s2[j];
    splA(ESg12s+i,ESg12s2+i,&dg0,&dg1);
    dg0	=ESg11c2[i];
    dg1	=ESg11c2[j];
    splA(ESg11c+i,ESg11c2+i,&dg0,&dg1);
    dg0	=ESg11s2[i];
    dg1	=ESg11s2[j];
    splA(ESg11s+i,ESg11s2+i,&dg0,&dg1);
  }
  return(0);
}

int ES3DMetrTnsrCSHoleV()
{
  int i,in,j,ji,n,n1,jni,im,im0,m,m1,mm;
  int K,k,ki,kj;
  double *rc,*rca,*rs,*rsa,*df1;

  double r0,rr0,b;
  double s,ba,rA;
  double raa,ragt,zaa,zagt,dg0,dg1;
  double *D,*L,*V,*G22,*G12,*G11;

  D	=dra+ESNp1;
  L	=D+ESNp1;
  rc	=L+ESNp1;

  V	=dza+ESNp1;
  G22	=V+ESNp1;
  rca	=G22+ESNp1;

  G12	=drgt+ESNp1;
  rs	=G12+ESNp1;

  G11	=dzgt+ESNp1;
  rsa	=G11+ESNp1;

  for(n=0; n < ESNLt; n++){
    K		=ESnAF*n;
    r0		=ESaR0[n];
    rr0		=1./r0;
    jni		=ESnAP*n;
    in		=ESNa1*n;
    im0		=ESnMp*n;
    jni		=ESnAP*n;
    i		=0;
    b		=ESsb1a[in];
    ki		=K+ESNa1;
    rc[1]	=2.*rcT1a[ki];
    rs[1]	=2.*rsT1a[ki];
    ba		=b*rc[1];
    for(j=0; j < ESNp; j++){
      drgt[j]	=-rc[1]*sn1[j]+rs[1]*cs1[j];
      dzgt[j]	=b*cs1[j];
      dra[j]	=rc[1]*cs1[j]+rs[1]*sn1[j];
      dza[j]	=b*sn1[j];
      L[j]	=ba;
      dg0	=rr0/ba;
      G11[j]	=(dra[j]*dra[j]+dza[j]*dza[j])*dg0;
      G12[j]	=(dra[j]*drgt[j]+dza[j]*dzgt[j])*dg0;
      G22[j]	=(drgt[j]*drgt[j]+dzgt[j]*dzgt[j])*dg0;
      rT1a[jni]	=dra[j];
      rT1gt[jni]=drgt[j];
      zT1a[jni]	=dza[j];
      zT1gt[jni]=dzgt[j];
      jni++;
    }
    L[j]	=L[0];
    G11[j]	=G11[0];
    G12[j]	=G12[0];
    G22[j]	=G22[0];
    rT1a[jni]	=dra[0];
    rT1gt[jni]	=drgt[0];
    zT1a[jni]	=dza[0];
    zT1gt[jni]	=dzgt[0];
    jni++;
    im		=im0;
    ESgP2gF(ESg22c+im,ESg22s+im,G22,ES2Mp);
    m1		=im;
    ESg11c[im]	=ESg22c[im];
    ESg11s[im]	=0.;
    ESg12c[im]	=0.;
    ESg12s[im]	=0.;
    ESDPc[im]	=ba;
    ESDPs[im]	=0.;
    ESLc[im]	=ba;
    ESLs[im]	=0.;
    ESVc2[im]	=0.;
    ESVs2[im]	=0.;
    m1	+=ESNa1;
    ESg11c[m1]	=0.;
    ESg11s[m1]	=0.;
    ESg12c[m1]	=0.;
    ESg12s[m1]	=0.;
    ESg22c[m1]	=0.;
    ESg22s[m1]	=0.;
    ESDPc[m1]	=0.;
    ESDPs[m1]	=0.;
    ESLc[m1]	=0.;
    ESLs[m1]	=0.;
    ESVc2[m1]	=ba*rc[1]*rr0;
    ESVs2[m1]	=ba*rs[1]*rr0;
    m1	+=ESNa1;
    ESg12c[m1]	=-ESg22s[m1];
    ESg12s[m1]	=ESg22c[m1];
    ESg11c[m1]	=-ESg22c[m1];
    ESg11s[m1]	=-ESg22s[m1];
    ESDPc[m1]	=0.;
    ESDPs[m1]	=0.;
    ESLc[m1]	=0.;
    ESLs[m1]	=0.;
    ESVc2[m1]	=0.;
    ESVs2[m1]	=0.;
    for(m=3; m < ES2Mp1; m++){
      m1	+=ESNa1;
      ESg11c[m1]	=0.;
      ESg11s[m1]	=0.;
      ESg12c[m1]	=0.;
      ESg12s[m1]	=0.;
      ESDPc[m1]		=0.;
      ESDPs[m1]		=0.;
      ESLc[m1]		=0.;
      ESLs[m1]		=0.;
      ESVc2[m1]		=0.;
      ESVs2[m1]		=0.;
    }
    ki		=K;
    rca[0]	=rcT2a[ki];
    rsa[0]	=rsT2a[ki];
    ki		+=ESNa1;
    k		=1;
    rca[1]	=2.*rcT1a[ki];
    rsa[1]	=2.*rsT1a[ki];
    ki		+=ESNa1;
    k		=2;
    rca[2]	=2.*rcT2a[ki];
    rsa[2]	=2.*rsT2a[ki];
    for(j=0; j < ESNp; j++){
      raa	=rca[0]+rca[2]*cs2[j]+rsa[2]*sn2[j];
      zaa	=rsa[0];
      ragt	=-rca[2]*sn2[j]+rsa[2]*cs2[j];
      dg1	=(b*(rca[0]+rca[2])-rsa[1]*rsa[0])*cs1[j]
	+(b*rsa[2]+rca[1]*rsa[0])*sn1[j];
      dg0	=-dra[j]*rr0;
      D[j]	=dg1;
      L[j]	=L[j]*dg0+dg1;
      dg0	-=dg1/ba;
      G11[j]	*=dg0;
      G12[j]	*=dg0;
      G22[j]	*=dg0;
      dg0	=rr0/ba;
      G11[j]	+=2.*(dra[j]*raa+dza[j]*zaa)*dg0;
      G12[j]	+=(raa*drgt[j]+dra[j]*ragt+zaa*dzgt[j])*dg0;
      G22[j]	+=2.*ragt*drgt[j]*dg0;
    }
    D[j]	=D[0];
    L[j]	=L[0];
    G11[j]	=G11[0];
    G12[j]	=G12[0];
    G22[j]	=G22[0];
    im		=im0+i;
    ESgP2gF(ESDPc2+im,ESDPs2+im,D,ES2Mp);
    ESgP2gF(ESLc2+im,ESLs2+im,L,ES2Mp);
    m1		=im;
    for(m=0; m < ES2Mp1; m++){
      ESVc[m1]	=0.;
      ESVs[m1]	=0.;
      m1	+=ESNa1;
    }
    ESgP2gF(ESg11c2+im,ESg11s2+im,G11,ES2Mp);
    ESgP2gF(ESg12c2+im,ESg12s2+im,G12,ES2Mp);
    ESgP2gF(ESg22c2+im,ESg22s2+im,G22,ES2Mp);
#ifdef H
    ki		=K;
    rca[0]	=rcT2a[ki];
    rsa[0]	=rsT2a[ki];
    ki	+=ESNa1;
    rc[1]	=2.*rcT1a[ki];
    rs[1]	=2.*rsT1a[ki];
    rca[1]	=2.*rcT2a[ki];
    rsa[1]	=2.*rsT2a[ki];
    for(k=2; k < ESFp1; k++){
      ki	+=ESNa1;
      rca[k]	=2.*rcT2a[ki];
      rsa[k]	=2.*rsT2a[ki];
    }
    for(j=0; j < ESNp; j++){
      drgt[j]	=-rc[1]*sn1[j]+rs[1]*cs1[j];
      ragt	=-rca[1]*sn1[j]+rsa[1]*cs1[j];
      dra[j]	=rc[1]*cs1[j]+rs[1]*sn1[j];
      raa	=rca[0];
      kj	=0;
      for(k=1; k < ESFp1; k++){
	kj	+=j;
	if(kj >= ESNp)
	  kj	-=ESNp;
	raa	+=rca[k]*cs1[kj]+rsa[k]*sn1[kj];
	ragt	=k*(-rca[k]*sn1[kj]+rsa[k]*cs1[kj]);
      }
      ragt	*=cr2;
      dzgt[j]	=ESsb1a[0]*cs1[j];
      zagt	=cr2*ESsb2a[0]*cs1[j];
      dza[j]	=ESsb1a[0]*sn1[j];
      zaa	=rsa[0]+ESsb2a[0]*sn1[j];
      dg1	=raa*dzgt[j]+dra[j]*zagt-ragt*dza[j]-drgt[j]*zaa;
      D[j]	=dg1;
      dg0	=-dra[j]*rr0;
      L[j]	=L[j]*dg0+dg1*rr0;
      dg0	-=dg1/ba;
      G11[j]	*=dg0;
      G12[j]	*=dg0;
      G22[j]	*=dg0;
      dg0	=rr0/ba;
      G11[j]	+=2.*(dra[j]*raa+dza[j]*zaa)*dg0;
      G12[j]	+=(raa*drgt[j]+dra[j]*ragt+zaa*dzgt[j])*dg0;
      G22[j]	+=2.*(drgt[j]*ragt+dzgt[j]*zagt)*dg0;
    }
    D[j]	=D[0];
    L[j]	=L[0];
    G11[j]	=G11[0];
    G12[j]	=G12[0];
    G22[j]	=G22[0];
    im		=im0+i;
    ESgP2gF(ESDPc2+im,ESDPs2+im,D,ES2Mp);
    ESgP2gF(ESLc2+im,ESLs2+im,L,ES2Mp);
    m1		=im;
    for(m=0; m < ES2Mp1; m++){
      ESVc[m1]	=0.;
      ESVs[m1]	=0.;
      m1	+=ESNa1;
    }
    ESgP2gF(ESg11c2+im,ESg11s2+im,G11,ES2Mp);
    ESgP2gF(ESg12c2+im,ESg12s2+im,G12,ES2Mp);
    ESgP2gF(ESg22c2+im,ESg22s2+im,G22,ES2Mp);
#endif
    in++;
    for(i=1; i < ESNa1; i++){
      rA	=1./ESsa[i];
      b		=ESsb[in];
      ba	=ESsb1a[in];
      ki	=K+i;
      rca[0]	=rcT1a[ki];
      rsa[0]	=rsT1a[ki];
      for(k=1; k < ESFp1; k++){
	ki	+=ESNa1;
	rca[k]	=2.*rcT1a[ki];
	rsa[k]	=2.*rsT1a[ki];
	rc[k]	=2.*k*rcT[ki];
	rs[k]	=2.*k*rsT[ki];
      }
      for(j=0; j < ESNp; j++){
	dra[j]	=rca[0];
	drgt[j]	=0.;
	kj	=0;
	for(k=1; k < ESFp1; k++){
	  kj		+=j;
	  if(kj >= ESNp)
	    kj	-=ESNp;
	  dra[j]	+=rca[k]*cs1[kj]+rsa[k]*sn1[kj];
	  drgt[j]	+=-rc[k]*sn1[kj]+rs[k]*cs1[kj];
	}
	dzgt[j]	=b*cs1[j];
	dza[j]	=rsa[0]+ba*sn1[j];
	D[j]	=dra[j]*dzgt[j]-drgt[j]*dza[j];
	if(D[j] <= 0.){
#ifdef H
	  printf("D[n=%3d][i=%2d][j=%2d]=%10.3e <= 0%c\n",n,i,j,D[j],7);
#endif
	  return(1);
	}
	L[j]	=D[j]*r0/ESsr[jni]*rA;
	V[j]	=D[j]*ESsr[jni]*rr0*rA-L[j];
	dg0	=1./(D[j]*ESsr[jni]);
	D[j]	*=rA;
	G11[j]	=ESsa[i]*(dra[j]*dra[j]+dza[j]*dza[j])*dg0;
	G12[j]	=(dra[j]*drgt[j]+dza[j]*dzgt[j])*dg0;
	G22[j]	=(drgt[j]*drgt[j]+dzgt[j]*dzgt[j])*dg0*rA;
	rT1a[jni]	=dra[j];
	rT1gt[jni]	=drgt[j];
	zT1a[jni]	=dza[j];
	zT1gt[jni]	=dzgt[j];
	jni++;
      }
      D[j]	=D[0];
      L[j]	=L[0];
      V[j]	=V[0];
      G11[j]	=G11[0];
      G12[j]	=G12[0];
      G22[j]	=G22[0];
      rT1a[jni]	=dra[0];
      rT1gt[jni]=drgt[0];
      zT1a[jni]	=dza[0];
      zT1gt[jni]=dzgt[0];
      jni++;
      im	=im0+i;
      ESgP2gF(ESDPc+im,ESDPs+im,D,ES2Mp);
      ESgP2gF(ESLc+im,ESLs+im,L,ES2Mp);
      ESgP2gF(ESVc+im,ESVs+im,V,ES2Mp);
      ESgP2gF(ESg22c+im,ESg22s+im,G22,ES2Mp);
      ESgP2gF(ESg12c+im,ESg12s+im,G12,ES2Mp);
      ESgP2gF(ESg11c+im,ESg11s+im,G11,ES2Mp);
      in++;
    }
    i		=ESNa;
    jni		-=ESNp1;
    in--;
    ki		=K+i;
    rca[0]	=rcT2a[ki];
    rsa[0]	=rsT2a[ki];
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      rca[k]	=2.*rcT2a[ki];
      rsa[k]	=2.*rsT2a[ki];
      b		=2.*k;
      rc[k]	=b*rcT1a[ki];
      rs[k]	=b*rsT1a[ki];
    }
    b		=ESsb1a[in];
    ba		=ESsb2a[in];
    for(j=0; j < ESNp; j++){
      raa	=rca[0];
      ragt	=0.;
      kj	=0;
      for(k=1; k < ESFp1; k++){
	kj	+=j;
	if(kj >= ESNp)
	  kj	-=ESNp;
	raa	+=rca[k]*cs1[kj]+rsa[k]*sn1[kj];
	ragt	+=-rc[k]*sn1[kj]+rs[k]*cs1[kj];
      }
      zaa	=rsa[0]+ba*sn1[j];
      zagt	=b*cs1[j];
      dg0	=-dra[j]/ESsr[jni];
      dg1	=raa*dzgt[j]+dra[j]*zagt-ragt*dza[j]-drgt[j]*zaa;
      L[j]	=(dg0-1.)*L[j]+dg1*r0/ESsr[jni];
      V[j]	=(D[j]*dra[j]+dg1*ESsr[jni]-D[j]*ESsr[jni])*rr0-L[j];
      dg0	-=dg1/D[j];
      G12[j]	*=dg0;
      G22[j]	*=(dg0-1.);
      dg0	+=1.;
      G11[j]	*=dg0;
      dg0	=1./(D[j]*ESsr[jni]);
      G11[j]	+=2.*(dra[j]*raa+dza[j]*zaa)*dg0*ESsa[i];
      G12[j]	+=(raa*drgt[j]+dra[j]*ragt+zaa*dzgt[j]+dza[j]*zagt)*dg0;
      G22[j]	+=2.*(drgt[j]*ragt+dzgt[j]*zagt)*dg0;
      D[j]	=dg1-D[j];
      jni++;
    }
    D[j]	=D[0];
    L[j]	=L[0];
    V[j]	=V[0];
    G11[j]	=G11[0];
    G12[j]	=G12[0];
    G22[j]	=G22[0];
    im		=im0+i;
    ESgP2gF(ESDPc2+im,ESDPs2+im,D,ES2Mp);
    ESgP2gF(ESLc2+im,ESLs2+im,L,ES2Mp);
    ESgP2gF(ESVc2+im,ESVs2+im,V,ES2Mp);
    ESgP2gF(ESg22c2+im,ESg22s2+im,G22,ES2Mp);
    ESgP2gF(ESg12c2+im,ESg12s2+im,G12,ES2Mp);
    ESgP2gF(ESg11c2+im,ESg11s2+im,G11,ES2Mp);
    im	=im0;
    in	=im+ESNa;
    dg0	=ESDPc2[im];
    dg1	=ESDPc2[in];
    splAA(ESDPc+im,ESDPc1+im,ESDPc2+im,&dg0,&dg1);
    dg0	=ESLc2[im];
    dg1	=ESLc2[in];
    splAA(ESLc+im,ESLc1+im,ESLc2+im,&dg0,&dg1);
    dg0	=ESVc2[im];
    dg1	=ESVc2[in];
    splAA(ESVc+im,ESVc1+im,ESVc2+im,&dg0,&dg1);
    dg0	=ESg22c2[im];
    dg1	=ESg22c2[in];
    splAA(ESg22c+im,ESg22c1+im,ESg22c2+im,&dg0,&dg1);
    dg0	=ESg12c2[im];
    dg1	=ESg12c2[in];
    splA(ESg12c+im,ESg12c2+im,&dg0,&dg1);
    dg0	=ESg11c2[im];
    dg1	=ESg11c2[in];
    splA(ESg11c+im,ESg11c2+im,&dg0,&dg1);
    for(m=1; m < 2; m++){
      im 	+=ESNa1;
      in	=im+ESNa;
      dg0	=ESDPc2[im];
      dg1	=ESDPc2[in];
      splAA2(ESDPc+im,ESDPc1+im,ESDPc2+im,ESsa,&dg1);
      dg0	=ESDPs2[im];
      dg1	=ESDPs2[in];
      splAA2(ESDPs+im,ESDPs1+im,ESDPs2+im,ESsa,&dg1);
      dg0	=ESLc2[im];
      dg1	=ESLc2[in];
      splAA2(ESLc+im,ESLc1+im,ESLc2+im,ESsa,&dg1);
      dg0	=ESLs2[im];
      dg1	=ESLs2[in];
      splAA2(ESLs+im,ESLs1+im,ESLs2+im,ESsa,&dg1);
      dg0	=ESVc2[im];
      dg1	=ESVc2[in];
      splAA2(ESVc+im,ESVc1+im,ESVc2+im,ESsa,&dg1);
      dg0	=ESVs2[im];
      dg1	=ESVs2[in];
      splAA2(ESVs+im,ESVs1+im,ESVs2+im,ESsa,&dg1);
      dg0	=ESg22c2[im];
      dg1	=ESg22c2[in];
      splAA2(ESg22c+im,ESg22c1+im,ESg22c2+im,ESsa,&dg1);
      dg0	=ESg22s2[im];
      dg1	=ESg22s2[in];
      splAA2(ESg22s+im,ESg22s1+im,ESg22s2+im,ESsa,&dg1);
      dg0	=ESg12c2[im];
      dg1	=ESg12c2[in];
      splA(ESg12c+im,ESg12c2+im,&dg0,&dg1);
      dg0	=ESg12s2[im];
      dg1	=ESg12s2[in];
      splA(ESg12s+im,ESg12s2+im,&dg0,&dg1);
      dg0	=ESg11c2[im];
      dg1	=ESg11c2[in];
      splA(ESg11c+im,ESg11c2+im,&dg0,&dg1);
      dg0	=ESg11s2[im];
      dg1	=ESg11s2[in];
      splA(ESg11s+im,ESg11s2+im,&dg0,&dg1);
    }
    for(m=2; m < ES2Mp1; m++){
      im 	+=ESNa1;
      in	=im+ESNa;
      dg1	=ESDPc2[in];
      splAA(ESDPc+im,ESDPc1+im,ESDPc2+im,ESsa,&dg1);
      dg1	=ESDPs2[in];
      splAA(ESDPs+im,ESDPs1+im,ESDPs2+im,ESsa,&dg1);
      dg1	=ESLc2[in];
      splAA(ESLc+im,ESLc1+im,ESLc2+im,ESsa,&dg1);
      dg1	=ESLs2[in];
      splAA(ESLs+im,ESLs1+im,ESLs2+im,ESsa,&dg1);
      dg1	=ESVc2[in];
      splAA(ESVc+im,ESVc1+im,ESVc2+im,ESsa,&dg1);
      dg1	=ESVs2[in];
      splAA(ESVs+im,ESVs1+im,ESVs2+im,ESsa,&dg1);
      if(m == 3){
	dg1	=ESg12c2[in];
	splAA2(ESg12c+im,ESg22c1+im,ESg12c2+im,ESsa,&dg1);
	dg1	=ESg12s2[in];
	splAA2(ESg12s+im,ESg22c1+im,ESg12s2+im,ESsa,&dg1);
	dg1	=ESg11c2[in];
	splAA2(ESg11c+im,ESg22c1+im,ESg11c2+im,ESsa,&dg1);
	dg1	=ESg11s2[in];
	splAA2(ESg11s+im,ESg22c1+im,ESg11s2+im,ESsa,&dg1);
	dg1	=ESg22c2[in];
	splAA2(ESg22c+im,ESg22c1+im,ESg22c2+im,ESsa,&dg1);
	dg1	=ESg22s2[in];
	splAA2(ESg22s+im,ESg22s1+im,ESg22s2+im,ESsa,&dg1);
      }
      else{
	dg1	=ESg22c2[in];
	splAA(ESg22c+im,ESg22c1+im,ESg22c2+im,ESsa,&dg1);
	dg1	=ESg22s2[in];
	splAA(ESg22s+im,ESg22s1+im,ESg22s2+im,ESsa,&dg1);
	dg1	=ESg12c2[in];
	splA(ESg12c+im,ESg12c2+im,ESsa,&dg1);
	dg1	=ESg12s2[in];
	splA(ESg12s+im,ESg12s2+im,ESsa,&dg1);
	dg1	=ESg11c2[in];
	splA(ESg11c+im,ESg11c2+im,ESsa,&dg1);
	dg1	=ESg11s2[in];
	splA(ESg11s+im,ESg11s2+im,ESsa,&dg1);
      }
    }
    n1	=n+ESNLt;
    while(n1 < ESNt1){
      im	=im0+ESnMp*n1;
      ji	=im0;
      for(m=0; m < ES2Mp1; m++){
	for(i=0; i < ESNa1; i++){
	  ESDPc[im]	=ESDPc[ji];
	  ESDPs[im]	=ESDPs[ji];
	  ESDPc1[im]	=ESDPc1[ji];
	  ESDPs1[im]	=ESDPs1[ji];
	  ESDPc2[im]	=ESDPc2[ji];
	  ESDPs2[im]	=ESDPs2[ji];

	  ESLc[im]	=ESLc[ji];
	  ESLs[im]	=ESLs[ji];
	  ESLc2[im]	=ESLc2[ji];
	  ESLs2[im]	=ESLs2[ji];

	  ESVc[im]	=ESVc[ji];
	  ESVs[im]	=ESVs[ji];
	  ESVc2[im]	=ESVc2[ji];
	  ESVs2[im]	=ESVs2[ji];

	  ESg22c[im]	=ESg22c[ji];
	  ESg22s[im]	=ESg22s[ji];
	  ESg22c1[im]	=ESg22c1[ji];
	  ESg22s1[im]	=ESg22s1[ji];
	  ESg22c2[im]	=ESg22c2[ji];
	  ESg22s2[im]	=ESg22s2[ji];

	  ESg12c[im]	=ESg12c[ji];
	  ESg12s[im]	=ESg12s[ji];
	  ESg12c2[im]	=ESg12c2[ji];
	  ESg12s2[im]	=ESg12s2[ji];

	  ESg11c[im]	=ESg11c[ji];
	  ESg11s[im]	=ESg11s[ji];
	  ESg11c2[im]	=ESg11c2[ji];
	  ESg11s2[im]	=ESg11s2[ji];
	  im++;
	  ji++;
	}
      }
      n1	+=ESNLt;
    }
  }
  return(0);
}

int ES3DMetrTnsrCS()
{
  int i,in,j,ji,n,n1,jni,im,im0,m,m1,mm;
  int K,k,ki,kj;
  double *rc,*rca,*rs,*rsa,*df1;

  double r0,rr0,b;
  double s,ba,rA;
  double raa,ragt,zaa,zagt,dg0,dg1;
  double *D,*L,*V,*G22,*G12,*G11;

  D	=dra+ESNp1;
  L	=D+ESNp1;
  rc	=L+ESNp1;

  V	=dza+ESNp1;
  G22	=V+ESNp1;
  rca	=G22+ESNp1;

  G12	=drgt+ESNp1;
  rs	=G12+ESNp1;

  G11	=dzgt+ESNp1;
  rsa	=G11+ESNp1;

  for(n=0; n < ESNLt; n++){
    K		=ESnAF*n;
    r0		=ESaR0[n];
    rr0		=1./r0;
    jni		=ESnAP*n;
    in		=ESNa1*n;
    im0		=ESnMp*n;
    jni		=ESnAP*n;
    i		=0;
    b		=ESsb1a[in];
    ki		=K+ESNa1;
    rc[1]	=2.*rcT1a[ki];
    rs[1]	=2.*rsT1a[ki];
    ba		=b*rc[1];
    for(j=0; j < ESNp; j++){
      drgt[j]	=-rc[1]*sn1[j]+rs[1]*cs1[j];
      dzgt[j]	=b*cs1[j];
      dra[j]	=rc[1]*cs1[j]+rs[1]*sn1[j];
      dza[j]	=b*sn1[j];
      L[j]	=ba;
      dg0	=rr0/ba;
      G11[j]	=(dra[j]*dra[j]+dza[j]*dza[j])*dg0;
      G12[j]	=(dra[j]*drgt[j]+dza[j]*dzgt[j])*dg0;
      G22[j]	=(drgt[j]*drgt[j]+dzgt[j]*dzgt[j])*dg0;
      rT1a[jni]	=dra[j];
      rT1gt[jni]=drgt[j];
      zT1a[jni]	=dza[j];
      zT1gt[jni]=dzgt[j];
      jni++;
    }
    L[j]	=L[0];
    G11[j]	=G11[0];
    G12[j]	=G12[0];
    G22[j]	=G22[0];
    rT1a[jni]	=dra[0];
    rT1gt[jni]	=drgt[0];
    zT1a[jni]	=dza[0];
    zT1gt[jni]	=dzgt[0];
    jni++;
    im		=im0;
    ESgP2gF(ESg22c+im,ESg22s+im,G22,ES2Mp);
    m1		=im;
    ESg11c[im]	=ESg22c[im];
    ESg11s[im]	=0.;
    ESg12c[im]	=0.;
    ESg12s[im]	=0.;
    ESDPc[im]	=ba;
    ESDPs[im]	=0.;
    ESLc[im]	=ba;
    ESLs[im]	=0.;
    ESVc2[im]	=0.;
    ESVs2[im]	=0.;
    m1	+=ESNa1;
    ESg11c[m1]	=0.;
    ESg11s[m1]	=0.;
    ESg12c[m1]	=0.;
    ESg12s[m1]	=0.;
    ESg22c[m1]	=0.;
    ESg22s[m1]	=0.;
    ESDPc[m1]	=0.;
    ESDPs[m1]	=0.;
    ESLc[m1]	=0.;
    ESLs[m1]	=0.;
    ESVc2[m1]	=ba*rc[1]*rr0;
    ESVs2[m1]	=ba*rs[1]*rr0;
    m1	+=ESNa1;
    ESg12c[m1]	=-ESg22s[m1];
    ESg12s[m1]	=ESg22c[m1];
    ESg11c[m1]	=-ESg22c[m1];
    ESg11s[m1]	=-ESg22s[m1];
    ESDPc[m1]	=0.;
    ESDPs[m1]	=0.;
    ESLc[m1]	=0.;
    ESLs[m1]	=0.;
    ESVc2[m1]	=0.;
    ESVs2[m1]	=0.;
    for(m=3; m < ES2Mp1; m++){
      m1	+=ESNa1;
      ESg11c[m1]	=0.;
      ESg11s[m1]	=0.;
      ESg12c[m1]	=0.;
      ESg12s[m1]	=0.;
      ESDPc[m1]		=0.;
      ESDPs[m1]		=0.;
      ESLc[m1]		=0.;
      ESLs[m1]		=0.;
      ESVc2[m1]		=0.;
      ESVs2[m1]		=0.;
    }
    ki		=K;
    rca[0]	=rcT2a[ki];
    rsa[0]	=rsT2a[ki];
    ki		+=ESNa1;
    k		=1;
    rca[1]	=2.*rcT1a[ki];
    rsa[1]	=2.*rsT1a[ki];
    ki		+=ESNa1;
    k		=2;
    rca[2]	=2.*rcT2a[ki];
    rsa[2]	=2.*rsT2a[ki];
    for(j=0; j < ESNp; j++){
      raa	=rca[0]+rca[2]*cs2[j]+rsa[2]*sn2[j];
      zaa	=rsa[0];
      ragt	=-rca[2]*sn2[j]+rsa[2]*cs2[j];
      dg1	=(b*(rca[0]+rca[2])-rsa[1]*rsa[0])*cs1[j]
	+(b*rsa[2]+rca[1]*rsa[0])*sn1[j];
      dg0	=-dra[j]*rr0;
      D[j]	=dg1;
      L[j]	=L[j]*dg0+dg1;
      dg0	-=dg1/ba;
      G11[j]	*=dg0;
      G12[j]	*=dg0;
      G22[j]	*=dg0;
      dg0	=rr0/ba;
      G11[j]	+=2.*(dra[j]*raa+dza[j]*zaa)*dg0;
      G12[j]	+=(raa*drgt[j]+dra[j]*ragt+zaa*dzgt[j])*dg0;
      G22[j]	+=2.*ragt*drgt[j]*dg0;
    }
    D[j]	=D[0];
    L[j]	=L[0];
    G11[j]	=G11[0];
    G12[j]	=G12[0];
    G22[j]	=G22[0];
    im		=im0+i;
    ESgP2gF(ESDPc2+im,ESDPs2+im,D,ES2Mp);
    ESgP2gF(ESLc2+im,ESLs2+im,L,ES2Mp);
    m1		=im;
    for(m=0; m < ES2Mp1; m++){
      ESVc[m1]	=0.;
      ESVs[m1]	=0.;
      m1	+=ESNa1;
    }
    ESgP2gF(ESg11c2+im,ESg11s2+im,G11,ES2Mp);
    ESgP2gF(ESg12c2+im,ESg12s2+im,G12,ES2Mp);
    ESgP2gF(ESg22c2+im,ESg22s2+im,G22,ES2Mp);
#ifdef H
    ki		=K;
    rca[0]	=rcT2a[ki];
    rsa[0]	=rsT2a[ki];
    ki	+=ESNa1;
    rc[1]	=2.*rcT1a[ki];
    rs[1]	=2.*rsT1a[ki];
    rca[1]	=2.*rcT2a[ki];
    rsa[1]	=2.*rsT2a[ki];
    for(k=2; k < ESFp1; k++){
      ki	+=ESNa1;
      rca[k]	=2.*rcT2a[ki];
      rsa[k]	=2.*rsT2a[ki];
    }
    for(j=0; j < ESNp; j++){
      drgt[j]	=-rc[1]*sn1[j]+rs[1]*cs1[j];
      ragt	=-rca[1]*sn1[j]+rsa[1]*cs1[j];
      dra[j]	=rc[1]*cs1[j]+rs[1]*sn1[j];
      raa	=rca[0];
      kj	=0;
      for(k=1; k < ESFp1; k++){
	kj	+=j;
	if(kj >= ESNp)
	  kj	-=ESNp;
	raa	+=rca[k]*cs1[kj]+rsa[k]*sn1[kj];
	ragt	=k*(-rca[k]*sn1[kj]+rsa[k]*cs1[kj]);
      }
      ragt	*=cr2;
      dzgt[j]	=ESsb1a[0]*cs1[j];
      zagt	=cr2*ESsb2a[0]*cs1[j];
      dza[j]	=ESsb1a[0]*sn1[j];
      zaa	=rsa[0]+ESsb2a[0]*sn1[j];
      dg1	=raa*dzgt[j]+dra[j]*zagt-ragt*dza[j]-drgt[j]*zaa;
      D[j]	=dg1;
      dg0	=-dra[j]*rr0;
      L[j]	=L[j]*dg0+dg1*rr0;
      dg0	-=dg1/ba;
      G11[j]	*=dg0;
      G12[j]	*=dg0;
      G22[j]	*=dg0;
      dg0	=rr0/ba;
      G11[j]	+=2.*(dra[j]*raa+dza[j]*zaa)*dg0;
      G12[j]	+=(raa*drgt[j]+dra[j]*ragt+zaa*dzgt[j])*dg0;
      G22[j]	+=2.*(drgt[j]*ragt+dzgt[j]*zagt)*dg0;
    }
    D[j]	=D[0];
    L[j]	=L[0];
    G11[j]	=G11[0];
    G12[j]	=G12[0];
    G22[j]	=G22[0];
    im		=im0+i;
    ESgP2gF(ESDPc2+im,ESDPs2+im,D,ES2Mp);
    ESgP2gF(ESLc2+im,ESLs2+im,L,ES2Mp);
    m1		=im;
    for(m=0; m < ES2Mp1; m++){
      ESVc[m1]	=0.;
      ESVs[m1]	=0.;
      m1	+=ESNa1;
    }
    ESgP2gF(ESg11c2+im,ESg11s2+im,G11,ES2Mp);
    ESgP2gF(ESg12c2+im,ESg12s2+im,G12,ES2Mp);
    ESgP2gF(ESg22c2+im,ESg22s2+im,G22,ES2Mp);
#endif
    in++;
    for(i=1; i < ESNa1; i++){
      rA	=1./ESsa[i];
      b		=ESsb[in];
      ba	=ESsb1a[in];
      ki	=K+i;
      rca[0]	=rcT1a[ki];
      rsa[0]	=rsT1a[ki];
      for(k=1; k < ESFp1; k++){
	ki	+=ESNa1;
	rca[k]	=2.*rcT1a[ki];
	rsa[k]	=2.*rsT1a[ki];
	rc[k]	=2.*k*rcT[ki];
	rs[k]	=2.*k*rsT[ki];
      }
      for(j=0; j < ESNp; j++){
	dra[j]	=rca[0];
	drgt[j]	=0.;
	kj	=0;
	for(k=1; k < ESFp1; k++){
	  kj		+=j;
	  if(kj >= ESNp)
	    kj	-=ESNp;
	  dra[j]	+=rca[k]*cs1[kj]+rsa[k]*sn1[kj];
	  drgt[j]	+=-rc[k]*sn1[kj]+rs[k]*cs1[kj];
	}
	dzgt[j]	=b*cs1[j];
	dza[j]	=rsa[0]+ba*sn1[j];
	D[j]	=dra[j]*dzgt[j]-drgt[j]*dza[j];
	if(D[j] <= 0.){
#ifdef H
	  printf("D[n=%3d][i=%2d][j=%2d]=%10.3e <= 0%c\n",n,i,j,D[j],7);
#endif
	  return(1);
	}
	L[j]	=D[j]*r0/ESsr[jni]*rA;
	V[j]	=D[j]*ESsr[jni]*rr0*rA-L[j];
	dg0	=1./(D[j]*ESsr[jni]);
	D[j]	*=rA;
	G11[j]	=ESsa[i]*(dra[j]*dra[j]+dza[j]*dza[j])*dg0;
	G12[j]	=(dra[j]*drgt[j]+dza[j]*dzgt[j])*dg0;
	G22[j]	=(drgt[j]*drgt[j]+dzgt[j]*dzgt[j])*dg0*rA;
	rT1a[jni]	=dra[j];
	rT1gt[jni]	=drgt[j];
	zT1a[jni]	=dza[j];
	zT1gt[jni]	=dzgt[j];
	jni++;
      }
      D[j]	=D[0];
      L[j]	=L[0];
      V[j]	=V[0];
      G11[j]	=G11[0];
      G12[j]	=G12[0];
      G22[j]	=G22[0];
      rT1a[jni]	=dra[0];
      rT1gt[jni]=drgt[0];
      zT1a[jni]	=dza[0];
      zT1gt[jni]=dzgt[0];
      jni++;
      im	=im0+i;
      ESgP2gF(ESDPc+im,ESDPs+im,D,ES2Mp);
      ESgP2gF(ESLc+im,ESLs+im,L,ES2Mp);
      ESgP2gF(ESVc+im,ESVs+im,V,ES2Mp);
      ESgP2gF(ESg22c+im,ESg22s+im,G22,ES2Mp);
      ESgP2gF(ESg12c+im,ESg12s+im,G12,ES2Mp);
      ESgP2gF(ESg11c+im,ESg11s+im,G11,ES2Mp);
      in++;
    }
    i		=ESNa;
    jni		-=ESNp1;
    in--;
    ki		=K+i;
    rca[0]	=rcT2a[ki];
    rsa[0]	=rsT2a[ki];
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      rca[k]	=2.*rcT2a[ki];
      rsa[k]	=2.*rsT2a[ki];
      b		=2.*k;
      rc[k]	=b*rcT1a[ki];
      rs[k]	=b*rsT1a[ki];
    }
    b		=ESsb1a[in];
    ba		=ESsb2a[in];
    for(j=0; j < ESNp; j++){
      raa	=rca[0];
      ragt	=0.;
      kj	=0;
      for(k=1; k < ESFp1; k++){
	kj	+=j;
	if(kj >= ESNp)
	  kj	-=ESNp;
	raa	+=rca[k]*cs1[kj]+rsa[k]*sn1[kj];
	ragt	+=-rc[k]*sn1[kj]+rs[k]*cs1[kj];
      }
      zaa	=rsa[0]+ba*sn1[j];
      zagt	=b*cs1[j];
      dg0	=-dra[j]/ESsr[jni];
      dg1	=raa*dzgt[j]+dra[j]*zagt-ragt*dza[j]-drgt[j]*zaa;
      L[j]	=(dg0-1.)*L[j]+dg1*r0/ESsr[jni];
      V[j]	=(D[j]*dra[j]+dg1*ESsr[jni]-D[j]*ESsr[jni])*rr0-L[j];
      dg0	-=dg1/D[j];
      G12[j]	*=dg0;
      G22[j]	*=(dg0-1.);
      dg0	+=1.;
      G11[j]	*=dg0;
      dg0	=1./(D[j]*ESsr[jni]);
      G11[j]	+=2.*(dra[j]*raa+dza[j]*zaa)*dg0*ESsa[i];
      G12[j]	+=(raa*drgt[j]+dra[j]*ragt+zaa*dzgt[j]+dza[j]*zagt)*dg0;
      G22[j]	+=2.*(drgt[j]*ragt+dzgt[j]*zagt)*dg0;
      D[j]	=dg1-D[j];
      jni++;
    }
    D[j]	=D[0];
    L[j]	=L[0];
    V[j]	=V[0];
    G11[j]	=G11[0];
    G12[j]	=G12[0];
    G22[j]	=G22[0];
    im		=im0+i;
    ESgP2gF(ESDPc2+im,ESDPs2+im,D,ES2Mp);
    ESgP2gF(ESLc2+im,ESLs2+im,L,ES2Mp);
    ESgP2gF(ESVc2+im,ESVs2+im,V,ES2Mp);
    ESgP2gF(ESg22c2+im,ESg22s2+im,G22,ES2Mp);
    ESgP2gF(ESg12c2+im,ESg12s2+im,G12,ES2Mp);
    ESgP2gF(ESg11c2+im,ESg11s2+im,G11,ES2Mp);
    im	=im0;
    in	=im+ESNa;
    dg0	=ESDPc2[im];
    dg1	=ESDPc2[in];
    splAA(ESDPc+im,ESDPc1+im,ESDPc2+im,&dg0,&dg1);
    dg0	=ESLc2[im];
    dg1	=ESLc2[in];
    splAA(ESLc+im,ESLc1+im,ESLc2+im,&dg0,&dg1);
    dg0	=ESVc2[im];
    dg1	=ESVc2[in];
    splAA(ESVc+im,ESVc1+im,ESVc2+im,&dg0,&dg1);
    dg0	=ESg22c2[im];
    dg1	=ESg22c2[in];
    splAA(ESg22c+im,ESg22c1+im,ESg22c2+im,&dg0,&dg1);
    dg0	=ESg12c2[im];
    dg1	=ESg12c2[in];
    splA(ESg12c+im,ESg12c2+im,&dg0,&dg1);
    dg0	=ESg11c2[im];
    dg1	=ESg11c2[in];
    splA(ESg11c+im,ESg11c2+im,&dg0,&dg1);
    for(m=1; m < 2; m++){
      im 	+=ESNa1;
      in	=im+ESNa;
      dg0	=ESDPc2[im];
      dg1	=ESDPc2[in];
      splAA2(ESDPc+im,ESDPc1+im,ESDPc2+im,ESsa,&dg1);
      dg0	=ESDPs2[im];
      dg1	=ESDPs2[in];
      splAA2(ESDPs+im,ESDPs1+im,ESDPs2+im,ESsa,&dg1);
      dg0	=ESLc2[im];
      dg1	=ESLc2[in];
      splAA2(ESLc+im,ESLc1+im,ESLc2+im,ESsa,&dg1);
      dg0	=ESLs2[im];
      dg1	=ESLs2[in];
      splAA2(ESLs+im,ESLs1+im,ESLs2+im,ESsa,&dg1);
      dg0	=ESVc2[im];
      dg1	=ESVc2[in];
      splAA2(ESVc+im,ESVc1+im,ESVc2+im,ESsa,&dg1);
      dg0	=ESVs2[im];
      dg1	=ESVs2[in];
      splAA2(ESVs+im,ESVs1+im,ESVs2+im,ESsa,&dg1);
      dg0	=ESg22c2[im];
      dg1	=ESg22c2[in];
      splAA2(ESg22c+im,ESg22c1+im,ESg22c2+im,ESsa,&dg1);
      dg0	=ESg22s2[im];
      dg1	=ESg22s2[in];
      splAA2(ESg22s+im,ESg22s1+im,ESg22s2+im,ESsa,&dg1);
      dg0	=ESg12c2[im];
      dg1	=ESg12c2[in];
      splA(ESg12c+im,ESg12c2+im,&dg0,&dg1);
      dg0	=ESg12s2[im];
      dg1	=ESg12s2[in];
      splA(ESg12s+im,ESg12s2+im,&dg0,&dg1);
      dg0	=ESg11c2[im];
      dg1	=ESg11c2[in];
      splA(ESg11c+im,ESg11c2+im,&dg0,&dg1);
      dg0	=ESg11s2[im];
      dg1	=ESg11s2[in];
      splA(ESg11s+im,ESg11s2+im,&dg0,&dg1);
    }
    for(m=2; m < ES2Mp1; m++){
      im 	+=ESNa1;
      in	=im+ESNa;
      dg1	=ESDPc2[in];
      splAA(ESDPc+im,ESDPc1+im,ESDPc2+im,ESsa,&dg1);
      dg1	=ESDPs2[in];
      splAA(ESDPs+im,ESDPs1+im,ESDPs2+im,ESsa,&dg1);
      dg1	=ESLc2[in];
      splAA(ESLc+im,ESLc1+im,ESLc2+im,ESsa,&dg1);
      dg1	=ESLs2[in];
      splAA(ESLs+im,ESLs1+im,ESLs2+im,ESsa,&dg1);
      dg1	=ESVc2[in];
      splAA(ESVc+im,ESVc1+im,ESVc2+im,ESsa,&dg1);
      dg1	=ESVs2[in];
      splAA(ESVs+im,ESVs1+im,ESVs2+im,ESsa,&dg1);
      if(m == 3){
	dg1	=ESg12c2[in];
	splAA2(ESg12c+im,ESg22c1+im,ESg12c2+im,ESsa,&dg1);
	dg1	=ESg12s2[in];
	splAA2(ESg12s+im,ESg22c1+im,ESg12s2+im,ESsa,&dg1);
	dg1	=ESg11c2[in];
	splAA2(ESg11c+im,ESg22c1+im,ESg11c2+im,ESsa,&dg1);
	dg1	=ESg11s2[in];
	splAA2(ESg11s+im,ESg22c1+im,ESg11s2+im,ESsa,&dg1);
	dg1	=ESg22c2[in];
	splAA2(ESg22c+im,ESg22c1+im,ESg22c2+im,ESsa,&dg1);
	dg1	=ESg22s2[in];
	splAA2(ESg22s+im,ESg22s1+im,ESg22s2+im,ESsa,&dg1);
      }
      else{
	dg1	=ESg22c2[in];
	splAA(ESg22c+im,ESg22c1+im,ESg22c2+im,ESsa,&dg1);
	dg1	=ESg22s2[in];
	splAA(ESg22s+im,ESg22s1+im,ESg22s2+im,ESsa,&dg1);
	dg1	=ESg12c2[in];
	splA(ESg12c+im,ESg12c2+im,ESsa,&dg1);
	dg1	=ESg12s2[in];
	splA(ESg12s+im,ESg12s2+im,ESsa,&dg1);
	dg1	=ESg11c2[in];
	splA(ESg11c+im,ESg11c2+im,ESsa,&dg1);
	dg1	=ESg11s2[in];
	splA(ESg11s+im,ESg11s2+im,ESsa,&dg1);
      }
    }
    n1	=n+ESNLt;
    while(n1 < ESNt1){
      im	=im0+ESnMp*n1;
      ji	=im0;
      for(m=0; m < ES2Mp1; m++){
	for(i=0; i < ESNa1; i++){
	  ESDPc[im]	=ESDPc[ji];
	  ESDPs[im]	=ESDPs[ji];
	  ESDPc1[im]	=ESDPc1[ji];
	  ESDPs1[im]	=ESDPs1[ji];
	  ESDPc2[im]	=ESDPc2[ji];
	  ESDPs2[im]	=ESDPs2[ji];

	  ESLc[im]	=ESLc[ji];
	  ESLs[im]	=ESLs[ji];
	  ESLc2[im]	=ESLc2[ji];
	  ESLs2[im]	=ESLs2[ji];

	  ESVc[im]	=ESVc[ji];
	  ESVs[im]	=ESVs[ji];
	  ESVc2[im]	=ESVc2[ji];
	  ESVs2[im]	=ESVs2[ji];

	  ESg22c[im]	=ESg22c[ji];
	  ESg22s[im]	=ESg22s[ji];
	  ESg22c1[im]	=ESg22c1[ji];
	  ESg22s1[im]	=ESg22s1[ji];
	  ESg22c2[im]	=ESg22c2[ji];
	  ESg22s2[im]	=ESg22s2[ji];

	  ESg12c[im]	=ESg12c[ji];
	  ESg12s[im]	=ESg12s[ji];
	  ESg12c2[im]	=ESg12c2[ji];
	  ESg12s2[im]	=ESg12s2[ji];

	  ESg11c[im]	=ESg11c[ji];
	  ESg11s[im]	=ESg11s[ji];
	  ESg11c2[im]	=ESg11c2[ji];
	  ESg11s2[im]	=ESg11s2[ji];
	  im++;
	  ji++;
	}
      }
      n1	+=ESNLt;
    }
  }
  return(0);
}

int ES3DInvG22()
{
  int i,j,ji,n,n1,im,im1,im0,m,m1,mm;
  
  double *p22c,*p22s,*pr22,*pr222;
  double *ac,*f,s;
  int *indx;

  ESReInitPlVeq(ES2Mp1,&ac,&f,&indx);
  
  return(0);

  im0	=ESNa1*ES2Mp1;
  for(n=0; n < ESNLt; n++){
    im		=im0*n;
    p22c	=ESg22c+im;
    p22s	=ESg22s+im;
    im		*=ES2Mp1;
    pr22	=ESr22+im;
    pr222	=ESr222+im;
    i		=0;
    j		=0;
    im		=i;
    ji		=0;
    ac[ji]	=p22c[im];
    ji++;
    im		+=ESNa1;
    for(mm=0; mm < ESMp; mm++){
      ac[ji]	=2.*p22c[im];
      ji++;
      ac[ji]	=-2.*p22s[im];
      ji++;
      im	+=ESNa1;
    }
    j++;
    for(m=0; m < ESMp; m++){
      ji	=ES2Mp1*j+j;
      m1	=2*m+2;
      im	=i+ESNa1*m1;
      im1	=i;
      for(mm=m; mm < ESMp; mm++){
	if(mm == m){
	  ac[ji]	=2.*(p22c[im]+ac[0]);
	  ji++;
	  ac[ji]	=-2.*p22s[im];
	}
	else{
	  ac[ji]	=2.*(p22c[im]+p22c[im1]);
	  ji++;
	  ac[ji]	=-2.*(p22s[im]+p22s[im1]);
	}
	ji++;
	im1	+=ESNa1;
	im	+=ESNa1;
      }
      j++;
      ji	=ES2Mp1*j+j;
      im	=i+ESNa1*m1;
      im1	=i;
      mm	=m;
      ac[ji]	=2.*(ac[0]-p22c[im]);
      ji++;
      im1	+=ESNa1;
      im	+=ESNa1;
      mm++;
      while(mm < ESMp){
	ac[ji]	=2.*(p22s[im1]-p22s[im]);
	ji++;
	ac[ji]	=2.*(p22c[im1]-p22c[im]);
	ji++;
	im1	+=ESNa1;
	im	+=ESNa1;
	mm++;
      }
      j++;
    }
    if(CholDc(ac,ES2Mp1) < 0){
#ifdef H
      printf("Nonpositive G22 Matrix at n=%3d i=%2d\n",n,i);
      printf("??? ------ ac[%2d] ------\n",i);
      for(m=0; m < ES2Mp1; m++){
	ji	=ES2Mp1*m+m;
	for(mm=0; mm < m; mm++){
	  printf("%10.3s","");
	}
	for(mm=m; mm < ES2Mp1; mm++){
	  printf("%10.3e",ac[ji]);
	  ji++;
	}
	printf("\n");
      }
#endif
    }
    ji	=i;
    for(m=0; m < ES2Mp1; m++){
      for(mm=0; mm <= m ; mm++){
	pr22[ji]	=ac[m*ES2Mp1+mm];
	ji	+=ESNa1;
      }
    }
    p22c	=ESg22c+im0*n;
    p22s	=ESg22s+im0*n;
    for(i=1; i < ESNa1; i++){
      j		=0;
      im	=i;
      ji	=0;
      ac[ji]	=p22c[im];
      ji++;
      im	+=ESNa1;
      for(mm=0; mm < ESMp; mm++){
	ac[ji]	=2.*p22c[im];
	ji++;
	ac[ji]	=-2.*p22s[im];
	ji++;
	im	+=ESNa1;
      }
      j++;
      for(m=0; m < ESMp; m++){
	ji	=ES2Mp1*j+j;
	m1	=2*m+2;
	im	=i+ESNa1*m1;
	im1	=i;
	for(mm=m; mm < ESMp; mm++){
	  if(mm == m){
	    ac[ji]	=2.*(p22c[im]+ac[0]);
	    ji++;
	    ac[ji]	=-2.*p22s[im];
	  }
	  else{
	    ac[ji]	=2.*(p22c[im]+p22c[im1]);
	    ji++;
	    ac[ji]	=-2.*(p22s[im]+p22s[im1]);
	  }
	  ji++;
	  im1	+=ESNa1;
	  im	+=ESNa1;
	}
	j++;
	ji	=ES2Mp1*j+j;
	im	=i+ESNa1*m1;
	im1	=i;
	mm	=m;
	ac[ji]	=2.*(ac[0]-p22c[im]);
	ji++;
	im1	+=ESNa1;
	im	+=ESNa1;
	mm++;
	while(mm < ESMp){
	  ac[ji]	=2.*(p22s[im1]-p22s[im]);
	  ji++;
	  ac[ji]	=2.*(p22c[im1]-p22c[im]);
	  ji++;
	  im1	+=ESNa1;
	  im	+=ESNa1;
	  mm++;
	}
	j++;
      }
      if(CholDc(ac,ES2Mp1) < 0){
#ifdef H
	printf("Nonpositive G22 Matrix at n=%3d i=%2d\n",n,i);
	printf("??? ------ ac[%2d] ------\n",i);
	for(m=0; m < ES2Mp1; m++){
	  ji	=ES2Mp1*m+m;
	  for(mm=0; mm < m; mm++){
	    printf("%10.3s","");
	  }
	  for(mm=m; mm < ES2Mp1; mm++){
	    printf("%10.3e",ac[ji]);
	    ji++;
	  }
	  printf("\n");
	}
#endif
      }
      ji	=i;
      for(m=0; m < ES2Mp1; m++){
	for(mm=0; mm <= m ; mm++){
	  pr22[ji]	=ac[m*ES2Mp1+mm];
	  ji	+=ESNa1;
	}
      }
    }
    ji	=0;
    for(m=0; m < ES2Mp1; m++){
#ifdef J
      printf("??? m=%2d\n",m);
#endif
      for(mm=0; mm <= m ; mm++){
#ifdef J
	printf("???    mm=%2d\n",mm);
	for(i=0; i< ESNa1; i++){
	  printf("??? pr22[%2d]=%10.3e\n",i,*(pr22+ji+i));
	}
#endif
	splA(pr22+ji,pr222+ji,NULL,NULL);
	ji	+=ESNa1;
      }
    }
    n1	=n+ESNLt;
    while(n1 < ESNt1){
      im1	=im0*ES2Mp1*n1;
      ji	=0;
      for(m=0; m < ES2Mp1; m++){
	for(mm=0; mm <= m; mm++){
	  for(i=0; i < ESNa1; i++){
	    ESr22[im1]	=pr22[ji];
	    ESr222[im1]	=pr222[ji];
	    im1++;
	    ji++;
	  }
	}
      }
      n1	+=ESNLt;
    }
  }
#ifdef h
  {
    int kW,k,ki;
    double x[2],y[2],Wx[300],W1[300][200];
    double *p,*p1,*p2,s,t,s1;

    p		=ESr22;
    p1		=rcT1a;
    p2		=ESr222;
    y[0]	=p[0];
    y[1]	=y[0];
    x[0]	=0.;
    x[1]	=1.;
    ki		=0;
    s		=0.125*ESsa[1];
    
    ki	=0;
    k	=0;
    for(m=0; m < ES2Mp1; m++){
      for(mm=0; mm <= m; mm++){
	kW	=0;
	t	=0.;
	while(t <= x[1]+s*0.5){
	  ESSetSplA(t);
	  Wx[kW]	=t;
	  splRA(&s1,NULL,p+ki,p2+ki);
	  if(y[0] > s1)
	    y[0]	=s1;
	  if(y[1] < s1)
	    y[1]	=s1;
	  W1[k][kW]	=s1;
	  t		+=s;
	  kW++;
	}
	k++;
	ki	+=ESNa1;
      }
    }
    Scale2D(4,2,x,y,2,6,2);
    ki	=0;
    k	=0;
    for(m=0; m < ES2Mp1; m++){
      for(mm=0; mm <= m; mm++){
	Plot2d(4,ESsa,p+ki,ESNa1,6,0,4,0);
	Plot2d(4,Wx,W1[k],kW,6,0,0,0);
	k++;
	ki	+=ESNa1;
      }
    }
    CbFlush();
  }
#endif
  return(0);
}    

int ESInitgYbY(double *gyr,double *gyi,double *Yr,double *Yi,double A, int n)
{
  int i,j,m,M,M0,mm,in,k,k1,k2,kk;
  int K,ki,kj;
  double b,ba,S,sr,si,r1c,r1s;

  double *fr,*fi,*dfr,*dfi,*fmr,*fmi,*dfmr,*dfmi;
  double Fc[128],Fs[128],dFc[128],dFs[128],g22c[128],g22s[128],
  g12c[128],g12s[128];

  double gnr,gni,gnn,gnmr,gnmi,gnnm;
  double s,zr,zi,zmr,zmi;
  double f1r,f1i,f2r,f2i;

  if(ES2Mp1 > 128){
    printf("2*Mp+1=%d > 128\n",ES2Mp1);
    ESexit(0);
  }

  K	=ESnAF*n;
  ESSetSplA(A);
  fr	=ESd2Faux+ESNa1;
  fi	=fr	+ESNp1;
  dfr	=fi	+ESNp1;
  dfi	=dfr	+ESNp1;
  fmr	=dfi	+ESNp1;
  fmi	=fmr	+ESNp1;
  dfmr	=fmi	+ESNp1;
  dfmi	=dfmr	+ESNp1;

  in	=ESNa1*n;
  ki	=K+ESNa1;
  zr	=rcT[ki]+cr2*ESsb1a[in];
  zi	=rsT[ki];

  s	=1./(zr*zr+zi*zi);
  zr	*=s;
  zi	*=s;

  b	=ESsb1a[in];
  sr	=2.*rcT[ki];
  s	=sr+b;
  si	=2.*rsT[ki];
  s	=1./(s*s+si*si);
  gnr	=(sr*sr-si*si-b*b)*s;
  gni	=2.*sr*si*s;
  gnn	=gnr*gnr+gni*gni;
  gnmr	=gnr;
  gnmi	=gni;
  gnnm	=gnn;
  zmr	=zr;
  zmi	=zi;
  splRA(&b,&ba,ESsb+in,ESsb2a+in);
  ba	*=A;
  ki	=K;
  for(k=0; k < ESFp1; k++){
    splRA(Fc+k,dFc+k,rcT+ki,rcT2a+ki);
    splRA(Fs+k,dFs+k,rsT+ki,rsT2a+ki);
    ki	+=ESNa1;
  }
  dFc[0]	*=A;
  dFs[0]	*=A;
  S		=2.*A;
  for(k=1; k < ESFp1; k++){
    Fc[k]	*=2.;
    Fs[k]	*=2.;
    dFc[k]	*=S;
    dFs[k]	*=S;
  }
  for(j=0; j < ESNp; j++){
    fr[j]	=Fc[0];
    dfr[j]	=dFc[0];
    kj	=j;
    for(k=1; k < ESFp; k++){
      fr[j]	+=Fc[k]*cs1[kj]+Fs[k]*sn1[kj];
      dfr[j]	+=dFc[k]*cs1[kj]+dFs[k]*sn1[kj];
      kj		+=j;
      if(kj >= ESNp)
	kj	-=ESNp;
    }
    fi[j]	=Fs[0]+b*sn1[j];
    dfi[j]	=dFs[0]+ba*sn1[j];
    fmr[j]	=fr[j];
    fmi[j]	=fi[j];
    dfmr[j]	=dfr[j];
    dfmi[j]	=dfi[j];
  }
  fr[ESNp]	=fr[0];
  fi[ESNp]	=fi[0];
  dfr[ESNp]	=dfr[0];
  dfi[ESNp]	=dfi[0];
  fmr[ESNp]	=fmr[0];
  fmi[ESNp]	=fmi[0];
  dfmr[ESNp]	=dfmr[0];
  dfmi[ESNp]	=dfmi[0];
  M	=0;
  k	=0;
  for(kk	=0; kk < ES2Mp1; kk++){
    gyr[k]	=0.;
    gyi[k]	=0.;
    Yr[k]	=0.;
    Yi[k]	=0.;
    k++;
  }
  M++;
  ESP2F(Fc,Fs,fmr,ESMp);
  ESP2F(dFc,dFs,dfmr,ESMp);
  k	=ES2Mp1*M+ESMp;
  k2	=k;
  k1	=k;
  mm	=0;
  gyr[k]	=Fc[mm];
  gyi[k]	=Fs[mm];
  Yr[k]		=dFc[mm];
  Yi[k]		=dFs[mm];
  while(mm < ESMp){
    k2++;
    k1--;
    mm++;
    gyr[k1]	=Fc[mm];
    gyi[k1]	=Fs[mm];
    Yr[k1]	=dFc[mm];
    Yi[k1]	=dFs[mm];
    gyr[k2]	=Fc[mm];
    gyi[k2]	=-Fs[mm];
    Yr[k2]	=dFc[mm];
    Yi[k2]	=-dFs[mm];
  }
  ESP2F(Fc,Fs,fmi,ESMp);
  ESP2F(dFc,dFs,dfmi,ESMp);
  k	=ES2Mp1*M+ESMp;
  k2	=k;
  k1	=k;
  mm	=0;
  gyr[k]	-=Fs[mm];
  gyi[k]	+=Fc[mm];
  Yr[k]		-=dFs[mm];
  Yi[k]		+=dFc[mm];
  while(mm < ESMp){
    k2++;
    k1--;
    mm++;
    gyr[k1]	-=Fs[mm];
    gyi[k1]	+=Fc[mm];
    Yr[k1]	-=dFs[mm];
    Yi[k1]	+=dFc[mm];
    gyr[k2]	+=Fs[mm];
    gyi[k2]	+=Fc[mm];
    Yr[k2]	+=dFs[mm];
    Yi[k2]	+=dFc[mm];
  }
  s		=1./(1.-gnnm);
  f1r		=zmr*s;
  f1i		=zmi*s;
  f2r		=(gnmr*zmr+gnmi*zmi)*s;
  f2i		=(gnmi*zmr-gnmr*zmi)*s;
  sr		=gnmr;
  si		=gnmi;
  gnmr		=sr*gnr-si*gni;
  gnmi		=si*gnr+sr*gni;
  gnnm		*=gnn;
  sr		=zmr;
  si		=zmi;
  zmr		=sr*zr-si*zi;
  zmi		=si*zr+sr*zi;
  k		=ES2Mp1*M+ESMp;
  sr		=gyr[k];
  si		=gyi[k];
  gyr[k]	=(f1r-f2r)*sr-(f1i+f2i)*si;
  gyi[k]	=(f1i-f2i)*sr+(f1r+f2r)*si;
  sr		=Yr[k];
  si		=Yi[k];
  Yr[k]		=(f1r-f2r)*sr-(f1i+f2i)*si;
  Yi[k]		=(f1i-f2i)*sr+(f1r+f2r)*si;
  for(mm=1; mm < ESMp1; mm++){
    k2		=k+mm;
    k1		=k-mm;
    sr		=gyr[k2];
    si		=gyi[k2];
    r1c		=gyr[k1];
    r1s		=gyi[k1];
    gyr[k2]	=f1r*sr-f1i*si-f2r*r1c-f2i*r1s;
    gyi[k2]	=f1i*sr+f1r*si-f2i*r1c+f2r*r1s;
    gyr[k1]	=f1r*r1c-f1i*r1s-f2r*sr-f2i*si;
    gyi[k1]	=f1i*r1c+f1r*r1s-f2i*sr+f2r*si;
    sr		=Yr[k2];
    si		=Yi[k2];
    r1c		=Yr[k1];
    r1s		=Yi[k1];
    Yr[k2]	=f1r*sr-f1i*si-f2r*r1c-f2i*r1s;
    Yi[k2]	=f1i*sr+f1r*si-f2i*r1c+f2r*r1s;
    Yr[k1]	=f1r*r1c-f1i*r1s-f2r*sr-f2i*si;
    Yi[k1]	=f1i*r1c+f1r*r1s-f2i*sr+f2r*si;
  }	
  for(M=2; M < ESMp1; M++){
    for(j=0; j < ESNp1; j++){
      sr	=fmr[j];
      si	=fmi[j];
      dfmr[j]	=M*(dfr[j]*sr-dfi[j]*si);
      dfmi[j]	=M*(dfr[j]*si+dfi[j]*sr);
      fmr[j]	=fr[j]*sr-fi[j]*si;
      fmi[j]	=fr[j]*si+fi[j]*sr;
    }
    ESP2F(Fc,Fs,fmr,ESMp);
    ESP2F(dFc,dFs,dfmr,ESMp);
    k		=ES2Mp1*M+ESMp;
    k2		=k;
    k1		=k;
    mm		=0;
    gyr[k]	=Fc[mm];
    gyi[k]	=0.;
    Yr[k]	=dFc[mm];
    Yi[k]	=0.;
    while(mm < ESMp){
      k2++;
      k1--;
      mm++;
      gyr[k1]	=Fc[mm];
      gyi[k1]	=Fs[mm];
      Yr[k1]	=dFc[mm];
      Yi[k1]	=dFs[mm];
      gyr[k2]	=Fc[mm];
      gyi[k2]	=-Fs[mm];
      Yr[k2]	=dFc[mm];
      Yi[k2]	=-dFs[mm];
    }
    ESP2F(Fc,Fs,fmi,ESMp);
    ESP2F(dFc,dFs,dfmi,ESMp);
    k		=ES2Mp1*M+ESMp;
    k2		=k;
    k1		=k;
    mm		=0;
    gyi[k]	+=Fc[mm];
    Yi[k]	+=dFc[mm];
    while(mm < ESMp){
      k2++;
      k1--;
      mm++;
      gyr[k1]	-=Fs[mm];
      gyi[k1]	+=Fc[mm];
      Yr[k1]	-=dFs[mm];
      Yi[k1]	+=dFc[mm];
      gyr[k2]	+=Fs[mm];
      gyi[k2]	+=Fc[mm];
      Yr[k2]	+=dFs[mm];
      Yi[k2]	+=dFc[mm];
    }
    s		=1./(1.-gnnm);
    f1r		=zmr*s;
    f1i		=zmi*s;
    f2r		=(gnmr*zmr+gnmi*zmi)*s;
    f2i		=(gnmi*zmr-gnmr*zmi)*s;
    sr		=gnmr;
    si		=gnmi;
    gnmr	=sr*gnr-si*gni;
    gnmi	=si*gnr+sr*gni;
    gnnm	*=gnn;
    sr		=zmr;
    si		=zmi;
    zmr		=sr*zr-si*zi;
    zmi		=si*zr+sr*zi;
    k		=ES2Mp1*M+ESMp;
    sr		=gyr[k];
    si		=gyi[k];
    gyr[k]	=(f1r-f2r)*sr-(f1i+f2i)*si;
    gyi[k]	=(f1i-f2i)*sr+(f1r+f2r)*si;
    sr		=Yr[k];
    si		=Yi[k];
    Yr[k]	=(f1r-f2r)*sr-(f1i+f2i)*si;
    Yi[k]	=(f1i-f2i)*sr+(f1r+f2r)*si;
    for(mm=1; mm < ESMp1; mm++){
      k2	=k+mm;
      k1	=k-mm;
      sr	=gyr[k2];
      si	=gyi[k2];
      r1c	=gyr[k1];
      r1s	=gyi[k1];
      gyr[k2]	=f1r*sr-f1i*si-f2r*r1c-f2i*r1s;
      gyi[k2]	=f1i*sr+f1r*si-f2i*r1c+f2r*r1s;
      gyr[k1]	=f1r*r1c-f1i*r1s-f2r*sr-f2i*si;
      gyi[k1]	=f1i*r1c+f1r*r1s-f2i*sr+f2r*si;
      sr	=Yr[k2];
      si	=Yi[k2];
      r1c	=Yr[k1];
      r1s	=Yi[k1];
      Yr[k2]	=f1r*sr-f1i*si-f2r*r1c-f2i*r1s;
      Yi[k2]	=f1i*sr+f1r*si-f2i*r1c+f2r*r1s;
      Yr[k1]	=f1r*r1c-f1i*r1s-f2r*sr-f2i*si;
      Yi[k1]	=f1i*r1c+f1r*r1s-f2i*sr+f2r*si;
    }	
  }
  k	=ESnMp*n;
  splRA(g22c,NULL,ESg22c+k,ESg22c2+k);
  splRA(g12c,NULL,ESg12c+k,ESg12c2+k);
  S		=1./A;
  g22c[0]	*=S;
  g12c[0]	*=S;
  g22s[0]	=0.;
  g12s[0]	=0.;
  Fc[0]		=0.;
  Fs[0]		=0.;
  k	 	+=ESNa1;
  for(m=1; m < ES2Mp1; m++){
    Fc[m]	=0.;
    Fs[m]	=0.;
    splRA(g22c+m,NULL,ESg22c+k,ESg22c2+k);
    splRA(g22s+m,NULL,ESg22s+k,ESg22s2+k);
    g22c[m]	*=S;
    g22s[m]	*=S;
    splRA(g12c+m,NULL,ESg12c+k,ESg12c2+k);
    splRA(g12s+m,NULL,ESg12s+k,ESg12s2+k);
    g12c[m]	*=S;
    g12s[m]	*=S;
    k 	+=ESNa1;
  }

  for(M=1; M < ESMp1; M++){
    M0		=ES2Mp1*M;
    for(k=0; k < ES2Mp1; k++){
      mm	=M0+k;
      m		=k-ESMp;
      si	=g12c[0]*m;
      sr	=g22c[0];
      Fc[k]	=sr*Yr[mm]+si*gyi[mm];
      Fs[k]	=sr*Yi[mm]-si*gyr[mm];
      for(kk=1, k1=k-kk; k1 >= 0; kk++,k1--){
	mm	=M0+k1;
	m	=k1-ESMp;
	sr	=m*g12c[kk];
	si	=m*g12s[kk];
	Fc[k]	+=g22c[kk]*Yr[mm]+g22s[kk]*Yi[mm]+sr*gyi[mm]-si*gyr[mm];
	Fs[k]	+=g22c[kk]*Yi[mm]-g22s[kk]*Yr[mm]-sr*gyr[mm]-si*gyi[mm];
      }
      for(kk=1, k1=k+kk; k1 < ES2Mp1; kk++, k1++){
	mm	=M0+k1;
	m	=k1-ESMp;
	sr	=m*g12c[kk];
	si	=m*g12s[kk];
	Fc[k]	+=g22c[kk]*Yr[mm]-g22s[kk]*Yi[mm]+sr*gyi[mm]+si*gyr[mm];
	Fs[k]	+=g22c[kk]*Yi[mm]+g22s[kk]*Yr[mm]-sr*gyr[mm]+si*gyi[mm];
      }
    }
    for(k=0; k < ES2Mp1; k++){
      mm	=M0+k;
      Yr[mm]	=Fc[k];
      Yi[mm]	=Fs[k];
    }
  }
  return(0);
}

int ES2DMapFlx2Lab(double *r, double *z
		   ,double *dra, double *dza
		   ,double *drp, double *dzp
		   ,double a, double gt)
{

  double cs,sn;
  double rc,rca,rs,rsa;
  int k,ki;

  cs	=cos(gt);
  sn	=sin(gt);
  ESSetSplA(a);
  splRA(&rc,&rca,ESsb,ESsb2a);
  splRA(&rs,&rsa,rsT,rsT2a);
  *z	=ESaZ0[0]+rs+rc*sn;
  *dza	=rsa+rca*sn;
  *dzp	=rc*cs;

  splRA(&rc,&rca,rcT,rcT2a);
  *r	=ESaR0[0]+rc;
  *dra	=rca;
  *drp	=0.;
  ki	=0;
  for(k=1; k < ESFp1; k++){
    ki	+=ESNa1;
    splRA(&rc,&rca,rcT+ki,rcT2a+ki);
    splRA(&rs,&rsa,rsT+ki,rsT2a+ki);
    cs	=cos(k*gt);
    sn	=sin(k*gt);
    *r	+=2.*(rc*cs+rs*sn);
    *dra+=2.*(rca*cs+rsa*sn);
    *drp+=2.*k*(-rc*sn+rs*cs);
  }
  return(0);
}

int ES2DMapLab2Flx0(double *A,double *t,double R, double Z)
{
  int Fl;
  double D;
  double cs,sn;
  double rc,rca,rc2,rs,rsa,rs2;
  int k,ki,i;
  double r,z,dra,dr2,dza,dz2,drp,dzp,a,gt;

  Fl	=0;
  a	=*A;
  gt	=*t;
  i	=0;
  do{
    ESSetSplA(a);
    cs	=cos(gt);
    sn	=sin(gt);
    splRA2(&rc,&rca,&rc2,ESsb,ESsb2a);
    splRA2(&rs,&rsa,&rs2,rsT,rsT2a);
    z	=ESaZ0[0]+rs+rc*sn-Z;
    dza	=rsa+rca*sn;
    dz2	=rs2+rc2*sn;
    dzp	=rc*cs;
    
    splRA2(&rc,&rca,&rc2,rcT,rcT2a);
    r	=ESaR0[0]+rc-R;
    dra	=rca;
    dr2	=rc2;
    drp	=0.;
    ki	=0;
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      splRA2(&rc,&rca,&rc2,rcT+ki,rcT2a+ki);
      splRA2(&rs,&rsa,&rs2,rsT+ki,rsT2a+ki);
      cs	=cos(k*gt);
      sn	=sin(k*gt);
      r		+=2.*(rc*cs+rs*sn);
      dra	+=2.*(rca*cs+rsa*sn);
      dr2	+=2.*(rc2*cs+rs2*sn);
      drp	+=2.*k*(-rc*sn+rs*cs);
    }
    D	=dra*dzp-drp*dza;
    drp	=2.*(z*drp-r*dzp);
    drp	/=sqrt(D*D+(dr2*dzp-drp*dz2)*drp)+D;
    a	+=drp;
    dra	=(r*dza-z*dra-cr2*(dra*dz2-dr2*dza)*drp*drp)/D;
    gt	+=dra;
    if(a < 0.){
      Fl	=1;
      a-=drp;
    }
    if(a > 1.){
      return(-1);
    }
    i++;
  }while(i < 10 && fabs(dra)+fabs(drp) > 0.001);
  if(1 || i< 10){
    *A	=a;
    *t	=gt;
    return(Fl);
  }
  return(1);
}

int ESSetMapBox()
{
  int j,n,k;

  j	=ESNa*ESNp1;
  n	=j+ESNp1;
  Zmx	=ESaZ0[0]+rsT[ESNa]+ESsb[ESNa];
  Zmn	=ESaZ0[0]+rsT[ESNa]-ESsb[ESNa];
  Rmn	=ESsr[j];
  Rmx	=ESsr[j];
  gtrmx	=0.;
  gtrmn	=0.;
  k	=1;
  for(j++; j < n; j++){
    if(Rmn > ESsr[j]){
      Rmn	=ESsr[j];
      Zrmn	=ESsz[j];
      gtrmn	=ESgt[k];
    }
    if(Rmx < ESsr[j]){
      Rmx	=ESsr[j];
      Zrmx	=ESsz[j];
      gtrmx	=ESgt[k];
    }
    k++;
  }
  return(0);
}

int ES1DMapZ2gt(double *gt,double *R1,double *R2, double z)
{
  double t,cs,sn,s;
  int k,ki;

  ki	=ESNa;
  t	=(z-ESaZ0[0]-rsT[ki])/ESsb[ESNa];
  if(t > 1. || t < -1.){
    return(1);
  }
  *gt	=asin(t);

  *R1	=ESaR0[0]+rcT[ki];
  *R2	=*R1;
  s	=-1;
  for(k=1; k < ESFp1; k++){
    t	=*gt*k;
    cs	=cos(t);
    sn	=sin(t);
    ki	+=ESNa1;
    *R1	+=2.*(rcT[ki]*cs+rsT[ki]*sn);
    *R2	+=2.*(rcT[ki]*cs-rsT[ki]*sn)*s;
    s	=-s;
  }
  return(0);
}

int ES1DMapR2gt(double *t1,double *t2,double *Z1,double *Z2, double R)
{
  double cs,sn,r,drp,gt;
  int i,k,ki;

  if(R < Rmn || R > Rmx){
    return(1);
  }
  if(R == Rmn){
    *Z1	=Zrmn;
    *Z2	=Zrmn;
    return(0);
  }
  if(R == Rmx){
    *Z1	=Zrmx;
    *Z2	=Zrmx;
    return(0);
  }

  gt	=*t1;
  i	=0;
  do{
    ki	=ESNa;
    r	=ESaR0[0]+rcT[ki]-R;
    drp	=0.;
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      cs	=cos(k*gt);
      sn	=sin(k*gt);
      r		+=2.*(rcT[ki]*cs+rsT[ki]*sn);
      drp	+=2.*k*(-rcT[ki]*sn+rsT[ki]*cs);
    }
    drp	=-r/drp;
    gt	+=drp;
    i++;
  }while(i < 10 && fabs(drp) > 0.001);
  if(i	< 10){
    *t1	=gt;
    *Z1	=ESaZ0[0]+rsT[ESNa]+ESsb[ESNa]*sin(gt);
  }
  gt	=*t2;
  i	=0;
  do{
    ki	=ESNa;
    r	=ESaR0[0]+rcT[ki]-R;
    drp	=0.;
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      cs	=cos(k*gt);
      sn	=sin(k*gt);
      r		+=2.*(rcT[ki]*cs+rsT[ki]*sn);
      drp	+=2.*k*(-rcT[ki]*sn+rsT[ki]*cs);
    }
    drp	=-r/drp;
    gt	+=drp;
    i++;
  }while(i < 10 && fabs(drp) > 0.001);
  if(i	< 10){
    *t2	=gt;
    *Z2	=ESaZ0[0]+rsT[ESNa]+ESsb[ESNa]*sin(gt);
    return(0);
  }
  return(1);
}

int ES2DMapLab2Flx(double *A,double *t,double R, double Z)
{
  int Fl;
  double D,Rt;
  double cs,sn;
  double rc,rca,rc2,rs,rsa,rs2;
  int k,ki,i;
  double r,z,dra,dr2,dza,dz2,drp,dzp,a,gt;

  R	-=ESaR0[0];
  Z	-=ESaZ0[0];
  Fl	=0;
  a	=*A;
  gt	=*t;
  
  ESSetSplA(a);
  r	=0.;
  Rt	=0.;
  ki	=0;
  dra	=Z < 0. ? -1. : 1.;
  for(k=1; k < ESFp1; k++){
    ki	+=ESNa1;
    splRA(&rc,NULL,rcT+ki,rcT2a+ki);
    splRA(&rs,NULL,rsT+ki,rsT2a+ki);
    cs	=cos(k*gt);
    sn	=sin(k*gt);
    r	+=(rc*cs+rs*sn);
    switch(k%4){
    case 0:
      Rt	+=rc;
      break;
    case 1:
      Rt	+=dra*rs;
      break;
    case 2:
      Rt	-=rc;
      break;
    case 3:
      Rt	-=dra*rs;
      break;
    }
  }
  splRA(&rc,NULL,rcT,rcT2a);
  splRA(&rs,NULL,rcT,rcT2a);
  Rt	=rc+2.*Rt;
  r	=rc+2.*r;
  if((R-Rt)*(r-Rt) < 0.){
    gt	=cgp-gt;
  }
  i	=0;
  do{
    ESSetSplA(a);
    cs	=cos(gt);
    sn	=sin(gt);
    splRA2(&rc,&rca,&rc2,ESsb,ESsb2a);
    splRA2(&rs,&rsa,&rs2,rsT,rsT2a);
    z	=rs+rc*sn-Z;
    dza	=rsa+rca*sn;
    dz2	=rs2+rc2*sn;
    dzp	=rc*cs;
    
    r	=0.;
    dra	=0.;
    dr2	=0.;
    drp	=0.;
    ki	=0;
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      splRA2(&rc,&rca,&rc2,rcT+ki,rcT2a+ki);
      splRA2(&rs,&rsa,&rs2,rsT+ki,rsT2a+ki);
      cs	=cos(k*gt);
      sn	=sin(k*gt);
      r		+=(rc*cs+rs*sn);
      dra	+=(rca*cs+rsa*sn);
      dr2	+=(rc2*cs+rs2*sn);
      drp	+=k*(-rc*sn+rs*cs);
    }
    splRA2(&rc,&rca,&rc2,rcT,rcT2a);
    r	=rc+2.*r-R;
    dra	=rca+2.*dra;
    dr2	=rc2+2.*dr2;
    drp	*=2.;
    D	=dra*dzp-drp*dza;
    drp	=2.*(z*drp-r*dzp);
    drp	/=sqrt(D*D+(dr2*dzp-drp*dz2)*drp)+D;
    a	+=drp;
    dra	=(r*dza-z*dra-cr2*(dra*dz2-dr2*dza)*drp*drp)/D;
    gt	+=dra;
    if(a < 0.){
      Fl	=2;
      a-=drp;
    }
    if(a > 1.){
      a	=1.;
#ifdef H
      return(-1);
#endif
    }
    i++;
  }while(i < 10 && fabs(dra)+fabs(drp) > 0.001);
  if(i < 10){
    while(gt > c2gp){
      gt	-=c2gp;
    }
    while(gt < 0.){
      gt	+=c2gp;
    }
    *A	=a;
    *t	=gt;
    return(Fl);
  }
  return(1);
}

int ESGetLabMetrics(double *dRa,double *dRp,double *dZa,double *dZp
		    ,double *A,double *t,double R, double Z)
{
  int Fl;
  double D,Rt;
  double cs,sn;
  double rc,rca,rc2,rs,rsa,rs2;
  int k,ki,i;
  double r,z,dra,dr2,dza,dz2,drp,dzp,a,gt,da,dt;
  
  R	-=ESaR0[0];
  Z	-=ESaZ0[0];
  Fl	=0;
  a	=*A;
  gt	=*t;
  
  ESSetSplA(a);
  r	=0.;
  Rt	=0.;
  ki	=0;
  dra	=Z < 0. ? -1. : 1.;
  for(k=1; k < ESFp1; k++){
    ki	+=ESNa1;
    splRA(&rc,NULL,rcT+ki,rcT2a+ki);
    splRA(&rs,NULL,rsT+ki,rsT2a+ki);
    cs	=cos(k*gt);
    sn	=sin(k*gt);
    r	+=(rc*cs+rs*sn);
    switch(k%4){
    case 0:
      Rt	+=rc;
      break;
    case 1:
      Rt	+=dra*rs;
      break;
    case 2:
      Rt	-=rc;
      break;
    case 3:
      Rt	-=dra*rs;
      break;
    }
  }
  splRA(&rc,NULL,rcT,rcT2a);
  splRA(&rs,NULL,rcT,rcT2a);
  Rt	=rc+2.*Rt;
  r	=rc+2.*r;
  if((R-Rt)*(r-Rt) < 0.){
    gt	=cgp-gt;
  }
  i	=0;
  do{
    ESSetSplA(a);
    cs	=cos(gt);
    sn	=sin(gt);
    splRA2(&rc,&rca,&rc2,ESsb,ESsb2a);
    splRA2(&rs,&rsa,&rs2,rsT,rsT2a);
    z	=rs+rc*sn-Z;
    dza	=rsa+rca*sn;
    dz2	=rs2+rc2*sn;
    dzp	=rc*cs;
    
    r	=0.;
    dra	=0.;
    dr2	=0.;
    drp	=0.;
    ki	=0;
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      splRA2(&rc,&rca,&rc2,rcT+ki,rcT2a+ki);
      splRA2(&rs,&rsa,&rs2,rsT+ki,rsT2a+ki);
      cs	=cos(k*gt);
      sn	=sin(k*gt);
      r		+=(rc*cs+rs*sn);
      dra	+=(rca*cs+rsa*sn);
      dr2	+=(rc2*cs+rs2*sn);
      drp	+=k*(-rc*sn+rs*cs);
    }
    splRA2(&rc,&rca,&rc2,rcT,rcT2a);
    r	=rc+2.*r-R;
    dra	=rca+2.*dra;
    dr2	=rc2+2.*dr2;
    drp	*=2.;
    D	=dra*dzp-drp*dza;
    da	=2.*(z*drp-r*dzp);
    da	/=sqrt(D*D+(dr2*dzp-da*dz2)*da)+D;
    a	+=da;
    dt	=(r*dza-z*dra-cr2*(dra*dz2-dr2*dza)*da*da)/D;
    gt	+=dt;
    if(a < 0.){
      Fl	=2;
      a	-=da;
    }
    if(a > 1.){
      return(-1);
    }
    i++;
  }while(i < 10 && fabs(dt)+fabs(da) > 0.001);
  if(i < 10){
    *A	=a;
    *t	=gt;
    *dRa	=dra;
    *dRp	=drp;
    *dZa	=dza;
    *dZp	=dzp;
    return(Fl);
  }
  return(1);
}

int ESGetMetrics(double *dRa,double *dRp,double *dZa,double *dZp
		    ,double a,double gt)
{
  int k,ki;
  double cs,sn;
  double rc,rca,rs,rsa;
  double dra,drp;
  
  ESSetSplA(a);
  cs	=cos(gt);
  sn	=sin(gt);
  splRA(&rc,&rca,ESsb,ESsb2a);
  splRA(&rs,&rsa,rsT,rsT2a);
  *dZa	=rsa+rca*sn;
  *dZp	=rc*cs;
  dra	=0.;
  drp	=0.;
  ki	=0;
  for(k=1; k < ESFp1; k++){
    ki	+=ESNa1;
    splRA(&rc,&rca,rcT+ki,rcT2a+ki);
    splRA(&rs,&rsa,rsT+ki,rsT2a+ki);
    cs	=cos(k*gt);
    sn	=sin(k*gt);
    dra	+=(rca*cs+rsa*sn);
    drp	+=k*(-rc*sn+rs*cs);
  }
  splRA(&rc,&rca,rcT,rcT2a);
  *dRa	=rca+2.*dra;
  *dRp	=2.*drp;
  return(0);
}

extern int EcNjet;
extern double EcRjet[],EcZjet[];
extern double *ESgY,*ESgY2a;

int FluxAtLabRZ(double *gY,double Br[2],double Bz[2],int nr1, int nz1)
     /* gY_{ij}=gY[nr1*i+j] */
{
  int i,j,j1,k;
  double r,z;
  double R1,R2;
  double a,gt;
  double dr,dz;
  
  dr	=(Br[1]-Br[0])/(nr1-1);
  dz	=(Bz[1]-Bz[0])/(nz1-1);

  EcNjet	=0;
  j1	=0;
  for(i=0; i < nz1;  i++){
    j	=j1;
    j1	+=nr1;
    z	=Bz[0]+i*dz;
    if(ES1DMapZ2gt(&gt,&R1,&R2,z) == 0){
      r	=Br[0];
      while(r < R1 && j < j1){
	gY[j]	=c2gp*ESgY[ESNa]; /* out of plasma */
	j++;
	r	+=dr;
      }
      gt	=cgp-gt;
      a		=0.95;
      while(r < R2 && j < j1){
	out("sidddd","r",i,R1,R2,r,z);
	k	=ES2DMapLab2Flx(&a,&gt,r,z);
	{
	  int ki,k;
	  double cs,sn,rc,rs;
	  ESSetSplA(a);
	  out("sI","EcNjet=",EcNjet);
	  EcRjet[EcNjet]	=0.;
	  ki	=0;
	  cs	=cos(gt);
	  sn	=sin(gt);
	  splRA(&rc,NULL,ESsb,ESsb2a);
	  splRA(&rs,NULL,rsT,rsT2a);
	  EcZjet[EcNjet]	=ESaZ0[0]+rs+rc*sn;
	  for(k=1; k < ESFp1; k++){
	    ki	+=ESNa1;
	    splRA(&rc,NULL,rcT+ki,rcT2a+ki);
	    splRA(&rs,NULL,rsT+ki,rsT2a+ki);
	    cs	=cos(k*gt);
	    sn	=sin(k*gt);
	    EcRjet[EcNjet]	+=rc*cs+rs*sn;
	  }
	  splRA(&rc,NULL,rcT,rcT2a);
	  EcRjet[EcNjet]	=rc+2.*EcRjet[EcNjet]+ESaR0[0];
	  EcNjet++;
	}

	if(k){
	  printf("Mapping failed\n");
	  return(-1);
	}
	ESSetSplA(a);
	splRA(gY+j,NULL,ESgY,ESgY2a);
	gY[j]	=c2gp*(gY[j]-ESgY[ESNa]);
	r	+=dr;
	j++;
      }
      while(j < j1){
	gY[j]	=c2gp*ESgY[ESNa];
	r	+=dr;
	j++;
      }
    }
    else{
      while(j < j1){
	gY[j]	=c2gp*ESgY[ESNa]; /* out of plasma */
	j++;
      }
    }
  }
  return(0);
}

/**********************/

int ES1DMapZ2gtP(double *gt,double *R1,double *R2, double z)
{
  double t,cs,sn,s;
  int k,ki;

  ki =ESNa;
  t =(z-ESaZ0[0]-rsT[ki])/ESsb[ESNa];
  if(t > 1. || t < -1.){
    return(1);
  }
  *gt =asin(t);

  *R1 =ESaR0[0]+rcT[ki];
  *R2 =*R1;
  s =-1.;
  for(k=1; k < ESFp1; k++){
    t =*gt*k;
    cs =cos(t);
    sn =sin(t);
    ki +=ESNa1;
    *R1 +=2.*(rcT[ki]*cs+rsT[ki]*sn);
    *R2 +=2.*(rcT[ki]*cs-rsT[ki]*sn)*s;
    s =-s;
  }
  return(0);
}

int ESIsOutsidePlasma(double *lr1,double *lr2,double r,double z)
{
  double a,gt,t,cs,sn,s,R1,R2;
  int k,ki;

  r	-=ESaR0[0];
  z	-=ESaZ0[0];

  ki =ESNa;
  a =(z-rsT[ki])/ESsb[ESNa];
  if(a > 1.){
    *lr1	=ESsr[ESNp1*ESNa+ESNp/2];
    *lr2	=*lr1;
    return(2);
  }
  if(a < -1.){
    *lr1	=ESsr[ESNp1*ESNa+3*ESNp/2];
    *lr2	=*lr1;
    return(2);
  }
  gt =asin(a);
  R1 	=0.;
  R2 	=0.;
  s	=-1.;
  for(k=1; k < ESFp1; k++){
    t =gt*k;
    cs =cos(t);
    sn =sin(t);
    ki +=ESNa1;
    R1 +=(rcT[ki]*cs-rsT[ki]*sn)*s;
    R2 +=rcT[ki]*cs+rsT[ki]*sn;
    s =-s;
  }
  R1	=2.*R1+rcT[ESNa];
  R2	=2.*R2+rcT[ESNa];
  if(r < R1 || R2 < r){
    *lr1	=R1+ESaR0[0];
    *lr2	=R2+ESaR0[0];
    return(2);
  }
  if(ESa0 != 0.){
    a =z/ESsb[0];
    if(fabs(a) < 1.){
      gt =asin(a);
      R1 	=0.;
      R2 	=0.;
      ki 	=0;
      s		=-1.;
      for(k=1; k < ESFp1; k++){
	t =gt*k;
	cs =cos(t);
	sn =sin(t);
	ki +=ESNa1;
	R1 +=(rcT[ki]*cs-rsT[ki]*sn)*s;
	R2 +=rcT[ki]*cs+rsT[ki]*sn;
	s =-s;
      }
      R1	=2.*R1+rcT[0];
      R2	=2.*R2+rcT[0];
      *lr1	=R1+ESaR0[0];
      *lr2	=R2+ESaR0[0];
      if(R1 < r && r < R2){
	return(1);
      }
    }
    else{
      *lr1	=a > 1 ? ESsr[ESNp/2] : ESsr[3*ESNp/2];
      *lr2	=*lr1;
    }
  }
  return(0);
}

int ES2DMapLab2FlxP(double *A,double *t,double R, double Z)
{
  int Fl;
  double D,Rt;
  double cs,sn;
  double rc,rca,rc2,rs,rsa,rs2;
  int k,ki,i;
  double r,z,dra,dr2,dza,dz2,drp,dzp,a,gt;

  R -=ESaR0[0];
  Z -=ESaZ0[0];
  Fl =0;
  a =*A;
  gt =*t;

  ESSetSplA(a);
  r =0.;
  Rt =0.;
  ki =0;
  dra =Z < 0. ? -1. : 1.;
  for(k=1; k < ESFp1; k++){
    ki +=ESNa1;
    splRA(&rc,NULL,rcT+ki,rcT2a+ki);
    splRA(&rs,NULL,rsT+ki,rsT2a+ki);
    cs =cos(k*gt);
    sn =sin(k*gt);
    r +=(rc*cs+rs*sn);
    switch(k%4){
    case 0:
      Rt +=rc;
      break;
    case 1:
      Rt +=dra*rs;
      break;
    case 2:
      Rt -=rc;
      break;
    case 3:
      Rt -=dra*rs;
      break;
    }
  }
  splRA(&rc,NULL,rcT,rcT2a);
  splRA(&rs,NULL,rcT,rcT2a);
  Rt =rc+2.*Rt;
  r =rc+2.*r;
  if((R-Rt)*(r-Rt) < 0.){
    gt =cgp-gt;
  }
  i =0;
  do{
    ESSetSplA(a);
    cs =cos(gt);
    sn =sin(gt);
    splRA2(&rc,&rca,&rc2,ESsb,ESsb2a);
    splRA2(&rs,&rsa,&rs2,rsT,rsT2a);
    z =rs+rc*sn-Z;
    dza =rsa+rca*sn;
    dz2 =rs2+rc2*sn;
    dzp =rc*cs;

    r =0.;
    dra =0.;
    dr2 =0.;
    drp =0.;
    ki =0;
    for(k=1; k < ESFp1; k++){
      ki +=ESNa1;
      splRA2(&rc,&rca,&rc2,rcT+ki,rcT2a+ki);
      splRA2(&rs,&rsa,&rs2,rsT+ki,rsT2a+ki);
      cs =cos(k*gt);
      sn =sin(k*gt);
      r  +=(rc*cs+rs*sn);
      dra +=(rca*cs+rsa*sn);
      dr2 +=(rc2*cs+rs2*sn);
      drp +=k*(-rc*sn+rs*cs);
    }
    splRA2(&rc,&rca,&rc2,rcT,rcT2a);
    r =rc+2.*r-R;
    dra =rca+2.*dra;
    dr2 =rc2+2.*dr2;
    drp *=2.;
    D =dra*dzp-drp*dza;
    drp =2.*(z*drp-r*dzp);
    drp /=sqrt(D*D+(dr2*dzp-drp*dz2)*drp)+D;
    a +=drp;
    dra =(r*dza-z*dra-cr2*(dra*dz2-dr2*dza)*drp*drp)/D;
    gt +=dra;
    if(a < 0.){
      Fl =2;
      a-=drp;
    }
    if(a > 1.){
      a	=1.;
    }
    i++;
  }while(i < 10 && fabs(dra)+fabs(drp) > 0.001);
  if(i < 10){
    *A =a;
    *t =gt;
    return(Fl);
  }
  return(1);
}

int FluxAtLabRZP(double *gY,double Br[2],double Bz[2],int nr1, int nz1)
     /* gY_{ij}=gY[nr1*i+j] */
{
  int i,j,j1,k;
  double r,z;
  double R1,R2;
  double a,gt;
  double dr,dz;

  printf("r1=%6g,  r2=%6g,  z1=%6g,  z2=%6g\n",Br[0],Br[1],Bz[0],Bz[1]);
  dr =(Br[1]-Br[0])/(nr1-1);
  dz =(Bz[1]-Bz[0])/(nz1-1);

  j1 =0;
  for(i=0; i < nz1;  i++){
    j =j1;
    j1 +=nr1;
    z =Bz[0]+i*dz;
    if(ES1DMapZ2gtP(&gt,&R1,&R2,z) == 0){
      r =Br[0];
      while(r < R1 && j < j1){
	gY[j] =0.; /* out of plasma */
        printf("A  %d  %10.3e  %10.3e    %10.3e\n",j,r,z,gY[j]);
	j++;
	r +=dr;
      }
      gt =cgp-gt;
      a =0.95;
      while(r < R2 && j < j1){
	k =ES2DMapLab2FlxP(&a,&gt,r,z);
	if(k){
	  printf("Mapping failed\n");
	  return(-1);
	}
	ESSetSplA(a);
	splRA(gY+j,NULL,ESgY,ESgY2a);
	gY[j] =c2gp*(gY[j]-ESgY[ESNa]);
        printf("B  %d  %10.3e  %10.3e    %10.3e\n",j,r,z,gY[j]);
	r +=dr;
	j++;
      }
      while(j < j1){
	gY[j] =0.;
        printf("C  %d  %10.3e  %10.3e    %10.3e\n",j,r,z,gY[j]);
	r +=dr;
	j++;
      }
    }
    else{
      while(j < j1){
	gY[j] =0.; /* out of plasma */
        printf("D  %d  %10.3e %10.3e\n",j,z,gY[j]);
	j++;
      }
    }
  }
  return(0);
}

int escpsi_(double *r1, double *r2, double *z1, double *z2)
{
  int i, j, j1, nr, nz;
  double Psi[625], Br[2], Bz[2];
  double r, z, dr, dz;

  printf("r1=%6g,  r2=%6g,  z1=%6g,  z2=%6g\n",*r1,*r2,*z1,*z2);
  nr = 10;  nz = 10;
  Br[0] = *r1;  Br[1] = *r2;  Bz[0] = *z1;  Bz[1] = *z2;        
  dr =(Br[1]-Br[0])/(nr-1);
  dz =(Bz[1]-Bz[0])/(nz-1);

  i = FluxAtLabRZP(&Psi[0], Br, Bz, nr, nz);
  i = 0;
  for (j=0; j < nr; j++){
    z =Bz[0]+j*dz;
    r = Br[0];
    for (j1=0; j1 < nz; j1++){
      printf("i=%d,  r=%6g,  z=%6g,  Psi=%g \n", i,r,z,Psi[i]);
      r +=dr;
      i++;
    }
  }
  return(0);
}

int ESrz2agt(double *la,double *lgt,double r,double z)
{
  int Fl;
  int k,ki,i;
  double a,gt,t,cs,sn,s,R1,R2;
  double x,y,z0,dr,dz,b,db;

  double D,Rt;
  double rc,rca,rc2,rs,rsa,rs2;
  double dra,dr2,dza,dz2,drp,dzp,da,dt;

  r	-=ESaR0[0];
  z	-=ESaZ0[0];

  ki =ESNa;
  a =(z-rsT[ki])/ESsb[ESNa];
  if(a > 1. || a < -1.){
    return(2);
  }
  gt =asin(a);
  R1 	=0.;
  R2 	=0.;
  s	=-1.;
  for(k=1; k < ESFp1; k++){
    t =gt*k;
    cs =cos(t);
    sn =sin(t);
    ki +=ESNa1;
    R1 +=(rcT[ki]*cs-rsT[ki]*sn)*s;
    R2 +=rcT[ki]*cs+rsT[ki]*sn;
    s =-s;
  }
  R1	=2.*R1+rcT[ESNa];
  R2	=2.*R2+rcT[ESNa];
  if(r < R1 || R2 < r){
    return(2);
  }
  if(ESa0 != 0.){
    x =z/ESsb[0];
    a	=fabs(x);
    if(a < 1.){
      gt 	=asin(x);
      R1 	=0.;
      R2 	=0.;
      ki 	=0;
      s		=-1.;
      for(k=1; k < ESFp1; k++){
	t =gt*k;
	cs =cos(t);
	sn =sin(t);
	ki +=ESNa1;
	R1 +=(rcT[ki]*cs-rsT[ki]*sn)*s;
	R2 +=rcT[ki]*cs+rsT[ki]*sn;
	s =-s;
      }
      R1	=2.*R1+rcT[0];
      R2	=2.*R2+rcT[0];
      if(R1 < r && r < R2){
	return(1);
      }
      x	=2.*r/(ESsr[0]-ESsr[ESNp/2]);
      y	=z/ESsb[0];
      a	=sqrt(y*y+x*x);	
      t 	=asin(y/a);
      if(r < R1){
	t	=cgp-t;
      }
      a	*=ESa0;
      if(a > ESsa[ESNa]){
	a	=ESsa[ESNa]*0.95;
      }
      if(a < ESa0){
	a	=ESa0*1.05;
      }
    }
    else{
      s	=2.*rcT1a[0]/ESsb1a[0];
      y	=z/ESsb[3];
      x	=r/(s*ESsb[3]);
      a	=sqrt(y*y+x*x);	
      if(a > 1.){
	k	=ESNp1*ESNa;
	x	=2.*r/(ESsr[k]-ESsr[k+ESNp/2]);
	y	=z/ESsb[ESNa];
	a	=sqrt(y*y+x*x);	
	t	=asin(y/a);
	a	*=ESsa[ESNa];
	if(a > ESsa[ESNa]){
	  a	=ESsa[ESNa]*0.95;
	}
      }
      else{
	t	=asin(y/a);
	a	*=ESsa[3];
      }
    }
  }
  
  gt	=t;
  ESSetSplA(a);
  dr	=0.;
  Rt	=0.;
  ki	=0;
  s	=z < 0. ? -1. : 1.;
  for(k=1; k < ESFp1; k++){
    ki	+=ESNa1;
    splRA(&rc,NULL,rcT+ki,rcT2a+ki);
    splRA(&rs,NULL,rsT+ki,rsT2a+ki);
    cs	=cos(k*gt);
    sn	=sin(k*gt);
    dr	+=(rc*cs+rs*sn);
    switch(k%4){
    case 0:
      Rt	+=rc;
      break;
    case 1:
      Rt	+=s*rs;
      break;
    case 2:
      Rt	-=rc;
      break;
    case 3:
      Rt	-=s*rs;
      break;
    }
  }
  splRA(&rc,NULL,rcT,rcT2a);
  splRA(&rs,NULL,rcT,rcT2a);
  Rt	=rc+2.*Rt;
  dr	=rc+2.*dr;

  if((r-Rt)*(dr-Rt) < 0.){
    gt	=cgp-gt;
  }
  i	=0;
  do{
    ESSetSplA(a);
    cs	=cos(gt);
    sn	=sin(gt);
    splRA2(&rc,&rca,&rc2,ESsb,ESsb2a);
    splRA2(&rs,&rsa,&rs2,rsT,rsT2a);
    dz	=rs+rc*sn-z;
    dza	=rsa+rca*sn;
    dz2	=rs2+rc2*sn;
    dzp	=rc*cs;
    
    dr	=0.;
    dra	=0.;
    dr2	=0.;
    drp	=0.;
    ki	=0;
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      splRA2(&rc,&rca,&rc2,rcT+ki,rcT2a+ki);
      splRA2(&rs,&rsa,&rs2,rsT+ki,rsT2a+ki);
      cs	=cos(k*gt);
      sn	=sin(k*gt);
      dr	+=(rc*cs+rs*sn);
      dra	+=(rca*cs+rsa*sn);
      dr2	+=(rc2*cs+rs2*sn);
      drp	+=k*(-rc*sn+rs*cs);
    }
    splRA2(&rc,&rca,&rc2,rcT,rcT2a);
    dr	=rc+2.*dr-r;
    dra	=rca+2.*dra;
    dr2	=rc2+2.*dr2;
    drp	*=2.;
    D	=dra*dzp-drp*dza;
    da	=2.*(dz*drp-dr*dzp);
    da	/=sqrt(D*D+(dr2*dzp-da*dz2)*da)+D;
    a	+=da;
    dt	=(dr*dza-dz*dra-cr2*(dra*dz2-dr2*dza)*da*da)/D;
    gt	+=dt;
    if(a < ESa0){
      if(ESa0 != 0.){
	a	=ESa0;
      }
      else{
	y	=z/ESsb1a[0];
	x	=r/(2.*rcT1a[0]);
	a	=sqrt(y*y+x*x);	
      }
    }
    if(a > 1.){
      a	=1;
    }
    i++;
  }while(i < 10 && fabs(dt)+fabs(da) > 0.001);
  if(i == 10){
    return(3);
  }
  *la	=a;
  *lgt	=gt;
  return(0);

#ifdef H
  a	=ESa0+(fabs(z)-ESsb[0])*(ESsa[ESNa]-ESa0)/(ESsb[ESNa]-ESsb[0]);
  x	=z > 0. ? a : -a;
  do{
    ESSetSplA(a);
    splRA(&z0,&dz,rsT,rsT2a);
    splRA(&b,&db,ESsb,ESsb2a);
    if(x < 0.){
      dz=-dz;
      b	=-b;
    }
    db	=(z-z0-b)/(dz+db);
    x	+=db;
    a	=fabs(x);
  }while(fabs(db) > 1e-5);
#endif
}

int ESErrorInGSh()
{
  static int Fl=0;
  int i,j,ji;
  int k,ki,kj;
#ifdef XWIN
  double rc[ESFp1],rca[ESFp1],rc2a[ESFp1],rs[ESFp1],rsa[ESFp1],rs2a[ESFp1];
#else
  double rc[33],rca[33],rc2a[33],rs[33],rsa[33],rs2a[33];
#endif
  double b,ba,b2a;
  double r,r0,rr0,s,A,rA;
  double ra,rt,za,zt,raa,rat,rtt,zaa,zat,ztt;
  double LHS,RHS,LHS0,RHS0;
  double K,Ka,N,Nt,D,L,V;
  double cs,sn;
  double Err,Max;
  int i0,j0;
  int fpa;
  int offset,Off0;
  static char RunIDa[16];
  char ln[64],*lc[24];
  extern double ESTime;
  extern int ESEqSolvRadC,ESEqSolvInPr,ESEqSolvInCr,ESnDtPr,ESnDtCr,ESFail;

  static double t=0.;
  ESTime	=t;

  offset	=0;
  if(Fl == 0){
    extern char ESRunID[];
#ifdef XWIN
    sprintf(RunIDa,"%s.err",ShotNm);
#else      
    k	=CbStrCpy(RunIDa,ESRunID);
    strcpy(RunIDa+k,".err");
#endif
    fpa	=open(RunIDa,O_RDWR|O_CREAT|O_TRUNC,S_IRWXU|S_IROTH|S_IXOTH);
    lc[0]	="      subroutine readescerr ! an example reading the file\n";
    lc[1]	="      real*8 t,err\n";
    lc[2]	="      real*8 LHS(65,21),RHS(65,21)\n";
    lc[3]	="      integer na1,nt1\n";
    lc[4]	="      integer i,j,k,u\n";
    lc[5]	="      character*64 ch\n";
    lc[6]	="      u=1\n";
    sprintf(ln,"      open(unit=u,file='%s')\n",RunIDa);
    lc[7]	=ln;
    lc[8]	="      do k=1,26\n";
    lc[9]	="        read(u,*)\n";
    lc[10]	="      enddo\n\n";
    lc[11]	="      read(u,'(a6,i3,a7,i3)')ch,na1,ch,nt1\n";
    lc[12]	="      do k=1,1000\n";
    lc[13]	="        read(u,'(a5,e12.5,a45,e9.2)',end=1)ch,t,ch,err\n";
    lc[14]	="        read(u,*)\n";
    lc[15]	="        do i=1,na1\n";
    lc[16]	="          do j=1,nt1\n";
    lc[17]	="          read(u,'(a5,2e12.4,e10.2)')ch,LHS(j,i),RHS(j,i)\n";
    lc[18]	="          enddo\n";
    lc[19]	="        enddo\n";
    lc[20]	="      enddo\n";
    lc[21]	=" 1    continue\n";
    lc[22]	="      close(u)\n";
    lc[23]	="      end\n\n";
    Fl	=1;
  }
  else{
    fpa	=open(RunIDa,O_RDWR);
    lseek(fpa,(off_t)0,SEEK_END);
    Fl	=2;
  }
  if(fpa == -1){
    printf("%s - ASCII Error file cannot be open\n",RunIDa);
    return(1);
  }
  if(Fl == 1){
    for(i=0; i < 24; i++){
      k	=strlen(lc[i]);
      write(fpa,lc[i],k);
      offset	+=k;
    }
    sprintf(ln,"ESNa1=%3d ESNp1=%3d\n",ESNa1,ESNp1);
    write(fpa,ln,20);
    offset	+=20;
  }
  Err	=0.;
  Max	=0.;
  sprintf(ln,"time=%12.5e RadC=%d InPr=%d InCr=%d nPr=%d nCr=%d 0x%8.8x"
	  ,ESTime,ESEqSolvRadC,ESEqSolvInPr,ESEqSolvInCr,ESnDtPr,ESnDtCr
	  ,ESFail);
  write(fpa,ln,57);
  offset	+=57;
  Off0	=offset;
  sprintf(ln," Err(%2d,%2d)=%8.2e\n",0,0,Err/Max);
  write(fpa,ln,21);
  offset	+=21;
  sprintf(ln,"%2s %2s %11s %11s %9s\n","i","j","LHS","RHS","LHS-RHS");
  write(fpa,ln,40);
  offset	+=40;

  r0	=ESaR0[0];
  rr0	=1./r0;
  i	=0;
  b	=ESsb1a[i];
  ba	=ESsb1a[i];
  ki	=ESNa1;
  rc[1]	=2.*rcT1a[ki];
  rs[1]	=2.*rsT1a[ki];
  rca[1]=2.*rcT1a[ki];
  rsa[1]=2.*rsT1a[ki];
  D	=b*rc[1];
  r	=r0;
  L	=D;
  s	=1./(r*D);

  j	=0;
  cs	=1.;
  rt	=rs[1];
  zt	=b*cs;
  ra	=rca[1];
  za	=0.;
  rat	=rsa[1];
  rtt	=-rc[1];
  zat	=ba;
  ztt	=0.;
  K	=(rt*rt+zt*zt)*s;
  N	=(ra*rt+za*zt)*s;
  Nt	=(rat*rt+ra*rtt+zat*zt+za*ztt)*s;
  LHS	=(2.*K-Nt)*ESdgY[0];
  RHS	=-L*ESjs[0];
  LHS0	=LHS;
  RHS0	=RHS;
  if(Max < fabs(RHS)){
    Max	=fabs(RHS);
  }
  if(Err < fabs(LHS-RHS)){
    i0	=i;
    j0	=j;
    Err	=fabs(LHS-RHS);
  }
  sprintf(ln,"%2d %2d %11.4e %11.4e %9.2e\n",i,j,LHS,RHS,LHS-RHS);
  write(fpa,ln,40);
  offset	+=40;
  for(j=1; j < ESNp; j++){
    cs	=cs1[j];
    sn	=sn1[j];
    rt	=-rc[1]*sn+rs[1]*cs;
    zt	=b*cs;
    ra	=rca[1]*cs+rsa[1]*sn;
    za	=ba*sn;
    rat	=-rca[1]*sn+rsa[1]*cs;
    rtt	=-rc[1]*cs-rs[1]*sn;
    zat	=ba*cs;
    ztt	=-b*sn;
    K	=(rt*rt+zt*zt)*s;
    N	=(ra*rt+za*zt)*s;
    Nt	=(rat*rt+ra*rtt+zat*zt+za*ztt)*s;
    LHS	=(2.*K-Nt)*ESdgY[0];
    RHS	=-L*ESjs[0];
    sprintf(ln,"%2d %2d %11.4e %11.4e %9.2e\n",i,j,LHS,RHS,LHS-RHS);
    write(fpa,ln,40);
    offset	+=40;
    if(Max < fabs(RHS)){
      Max	=fabs(RHS);
    }
    if(Err < fabs(LHS-RHS)){
      i0	=i;
      j0	=j;
      Err	=fabs(LHS-RHS);
    }
  }
  LHS	=LHS0;
  RHS	=RHS0;
  sprintf(ln,"%2d %2d %11.4e %11.4e %9.2e\n",i,j,LHS,RHS,LHS-RHS);
  write(fpa,ln,40);
  offset	+=40;
  ji	=ESNp1;
  for(i=1; i < ESNa1; i++){
    A	=ESsa[i];
    rA	=1./A;
    b	=ESsb[i];
    ba	=ESsb1a[i];
    b2a	=ESsb2a[i];
    ki	=i;
    rc[0]	=rcT[ki];
    rs[0]	=rsT[ki];
    rca[0]	=rcT1a[ki];
    rsa[0]	=rsT1a[ki];
    rc2a[0]	=rcT2a[ki];
    rs2a[0]	=rsT2a[ki];
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      rc[k]	=2.*rcT[ki];
      rs[k]	=2.*rsT[ki];
      rca[k]	=2.*rcT1a[ki];
      rsa[k]	=2.*rsT1a[ki];
      rc2a[k]	=2.*rcT2a[ki];
      rs2a[k]	=2.*rsT2a[ki];
    }
    j	=0;
    ra	=rca[0];
    raa	=rc2a[0];
    rat	=0.;
    rtt	=0.;
    rt	=0.;
    kj	=0;
    for(k=1; k < ESFp1; k++){
      kj	+=j;
      if(kj >= ESNp){
	kj	-=ESNp;
      }
      ra	+=rca[k];
      raa	+=rc2a[k];
      rt	+=rs[k]*k;
      rat	+=rsa[k]*k;
      rtt	-=rc[k]*k*k;
    }
    zt	=b;
    za	=rsa[0];
    zaa	=rs2a[0];
    zat	=ba;
    ztt	=0.;
    D	=ra*zt-rt*za;
    r	=ESsr[ji];
    L	=D*rA*r0/r;
    V	=D*rA*r*rr0-L;
    s	=1./(r*D);
    N	=(ra*rt+za*zt)*s;
    Nt	=(rat*rt+ra*rtt+zat*zt+za*ztt)*s
      -N*(rt/r+(rat*zt+ra*ztt-rtt*za-rt*zat)/D);
    K	=(rt*rt+zt*zt)*s*rA;
    Ka	=2.*(rt*rat+zt*zat)*s
      -A*K*(ra/r+(raa*zt+ra*zat-rat*za-rt*zaa)/D);
    LHS	=(K+Ka-Nt)*ESdgY[i]+A*K*ESdgY1a[i];
    RHS	=-L*ESjs[i]-V*ESjp[i];
    LHS0=LHS;
    RHS0=RHS;
    sprintf(ln,"%2d %2d %11.4e %11.4e %9.2e\n",i,j,LHS,RHS,LHS-RHS);
    write(fpa,ln,40);
    offset	+=40;
    if(Max < fabs(RHS)){
      Max	=fabs(RHS);
    }
    if(Err < fabs(LHS-RHS)){
      i0	=i;
      j0	=j;
      Err	=fabs(LHS-RHS);
    }
    ji++;
    for(j=1; j < ESNp; j++){
      ra	=rca[0];
      raa	=rc2a[0];
      rat	=0.;
      rtt	=0.;
      rt	=0.;
      kj	=0;
      for(k=1; k < ESFp1; k++){
	kj	+=j;
	if(kj >= ESNp){
	  kj	-=ESNp;
	}
	cs	=cs1[kj];
	sn	=sn1[kj];
	ra	+=rca[k]*cs+rsa[k]*sn;
	raa	+=rc2a[k]*cs+rs2a[k]*sn;
	rt	+=(-rc[k]*sn+rs[k]*cs)*k;
	rat	+=(-rca[k]*sn+rsa[k]*cs)*k;
	rtt	-=(rc[k]*cs+rs[k]*sn)*k*k;
      }
      cs	=cs1[j];
      sn	=sn1[j];
      zt	=b*cs;
      za	=rsa[0]+ba*sn;
      zaa	=rs2a[0]+b2a*sn;
      zat	=ba*cs;
      ztt	=-b*sn;
      D		=ra*zt-rt*za;
      r		=ESsr[ji];
      L		=D*rA*r0/r;
      V		=D*rA*r*rr0-L;
      s		=1./(r*D);
      N		=(ra*rt+za*zt)*s;
      Nt	=(rat*rt+ra*rtt+zat*zt+za*ztt)*s
	-N*(rt/r+(rat*zt+ra*ztt-rtt*za-rt*zat)/D);
      K	=(rt*rt+zt*zt)*s*rA;
      Ka	=2.*(rt*rat+zt*zat)*s
	-A*K*(ra/r+(raa*zt+ra  *zat-rat*za-rt*zaa)/D);
      LHS	=(K+Ka-Nt)*ESdgY[i]+A*K*ESdgY1a[i];
      RHS	=-L*ESjs[i]-V*ESjp[i];
      sprintf(ln,"%2d %2d %11.4e %11.4e %9.2e\n",i,j,LHS,RHS,LHS-RHS);
      write(fpa,ln,40);
      offset	+=40;
      if(Max < fabs(RHS)){
	Max	=fabs(RHS);
      }
      if(Err < fabs(LHS-RHS)){
	i0	=i;
	j0	=j;
	Err	=fabs(LHS-RHS);
      }
      ji++;
    }
    LHS	=LHS0;
    RHS	=RHS0;
    sprintf(ln,"%2d %2d %11.4e %11.4e %9.2e\n",i,j,LHS,RHS,LHS-RHS);
    write(fpa,ln,40);
    offset	+=40;
    ji++;
  }
  offset	=Off0-offset;
  lseek(fpa,(off_t)offset,SEEK_END);
  sprintf(ln," Err(%2d,%2d)=%8.2e\n",i0,j0,Err/Max);
  write(fpa,ln,21);
  close(fpa);
  t	+=1.;
  return(0);
}

int ESCheckGSh(int i0,int i1,int m0,int m1,int Fl)
{
  int i,j,ji,im,jm;
  int k,ki,kj;
  double rc[ESFp1],rca[ESFp1],rc2a[ESFp1],rs[ESFp1],rsa[ESFp1],rs2a[ESFp1];
  double Kc[ESFp1],Kca[ESFp1],Ks[ESFp1],Ksa[ESFp1];
  double b,ba,b2a;
  double r,r0,rr0,s,A,rA;
  double raa,rat,rtt,zaa,zat,ztt;
  double LHS[ESNp1*ESNa1],RHS[ESNp1*ESNa1];
  double LHSc[ES2Mp1],LHSs[ES2Mp1],RHSc[ES2Mp1],RHSs[ES2Mp1];
  double K,Ka,N,Nt,D,L,V;
  double KK[ESNp1],KKa[ESNp1];
  double cs,sn;
  double *lL,*lR;
  double Err,Max;

  Err	=0.;
  Max	=0.;
  im	=0;
  jm	=0;

  r0	=ESaR0[0];
  rr0	=1./r0;
  i	=0;
  b	=ESsb1a[i];
  ba	=ESsb1a[i];
  ki	=ESNa1;
  rc[1]	=2.*rcT1a[ki];
  rs[1]	=2.*rsT1a[ki];
  rca[1]=2.*rcT1a[ki];
  rsa[1]=2.*rsT1a[ki];
  D	=b*rc[1];
  r	=r0;
  s	=1./(r*D);
  for(j=0; j < ESNp; j++){
    cs	=cs1[j];
    sn	=sn1[j];
    drgt[j]	=-rc[1]*sn+rs[1]*cs;
    dzgt[j]	=b*cs;
    dra[j]	=rca[1]*cs+rsa[1]*sn;
    dza[j]	=ba*sn;
    rat		=-rca[1]*sn+rsa[1]*cs;
    rtt		=-rc[1]*cs-rs[1]*sn;
    zat		=ba*cs;
    ztt		=-b*sn;
    K	=(drgt[j]*drgt[j]+dzgt[j]*dzgt[j])*s;
    N	=(dra[j]*drgt[j]+dza[j]*dzgt[j])*s;
    Nt	=(rat*drgt[j]+dra[j]*rtt+zat*dzgt[j]+dza[j]*ztt)*s;
    LHS[j]	=(2.*K-Nt)*ESdgY[0];
    RHS[j]	=-D*ESjs[0];
    LHS[j]	-=RHS[j];
    if(Max < fabs(RHS[j])){
      Max	=fabs(RHS[j]);
    }
    if(Err < fabs(LHS[j])){
      im	=i;
      jm	=j;
      Err	=fabs(LHS[j]);
    }
  }
  LHS[j]	=LHS[0];
  RHS[j]	=RHS[0];
  ESP2F(LHSc,LHSs,LHS,ES2Mp);
  ESP2F(RHSc,RHSs,RHS,ES2Mp);
  ji	=ESNp1;
  for(i=1; i < ESNa1; i++){
    A	=ESsa[i];
    rA	=1./A;
    b	=ESsb[i];
    ba	=ESsb1a[i];
    b2a	=ESsb2a[i];
    ki	=i;
    rc[0]	=rcT[ki];
    rs[0]	=rsT[ki];
    rca[0]	=rcT1a[ki];
    rsa[0]	=rsT1a[ki];
    rc2a[0]	=rcT2a[ki];
    rs2a[0]	=rsT2a[ki];

    Kc[0]	=ESg22c[ki];
    Ks[0]	=ESg22s[ki];
    Kca[0]	=ESg22c1[ki];
    Ksa[0]	=ESg22s1[ki];

    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      rc[k]	=2.*rcT[ki];
      rs[k]	=2.*rsT[ki];
      rca[k]	=2.*rcT1a[ki];
      rsa[k]	=2.*rsT1a[ki];
      rc2a[k]	=2.*rcT2a[ki];
      rs2a[k]	=2.*rsT2a[ki];

      if(k < ESMp1){
	Kc[k]	=2.*ESg22c[ki];
	Ks[k]	=2.*ESg22s[ki];
	Kca[k]	=2.*ESg22c1[ki];
	Ksa[k]	=2.*ESg22s1[ki];
      }
    }
    for(j=0; j < ESNp; j++){
      dra[j]	=rca[0];
      raa	=rc2a[0];
      rat	=0.;
      rtt	=0.;
      drgt[j]	=0.;
      K		=Kc[0];
      Ka	=Kca[0];
      kj	=0;
      for(k=1; k < ESFp1; k++){
	kj	+=j;
	if(kj >= ESNp){
	  kj	-=ESNp;
	}
	cs	=cs1[kj];
	sn	=sn1[kj];
	dra[j]	+=rca[k]*cs+rsa[k]*sn;
	raa	+=rc2a[k]*cs+rs2a[k]*sn;
	drgt[j]	+=(-rc[k]*sn+rs[k]*cs)*k;
	rat	+=(-rca[k]*sn+rsa[k]*cs)*k;
	rtt	-=(rc[k]*cs+rs[k]*sn)*k*k;
	if(k < ESMp1){
	  K	+=Kc[k]*cs+Ks[k]*sn;
	  Ka	+=Kca[k]*cs+Ksa[k]*sn;
	}
      }
      cs	=cs1[j];
      sn	=sn1[j];
      dzgt[j]	=b*cs;
      dza[j]	=rsa[0]+ba*sn;
      zaa	=rs2a[0]+b2a*sn;
      zat	=ba*cs;
      ztt	=-b*sn;
      D		=dra[j]*dzgt[j]-drgt[j]*dza[j];
      r		=ESsr[ji];
      L		=D*rA*r0/r;
      V		=D*rA*r*rr0-L;
      s		=1./(r*D);
      N		=(dra[j]*drgt[j]+dza[j]*dzgt[j])*s;
      Nt	=(rat*drgt[j]+dra[j]*rtt+zat*dzgt[j]+dza[j]*ztt)*s
	-N*(drgt[j]/r+(rat*dzgt[j]+dra[j]*ztt-rtt*dza[j]-drgt[j]*zat)/D);
#ifdef H
      LHS[ji]	=(2.*K+A*Ka-Nt)*ESdgY[i]+A*K*ESdgY1a[i];
      if(i == ESNa){
	K	=(drgt[j]*drgt[j]+dzgt[j]*dzgt[j])*s*rA;
	Ka	=2.*(drgt[j]*rat+dzgt[j]*zat)*s
	  -A*K*(dra[j]/r+(raa*dzgt[j]+dra[j]*zat-rat*dza[j]-drgt[j]*zaa)/D);
	KK[j]	=V;
	KKa[j]	=(raa*dzgt[j]+dra[j]*zat-rat*dza[j]-drgt[j]*zaa)*(r*rr0-r0/r)
	  +D*dra[j]*(rr0+r0/(r*r));
      }
#endif

      K	=(drgt[j]*drgt[j]+dzgt[j]*dzgt[j])*s*rA;
      Ka	=2.*(drgt[j]*rat+dzgt[j]*zat)*s
	-A*K*(dra[j]/r+(raa*dzgt[j]+dra[j]*zat-rat*dza[j]-drgt[j]*zaa)/D);
      LHS[ji]	=(K+Ka-Nt)*ESdgY[i]+A*K*ESdgY1a[i];
      RHS[ji]	=-L*ESjs[i]-V*ESjp[i];
      LHS[ji]	-=RHS[ji];
      if(Max < fabs(RHS[ji])){
	Max	=fabs(RHS[ji]);
      }
      if(Err < fabs(LHS[ji])){
	im	=i;
	jm	=j;
	Err	=fabs(LHS[ji]);
      }
      ji++;
    }
    j	=ji-ESNp;
    LHS[ji]	=LHS[j];
    RHS[ji]	=RHS[j];
    
#ifdef H
    if(i == ESNa){
      KK[j]	=KK[0];
      KKa[j]	=KKa[0];
      ESP2F(LHSc,LHSs,KK,ESMp1);
      ESP2F(RHSc,RHSs,KKa,ESMp1);
      ki	=ESNa;
      for(k=0; k < ESMp1; k++){
	if(k){
	  out("sidddd","KK",k,LHSc[k],LHSs[k],RHSc[k],RHSs[k]);
	}
	else{
	  out("sidddd","KK",k,LHSc[k],LHSs[k],RHSc[k],RHSs[k]);
	}
	out("sidddd","K ",k,ESVc[ki],ESVs[ki]
	    ,ESVc[ki]+ESVc1[ki],ESVs[ki]+ESVs1[ki]);
	ki	+=ESNa1;
      }
    }
#endif
    ji++;
    if(m0 < m1){
      lL	=LHS+j;
      lR	=RHS+j;
      ESP2F(LHSc,LHSs,lL,m1-1);
      ESP2F(RHSc,RHSs,lR,m1-1);
      for(j=0; j < ESNp; j++){
	r	=0.;
	s	=0.;
	kj	=0;
	for(k=m0 ? m0 : 1; k < m1; k++){
	  kj	+=j;
	  if(kj >= ESNp){
	    kj	-=ESNp;
	  }
	  cs	=cs1[kj];
	  sn	=sn1[kj];
	  s	+=LHSc[k]*cs+LHSs[k]*sn;
	  r	+=RHSc[k]*cs+RHSs[k]*sn;
	}
	s	*=2.;	
	r	*=2.;
	if(m0 == 0){
	  s	+=LHSc[0];
	  r	+=RHSc[0];
	}
	lL[j]	=s;
	lR[j]	=r;
      }
      lL[j]	=lL[0];
      lR[j]	=lR[0];
    }
  }
  printf(" Err(%2d,%2d)=%9.2e %9.2e %9.2e%% %9.2e\n",im,jm,Err,Max,Err*100./Max
	 ,log10(Err/Max));
#ifdef XWIN
  {
    extern int ESEqSolvFl;
    double x[2],y[ESNp1],y0[ESNp1];
    char ln[32];

    j	=ESNp1*i0;
    j	=0;
    r	=log10(fabs(RHS[j])+1e-20);
    y[0]	=-5.;
    y[1]	=0.;
    x[0]	=ESgt[0];
    x[1]	=ESgt[ESNp];

    if(Fl){
      static int inx=1,iny=1;
      static int kPS=1;
      static char PSFileNm[16];
      if(Fl > 1){
	if(Fl != 4){
	  inx	=Fl-1;
	  iny	=Fl-1;
	}
	else{
	  inx	=4;
	  iny	=4;
	}
      }
      if(kPS && *ShotNm != '\0'){
	sprintf(PSFileNm,"%s.ps",ShotNm);
	PSSetFileName(PSFileNm);
	kPS	=0;
      }
      PSFileOpen(inx,iny);
    }
    sprintf(ln,"%d<=i<%d m=%d (%s) %s",i0,i1,ESMp
	    ,(ESEqSolvFl&0x0F) ? "RungK" : "Lsode",ShotNm);
    SetPlotName("sqrt(gF)","GSh log10|LHS-RHS|",ln);
    Scale2d(4,x,y,2,6,2);
    if(Fl){
      PSNewFrame();
    }
    
    if(i0 < 0){
      i0	=0;
    }
    if(i1 > ESNa1){
      i1	=ESNa1;
    }
    ji	=ESNp1*i0;
    for(i=i0; i < i1; i++){
      lL	=LHS+ji;
      lR	=RHS+ji;
      for(j=0; j < ESNp1; j++){
	y[j]	=log10(fabs(lL[j])+1e-20);
	y0[j]	=log10(fabs(lR[j])+1e-20);
      }
      Plot2d(4,ESgt,y,ESNp1,6,0,14,0);
      Plot2d(4,ESgt,y0,ESNp1,6,0,4,0);
      if(Fl){
	PSPlot2d(ESgt,y,ESNp1,6,0,14,0);
	PSPlot2d(ESgt,y0,ESNp1,6,0,4,0);
      }
      ji	+=ESNp1;
    }
    CbFlush();
    if(Fl){
      PSFileClose();
      ESErrorInGSh();
    }
  }
#endif

  return(0);
}
