#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/shm.h>

#if defined(_SEM_SEMUN_UNDEFINED)
union semun {
  int val;                    /* value for SETVAL */
  struct semid_ds *buf;       /* buffer for IPC_STAT, IPC_SET */
  unsigned short int *array;  /* array for GETALL, SETALL */
  struct seminfo *__buf;      /* buffer for IPC_INFO */
};
#endif

/* Number of elements in shared memory buffer */
#define NUM_ELEM 10

#define SHMEMLENGTH 65536
#define S0 0
#define S1 1

static long int BegOfRecord=0L;
int SemID;
int ShmID;
static struct sembuf bufV={S0, 1,IPC_NOWAIT};
static struct sembuf bufP={S1, -1,SEM_UNDO};
void *lShm;
static int *lRetFl;

#ifndef FORK
int ESUpDateCleanFile()
{
  FILE *lF;

  lF	=fopen("escCLEAN","r+");
  if(lF == NULL){
    lF	=fopen("escCLEAN","w+");
    BegOfRecord=0L;
  }
  else{
    fseek(lF,(long int)(-12),SEEK_END);
    BegOfRecord=ftell(lF);
  }
  fprintf(lF,"ipcrm sem %d\n",SemID);
  fprintf(lF,"ipcrm shm %d\n",ShmID);
  fprintf(lF,"kill %d\n",getpid());
  fprintf(lF,"rm escCLEAN\n");
  fclose(lF);
  chmod("escCLEAN",00777);
  printf("escCLEAN - if exists, then use for cleaning system resources\n");
  return(0);
}

int ESRestoreCleanFile()
{
  int k;
  k	=open("escCLEAN",O_RDWR);
  if(k == -1){
    return(0);
  }
  if(BegOfRecord == 0L){
    close(k);
    system("rm escCLEAN");
    return(0);
  }
  lseek(k,BegOfRecord,SEEK_SET);
  write(k,"rm escCLEAN\n",12);
  ftruncate(k,BegOfRecord+12);
  close(k);
  return(0);
}

int ESCInitStr()
{
  Flesc0=1;
  ESiAx	=0;
  ESNp	=64;
  ESMp	=12;
  ESMp	=6;
  ESFp	=2*ESMp;
  TFCNt	=0;
  TFCNp	=1;
  ESLt	=0;
  ESNt	=0;

  InitConstants();
  ESInitPlGeomInt();
  ESReInit1DPlV();
  ESReInit2DTPlV();
  ESReInitPlProf(21);
  EcInitArrays();

  ESReInitMetrTnsrCS();
  ECReInitFgY();
  
  ESGetPlVDAddr(&rdPV,&zdPV);
  ESInit1DPlV();
  ESInit2DTPlV();

  cssp	=cgm0;
  cssj	=cgm0;
  cssr	=1.;
#ifdef SHMEM
  kI	=sizeof(int);
  kD	=sizeof(double);
#endif
  lProf[0]	=ESgY;
  lProf[1]	=ESgF;
  lProf[2]	=ESaF;
  lProf[3]	=ESgm;
  lProf[4]	=ESsq;
  lProf[5]	=ESsp;

  lProf2[0]	=ESgY2a;
  lProf2[1]	=ESgF2a;
  lProf2[2]	=ESaF2a;
  lProf2[3]	=ESgm2a;
  lProf2[4]	=ESsq2a;
  lProf2[5]	=ESsp2a;
  return(0);
}

void esc00_(char *MachineRunID, int *na)
{
  if(MachineRunID != NULL){
    char *p0,*p1;
    int L;

    char *p2;
    p2	=MachineRunID;
    while(isspace(*MachineRunID)){
      MachineRunID++;
    }
    p0	=ESMachineNm;
    p1	=MachineRunID;
    while(*p1 != '.' && *p1 != '\0' && !isspace(*p1)){
      *p0++	=*p1++;
    }
    if(p0-ESMachineNm){
      *p0	='\0';
    }
    while(*p1 != '.' && *p1 != '\0'){
      p1++;
    }
    if(*p1 == '.'){
      p1++;
      while(isspace(*p1)){
	p1++;
      }
      if(*p1 != '\0'){
	CbStrCpy(ESRunID,p1);
      }
    }
#ifdef H
    printf("|%s| |%s|.|%s|\n",MachineRunID,ESMachineNm,ESRunID);
#endif
  }
  if(*na > 128){
    printf("ESC::WARNING too many radial zones %d > 128\n",*na);
    ESNa=128;
  }
  else{
    ESNa	=*na;
  }
  ESCInitStr();
#ifdef SHMEM
  memcpy(lShm,(void*)&ESNa,kI);
  memcpy(lShm+kI,(void*)&ESNp,kI);
  memcpy(lShm+2*kI,(void*)&ESFp,kI);
#endif
  return;
}

void esc1_()
{
  ESDeInitEsc();
  ESDeInitEc();
  DeInitSplines();
  DeInitLUdcmp();
}

void EscEq()
{
  int i;
  int Ffail,Fjob,Mpol;
  double sTime;
  void *lc;

  int k;
  lc	=lShm+kI;
  Ffail	=*(int*)lc;
  lc	+=kI;
  Fjob	=*(int*)lc;
  lc	+=kI;
  Mpol	=*(int*)lc;
  lc	+=kI;
  sTime	=*(double*)lc;

  if(2*(Mpol) < ESFp){
    ESMp	=Mpol;
    ESMp1	=ESMp+1;
    ES2Mp	=2*ESMp;
    ES2Mp1	=ES2Mp+1;
    ESnMp	=ESNa1*ES2Mp1;
  }
  else{
    printf("ESC::WARNING too many Fourier modes %d >= %d \n",Mpol,ESFp/2);
  }
  ESTime	=sTime; 
  if(ESiAx){
    ESReal2RefDataX(0);
    ESPlVdata2PlVX();
    ESGapCoreSepX();
    ESCoreX();
  }
  else{
    ESReal2RefData(0);
    ESReal2RefGeom(0);
    if(FlagPoints){
      ESPlVPoints2PlV(EcRjet,EcZjet,EcNjet);
    }
    else{
      ESPlVdata2PlV();
    }
    ESRef2RealGeom(0);
    if((Fjob&0x02)){
      ECRestoreESC(ESTime);
    }
  }
  ESRealGeom(0);
  ESReInitMetrTnsrCS();
  ECRcs3D2Rcs2D(0);
  ES3DMetrTnsrCS();
  if((Fjob&0x02)){
    Ffail	=0;
    if((ESFail&0x0000000F))
      Ffail	+=1;
    if((ESFail&0x000000F0) || (ESFail&0x0F000000))
      Ffail	+=8;
    if((ESFail&0x00F00000) || (ESFail&0x0000F000))
      Ffail	+=4;
    if((ESFail&0x000F0000) || (ESFail&0x00000F00))
      Ffail	+=2;
    return;
  }
  ECReInitFgY();
  ESInpPlPrCr();
  EC1DEqSolv();
  ESPlProf2Data();
  ECRescue2DGeom(0);
  do{
    if(EC2DEqSolv(0) == 0){
      ECgYcs2dRZcsBound(0);
      ECVirtualPlV2vdrPlV();
      ECgYrc2dRZcs();
      ECvdrz2vrz();
      ECCheckJacob(0);
    }
    else{
      ECRestore2DGeom();
    }
    ECRcs2D2Rcs3D(0);
    if(ESiAx != 0){
      EcMetrTnsrX();
    }
    ESRealGeom(0);
    if(ES3DMetrTnsrCS()){
      ESFail	=0x0FFFFFFF;
      break;
    }
    EC1DEqSolv();
  }while(ECIfEqFound());
  Ffail	=0;
  if((ESFail&0x0000000F))
    Ffail	+=1;
  if((ESFail&0x000000F0) || (ESFail&0x0F000000))
    Ffail	+=8;
  if((ESFail&0x00F00000) || (ESFail&0x0000F000))
    Ffail	+=4;
  if((ESFail&0x000F0000) || (ESFail&0x00000F00))
    Ffail	+=2;
  if(Ffail){
    ECRestore2DGeom();
    ECRcs2D2Rcs3D(0);
    if(ESiAx != 0){
      EcMetrTnsrX();
    }
    ESRealGeom(0);
    ES3DMetrTnsrCS();
    EC1DEqSolv();
    ECEqIterFl	=0;
  }
  else{
    ECEqIterFl	=1;
  }
  ECRefGeom(0);
  if((Fjob&0x01)){
    ECSaveESC();
  }
  if((Fjob&0x08)){
    ESBasicFunctions();
#ifdef H
    ESWriteESI0();
#endif
    ESWriteESI(NULL,ESsr,ESsz,ESaB,ESgH,ESNa,ESNp);
  }
  if((Fjob&4)){
    ESBalStab();
  }
  memcpy(lShm+kI,(void*)&Ffail,kI);
  return;
}

void EscIn()
{
  int i,k;
  static double *A=NULL;

  double *pProf,*jProf,*aProf;
  int nProf,hH,Npv,kP,kB;
  double *Rpv,*Zpv;
  double RBtor,Rext;
  char ch;
  void *lv;

  lv	=lShm+kI;
  ch	=*(char*)lv;
  lv++;
  nProf	=*(int*)lv;

  kP	=(nProf)*kD;
  lv	+=kI;
  if(ch == 'y'){
    aProf	=(double*)lv;
    lv	+=kP;
  }
  pProf	=(double*)lv;
  lv	+=kP;
  jProf	=(double*)lv;
  lv	+=kP;
  hH	=*(int*)lv;
  lv	+=kI;
  Npv	=*(int*)lv;
  lv	+=kI;
  kB	=(Npv)*kD;
  Rpv	=(double*)lv;
  lv	+=kB;
  Zpv	=(double*)lv;
  lv	+=kB;
  RBtor	=*(double*)lv;
  lv	+=kD;
  Rext	=*(double*)lv;
  cssr	=1.;
  if(Npv > 12){
    FlagPoints	=1;
    EcNjet	=Npv;
    for(i=0; i < EcNjet; i++){
      EcRjet[i]	=Rpv[i];
      EcZjet[i]	=Zpv[i];
    }
  }
  else{
    FlagPoints	=0;
  }
  
  i	=(hH)/10;
  if(ESEqSolvInPr != i){
    np	=0;
    ESEqSolvInPr	=i;
    switch(i){
    case 0:	/* convert jp [MA/m^2] to gm_0 jp */
    case 2:	/* convert p [MPa] to gm_0 p */
      cssp	=cgm0;
      break;
    default:
      cssp	=1.;
      break;
    }
  }
  k	=(hH)%10;
  i	=k != 6 ? k : 7;
  if(ESEqSolvInCr != i){
    np	=0;
    ESEqSolvInCr	=i;
    switch(i){
    case 0:	/* convert js [MA/m^2] to gm_0 js */
    case 1:	/* convert j|| [MA/m^2] to gm_0 j|| */
      cssj	=cgm0;
      break;
    default:
      cssj	=1.;
      break;
    }
  }
  if(FlStart || np == 0){
    ESGetPlPrfAddr(&ydPr,&ydCr,&xdPr,&xdCr,&kdPr,&kdCr);
  }

  ESRext	=Rext;
  ESRBt		=RBtor;
  if(FlStart){
    ch2hp	=cr12*ESgt[1]*ESgt[1];
    crHp	=1./ESNp;
    npv		=Npv < 13 ? Npv : 12;
    mpv		=FlagPoints ? 8 : npv/2+1;
    ESSetPlVInt(npv,mpv);
    ESEqSolvFl	=0x31;
    ESEqSolvIt	=20;
    ESEqSolvRadC=2;
    for(i=0; i < ESNa1; i++){
      ESaF[i]	=ESRBt;
      ESaF1a[i]	=0;
      ESaF2a[i]	=0;
    }
  }
  for(i=0; i < npv; i++){
    rdPV[i]	=Rpv[i]*cssr;
    zdPV[i]	=Zpv[i]*cssr;
  }
  if(FlStart){
    if(FlagPoints){
      double zmx,zmn,rmx,rmn;
      zmx	=EcZjet[0];
      zmn	=zmx;
      rmx	=EcRjet[0];
      rmn	=rmx;
      for(i=1;  i < EcNjet; i++){
	if(zmx < EcZjet[i]){
	  zmx	=EcZjet[i];
	}
	if(zmn > EcZjet[i]){
	  zmn	=EcZjet[i];
	}
	if(rmx < EcRjet[i]){
	  rmx	=EcRjet[i];
	}
	if(rmn > EcRjet[i]){
	  rmn	=EcRjet[i];
	}
      }
      ESaR0[0]	=cr2*(0.9*rmn+1.1*rmx);
      ESaZ0[0]	=cr2*(zmx+zmn);
      rdPV[0]	=cr2*(rmn+rmx);
      zdPV[0]	=zmx;
      rdPV[1]	=cr2*(rmn+rmx);
      zdPV[1]	=zmn;
      rdPV[2]	=rmn;
      zdPV[2]	=cr2*(zmx+zmn);
      rdPV[3]	=rmx;
      zdPV[3]	=cr2*(zmx+zmn);
    }
    else{
      ESaR0[0]	=cr2*(0.9*Rpv[2]+1.1*Rpv[3]);
      ESaZ0[0]	=cr2*(Zpv[0]+Zpv[1]);
      if(ESiAx != 0){
	ESaZ0[0]	+=0.2*cr2*(Zpv[0]-Zpv[1]);
      }
    }
    ESInit2DPlGeom();
    EcInitPlVacData();
    ECRcs3D2Rcs2D(0);
    ECvr2r(0);
    ECvz2z(0);
    ESSwitchInpRadC();
    FlStart	=0;
  }

  if(np != nProf){
    double dx;
    np	=nProf;
    nj	=np;
    ESnDtPr	=np-1;
    ESnDtCr	=nj-1;
    dx		=1./ESnDtPr;
    for(i=0; i < np; i++){
      xdPr[i]	=dx*i;
      xdCr[i]	=dx*i;
      ESxDtPr[i]=xdPr[i];
      ESxDtCr[i]=xdCr[i];
      kdPr[i]	=1;
      kdCr[i]	=1;
    }
  }
  if(aProf != NULL){
    for(i=0; i < np; i++){
      xdPr[i]	=aProf[i];
      xdCr[i]	=aProf[i];
      ESxDtPr[i]=aProf[i];
      ESxDtCr[i]=aProf[i];
    }
    ESFirstSetSplDPr();
    ESFirstSetSplDCr();
  }
  else{
    if(A != NULL){
      double dx;
      dx	=1./ESnDtPr;
      for(i=0; i < np; i++){
	xdPr[i]	=dx*i;
	xdCr[i]	=dx*i;
	ESxDtPr[i]=xdPr[i];
	ESxDtCr[i]=xdCr[i];
      }
      ESFirstSetSplDPr();
      ESFirstSetSplDCr();
    }
  }
  A	=aProf;
  for(i=0; i < np; i++){
    ydPr[i]	=pProf[i]*cssp;
    ydCr[i]	=jProf[i]*cssj;
  }
  if(k == 6){
    for(i=0; i < np; i++){
      ydCr[i]	=1./jProf[i];
    }
  }
  return;
}

void E2Astra()
{
  int i,kA;
  double rR0,R0;
  int j,ji,k,ki,kj;
  double b,ba;
  double ra,rgt,za,zgt;
  double *ld;
  double Sside[ESNa1],Sside1[ESNa1];

  kA	=kD*ESNa1;
  ld	=(double*)(lShm+kI);
  *ld++	=ESaR0[0];
  *ld++	=ESaZ0[0];
  *ld++	=ESgF[ESNa]*c2gp;
  memcpy((void*)ld,(void*)ESaF,kA);
  ld	+=ESNa1;
  memcpy((void*)ld,(void*)ESgm,kA);
  ld	+=ESNa1;

  R0		=ESaR0[0];
  rR0		=1./R0;
  /* G22 */
  for(i=0; i < ESNa1; i++){
    *ld++	=ESsa[i]*ESg22c[i];
  }
  /* G33 */
  for(i=0; i < ESNa1; i++){
    *ld++	=ESsa[i]*ESLc[i]*rR0;
  }
  /* D2 */
  for(i=0; i < ESNa1; i++){
    *ld++	=ESsa[i]*ESDPc[i];
  }
  /* D3 */
  for(i=0; i < ESNa1; i++){
    *ld++	=ESsa[i]*R0*(ESLc[i]+ESVc[i]);
  }

  Sside[0]	=0.;
  Sside1[0]	=0.;
  for(i=1; i < ESNa1; i++){
    b		=ESsb[i];
    ba		=ESsb1a[i];
    ki		=i;
    rca[0]	=rcT1a[i];
    rsa[0]	=rsT1a[i];
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      rca[k]	=2.*rcT1a[ki];
      rsa[k]	=2.*rsT1a[ki];
      rc[k]	=2.*k*rcT[ki];
      rs[k]	=2.*k*rsT[ki];
    }
    ji	=ESNp1*i;
    for(j=0; j < ESNp; j++){
      ra	=rca[0];
      rgt	=0.;
      kj	=0;
      for(k=1; k < ESFp1; k++){
	kj	+=j;
	if(kj >= ESNp)
	  kj	-=ESNp;
	ra	+=rca[k]*EScs1[kj]+rsa[k]*ESsn1[kj];
	rgt	+=-rc[k]*ESsn1[kj]+rs[k]*EScs1[kj];
      }
      zgt	=b*EScs1[j];
      za	=rsa[0]+ba*ESsn1[j];
      as[j]	=rgt*rgt+zgt*zgt;
      as1[j]	=sqrt(rgt*rgt+zgt*zgt)*ESsr[ji];
      as[j]	*=ESsr[ji]/(ra*zgt-rgt*za);
      ji++;
    }
    as[j]	=as[0];
    as1[j]	=as1[0];
    splP(as,as2p);
    splP(as1,as12p);
    b	=0.;
    ba	=0.;
    for(j=0; j < ESNp; j++){
      b		+=as[j]-ch2hp*as2p[j];
      ba	+=as1[j]-ch2hp*as12p[j];
    }
    Sside[i]	=b*crHp;
    Sside1[i]	=ba*crHp;
  }
  memcpy((void*)ld,(void*)Sside,kA);
  ld	+=ESNa1;
  memcpy((void*)ld,(void*)Sside1,kA);
  ld	+=ESNa1;
  return;
}
int ESSendProfile()
{
  int i,k,n;
  double dx;
  double *ld,*lp,*lp2;
  int *li;

  li	=(int*)(lShm+kI);
  ld	=(double*)li;
  k	=*li++;
  n	=*li++;

  lp	=lProf[k];
  lp2	=lProf2[k];

  dx 	=ESsa[ESNa]/(n-1);
  for(i=0; i < n; i++){
    ESSetSplA(dx*i);
    splRA(ld,NULL,lp,lp2);
    ld++;
  }
  return(0);
}

int ESSendFourierR()
{
  int I,ic,inc;
  int i,j,k,n;
  int nfmax,mi;
  double dx;
  double *ld;
  int *li;

  li	=(int*)(lShm+kI);
  ld	=(double*)li;

  k	=*li++;
  n	=*li++;

  nfmax =k > ESFp1 ? ESFp1 : k;

  dx =ESsa[ESNa]/(n-1);

  inc	=2*k*kD;
  I	=SHMEMLENGTH-inc;
  ic	=0;
  for(i=0; i < n; i++){
    ESSetSplA(dx*i);
    mi  =0;
    splRA(ld,NULL,rcT,rcT2a);
    *ld++	+=ESaR0[0];
    *ld++	 =0.0;
    for(j=1; j < nfmax; j++){
      mi	+=ESNa1;
      splRA(ld,NULL,rcT+mi, rcT2a+mi);
      *ld++ *=2.0;
      splRA(ld,NULL,rsT+mi, rsT2a+mi);
      *ld++ *=2.0;
    }
    for(j=nfmax; j < k; j++){
      *ld++	=0.0;
      *ld++	=0.0;
    }
    ic	+=inc;
    if(ic > I){
      ld	=(double*)(lShm+kI);
      ic	=0;
      semop(SemID, &bufV, 1);   		   
      semop(SemID, &bufP, 1);   
    }
  }
}

int ESSendFourierZ()
{
  int i,j,k,n;
  double dx;
  double *ld;
  int *li;

  li	=(int*)(lShm+kI);
  ld	=(double*)li;

  k	=*li++;
  n	=*li++;
  dx =ESsa[ESNa]/(n-1);
  for(i=0; i < n; i++){
    ESSetSplA(dx*i);
    splRA(ld,NULL,rsT,rsT2a);
    *ld++	+=ESaZ0[0];
    splRA(ld,NULL,ESsb,ESsb2a);
    *ld++;
  }
}

int main(int argc, char **argv)
{
  int Fl=0;
  int Pid;
  int na;
  char ln[32];
  int i,k;
  char elem;
  union semun Mysemun;
  
  sscanf(argv[1],"%d",&Pid);

  /* Init Shared memory */
  ShmID	=shmget((key_t)(Pid+1), 0, 0666|IPC_CREAT);

  /* Init semaphores */
  /* Two semaphores (Empty and Full) are created in one set */
  SemID	=semget((key_t)Pid, 0,0666|IPC_CREAT);
  /* attach shared memory to process */
  lShm	=shmat(ShmID,NULL,0);
  lRetFl	=(int*)lShm;
  *lRetFl	=0;
  lShm	+=sizeof(int);
  
#ifdef H
  printf("Esc: SemID=%d %d %lx\n",SemID,ShmID,lShm);
#endif
  semop(SemID, &bufP, 1);   

  esc00_(argv[1],&na);

  semop(SemID, &bufV, 1);   		   

  ESUpDateCleanFile();
  while(1){
    semop(SemID, &bufP, 1);   
    Fl	=*(int*)lShm;
    switch(Fl){
    case 1:
      EscIn();
      *lRetFl	=1;
      semop(SemID, &bufV, 1);   		   
      semop(SemID, &bufP, 1);   
      break;
    case 2:
      EscEq();
      *lRetFl	=2;
      semop(SemID, &bufV, 1);   		   
      semop(SemID, &bufP, 1);   
      break;
    case 3:
      E2Astra();
      *lRetFl	=3;
      semop(SemID, &bufV, 1);   		   
      semop(SemID, &bufP, 1);   
      break;
    case 5:
      ESSendProfile();
      *lRetFl	=5;
      semop(SemID, &bufV, 1);   		   
      semop(SemID, &bufP, 1);   
      break;
    case 6:
      ESSendFourierR();
      *lRetFl	=6;
      semop(SemID, &bufV, 1);   		   
      semop(SemID, &bufP, 1);   
      break;
    case 7:
      ESSendFourierZ();
      *lRetFl	=7;
      semop(SemID, &bufV, 1);   		   
      semop(SemID, &bufP, 1);   
      break;
    case 4:
      *lRetFl	=4;
      semop(SemID, &bufV, 1);   		   
      esc1_();
      ESRestoreCleanFile();
      exit(0);
      break;
    default:
      break;
    }
    semop(SemID, &bufV, 1);   		   
  }

#ifdef H
  i=0;
  while(i < 26){
    /* Wait(Full) */
    semop(SemID, &bufP, 1);   
    for(k=0; k < 10 && i < 26; k++){
      /* Get and consume element, Example element are chars 'a'-'z' */
      elem	=*((char *)lPtr+(i%NUM_ELEM));
      printf("                             Consumed Elem '%c'\n", elem);
      i++;
    }
    /* Signal(Empty) */
    semop(SemID, &bufV, 1);   		   
  }
#endif
  exit(0);
}

#else
#include <wait.h>
int main(int argc, char **argv)
{
  pid_t pid;
  int Fl=0;
  int Pid;
  int na;
  char ln[32];
  int i,k;
  char elem;
  union semun Mysemun;
  
  strcpy(ln,argv[1]);
  sscanf(argv[2],"%d",&Pid);
  sscanf(argv[3],"%d",&na);

  /* Init Shared memory */
  ShmID	=shmget((key_t)(Pid+1), 0, 0666|IPC_CREAT);

  /* Init semaphores */
  /* Two semaphores (Empty and Full) are created in one set */
  SemID	=semget((key_t)Pid, 0,0666|IPC_CREAT);
  /* attach shared memory to process */
  lShm	=shmat(ShmID,NULL,0);
  lRetFl	=(int*)lShm;
  *lRetFl	=0;
  lShm	+=sizeof(int);

  pid = fork();
  if (pid == -1) {
    fprintf(stderr,"fork failed\n");
    exit(1);
  }
  if (pid == 0) {/* Child */
    if(execlp("escChild","esc0",argv[1],argv[2],argv[3],NULL) == -1){
      fprintf(stderr,"execlp failed\n");
      exit(0);
    }
  }
  if (pid > 0) {/* parent */
    int status;
    wait(&status);
    if(WIFEXITED(status) == 0){
      *lRetFl	=-1;
      semop(SemID, &bufV, 1);   		   
    }

    printf("status=%d %d: %s\n",status,WIFEXITED(status)
	   ,WIFEXITED(status) ? "Normal" : "Crash");
    exit(0);
  }
}
#endif /*FORK*/
