#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "esc.h"

#ifndef mzl_ESmain
extern double cr2,cr3,cr4,cr6,cr12,cgp,c2gp,cgp_4,cr2gp,cgm0,crgm0;
extern int ESmemlev;
#endif

#ifndef mzl_1Dcore
#endif

#ifndef mzl_2Dcore
extern int ESNp,ESNp1;
extern int *ESnF,*ESmF,*ESkF,ESNf,ESNf1;
extern double R0,Z0,*r,*z,*rc1,*rs1,*r2,*z2;
extern double *dra,*drt,*dza,*dzt;
#endif

#ifndef mzl_3Dcore
extern int ESLt,ESNt,ESNt1;
extern double *gf1T,*csLT,*snLT,*cs1T,*sn1T;
#endif

#ifndef mzl_TFC
extern int TFCLt,TFCNt,TFCNp,TFCNp1;
extern double *xTFC,*yTFC,*zTFC,*gfTFC,*rTFC,ITFC;

double ESBgf;

static int NTP=0,nTP=0,NT=0,nT=0,LT=0;
static double *hTFC,*gf0TFC;
static double *gfL,*csL,*snL;
static double Rmn,Rmx,Zmn,Zmx,Hmn,Hmx;

static int NPncr=0,nPncr=0,nPsrf=1,KPncr=1,iPncr=0,LPncr;
static double *rPncr,*zPncr,*xPncr,*yPncr,gfPncr0,gfPncr1;

static double Bextz=0.,Bgf;

double Ax,Ay,Az,Bx,By,Bz;

#endif

#ifndef mzl_Lsode
static int lrw=68,liw=20;
static double *yeq,*rwork,*yh,*acor,*vtol,*rvtol;
static int iwork[20],istate,Neq=0,neq=3,itol=1,itask=4,mf=10,iopt=0;
static double ErrR=1e-8,ErrA=1e-12;

int ESReInitLsode()
{
  if(Neq < lrw+2*neq){
    lrw		=liw+16*neq;
    if(ReInitArray((void**)&rwork,Neq,lrw+3*neq,sizeof(double)) < 0){
      FailureAlarm((char*)rwork,"ESReInitLsode() - no memory for rwork");
      ESexit(0);
    }
    Neq	=lrw+2*neq;
    yh		=rwork+20;
    acor	=rwork+lrw-neq;
    vtol	=rwork+lrw;
    rvtol	=vtol+neq;
    yeq		=rvtol+neq;
    ESmemlev	|=0x00010000;
  }
  return(0);
}

int ESDeInitLsode()
{
  free(rwork);
  return(0);
}
#endif

#ifndef mzl_TFC
#ifndef stg_DataTFC
int ESGetTFCAddr(double **gf0,double **gfL0,double **csL0,double **snL0)
{
  *gf0=gf0TFC;
  *gfL0=gfL;
  *csL0=csL;
  *snL0=snL;
  return(0);
}

int ESReInit1DTFC()
{
  if(NT < TFCNt){
    if(ReInitArray((void**)&gf0TFC,NT,TFCNt+1,sizeof(double)) < 0){
      FailureAlarm((char*)gf0TFC,"ESReInit1DTFC() - no memory for gf0TFC");
      ESexit(0);
    }
    NT =TFCNt+1;
    ESmemlev |=0x00004000;
  }
  return(0);
}

int ESDeInit1DTFC()
{
  if(NT){
    free(gf0TFC);
    NT=0;
  }
  return(0);
}

int ESInit1LTFC()
{
  int L;
  double t,dt;

  dt	=c2gp/abs(ESLt);
  for(L=0; L < abs(ESLt); L++){
    t		=L*dt;
    gfL[L]	=t;
    csL[L]	=cos(t);
    snL[L]	=sin(t);
  }
  gfL[L]	=c2gp;
  csL[L]	=1.;
  snL[L]	=0.;
  return(0);
}

int ESInit3DTFC()
{
  int i,n,in;
  double cgp05,x,y,z,gf,r,cs,sn,X,Y;

  cgp05	=cr2*cgp;
  Rmn=1e+20;
  Rmx=-Rmn;
  Zmn=Rmn;
  Zmx=Rmx;
  Hmn=Rmn;
  Hmx=Rmx;
  in=0;
  for(n=0; n < TFCNt; n++){
    x	=0.;
    y	=0.;
    for(i=0; i < TFCNp; i++){
      x	+=xTFC[in];
      y	+=yTFC[in];
      in++;
    }
    in++;
    if(x == 0. && y == 0.){
      gf=0.;
    }
    else{
      if(fabs(y) < fabs(x)){
	gf	=atan(y/x);
	if(x < 0.){
	  gf	+=cgp;
	}
      }
      else{
	gf	=cgp05-atan(x/y);
	if(y < 0.){
	  gf	+=cgp;
	}
      }
    }
    gf0TFC[n]	=n && gf < gf0TFC[n-1] ? gf+c2gp : gf;
  }
  in	=0;
  for(n=0; n < TFCNt; n++){
    cs	=cos(gf0TFC[n]);
    sn	=sin(gf0TFC[n]);
    for(i=0; i < TFCNp1; i++){
      x	=xTFC[in];
      y	=yTFC[in];
      z	=zTFC[in];
      r	=sqrt(x*x+y*y);
      X	=x*cs+y*sn;
      Y	=-x*sn+y*cs;
      if(fabs(Y) < fabs(X)){
	gf	=atan(Y/X);
	if(X < 0.){
	  if(Y < 0.)
	    gf	-=cgp;
	  else
	    gf	+=cgp;
	}
      }
      else{
	gf	=cgp05-atan(X/Y);
	if(Y < 0.){
	  gf	-=cgp;
	}
      }
      gfTFC[in]	=gf+gf0TFC[n];
      rTFC[in]	=r;
      if(Rmn > r){
	Rmn=r;
      }
      if(Rmx < r){
	Rmx=r;
      }
      if(Zmn > z){
	Zmn=z;
      }
      if(Zmx < z){
	Zmx=z;
      }
      in++;
    }
  }
  for(n=0; n <TFCNt; n++){
    in=TFCNp1*n;
    gf=0;
    for(i=0; i < TFCNp1; i++){
      gf =gfTFC[in]-gf0TFC[n];
      hTFC[in]=rTFC[in]*sin(gf);
      if(Hmn > hTFC[in]){
	Hmn=hTFC[in];
      }
      if(Hmx < hTFC[in]){
	Hmx=hTFC[in];
      }
      in++;
    }
  }
  gf0TFC[n]	=gf0TFC[0]+c2gp/abs(ESLt);
  return(0);
}

int ESReInit3DTFC()
{
  if(NTP < nTP){
    if(ReInitArray((void**)&hTFC,NTP,nTP,sizeof(double)) < 0){
      FailureAlarm((char*)hTFC,"ESReInit3DTFC() - no memory for hTFC");
      ESexit(0);
    }
    if(ReInitArray((void**)&gfTFC,NTP,nTP,sizeof(double)) < 0){
      FailureAlarm((char*)gfTFC,"ESReInit3DTFC() - no memory for gfTFC");
      ESexit(0);
    }
    if(ReInitArray((void**)&rTFC,NTP,nTP,sizeof(double)) < 0){
      FailureAlarm((char*)rTFC,"ESReInit3DTFC() - no memory for rTFC");
      ESexit(0);
    }
    if(ReInitArray((void**)&zTFC,NTP,nTP,sizeof(double)) < 0){
      FailureAlarm((char*)zTFC,"ESReInit3DTFC() - no memory for zTFC");
      ESexit(0);
    }
    if(ReInitArray((void**)&yTFC,NTP,nTP,sizeof(double)) < 0){
      FailureAlarm((char*)yTFC,"ESReInit3DTFC() - no memory for yTFC");
      ESexit(0);
    }
    if(ReInitArray((void**)&xTFC,NTP,nTP,sizeof(double)) < 0){
      FailureAlarm((char*)xTFC,"ESReInit3DTFC() - no memory for xTFC");
      ESexit(0);
    }
    NTP =nTP;
    ESmemlev |=0x00002000;
  }
  return(0);
}

int ESDeInit3DTFC()
{
  if(NTP){
    free(xTFC);
    free(yTFC);
    free(zTFC);
    free(rTFC);
    free(gfTFC);
    free(hTFC);
    NTP=0;
  }
  return(0);
}

int ESInitTFC()
{
  int i;
  static int iT=0;
  double s,*pr,*ph,*pz;

  if(ESLt == 0){
    return(0);
  }
#ifndef AscI_TFC
#undef Ins1_TFC
  if(nTP == 0){
    return(-1);
  }
  TFCLt=abs(ESLt);
  TFCNp1=TFCNp+1;;
  TFCNt=nTP/TFCNp1;

  if(NT < TFCNt){
    ESReInit1DTFC();
  }
  if(NTP < nTP){
    ESReInit3DTFC();
  }
#undef Ins2_TFC
#endif /*AscI_TFC*/
  ESInit3DTFC();
#ifndef Tbl_TFC
  if(iT < 0)
    iT=0;
  if(iT >= TFCNt)
    iT=TFCNt-1;
  pr	=rTFC+TFCNp1*iT;
  ph	=hTFC+TFCNp1*iT;
  pz	=zTFC+TFCNp1*iT;
#endif /*Tbl_TFC*/
  if(LT < TFCLt+1){
    ESReInit1LTFC();
  }
  ESInit1LTFC();
  return(0);
}
#endif

#ifndef stg_VacMF
int ESReInit1LTFC()
{
  if(LT < TFCLt+1){
    if(ReInitArray((void**)&gfL,LT,TFCLt+1,sizeof(double)) < 0){
      FailureAlarm((char*)gfL,"ESReInit1LTFC() - no memory for gfL");
      ESexit(0);
    }
    if(ReInitArray((void**)&csL,LT,TFCLt+1,sizeof(double)) < 0){
      FailureAlarm((char*)csL,"ESReInit1LTFC() - no memory for csL");
      ESexit(0);
    }
    if(ReInitArray((void**)&snL,LT,TFCLt+1,sizeof(double)) < 0){
      FailureAlarm((char*)snL,"ESReInit1LTFC() - no memory for snL");
      ESexit(0);
    }
    LT =TFCLt+1;
    ESmemlev |=0x00008000;
  }
  return(0);
}

int ESDeInit1LTFC()
{
  if(LT){
    free(snL);
    free(csL);
    free(gfL);
    LT=0;
  }
  return(0);
}

int ESReInitPncr()
{
  if(NPncr < nPncr){
    if(ReInitArray((void**)&rPncr,NPncr,nPncr,sizeof(double)) < 0){
      FailureAlarm((char*)rPncr,"ESReInitPncr() - no memory for rPncr");
      ESexit(0);
    }
    if(ReInitArray((void**)&zPncr,NPncr,nPncr,sizeof(double)) < 0){
      FailureAlarm((char*)zPncr,"ESReInitPncr() - no memory for zPncr");
      ESexit(0);
    }
    if(ReInitArray((void**)&xPncr,NPncr,nPncr,sizeof(double)) < 0){
      FailureAlarm((char*)xPncr,"ESReInitPncr() - no memory for xPncr");
      ESexit(0);
    }
    if(ReInitArray((void**)&yPncr,NPncr,nPncr,sizeof(double)) < 0){
      FailureAlarm((char*)yPncr,"ESReInitPncr() - no memory for yPncr");
      ESexit(0);
    }
    NPncr	=nPncr;
    ESmemlev	|=0x00020000;
  }
  return(0);
}

int ESDeInitPncr()
{
  free(zPncr);
  free(rPncr);
  return(0);
}

int ESGetPncrAddr(int *nPncr0,int *nPsrf0,double **rPncr0,double **zPncr0,
		  double **xPncr0,double **yPncr0)
{
  *nPncr0=nPncr;
  *nPsrf0=nPsrf;
  *rPncr0=rPncr;
  *zPncr0=zPncr;
  *xPncr0=xPncr;
  *yPncr0=yPncr;
  return(0);
}

int ESbVAbVBTFC(double X,double Y,double Z)
{
  int j,n,jn,L;
  double x01,y01,x02,y02,x1,y1,z1,x2,y2,z2,Ix,Iy,Iz,dL,
  R1,R2,Z1,Z2,R1x,R1y,R1z,R2x,R2y,R2z;
  double aL;
  
  Ax	=0.;
  Ay	=0.;
  Az	=0.;
  Bx	=0.;
  By	=0.;
  Bz	=0.;
  jn	=0;
  for(n=0; n < TFCNt; n++){
    x02	=xTFC[jn];
    y02	=yTFC[jn];
    z2	=zTFC[jn];
    jn++;
    for(j=1; j < TFCNp1; j++){
      x01	=x02;
      y01	=y02;
      z1	=z2;
      x02	=xTFC[jn];
      y02	=yTFC[jn];
      z2	=zTFC[jn];
      Iz	=z2-z1;
      R1z	=z1-Z;
      R2z	=z2-Z;
      for(L=0; L < TFCLt; L++){
	x1	=x01*csL[L]-y01*snL[L];
	y1	=x01*snL[L]+y01*csL[L];
	x2	=x02*csL[L]-y02*snL[L];
	y2	=x02*snL[L]+y02*csL[L];	
	Ix	=x2-x1;
	Iy	=y2-y1;
	dL	=1./sqrt(Ix*Ix+Iy*Iy+Iz*Iz);
	R1x	=x1-X;
	R1y	=y1-Y;
	R1	=sqrt(R1x*R1x+R1y*R1y+R1z*R1z);
	Z1	=(Ix*R1x+Iy*R1y+Iz*R1z)*dL;
	R2x	=x2-X;
	R2y	=y2-Y;
	R2	=sqrt(R2x*R2x+R2y*R2y+R2z*R2z);
	Z2	=(Ix*R2x+Iy*R2y+Iz*R2z)*dL;
	aL	=log((Z2+R2)/(Z1+R1))*dL;
	Ax	+=Ix*aL;
	Ay	+=Iy*aL;
	Az	+=Iz*aL;
	aL	=dL/(R2*(R2+Z2))-dL/(R1*(R1+Z1));
	Bx	+=(Iy*R1z-Iz*R1y)*aL;
	By	+=(Iz*R1x-Ix*R1z)*aL;
	Bz	+=(Ix*R1y-Iy*R1x)*aL;
      }
      jn++;
    }
  }
  aL	=0.1*ITFC;
  Ax	*=aL;
  Ay	*=aL;
  Az	*=aL;
  Bx	*=aL;
  By	*=aL;
  Bz	*=aL;
  Bz	+=Bextz;
  Bgf	=(-Bx*Y+By*X)/sqrt(X*X+Y*Y);
  Bgf	=-Bx*Y+By*X; /*  $rB_\ph$ */
  return(0);
}

int ESbVAbVBTFC0(double *BR,double *BF,double *BZ,double R,double Z,int N)
{

  double X,Y;
  int j,n,jn,L;
  double x01,y01,x02,y02,x1,y1,z1,x2,y2,z2,Ix,Iy,Iz,dL,
  R1,R2,Z1,Z2,R1x,R1y,R1z,R2x,R2y,R2z;
  double aL;
  

  X	=R*cs1T[N];
  Y	=R*sn1T[N];

  Ax	=0.;
  Ay	=0.;
  Az	=0.;
  Bx	=0.;
  By	=0.;
  Bz	=0.;
  jn	=0;
  for(n=0; n < TFCNt; n++){
    x02	=xTFC[jn];
    y02	=yTFC[jn];
    z2	=zTFC[jn];
    jn++;
    for(j=1; j < TFCNp1; j++){
      x01	=x02;
      y01	=y02;
      z1	=z2;
      x02	=xTFC[jn];
      y02	=yTFC[jn];
      z2	=zTFC[jn];
      Iz	=z2-z1;
      R1z	=z1-Z;
      R2z	=z2-Z;
      for(L=0; L < TFCLt; L++){
	x1	=x01*csL[L]-y01*snL[L];
	y1	=x01*snL[L]+y01*csL[L];
	x2	=x02*csL[L]-y02*snL[L];
	y2	=x02*snL[L]+y02*csL[L];	
	Ix	=x2-x1;
	Iy	=y2-y1;
	dL	=1./sqrt(Ix*Ix+Iy*Iy+Iz*Iz);
	R1x	=x1-X;
	R1y	=y1-Y;
	R1	=sqrt(R1x*R1x+R1y*R1y+R1z*R1z);
	Z1	=(Ix*R1x+Iy*R1y+Iz*R1z)*dL;
	R2x	=x2-X;
	R2y	=y2-Y;
	R2	=sqrt(R2x*R2x+R2y*R2y+R2z*R2z);
	Z2	=(Ix*R2x+Iy*R2y+Iz*R2z)*dL;
	aL	=log((Z2+R2)/(Z1+R1))*dL;
	Ax	+=Ix*aL;
	Ay	+=Iy*aL;
	Az	+=Iz*aL;
	aL	=dL/(R2*(R2+Z2))-dL/(R1*(R1+Z1));
	Bx	+=(Iy*R1z-Iz*R1y)*aL;
	By	+=(Iz*R1x-Ix*R1z)*aL;
	Bz	+=(Ix*R1y-Iy*R1x)*aL;
      }
      jn++;
    }
  }
  aL	=0.1*ITFC;
  Ax	*=aL;
  Ay	*=aL;
  Az	*=aL;
  Bx	*=aL;
  By	*=aL;
  Bz	*=aL;
  Bz	+=Bextz;
  Bgf	=(-Bx*Y+By*X)/sqrt(X*X+Y*Y);
  Bgf	=-Bx*Y+By*X; /*  $rB_\ph$ */

  *BR	=Bx*cs1T[N]+By*sn1T[N];
  *BF	=-Bx*sn1T[N]+By*cs1T[N];
  *BZ	=Bz;

  return(0);
}

void ESMFLineTraceStep0(nq,x,yeq,dy)
     int *nq;
     double *x,*yeq,*dy;
{
  double X,Y,Z;
  double aX,aXz,aXy;
  double aY,aYx,aYz;
  double aZ,aZx,aZy;
  double F,Fx,Fy,Fz,rF;

  int j,n,jn,L;
  double csLL,snLL,x02,y02,x1,y1,z1,x2,y2,z2;
  double Ix,Iy,Iz,dL,R,rR,Z1,Rx,Ry,Rz;
  
  X	=yeq[0];
  Y	=yeq[1];
  Z	=yeq[2];

  aX	=0.;
  aXz	=0.;
  aXy	=0.;
  aY	=0.;
  aYx	=0.;
  aYz	=0.;
  aZ	=0.;
  aZx	=0.;
  aZy	=0.;
  for(L=0; L < TFCLt; L++){
    csLL	=csL[L];
    snLL	=snL[L];
    jn	=0;
    for(n=0; n < TFCNt; n++){
      x02	=xTFC[jn];
      y02	=yTFC[jn];
      x2	=x02*csLL-y02*snLL;
      y2	=x02*snLL+y02*csLL;	
      z2	=zTFC[jn];
      Rx	=x2-X;
      Ry	=y2-Y;
      Rz	=z2-Z;
      R		=sqrt(Rx*Rx+Ry*Ry+Rz*Rz);
      rR	=1./R;
      Rx	*=rR;
      Ry	*=rR;
      Rz	*=rR;
      jn++;
      for(j=1; j < TFCNp1; j++){
	z1	=z2;
	x1	=x2;
	y1	=y2;
	x02	=xTFC[jn];
	y02	=yTFC[jn];
	x2	=x02*csLL-y02*snLL;
	y2	=x02*snLL+y02*csLL;	
	z2	=zTFC[jn];
	Ix	=x2-x1;
	Iy	=y2-y1;
	Iz	=z2-z1;
	dL	=1./sqrt(Ix*Ix+Iy*Iy+Iz*Iz);
	Ix	*=dL;
	Iy	*=dL;
	Iz	*=dL;
	Z1	=(Ix*Rx+Iy*Ry+Iz*Rz)*R;
	rF	=1./(Z1+R);

	Fx	=(Rx+Ix)*rF;
	Fy	=(Ry+Iy)*rF;
	Fz	=(Rz+Iz)*rF;

	aXy   	+=Ix*Fy   ;
	aXz   	+=Ix*Fz   ;

	aYx   	+=Iy*Fx   ;
	aYz   	+=Iy*Fz   ;
	aZx   	+=Iz*Fx   ;
	aZy   	+=Iz*Fy   ;

	Rx	=x2-X;
	Ry	=y2-Y;
	Rz	=z2-Z;
	R		=sqrt(Rx*Rx+Ry*Ry+Rz*Rz);
	rR	=1./R;
	Rx	*=rR;
	Ry	*=rR;
	Rz	*=rR;
	Z1	=(Ix*Rx+Iy*Ry+Iz*Rz)*R;
	F		=log((Z1+R)*rF);
	aX	+=Ix*F;
	aY	+=Iy*F;
	aZ	+=Iz*F;
	rF	=1./(Z1+R);

	Fx	=(Rx+Ix)*rF;
	Fy	=(Ry+Iy)*rF;
	Fz	=(Rz+Iz)*rF;

	aXy   	-=Ix*Fy   ;
	aXz   	-=Ix*Fz   ;
	aYx   	-=Iy*Fx   ;
	aYz   	-=Iy*Fz   ;
	aZx   	-=Iz*Fx   ;
	aZy   	-=Iz*Fy   ;
	jn++;
      }
    }
  }
  R	=0.1*ITFC;
  Ax	=R*aX;
  Ay	=R*aY;
  Az	=R*aZ;
  Bx	=R*(aZy-aYz);
  By	=R*(aXz-aZx);
  Bz	=R*(aYx-aXy);
  R	=(X*X+Y*Y)/(By*X-Bx*Y);
  dy[0]	=R*Bx;
  dy[1]	=R*By;
  dy[2]	=R*Bz;
}

void ESMFLineTraceStep(nq,x,yeq,dy)
     int *nq;
     double *x,*yeq,*dy;
{
  int j,n,jn,L;
  double x01,y01,x02,y02,x1,y1,z1,x2,y2,z2,Ix,Iy,Iz,dL,
  R1,R2,Z1,Z2,R1x,R1y,R1z,R2x,R2y,R2z;
  double aL;
  
  Ax	=0.;
  Ay	=0.;
  Az	=0.;
  Bx	=0.;
  By	=0.;
  Bz	=0.;
  jn	=0;
  for(n=0; n < TFCNt; n++){
    x02	=xTFC[jn];
    y02	=yTFC[jn];
    z2	=zTFC[jn];
    jn++;
    for(j=1; j < TFCNp1; j++){
      x01	=x02;
      y01	=y02;
      z1	=z2;
      x02	=xTFC[jn];
      y02	=yTFC[jn];
      z2	=zTFC[jn];
      Iz	=z2-z1;
      R1z	=z1-yeq[2];
      R2z	=z2-yeq[2];
      for(L=0; L < TFCLt; L++){
	x1	=x01*csL[L]-y01*snL[L];
	y1	=x01*snL[L]+y01*csL[L];
	x2	=x02*csL[L]-y02*snL[L];
	y2	=x02*snL[L]+y02*csL[L];	
	Ix	=x2-x1;
	Iy	=y2-y1;
	dL	=1./sqrt(Ix*Ix+Iy*Iy+Iz*Iz);
	R1x	=x1-yeq[0];
	R1y	=y1-yeq[1];
	R1	=sqrt(R1x*R1x+R1y*R1y+R1z*R1z);
	Z1	=(Ix*R1x+Iy*R1y+Iz*R1z)*dL;
	R2x	=x2-yeq[0];
	R2y	=y2-yeq[1];
	R2	=sqrt(R2x*R2x+R2y*R2y+R2z*R2z);
	Z2	=(Ix*R2x+Iy*R2y+Iz*R2z)*dL;
	aL	=log((Z2+R2)/(Z1+R1))*dL;
	Ax	+=Ix*aL;
	Ay	+=Iy*aL;
	Az	+=Iz*aL;
	aL	=dL/(R2*(R2+Z2))-dL/(R1*(R1+Z1));
	Bx	+=(Iy*R1z-Iz*R1y)*aL;
	By	+=(Iz*R1x-Ix*R1z)*aL;
	Bz	+=(Ix*R1y-Iy*R1x)*aL;
      }
      jn++;
    }
  }
  aL	=0.1*ITFC;
  Ax	*=aL;
  Ay	*=aL;
  Az	*=aL;
  Bx	*=aL;
  By	*=aL;
  Bz	*=aL;
  Bz	+=Bextz;
#ifdef H
  R1	=-Bx*sin(*x)+By*cos(*x);
#endif
  Bgf	=-Bx*yeq[1]+By*yeq[0]; /*  $rB_\ph$ */
  R1	=(yeq[0]*yeq[0]+yeq[1]*yeq[1])/Bgf;
  dy[0]	=R1*Bx;
  dy[1]	=R1*By;
  dy[2]	=R1*Bz;
}

int ESMFLineTrace(int Fl)
{
  static int i,ii,k,kP;
  static double gf0,gf1,dgf,RBgf0,ddY[3];
  void ESMFLineSplStep();

  switch(Fl){
  case 1:
    gf0	=0.;
    gf1	=0.;
    i	=iPncr;
    yeq[0]=rPncr[i];
    yeq[1]=0.;
    yeq[2]=zPncr[i];
    i++;
    ii	=1;
    k	=0;
    dgf	=0.;
    ESMFLineSplStep(&neq,&gf0,yeq,ddY);
    printf("??? ESBgf=%10.3e\n",ESBgf);
    
    RBgf0=fabs(ESBgf);
    kP	=iPncr+KPncr;
    break;
  case 2:
    kP	=nPncr;
    break;
  case 3:
    gf0	=0.;
    gf1	=0.;
    dgf	=0.;
    k	=0;
    i	=iPncr;
    yeq[0]=rPncr[i];
    yeq[1]=0.;
    yeq[2]=zPncr[i];
    i++;
    ii	=1;
    kP	=iPncr+KPncr;
    break;
  }
#ifdef H
  iwork[4]	=3;
#endif
  while(i < kP){
    gf1	=dgf+gf1T[ii];
    lsode_(ESMFLineSplStep,&neq,yeq,&gf0,&gf1,&itol,&ErrR,&ErrA,&itask,
	   &istate,&iopt,rwork,&lrw,iwork,&liw,NULL,&mf);
    rPncr[i]	=yeq[0];
    zPncr[i]	=yeq[2];
    i++;
    ii++;
    if(ii == ESNt){
      ii=0;
      k++;
      dgf=c2gp*k;
    }
    if(fabs(ESBgf) < 0.25*RBgf0){
      printf("???? ESBgf[%d]=%10.3e < 0.25*%10.3e %10.3e %10.3e \n"
	     ,i-1,ESBgf,RBgf0,yeq[0],yeq[2]);
      nPncr	=i;
      return(1);
    }
  }
  printf("%6d Field callculations has been made in %6d steps ,ii=%d %d %d\n"
	 ,iwork[11],iwork[10],ii,nPncr,i);
  return(0);
}

int ESMFLineTrace0(int Fl)
{
  static int i,ii,k,kP;
  static double gf0,gf1,dgf,RBgf0;
  void ESMFLineTraceStep();

  switch(Fl){
  case 1:
    gf0	=0.;
    gf1	=0.;
    i	=iPncr;
    yeq[0]=rPncr[i];
    yeq[1]=0.;
    yeq[2]=zPncr[i];
    i++;
    ii	=1;
    k	=0;
    dgf	=0.;
    ESbVAbVBTFC(rPncr[0],0.,zPncr[0]);
    if(Bgf < 0.)
      ITFC	=-ITFC;
    RBgf0=fabs(Bgf);
    kP	=iPncr+KPncr;
    break;
  case 2:
    kP	=nPncr;
    break;
  case 3:
    gf0	=0.;
    gf1	=0.;
    dgf	=0.;
    k	=0;
    i	=iPncr;
    yeq[0]=rPncr[i];
    yeq[1]=0.;
    yeq[2]=zPncr[i];
    i++;
    ii	=1;
    kP	=iPncr+KPncr;
    break;
  }
  while(i < kP){
    gf1	=dgf+gf1T[ii];
    lsode_(ESMFLineTraceStep,&neq,yeq,&gf0,&gf1,&itol,&ErrR,&ErrA,&itask,
	   &istate,&iopt,rwork,&lrw,iwork,&liw,NULL,&mf);
    rPncr[i]	=sqrt(yeq[0]*yeq[0]+yeq[1]*yeq[1]);
    zPncr[i]	=yeq[2];
    i++;
    ii++;
    if(ii == ESNt){
      ii=0;
      k++;
      dgf=c2gp*k;
    }
    if(fabs(Bgf) < 0.25*RBgf0){
      nPncr	=i;
      return(1);
    }
  }
  printf("%6d Field callculations has been made in %6d steps ,ii=%d %d %d\n"
	 ,iwork[11],iwork[10],ii,nPncr,i);
  return(0);
}

int ESRectGr(double *R0, double *Z0,double *Rg, double *Zg)
{
  double hx,hy,wx,wy,h,w;
  hx	=R0[1]-R0[0];
  hy	=Z0[1]-Z0[0];
  h	=sqrt(hx*hx+hy*hy);
  hx	/=h;
  hy	/=h;
  wx	=R0[2]-R0[0];
  wy	=Z0[2]-Z0[0];
  w	=fabs(wx*hy-wy*hx);
  R0[2]	=R0[0]+w*hy;
  Z0[2]	=Z0[0]-w*hx;
  Rg[0]	=R0[0]-w*hy+h*hx;
  Zg[0]	=Z0[0]+w*hx+h*hy;
  Rg[1]	=R0[0]+w*hy+h*hx;
  Zg[1]	=Z0[0]-w*hx+h*hy;
  Rg[2]	=R0[0]+w*hy-h*hx;
  Zg[2]	=Z0[0]-w*hx-h*hy;
  Rg[3]	=R0[0]-w*hy-h*hx;
  Zg[3]	=Z0[0]+w*hx-h*hy;
  Rg[4]	=Rg[0];
  Zg[4]	=Zg[0];

  return(0);
}

int ESInitVacMF()
{
  int i;
  static int iT=0,iT1=1,kT=0,Fl=1,Fl1=0,nT,kPncr=0,kPsrf=1,kP=0,iPsrf;
  double *pr,*pz,*pr1,*pz1,X,Y,gF;
  static int Nsrf=11;
  double Rst[11],Zst[11],gFs,Ar,At,Br,Bt;

  if(ESLt == 0){
    return(0);
  }
  ESReInitLsode();
  ESReInitPncr();
  if(Fl){
    i	=TFCNp1*iT;
    Rst[0]=cr2*(rTFC[i]+rTFC[i+TFCNp/2]);
    Zst[0]=cr2*(zTFC[i]+zTFC[i+TFCNp/2]);
    Rst[kPsrf]=Rst[0]+0.05*(rTFC[i]-rTFC[i+TFCNp/2]);
    Zst[kPsrf]=Zst[0];
    for(i=kPsrf+1; i < Nsrf; i++){
      Rst[i]	=Rst[kPsrf];
      Zst[i]	=Zst[kPsrf];
    }
    gFs	=0.;
    Fl	=0;
  }
  LPncr	=ESNt/TFCLt;
#ifndef Tbl_VacMF
  nT	=ESNt;
  if(kT < 0)
    kT=0;
  while(kT >= ESNt){
    kT =ESNt-1;
  }
  gF	=gf0TFC[0];
  while(gF+gfL[1] <= gf1T[kT]){
    gF	+=gfL[1];
  }
  while(gF > gf1T[kT]){
    gF	-=gfL[1];
  }
  gF	=gf1T[kT]-gF+gf0TFC[0];
  iT1	=iT+1;
  while(iT < TFCNt && gf0TFC[iT1] <= gF){
    iT1++;
    iT++;
  }
  while(iT && gf0TFC[iT] > gF){
    iT1--;
    iT--;
  }
  pr	=rTFC+TFCNp1*iT;
  pz	=zTFC+TFCNp1*iT;
  if(iT1 != TFCNt){
    pr1	=rTFC+TFCNp1*iT1;
    pz1	=zTFC+TFCNp1*iT1;
  }
  else{
    pr1	=rTFC;
    pz1	=zTFC;
  }
  gF	=c2gp/ESNt*gFs;
  ESbVAbVBTFC(Rst[0]*cos(gF),Rst[0]*sin(gF),Zst[0]);
  Ar	= Ax*cos(gF)+Ay*sin(gF);
  At	= Rst[0]*(-Ax*sin(gF)+Ay*cos(gF));
  Br	= Bx*cos(gF)+By*sin(gF);
  Bt	=-Bx*sin(gF)+By*cos(gF);
  printf("=== R=%10.3e %10.3e %10.3e\n",Rst[0],gFs,Zst[0]);
  printf("??? Biot-Savart/Splines\n");
  printf("??? %13s %13s %13s %13s %13s %13s\n" ,"Ar","At","Az","Br","Bt","Bz");
  printf("??? %13.6e %13.6e %13.6e %13.6e %13.6e %13.6e\n",Ar,At,Az,Br,Bt,Bz);
  ESSplineArtz(Rst[0],gF,Zst[0],&Ar,&At,&Az,&Br,&Bt,&Bz);
  printf("??? %13.6e %13.6e %13.6e %13.6e %13.6e %13.6e\n",Ar,At,Az,Br,Bt,Bz);

  if(kPncr < 0)
    kPncr=0;
  if(kPsrf < 1)
    kPsrf=1;
  if(kPsrf > Nsrf-1)
    kPsrf=Nsrf-1;
  if(kPsrf > 1){
    if((Fl1%10) == 2){
      printf("Nsrf =%d and K tr=%d are incompatible\n",kPsrf,Fl1);
      Fl1	-=(Fl1%10);
      printf("K tr has been changed on %d%c\n",Fl1,7);
    }
    if(kPsrf != nPsrf){
      double dR,dZ;
      dR	=(Rst[kPsrf]-Rst[0])/kPsrf;
      dZ	=(Zst[kPsrf]-Zst[0])/kPsrf;
      for(i=1; i < kPsrf+1; i++){
	Rst[i]	=Rst[0]+dR*i;
	Zst[i]	=Zst[0]+dZ*i;
      }
      for(i=kPsrf+1; i < Nsrf; i++){
	Rst[i]	=Rst[kPsrf];
	Zst[i]	=Zst[kPsrf];
      }
    }
  }
  KPncr	=ESNt*kPncr+1;
  nPsrf	=kPsrf;
  if(kPncr && (Fl1%10)){
    gfPncr0	=0.;
    gfPncr1	=kPncr*c2gp;

    /*\bf========== input for lsode =================*/
    itask	=5;		/*take one step*/
    itask	=4;		/*give the output at x=xout*/
    rwork[0]	=gfPncr1+cgp;	/*limit for overshooting*/
    
    /*\bf========== optional input ==================*/
    iopt	=1; 
    rwork[4]	=cgp/ESNt;	/* guess for the step size*/
    rwork[5]	=cgp;		/* maximum step size*/
    rwork[6]	=0.;		/* minimum step size*/
    iwork[4]	=12;		/* maximum order allowed*/
    iwork[5]	=5000;		/* maximum number of internal steps allowed*/
    iwork[6]	=10;		/* number of messages printed*/
    switch((Fl1%10)){
    case 1:
      nPncr	=KPncr*nPsrf;
      if(NPncr < nPncr){
	ESReInitPncr();
      }
      iPncr	=0;
      for(iPsrf=1; iPsrf < nPsrf+1; iPsrf++){
	rPncr[iPncr]=Rst[iPsrf];
	zPncr[iPncr]=Zst[iPsrf];
	istate	=1;		/*first call of lsode()*/
	if((Fl1/10) == 0){
	  if(ESMFLineTrace0((Fl1%10))){
	    printf(" The MF-line seems to be unclosed%c\n",7);
	    Fl1=0;
	  }
	}    
	else{
	  if(ESMFLineTrace((Fl1%10))){
	    printf(" The MF-line seems to be unclosed%c\n",7);
	    Fl1=0;
	  }
	}
	iPncr	+=KPncr;
      }
      break;
    case 2:
      iPncr	=nPncr-1;
      nPncr	=KPncr*nPsrf;
      if(NPncr < nPncr){
	ESReInitPncr();
      }
      istate	=2;		/*second call of lsode()*/
      if((Fl1/10) == 0){
	if(ESMFLineTrace0((Fl1%10))){
	  printf(" The MF-line seems to be unclosed%c\n",7);
	  Fl1=0;
	}
      }    
      else{
	if(ESMFLineTrace((Fl1%10))){
	  printf(" The MF-line seems to be unclosed%c\n",7);
	  Fl1=0;
	}
      }
      break;
    case 3:
      iPncr	=nPncr-1;
      nPncr	=KPncr*nPsrf;
      if(NPncr < nPncr){
	ESReInitPncr();
      }
      istate	=1;		/*first call of lsode()*/
      rPncr[iPncr]=Rst[0];
      zPncr[iPncr]=Zst[0];
      if((Fl1/10) == 0){
	if(ESMFLineTrace0((Fl1%10))){
	  printf(" The MF-line seems to be unclosed%c\n",7);
	  Fl1=0;
	}
      }    
      else{
	if(ESMFLineTrace((Fl1%10))){
	  printf(" The MF-line seems to be unclosed%c\n",7);
	  Fl1=0;
	}
      }
      break;
    default:
      break;
    }
  }
  if(nPncr){
    int ii;
#undef Ins_VacMF
    kP	=0;
    for(iPsrf=0; iPsrf < nPsrf; iPsrf++){
      ii	=KPncr*iPsrf+kT;
      for(i=kT; i < KPncr; i +=LPncr){
	xPncr[kP]	=rPncr[ii];
	yPncr[kP]	=zPncr[ii];
	kP++;
	ii	+=LPncr;
      }
    }
  }
#endif /*Tbl_VacMF*/
  return(0);
}

#endif
#endif
