#ifndef Blb_esc.c
#ifdef Doc_Blb_esc.c
\section{Layoff of shared memory for ESC}
\ESC exchanges with other codes through the shared memory which has
the following laying off (where \c{Li,Ld} are the lentgh of \c{int,double})
\vskip1ex
\noindent\begin{tabular}{|llrrp{2.2in}|}\hline
ID 	& Byte		& Length	&Section & Comment\\\hline
\c{Fl}&\c{0 Li}	& \c{1 Li}	&
&Return flag\\
\c{Na}&\c{1 Li}	& \c{1 Li}	&\c{Machine} 
&number of radial intervals\\
\c{Np}&\c{2 Li}	& \c{1 Li}	&\c{Machine}
&number of poloidal intervals\\
\c{Fp}&\c{3 Li}	& \c{1 Li}	&\c{Machine}
&number of Fourier coefficients\\
\c{hH}&\c{4 Li}	& \c{1 Li}	&\c{PProf, CProf}
& Code of the profiles and radial\\
\c{Npv}&\c{5 Li}	& \c{1 Li}	&\c{PlVac}
& number of boundary data points coordinate\\
\hline
\c{nP}&\c{6 Li}	& \c{1 Li}	&\c{RadCoord}
& Number of profile data points\\
\c{Mp}&\c{7 Li}	& \c{1 Li}	&\c{RadCoord}
& Highest poloidal harmonics in $\bgY$\\
\c{Fjob}&\c{8 Li}	& \c{1 Li}	&
& Flag for equilibrium solver\\
\c{Ffail}&\c{9 Li}	& \c{1 Li}	&
& Error Flag of equilibrium solver\\
\hline
\c{Time}&\c{32 Li}	& \c{1 Ld}	&\c{PlVac}&$t$ reference time\\
\c{RBtor}&\c{\&Time+Ld}& \c{1 Ld}	&\c{PlVac}&$R_{ext}B_{tor}$ data\\
\c{Rext}&\c{\&Time+2 Ld}	& \c{1 Ld}	&\c{PlVac}
&$R_{ext}$ data - reference radius\\
\hline
\c{Rpv}&\c{\&Time+16 Ld}& \c{Npv Ld}	&\c{PlVac}
& boundary $r_i$ - data points\\
\c{Zpv}&\c{\&Rpv+Npv Ld}& \c{Npv Ld}	&\c{PlVac}
& boundary $z_i$ - data points\\
\hline
\c{a}	&\c{\&Rpv+384 Ld}	& \c{nP Ld}	&\c{RadCoord}
&$a_i$-radial grid data points\\
\c{p}	&\c{\&a+nP Ld}	& \c{nP Ld}	&\c{PProf}
&$p_i$-pressure related data\\
\c{j}	&\c{\&p+nP Ld}	& \c{nP Ld}	&\c{CProf}
&$j_i$-current related data\\
\hline
\c{esi}&\c{1024 Ld}	& \c{Np1+(12+16*Np1)*Na1}	&\c{Equil}
&\c{esi} interface output data\\
\hline
\end{tabular}
\vskip1ex
Temporary storage for ASTRA
\vskip1ex
\noindent\begin{tabular}{|llrrp{2.2in}|}\hline
ID 	& Byte		& Length	&Section & Comment\\\hline
\c{Rmaxis}&\c{esi[0]}	& \c{1 Ld}	&EqOut&$R_0$\\
\c{Zmaxis}&\c{esi[1]}	& \c{1 Ld}	&EqOut&$Z_0$\\\hline
\c{*ESsb}&\c{esi+2}	& \c{1 Ld}	&EqOut&$r$\\\hline
\c{*ESsr}&\c{esb+Na1}	& \c{1 Ld}	&EqOut&$r$\\
\c{*ESsz}&\c{ESsr+Np1*Na1}& \c{1 Ld}	&EqOut&$r$\\
\c{*rcT}&\c{ESsz+Np1*Na1}& \c{1 Ld}	&EqOut&$r$\\
\c{*rsT}&\c{rcT+Na1*Fp1}& \c{1 Ld}	&EqOut&$r$\\
\c{*rcT1a}&\c{rsT+Na1*Fp1}& \c{1 Ld}	&EqOut&$r$\\
\c{*rsT1a}&\c{rcT1a+Na1*Fp1}& \c{1 Ld}	&EqOut&$r$\\\hline
\c{gFtor}&\c{rsT1a+Na1*Fp1}& \c{1 Ld}	&EqOut&$\bgF_0$\\
\c{Fpol}&\c{gFtor+1}	& \c{1 Ld}	&EqOut&$\gm_i$\\
\c{*gM}&\c{Fpol+Na1}	& \c{Na1 Ld}	&EqOut&$\gm_i$\\
\c{*G22}&\c{gm+Na1}	& \c{Na1 Ld}	&EqOut&$G22_i$\\
\c{*G33}&\c{G22+Na1}	& \c{Na1 Ld}	&EqOut&$G33_i$\\
\c{*D2}&\c{G33+Na1}	& \c{Na1 Ld}	&EqOut&$D2_i$\\
\c{*D3}&\c{D2+Na1}	& \c{Na1 Ld}	&EqOut&$D3_i$\\
\c{*Sside}&\c{D3+Na1}	& \c{Na1 Ld}	&EqOut&$S_i$\\
\c{*Sside1}&\c{Sside+Na1}& \c{Na1 Ld}	&EqOut&$S1_i$\\
\hline
\end{tabular}

\vskip1ex
Temporary storage for TRANSP
\vskip1ex
\noindent\begin{tabular}{|llrrp{2.2in}|}\hline
ID 	& Byte		& Length	&Section & Comment\\\hline
\c{Rmaxis}&\c{esi[0]}	& \c{1 Ld}	&EqOut&$R_0$\\
\c{Zmaxis}&\c{esi[1]}	& \c{1 Ld}	&EqOut&$Z_0$\\\hline
\c{*ESsb}&\c{esi+2}	& \c{1 Ld}	&EqOut&$r$\\\hline
\c{*ESsr}&\c{esb+Na1}	& \c{1 Ld}	&EqOut&$r$\\
\c{*ESsz}&\c{ESsr+Np1*Na1}& \c{1 Ld}	&EqOut&$r$\\
\c{*rcT}&\c{ESsz+Np1*Na1}& \c{1 Ld}	&EqOut&$r$\\
\c{*rsT}&\c{rcT+Na1*Fp1}& \c{1 Ld}	&EqOut&$r$\\
\c{*rcT1a}&\c{rsT+Na1*Fp1}& \c{1 Ld}	&EqOut&$r$\\
\c{*rsT1a}&\c{rcT1a+Na1*Fp1}& \c{1 Ld}	&EqOut&$r$\\\hline
\c{gFtor}&\c{rsT1a+Na1*Fp1}& \c{1 Ld}	&EqOut&$\bgF_0$\\
\c{*gM}&\c{gFtor+1}	& \c{1 Ld}	&EqOut&$\gm_i$\\
\c{*gM}&\c{esi+3+Na1}	& \c{Na1 Ld}	&EqOut&$\gm_i$\\
\c{*G22}&\c{gm+Na1}	& \c{Na1 Ld}	&EqOut&$G22_i$\\
\c{*G33}&\c{G22+Na1}	& \c{Na1 Ld}	&EqOut&$G33_i$\\
\c{*D2}&\c{G33+Na1}	& \c{Na1 Ld}	&EqOut&$D2_i$\\
\c{*D3}&\c{D2+Na1}	& \c{Na1 Ld}	&EqOut&$D3_i$\\
\c{*Sside}&\c{D3+Na1}	& \c{Na1 Ld}	&EqOut&$S_i$\\
\c{*Sside1}&\c{Sside+Na1}& \c{Na1 Ld}	&EqOut&$S1_i$\\
\hline
\end{tabular}
\vskip1ex

\section{Skeleton of Shared memory communication}
\noindent\begin{tabular}{|rrlrrlp{2.2in}|}\hline
\multicolumn{3}{|c|}{\c{ASTRA}}&\multicolumn{3}{|c|}{\c{ESC}}&Comment\\\hline
\c{ASTRA}&\c{Command}&\c{ESC}&\c{ESC}&\c{Command}&\c{ASTRA}
&Semaphore positions: \c{L}ocked, 
\c{O}pen\\\hline
&\c{system();}&\c{O}&&&& Launching \ESC\\
\c{L}&&&\c{uL}&\c{shmget();}&\c{O}& Allocating shared memory in \ESC\\
\c{uL}&\c{shmat();}&\c{O}&\c{L}&&& Allocating shared memory in \c{ASTRA}\\
\c{L}&&&\c{u}&\c{PFC-WaveF}&& Hardware and WaveForm processing\\\hline
\c{L}&&&\c{uL}&&\c{O}& Openning a freeway for \c{ASTRA} loop\\
\c{uL}&\c{escin(}&\c{O}&\c{L}&&& Input data for \ESC\\
\c{L}&&&\c{u}&\c{PlVacShM();}&& Reading Plasma boundary data\\
\c{L}&&&\c{u}&\c{PlPrShM();}&& Reading Plasma pressure data\\
\c{L}&&&\c{uL}&\c{PlCrShM();}&\c{O}& Reading Plasma current data\\
\c{uL}&\c{IfFailure();}&\c{O}&\c{L}&&& Confirmation of data processing\\
\c{L}&\c{);}&&\c{u}&&&\\
\c{L}&&&\c{uL}&&\c{O}& Openning a freeway for \c{esc()} in \c{ASTRA}\\
\c{uL}&\c{esc(}&\c{O}&\c{L}&&& Putting data for equilibrium solver\\
\c{L}&&&\c{uL}&\c{2DEqSolv();}&\c{O}& Equilibrium solving\\
\c{uL}&\c{IfFailure();}&\c{O}&\c{L}&&& Confirmation of data processing\\
\c{L}&\c{);}&&\c{u}&&&\\
\c{L}&&&\c{uL}&\c{EqOutShM();}&\c{O}& Output into shared memory\\
\c{uL}&\c{e2astra();}&\c{O}&\c{L}&&&Taking equilibrium data\\
\hline
\end{tabular}
#endif

#ifndef mzl_System_esc.c /* --------------- Use of Variables --------{ */
#ifdef Tbl_UnusedID/* ------------- List of Unused variables ------{ */
  extern unsigned int ESmemlev;	/*            :esc.c */
  extern unsigned int ESmemlevP;/*            :esc.c */
  extern double cr2;		/*            :esc.c */
  extern double cr3;		/*            :esc.c */
  extern double cr4;		/*            :esc.c */
  extern double cr6;		/*            :esc.c */
  extern double cr12;		/*            :esc.c */
  extern double cgp;		/*            :esc.c */
  extern double c2gp;		/*            :esc.c */
  extern double cgp_4;		/*            :esc.c */
  extern double cr2gp;		/*            :esc.c */
  extern double cgm0;		/*            :esc.c */
  extern double crgm0;		/*            :esc.c */
  extern double GlTime;		/*            :esc.c */
  extern double dGlTime;	/*            :esc.c */
  extern double StTime;		/*            :esc.c */
  extern double ETime;		/*            :esc.c */
  extern int ESEqSolvIt;	/*            :esc.c */
  extern int ECEqFound();	/*            :esc.c */
  int main();			/*        <ESC:esc.c */
  int argc;			/* .main()<ESC:esc.c */
  char **argv;			/* .main()<ESC:esc.c */
#endif /* }=== End of List of Unused variables ======*/
#ifndef H/* ---------- Used External Variables ------{ */
  Systm NULL;			/* undefined */
  Undef PlBlockLim();		/* NO prototype */
  Undef CHIElectrodes();	/* NO prototype */
  Undef PFVacVessel();		/* NO prototype */
  Undef PSBElecPar();		/* NO prototype */
  Undef MFProbe();		/* NO prototype */
  Undef PFBlocks();		/* NO prototype */
  Undef PFBElecPar();		/* NO prototype */
  Undef PFCSplD();		/* NO prototype */
  Undef PFFirstSetSpl();	/* NO prototype */
  Undef ExtMField();		/* NO prototype */
  Undef MSEGeom();		/* NO prototype */
  Undef PFSetShotNm();		/* NO prototype */
  Undef tsc2e();		/* NO prototype */
  Undef PFWaveFormIp();		/* NO prototype */
  Undef PFWaveFormY();		/* NO prototype */
  Undef PFWaveFormB();		/* NO prototype */
  Undef PFWaveFormI();		/* NO prototype */
  Undef SnapShot();		/* NO prototype */
  Undef GetExpData();		/* NO prototype */
  Undef ESInpRadC();		/* NO prototype */
  Undef ESInputPlVac();		/* NO prototype */
  Undef ESInput2DEqSolv();	/* NO prototype */
  Undef ECReInitFgY();		/* NO prototype */
  Undef PFCoils();		/* NO prototype */
  Undef PFCOptPar();		/* NO prototype */
  Undef PFCInitEq();		/* NO prototype */
  Undef CHIEquil();		/* NO prototype */
  Undef ESInpPlPr();		/* NO prototype */
  Undef ESInpMSE();		/* NO prototype */
  Undef ESInpPlCr();		/* NO prototype */
  Undef ECRescue2DGeom();	/* NO prototype */
  Undef EC2DEqSolv();		/* NO prototype */
  Undef ESCheckGShFourier();	/* NO prototype */
  Undef ECgYcs2dRZcsBound();	/* NO prototype */
  Undef ECVirtualPlV2vdrPlV();	/* NO prototype */
  Undef ECgYrc2dRZcs();		/* NO prototype */
  Undef ECvdrz2vrz();		/* NO prototype */
  Undef ECgY2RZ();		/* NO prototype */
  Undef ECCheckJacob();		/* NO prototype */
  Undef ECgYrc2dRZcsHole();	/* NO prototype */
  Undef ECvdrz2vrzHole();	/* NO prototype */
  Undef ECCheckJacobHole();	/* NO prototype */
  Undef ECRestore2DGeom();	/* NO prototype */
  Undef ECRcs2D2Rcs3D();	/* NO prototype */
  Undef ESHoleEdgeBoundary();	/* NO prototype */
  Undef ESRealGeom();		/* NO prototype */
  Undef ES3DMetrTnsrCSHole();	/* NO prototype */
  Undef ES3DMetrTnsrCS();	/* NO prototype */
  Undef EC1DEqSolv();		/* NO prototype */
  Undef ECIfEqFound();		/* NO prototype */
  Undef ESEquilOut();		/* NO prototype */
  Undef ESBalStab();		/* NO prototype */
  Undef ESMHDStabLowN();	/* NO prototype */
  Undef ESNonMHDStabLowN();	/* NO prototype */
  Undef ESOrbits();		/* NO prototype */
  Undef ESDeInitTsc();		/* NO prototype */
  extern double ESa0;		/* External */
  extern void esc0_();		/* External() */
  extern clock_t clock();	/* External() */
  extern void esc1_();		/* External() */
#ifdef Tbl_ESC
  Undef PlBlockLim();		/* NO prototype */
  Undef CHIElectrodes();	/* NO prototype */
  Undef PFVacVessel();		/* NO prototype */
  Undef PSBElecPar();		/* NO prototype */
  Undef MFProbe();		/* NO prototype */
  Undef PFBlocks();		/* NO prototype */
  Undef PFBElecPar();		/* NO prototype */
  Undef PFCSplD();		/* NO prototype */
  Undef PFFirstSetSpl();	/* NO prototype */
  Undef ExtMField();		/* NO prototype */
  Undef MSEGeom();		/* NO prototype */
  Undef PFSetShotNm();		/* NO prototype */
  Undef tsc2e();		/* NO prototype */
  Undef PFWaveFormIp();		/* NO prototype */
  Undef PFWaveFormY();		/* NO prototype */
  Undef PFWaveFormB();		/* NO prototype */
  Undef PFWaveFormI();		/* NO prototype */
  Undef SnapShot();		/* NO prototype */
  Undef GetExpData();		/* NO prototype */
  Undef ESInpRadC();		/* NO prototype */
  Undef ESInputPlVac();		/* NO prototype */
  Undef ESInput2DEqSolv();	/* NO prototype */
  Undef ECReInitFgY();		/* NO prototype */
  Undef PFCoils();		/* NO prototype */
  Undef PFCOptPar();		/* NO prototype */
  Undef PFCInitEq();		/* NO prototype */
  Undef CHIEquil();		/* NO prototype */
  Undef ESInpPlPr();		/* NO prototype */
  Undef ESInpMSE();		/* NO prototype */
  Undef ESInpPlCr();		/* NO prototype */
  Undef ECRescue2DGeom();	/* NO prototype */
  Undef EC2DEqSolv();		/* NO prototype */
  Undef ESCheckGShFourier();	/* NO prototype */
  Undef ECgYcs2dRZcsBound();	/* NO prototype */
  Undef ECVirtualPlV2vdrPlV();	/* NO prototype */
  Undef ECgYrc2dRZcs();		/* NO prototype */
  Undef ECvdrz2vrz();		/* NO prototype */
  Undef ECgY2RZ();		/* NO prototype */
  Undef ECCheckJacob();		/* NO prototype */
  Undef ECgYrc2dRZcsHole();	/* NO prototype */
  Undef ECvdrz2vrzHole();	/* NO prototype */
  Undef ECCheckJacobHole();	/* NO prototype */
  Undef ECRestore2DGeom();	/* NO prototype */
  Undef ECRcs2D2Rcs3D();	/* NO prototype */
  Undef ESHoleEdgeBoundary();	/* NO prototype */
  Undef ESRealGeom();		/* NO prototype */
  Undef ES3DMetrTnsrCSHole();	/* NO prototype */
  Undef ES3DMetrTnsrCS();	/* NO prototype */
  Undef EC1DEqSolv();		/* NO prototype */
  Undef ECIfEqFound();		/* NO prototype */
  Undef ESEquilOut();		/* NO prototype */
  Undef ESBalStab();		/* NO prototype */
  Undef ESMHDStabLowN();	/* NO prototype */
  Undef ESNonMHDStabLowN();	/* NO prototype */
  Undef ESOrbits();		/* NO prototype */
  Undef ESDeInitTsc();		/* NO prototype */
  extern double ESa0;		/* External */
  extern clock_t clock();	/* External() */
  extern void esc1_();		/* External() */
#endif
#endif /* }======== End of Used External Variables ======= */

#endif /* }======== End of Use of Variables ======== */


#ifndef mzl_ESC
extern clock_t clock();		/* undefined */
extern unsigned int ESmemlev;	/* undefined */
extern unsigned int ESmemlevP;	/* undefined */
extern double cr2;		/* undefined */
extern double cr3;		/* undefined */
extern double cr4;		/* undefined */
extern double cr6;		/* undefined */
extern double cr12;		/* undefined */
extern double cgp;		/* undefined */
extern double c2gp;		/* undefined */
extern double cgp_4;		/* undefined */
extern double cr2gp;		/* undefined */
extern double cgm0;		/* undefined */
extern double crgm0;		/* undefined */
extern double GlTime;		/* undefined */
extern double dGlTime;		/* undefined */
extern double StTime;		/* undefined */
extern double ETime;		/* undefined */
extern int ESEqSolvIt;		/* undefined */
extern int ECEqFound();		/* undefined */
extern void esc0_();		/* undefined */
extern void esc1_();		/* undefined */
extern double ESa0;		/* undefined */
static clock_t Ect0;		/*  */

int main(argc,argv)
  int argc;	/*  */
  char **argv;	/*  */
{

#ifndef mzl_ESC

#ifndef blb_Machine
  Undef NULL;
  extern void esc0_();	/* undefined */

#ifndef mzl_ESC

#ifndef stg_PlLim
  Undef PlBlockLim();
#endif

#ifndef stg_CHI
  Undef CHIElectrodes();
#endif

#ifndef stg_VacV
  Undef PFVacVessel();
#endif

#ifndef stg_PSElecP
  Undef PSBElecPar();
#endif

#ifndef stg_MFProbe
  Undef MFProbe();
#endif

#ifndef stg_PFBlocks
  Undef PFBlocks();
#endif

#ifndef stg_ElecPar
  Undef PFBElecPar();
#endif

#ifndef stg_PFBSplD
  Undef PFCSplD();
#endif
  Undef PFFirstSetSpl();

#ifndef stg_ExtMF
  Undef ExtMField();
#endif

#ifndef blb_Year

#ifndef mzl_ESC

#ifndef stg_MSE_Geom
  Undef MSEGeom();
#endif

#ifndef blb_Shot

#ifndef mzl_ESC
  Undef PFSetShotNm();
  Undef tsc2e();

#ifndef stg_WFormIp
  Undef PFWaveFormIp();
#endif

#ifndef stg_WFormY
  Undef PFWaveFormY();
#endif

#ifndef stg_WFormB
  Undef PFWaveFormB();
#endif

#ifndef stg_WFormI
  Undef PFWaveFormI();
#endif

#ifndef blb_Equil

#ifndef mzl_ESC

#ifndef stg_SnapShot
  Undef SnapShot();
#endif

#ifndef stg_ExpData
  Undef GetExpData();
#endif

#ifndef stg_RadC
  Undef ESInpRadC();
#endif

#ifndef stg_PlVac
  Undef ESInputPlVac();
#endif

#ifndef stg_EqSolv
  Undef ESInput2DEqSolv();
  Undef ECReInitFgY();
#endif

#ifndef stg_PFCoils
  Undef PFCoils();
#endif

#ifndef stg_PFCOpt
  Undef PFCOptPar();
#endif

#ifndef stg_PFCIeq
  Undef PFCInitEq();
#endif

#ifndef stg_EqCHI
  Undef CHIEquil();
#endif

#ifndef stg_PlPr
  Undef ESInpPlPr();
#endif

#ifndef stg_MSE
  Undef ESInpMSE();
#endif

#ifndef stg_PlCr
  Undef ESInpPlCr();
#endif
  Undef ECRescue2DGeom();
  Undef EC2DEqSolv();
  Undef ESCheckGShFourier();
  Undef ECgYcs2dRZcsBound();
  Undef ECVirtualPlV2vdrPlV();
  Undef ECgYrc2dRZcs();
  Undef ECvdrz2vrz();
  Undef ECgY2RZ();
  Undef ECCheckJacob();
  Undef ECgYrc2dRZcsHole();
  Undef ECvdrz2vrzHole();
  Undef ECCheckJacobHole();
  Undef ECRestore2DGeom();
  Undef ECRcs2D2Rcs3D();
  Undef ESHoleEdgeBoundary();
  Undef ESRealGeom();
  Undef ES3DMetrTnsrCSHole();
  Undef ES3DMetrTnsrCS();
  Undef EC1DEqSolv();
  Undef ECIfEqFound();
  int Fl;	/*  .main()<ESC:esc.c */

#ifndef stg_EqOut
  Undef ESEquilOut();
#endif

#ifndef stg_BalSt
  Undef ESBalStab();
#endif

#ifndef stg_LowNSt
  Undef ESMHDStabLowN();
#endif

#ifndef stg_NonMHD
  Undef ESNonMHDStabLowN();
#endif

#ifndef stg_Orbits
  Undef ESOrbits();
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
  Undef ESDeInitTsc();
#endif
  /*
   */
#ifdef Doc_main
\section{Skeleton of main()}
  \begin{verbatim}
    main(){
    }
  \end{verbatim}
\section{main()}
\subsection{Description}
\subsection{Algorithm}
\subsection{ToDo}
\subsection{References}
#endif
}
#endif
#endif

#undef Doc_Cb_Structure
#ifdef Blb_esc.c
Blb_esc.c{
  mzl_ESC{
    int main(){
      mzl_ESC{
        blb_Machine{
          mzl_ESC{
            stg_PlLim{
            }
            stg_CHI{
            }
            stg_VacV{
            }
            stg_PSElecP{
            }
            stg_MFProbe{
            }
            stg_PFBlocks{
            }
            stg_ElecPar{
            }
            stg_PFBSplD{
            }
            stg_ExtMF{
            }
            blb_Year{
              mzl_ESC{
                stg_MSE_Geom{
                }
                blb_Shot{
                  mzl_ESC{
                    stg_WFormIp{
                    }
                    stg_WFormY{
                    }
                    stg_WFormB{
                    }
                    stg_WFormI{
                    }
                    blb_Equil{
                      mzl_ESC{
                        stg_SnapShot{
                        }
                        stg_ExpData{
                        }
                        stg_RadC{
                        }
                        stg_PlVac{
                        }
                        stg_EqSolv{
                        }
                        stg_PFCoils{
                        }
                        stg_PFCOpt{
                        }
                        stg_PFCIeq{
                        }
                        stg_EqCHI{
                        }
                        stg_PlPr{
                        }
                        stg_MSE{
                        }
                        stg_PlCr{
                        }
                        stg_EqOut{
                        }
                        stg_BalSt{
                        }
                        stg_LowNSt{
                        }
                        stg_NonMHD{
                        }
                        stg_Orbits{
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }
}
#endif

#undef Doc_Table_Of_Content
#ifndef Blb_esc.c
		PS p= -1 (  0);
#ifndef mzl_ESC
		PS p=  1 (  1);
#ifndef blb_main
int main();	PS p=  4 (  2);
#ifndef mzl_ESC
		PS p=  5 (  3);
#ifndef blb_Machine
		PS p=  5 (  3);
#ifndef mzl_ESC
		PS p=  5 (  3);
#ifndef stg_PlLim
		PS p=  5 (  3);
#endif
#ifndef stg_CHI
		PS p=  5 (  3);
#endif
#ifndef stg_VacV
		PS p=  5 (  3);
#endif
#ifndef stg_PSElecP
		PS p=  5 (  3);
#endif
#ifndef stg_MFProbe
		PS p=  5 (  3);
#endif
#ifndef stg_PFBlocks
		PS p=  5 (  3);
#endif
#ifndef stg_ElecPar
		PS p=  5 (  3);
#endif
#ifndef stg_PFBSplD
		PS p=  5 (  3);
#endif
#ifndef stg_ExtMF
		PS p=  5 (  3);
#endif
#ifndef blb_Year
		PS p=  5 (  3);
#ifndef mzl_ESC
		PS p=  5 (  3);
#ifndef stg_MSE_Geom
		PS p=  5 (  3);
#endif
#ifndef blb_Shot
		PS p=  5 (  3);
#ifndef mzl_ESC
		PS p=  5 (  3);
#ifndef stg_WFormIp
		PS p=  5 (  3);
#endif
#ifndef stg_WFormY
		PS p=  5 (  3);
#endif
#ifndef stg_WFormB
		PS p=  6 (  3);
#endif
#ifndef stg_WFormI
		PS p=  6 (  3);
#endif
#ifndef blb_Equil
		PS p=  6 (  3);
#ifndef mzl_ESC
		PS p=  6 (  3);
#ifndef stg_SnapShot
		PS p=  6 (  3);
#endif
#ifndef stg_ExpData
		PS p=  6 (  3);
#endif
#ifndef stg_RadC
		PS p=  6 (  3);
#endif
#ifndef stg_PlVac
		PS p=  6 (  3);
#endif
#ifndef stg_EqSolv
		PS p=  6 (  3);
#endif
#ifndef stg_PFCoils
		PS p=  6 (  3);
#endif
#ifndef stg_PFCOpt
		PS p=  6 (  3);
#endif
#ifndef stg_PFCIeq
		PS p=  6 (  3);
#endif
#ifndef stg_EqCHI
		PS p=  6 (  3);
#endif
#ifndef stg_PlPr
		PS p=  6 (  3);
#endif
#ifndef stg_MSE
		PS p=  6 (  3);
#endif
#ifndef stg_PlCr
		PS p=  6 (  3);
#endif
#ifndef stg_EqOut
		PS p=  7 (  4);
#endif
#ifndef stg_BalSt
		PS p=  7 (  4);
#endif
#ifndef stg_LowNSt
		PS p=  7 (  4);
#endif
#ifndef stg_NonMHD
		PS p=  7 (  4);
#endif
#ifndef stg_Orbits
		PS p=  7 (  4);
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
