\Section{\c{PlLim} \c{stg}-section}
\vskip-2ex
\hskip4ex{August 29, 2003 by\em Leonid E. Zakharov}
\hfill source file \c{Hlp/PlLim.Hlp}
\Subsection{Overview}
The section specifies the geometry of a grid box containing the plasma
region and the contour of the plasma facing Limiter.  {\em  At present,
  the Limiter contour should be closed.} 

It calculates the plasma grid and parameters of the ``numerical''
limiter, which consists of intersections of the limiter with the
grid lines or the boundary of the plasma box. 

\Subsection{Tutorial}
By default, when the code enters the the \c{PlLim} section, it reads
ASCII data on the plasma gdid box and on limiter contour. Then it enters
into the interactive regime
\begin{enumerate}
\item Click \c{Enter}, if there is a message. Message disappears.
  \item Set the Plasma box:
    \begin{enumerate}
      \item touch a corner of the plasma box (small empty rectangle)
      with \c{<mouse-M,R>}; 
      \item type the r-data value into the active (green) table data box;
      \item touch the corresponding cell for z-data in the table and
      type the z-value; 
      \item alternatively, move the corner into the necessary position
      by pressed \c{<mouse-L>}.
      \item type the number of radial and vertical intervals.
    \end{enumerate}
  \item Set the Limiter contour:
    \begin{enumerate}
      \item touch a point on the Limiter contour with \c{<mouse-M,R>}; 
      \item type the r-data value into the active (green) table data box;
      \item type the z-data (after switching to the \c{ZLm} line);
      \item alternatively, move the Limiter data point by pressed
      \c{<mouse-L>};
    \end{enumerate}
  \item Desactivate some of Limiter points:
    \begin{enumerate}
      \item to see the effect, move one of Limiter point out of contour; 
      Click \c{Enter} to accept the data.
      \item switch the index on the active cell in the Table from
      \c{'x'} to  \c{' '} position by clicking \c{<mouse-M>}. Click
      \c{Enter} to accept the data.
      \item Use \c{<mouse-M>} to toggle the index in other Limiter
      points. Click \c{Enter} to accept the data.
      \item Watch how the number of points \c{NLm} follows the number of
      active points.
    \end{enumerate}
  \item Save data in the Table:
    \begin{enumerate}
      \item Switch the \c{Table} box on the top line of window into the
      \c{Save} position by several clicks of \c{<mouse-L>}, which
      rotates the list. 
      Click \c{Enter} to accept the data.
      \item Switch it into \c{StrmPS} in order to save the PostScript
      plot. 
    \end{enumerate}
  \item Read data from the Table file:
    \begin{enumerate}
      \item Switch the \c{Table} box on the top line of window into the
      \c{Read} position by clicking \c{<mouse-M>}, which
      rotates the list backward;
      \item Click \c{Enter} to accept the data from the Table.
    \end{enumerate}
    \item Test different values of bitwise \c{Fl} described below in
    Control Parameters section.
    \item Activate the output switch \c{As} (\c{ASCII to .ao}) on
    \c{CPanel}. Activate \c{Lp} (\c{Loop On/Off}) mode switch and click
    \c{Esc}. The code confirms writing the output files with the current
    data on the Plasma Box and Limiter. Take a look in the content of
    the ASCII file. 
\end{enumerate}
Hit \c{Esc} to exit the section.

\Subsection{Known Bugs}
\Subsection{Control Parameters}
File: \c{PlLim.tb}.
\hskip1ex
The following table lists the control parameters and their
characteristic values. In the table, the bitwise C-operator
\c{Fl\&0x10} shows commonly set bits in two numbers \c{Fl} and
\c{0x10} (\c{0x} is the hexadecimal representation of integers).

\noindent\begin{tabular}{|rrcrp{4in}|}\hline
\bf Name&\bf Units&&\bf Type&\bf Comment\\\hline
\c{Fl}&&=&\c{int}&\em Bitwise GUI Flag controlling the plot. Lowest 7
bits of \c{Fl} are essential:\\
\c{Fl}&&=&\c{0}&\em  show the intersections of horizontal grid lines 
with the limiter;\\
\c{Fl\&1}&&=&\c{0x01}&\em show the intersections of horizontal grid 
lines with the limiter;\\
\c{Fl\&2}&&=&\c{0x02}&\em show all grid points inside the limiter;\\
\c{Fl\&4}&&=&\c{0x04}&\em show the grid points used for calculation of 
horizontal derivatives of $\bgY$;\\
\c{Fl\&8}&&=&\c{0x08}&\em show the grid points used for calculation of
vertical derivatives of $\bgY$;\\
\c{Fl\&16}&&=&\c{0x10}&\em show all the grid points;\\
\c{Fl\&32}&&=&\c{0x20}&\em show all the grid points along the grid
boundary used for calculation of derivatives at the boundary;\\
\c{Fl\&64}&&=&\c{0x40}&\em make a default limiter;\\
\hline
\c{NLm}&&=&\c{int}&\em  Number of Limiter segments.\\
\c{Nr}&&=&\c{int}&\em Radial number of subdivisions.\\
\c{Nz}&&=&\c{int}&\em Vertical number of subdivisions.\\
\hline
\c{Rp1}&L.m&=&\c{Dbl}&\em r of a corner of Plasma grid box.\\
\c{Rp2}&L.m&=&\c{Dbl}&\em r of another corner of the Plasma grid box.\\
\c{Zp1}&L.m&=&\c{Dbl}&\em r of a corner of Plasma grid box.\\
\c{Zp2}&L.m&=&\c{Dbl}&\em z of another corner of the Plasma grid box.\\
\hline
\c{RLm}&L.m&=&\c{Dbl*}&\em z - of the Limiter countour.\\
\c{ZLm}&L.m&=&\c{Dbl*}&\em z - of the Limiter countour.\\
\multicolumn{5}{|l|}{\bf Index:}\\
&\c{' '}&=&&\em the point is ignored;\\
&\c{'x'}&=&&\em the point is taken into account;\\
\hline
\end{tabular}
\vskip1ex
GUI allows to change the position of Limiter points, add or 
desactivate some of the points. Only points left active are taken into
account later in the code. 

\Subsection{Input}
\begin{verbatim}
 \c{<PlLim.bi>}, \c{<PlLim.ai>}
\end{verbatim}
\c{<PlLim.ai>} contains the records shown with their formats in the
Table (\c{*} in the Type means a vector data).
\vskip1ex
\begin{tabular}{|lll|}\hline
\multicolumn{2}{|l|}{\c{<Plasma Box>}}&\c{(\%e \%e \%e \%e \%d x \%d)}\\\hline

Data Name:&Type&\\
\c{Rpl 1}&\c{dbl}&r-coordinate of the plasma grid box\\
\c{Zpl 1}&\c{dbl}&z-coordinate of the plasma grid box\\
\c{Rpl 2}&\c{dbl}&r-coordinate of the plasma grid box\\
\c{Zpl 2}&\c{dbl}&z-coordinate of the plasma grid box\\
\c{Nr}&\c{int}&number of radial grid intervals\\
\c{Nz}&\c{int}&number of vertical grid intervals\\\hline

\multicolumn{2}{|l|}{\c{<Limiter>}}&\c{(\%d \%e \%e)}\\\hline

\c{RLm}&\c{dbl*}&r-coordinates of the limiter contour\\
\c{ZLm}&\c{dbl*}&z-coordinates of the limiter contour\\
\hline
\end{tabular}
\vskip1ex
The format of \c{PlLim.ai} is explained in the general help for \c{Input
  Switch} in window \c{\cdnm\ 0} and assumes:
\begin{itemize}
  \item \c{<Plasma Box>} and \c{<Limiter>} are the key words of the
    record which are used for searching the chunk of data.
  \item Text between records is ignored.
  \item Inside the data, the comment lines, starting with \c{!}, are
    allowed.
  \item In each data line, a comment is allowed after the actual data. 
  \item Enumeration of vector data in the first column is necessary,
    but its actiual value is ignored in the code.
\end{itemize}
It is illustrated below
\hskip1ex
\begin{verbatim}
<Plasma Box>[1](%e %e %e %e %d x %d)
 1.8000 -1.6000  3.9000  2.1000  24 x  33
  Rpl 1   Zpl 1   Rpl 2   Zpl 2  Nr x  Nz
Any text between the records
<Limiter>[142](%d %e %e)
  0  2.0481 -0.9298
  1  2.1316 -1.1314
  2  2.2927 -1.3368
  3  2.3611 -1.3368
  4  2.3974 -1.3757
  5  2.4104 -1.4028 comment behind the data
  6  2.4060 -1.3882
  7  2.4137 -1.4258
    !comment inside the data
  8  2.4142 -1.4328
.............
  #     RLm     ZLm
\end{verbatim}
\hskip1ex

\Subsection{Output}
Files: \c{PlLim.bo}, \c{PlLim.ao} - optional.
\Subsection{Tips}
In order to generate a Limiter input file, use either existing in the
DataBase \c{In/PlLim.ai} file or default setting as a template.
\begin{enumerate}
  \item Create graphically Plasma grid and Limeter contour;
  \item Activate the \c{ASCII} output switch on Control Panel;
  \item Exit the section in order to write the file;
  \item Move the file in the DataBase from \c{Out/PlLim.ao} into
  \c{In/PlLim.ai}. It will be ready for Input.
\end{enumerate}

\Subsection{Description}
 Plasma grid is used later on for either filament 
representation of the plasma current distribution and Green function 
based solver of the Grad-Shafranov equation or for the finite
 difference solver. 

In ESC the plasma cross-section is determined either by magnetic 
separatrix (if the plasma is entirely inside the limiter contour) 
or by the magnetic surface which touches the Limiter contour.
\Subsection{Algorithm}
\Subsection{ToDo}
\Subsection{References}
