#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>

#include "XCbType.h"
     
extern Display*theD;
extern Window theHW,theMW;
extern GC theGC,theGCM,theGCH;
extern WinPar*WinP;
extern int NWin;
extern XEvent msg;
extern int wF,hF,wFHelp,hFHelp;
extern int XCbNColor;
extern char **ColorNames;
extern unsigned long ColorPix[];

extern int theD_w,theD_h;
extern int icon_width;
extern int icon_height;
extern char*icon_bits;
extern Window theRW;

static int scr_n;
static int count_W=0;
static unsigned long theBGColor;
static Font theFont,theHelpFont;
static XFontStruct*HelpFontStr;
static char*ln=NULL;
static int Nln=120;
     
int screen_attributes(int *theD_w,int *theD_h)
{
  if((theD=XOpenDisplay(NULL)) == NULL){
    (void)fprintf(stderr,"cannot connect to X server %s\n",
		  XDisplayName(NULL));
    exit(-1);
  }
  scr_n		=DefaultScreen(theD);
  *theD_h	=DisplayHeight(theD,scr_n);
  *theD_w	=DisplayWidth(theD,scr_n);
  theRW		=RootWindow(theD,scr_n);
  XSetBGColor();
  return(0);
}

int XCbSpecifyColors()
{
  int i;
  Colormap theColormap;
  XColor theColor;

  theColormap	=DefaultColormap(theD,scr_n);
  for(i=0; i < XCbNColor; i++){
    if(XParseColor(theD,theColormap,ColorNames[i],&theColor)==0){
      printf("%s[%2d] - Color-name is not resolved in specify_colors()%c\n",
	     ColorNames[i],i,7);
    }
    XAllocColor(theD,theColormap,&theColor);
    ColorPix[i]	=theColor.pixel;
  }
  return(0);
}

int create_window(WinPar* WP)
{
  unsigned int border_width= 4;
  Pixmap icon_pixmap;
  XSizeHints size_hints;
  XIconSize*size_list;
  XEvent rep;

  if(ln==NULL){
    ln= (char*)malloc(Nln*sizeof(char));
  }

  WP->W= XCreateSimpleWindow(theD,theRW,WP->iX,WP->iY,WP->iW,WP->iH,
			     border_width,BlackPixel(theD,scr_n),
			     theBGColor);
  icon_pixmap= XCreateBitmapFromData(theD,WP->W,icon_bits,
				     icon_width,icon_height);
  size_hints.flags= USPosition|USSize|PMinSize;
  size_hints.min_width= 217;
  size_hints.min_height= 150;
  {
    XWMHints wm_hints;
    XClassHint class_hints;
    XTextProperty windowName,iconName;

    if(XStringListToTextProperty(&WP->nm,1,&windowName)==0){
      (void)fprintf(stderr," structure allocation for window failed.\n");
      exit(-1);
    }
    if(XStringListToTextProperty(&WP->nm,1,&iconName)==0){
      (void)fprintf(stderr,"structure allocation for icon failed.\n");
      exit(-1);
    }
    wm_hints.initial_state= WP->state ? NormalState : IconicState;
    wm_hints.input= True;
    wm_hints.icon_pixmap= icon_pixmap;
    wm_hints.icon_x= WP->iconx;
    wm_hints.icon_y= WP->icony;
    wm_hints.flags	=StateHint|IconPixmapHint|InputHint|IconPositionHint;
    class_hints.res_name= NULL;
    class_hints.res_class= WP->nm;
    XSetWMProperties(theD,WP->W,&windowName,&iconName,
		     NULL,0,&size_hints,&wm_hints,
		     &class_hints);
  }
  XSelectInput(theD,WP->W,ExposureMask|StructureNotifyMask|KeyPressMask
	       |EnterWindowMask
	       |Button1MotionMask|ButtonPressMask|ButtonReleaseMask);
  XMapWindow(theD,WP->W);
  if(count_W == 0){
    XGCValues val;
    char fontname[6];
    if(wF<10){
      if(hF<10){
	sprintf(fontname,"%1dx%1d",wF,hF);
      }
      else{
	sprintf(fontname,"%1dx%2d",wF,hF);
      }
    }
    else{
      if(hF<10){
	sprintf(fontname,"%2dx%1d",wF,hF);
      }
      else{
	sprintf(fontname,"%2dx%2d",wF,hF);
      }
    }
    theFont= XLoadFont(theD,fontname);

    val.font= theFont;
    val.foreground= BlackPixel(theD,scr_n);
    val.background= WhitePixel(theD,scr_n);
    theGC= XCreateGC(theD,WP->W,GCFont|GCForeground|GCBackground,&val);
    XCbSpecifyColors();
    XWarpPointer(theD,theRW,WP->W,0,0,theD_w,theD_h,100,100);
  }
  count_W++;
  return(0);
}

static XSizeHints HlpSize;

int XCbCreateHelpWindow(WinPar*WP)
{
  XSetWindowAttributes Attr;
  unsigned int border_width=1;
  
  WP->state	=0;
  WP->W	=XCreateSimpleWindow(theD,theRW,WP->iX,WP->iY,WP->iW,WP->iH,
			     border_width,BlackPixel(theD,scr_n),
			     WhitePixel(theD,scr_n));
  theHW	=WP->W;
  Attr.override_redirect=1;
  Attr.save_under=True;
  XChangeWindowAttributes(theD,theHW,
			  CWOverrideRedirect|CWSaveUnder,&Attr);
  
  HlpSize.flags		=USPosition|USSize|PMinSize;
  HlpSize.min_width	=217;
  HlpSize.min_height	=150;
  HlpSize.min_width	=400;
  HlpSize.min_height	=300;
  {
    XWMHints wm_hints;
    XClassHint class_hints;
    XTextProperty windowName;

    if(XStringListToTextProperty(&WP->nm,1,&windowName)==0){
      (void)fprintf(stderr," structure allocation for window failed.\n");
      exit(-1);
    }
    wm_hints.initial_state	=WP->state ? NormalState:IconicState;
    wm_hints.input= True;
    wm_hints.flags= StateHint|InputHint;
    class_hints.res_name= NULL;
    class_hints.res_class= WP->nm;

    XSetWMProperties(theD,theHW,&windowName,NULL,
		     NULL,0,&HlpSize,&wm_hints,
		     &class_hints);
  }
  XSelectInput(theD,theHW,ExposureMask|
	       ButtonPressMask|ButtonReleaseMask|StructureNotifyMask);
  
  {
    XGCValues val;
    
    char fontname[6];
    if(wFHelp < 10){
      if(hFHelp < 10)
	sprintf(fontname,"%1dx%1d\0",wFHelp,hFHelp);
      else
	sprintf(fontname,"%1dx%2d\0",wFHelp,hFHelp);
    }
    else{
      if(hFHelp<10)
	sprintf(fontname,"%2dx%1d\0",wFHelp,hFHelp);
      else
	sprintf(fontname,"%2dx%2d\0",wFHelp,hFHelp);
    }
    HelpFontStr= XLoadQueryFont(theD,fontname);
    if(HelpFontStr == NULL){
      out("ss",fontname," is not available");
    }
#ifdef H
    HelpFontStr= XLoadQueryFont(theD,"*helvetica-medium-r-normal--12*");
#endif
    
    hFHelp= HelpFontStr->ascent+HelpFontStr->descent;
    theHelpFont= HelpFontStr->fid;
    val.font= theHelpFont;
    val.foreground	=BlackPixel(theD,scr_n);
    val.background	=WhitePixel(theD,scr_n);
    theGCH	=XCreateGC(theD,theHW,GCFont|GCForeground|GCBackground,&val);
    val.foreground	=WhitePixel(theD,scr_n);
    val.background	=ColorPix[14];
    theGCM=	XCreateGC(theD,theHW,GCFont|GCForeground|GCBackground,&val);
  }
  count_W++;
  return(0);
}

int XCbCreateMenueWindow(WinPar*WP)
{
  XSetWindowAttributes Attr;

  theMW	=XCreateSimpleWindow(theD,theRW,WP->iX,WP->iY,WP->iW,WP->iH,
			     0,ColorPix[0],ColorPix[8]);
  WP->W	=theMW;
  Attr.override_redirect	=1;
  Attr.save_under	=1;
  XChangeWindowAttributes(theD,theMW,
			  CWOverrideRedirect|CWSaveUnder,&Attr);
  XSelectInput(theD,theMW,ButtonPressMask|ButtonReleaseMask);
  count_W++;

  XCbSetMenueTexts();
  return(0);
}

int XSetBGColor()
{
  Colormap theColormap;
  XColor theColor;

  theColormap= DefaultColormap(theD,scr_n);
  if(XParseColor(theD,theColormap,ColorNames[44],&theColor)==0){
    printf("%s - Color-name is not resolved in XSetBGColor()%c\n",
	   ColorNames[44],7);
  }
  
  XAllocColor(theD,theColormap,&theColor);
  theBGColor= theColor.pixel;
  return(0);
}

int XCbHelpText(char*text)
{
  int n,Fl,Flsc;
  int x,y,W,H,weF,w,Nw;
  char*p,*p1;

  W	=WinP[NWin].iW;
  H	=WinP[NWin].iH;

  weF	=XTextWidth(HelpFontStr,"eeeeeeee",8)/8;

  Nw	=W/weF;
  x	=weF;
  y	=hFHelp+2;
  p1	=text;

  p	=ln;
  ln[0]	='\0';
  n	=0;
  w	=0;
  Fl	=isspace(*p1)?0:1;
  Flsc	=0;
  while(*p1!='\0'&&y<H){
    if(isspace(*p1)){
      if(Fl==1){
	if(*p1=='\n'&&*(p1-1)==';'){
	  n--;
	  ln[n]	='\0';
	  Flsc	=1;
	}
	else{
	  Flsc	=0;
	}
	if(x+w>W){
	  y	+=hFHelp+2;
	  x	=weF;
	}
	XDrawString(theD,theHW,theGCH,x,y,ln,n);
	ln[n]	='\0';
	if(Flsc==1){
	  y	+=hFHelp+2;
	  x	=weF;
	}
	else{
	  x	+=w;
	}
      }
      Fl	=0;
      n	=0;

      while(isspace(*p1)){
	if(*p1=='\n'){
	  if(Fl==0){
	    n	=Flsc?0:1;
	  }
	  else{
	    n	=0;
	  }

	  Fl++;
	  if(Fl==2){
	    x	=weF;
	    y	+=hFHelp+2;
	    if(Flsc==0){
	      y	+=hFHelp+2;
	    }
	  }
	}
	else{
	  if(*p1=='\t'){
	    n	=(n/8+1)*8;
	  }
	  else{
	    n++;
	  }
	}
	*p1++;
      }
      x	+=n*weF;

      p	=ln;
      n	=0;
      w	=0;
      Fl	=0;
    }
    if(n>=Nln){
      Nln	+=120;
      ln	=(char*)realloc((void*)ln,Nln*sizeof(char));
      p	=ln+n;
    }
    if(*p1=='\0')
      break;
    Fl	=1;
    *p++=*p1++;
    w	+=XTextWidth(HelpFontStr,ln+n,1);
    n++;
    if(w>=W){
      y	+=hFHelp+2;
      x	=weF;
      *p	='\0';
      XDrawString(theD,theHW,theGCH,x,y,ln,n-2);
      x	+=w;
      w	=XTextWidth(HelpFontStr,ln+n-2,2);
      XDrawString(theD,theHW,theGCH,x,y,"-",1);
      ln[0]	=ln[n-2];
      ln[1]	=ln[n-1];
      n	=2;
      p	=ln+n;
    }
  }

  if(n){
    w	=XTextWidth(HelpFontStr,ln,n);
    if(x+w>W){
      y	+=hFHelp+2;
      x	=weF;
    }
    XDrawString(theD,theHW,theGCH,x,y,ln,n);
  }
  XFlush(theD);
  return(0);
}

int XCbHelpTextHeight(char*text)
{
  int n,Fl,Flsc;
  int x,y,W,H,weF,w,Nw;
  char*p,*p1;

  W	=WinP[NWin].iW;
  H	=WinP[NWin].iH;

  weF	=XTextWidth(HelpFontStr,"eeeeeeee",8)/8;

  Nw	=W/weF;
  x	=weF;
  y	=hFHelp+2;
  p1	=text;

  p	=ln;
  ln[0]	='\0';
  n	=0;
  w	=0;
  Fl	=isspace(*p1) ? 0 : 1;
  Flsc	=0;
  while(*p1 != '\0'){
    if(isspace(*p1)){
      if(Fl == 1){
	if(*p1 == '\n' && *(p1-1) == ';'){
	  n--;
	  ln[n]	='\0';
	  Flsc	=1;
	}
	else{
	  Flsc	=0;
	}
	if(x+w > W){
	  y  	+=hFHelp+2;
	  x	=weF;
	}
	ln[n]	='\0';
	if(Flsc == 1){
	  y	+=hFHelp+2;
	  x	=weF;
	}
	else{
	  x	+=w;
	}
      }
      Fl	=0;
      n	=0;

      while(isspace(*p1)){
	if(*p1 == '\n'){
	  if(Fl == 0){
	    n	=Flsc ? 0 : 1;
	  }
	  else{
	    n	=0;
	  }
	  Fl++;
	  if(Fl==2){
	    x	=weF;
	    y	+=hFHelp+2;
	    if(Flsc == 0){
	      y	+=hFHelp+2;
	    }
	  }
	}
	else{
	  if(*p1=='\t'){
	    n	=(n/8+1)*8;
	  }
	  else{
	    n++;
	  }
	}
	*p1++;
      }
      x	+=n*weF;

      p	=ln;
      n	=0;
      w	=0;
      Fl	=0;
    }
    if(n >= Nln){
      Nln	+=120;
      ln	=(char*)realloc((void*)ln,Nln*sizeof(char));
      p	=ln+n;
    }
    if(*p1 == '\0')
      break;
    Fl	=1;
    *p++	=*p1++;
    w	+=XTextWidth(HelpFontStr,ln+n,1);
    n++;
    if(w >= W){
      y	+=hFHelp+2;
      x	=weF;
      *p	='\0';
      x	+=w;
      w	=XTextWidth(HelpFontStr,ln+n-2,2);
      ln[0]	=ln[n-2];
      ln[1]	=ln[n-1];
      n	=2;
      p	=ln+n;
    }
  }

  if(n){
    w	=XTextWidth(HelpFontStr,ln,n);
    if(x+w > W){
      y	+=hFHelp+2;
      x	=weF;
    }
  }
  XDrawString(theD,theHW,theGCH,x,y,ln,n);
  return(y+2);
}

int XCbHelpWindow(int iWin,int x,int y,char*ptxt)
{
  WinP[NWin].iH	=ptxt == NULL ? XCbHelpTextHeight("No Help was available") :
    XCbHelpTextHeight(ptxt);
  x	+=WinP[iWin].iX;
  y	+=WinP[iWin].iY+hFHelp;

  if(x+WinP[NWin].iW > theD_w){
    x	+=theD_w-x-WinP[NWin].iW;
  }
  WinP[NWin].iX	=x;
  WinP[NWin].iY	=y;
  XMoveWindow(theD,theHW,x,y);
  XResizeWindow(theD,theHW,WinP[NWin].iW,WinP[NWin].iH);
  XMapWindow(theD,theHW);
  XRaiseWindow(theD,theHW);
  while(XCheckTypedEvent(theD,Expose,&msg)){
    ;
  }
#ifdef H
  do{
    XNextEvent(theD,&msg);
  }while(msg.type != Expose);
#endif
  if(ptxt == NULL){
    XCbHelpText("No Help was available");
  }
  else{
    XCbHelpText(ptxt);
  }
  do{
    XNextEvent(theD,&msg);
  }while(msg.type != ButtonRelease);
  XUnmapWindow(theD,theHW);
  while(XCheckTypedEvent(theD,Expose,&msg)
	|| XCheckTypedEvent(theD,UnmapNotify,&msg)){
    ;
  }
  return(0);
}

void endX()
{
  if(ln != NULL){
    free(ln);
  }
  XFreeGC(theD,theGCM);
  XFreeGC(theD,theGCH);
  XFreeGC(theD,theGC);
  XUnloadFont(theD,theFont);
  XFreeFont(theD,HelpFontStr);
  while(count_W){
    count_W--;
    XDestroyWindow(theD,WinP[count_W].W);
  }
  XCloseDisplay(theD);
  exit(0);
}

int CloseX()
{
  if(ln != NULL){
    free(ln);
  }
  XFreeGC(theD,theGCM);
  XFreeGC(theD,theGCH);
  XFreeGC(theD,theGC);
  XUnloadFont(theD,theFont);
  XFreeFont(theD,HelpFontStr);
  while(count_W){
    count_W--;
    XDestroyWindow(theD,WinP[count_W].W);
  }
  XCloseDisplay(theD);
  return(0);
}
