#ifndef Blb_esiZNew.c
/*  */
#ifdef Doc_esiZNew.c
\title{\vskip-0.9in Equilibrium Spline Interface (\c{ESI}) for magnetic 
confinement codes}
\author{Leonid E. Zakharov, Stephen C. Jardin, Morrell S. Chance, Roscoe 
B. White\\
{\em Princeton University, Princeton Plasma Physics Laboratory, 
Princeton NJ}\\
Grigorii V. Pereverzev\\
{\em Max-Plank Institute of Plasma Physics, Garching, Germany}\\
}
\date{}%{December 27, 2006}
\maketitle
\begin{abstract}
A compact and comprehensive interface between magneto-hydrodynamic (MHD)
equilibrium codes and gyro-kinetic, particle orbit, MHD stability, and
transport codes is presented. Its irreducible set of equilibrium data
consists of four 2- or 3-D functions of coordinates and four
1-D radial profiles together with their first derivatives. The C
reconstruction routines, accessible also from Fortran, allow the 
calculation of basis functions and their first derivatives at any position
inside the plasma. After this all vector fields and geometric
coefficients required for the above mentioned types of codes can be
calculated using only algebraic operations with no further
interpolation or differentiation.
\end{abstract}
\HyperPdf
\ToC
\Section{Introduction}
In plasma simulations, which deal with highly anisotropic equations,
the equilibrium codes play a fundamental role in supplying the
information about the magnetic configuration. For more sophisticated
plasma models: stability, transport, particle orbit or
gyro-kinetic, this information is needed in flux coordinates related to
the geometry of the magnetic field. 

While the equilibrium configurations depend essentially only on two
radial profiles and the shape of the plasma boundary, the request of
other codes may contain numerous 2- or 3-dimensional functions
representing components of the magnetic fields, current density, 
curvature of the field lines, or the magnitude of the magnetic field and 
its derivatives; all of these functions are inter-related. Because 
of finite accuracy of the equilibrium codes it is always a matter of
preference as to what should be taken as input data and what should be
calculated inside the ``client'' codes.

There are two extreme approaches to making such a decision, and both
rely on the exchange of data files. One of them requires all functions
necessary for the client code at all necessary positions to be supplied
by the equilibrium codes. Working only for closely related pairs of codes
and making them highly dependent, this approach requires vast storage
capacities because the required data (e.g., some combinations of metric
coefficients) are not very smooth. Being widespread in the plasma
physics community, it is totally irrelevant to the situation when the
client codes (like particle motion) need physics variables at
unpredictable positions inside the plasma.

Another extreme approach uses only primitive information from
the equilibrium codes, like coordinates of grid points and a few 
radial profiles, while all necessary functions are reconstructed
internally inside the client codes. The problem here is in calculation of
space derivatives, entering into many physics variables. With no
universal mathematical rules of doing numerical differentiation, this
approach unavoidably introduces its own inaccuracies and causes
potential convergence problems in interaction with equilibrium codes.

The idea of Equilibrium Spline Interface is to have a universal buffer,
which has a comprehensive information about magnetic configuration and is
able to provide it to magnetic confinement codes in a ready to use form.
The interface requires from the equilibrium codes a standard, easy to
generate, set of data together with the ID of coordinates and
representation. In its turn, the user side provide its own ID
information during initiation of interface. After this the interface can
generate all necessary data for magnetic confinement codes at any point
in the calculation area.

The proposed Equilibrium Spline Interface (\c{ESI}) consists of data
(stored in files or shared memory) and a C-source file (e.g., \c{escZ.c})
with C- or FORTRAN callable \c{ESI} initiation and reconstruction
routines. This paper and other documentation on the current state of
\c{ESI} can be found in \c{esiZ.c.d} file.

The interface is compact and comprehensive. \c{ESI} data represent a
Hermit polynomial representation of only well behaved functions in terms
of their values and first derivatives on a mesh. A limited set of basis
functions which consists of four radial profiles and four 2-D (for 
axisymmetric configurations like tokamaks) or 3-D (for stellarators)
functions of space coordinates.

The reconstruction routines can calculate basis functions and their
first derivatives at any point inside the plasma. Then, the client code
can calculate the necessary vector or scalar field variables by
algebraic operations without use of higher derivatives.

Sect.~2 introduces notations and basis of \c{ESI} functions and their
relations with the physics variables and plasma profiles. Sec.~3
describes initiation of \c{ESI} and a call main reconstruction routine
at the use side. Sec.~4 contains basic geometrical relationships, while
Sec.~5 provides the self consistent recipes for calculating parameters
of magnetic configurations.

Sect.~6 outlines the use of \c{ESI} for tracing field lines and particle
orbits. The Hamiltonian canonical coordinates are introduced for guiding
center motion in 3-D nested configurations. Boozer coordinates are not
canonical, as is often mistakenly believed. Sect.~7 shows sufficiency of
\c{ESI} for MHD stability codes. 

Finally, Sect.~8 and Sect.~9 describes the data storage structure, file
formats and the set of \c{ESI} routines.

\Section{The basis functions of \c{ESI}}

The curvilinear coordinates $a,\gq,\gz$ for describing magnetic
configurations for plasma confinement can be represented
parametrically by equations 
\bqbl
r=r(a,\gq,\gz)
,\quad
z=z(a,\gq,\gz)
,\quad
\gf=\gf(a,\gq,\gz)
\lbq{eq:rzgf}
,\eqbl
where $r,\gf,z$ are laboratory cylindrical coordinates. Coordinates are 
assumed to be nested and their Jacobian $J$
\bqbl
J\equiv\R{1}{(\na a\times\na\gq)\cdot\na\gz}
\eqbl
is not vanishing.

In the ideal situation, the magnetic surfaces are also
nested and it is possible to consider so-called ``flux'' coordinates,
when 
\bqbl
\waB\cdot\na a=0
.\eqbl

The vector potential of the magnetic field $\waB$ has the following
covariant representation 
\bqbl
\ESIA
\lbq{ESIA}
.\eqbl
In nested coordinates the dependencies on angles $\gq,\gz$ in function
$\bgF$ can be eliminated by adding $\na u$ to the vector potential and then
by massaging the radial coordinate $a=a+\gx(a,\gz)$
\bqbl
\bgF=\bgF(a)
.\eqbl
After this, $\gF(a)\equiv2\gp\bgF(a)$ becomes the flux of the magnetic
field through the contour $a=const,\gz=const$.
Only the reverse field pinch (RFP) configuration would require
$\bgF=\bgF(a,\gz)$.

In the function $\bgY=\bgY(a,\gq,\gz)$, {\em it is not possible} to
eliminate the dependence on angle coordinates in all situations. The
best representation 
\bqbl
\bgY
=
\bgY_{00}(a)
+\sum_{m,n}\gy_{mn}(a,\gq,\gz)
,\quad
\bgY_{00}(a)
\equiv
\R{1}{4\gp^2}\oint\oint\bgY d\gq\ d\gz
\lbq{RMCgY}
\eqbl
can be achieved in a special coordinate system, called the Reference
Magnetic Coordinates (RMC), where only resonant Fourier harmonics
$\gy_{mn}$ enter into $\bgY$. In equilibrium configurations these
oscillating terms determine the magnetic islands, which make the topology
of the magnetic field different from the coordinate system.

The averaged $2\gp\bgY_{00}$ is equal to the poloidal $\gY$ flux of
the magnetic field through the contours $a=const,\gq=const$.
For the purpose of \c{ESI} we consider only  simple nested magnetic
configurations, where
\bqbl
\bgY
=\bgY_{00}(a)
=\bgY(a)
.\eqbl
\c{ESI} can be easily extended in order to cover the general case
(\rfq{RMCgY}) as well.

The periodic function $\gh=\gh(a,\gq,\gz)$ is oscillating in both
$\gq,\gz$ 
\bqbl
\int\gh d\gq=\int\gh d\gz=0
.\eqbl
For special purposes, it can be eliminated by massaging the angles
\bqbl
\gq\to
\bgq
=
\gq
+\ga
,\quad
\gz\to
\bgz+\gb
,\quad
\bgF'\ga+\bgY'\gb
=
\bgF'\gh
\lbq{eq:SFL}
\eqbl
in order to produce the so-called ``straight field line'' coordinates
\bqbl
\waA
=
\bgF\na\bgq
+\bgY\na\gf
.\eqbl
{\em ``Barred'' notations $a,\bgq,\bgz$ are used for the straight field line
coordinates.}

In RMC, the magnetic field has the following contravariant
representation 
\bqbl
\ESIB
\lbq{ESIB}
,\eqbl
where $\gy$ with resonant harmonics describes magnetic islands. 
In flux coordinates ($\waB\cdot\na a=0$, $\gy=0$), the magnetic field
$\waB$ has a simpler form 
\bqbl
\ESIeqB
\lbq{ESIeqB}
.\eqbl
In  2-D case $\waB$ is reduced to
\bqbl
\ESIeqBA
\lbq{ESIeqBA}
.\eqbl

The definitions (\rfq{eq:rzgf}, \rfq{ESIeqB}) introduce two (out of a
total of four in \c{ESI}) 1-D profiles
\bqbl
\bgF'(a)
,\quad
\bgY'(a)
\lbq{eq:gFgY}
\eqbl
and, together with $|\waB|$, four basis functions  of 
\c{ESI} interface
\bqbl
r=r(a,\gq,\gz)
,\quad
z=z(a,\gq,\gz)
,\quad
|\waB|=B(a,\gq,\gz)
,\quad
\gh'_\gq\equiv\gh'_\gq(a,\gq,\gz)
,\\
\gh'_\gz\equiv\gh'_\gz(a,\gq,\gz)
,\quad
\gf=\gf(a,\gq,\gz)
\lbq{eq:rzBgh}
.\eqbl
In \c{ESI}, the units of $r,z$ are in [m], $B$ is in [T], $\gh,\gf$ are
dimensionless, while fluxes $\gF,\gY$ are in [V$\cdot$sec]. Units of
$a,\gq,\gz$ are not essential.

Note, that the basic functions (\rfq{eq:gFgY},\rfq{eq:rzBgh}) represent 
any nested magnetic configurations, even unrelated to a static
equilibrium.

In magneto-static equilibrium
\bqbl
\ESIeq
\lbq{ESIeq}
,\eqbl
where $p$ is the plasma pressure in [MPa], $\wsj$ is the current density
in [MA/m$^2$], and $\gm_0=0.4\gp$, one can introduce two additional 1-D
profiles, i.e., $P(a),T(a)$
\bqbl
\ESIPT
\lbq{ESIPT}
.\eqbl
Here $F=5\baF$ is the total poloidal current in [MA] through the contour
$a=const,\gq=const$ (analog of magnetic flux $\gY$). In axisymmetric 
configurations
\bqbl
\baF=rB_\gf
.\eqbl

\c{ESI} relies on the fact that four space coordinated functions
(\rfq{eq:rzBgh}) and four 1-D profiles
(\rfq{eq:gFgY}), (\rfq{ESIPT}) together with their first derivatives 
contain all necessary information for transport, stability, particle
motion and gyro-kinetic codes. They are called hereafter the ``basis''
functions.

As a special case, \c{ESI} accepts 2-D data on a
rectangular grid $r,z$ from free boundary codes in the form of
$\bgY(r,z),\bgY'_r,\bgY'_z,\bgY''_{rz}$. This is not sufficient to 
generate all possible information for other magnetic confinement 
codes. The special \c{ESI} routine can convert $r-z$ data into basis 
functions of flux coordinates.

{\em \c{ESI} reconstruction routines do not provide the second derivatives
 of basis functions}. The following sections describe the
 self-consistent rules of using basis function for different kind of
 numerical codes without use of second derivatives.

\Section{Main reconstruction routine of \c{ESI}}
Technically, \c{ESI} consists of a data set (file, or a segment of the
shared memory), which should be provided by an equilibrium code, and a
universal C-source file (\c{esiZ.c}), which contains \c{ESI} initiation
routines, the main reconstruction routine, and an unspecified set of
service routines (see Fig.1)

\btbl{c}
\psfig{figure=/w/Esc/Doc/esiZ.f0.eps,width=5in}\\
{\em \parbox{6in}{Fig.1. Data generating equilibrium code, \c{ESI}, and
the user's sides.}}
\etbl
\noindent The source code \c{esiZ.c} is self-contained and can be
compiled by a simple command
\vskip0.5em
\c{cc -c -o esiZ.o esiZ.c}
\vskip0.5em
All its routines are \c{int} functions, which return 0 in the case of
success, and non-zero otherwise. In order to initiate \c{ESI} from a 
data file (which should be created by an equilibrium code), the user
should use the C-code, like 
\begin{verbatim}
  if(File2ESI(FileName)){
    printf("Failure: %s is not an ESI-file\n'',FileName);
  }
\end{verbatim} 
or FORTRAN-code, like 
\begin{verbatim}
     external integer file2esi
     if(file2esi(FileName).ne.0) then
       write(*,'(aaa)')'Failure: ',FileName,' is not an ESI-file'
     endif
\end{verbatim} 
{\em The FORTRAN file name \c{FileName} in \c{file2esi} {\bf\em should
 contain} a space character \c{' '} at the end !} 

In the case of using shared memory the corresponding function is
\c{Shmem2ESI(int key)}, where \c{'key'} is a unique identification
number for the memory segment, known to mutually communicating
equilibrium and user codes.

The main reconstruction routine of \c{ESI} takes coordinates from a 
number of points inside the plasma and puts the values of the basis 
profiles and space functions into the arrays, allocated at the user
side. 
{\em The user code should provide the storage for these arrays.}
The example is the following C-code
\begin{verbatim}
  int n;
  double a[N],gq[N];
  double F[N],Fa[N],gFa[N][N],gFaa[N],gYa[N],gYaa[N]
    ,T[N],Ta[N],P[N],Pa[N]
    ,r[N],ra[N],rq[N],z[N],za[N],zq[N]
    ,B[N],Ba[N],Bq[N],gh[N],gha[N],ghq[N];
  int isw[N];
  double gz[N],rz[N],zz[N],Bz[N],ghz[N];
\end{verbatim} 
The number $N$ should not be less than the number of points requested in
reconstruction. The last line is necessary only for the 3-D case. The
FORTRAN analog of the same would be 
\begin{verbatim}
      integer n;
      double precision a(N),gq(N)
      double precision F(N),Fa(N),gFa(N)(N),gFaa(N),gYa(N),gYaa(N)
    &      ,T(N),Ta(N),P(N),Pa(N)
    &      ,r(N),ra(N),rq(N),z(N),za(N),zq(N)
    &      ,B(N),Ba(N),Bq(N),gh(N),gha(N),ghq(N)
      integer isw(N);
      double precision gz(N),rz(N),zz(N),Bz(N),ghz(N);
      external integer link2esi,link2esi3d,esi2all,esi2all3d
\end{verbatim} 
The meaning of this arrays are specified in the Table 1.
\btbl{|l|l||l|l|}\hline
\multicolumn{3}{|c|}{\bf Set of output radial profiles}&\hfil\em Table 1
\\\hline
\c{C}-code& Math &\c{C}-code& Math\\\hline
\c{F[],Fa[]}&$\baF,\baF'$&\c{isw[]}&index of the particle\\\hline
\c{gFa[],gFaa[]}&$\bgF',\bgF''$
&\c{gYa[],gYaa[]}&$\bgY',\bgY''$\\\hline
\c{T[],Ta[]}&$\baF\baF'/\bgY'$
&\c{P[],Pa[]}&$\bsp'/\bgY'$\\\hline\hline
\multicolumn{4}{|c|}{\bf Set of 2-D and 3-D output space functions}\\\hline
\c{C}-code& Math &\c{C}-code& Math\\\hline
\c{r[],ra[],rq[],rz[]}	&$r,r'_a,r'_\gq,r'_\gz$
&\c{z[],za[],zq[],zz[]}	&$z,z'_a,z'_\gq,z'_\gz$\\\hline 
\c{B[],Ba[],Bq[],Bz[]}	&$B,B'_a,B'_\gq,B'_\gz$
&\c{gh[],gha[],ghq[],ghz[]}&$\gh,\gh'_a,\gh'_\gq,\gh'_\gz$\\\hline
\etbl

The addresses of these arrays should be given to \c{ESI}, at least once,
using 
\begin{verbatim}
i=Link2ESI(F,Fa,gFa,gFaa,gYa,gYaa,T,Ta,P,Pa
           ,r,ra,rq,z,za,zq,B,Ba,Bq,gh,gha,ghq,isw);
i=Link2ESI3d(rz,zz,Bz,ghz);
\end{verbatim} 
from C-code or from FORTRAN as
\begin{verbatim}
        i=link2esi(F,Fa,gFa,gFaa,gYa,gYaa,T,Ta,P,Pa
     &    ,r,ra,rq,z,za,zq,B,Ba,Bq,gh,gha,ghq,isw)
        i=link2esi3d_(rz,zz,Bz,ghz)
\end{verbatim} 
Again, the last line is necessary only for the 3-D case. At present, the
main reconstruction routine does not use the index array \c{isw[i]},
which can be used as a marker of points to control some service
routines.

After all of this, the call of the main reconstruction routine is
simple, as soon as the coordinates of $\c{n}\leq\c{N}$ points $a,\gq$
(and $\gz$ in 3-D case) are put into array \c{a[],gq[]} (and \c{gz[]})
\begin{verbatim}
  i=ESI2all(a,gq,n); 
\end{verbatim} 
or for 3-D case
\begin{verbatim}
  i=ESI2all3d(a,gq,gz,n);
\end{verbatim} 
The FORTRAN analog of the same is 
\begin{verbatim}
        i=esi2all(a,gq,n);
        i=esi2all3d(a,gq,gz,n);
\end{verbatim} 
After these calls the information is is the mentioned arrays. In
addition to the main reconstruction routine, \c{ESI} contains a set of
service routine for calculating specialized information. Some of them
are listed at the end of this document, and their set can be extended.

\Section{Metrics of toroidal configurations}
The metric tensor $g_{ik}$ ($i=1,2,3$, $k=1,2,3$) of the coordinate
system $\{a,\gq,\gz\}=\{x^1,x^2,x^3\}$ is defined by the element of
length $dl$ 
\bqbl
\ESIgik
\lbq{ESIgik}
.\eqbl

Gradient vectors of cylindrical and curvilinear coordinates are related
with each other by a matrix of first derivatives $\maD$
\bqbl
\maD
\equiv
\left|\arr
r'_a&r'_\gq&r'_\gz\\
\gf'_a&\gf'_\gq&\gf'_\gz\\
z'_a&z'_\gq&z'_\gz\\
\ear\right|
,\quad
\left|\arr
\na r\\
\na\gf\\
\na z\\
\ear\right|
=
\maD
\cdot
\left|\arr
\na a\\
\na\gq\\
\na\gz\\
\ear\right|
,\quad
\left|\arr
\na a\\
\na\gq\\
\na\gz\\
\ear\right|
=
\maD^{-1}
\cdot
\left|\arr
\na r\\
\na\gf\\
\na z\\
\ear\right|
.\eqbl
The Jacobian $J$ of the coordinate system can be calculated as the
determinant $D$ of the matrix of first derivatives of cylindrical
coordinates 
\bqbl
D
=
\mbox{Det}(\maD)
,\quad
J=
rD
=
\q{g}
,\eqbl
where $g$ is the determinant of the matrix $g_{ik}$.

In axisymmetric configurations, only
 $g_{aa},g_{a\gq},g_{\gq\gq},g_{\gf\gf}$ are non-vanishing
\bqbl
\ESIgikA
\lbq{ESIgikA}
.\eqbl
\c{ESI} reconstruction routines provide all necessary derivatives for
calculation of $g_{ik}$, elements of matrix $\maD$, and Jacobian $J$ at
 the user side. {\em The derivatives of $g_{ik},\maD,J$ are not defined
 by \c{ESI}}.

\Subsection{Behavior near the origin $a=0$}
In configurations with a smooth current density near the axis $a=0$, flux
coordinates have a special behavior near the axis.
Thus, in some coordinate systems (n.n. 0,1,2,3,7 in Table 2), 
Fourier coefficients of $r(a,\gq,\gz), z(a,\gq,\gz)$ 
\bqbl
r
=
r_0+\sum^{m\leq M_\gq}_{m=1}\left[r_me^{im\gq}
 +r^*_me^{-im\gq}\right]
,\quad
z
=
z_0+\sum^{m\leq M_\gq}_{m=1}\left[z_me^{im\gq}
 +z^*_me^{-im\gq}\right]
\eqbl
vanish at $a=0$ at a certain rate. If $b=b(a)$ is a function proportinal to
the distance from the axis, then asymptotically
\bqbl
r_0(a,\gz)
\simeq
R_0(\gz)+\bsr_0(\gz)b^2 
,\quad
r_1(a,\gz)
\simeq
\bsr_1(\gz)b
,\quad
r_2(a,\gz)
\simeq
\bsr_m(\gz)b^2
,\quad
r_{m>2}(a,\gz)
\simeq
\bsr_m(\gz)b^m
.\eqbl
{\em The \c{ESC} data generated by the equilibrium codes in these
  coordinates should reflect this behaviour for
  $r_0,r_1,r_2,z_0,z_1,z_2$.} All other coefficients should vanish near
the axis, at least, like $b^2$. This would allow to resolve the details,
essential for some instabilities.

In other coordinates like polar ($\c{ID}_\gq$=8 in
Table 2), otherwise very good, such a bahavior is hidden more
deeply. This may slightly limit the application of \c{ESI} in such
coorinates.

\Subsection{Conversion of laboratory $r,\gf,z$ into $a,\gq,\gz$}
For the purpose of diagnostics on the experimental machines, it is
necessary to have an ablity of conversion of laboratory coordinates
$r,\gf,z$ into curvilinear coordinates $a,\gq,\gz$. This is
done in \c{ESI} by the Newton iterative procedure 
\bqbl
\left|\arr
da^{(k)}\\
d\gq^{(k)}\\
d\gz^{(k)}\\
\ear\right|
=
\left(\maD^{-1}\right)^{(k)}
\cdot
\left|\arr
r-r^{(k)}\\
\gf-\gf^{(k)}\\
z-z^{(k)}\\
\ear\right|
,\quad
\arr
a^{(k+1)}&=a^{(k)}&+da^{(k)}\\
d\gq^{(k+1)}&=\gq^{(k)}&+d\gq^{(k)}\\
d\gz^{(k+1)}&=\gz^{(k)}&+d\gz^{(k)}\\
\ear
,\eqbl
where $k$ is the iteration counter.

\Section{Nested magnetic configurations}
The vector potential $\waA$ in Eq.(\rfq{ESIA}) contains integrals from
the basis profiles of
\c{ESI}
\bqbl
\gF
=
2\gp\int^a_0\bgF'(a)da 
,\quad
\gY
=
2\gp\int^a_0\bgY'(a)da 
,\quad
\gh
=
\int^\gq_0\gh'_\gq d\gq 
+\int^\gz_0\left(\R{1}{2\gp}\oint\gh'_\gz d\gq\right)d\gz
.\eqbl
There is a special service routine in \c{ESI} which calculates $\gY,\gF$
for the transport codes, while $\gh$ and its derivatives are calculated
by the main reconstruction routine.

\Subsection{Magnetic field}
The contravariant representation of the magnetic field is given by
Eq.(\rfq{ESIeqB}), or in the 2-D case by Eq.(\rfq{ESIeqBA}). It can be
reconstructed from \c{ESI} basis functions in a straightforward manner.
On the other hand the derivatives of individual components of the
magnetic field are not defined.

The covariant components of magnetic field $B_i$ (integer subscript $i$
is used to distinguish them from the physical components),
\bqbl
\ESIBco
\lbq{ESIBco}
\eqbl
are useful for particle motion. Herethe  summation convention is assumed.

In flux coordinates (no islands) $B^1=B^a=0$ and only the following metric
tensor combinations are present in Eq.(\rfq{ESIBco})
\bqbl
\ESIeqKMN
\lbq{ESIeqKMN}
.\eqbl
All of them can be calculated at the user side using the relationship
between co- and contra-variant components of the vector $\waB$.

In the case of an ``ideal'' (isotropic) equilibrium, the covariant
components have a special form
\bqbl
\ESIeqBco
\lbq{ESIeqBco}
\eqbl
and can be calculated using
\bqbl
\ESIeqBcocn
\lbq{ESIeqBcocn}
.\eqbl
For the case of axisymmetry
\bqbl
\ESIeqBcocnA
\lbq{ESIeqBcocnA}
.\eqbl

\Subsection{Current density}
In equilibrium configurations the current density $\b{\wsj}$ has the
following contravariant form 
\bqbl
\ESIeqbsj
&&
\lbq{ESIeqbsj}
,\eqbl
where $I(a)$ is the toroidal current through the poloidal cross-section
of magnetic surfaces. Both $\baF,\baF'$, together with $\bsp,\bsp'$ are
calculated inside \c{ESI} using
\bqbl
\baF'=
\R{T\bgY'}{\baF}
,\quad
\baF(a)=\q{\baF^2_{ref}+2\int^a_{a_{ref}}T(a)\bgY'da}
,\quad
\bsp'=
P\bgY'
,\quad
\bsp(a)
=
\int^a_{a_v}P\bgY'da
,\quad
\lbq{eq:baF}
.\eqbl
Here, $a_{ref}$ is a reference radius, where the function $\baF$ has a
prescribed value, $\baF(a_{ref})=\baF_{ref}$. Typically $a_{ref}$
corresponds to the plasma boundary. The exceptional case is the RFP
configurations, where $a_{ref}$ should specify the surface with 
$\baF(a_{ref})=0$. The main \c{ESI} reconstruction routine delivers
$\baF,\baF'$, while $\bsp,\bsp'$ are provided by a special service
routine. {\em For RFP case the sign of $\baF$ cannot be determined from
 the \c{ESI} data. It is chosen using convention $\baF(0) > 0$}.

The radial component of equilibrium equation (\rfq{ESIeq}) (the pressure
balance)
\bqbl
\ESIeqpb
\lbq{ESIeqpb}
\eqbl
determines $\baI'(a)$ inside \c{ESI} from the averaged
Eq.(\rfq{ESIeqpb})
\bqbl
PJ_{00}
=
-\bgF'\R{T}{\baF}+\baI'
,\quad
J_{00}
\equiv
\R{1}{4\gp^2}\oint\oint Jd\gq d\gz
,\quad
J
\equiv
J_{00}
+J_{\til\til}
.\eqbl
The oscillatory part of this equation, having the form of the so-called
magnetic differential equation (MDE)
\bqbl
\ESIeqgn
\lbq{ESIeqgn}
\eqbl
allows one to determine the function $\gn$. In the case of axisymmetry this
is simply given by an integral
\bqbl
\baI'
-\gn'_\gq
=
TL
+PJ
,\quad
\gn'_\gq
=
-TL_{\til\til}
-PJ_{\til\til}
,\quad
\gn
=
-\int^\gq_0(TL_{\til\til}+PJ_{\til\til})d\gq
.\eqbl
In the case of 3-D equilibrium configurations, it necessary to solve MDE
(\rfq{ESIeqgn}). In configurations with simple nested surfaces (with no
islands), the Jacobian does not contain resonance harmonics, and MDE for
$\gn$ has a nonsingular periodic solution. This is described in the next
section. A special service routine of \c{ESI} generates $\gn$.

Thus, all physical quantities in the equilibrium configurations can be
calculated from \c{ESI} basis functions without using their second
derivatives.

Note, that the output information from \c{ESI} can appear to be
inconsistent. For example, in 2-D, the third equation in
(\rfq{ESIeqBcocnA}) gives the relationship
\bqbl
\bgF'=L_0\baF
,\quad
L_0\equiv\R{1}{2\gp}\oint\R{D}{r}d\gq
\lbq{eq:L0}
,\eqbl
which gives an alternative to the Eq.(\rfq{eq:baF}) way of calculating 
$\baF$. The level of discrepancy between the two calculations may be
used for testing the accuracy of \c{ESI} data.

In fact, the user has no information from \c{ESI} on how to perform
integration without interpolation of the integrand, which makes such
comparison ``illegal'' within \c{ESI} rules. Instead, the main
reconstruction routine of \c{ESI} provides the values of
$\baF,\baF',\bgF,\bgF'$ explicitly, thus, giving
$L_0,L'_0$. Alternatively, a specially written service routine of \c{ESI}
can provide $L_0,L'_0$, consistent with relation (\rfq{eq:L0}).

\Subsection{Magnetic differential equation (MDE)}
Assume straight field line coordinates $a,\gq,\gz$ with
$\gh=0$. Then, the magnetic differential equation (\rfq{ESIeqgn}) has a
solution 
\bqbl
q
\equiv
-\R{\bgF'}{\bgY'}
,\quad
\gn'_\gq+q\gn'_\gz
=
P(a)\taJ(a,\gq,\gz)
\equiv
S(a,\gq,\gz)
,\\
\gn(a,\gq,\gz)
=
A(a,\gz-q\gq)
-\int^\gq_0S(a,\ga,\gz+q\ga-q\gq)d\ga
,\\
A(a,\gz)
-A(a,\gz-2\gp q)
=
\int^{2\gp}_0S(a,\ga,\gz+q\ga-2\gp q)d\ga
\lbq{eq:gn}
,\eqbl
where a 2-D function $A(a,\gz)$ is added in order to provide the
periodicity of $\gn$ as function of $\gq$. The equation for $A$ can be
solved directly numerically in Fourier space
\bqbl
S(a,\gq,\gz)
\equiv
\sum_nS_n(a,\gq)e^{in\gz}
,\quad
A(a,\gz)
\equiv
\sum_nA_n(a)e^{in\gz}
,\quad
A_n(a)
=
\R{e^{-in\gp q}}{2\sin n\gp q}\int^{2\gp}_0S_n(a,\ga)e^{inq\ga}d\ga
\lbq{eq:gnA}
.\eqbl
In the general case when $\gh\neq0$, the equation for $\gn$ (\rfq{ESIeqgn})
can be written as
\bqbl
q(1+\gh'_\gq)\gn'_\gz
+(1-q\gh'_\gz)\gn'_\gq
=
\taJ P
-q(\baF'\gh'_\gq-\baI'\gh'_\gz)
\equiv
(1+\gh'_\gq)S(a,\gq,\gz)
.\eqbl
A transition to a new variable $\bgq=\gq+\gh$ in $\gn(a,\gq+\gh,\gz)$
leads to 
\bqbl
q\gn'_\gz
+\gn'_\bgq
=
S(a,\bgq-\gh,\gz)
,\eqbl
whose solution is given by Eqs.(\rfq{eq:gn},\rfq{eq:gnA}). 

Because of resonant denominators $nq=m$ ($m,n$ are integers), the function
$\gn$ may have a rather complicated behavior even when the MDE can be
resolved. It cannot be well represented by smooth polynomials on a
finite grid, and its calculations cannot be delegated to the
user. Specially designed \c{ESI} routines should reconstruct $\gn$ upon the
user's request. 

\Subsection{Curvature of the magnetic field lines}
The curvature of the magnetic field lines, essential for some
instabilities,
\bqbl
&&
\wsk
\equiv
\left(\R{\waB}{|B|}\cdot\na\right)\R{\waB}{|B|}
=
-\left(\R{\waB}{|B|}\times\left(\na\times\R{\waB}{|B|}\right)\right)
=
\R{\bsp'}{|B|^2}\na a
-\R{((\waB\cdot\na)|B|)}{|B|^3}\waB
+\R{|B|'_a}{|B|}\na a
+\R{|B|'_\gq}{|B|}\na\gq
\eqbl
can be calculated at the user's side using the data from \c{ESI}
reconstruction routines.

\Subsection{Evolution equation for magnetic fluxes}
In evolving magnetic configurations, the electric field $\waE$
can be written as
\bqbl
\waE
=
&&
-\R{\pa \waA}{\pa t}
-\na \gF_E
.\eqbl
Elimination of the scalar potential $\na \gF_E$ from the classical parallel
Ohm's law 
\bqbl
\gs_\parallel(\waB\cdot\waE)=(\waB\cdot\wsj) 
\eqbl
using averaging gives the evolution equation for magnetic fluxes
\bqbl
\bgF'_a\gY'_t
-\bgY'_a\gF'_t
=
\R{1}{\gs_\parallel}(\baF\baI'-\baI\baF')
.\eqbl
It can be rewritten as a magnetic diffusion equation
\bqbl
\gY'_t
-\R{\bgY'_a}{\bgF'_a}\gF'_t
=
\R{1}{\gs_\parallel}\left[\R{\bgF'\baF-\bgY'\baI}{\bgF'^2}
\left(
   {g_{\gq\gq}\over\q{g}}(\gY'+\gh'_\gz)-{g_{\gq\gz}\over\q{g}}(\gF'+\gh'_\gq)
  \right)'_{00} 
-\R{\baI\bsp'}{\bgF'^2}(\q{g})_{00}
\right]
,\\
\baI
=
-\left(\R{g_{\gq\gq}}{\q{g}}(\gY'+\gh'_\gz)
-\R{g_{\gq\gz}}{\q{g}}(\gF'+\gh'_\gq)\right)_{00} 
.\eqbl
The averaging can be performed on the user's side, while \c{ESI}
reconstruction provides data for all coefficients in this equation.

When the radial coordinate $a$ represents the toroidal magnetic flux
(e.g., in the ASTRA code $a=\q{\R{2\bgF}{B_0}}$, $B_0=$ const), the
convective term in the left hand side vanishes, $\bgF'_t=0$.

\Subsection{Transport equations}
The structure of the transport equations is given by
\bqbl
\R{\pa }{\pa t}(Jn)_{00}
+\R{\pa }{\pa a}\left(J\vgG\cdot\na a\right)_{00}
=
(JS)_{00}
,\quad
J\equiv\q{g}
,\eqbl
where $n$ is a physics variable, $\gG$ is its flux through the magnetic
surface (both diffusive and convective), and $S$ are the sources and
sinks.

\c{ESI} reconstruction routines provide the necessary information for
calculating coefficients of the transport equations on the user's side. In
contrast, the approach based on ``cooking'' out all the inforamtion
inside the equilibrium codes is always deficient due to the
unpredictable nature of plasma transport models.

\Subsection{Transition to the straight field line (SFL) coordinates} 
Straight field line coordinates are important for stability theory
and for stellarator equilibrium calculations. \c{ESI} reconstruction
routines provide the necessary information for making transition to the 
SFL coordinates $a,\gq,\bgz$
\bqbl
\gq=\gq-\ga(a,\gq,\bgz)
,\quad
\gz=\bgz-\gb(a,\gq,\bgz)
,\\
d\gq
=
(1-\ga'_\gq)d\gq
-\ga'_ada
-\ga'_\gz d\bgz
,\quad
d\gz
=
(1-\gb'_\bgq)d\bgq
-\gb'_ada
-\gb'_\gq d\gq
\lbq{eq:SFL}
,\eqbl
where either $\ga$ or $\gb$ is an arbitrary function, related by
Eq. (\rfq{eq:SFL})
\bqbl
\bgF'\ga+\bgY'\gb=\bgF'\gh
.\eqbl
Substitution of $d\gq,d\gz$ (\rfq{eq:SFL}) info element of the length
(\rfq{ESIgik}) allows calculation of the metric tensor in new
coordinates. Thus, with the 
function $\gh$ in the set of basis function the \c{ESI} main routine
provides information for transition into any SFL coordinates.

\Section{Field lines and particle motion} 
\Subsection{Equation for magnetic field lines} 
In curvilinear coordinates, the magnetic field lines are determined by
three ordinary differential equations 
\bqbl
\R{d\gq}{dl}
=
\R{B^\gq}{|B|}
,\quad
\R{d\gz}{dl}
=
\R{B^\gz}{|B|}
,\quad
\R{da}{dl}
=
\R{B^a}{|B|}
\lbq{GCmfl}
,\eqbl
where $B^a,B^\gq,B^\gz$ are given by Eq.(\rfq{ESIB}). In the case of
of magnetic configurations with the nested surfaces, $a=$const, and only
the two first equations are essential. \c{ESI} provides a special step
advancing routine \c{ESI2mfl()} for calculating derivatives for further
use in ODE solvers.

\Subsection{Guiding center Lagrangian equations} 
The guiding center Lagrangian  obtained by Littlejohn has the following
normalized form
\bqbl
&&
L
=
\left(\waA+\gr_\parallel\waB\right)\cdot\wsv-H
,\quad
\wsv
\equiv
\R{d\wsr}{d\gt}
,\quad
\gt
\equiv
\gW_ct
,\quad
\gW_c
\equiv
\R{eB_0}{mc}
,\\&&
H
\equiv
\gr^2_\parallel\R{B^2}{2}
+\gm B
+\gj_E
,\quad
\gm
\equiv
\R{\gr^2_\perp}{2B}
,\quad
\gj_E
\equiv
\R{mc^2\gF_E}{e}
\equiv
\gr_ER_0B^2_0
,\quad
\gr_E
\equiv
\R{c\gF_E}{R_0B_0\gW_c}
.\eqbl
Here, $\wsv$ is considered as the normalized time derivative of the
particle position, $\gW_c$ is the ion cyclotron frequency in the
reference magnetic field $B_0$ whatever is used for normalization (e.g.,
for \c{ESI} Units, $B_0=1$ T and $\gW_c=95.79\cdot10^{+6}$ sec$^{-1}$ for
protons), $\gr_\parallel,\gr_\perp$ are the ion Larmor radii calculated
based on particle velocities parallel and perpendicular to the field. In
the Hamiltonian $H$ the last term $\gj_E$ is associated with the electric
field potential $\gF_E$ and can be expressed in terms of the Larmor radius
$\gr_E$ calculated based on drift velocity in the effective electric
field $\gF_E/R_0$ where $R_0$ is some characteristic unit length (e.g.,
1 m).

Written for generalized coordinates of a particle $a,\gq,\gz$, the
Lagrangian has the form
\bqbl
&&
L=
P_a\d{a}
+P_\gq\d{\gq}
+P_\gz\d{\gz}
-H
,\\&&
P_a
\equiv
\gr_\parallel B_a
,\quad
P_\gq
\equiv
\bgF+\int^a_0\bgF'\gh'_\gq da+\gr_\parallel B_\gq
,\quad
P_\gz
\equiv
\bgY+\int^a_0\bgF'\gh'_\gz da+\gr_\parallel B_\gz
.\eqbl
The straightforward variation of $L$ with respect to
$\gr_\parallel,a,\gq,\gz$ leads to the following system of coupled
equations of motion
\bqbl
\left|\arr
0
&B_a
&B_\gq
&B_\gz
\vrule height20pt width0pt\\
-B_a
&0
&J(B^\gz+\gr_\parallel\bsj^\gz)
&-J(B^\gq+\gr_\parallel\bsj^\gq)
\vrule height20pt width0pt\\
B_\gq
&J(B^\gz+\gr_\parallel\bsj^\gz)
&0
&-J(B^a+\gr_\parallel\bsj^a)
\vrule height20pt width0pt\\
B_\gz
&-J(B^\gq+\gr_\parallel\bsj^\gq)
& J(B^a+\gr_\parallel\bsj^a)
&0
\vrule height20pt width0pt\\
\ear
\right|
\cdot
\left|\arr
\d{\gr}_\parallel\vrule height22pt width0pt\\
\d{a}\vrule height22pt width0pt\\
\dgq\vrule height22pt width0pt\\
\d\gz\vrule height22pt width0pt\\
\ear
\right|
=
\left|\arr
\R{\pa H}{\pa\gr_\parallel}\vrule height16pt width0pt\\
\R{\pa H}{\pa a}\vrule height16pt width0pt\\
-\R{\pa H}{\pa\gq}\vrule height16pt width0pt\\
-\R{\pa H}{\pa\gz}\vrule height16pt width0pt\\
\ear
\right|
.\eqbl
or to wrong ones
\begin{eqnarray}
&&
\left|
\begin{array}{rrrr}
\db0
\da B_a
\da B_\gq
\da B_\gz
\dd
\dd-B_a
\da0
\da \q{g}(B^\gz+\gr_\parallel j^\gz)
\da -\q{g}(B^\gq+\gr_\parallel j^\gq)
\dd
\dd B_\gq
\da \q{g}(B^\gz+\gr_\parallel j^\gz)
\da0
\da0
\dd
\dd B_\gz
\da-\q{g}(B^\gq+\gr_\parallel j^\gq)
\da0
\da0
\end{array}
\right|
\cdot
\left|
\begin{array}{l}
\db\d{\gr}_\parallel
\dd
\dd\d{a}
\dd
\dd\d{\gq}
\dd
\dd\d{\gz}
\end{array}
\right|
=
\left|
\begin{array}{r}
\db\R{\pa H}{\pa\gr_\parallel}
\dd\R{\pa H}{\pa a}
\dd-\R{\pa H}{\pa\gq}
\dd-\R{\pa H}{\pa\gz}
\end{array}
\right|
.\end{eqnarray}
Using the matrix
\begin{eqnarray}
&&
\maA
\equiv
\left|
\begin{array}{rr}
\db a_{11}
\da a_{12}
\dd a_{21}
\da a_{22}
\end{array}
\right|
,\quad
|\maA|
\equiv
a_{11}a_{22}-a_{12}a_{21}
=
\q{g}[|B|^2+\gr_\parallel(\waB\cdot(\na\times\waB))]
,\end{eqnarray}
where
\bqbl
&&
a_{11}\equiv B_\gq
,\quad a_{12}\equiv \q{g}(B^\gz+\gr_\parallel j^\gz)
,\quad a_{21}\equiv B_\gz
,\quad a_{22}\equiv-\q{g}(B^\gq+\gr_\parallel j^\gq)
,\eqbl
the equation of drift motion can be written in the explicit form
\begin{eqnarray}
&&
\left|
\begin{array}{l}
\db\dgr_\parallel
\dd\dd
\dd\dsa
\dd\dd
\dd\dgw
\dd\dd
\dd\dgz
\end{array}
\right|
=
\left|
\begin{array}{rrrr}
\db0
\da0
\da \R{a_{22}}{|\maA|}
\da -\R{a_{12}}{|\maA|}
\dd
\dd0
\da0
\da -\R{a_{21}}{|\maA|}
\da \R{a_{11}}{|\maA|}
\dd
\dd \R{a_{22}}{|\maA|}
\da -\R{a_{21}}{|\maA|}
\da0
\da-\R{B_a}{|\maA|}
\dd
\dd -\R{a_{12}}{|\maA|}
\da \R{a_{11}}{|\maA|}
\da\R{B_a}{|\maA|}
\da0
\end{array}
\right|
\cdot
\left|
\begin{array}{r}
\db\R{\pa H}{\pa\gr_\parallel}
\dd
\dd\R{\pa H}{\pa a}
\dd
\dd-\R{\pa H}{\pa\gq}
\dd
\dd-\R{\pa H}{\pa\gz}
\end{array}
\right|
\lbq{GCgcm}
.\end{eqnarray}
\c{ESI} allows calculation of the right hand side of the guiding center
equations at any point inside the plasma without interpolations. In
fact, \c{ESI} contains a guiding center motion routine which calculates
the derivatives $\dgr_\parallel,\dsa,\dgw,\dgz$ given
$\gr_\parallel,a,\gq,\gz$.

\Subsection{Hamiltonian equations for guiding center motion} 
By transformation of angle coordinates it is possible to eliminate the 
$B_a=\gn+\gs_a$ term in the covariant representation of $\waB$ 
(\rfq{ESIeqBco}) together with the $P_a$ term in Lagrangian. For this 
purpose, new, canonical coordinates $\hgq,\hgz$ are determined by 
\bqbl
\hgq
=
\gq
+\ga
,\quad
\hgz
=
\gz
+\gb
\lbq{GCCan0}
,\eqbl
where $\ga,\gb$ satisfy equations
\bqbl
\gn\na a
+\na\gs
+\na(\baI\ga)
+\na(\baF\gb)
-\baI'\ga\na a
-\baF'\gb\na a
=
0
,\\
\baI'\ga
+\baF'\gb
=
\gn
,\quad
\baI\ga
+\baF\gb
=
-\gs
,\quad
\ga
=
-\R{\baF\gn+\baF'\gs}{\baF\baI'-\baI\baF'}
,\quad
\gb
=
\R{\baI\gn+\baI'\gs}{\baF\baI'-\baI\baF'}
\lbq{GCCan}
.\eqbl
In canonical coordinates $a,\hgq,\hgz$ the covariant representation of
the magnetic field is very simple 
\bqbl
\waB
=
\baI\na\hgq
+\baF\na\hgz
.\eqbl
The vector potential becomes
\bqbl
\waA
=
-\bgF'\hgh\na a
+\bgF\na\hgq
+\bgY\na\hgz
,\quad
\bgF'\hgh
\equiv
\bgF'\gh
-\bgF'\ga
-\bgY'\gb
\eqbl
Accordingly, the Lagrangian is reduced to the Hamiltonian form
\bqbl
L=
P_\bgq\d{\hgq}
+P_\hgz\d{\hgz}
-H
,\quad
P_\hgq
\equiv
\bgF+\int^a_0\bgF'\hgh'_\gq da+\gr_\parallel\baI
,\quad
P_\hgz
\equiv
\bgY+\int^a_0\bgF'\hgh'_\gz da+\gr_\parallel\baF
.\eqbl
with no explicit time derivative $\d{a}$. The equation of motion for
$a,\gq,\gz$ can be now written as 
\bqbl
\left|\arr
0&0&\R{\pa P_\hgq}{\pa\gr_\parallel}&\R{\pa P_\hgz}{\pa\gr_\parallel}\\
0&0&\R{\pa P_\hgq}{\pa a}&\R{\pa P_\hgz}{\pa a}\\
\R{\pa P_\hgq}{\pa\gr_\parallel}&\R{\pa P_\hgq}{\pa a}&0&0\\
\R{\pa P_\hgz}{\pa\gr_\parallel}&\R{\pa P_\hgz}{\pa a}&0&0\\
\ear\right|
\cdot
\left|\arr
\d{\gr}_\parallel\\
\\
\d{a}\\
\\
\d{\hgq}\\
\\
\d{\hgz}\\
\ear\right|
=
\left|\arr
\R{\pa H}{\pa\gr_\parallel}\\
\R{\pa H}{\pa a}\\
-\R{\pa H}{\pa\hgq}\\
-\R{\pa H}{\pa\hgz}\\
\ear\right|
.\eqbl
These equations are equivalent to Hamiltonian equations, if momenta
$P_\hgq,P_\hgz$ are used instead of $\gr_\parallel,a$ 
\bqbl
\dgq
=
\R{\pa H}{\pa P_\hgq}
,\quad
\dgz
=
\R{\pa H}{\pa P_\hgz}
,\quad
\daP_\hgq
=
-\R{\pa H}{\pa\hgq}
,\quad
\daP_\hgz
=
-\R{\pa H}{\pa\hgz}
.\eqbl
The Hamiltonian form of the equations of motion guarantees that the
guiding center motion not only preserves the value of Hamiltonian $H$ (and,
in the case of symmetry, the momentum $P_\hgz$), but also
preserves the phase volume of an ensemble of particles in the phase
space $\hgq,\hgz,P_\hgw,P_\hgz$.

For calculations of guiding center motion in unperturbed equilibrium
configurations the Hamiltonian form does not suggest any advantage
with respect to the Lagrangian form. Canonical variables are essential
only for studies of perturbed particle motion, where adiabatic
invariants play the key role.

Note, that there are other choices of canonic variables than
Eq.(\rfq{GCCan}). Indeed, in order to eliminate $B_a$ it is sufficient
to satisfy only the $\na a$ component of the first equation in
Eq.(\rfq{GCCan}) 
\bqbl
\gn
+\gs'_a
+\baI\ga'_a
+\baF\gb'_a
=
0
,\quad
\gb
=
-\int^a_0
\R{\gn+\gs'_a+\baI\ga'_a}{\baF}da
.\eqbl
The case with $\ga=0$ was pointed out earlier. The attempt to use the
freedom for elimination of the $\gh$ term in the vector potential (making SFL
canonical coordinates) leads to substantial, unjustified
complications in calculations. The canonic variables $a,\hgq,\hgz$,
suggested by Eq.(\rfq{GCCan}) represent the best choice.

\c{ESI} interface is capable of providing a transition to canonical
coordinates $a,\hgq,\hgz$ and to a Hamiltonian representation of guiding
center motion.

\Section{The MHD energy principle}
The well-known functional of potential energy in ideal magneto-hydrodynamics
\bqbl
W
=
\R{1}{8\gp}\int\left\{
\t\waB^2
+(\vgx\cdot\na\bsp)(\na\cdot\vgx)
-\vgx\times(\na\times\waB)\cdot\t\waB
+\gg_0\bsp(\na\cdot\vgx)^2
\right\}dV
,\quad
\t\waB
\equiv
(\waB\times(\na\times\vgx))
,\eqbl
where $\vgx$ is the test function, does not contain anything that
cannot be calculated using basis functions of \c{ESI}. Also, the kinetic energy
term
\bqbl
K
=
\gg^2\int\gr\vgx^2dV
\eqbl
needs only the metric tensor for its calculation.

At a deeper level, instead of three contravariant components of $\vgx$
\bqbl
&&
\gx^{a,\gq,\gz}\equiv
\left\{\gx,\gx^\gq,\gx^\gz\right\}
,\quad
\gx
\equiv
\gx^a
\eqbl
another set of test functions is used, i.e, $\gx,\gL,X$, where
\bqbl
&&
X\equiv
(\na\cdot\vgx)
,\quad
\gL\equiv\bgF'\gx^\gq+\bgY'\gx^\gz
.\eqbl
In terms of $\gx$ and $\gL$ the perturbation of the magnetic field can
be written as
\bqbl
J\t{\waB}
=
\gL(\na\gq\times\na\gz)
-\bgF'\gx(\na\gz\times\na a)
-\bgY'\gx(\na a\times\na\gq)
.\eqbl
In new variables the potential energy has the form
\bqbl
W
=
\r{8\gp}&&\int\left\{
\R{\gg_0\bsp}{\q{g}}X^2
+\q{g}\left(\taB_a\taB^a+\taB_\gq\taB^\gq+\taB_\gz\taB^\gz\right)
-P'\bgY'\q{g}\gx^2
+[P\bgY'\q{g}\gx^2]'_a
\right.\\&&\left.
+2(\baI'-\gn'_\gq)\gL'_\gz\gx
-2(\baF'-\gn'_\gz)\gL'_\gq\gx
+q'(\baF'-\gn'_\gz)\bgY'\gx^2
\right\}da d\gq d\gz
\lbq{Wstd}
.\eqbl
\c{ESI} reconstruction routines provide sufficient data for calculating the
coefficients in this functional on the user side.

\Section{Storage and \c{ESI} format of the equilibrium output file}
Full description on the data structure and \c{ESI} routines 
can be found in the \c{esiZ.c.d} documentation file. This section
specifies what information should be provided by the equilibrium codes
and in what format in order to be interfaced by \c{ESI}.

Both binary and ASCII files are suitable for storing the \c{ESI}
data. The suggested names of the data files are constructed as 

\c{esiE.00} for ASCII data-file, or a pair of

\c{esiE.00b} binary data-file and \c{esiE.00m} (ASCII map-file),

\noindent The extension \c{.00} specifies the sequence number. The pair
of binary data- and map files has \c{b} and \c{m} at the end of the
extension. The capital letters in the name stand for 
different coordinate systems. The suggested choices are listed in Table
2.

\btbl{|r|r|l||p{3.5in}|}\hline
\multicolumn{3}{|c|}{\bf $\c{ID}_\gq$ of angle $\gq$}&\hfil\em Table 2\\\hline 
$\c{ID}_\gq$&Symbol&Name&Specific property\\\hline
0&\c{E}&ESC&$a,\gq,\gf$, $r=r(a,\gq)$, $z=z_0(a)+b(a)\sin\gq$\\
1&\c{K}&KINX, polar&$r=r_0+\gr(a,\gq,\gz)\cos\gq$,
$z=z_0+\gr(a,\gq,\gz)\sin\gq$\\ 
2&\c{H}&Hamada&$J=J(a)$\\
3&\c{P}&PEST&$a,\gq,\gf$, $J=f(a)r^2$\\
4&\c{B}&Boozer&SFL $a,\bgq,\bgz$ with $\gh=0$ and $J=\R{f(a)}{|B|^2}$\\
5&\c{L}&Laboratory $rz$&$\bgY(r,z),\bgY'_r(r,z),\bgY'_z(r,z)$\\
6&\c{C}&Canonical&$a,\hgq,\hgz$, Eqs.(\rfq{GCCan0},\rfq{GCCan})\\
9&\c{U}&General&unspecified general coordinates, Eq.(\rfq{eq:rzgf})\\\hline
10+\c{*}&\c{1*}&VMEC-like&coordinates with separation of odd and even parts,
Eq.(\rfq{eq:rzgf})\\
\hline\etbl

Table 2 introduces also a numerical identification number \c{ID}$_\gq$
depending on the choice of the polidal angle. It should be ennhanced by
10, if there is a separation of the odd and even parts in presentation
of $r,z$ coordinates. Correspondingly the digit '\c{1}' is recommended
in the names of files before the symbol of coordiantes.

\btbl{ccc}\\
\psfig{figure=/w/Esc/Doc/esiZ.c.f1e.ps,width=2in}
&\psfig{figure=/w/Esc/Doc/esiZ.c.f1c.ps,width=2in}
&\psfig{figure=/w/Esc/Doc/esiZ.c.f1h.ps,width=2in}
\\
a)&b)&c)
\\
\psfig{figure=/w/Esc/Doc/esiZ.c.f1p.ps,width=2in}
&\psfig{figure=/w/Esc/Doc/esiZ.c.f1b.ps,width=2in}
&\psfig{figure=/w/Esc/Doc/esiZ.c.f1k.ps,width=2in}
\\
d)&e)&f)
\\
\multicolumn{3}{c}{\parbox{6in}{\em Fig.2. Typical coordinate systems
    for equilibrium configurations:
 a) ECS coordinates,
 b) canonical ``hat''-coordinates,
 c) Hamada coordinates,
 d) PEST coordinates,
 e) Boozer coordinates,
 f) KINX (polar) coordinates.
}}
\etbl

Before specifying the format of \c{ESI} data files, the distribution of
storage in the case of theuse of shared memory is presented in the next
section.

\Subsection{Storage in the shared memory}
Table 3 specifies all variables of \c{ESI} routines.
Parameters $L_\gq$, and $L_\gz$ represent periodicity of the system, 
\bqbl
f(a,\gq,\gz)
=
f(a,\gq,L_\gq\gq+L_\gz\gz),
.\eqbl
$N_a,N_\gq$ are the number of mesh intervals along $a,\gq$. In the
toroidal 
direction only one period is represented by the number $N_\gz$.
In the case of axisymmetry $N^\gz=0$ or $L_\gz=0$, while a special value
of $L_\gq=-1$ is for up-down symmetry.

\btbl{|l|l||l|r|l|}\hline
\multicolumn{3}{|c|}{\bf \c{ESI} data structure}
&\multicolumn{2}{|r|}{\em Table 3}\\\hline
\c{C} name&Type & Math&\#&Group name\\\hline
\c{Np1}&\c{int}&$N_\gq+1$&0&\c{<ESI dimensions>}\\
\c{Na1}&\c{int}&$N_a+1$&&\\
\c{ID}&\c{int}&Type of $a$&&\\
\c{Nt1}&\c{int}&$N_\gz+1$&&\\
\c{Lp}&\c{int}&$L_\gq$&&\\
\c{Lt}&\c{int}&$L_\gz$&&\\
\c{rBtor}&\c{double}&$\baF_{ref}\equiv\baF(a_{ref})$&&\\
\c{Rext}&\c{double}&$R_{ext}$&&\\
\c{aref}&\c{double}&$a_{ref}$&&\\
\c{bsp1}&\c{double}&$\bsp_{boundary}$&&\\\hline
\c{ToC[0x40][2]}&\c{int}&&1&\em Holds group addresses and sizes\\\hline
\c{gq[Na1]}&\c{double}&$\gq_j$&2&\c{<ESI [gq]>}\\\hline
\c{sa[Na1]}&\c{double}&$a_i$&3&\c{<ESI [sa]>}\\\hline
\c{aF [Na1]}&\c{double}&$\baF(a_i)$&4&\c{<ESI [baF bsp gF gY]>}\\
\c{bsp[Na1]}&\c{double}&$\bsp(a_i)$&&\em The group is informational only.\\
\c{gF  [Na1]}&\c{double}&$\gF(a_i)$&&\em It is generated during \\
\c{gY  [Na1]}&\c{double}&$\gY(a_i)$&&\em initialization of \c{ESI}\\\hline
\c{gFa [Na1]}&\c{double}&$\left(\R{\bgF'}{a}\right)(a_i)$
&5&\c{<ESI bgFa bgFaa bgYa bgYaa>}\\
\c{gFaa[Na1]}&\c{double}&$\left(\R{\bgF'}{a}\right)'_i$&&\\
\c{gYa [Na1]}&\c{double}&$\left(\R{\bgY'}{a}\right)(a_i)$&&\\
\c{gYaa[Na1]}&\c{double}&$\left(\R{\bgY'}{a}\right)'_i$&&\\\hline
\c{T [Na1]}&\c{double}&$T(a_i)$&6&\c{<ESI aT aTa aP aPa>}\\
\c{Ta[Na1]}&\c{double}&$T'(a_i)$&&\\
\c{P [Na1]}&\c{double}&$P(a_i)$&&\\
\c{Pa[Na1]}&\c{double}&$P'(a_i)$&&\\\hline\hline
\multicolumn{3}{|c|}{\bf \c{ESC} geometry data}&7&Group name\\\hline
\c{C} name&Type & Math&8&\c{<ESI r ra rq raq>}\\\hline
\c{r\ \ [Nt1][Na1][Np1]}&\c{double}
&$r(a_i,\gq_j,\gz_k)$&9&\\
\c{ra [Nt1][Na1][Np1]}&\c{double}
&$r'_a(a_i,\gq_j,\gz_k)$&&\\
\c{rq [Nt1][Na1][Np1]}&\c{double}
&$r'_\gq(a_i,\gq_j,\gz_k)$&&\\
\c{raq[Nt1][Na1][Np1]}&\c{double}
&$r''_{a\gq}(a_i,\gq_j,\gz_k)$&&\\\hline
\multicolumn{3}{|c|}{ same for $z,z'_a,z'_\gq,z''_{a\gq}$}
&10&\c{<ESI z za zq zaq>}\\\hline
\multicolumn{3}{|c|}{ same for $|B|,|B|'_a,|B|'_\gq,|B|''_{a\gq}$}
&11&\c{<ESI B Ba Bq Baq>}\\\hline
\c{ghq\ \ [Nt1][Na1][Np1]}&\c{double}
&$\gh'_\gq(a_i,\gq_j,\gz_k)$&12&\c{<ESI ghq ghaq ghqq gaqq>}\\
\c{ghaq [Nt1][Na1][Np1]}&\c{double}
&$\gh''_{a\gq}(a_i,\gq_j,\gz_k)$&&\\
\c{ghqq [Nt1][Na1][Np1]}&\c{double}
&$\gh''_{\gq\gq}(a_i,\gq_j,\gz_k)$&&\\
\c{ghaqq[Nt1][Na1][Np1]}&\c{double}
&$\gh'''_{a\gq\gq}(a_i,\gq_j,\gz_k)$&&\\
\c{gh\ \ \ [Nt1][Na1][Np1]}&\c{double}
&$\gh(a_i,\gq_j,\gz_k)$&&\em generated during initialization\\
\c{gha\ \ [Nt1][Na1][Np1]}&\c{double}
&$\gh'_a(a_i,\gq_j,\gz_k)$&&\em generated during initialization\\\hline
\etbl
\noindent  Functions $\gF,\gY,\baF,\bsp,\gh,\gh'_a$ are calculated
during the initialization of \c{ESI}. Indexes $i,j,k$ are used for the mesh
points along $a,\gq,\gz$ correspondingly
\bqbl
0\leq i\leq N_a
,\quad
0\leq j\leq N_\gq
,\quad
0\leq k\leq N_\gz
.\eqbl
The multidimensional arrays here are equivalent to a 1-D array
referenced as \c{[(k*Na1+i)*Np1+j]}.
In the 3-D case the following storage is added
for derivatives with respect to $\gz$.
\btbl{|l|l||l|r|l|}\hline
\multicolumn{3}{|c|}{\bf \c{ESI} profile data}
&\multicolumn{2}{|r|}{\em Table 3 (cont)}\\\hline
\c{C} name&Type & Math&\#&Group name\\\hline
\c{gz [Nt1]}&\c{double}&$\gz_k$&13&\c{<ESI [gz]>}\\\hline
\c{rz[Nt1][Na1][Np1]}&\c{double}&line$r'_\gz(a_i,\gq_j,\gz_k)$
&14&\c{<ESI rz raz rqz>}\\
\c{raz[Nt1][Na1][Np1]}&\c{double}
 &$r''_{a\gz}(a_i,\gq_j,\gz_k)$&&\\
\c{rqz[Nt1][Na1][Np1]}&\c{double}
  &$r''_{\gq\gz}(a_i,\gq_j,\gz_k)$&&\c{}\\
\c{raqz[Nt1][Na1][Np1]}&\c{double}
  &$r'''_{a\gq\gz}(a_i,\gq_j,\gz_k)$&&\c{}\\\hline
\multicolumn{3}{|c|}{ same for $z'_\gz,z''_{a\gz},z''_{\gq\gz}$}
 &15&\c{<ESI rz raz rqz>}\\\hline
\multicolumn{3}{|c|}{ same for $|B|'_\gz,|B|''_{a\gz},|B|''_{\gq\gz}$}
&16&\c{<ESI Bz Baz Bqz>}\\\hline
\c{ghz [Nt1][Na1][Np1]}&\c{double}
&$\gh''_{a\gq}(a_i,\gq_j,\gz_k)$&17&\c{<ESI ghqz ghaqz ghqqz>}\\
\c{ghaz [Nt1][Na1][Np1]}&\c{double}
&$\gh''_{\gq\gq}(a_i,\gq_j,\gz_k)$&&\c{}\\
\c{ghqz[Nt1][Na1][Np1]}&\c{double}
&$\gh''_{\gq\gz}(a_i,\gq_j,\gz_k)$&&\c{}\\
\c{ghaqz[Nt1][Na1][Np1]}&\c{double}
&$\gh'''_{a\gq\gz}(a_i,\gq_j,\gz_k)$&&\c{}\\\hline
\etbl
\noindent The resulting total size of \c{ESI} data (without counting
generated by \c{ESC}) is

\c{(6+128)*sizeof(int)+3+Np1+9*Na1+Nt1+16*Na1*Np1*Nt1*sizeof(double)}

\noindent and, e.g., for \c{Na1=21,Np1=65,Nt1=33} this is equal to 5,768,352
bytes only. 

The parameter \c{ID} specifies the type of the coordinate system
$a,\gq,\gz$. It is composed from $\c{ID}_a$ (Table 4) $\c{ID}_\gq$
(Table 2), and $\c{ID}_\gf$ (Table 4) in the following manner
\bqbl
\c{ID}=\c{ID}_\gz\c{*1000+}\c{ID}_\gq\c{*100+}\c{ID}_a
\eqbl
The $\c{ID}_a$ is a two digit decimal number \c{'Dd'}, where \c{'d'}
determine the physics meaning of  $a$ (e.g., vertical size, volume,
magnetic fluxes, etc), while tens \c{'D'} is a modifier. \c{D=0} and
\c{D=2} stands for normalized $0\leq a\leq1$, \c{D=0} and \c{D=1}
specify ``$\gr$''-like $a$ (like ``minor'' radius) when, e.g.,
$\bgF'(0)=0$, while \c{D=2} and \c{D=3} are for ``V''-like $a$ (similar
to volume of magnetic surfaces). Table 4 shows the possible values of
$\c{ID}_a$ and $\c{ID}_\gz$.

\btbl{|r|l||r|l|p{3in}|}\hline
\multicolumn{4}{|c|}{\bf $\c{ID}_a$ of radial coordinate $a$ in \c{ESI}}
&\hfill\em Table 4\\\hline
$\c{ID}_a$&$a$&\c{ID}&$a$&Comment\\\hline
\c{0}&$a=\R{b}{b_{boundary}}$&\c{20}&$a=\R{b^2}{b^2_{boundary}}$
 &$b$ is a vertical semiaxis of the cross-section of the magnetic surfaces
\\
\c{10}&$a=b$&\c{30}&$a=b^2$
&
\\\hline
\c{1}&$a=\q{\R{V}{V_{boundary}}}$&\c{21}&$a=\R{V}{V_{boundary}}$
 &$V$ is a volume of magnetic surface, $a_{ref}$ [m], $V_{ref}$ [m$^3$]
are some dimensional reference numbers.
\\
\c{11}&$a=a_{ref}\q{\R{V}{V_{ref}}}$&\c{31}&$a=V$
&
\\\hline
\c{2}&$a=\q{\R{\gF}{\gF_{boundary}}}$&\c{22}&$a=\R{\gF}{\gF_{boundary}}$
 &$\gF$ is a volume of magnetic surface, $B_{ref}$ [T] is some
dimensional reference numbers (ASTRA code).
\\
\c{12}&$a=\q{\R{\gF}{\gp B_{ref}}}$&\c{32}&$a=\gF$
&
\\\hline
\c{3}&$a=\q{\R{\gY}{\gY_{boundary}}}$&\c{23}&$a=\R{\gY}{\gY_{boundary}}$
 &$\gY$ is a volume of magnetic surface, $B_{ref}$ [T] is some
dimensional reference numbers (ASTRA code).
\\
\c{13}&$a=\q{\R{\gY}{\gp B_{ref}}}$&\c{33}&$a=\gY$
&
\\\hline
\c{9}&$a,\bgF'(0)=0$&\c{29}&$a,\bgF'(0)\neq0$
 &unspecified normalized  $a$
\\
\c{19}&$a,\bgF'(0)=0$&\c{39}&$a,\bgF'(0)\neq0$
 &unspecified dimensional $a$
\\\hline\hline
\multicolumn{4}{|c|}{\bf $\c{ID}_\gf$ of angle $\gz$}
&\hfill\em \\\hline
$\c{ID}_\gz$&$\gz$&$\c{ID}_\gz$&$\gz$&Comment\\\hline
\c{0}&$\gz=\gf$&\c{1}&$\gz=\bgz$
 &$\gz$ \vrule width0em height1.1em is a cylindrical azimuth or the
straight field line angle 
\\\hline
\c{2}&$\gz=\hgz$&\c{9}&$\gz$
&$\gz$ \vrule width0em height1.1em is a canonical or unspecified
\\\hline
\etbl

{\em Use of ``V''-like radial coordinates is discouraged because of
possible singularities in first derivatives of space functions at the magnetic
axis.}

In specific cases some groups of \c{ESI} data could be dropped. Thus,
transport codes do not need the group $\gh$. For PEST or SFL coordinates
this group is automatically absent because $\gh=0$. {\em The array
  \c{ToC}, which keeps addresses and sizes of the groups, marks the 
  absent groups by zero size.} 

\Subsection{Format of ASCII \c{ESI} data file}
Every equilibrium code is capable of creating the \c{ESI} data
file. This section specifies the format of the ASCII file, which should
be created by individual equilibrium codes.

The ASCII \c{ESI} file consists of a number of groups of records. Each
group starts with an identification line which contain the predefined
name of the group, number of records and a string with format of the 
records. The beginning of an \c{ESI} ASCII file can serve as a
representative example

\begin{verbatim}
<ESI dimensions>[1](%d x %d %d %e %s) ESC Date: 12/17/06 at 23:05
!Np1 x Na1   ID RBtor Name
 65 x  21   102 1.50  PEST
<ESI [gq]>[1](65%e)
\end{verbatim} 

Here '\c{<ESI dimensions>}' is the predefined name of the very first
group of \c{ESI} file, '\c{[1]}' means 1 record in the group, and the
string '\c{(\%d x \%d \%e)}' specifies the format for reading (3
integers with \c{'x'} between the first two, one double, and one
string). What is after the record head information (the name of the code
and the date) is a comment, ignored by the reading routine.

The following line, starting with '\c{!}' is also a comment. The next
line contains the data itself. (The string \c{"PEST"} is, in fact,
only informational, rather than a part of \c{ESI}).

After this there is the head line of the next group. All names of the groups
are given in the Table 3. Groups with names of variables inside
'\c{[]}', like '\c{<ESI [baF bsp gF gY]>}' are optional. These data are
initiated by \c{ESI}. Also '\c{<ESI [gq]>}', '\c{<ESI [sa]>}', '\c{<ESI
  [gq]>}' are optional. Their data are replaced by equidistant $0\leq
a_i\leq1$, $0\leq\gq_j\leq2\gp$, $0\leq\gz_k\leq2\gp/L_\gz$. 

The comments can be freely distributed outside the group of data. The
short comments can be placed into the data lines: (a) after the format
string in the ID line, and (b) after the end of each group of data. With
leading '\c{!}' comment lines can be placed between groups of data.

Note, that only 3 parameters, out of 8 from Table 3, are present in the
example. Parameters \c{Nt1,Lp,Lt,bspb} are dropped from the
record. Their default values are \c{1,0,0,0}. Because the order of
parameters is predefined by Table 3 the presence of a format string
makes reading unambiguous.

The groups of arrays are recorded in the file by columns with  one column
for each variable, listed in the name of the group, i.e.,
{\obeylines\obeyspaces\Ci{
<ESI r ra rgq raq>[1365](\%e \%e \%e \%e)
!          r                    r'\_a                    r'\_q                   r''\_{aq}
1.1189417832363331e+00  6.1752680497674650e-01 -0.0000000000000000e+00 -9.1619079100425833e-03
1.1189417832363331e+00  6.1365522061023337e-01 -0.0000000000000000e+00 -6.9646002319922323e-02
 }}

{\em The use of named groups and format lines makes \c{ESI} format very
flexible.} Thus, reduced versions of \c{ESI} data set are possible. For
example, in communication with the transport simulation codes, the
function $\gh$ is not necessary and its group of data may not be
present. The format string identifies which data are present in the
records inside the group. {\em If specified in the format string,
  enumeration can be in front of the records for human
  readability. The reading routine would ignore the enumeration.} 


\Subsection{\c{ESI} binary data- and map- files}
In this case, the short ASCII file represents the data for reading the
binary file.

It starts with the group \c{<ESI dimensions>}, described in the
previous section. It may contain groups with plasma profiles.

After this it lists groups, present in the binary file (the group
'\c{<ESI [baF bsp gF gY]>}' is useless to keep in the binary
files). Instead of real data, the data in the map file represent the
address (in bytes) of the beginning of the group and its size (in bytes
as well), e.g.

\begin{verbatim} 
  <ESI r ra rq raq}>[2](%d %d)
672 43680
\end{verbatim} 

\Section{Basic set of \c{ESI} routines}
The prototypes of \c{ESI} C-routines for the user's side (some not yet
available) are listed in Table 5. The non-descriptive names of variables
are constructed using the following conventions: \c{'g'} prefixes the
name of Greek math variables, \c{'b'} prefixes ``barred'' variables,
\c{'d'} prefixes time derivatives, names with \c{'a'},\c{'q'}\c{'z'}
after the symbol of are used for derivatives. FORTRAN-callable analogs
of C-routines have the same names with capital latters replaced by low 
case ones.

\btbl{|p{1.35in}|c|p{4in}|}\hline
\multicolumn{3}{|c|}{{\bf Minimum set of ESI routines. Marked by \c{*}
    are not yet available}\hfill\em Table 5}\\\hline
example of C call&\c{*}& Comment\\\hline 
\c{i=File2ESC(FNm);}&& \c{'FNm'} is the name of the \c{ESI} data-file.
\\
&&\\
\c{i=FreeESI();}&& frees the memory allocated for \c{ESI} by \c{File2ESC()}.
\\\hline
\c{i=FShM2ESC(key);}&& \c{'key'} is the ID of the shared memory segment
with \c{ESI} data\\\hline
\c{i=Link2ESC(F,Fa ,gFa,gFaa ,gYa,gYaa ,T,Ta ,P,Pa 
,r,ra,rq,z,za,zq ,B,Ba,Bq ,gh,gha,ghq,isw);}
&& Called once, it sends to \c{ESI} the addresses of arrays, which will contain
values of the basic functions.

\c{isw} is the integer array which could be necessary for marking
the particles.
\\
&&\\
\c{i=Link2ESI3d(rz,zz ,Bz,ghz);}&\c{*}
&Specifies additional links for 3-D case.
\\\hline 
\c{i=ESI2all(a,gq,n);}
&& The main reconstruction routine, which calculates the basic functions
for $n$ particles in positions given by \c{a[i],q[i]}.
\\
&&\\
\c{i=ESI2all3d(a,gq ,gz,n);}&\c{*}& 3-D analog of \c{ESI2all()}.
\\\hline 
\c{i=ESI2PlP(F,Fa ,gFa,gFaa ,gYa,gYaa ,T,Ta ,P,Pa,a,n);}
&&Puts the values of plasma profiles
$\baF,\baF',\bgF',\bgF'',\bgY',\bgY'',T,T',P,P'$ into 
arrays \c{F[], Fa[], gFa[], gFaa[], gYa[], gYaa[], T[], Ta[], P[], Pa[]} for
$n$ points specified in \c{a[]}.
\\\hline
\c{i=ESI2bsp(bsp,bspa ,a,n);}
&&Puts the values of $\bsp,\bsp'$ into
arrays \c{bsp[], bspa[]} for $n$ points specified in \c{a[]}.
\\\hline
\c{i=ESI2gFgY(gF,gY ,a,n);}
&&Puts the values of magnetic fluxes $\gF,\gY$ (un-barred) into
arrays \c{gF[], gY[]} for $n$ points specified in \c{a[]}.
\\\hline
\c{i=ESIrz2agq(a,gq ,r,z,n);}
&&Converts $n$ cylindrical coordinates $r,z$ into $a,\gq$. 
\\\hline
\c{i=ESIrzf2agq3d(a,gq ,gz ,r,z,gf,n);}
&\c{*}&3D version of \c{ESIrz2agq()}.
\\\hline
\c{i=rzESI2agq(ID ,na,nq);}
&\c{*}&Converts original \c{ESI} data on $r-z$ grid into \c{ESI} data in
nested flux coordinates specified by the identification number \c{ID}
with \c{na,nq} radial and poloidal intervals.
\\\hline
\c{i=ESI2gcm(dgr,da ,dgq,dgf ,gr,a,gq,gm,n);}
&2D&Time advancing routine for guiding center motion. Calculates the time
derivatives $\dgr_\parallel,\dsa,\dgq,\dgf$ in Eq.(\rfq{GCgcm}) for
$n$ particles. \c{gm[]} contains magnetic moments of the
particles. Integer marker \c{isw[]} is used behind the scene.
{\em Only 2-D version is available.}
\\\hline
\c{i=ESI2mfl(dgqL,dgfL ,a,gq,n);}
&2D&Calculates derivatives of $\gq'_l,\gf'_l$ in Eq.(\rfq{GCmfl}) with
respect to length $l$ along $n$ magnetic field lines.
{\em Only 2-D version is available.}
\\\hline
\etbl
More complete information is given in the \c{escZ.c.d} document
file (which contains as a part also this text).
At present \c{ESI} is functional with Equilibrium and Stability Code
(ESC) for axisymmetric tokamak equilibrium configurations. ESC is
interfaced with the transport simulation ASTRA code using \c{ESI}
through the shared memory, and using the data files with a number of
author's codes.

\Subsection{Service routines of \c{ESI}.}
Some available and possible ``convenience'' routines are listed in
Table~6. 
\btbl{|p{1.35in}|c|p{4in}|}\hline
\multicolumn{3}{|c|}{{\bf \c{ESI} service routines. Marked by \c{*}
    are not yet available}\hfill\em Table 6}\\\hline
example of C call&\c{*}& Comment\\\hline 
\c{i=ESI2geom(r,ra,rq, z,za,zq, a,gq,n);}
&&Calculates $r,r'_a,r'_\gq,z,z'_a,z'_\gq$ for $n$ points.
\\\hline
\c{i=ESI2rzB(r,z,B ,a,gq,n);}
&&Calculates $r,z,|B|$ for $n$ points.
\\\hline
\c{i=rzESI2gcm(dgr,dr ,dz,dgf ,r,z,gm,n);}
&&Analog of ESI2gcm() with use of $r-z$ \c{ESI} data. Calculates the time
derivatives $\dgr_\parallel,\dsr,\dsz,\dgf$ for guiding center motion
for $n$ particle specified by their $r,z$ and magnetic moment $\gm$.
\\\hline
\etbl

The set of service routines can be expanded with more practice.

\Subsection{\c{ESI} as a structure.}
In fact, the Table 3 shows the necessary elements of a single C data-structure
which is \c{typedef}ined as \c{ESIstruct}. The user can work simultaneously
with several of them (from 0 to 7), e.g., for comparison of different
codes. But only one structure is functional at the \c{ESI} side. By
default the \c{ESI[0]} is activated. Using the call 
\vskip1ex

\c{i=ESI2up(1);}
\vskip1ex

\noindent one can switch between active structures.

\Subsection{\c{ESI} as a virtual machine.}
In a primitive manner, \c{ESI} can mimic the ``virtual machine''
approach of OpenGL. A particular service, like
calculation of function $|B|$, $gh$,
or graphics output can be activated any time by using
\vskip1ex
\c{i=esiEnable(esiGL,NULL);}

\c{i=esiEnable(esiBASIS,ESIB);}

\c{i=esiEnable(esiBASIS,ESIgh);}
\vskip1ex
\noindent and disabled by
\vskip1ex
\c{i=esiDisable(esiGL ,NULL);}

\c{i=esi2Disable(esiBASIS,ESIB);}
\vskip1ex
when it is not needed anymore. By default all calculations corresponding
to a predefined parameter \c{esiBASIS} are activated.

The full set of possibilities is specified \c{ESI} documentation file
\c{esiZ.c.d}. 

{\em For use of possibilities of a virtual machine the
  definition file \c{esiZ.h} (or \c{esiZ.inc} for FORTRAN) should be
  included into the user source code. Their content can be found in
  the \c{esiZ.c} file.}

\Section{Summary}
The described \c{ESI} interface has unique properties in being
organized, comprehensive, compact and independent of the computer
architecture. For many plasma physics
codes it can provide a uniform access to the background information
related to magnetic configurations, independent of how this information
was created and by what equilibrium code.

Without change in the format of communications, \c{ESI} allows one to
drop some groups of data if they are not necessary for a particular
communication. On the other hand, within the same approach it is
extendible to more general cases, e.g., anisotropic equilibria or
equilibria with perturbed magnetic surfaces. At this moment, one of the
crucial extension should be introduction of the separatrix boundary
layer, where the flux coordinates have a special behavior near the X-points.
#endif

#ifndef mzl_System_esiZNew.c /* --------------- Use of Variables --------{ */
#ifndef H/* ---------- Used External Variables ------{ */
  Ignor double;
  Ignor REAL;
  Ignor char;
  Ignor int;
  Systm NULL;
  Systm sqrt();
  Systm acos();
  Systm cos();
  Systm pow();
  Systm atan();
  Systm free();
  Systm malloc();
  Systm printf();
  Systm feof();
  Systm fgetc();
  Systm fopen();
  Systm fread();
  Systm fclose();
  Systm fscanf();
  Systm fabs();
  Systm isspace();
  Systm sin();
#endif /* }======== End of Used External Variables ======= */

#endif /* }======== End of Use of Variables ======== */

static double cr2;	/*  */
static double cr3;	/*  */
static double cr4;	/*  */
static double cr5;	/*  */
static double cr6;	/*  */
static double cr7;	/*  */
static double cr8;	/*  */
static double cr12;	/*  */
static double cgp;	/*  */
static double c2gp;	/*  */
static double cgp_4;	/*  */
static double cr2gp;	/*  */
static double cgm0;	/*  */
static double crgm0;	/*  */
static double cbsp2ne;	/*  */
static int Na;		/*  */
static int Na1;		/*  */
static int Np;		/*  */
static int Np1;		/*  */
static REAL RBtor;	/*  */
static REAL aRBtor;	/*  */
static REAL *gq;	/*  */
static REAL *sa;	/*  */
static REAL *dgF;	/*  */
static REAL *d2gF;	/*  */
static REAL *dgY;	/*  */
static REAL *d2gY;	/*  */
static REAL *aT;	/*  */
static REAL *daT;	/*  */
static REAL *aP;	/*  */
static REAL *daP;	/*  */
static REAL *gF;	/*  */
static REAL *gY;	/*  */
static REAL *bsp;	/*  */
static REAL *dbsp;	/*  */
static REAL *aF;	/*  */
static REAL *daF;	/*  */
static REAL *sr;	/*  */
static REAL *sra;	/*  */
static REAL *srq;	/*  */
static REAL *sraq;	/*  */
static REAL *sz;	/*  */
static REAL *sza;	/*  */
static REAL *szq;	/*  */
static REAL *szaq;	/*  */
static REAL *aB;	/*  */
static REAL *aBa;	/*  */
static REAL *aBq;	/*  */
static REAL *aBaq;	/*  */
static REAL *gH;	/*  */
static REAL *gHa;	/*  */
static REAL *gHq;	/*  */
static REAL *gHaq;	/*  */
static REAL *gHqq;	/*  */
static REAL *gHaqq;	/*  */
static int i0;		/*  */
static int i1;		/*  */
static int j00;		/*  */
static int j10;		/*  */
static int j01;		/*  */
static int j11;		/*  */
static REAL A;		/*  */
static REAL ha;		/*  */
static REAL rha;	/*  */
static REAL hq;		/*  */
static REAL rhq;	/*  */
static REAL cgq0;	/*  */
static REAL crgq0;	/*  */
static REAL xF;		/*  */
static REAL XF;		/*  */
static REAL xD;		/*  */
static REAL XD;		/*  */
static REAL dX;		/*  */
static REAL dxD;	/*  */
static REAL dXD;	/*  */
static REAL yF;		/*  */
static REAL YF;		/*  */
static REAL yD;		/*  */
static REAL YD;		/*  */
static REAL dY;		/*  */
static REAL dyD;	/*  */
static REAL dYD;	/*  */
static REAL y;		/*  */
static REAL Y;		/*  */
static REAL yy;		/*  */
static REAL YY;		/*  */
static int *sw;		/*  */
static REAL *r;		/*  */
static REAL *ra;	/*  */
static REAL *rq;	/*  */
static REAL *z;		/*  */
static REAL *za;	/*  */
static REAL *zq;	/*  */
static REAL *B;		/*  */
static REAL *Ba;	/*  */
static REAL *Bq;	/*  */
static REAL *gh;	/*  */
static REAL *gha;	/*  */
static REAL *ghq;	/*  */
static REAL *gFa;	/*  */
static REAL *gFaa;	/*  */
static REAL *gYa;	/*  */
static REAL *gYaa;	/*  */
static REAL *T;		/*  */
static REAL *Ta;	/*  */
static REAL *P;		/*  */
static REAL *Pa;	/*  */
static REAL *F;		/*  */
static REAL *Fa;	/*  */
static REAL *sp;	/*  */
static REAL *spa;	/*  */
static double cme;	/*  */
static double cmi;	/*  */
static double cc;	/*  */
static double ce;	/*  */
static double cEeV;	/*  */
static double cEkeV;	/*  */

int PHermit(f,df,x,y,dy)
  double *f;	/*  */
  double *df;	/*  */
  double x;	/*  */
  double *y;	/*  */
  double *dy;	/*  */
{
  double *dy;		/*  .PHermit():esiZNew.c */
  double *y;		/*  .PHermit():esiZNew.c */
  double x;		/*  .PHermit():esiZNew.c */
  double *f;		/*  .PHermit():esiZNew.c */
  double *df;		/*  .PHermit():esiZNew.c */
  static REAL ha;	/*            :esiZNew.c */
  static REAL rha;	/*            :esiZNew.c */
/*
   */
#ifdef Doc_PHermit
\Section{Skeleton of \c{PHermit()}}
  \begin{verbatim}
    PHermit(){
    }
  \end{verbatim}
\Section{\c{PHermit()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*f}&$$&\\
\c{double }&\c{*df}&$$&\\
\c{double }&\c{x}&$$&\\
\c{double }&\c{*y}&$$&\\
\c{double }&\c{*dy}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{static REAL }&\c{ha}&$$&\\
\c{double }&\c{*dy}&$$&\\
\c{double }&\c{*y}&$$&\\
\c{double }&\c{x}&$$&\\
\c{static REAL }&\c{rha}&$$&\\
\c{double }&\c{*f}&$$&\\
\c{double }&\c{*df}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int EPPAHermit(f,df,x,i0)
  double *f;	/*  */
  double *df;	/*  */
  double x;	/*  */
  int i0;	/*  */
{
  int i0;		/*  .EPPAHermit():esiZNew.c */
  double x;		/*  .EPPAHermit():esiZNew.c */
  double *f;		/*  .EPPAHermit():esiZNew.c */
  double *df;		/*  .EPPAHermit():esiZNew.c */
  static REAL rha;	/*               :esiZNew.c */
  static REAL *aT;	/*               :esiZNew.c */
  static REAL *dgY;	/*               :esiZNew.c */
  static REAL *sa;	/*               :esiZNew.c */
  static REAL ha;	/*               :esiZNew.c */
  static REAL *d2gY;	/*               :esiZNew.c */
  static double cr2;	/*               :esiZNew.c */
  static double cr3;	/*               :esiZNew.c */
  static double cr4;	/*               :esiZNew.c */
  static double cr5;	/*               :esiZNew.c */
  static double cr6;	/*               :esiZNew.c */
  static double cr7;	/*               :esiZNew.c */
  static double cr8;	/*               :esiZNew.c */
  static REAL *daT;	/*               :esiZNew.c */
/*
   */
#ifdef Doc_EPPAHermit
\Section{Skeleton of \c{EPPAHermit()}}
  \begin{verbatim}
    EPPAHermit(){
    }
  \end{verbatim}
\Section{\c{EPPAHermit()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*f}&$$&\\
\c{double }&\c{*df}&$$&\\
\c{double }&\c{x}&$$&\\
\c{int }&\c{i0}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{int }&\c{i0}&$$&\\
\c{double }&\c{x}&$$&\\
\c{static REAL }&\c{rha}&$$&\\
\c{double }&\c{*f}&$$&\\
\c{double }&\c{*df}&$$&\\
\c{static REAL }&\c{*aT}&$$&\\
\c{static REAL }&\c{*dgY}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{ha}&$$&\\
\c{static REAL }&\c{*d2gY}&$$&\\
\c{static double }&\c{cr2}&$$&\\
\c{static double }&\c{cr3}&$$&\\
\c{static double }&\c{cr4}&$$&\\
\c{static double }&\c{cr5}&$$&\\
\c{static double }&\c{cr6}&$$&\\
\c{static double }&\c{cr7}&$$&\\
\c{static double }&\c{cr8}&$$&\\
\c{static REAL }&\c{*daT}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int ZeroOfHermit(ts,df,i0)
  double *ts;	/*  */
  double *df;	/*  */
  int i0;	/*  */
{
  int i0;		/*  .ZeroOfHermit():esiZNew.c */
  double *ts;		/*  .ZeroOfHermit():esiZNew.c */
  double *df;		/*  .ZeroOfHermit():esiZNew.c */
  static REAL *aT;	/*                 :esiZNew.c */
  static REAL ha;	/*                 :esiZNew.c */
  static REAL *daT;	/*                 :esiZNew.c */
  static REAL rha;	/*                 :esiZNew.c */
  static double cr3;	/*                 :esiZNew.c */
  static double cr2;	/*                 :esiZNew.c */
  static double c2gp;	/*                 :esiZNew.c */
/*
   */
#ifdef Doc_ZeroOfHermit
\Section{Skeleton of \c{ZeroOfHermit()}}
  \begin{verbatim}
    ZeroOfHermit(){
    }
  \end{verbatim}
\Section{\c{ZeroOfHermit()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*ts}&$$&\\
\c{double }&\c{*df}&$$&\\
\c{int }&\c{i0}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{Undef }&\c{sqrt()}&$$&\\
\c{Undef }&\c{acos()}&$$&\\
\c{Undef }&\c{cos()}&$$&\\
\c{Undef }&\c{pow()}&$$&\\
\c{int }&\c{i0}&$$&\\
\c{static REAL }&\c{*aT}&$$&\\
\c{static REAL }&\c{ha}&$$&\\
\c{static REAL }&\c{*daT}&$$&\\
\c{double }&\c{*ts}&$$&\\
\c{double }&\c{*df}&$$&\\
\c{static REAL }&\c{rha}&$$&\\
\c{static double }&\c{cr3}&$$&\\
\c{static double }&\c{cr2}&$$&\\
\c{static double }&\c{c2gp}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int InitLocalConstants()
{
  static double cr2;	/*  :esiZNew.c */
  static double cr3;	/*  :esiZNew.c */
  static double cr4;	/*  :esiZNew.c */
  static double cr5;	/*  :esiZNew.c */
  static double cr6;	/*  :esiZNew.c */
  static double cr7;	/*  :esiZNew.c */
  static double cr8;	/*  :esiZNew.c */
  static double cr12;	/*  :esiZNew.c */
  static double cgp_4;	/*  :esiZNew.c */
  static double cgp;	/*  :esiZNew.c */
  static double c2gp;	/*  :esiZNew.c */
  static double cr2gp;	/*  :esiZNew.c */
  static double cgm0;	/*  :esiZNew.c */
  static double crgm0;	/*  :esiZNew.c */
  static double cbsp2ne;/*  :esiZNew.c */
/*
   */
#ifdef Doc_InitLocalConstants
\Section{Skeleton of \c{InitLocalConstants()}}
  \begin{verbatim}
    InitLocalConstants(){
    }
  \end{verbatim}
\Section{\c{InitLocalConstants()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{Undef }&\c{atan()}&$$&\\
\c{Undef }&\c{double}&$$&\\
\c{static double }&\c{cr2}&$$&\\
\c{static double }&\c{cr3}&$$&\\
\c{static double }&\c{cr4}&$$&\\
\c{static double }&\c{cr5}&$$&\\
\c{static double }&\c{cr6}&$$&\\
\c{static double }&\c{cr7}&$$&\\
\c{static double }&\c{cr8}&$$&\\
\c{static double }&\c{cr12}&$$&\\
\c{static double }&\c{cgp_4}&$$&\\
\c{static double }&\c{cgp}&$$&\\
\c{static double }&\c{c2gp}&$$&\\
\c{static double }&\c{cr2gp}&$$&\\
\c{static double }&\c{cgm0}&$$&\\
\c{static double }&\c{crgm0}&$$&\\
\c{static double }&\c{cbsp2ne}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int ESISetMem(gq0,na1,np1)
  double *gq0;	/*  */
  int na1;	/*  */
  int np1;	/*  */
{
  int np1;			/*  .ESISetMem():esiZNew.c */
  int na1;			/*  .ESISetMem():esiZNew.c */
  double *gq0;			/*  .ESISetMem():esiZNew.c */
  static int Np1;		/*              :esiZNew.c */
  static int Na1;		/*              :esiZNew.c */
  static int Na;		/*              :esiZNew.c */
  static int Np;		/*              :esiZNew.c */
  static REAL *gq;		/*              :esiZNew.c */
  static REAL *sa;		/*              :esiZNew.c */
  static REAL *gF;		/*              :esiZNew.c */
  static REAL *dgF;		/*              :esiZNew.c */
  static REAL *d2gF;		/*              :esiZNew.c */
  static REAL *gY;		/*              :esiZNew.c */
  static REAL *dgY;		/*              :esiZNew.c */
  static REAL *d2gY;		/*              :esiZNew.c */
  static REAL *aT;		/*              :esiZNew.c */
  static REAL *daT;		/*              :esiZNew.c */
  static REAL *aP;		/*              :esiZNew.c */
  static REAL *daP;		/*              :esiZNew.c */
  static REAL *aF;		/*              :esiZNew.c */
  static REAL *daF;		/*              :esiZNew.c */
  static REAL *bsp;		/*              :esiZNew.c */
  static REAL *dbsp;		/*              :esiZNew.c */
  static REAL *sr;		/*              :esiZNew.c */
  static REAL *sra;		/*              :esiZNew.c */
  static REAL *srq;		/*              :esiZNew.c */
  static REAL *sraq;		/*              :esiZNew.c */
  static REAL *sz;		/*              :esiZNew.c */
  static REAL *sza;		/*              :esiZNew.c */
  static REAL *szq;		/*              :esiZNew.c */
  static REAL *szaq;		/*              :esiZNew.c */
  static REAL *aB;		/*              :esiZNew.c */
  static REAL *aBa;		/*              :esiZNew.c */
  static REAL *aBq;		/*              :esiZNew.c */
  static REAL *aBaq;		/*              :esiZNew.c */
  static REAL *gH;		/*              :esiZNew.c */
  static REAL *gHa;		/*              :esiZNew.c */
  static REAL *gHq;		/*              :esiZNew.c */
  static REAL *gHaq;		/*              :esiZNew.c */
  static REAL *gHqq;		/*              :esiZNew.c */
  static REAL *gHaqq;		/*              :esiZNew.c */
  int InitLocalConstants();	/*              :esiZNew.c */
/*
   */
#ifdef Doc_ESISetMem
\Section{Skeleton of \c{ESISetMem()}}
  \begin{verbatim}
    ESISetMem(){
    }
  \end{verbatim}
\Section{\c{ESISetMem()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*gq0}&$$&\\
\c{int }&\c{na1}&$$&\\
\c{int }&\c{np1}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{static int }&\c{Np1}&$$&\\
\c{int }&\c{np1}&$$&\\
\c{static int }&\c{Na1}&$$&\\
\c{int }&\c{na1}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{static int }&\c{Np}&$$&\\
\c{int }&\c{InitLocalConstants()}&$$&\\
\c{static REAL }&\c{*gq}&$$&\\
\c{double }&\c{*gq0}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{*gF}&$$&\\
\c{static REAL }&\c{*dgF}&$$&\\
\c{static REAL }&\c{*d2gF}&$$&\\
\c{static REAL }&\c{*gY}&$$&\\
\c{static REAL }&\c{*dgY}&$$&\\
\c{static REAL }&\c{*d2gY}&$$&\\
\c{static REAL }&\c{*aT}&$$&\\
\c{static REAL }&\c{*daT}&$$&\\
\c{static REAL }&\c{*aP}&$$&\\
\c{static REAL }&\c{*daP}&$$&\\
\c{static REAL }&\c{*aF}&$$&\\
\c{static REAL }&\c{*daF}&$$&\\
\c{static REAL }&\c{*bsp}&$$&\\
\c{static REAL }&\c{*dbsp}&$$&\\
\c{static REAL }&\c{*sr}&$$&\\
\c{static REAL }&\c{*sra}&$$&\\
\c{static REAL }&\c{*srq}&$$&\\
\c{static REAL }&\c{*sraq}&$$&\\
\c{static REAL }&\c{*sz}&$$&\\
\c{static REAL }&\c{*sza}&$$&\\
\c{static REAL }&\c{*szq}&$$&\\
\c{static REAL }&\c{*szaq}&$$&\\
\c{static REAL }&\c{*aB}&$$&\\
\c{static REAL }&\c{*aBa}&$$&\\
\c{static REAL }&\c{*aBq}&$$&\\
\c{static REAL }&\c{*aBaq}&$$&\\
\c{static REAL }&\c{*gH}&$$&\\
\c{static REAL }&\c{*gHa}&$$&\\
\c{static REAL }&\c{*gHq}&$$&\\
\c{static REAL }&\c{*gHaq}&$$&\\
\c{static REAL }&\c{*gHqq}&$$&\\
\c{static REAL }&\c{*gHaqq}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int esifree_()
{
  static REAL *gq;	/*  :esiZNew.c */
/*
   */
#ifdef Doc_esifree_
\Section{Skeleton of \c{esifree_()}}
  \begin{verbatim}
    esifree_(){
    }
  \end{verbatim}
\Section{\c{esifree_()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{Undef }&\c{NULL}&$$&\\
\c{Undef }&\c{free()}&$$&\\
\c{static REAL }&\c{*gq}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int ESIMemAlloc(na1,np1)
  int na1;	/*  */
  int np1;	/*  */
{
  int na1;		/*  .ESIMemAlloc():esiZNew.c */
  int np1;		/*  .ESIMemAlloc():esiZNew.c */
  static REAL *gq;	/*                :esiZNew.c */
  static int Na1;	/*                :esiZNew.c */
  static int Np1;	/*                :esiZNew.c */
  static int Na;	/*                :esiZNew.c */
  static int Np;	/*                :esiZNew.c */
  int ESISetMem();	/*                :esiZNew.c */
/*
   */
#ifdef Doc_ESIMemAlloc
\Section{Skeleton of \c{ESIMemAlloc()}}
  \begin{verbatim}
    ESIMemAlloc(){
    }
  \end{verbatim}
\Section{\c{ESIMemAlloc()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{int }&\c{na1}&$$&\\
\c{int }&\c{np1}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{Undef }&\c{NULL}&$$&\\
\c{Undef }&\c{free()}&$$&\\
\c{Undef }&\c{REAL}&$$&\\
\c{Undef }&\c{malloc()}&$$&\\
\c{Undef }&\c{printf()}&$$&\\
\c{static REAL }&\c{*gq}&$$&\\
\c{static int }&\c{Na1}&$$&\\
\c{int }&\c{na1}&$$&\\
\c{static int }&\c{Np1}&$$&\\
\c{int }&\c{np1}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{static int }&\c{Np}&$$&\\
\c{int }&\c{ESISetMem()}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int IsStrFound(lF,Str)
  FILE *lF;	/*  */
  char *Str;	/*  */
{
  char *Str;	/*  .IsStrFound():esiZNew.c */
  FILE *lF;	/*  .IsStrFound():esiZNew.c */
/*
   */
#ifdef Doc_IsStrFound
\Section{Skeleton of \c{IsStrFound()}}
  \begin{verbatim}
    IsStrFound(){
    }
  \end{verbatim}
\Section{\c{IsStrFound()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{FILE }&\c{*lF}&$$&\\
\c{char }&\c{*Str}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{Undef }&\c{feof()}&$$&\\
\c{Undef }&\c{char}&$$&\\
\c{Undef }&\c{fgetc()}&$$&\\
\c{char }&\c{*Str}&$$&\\
\c{FILE }&\c{*lF}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int ESIReadBD(FNm)
  char *FNm;	/*  */
{
  char *FNm;		/*  .ESIReadBD():esiZNew.c */
  static REAL *gq;	/*              :esiZNew.c */
  static int Np1;	/*              :esiZNew.c */
  static int Na1;	/*              :esiZNew.c */
  static REAL RBtor;	/*              :esiZNew.c */
  static REAL *sa;	/*              :esiZNew.c */
  static REAL *aF;	/*              :esiZNew.c */
  static REAL *daF;	/*              :esiZNew.c */
  static REAL *dgF;	/*              :esiZNew.c */
  static REAL *d2gF;	/*              :esiZNew.c */
  static REAL *dgY;	/*              :esiZNew.c */
  static REAL *d2gY;	/*              :esiZNew.c */
  static REAL *aT;	/*              :esiZNew.c */
  static REAL *daT;	/*              :esiZNew.c */
  static REAL *aP;	/*              :esiZNew.c */
  static REAL *daP;	/*              :esiZNew.c */
  static REAL *sr;	/*              :esiZNew.c */
  static REAL *sra;	/*              :esiZNew.c */
  static REAL *srq;	/*              :esiZNew.c */
  static REAL *sraq;	/*              :esiZNew.c */
  static REAL *sz;	/*              :esiZNew.c */
  static REAL *sza;	/*              :esiZNew.c */
  static REAL *szq;	/*              :esiZNew.c */
  static REAL *szaq;	/*              :esiZNew.c */
  static REAL *aB;	/*              :esiZNew.c */
  static REAL *aBa;	/*              :esiZNew.c */
  static REAL *aBq;	/*              :esiZNew.c */
  static REAL *aBaq;	/*              :esiZNew.c */
  static REAL *gHq;	/*              :esiZNew.c */
  static REAL *gHaq;	/*              :esiZNew.c */
  static REAL *gHqq;	/*              :esiZNew.c */
  static REAL *gHaqq;	/*              :esiZNew.c */
  int ESIMemAlloc();	/*              :esiZNew.c */
/*
   */
#ifdef Doc_ESIReadBD
\Section{Skeleton of \c{ESIReadBD()}}
  \begin{verbatim}
    ESIReadBD(){
    }
  \end{verbatim}
\Section{\c{ESIReadBD()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{char }&\c{*FNm}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{Undef }&\c{fopen()}&$$&\\
\c{Undef }&\c{NULL}&$$&\\
\c{Undef }&\c{printf()}&$$&\\
\c{Undef }&\c{REAL}&$$&\\
\c{Undef }&\c{fread()}&$$&\\
\c{Undef }&\c{int}&$$&\\
\c{Undef }&\c{free()}&$$&\\
\c{Undef }&\c{fclose()}&$$&\\
\c{char }&\c{*FNm}&$$&\\
\c{static REAL }&\c{*gq}&$$&\\
\c{static int }&\c{Np1}&$$&\\
\c{static int }&\c{Na1}&$$&\\
\c{int }&\c{ESIMemAlloc()}&$$&\\
\c{static REAL }&\c{RBtor}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{*aF}&$$&\\
\c{static REAL }&\c{*daF}&$$&\\
\c{static REAL }&\c{*dgF}&$$&\\
\c{static REAL }&\c{*d2gF}&$$&\\
\c{static REAL }&\c{*dgY}&$$&\\
\c{static REAL }&\c{*d2gY}&$$&\\
\c{static REAL }&\c{*aT}&$$&\\
\c{static REAL }&\c{*daT}&$$&\\
\c{static REAL }&\c{*aP}&$$&\\
\c{static REAL }&\c{*daP}&$$&\\
\c{static REAL }&\c{*sr}&$$&\\
\c{static REAL }&\c{*sra}&$$&\\
\c{static REAL }&\c{*srq}&$$&\\
\c{static REAL }&\c{*sraq}&$$&\\
\c{static REAL }&\c{*sz}&$$&\\
\c{static REAL }&\c{*sza}&$$&\\
\c{static REAL }&\c{*szq}&$$&\\
\c{static REAL }&\c{*szaq}&$$&\\
\c{static REAL }&\c{*aB}&$$&\\
\c{static REAL }&\c{*aBa}&$$&\\
\c{static REAL }&\c{*aBq}&$$&\\
\c{static REAL }&\c{*aBaq}&$$&\\
\c{static REAL }&\c{*gHq}&$$&\\
\c{static REAL }&\c{*gHaq}&$$&\\
\c{static REAL }&\c{*gHqq}&$$&\\
\c{static REAL }&\c{*gHaqq}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int ESIReadAD(FNm)
  char *FNm;	/*  */
{
  char *FNm;		/*  .ESIReadAD():esiZNew.c */
  static REAL RBtor;	/*              :esiZNew.c */
  static REAL *gq;	/*              :esiZNew.c */
  static int Np1;	/*              :esiZNew.c */
  static int Na1;	/*              :esiZNew.c */
  static REAL *sa;	/*              :esiZNew.c */
  static REAL *dgF;	/*              :esiZNew.c */
  static REAL *d2gF;	/*              :esiZNew.c */
  static REAL *dgY;	/*              :esiZNew.c */
  static REAL *d2gY;	/*              :esiZNew.c */
  static REAL *aT;	/*              :esiZNew.c */
  static REAL *daT;	/*              :esiZNew.c */
  static REAL *aP;	/*              :esiZNew.c */
  static REAL *daP;	/*              :esiZNew.c */
  static REAL *sr;	/*              :esiZNew.c */
  static REAL *sra;	/*              :esiZNew.c */
  static REAL *srq;	/*              :esiZNew.c */
  static REAL *sraq;	/*              :esiZNew.c */
  static REAL *sz;	/*              :esiZNew.c */
  static REAL *sza;	/*              :esiZNew.c */
  static REAL *szq;	/*              :esiZNew.c */
  static REAL *szaq;	/*              :esiZNew.c */
  static REAL *aB;	/*              :esiZNew.c */
  static REAL *aBa;	/*              :esiZNew.c */
  static REAL *aBq;	/*              :esiZNew.c */
  static REAL *aBaq;	/*              :esiZNew.c */
  static REAL *gHq;	/*              :esiZNew.c */
  static REAL *gHaq;	/*              :esiZNew.c */
  static REAL *gHqq;	/*              :esiZNew.c */
  static REAL *gHaqq;	/*              :esiZNew.c */
  int IsStrFound();	/*              :esiZNew.c */
  int ESIMemAlloc();	/*              :esiZNew.c */
/*
   */
#ifdef Doc_ESIReadAD
\Section{Skeleton of \c{ESIReadAD()}}
  \begin{verbatim}
    ESIReadAD(){
    }
  \end{verbatim}
\Section{\c{ESIReadAD()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{char }&\c{*FNm}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{Undef }&\c{fopen()}&$$&\\
\c{Undef }&\c{NULL}&$$&\\
\c{Undef }&\c{printf()}&$$&\\
\c{Undef }&\c{fclose()}&$$&\\
\c{Undef }&\c{feof()}&$$&\\
\c{Undef }&\c{fgetc()}&$$&\\
\c{Undef }&\c{fscanf()}&$$&\\
\c{char }&\c{*FNm}&$$&\\
\c{int }&\c{IsStrFound()}&$$&\\
\c{static REAL }&\c{RBtor}&$$&\\
\c{static REAL }&\c{*gq}&$$&\\
\c{static int }&\c{Np1}&$$&\\
\c{static int }&\c{Na1}&$$&\\
\c{int }&\c{ESIMemAlloc()}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{*dgF}&$$&\\
\c{static REAL }&\c{*d2gF}&$$&\\
\c{static REAL }&\c{*dgY}&$$&\\
\c{static REAL }&\c{*d2gY}&$$&\\
\c{static REAL }&\c{*aT}&$$&\\
\c{static REAL }&\c{*daT}&$$&\\
\c{static REAL }&\c{*aP}&$$&\\
\c{static REAL }&\c{*daP}&$$&\\
\c{static REAL }&\c{*sr}&$$&\\
\c{static REAL }&\c{*sra}&$$&\\
\c{static REAL }&\c{*srq}&$$&\\
\c{static REAL }&\c{*sraq}&$$&\\
\c{static REAL }&\c{*sz}&$$&\\
\c{static REAL }&\c{*sza}&$$&\\
\c{static REAL }&\c{*szq}&$$&\\
\c{static REAL }&\c{*szaq}&$$&\\
\c{static REAL }&\c{*aB}&$$&\\
\c{static REAL }&\c{*aBa}&$$&\\
\c{static REAL }&\c{*aBq}&$$&\\
\c{static REAL }&\c{*aBaq}&$$&\\
\c{static REAL }&\c{*gHq}&$$&\\
\c{static REAL }&\c{*gHaq}&$$&\\
\c{static REAL }&\c{*gHqq}&$$&\\
\c{static REAL }&\c{*gHaqq}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int ESIInit()
{
  static REAL cgq0;	/*  :esiZNew.c */
  static REAL *gq;	/*  :esiZNew.c */
  static int Np;	/*  :esiZNew.c */
  static REAL crgq0;	/*  :esiZNew.c */
  static REAL hq;	/*  :esiZNew.c */
  static REAL rhq;	/*  :esiZNew.c */
  static REAL ha;	/*  :esiZNew.c */
  static REAL *sa;	/*  :esiZNew.c */
  static REAL rha;	/*  :esiZNew.c */
  static int Na1;	/*  :esiZNew.c */
  static REAL *gH;	/*  :esiZNew.c */
  static REAL *gHa;	/*  :esiZNew.c */
  static int Np1;	/*  :esiZNew.c */
  static REAL *gHq;	/*  :esiZNew.c */
  static double cr6;	/*  :esiZNew.c */
  static REAL *gHqq;	/*  :esiZNew.c */
  static REAL *gHaq;	/*  :esiZNew.c */
  static REAL *gHaqq;	/*  :esiZNew.c */
  static REAL *gF;	/*  :esiZNew.c */
  static REAL *gY;	/*  :esiZNew.c */
  static REAL *bsp;	/*  :esiZNew.c */
  static REAL *aF;	/*  :esiZNew.c */
  static REAL *dgF;	/*  :esiZNew.c */
  static REAL *d2gF;	/*  :esiZNew.c */
  static double cr2;	/*  :esiZNew.c */
  static double cr3;	/*  :esiZNew.c */
  static double cr4;	/*  :esiZNew.c */
  static double cr5;	/*  :esiZNew.c */
  static REAL *dgY;	/*  :esiZNew.c */
  static REAL *d2gY;	/*  :esiZNew.c */
  static double cr7;	/*  :esiZNew.c */
  static double cr8;	/*  :esiZNew.c */
  static REAL *aT;	/*  :esiZNew.c */
  static REAL *daT;	/*  :esiZNew.c */
  static REAL *aP;	/*  :esiZNew.c */
  static REAL *daP;	/*  :esiZNew.c */
  static int Na;	/*  :esiZNew.c */
  static REAL *dbsp;	/*  :esiZNew.c */
  static REAL aRBtor;	/*  :esiZNew.c */
  static REAL RBtor;	/*  :esiZNew.c */
  static REAL *daF;	/*  :esiZNew.c */
  int ZeroOfHermit();	/*  :esiZNew.c */
  int EPPAHermit();	/*  :esiZNew.c */
/*
   */
#ifdef Doc_ESIInit
\Section{Skeleton of \c{ESIInit()}}
  \begin{verbatim}
    ESIInit(){
    }
  \end{verbatim}
\Section{\c{ESIInit()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{Undef }&\c{sqrt()}&$$&\\
\c{Undef }&\c{fabs()}&$$&\\
\c{static REAL }&\c{cgq0}&$$&\\
\c{static REAL }&\c{*gq}&$$&\\
\c{static int }&\c{Np}&$$&\\
\c{static REAL }&\c{crgq0}&$$&\\
\c{static REAL }&\c{hq}&$$&\\
\c{static REAL }&\c{rhq}&$$&\\
\c{static REAL }&\c{ha}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{rha}&$$&\\
\c{static int }&\c{Na1}&$$&\\
\c{static REAL }&\c{*gH}&$$&\\
\c{static REAL }&\c{*gHa}&$$&\\
\c{static int }&\c{Np1}&$$&\\
\c{static REAL }&\c{*gHq}&$$&\\
\c{static double }&\c{cr6}&$$&\\
\c{static REAL }&\c{*gHqq}&$$&\\
\c{static REAL }&\c{*gHaq}&$$&\\
\c{static REAL }&\c{*gHaqq}&$$&\\
\c{static REAL }&\c{*gF}&$$&\\
\c{static REAL }&\c{*gY}&$$&\\
\c{static REAL }&\c{*bsp}&$$&\\
\c{static REAL }&\c{*aF}&$$&\\
\c{static REAL }&\c{*dgF}&$$&\\
\c{static REAL }&\c{*d2gF}&$$&\\
\c{static double }&\c{cr2}&$$&\\
\c{static double }&\c{cr3}&$$&\\
\c{static double }&\c{cr4}&$$&\\
\c{static double }&\c{cr5}&$$&\\
\c{static REAL }&\c{*dgY}&$$&\\
\c{static REAL }&\c{*d2gY}&$$&\\
\c{static double }&\c{cr7}&$$&\\
\c{static double }&\c{cr8}&$$&\\
\c{static REAL }&\c{*aT}&$$&\\
\c{static REAL }&\c{*daT}&$$&\\
\c{static REAL }&\c{*aP}&$$&\\
\c{static REAL }&\c{*daP}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{static REAL }&\c{*dbsp}&$$&\\
\c{static REAL }&\c{aRBtor}&$$&\\
\c{static REAL }&\c{RBtor}&$$&\\
\c{static REAL }&\c{*daF}&$$&\\
\c{int }&\c{ZeroOfHermit()}&$$&\\
\c{int }&\c{EPPAHermit()}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int esiread_(FName)
  char *FName;	/*  */
{
  char *FName;		/*  .esiread_():esiZNew.c */
  int ESIReadAD();	/*             :esiZNew.c */
  int ESIReadBD();	/*             :esiZNew.c */
  int ESIInit();	/*             :esiZNew.c */
/*
   */
#ifdef Doc_esiread_
\Section{Skeleton of \c{esiread_()}}
  \begin{verbatim}
    esiread_(){
    }
  \end{verbatim}
\Section{\c{esiread_()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{char }&\c{*FName}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{Undef }&\c{isspace()}&$$&\\
\c{char }&\c{*FName}&$$&\\
\c{int }&\c{ESIReadAD()}&$$&\\
\c{int }&\c{ESIReadBD()}&$$&\\
\c{int }&\c{ESIInit()}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

void esilink2c_(XaF,XaFa,XgFa,XgFaa,XgYa,XgYaa,XT,XTa,XP,XPa,Xr,Xra,Xrq,Xz,Xza,Xzq,XB,XBa,XBq,Xgh,Xgha,Xghq,Xsw)
  REAL *XaF;	/*  */
  REAL *XaFa;	/*  */
  REAL *XgFa;	/*  */
  REAL *XgFaa;	/*  */
  REAL *XgYa;	/*  */
  REAL *XgYaa;	/*  */
  REAL *XT;	/*  */
  REAL *XTa;	/*  */
  REAL *XP;	/*  */
  REAL *XPa;	/*  */
  REAL *Xr;	/*  */
  REAL *Xra;	/*  */
  REAL *Xrq;	/*  */
  REAL *Xz;	/*  */
  REAL *Xza;	/*  */
  REAL *Xzq;	/*  */
  REAL *XB;	/*  */
  REAL *XBa;	/*  */
  REAL *XBq;	/*  */
  REAL *Xgh;	/*  */
  REAL *Xgha;	/*  */
  REAL *Xghq;	/*  */
  int *Xsw;	/*  */
{
  REAL *XaF;		/*  .esilink2c_():esiZNew.c */
  REAL *XaFa;		/*  .esilink2c_():esiZNew.c */
  REAL *XgFa;		/*  .esilink2c_():esiZNew.c */
  REAL *XgFaa;		/*  .esilink2c_():esiZNew.c */
  REAL *XgYa;		/*  .esilink2c_():esiZNew.c */
  REAL *XgYaa;		/*  .esilink2c_():esiZNew.c */
  REAL *XT;		/*  .esilink2c_():esiZNew.c */
  REAL *XTa;		/*  .esilink2c_():esiZNew.c */
  REAL *XP;		/*  .esilink2c_():esiZNew.c */
  REAL *XPa;		/*  .esilink2c_():esiZNew.c */
  REAL *Xr;		/*  .esilink2c_():esiZNew.c */
  REAL *Xra;		/*  .esilink2c_():esiZNew.c */
  REAL *Xrq;		/*  .esilink2c_():esiZNew.c */
  REAL *Xz;		/*  .esilink2c_():esiZNew.c */
  REAL *Xza;		/*  .esilink2c_():esiZNew.c */
  REAL *Xzq;		/*  .esilink2c_():esiZNew.c */
  REAL *XB;		/*  .esilink2c_():esiZNew.c */
  REAL *XBa;		/*  .esilink2c_():esiZNew.c */
  REAL *XBq;		/*  .esilink2c_():esiZNew.c */
  REAL *Xgh;		/*  .esilink2c_():esiZNew.c */
  REAL *Xgha;		/*  .esilink2c_():esiZNew.c */
  REAL *Xghq;		/*  .esilink2c_():esiZNew.c */
  int *Xsw;		/*  .esilink2c_():esiZNew.c */
  static REAL *F;	/*               :esiZNew.c */
  static REAL *Fa;	/*               :esiZNew.c */
  static REAL *gFa;	/*               :esiZNew.c */
  static REAL *gFaa;	/*               :esiZNew.c */
  static REAL *gYa;	/*               :esiZNew.c */
  static REAL *gYaa;	/*               :esiZNew.c */
  static REAL *T;	/*               :esiZNew.c */
  static REAL *Ta;	/*               :esiZNew.c */
  static REAL *P;	/*               :esiZNew.c */
  static REAL *Pa;	/*               :esiZNew.c */
  static REAL *r;	/*               :esiZNew.c */
  static REAL *ra;	/*               :esiZNew.c */
  static REAL *rq;	/*               :esiZNew.c */
  static REAL *z;	/*               :esiZNew.c */
  static REAL *za;	/*               :esiZNew.c */
  static REAL *zq;	/*               :esiZNew.c */
  static REAL *B;	/*               :esiZNew.c */
  static REAL *Ba;	/*               :esiZNew.c */
  static REAL *Bq;	/*               :esiZNew.c */
  static REAL *gh;	/*               :esiZNew.c */
  static REAL *gha;	/*               :esiZNew.c */
  static REAL *ghq;	/*               :esiZNew.c */
  static int *sw;	/*               :esiZNew.c */
/*
   */
#ifdef Doc_esilink2c_
\Section{Skeleton of \c{esilink2c_()}}
  \begin{verbatim}
    esilink2c_(){
    }
  \end{verbatim}
\Section{\c{esilink2c_()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{REAL }&\c{*XaF}&$$&\\
\c{REAL }&\c{*XaFa}&$$&\\
\c{REAL }&\c{*XgFa}&$$&\\
\c{REAL }&\c{*XgFaa}&$$&\\
\c{REAL }&\c{*XgYa}&$$&\\
\c{REAL }&\c{*XgYaa}&$$&\\
\c{REAL }&\c{*XT}&$$&\\
\c{REAL }&\c{*XTa}&$$&\\
\c{REAL }&\c{*XP}&$$&\\
\c{REAL }&\c{*XPa}&$$&\\
\c{REAL }&\c{*Xr}&$$&\\
\c{REAL }&\c{*Xra}&$$&\\
\c{REAL }&\c{*Xrq}&$$&\\
\c{REAL }&\c{*Xz}&$$&\\
\c{REAL }&\c{*Xza}&$$&\\
\c{REAL }&\c{*Xzq}&$$&\\
\c{REAL }&\c{*XB}&$$&\\
\c{REAL }&\c{*XBa}&$$&\\
\c{REAL }&\c{*XBq}&$$&\\
\c{REAL }&\c{*Xgh}&$$&\\
\c{REAL }&\c{*Xgha}&$$&\\
\c{REAL }&\c{*Xghq}&$$&\\
\c{int }&\c{*Xsw}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{static REAL }&\c{*F}&$$&\\
\c{REAL }&\c{*XaF}&$$&\\
\c{static REAL }&\c{*Fa}&$$&\\
\c{REAL }&\c{*XaFa}&$$&\\
\c{static REAL }&\c{*gFa}&$$&\\
\c{REAL }&\c{*XgFa}&$$&\\
\c{static REAL }&\c{*gFaa}&$$&\\
\c{REAL }&\c{*XgFaa}&$$&\\
\c{static REAL }&\c{*gYa}&$$&\\
\c{REAL }&\c{*XgYa}&$$&\\
\c{static REAL }&\c{*gYaa}&$$&\\
\c{REAL }&\c{*XgYaa}&$$&\\
\c{static REAL }&\c{*T}&$$&\\
\c{REAL }&\c{*XT}&$$&\\
\c{static REAL }&\c{*Ta}&$$&\\
\c{REAL }&\c{*XTa}&$$&\\
\c{static REAL }&\c{*P}&$$&\\
\c{REAL }&\c{*XP}&$$&\\
\c{static REAL }&\c{*Pa}&$$&\\
\c{REAL }&\c{*XPa}&$$&\\
\c{static REAL }&\c{*r}&$$&\\
\c{REAL }&\c{*Xr}&$$&\\
\c{static REAL }&\c{*ra}&$$&\\
\c{REAL }&\c{*Xra}&$$&\\
\c{static REAL }&\c{*rq}&$$&\\
\c{REAL }&\c{*Xrq}&$$&\\
\c{static REAL }&\c{*z}&$$&\\
\c{REAL }&\c{*Xz}&$$&\\
\c{static REAL }&\c{*za}&$$&\\
\c{REAL }&\c{*Xza}&$$&\\
\c{static REAL }&\c{*zq}&$$&\\
\c{REAL }&\c{*Xzq}&$$&\\
\c{static REAL }&\c{*B}&$$&\\
\c{REAL }&\c{*XB}&$$&\\
\c{static REAL }&\c{*Ba}&$$&\\
\c{REAL }&\c{*XBa}&$$&\\
\c{static REAL }&\c{*Bq}&$$&\\
\c{REAL }&\c{*XBq}&$$&\\
\c{static REAL }&\c{*gh}&$$&\\
\c{REAL }&\c{*Xgh}&$$&\\
\c{static REAL }&\c{*gha}&$$&\\
\c{REAL }&\c{*Xgha}&$$&\\
\c{static REAL }&\c{*ghq}&$$&\\
\c{REAL }&\c{*Xghq}&$$&\\
\c{static int }&\c{*sw}&$$&\\
\c{int }&\c{*Xsw}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int ESIGetSize(na,np)
  int *na;	/*  */
  int *np;	/*  */
{
  int *na;	/*  .ESIGetSize():esiZNew.c */
  int *np;	/*  .ESIGetSize():esiZNew.c */
  static int Na;/*               :esiZNew.c */
  static int Np;/*               :esiZNew.c */
/*
   */
#ifdef Doc_ESIGetSize
\Section{Skeleton of \c{ESIGetSize()}}
  \begin{verbatim}
    ESIGetSize(){
    }
  \end{verbatim}
\Section{\c{ESIGetSize()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{int }&\c{*na}&$$&\\
\c{int }&\c{*np}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{int }&\c{*na}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{int }&\c{*np}&$$&\\
\c{static int }&\c{Np}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int EsiGetLink2c(Xr,Xra,Xrq,Xz,Xza,Xzq)
  double **Xr;	/*  */
  double **Xra;	/*  */
  double **Xrq;	/*  */
  double **Xz;	/*  */
  double **Xza;	/*  */
  double **Xzq;	/*  */
{
  double **Xr;		/*  .EsiGetLink2c():esiZNew.c */
  double **Xra;		/*  .EsiGetLink2c():esiZNew.c */
  double **Xrq;		/*  .EsiGetLink2c():esiZNew.c */
  double **Xz;		/*  .EsiGetLink2c():esiZNew.c */
  double **Xza;		/*  .EsiGetLink2c():esiZNew.c */
  double **Xzq;		/*  .EsiGetLink2c():esiZNew.c */
  static REAL *r;	/*                 :esiZNew.c */
  static REAL *ra;	/*                 :esiZNew.c */
  static REAL *rq;	/*                 :esiZNew.c */
  static REAL *z;	/*                 :esiZNew.c */
  static REAL *za;	/*                 :esiZNew.c */
  static REAL *zq;	/*                 :esiZNew.c */
/*
   */
#ifdef Doc_EsiGetLink2c
\Section{Skeleton of \c{EsiGetLink2c()}}
  \begin{verbatim}
    EsiGetLink2c(){
    }
  \end{verbatim}
\Section{\c{EsiGetLink2c()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{**Xr}&$$&\\
\c{double }&\c{**Xra}&$$&\\
\c{double }&\c{**Xrq}&$$&\\
\c{double }&\c{**Xz}&$$&\\
\c{double }&\c{**Xza}&$$&\\
\c{double }&\c{**Xzq}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{**Xr}&$$&\\
\c{static REAL }&\c{*r}&$$&\\
\c{double }&\c{**Xra}&$$&\\
\c{static REAL }&\c{*ra}&$$&\\
\c{double }&\c{**Xrq}&$$&\\
\c{static REAL }&\c{*rq}&$$&\\
\c{double }&\c{**Xz}&$$&\\
\c{static REAL }&\c{*z}&$$&\\
\c{double }&\c{**Xza}&$$&\\
\c{static REAL }&\c{*za}&$$&\\
\c{double }&\c{**Xzq}&$$&\\
\c{static REAL }&\c{*zq}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int esi2gFgY(gf,gy,ap,n)
  double *gf;	/*  */
  double *gy;	/*  */
  double *ap;	/*  */
  int n;	/*  */
{
  int n;		/*  .esi2gFgY():esiZNew.c */
  double *ap;		/*  .esi2gFgY():esiZNew.c */
  double *gf;		/*  .esi2gFgY():esiZNew.c */
  double *gy;		/*  .esi2gFgY():esiZNew.c */
  static REAL *sa;	/*             :esiZNew.c */
  static REAL rha;	/*             :esiZNew.c */
  static int Na;	/*             :esiZNew.c */
  static REAL *gF;	/*             :esiZNew.c */
  static REAL *gY;	/*             :esiZNew.c */
  static double cr2;	/*             :esiZNew.c */
  static double cr3;	/*             :esiZNew.c */
  static double cr4;	/*             :esiZNew.c */
  static double cr5;	/*             :esiZNew.c */
  static REAL ha;	/*             :esiZNew.c */
  static REAL *dgF;	/*             :esiZNew.c */
  static REAL *d2gF;	/*             :esiZNew.c */
  static REAL *dgY;	/*             :esiZNew.c */
  static REAL *d2gY;	/*             :esiZNew.c */
  static double c2gp;	/*             :esiZNew.c */
/*
   */
#ifdef Doc_esi2gFgY
\Section{Skeleton of \c{esi2gFgY()}}
  \begin{verbatim}
    esi2gFgY(){
    }
  \end{verbatim}
\Section{\c{esi2gFgY()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*gf}&$$&\\
\c{double }&\c{*gy}&$$&\\
\c{double }&\c{*ap}&$$&\\
\c{int }&\c{n}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{int }&\c{n}&$$&\\
\c{double }&\c{*ap}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{rha}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{double }&\c{*gf}&$$&\\
\c{static REAL }&\c{*gF}&$$&\\
\c{double }&\c{*gy}&$$&\\
\c{static REAL }&\c{*gY}&$$&\\
\c{static double }&\c{cr2}&$$&\\
\c{static double }&\c{cr3}&$$&\\
\c{static double }&\c{cr4}&$$&\\
\c{static double }&\c{cr5}&$$&\\
\c{static REAL }&\c{ha}&$$&\\
\c{static REAL }&\c{*dgF}&$$&\\
\c{static REAL }&\c{*d2gF}&$$&\\
\c{static REAL }&\c{*dgY}&$$&\\
\c{static REAL }&\c{*d2gY}&$$&\\
\c{static double }&\c{c2gp}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int esi2bsp(bsp0,dbsp0,ap,n)
  double *bsp0;		/*  */
  double *dbsp0;	/*  */
  double *ap;		/*  */
  int n;		/*  */
{
  int n;		/*  .esi2bsp():esiZNew.c */
  double *ap;		/*  .esi2bsp():esiZNew.c */
  double *dbsp0;	/*  .esi2bsp():esiZNew.c */
  double *bsp0;		/*  .esi2bsp():esiZNew.c */
  static REAL *sa;	/*            :esiZNew.c */
  static REAL rha;	/*            :esiZNew.c */
  static int Na;	/*            :esiZNew.c */
  static REAL *aP;	/*            :esiZNew.c */
  static REAL *dgY;	/*            :esiZNew.c */
  static REAL *sp;	/*            :esiZNew.c */
  static REAL ha;	/*            :esiZNew.c */
  static REAL *d2gY;	/*            :esiZNew.c */
  static double cr2;	/*            :esiZNew.c */
  static double cr3;	/*            :esiZNew.c */
  static double cr4;	/*            :esiZNew.c */
  static double cr5;	/*            :esiZNew.c */
  static double cr6;	/*            :esiZNew.c */
  static double cr7;	/*            :esiZNew.c */
  static double cr8;	/*            :esiZNew.c */
  static REAL *daP;	/*            :esiZNew.c */
  static REAL *bsp;	/*            :esiZNew.c */
/*
   */
#ifdef Doc_esi2bsp
\Section{Skeleton of \c{esi2bsp()}}
  \begin{verbatim}
    esi2bsp(){
    }
  \end{verbatim}
\Section{\c{esi2bsp()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*bsp0}&$$&\\
\c{double }&\c{*dbsp0}&$$&\\
\c{double }&\c{*ap}&$$&\\
\c{int }&\c{n}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{int }&\c{n}&$$&\\
\c{double }&\c{*ap}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{rha}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{double }&\c{*dbsp0}&$$&\\
\c{static REAL }&\c{*aP}&$$&\\
\c{static REAL }&\c{*dgY}&$$&\\
\c{double }&\c{*bsp0}&$$&\\
\c{static REAL }&\c{*sp}&$$&\\
\c{static REAL }&\c{ha}&$$&\\
\c{static REAL }&\c{*d2gY}&$$&\\
\c{static double }&\c{cr2}&$$&\\
\c{static double }&\c{cr3}&$$&\\
\c{static double }&\c{cr4}&$$&\\
\c{static double }&\c{cr5}&$$&\\
\c{static double }&\c{cr6}&$$&\\
\c{static double }&\c{cr7}&$$&\\
\c{static double }&\c{cr8}&$$&\\
\c{static REAL }&\c{*daP}&$$&\\
\c{static REAL }&\c{*bsp}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int esi2EqProfiles(sF,sFa,sgFa,sgFaa,sgYa,sgYaa,sT,sTa,sP,sPa,a,n)
  REAL *sF;	/*  */
  REAL *sFa;	/*  */
  REAL *sgFa;	/*  */
  REAL *sgFaa;	/*  */
  REAL *sgYa;	/*  */
  REAL *sgYaa;	/*  */
  REAL *sT;	/*  */
  REAL *sTa;	/*  */
  REAL *sP;	/*  */
  REAL *sPa;	/*  */
  REAL *a;	/*  */
  int n;	/*  */
{
  int n;		/*  .esi2EqProfiles():esiZNew.c */
  REAL *a;		/*  .esi2EqProfiles():esiZNew.c */
  REAL *sF;		/*  .esi2EqProfiles():esiZNew.c */
  REAL *sFa;		/*  .esi2EqProfiles():esiZNew.c */
  REAL *sgFa;		/*  .esi2EqProfiles():esiZNew.c */
  REAL *sgFaa;		/*  .esi2EqProfiles():esiZNew.c */
  REAL *sgYa;		/*  .esi2EqProfiles():esiZNew.c */
  REAL *sgYaa;		/*  .esi2EqProfiles():esiZNew.c */
  REAL *sT;		/*  .esi2EqProfiles():esiZNew.c */
  REAL *sTa;		/*  .esi2EqProfiles():esiZNew.c */
  REAL *sP;		/*  .esi2EqProfiles():esiZNew.c */
  REAL *sPa;		/*  .esi2EqProfiles():esiZNew.c */
  static REAL *aF;	/*                   :esiZNew.c */
  static REAL *dgF;	/*                   :esiZNew.c */
  static REAL *dgY;	/*                   :esiZNew.c */
  static REAL *aT;	/*                   :esiZNew.c */
  static REAL *aP;	/*                   :esiZNew.c */
  static REAL *sp;	/*                   :esiZNew.c */
  static REAL *bsp;	/*                   :esiZNew.c */
  static REAL *spa;	/*                   :esiZNew.c */
  static REAL A;	/*                   :esiZNew.c */
  static int i0;	/*                   :esiZNew.c */
  static REAL *sa;	/*                   :esiZNew.c */
  static REAL rha;	/*                   :esiZNew.c */
  static int Na;	/*                   :esiZNew.c */
  static int i1;	/*                   :esiZNew.c */
  static REAL ha;	/*                   :esiZNew.c */
  static REAL *daF;	/*                   :esiZNew.c */
  static REAL *d2gF;	/*                   :esiZNew.c */
  static REAL *d2gY;	/*                   :esiZNew.c */
  static REAL *daT;	/*                   :esiZNew.c */
  static REAL *daP;	/*                   :esiZNew.c */
  static REAL *dbsp;	/*                   :esiZNew.c */
/*
   */
#ifdef Doc_esi2EqProfiles
\Section{Skeleton of \c{esi2EqProfiles()}}
  \begin{verbatim}
    esi2EqProfiles(){
    }
  \end{verbatim}
\Section{\c{esi2EqProfiles()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{REAL }&\c{*sF}&$$&\\
\c{REAL }&\c{*sFa}&$$&\\
\c{REAL }&\c{*sgFa}&$$&\\
\c{REAL }&\c{*sgFaa}&$$&\\
\c{REAL }&\c{*sgYa}&$$&\\
\c{REAL }&\c{*sgYaa}&$$&\\
\c{REAL }&\c{*sT}&$$&\\
\c{REAL }&\c{*sTa}&$$&\\
\c{REAL }&\c{*sP}&$$&\\
\c{REAL }&\c{*sPa}&$$&\\
\c{REAL }&\c{*a}&$$&\\
\c{int }&\c{n}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{Undef }&\c{NULL}&$$&\\
\c{int }&\c{n}&$$&\\
\c{REAL }&\c{*a}&$$&\\
\c{REAL }&\c{*sF}&$$&\\
\c{static REAL }&\c{*aF}&$$&\\
\c{REAL }&\c{*sFa}&$$&\\
\c{REAL }&\c{*sgFa}&$$&\\
\c{REAL }&\c{*sgFaa}&$$&\\
\c{static REAL }&\c{*dgF}&$$&\\
\c{REAL }&\c{*sgYa}&$$&\\
\c{REAL }&\c{*sgYaa}&$$&\\
\c{static REAL }&\c{*dgY}&$$&\\
\c{REAL }&\c{*sT}&$$&\\
\c{static REAL }&\c{*aT}&$$&\\
\c{REAL }&\c{*sTa}&$$&\\
\c{REAL }&\c{*sP}&$$&\\
\c{static REAL }&\c{*aP}&$$&\\
\c{REAL }&\c{*sPa}&$$&\\
\c{static REAL }&\c{*sp}&$$&\\
\c{static REAL }&\c{*bsp}&$$&\\
\c{static REAL }&\c{*spa}&$$&\\
\c{static REAL }&\c{A}&$$&\\
\c{static int }&\c{i0}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{rha}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{static int }&\c{i1}&$$&\\
\c{static REAL }&\c{ha}&$$&\\
\c{static REAL }&\c{*daF}&$$&\\
\c{static REAL }&\c{*d2gF}&$$&\\
\c{static REAL }&\c{*d2gY}&$$&\\
\c{static REAL }&\c{*daT}&$$&\\
\c{static REAL }&\c{*daP}&$$&\\
\c{static REAL }&\c{*dbsp}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int esiget2dfunctions_(a0,gq0,n)
  REAL *a0;	/*  */
  REAL *gq0;	/*  */
  int *n;	/*  */
{
  int *n;		/*  .esiget2dfunctions_():esiZNew.c */
  REAL *gq0;		/*  .esiget2dfunctions_():esiZNew.c */
  REAL *a0;		/*  .esiget2dfunctions_():esiZNew.c */
  static REAL crgq0;	/*                       :esiZNew.c */
  static REAL cgq0;	/*                       :esiZNew.c */
  static REAL rhq;	/*                       :esiZNew.c */
  static int Np;	/*                       :esiZNew.c */
  static REAL *gq;	/*                       :esiZNew.c */
  static double cr4;	/*                       :esiZNew.c */
  static double cr2;	/*                       :esiZNew.c */
  static double cr3;	/*                       :esiZNew.c */
  static REAL hq;	/*                       :esiZNew.c */
  static int i0;	/*                       :esiZNew.c */
  static REAL *sa;	/*                       :esiZNew.c */
  static REAL rha;	/*                       :esiZNew.c */
  static int Na;	/*                       :esiZNew.c */
  static int i1;	/*                       :esiZNew.c */
  static REAL ha;	/*                       :esiZNew.c */
  static int j00;	/*                       :esiZNew.c */
  static int Np1;	/*                       :esiZNew.c */
  static int j01;	/*                       :esiZNew.c */
  static int j10;	/*                       :esiZNew.c */
  static int j11;	/*                       :esiZNew.c */
  static REAL *sr;	/*                       :esiZNew.c */
  static REAL *sra;	/*                       :esiZNew.c */
  static REAL *srq;	/*                       :esiZNew.c */
  static REAL *sraq;	/*                       :esiZNew.c */
  static REAL *r;	/*                       :esiZNew.c */
  static REAL *rq;	/*                       :esiZNew.c */
  static REAL *ra;	/*                       :esiZNew.c */
  static REAL *sz;	/*                       :esiZNew.c */
  static REAL *sza;	/*                       :esiZNew.c */
  static REAL *szq;	/*                       :esiZNew.c */
  static REAL *szaq;	/*                       :esiZNew.c */
  static REAL *z;	/*                       :esiZNew.c */
  static REAL *zq;	/*                       :esiZNew.c */
  static REAL *za;	/*                       :esiZNew.c */
  static REAL *aB;	/*                       :esiZNew.c */
  static REAL *aBa;	/*                       :esiZNew.c */
  static REAL *aBq;	/*                       :esiZNew.c */
  static REAL *aBaq;	/*                       :esiZNew.c */
  static REAL *B;	/*                       :esiZNew.c */
  static REAL *Bq;	/*                       :esiZNew.c */
  static REAL *Ba;	/*                       :esiZNew.c */
  static REAL *gHq;	/*                       :esiZNew.c */
  static REAL *gHaq;	/*                       :esiZNew.c */
  static REAL *gHqq;	/*                       :esiZNew.c */
  static REAL *gHaqq;	/*                       :esiZNew.c */
  static REAL *ghq;	/*                       :esiZNew.c */
  static REAL *gh;	/*                       :esiZNew.c */
  static REAL *gH;	/*                       :esiZNew.c */
  static REAL *gHa;	/*                       :esiZNew.c */
  static REAL *gha;	/*                       :esiZNew.c */
  static REAL *F;	/*                       :esiZNew.c */
  static REAL *aF;	/*                       :esiZNew.c */
  static REAL *daF;	/*                       :esiZNew.c */
  static REAL *Fa;	/*                       :esiZNew.c */
  static REAL *gFa;	/*                       :esiZNew.c */
  static REAL *dgF;	/*                       :esiZNew.c */
  static REAL *d2gF;	/*                       :esiZNew.c */
  static REAL *gFaa;	/*                       :esiZNew.c */
  static REAL *gYa;	/*                       :esiZNew.c */
  static REAL *dgY;	/*                       :esiZNew.c */
  static REAL *d2gY;	/*                       :esiZNew.c */
  static REAL *gYaa;	/*                       :esiZNew.c */
  static REAL *T;	/*                       :esiZNew.c */
  static REAL *aT;	/*                       :esiZNew.c */
  static REAL *daT;	/*                       :esiZNew.c */
  static REAL *Ta;	/*                       :esiZNew.c */
  static REAL *P;	/*                       :esiZNew.c */
  static REAL *aP;	/*                       :esiZNew.c */
  static REAL *daP;	/*                       :esiZNew.c */
  static REAL *Pa;	/*                       :esiZNew.c */
/*
   */
#ifdef Doc_esiget2dfunctions_
\Section{Skeleton of \c{esiget2dfunctions_()}}
  \begin{verbatim}
    esiget2dfunctions_(){
    }
  \end{verbatim}
\Section{\c{esiget2dfunctions_()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{REAL }&\c{*a0}&$$&\\
\c{REAL }&\c{*gq0}&$$&\\
\c{int }&\c{*n}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{int }&\c{*n}&$$&\\
\c{REAL }&\c{*gq0}&$$&\\
\c{static REAL }&\c{crgq0}&$$&\\
\c{static REAL }&\c{cgq0}&$$&\\
\c{static REAL }&\c{rhq}&$$&\\
\c{static int }&\c{Np}&$$&\\
\c{static REAL }&\c{*gq}&$$&\\
\c{static double }&\c{cr4}&$$&\\
\c{static double }&\c{cr2}&$$&\\
\c{static double }&\c{cr3}&$$&\\
\c{static REAL }&\c{hq}&$$&\\
\c{REAL }&\c{*a0}&$$&\\
\c{static int }&\c{i0}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{rha}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{static int }&\c{i1}&$$&\\
\c{static REAL }&\c{ha}&$$&\\
\c{static int }&\c{j00}&$$&\\
\c{static int }&\c{Np1}&$$&\\
\c{static int }&\c{j01}&$$&\\
\c{static int }&\c{j10}&$$&\\
\c{static int }&\c{j11}&$$&\\
\c{static REAL }&\c{*sr}&$$&\\
\c{static REAL }&\c{*sra}&$$&\\
\c{static REAL }&\c{*srq}&$$&\\
\c{static REAL }&\c{*sraq}&$$&\\
\c{static REAL }&\c{*r}&$$&\\
\c{static REAL }&\c{*rq}&$$&\\
\c{static REAL }&\c{*ra}&$$&\\
\c{static REAL }&\c{*sz}&$$&\\
\c{static REAL }&\c{*sza}&$$&\\
\c{static REAL }&\c{*szq}&$$&\\
\c{static REAL }&\c{*szaq}&$$&\\
\c{static REAL }&\c{*z}&$$&\\
\c{static REAL }&\c{*zq}&$$&\\
\c{static REAL }&\c{*za}&$$&\\
\c{static REAL }&\c{*aB}&$$&\\
\c{static REAL }&\c{*aBa}&$$&\\
\c{static REAL }&\c{*aBq}&$$&\\
\c{static REAL }&\c{*aBaq}&$$&\\
\c{static REAL }&\c{*B}&$$&\\
\c{static REAL }&\c{*Bq}&$$&\\
\c{static REAL }&\c{*Ba}&$$&\\
\c{static REAL }&\c{*gHq}&$$&\\
\c{static REAL }&\c{*gHaq}&$$&\\
\c{static REAL }&\c{*gHqq}&$$&\\
\c{static REAL }&\c{*gHaqq}&$$&\\
\c{static REAL }&\c{*ghq}&$$&\\
\c{static REAL }&\c{*gh}&$$&\\
\c{static REAL }&\c{*gH}&$$&\\
\c{static REAL }&\c{*gHa}&$$&\\
\c{static REAL }&\c{*gha}&$$&\\
\c{static REAL }&\c{*F}&$$&\\
\c{static REAL }&\c{*aF}&$$&\\
\c{static REAL }&\c{*daF}&$$&\\
\c{static REAL }&\c{*Fa}&$$&\\
\c{static REAL }&\c{*gFa}&$$&\\
\c{static REAL }&\c{*dgF}&$$&\\
\c{static REAL }&\c{*d2gF}&$$&\\
\c{static REAL }&\c{*gFaa}&$$&\\
\c{static REAL }&\c{*gYa}&$$&\\
\c{static REAL }&\c{*dgY}&$$&\\
\c{static REAL }&\c{*d2gY}&$$&\\
\c{static REAL }&\c{*gYaa}&$$&\\
\c{static REAL }&\c{*T}&$$&\\
\c{static REAL }&\c{*aT}&$$&\\
\c{static REAL }&\c{*daT}&$$&\\
\c{static REAL }&\c{*Ta}&$$&\\
\c{static REAL }&\c{*P}&$$&\\
\c{static REAL }&\c{*aP}&$$&\\
\c{static REAL }&\c{*daP}&$$&\\
\c{static REAL }&\c{*Pa}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

void esigetdensity_(ne,dne,Te,Ti,a)
  double *ne;	/*  */
  double *dne;	/*  */
  double *Te;	/*  */
  double *Ti;	/*  */
  double *a;	/*  */
{
  double *Te;		/*  .esigetdensity_():esiZNew.c */
  double *Ti;		/*  .esigetdensity_():esiZNew.c */
  double *a;		/*  .esigetdensity_():esiZNew.c */
  double *ne;		/*  .esigetdensity_():esiZNew.c */
  double *dne;		/*  .esigetdensity_():esiZNew.c */
  static double cbsp2ne;/*                   :esiZNew.c */
  static REAL *sa;	/*                   :esiZNew.c */
  static REAL rha;	/*                   :esiZNew.c */
  static int Na;	/*                   :esiZNew.c */
  static REAL ha;	/*                   :esiZNew.c */
  static REAL *dbsp;	/*                   :esiZNew.c */
  static REAL *bsp;	/*                   :esiZNew.c */
/*
   */
#ifdef Doc_esigetdensity_
\Section{Skeleton of \c{esigetdensity_()}}
  \begin{verbatim}
    esigetdensity_(){
    }
  \end{verbatim}
\Section{\c{esigetdensity_()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*ne}&$$&\\
\c{double }&\c{*dne}&$$&\\
\c{double }&\c{*Te}&$$&\\
\c{double }&\c{*Ti}&$$&\\
\c{double }&\c{*a}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{Undef }&\c{NULL}&$$&\\
\c{static double }&\c{cbsp2ne}&$$&\\
\c{double }&\c{*Te}&$$&\\
\c{double }&\c{*Ti}&$$&\\
\c{double }&\c{*a}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{rha}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{static REAL }&\c{ha}&$$&\\
\c{static REAL }&\c{*dbsp}&$$&\\
\c{static REAL }&\c{*bsp}&$$&\\
\c{double }&\c{*ne}&$$&\\
\c{double }&\c{*dne}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

void esi2baIpl(Ipl)
  double *Ipl;	/*  */
{
  double *Ipl;		/*  .esi2baIpl():esiZNew.c */
  static int Np1;	/*              :esiZNew.c */
  static int Na;	/*              :esiZNew.c */
  static int Np;	/*              :esiZNew.c */
  static REAL *srq;	/*              :esiZNew.c */
  static REAL *szq;	/*              :esiZNew.c */
  static REAL *sr;	/*              :esiZNew.c */
  static REAL *sra;	/*              :esiZNew.c */
  static REAL *sza;	/*              :esiZNew.c */
  static REAL *dgY;	/*              :esiZNew.c */
/*
   */
#ifdef Doc_esi2baIpl
\Section{Skeleton of \c{esi2baIpl()}}
  \begin{verbatim}
    esi2baIpl(){
    }
  \end{verbatim}
\Section{\c{esi2baIpl()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*Ipl}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*Ipl}&$$&\\
\c{static int }&\c{Np1}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{static int }&\c{Np}&$$&\\
\c{static REAL }&\c{*srq}&$$&\\
\c{static REAL }&\c{*szq}&$$&\\
\c{static REAL }&\c{*sr}&$$&\\
\c{static REAL }&\c{*sra}&$$&\\
\c{static REAL }&\c{*sza}&$$&\\
\c{static REAL }&\c{*dgY}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

void esiplcurrent_(Ipl)
  double *Ipl;	/*  */
{
  double *Ipl;		/*  .esiplcurrent_():esiZNew.c */
  void esi2baIpl();	/*                  :esiZNew.c */
/*
   */
#ifdef Doc_esiplcurrent_
\Section{Skeleton of \c{esiplcurrent_()}}
  \begin{verbatim}
    esiplcurrent_(){
    }
  \end{verbatim}
\Section{\c{esiplcurrent_()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*Ipl}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{void }&\c{esi2baIpl()}&$$&\\
\c{double }&\c{*Ipl}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int esiGetgm(gm,gma,a)
  double *gm;	/*  */
  double *gma;	/*  */
  double a;	/*  */
{
  double a;		/*  .esiGetgm():esiZNew.c */
  double *gm;		/*  .esiGetgm():esiZNew.c */
  double *gma;		/*  .esiGetgm():esiZNew.c */
  static int i0;	/*             :esiZNew.c */
  static REAL *sa;	/*             :esiZNew.c */
  static REAL rha;	/*             :esiZNew.c */
  static int Na;	/*             :esiZNew.c */
  static int i1;	/*             :esiZNew.c */
  static REAL XF;	/*             :esiZNew.c */
  static REAL XD;	/*             :esiZNew.c */
  static REAL ha;	/*             :esiZNew.c */
  static REAL xF;	/*             :esiZNew.c */
  static REAL xD;	/*             :esiZNew.c */
  static REAL dX;	/*             :esiZNew.c */
  static REAL dXD;	/*             :esiZNew.c */
  static REAL dxD;	/*             :esiZNew.c */
  static REAL *dgF;	/*             :esiZNew.c */
  static REAL *d2gF;	/*             :esiZNew.c */
  static REAL *dgY;	/*             :esiZNew.c */
  static REAL *d2gY;	/*             :esiZNew.c */
/*
   */
#ifdef Doc_esiGetgm
\Section{Skeleton of \c{esiGetgm()}}
  \begin{verbatim}
    esiGetgm(){
    }
  \end{verbatim}
\Section{\c{esiGetgm()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*gm}&$$&\\
\c{double }&\c{*gma}&$$&\\
\c{double }&\c{a}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{Undef }&\c{NULL}&$$&\\
\c{double }&\c{a}&$$&\\
\c{static int }&\c{i0}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{rha}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{static int }&\c{i1}&$$&\\
\c{static REAL }&\c{XF}&$$&\\
\c{static REAL }&\c{XD}&$$&\\
\c{static REAL }&\c{ha}&$$&\\
\c{static REAL }&\c{xF}&$$&\\
\c{static REAL }&\c{xD}&$$&\\
\c{static REAL }&\c{dX}&$$&\\
\c{static REAL }&\c{dXD}&$$&\\
\c{static REAL }&\c{dxD}&$$&\\
\c{static REAL }&\c{*dgF}&$$&\\
\c{static REAL }&\c{*d2gF}&$$&\\
\c{static REAL }&\c{*dgY}&$$&\\
\c{static REAL }&\c{*d2gY}&$$&\\
\c{double }&\c{*gm}&$$&\\
\c{double }&\c{*gma}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int esiget2d_(m,G,Ga,Gq,a0,gq0,n)
  int m;	/*  */
  REAL *G;	/*  */
  REAL *Ga;	/*  */
  REAL *Gq;	/*  */
  REAL *a0;	/*  */
  REAL *gq0;	/*  */
  int *n;	/*  */
{
  int m;		/*  .esiget2d_():esiZNew.c */
  int *n;		/*  .esiget2d_():esiZNew.c */
  REAL *a0;		/*  .esiget2d_():esiZNew.c */
  REAL *gq0;		/*  .esiget2d_():esiZNew.c */
  REAL *G;		/*  .esiget2d_():esiZNew.c */
  REAL *Gq;		/*  .esiget2d_():esiZNew.c */
  REAL *Ga;		/*  .esiget2d_():esiZNew.c */
  static REAL *sr;	/*              :esiZNew.c */
  static REAL *sra;	/*              :esiZNew.c */
  static REAL *srq;	/*              :esiZNew.c */
  static REAL *sraq;	/*              :esiZNew.c */
  static REAL *sz;	/*              :esiZNew.c */
  static REAL *sza;	/*              :esiZNew.c */
  static REAL *szq;	/*              :esiZNew.c */
  static REAL *szaq;	/*              :esiZNew.c */
  static REAL *aB;	/*              :esiZNew.c */
  static REAL *aBa;	/*              :esiZNew.c */
  static REAL *aBq;	/*              :esiZNew.c */
  static REAL *aBaq;	/*              :esiZNew.c */
  static REAL crgq0;	/*              :esiZNew.c */
  static REAL cgq0;	/*              :esiZNew.c */
  static REAL rhq;	/*              :esiZNew.c */
  static int Np;	/*              :esiZNew.c */
  static REAL y;	/*              :esiZNew.c */
  static REAL *gq;	/*              :esiZNew.c */
  static REAL Y;	/*              :esiZNew.c */
  static REAL YY;	/*              :esiZNew.c */
  static REAL yy;	/*              :esiZNew.c */
  static REAL YF;	/*              :esiZNew.c */
  static REAL YD;	/*              :esiZNew.c */
  static REAL hq;	/*              :esiZNew.c */
  static REAL yF;	/*              :esiZNew.c */
  static REAL yD;	/*              :esiZNew.c */
  static REAL dY;	/*              :esiZNew.c */
  static REAL dYD;	/*              :esiZNew.c */
  static REAL dyD;	/*              :esiZNew.c */
  static int i0;	/*              :esiZNew.c */
  static REAL *sa;	/*              :esiZNew.c */
  static REAL rha;	/*              :esiZNew.c */
  static int Na;	/*              :esiZNew.c */
  static int i1;	/*              :esiZNew.c */
  static REAL XF;	/*              :esiZNew.c */
  static REAL XD;	/*              :esiZNew.c */
  static REAL ha;	/*              :esiZNew.c */
  static REAL xF;	/*              :esiZNew.c */
  static REAL xD;	/*              :esiZNew.c */
  static REAL dX;	/*              :esiZNew.c */
  static REAL dXD;	/*              :esiZNew.c */
  static REAL dxD;	/*              :esiZNew.c */
  static int j00;	/*              :esiZNew.c */
  static int Np1;	/*              :esiZNew.c */
  static int j01;	/*              :esiZNew.c */
  static int j10;	/*              :esiZNew.c */
  static int j11;	/*              :esiZNew.c */
/*
   */
#ifdef Doc_esiget2d_
\Section{Skeleton of \c{esiget2d_()}}
  \begin{verbatim}
    esiget2d_(){
    }
  \end{verbatim}
\Section{\c{esiget2d_()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{int }&\c{m}&$$&\\
\c{REAL }&\c{*G}&$$&\\
\c{REAL }&\c{*Ga}&$$&\\
\c{REAL }&\c{*Gq}&$$&\\
\c{REAL }&\c{*a0}&$$&\\
\c{REAL }&\c{*gq0}&$$&\\
\c{int }&\c{*n}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{int }&\c{m}&$$&\\
\c{static REAL }&\c{*sr}&$$&\\
\c{static REAL }&\c{*sra}&$$&\\
\c{static REAL }&\c{*srq}&$$&\\
\c{static REAL }&\c{*sraq}&$$&\\
\c{static REAL }&\c{*sz}&$$&\\
\c{static REAL }&\c{*sza}&$$&\\
\c{static REAL }&\c{*szq}&$$&\\
\c{static REAL }&\c{*szaq}&$$&\\
\c{static REAL }&\c{*aB}&$$&\\
\c{static REAL }&\c{*aBa}&$$&\\
\c{static REAL }&\c{*aBq}&$$&\\
\c{static REAL }&\c{*aBaq}&$$&\\
\c{int }&\c{*n}&$$&\\
\c{REAL }&\c{*a0}&$$&\\
\c{REAL }&\c{*gq0}&$$&\\
\c{static REAL }&\c{crgq0}&$$&\\
\c{static REAL }&\c{cgq0}&$$&\\
\c{static REAL }&\c{rhq}&$$&\\
\c{static int }&\c{Np}&$$&\\
\c{static REAL }&\c{y}&$$&\\
\c{static REAL }&\c{*gq}&$$&\\
\c{static REAL }&\c{Y}&$$&\\
\c{static REAL }&\c{YY}&$$&\\
\c{static REAL }&\c{yy}&$$&\\
\c{static REAL }&\c{YF}&$$&\\
\c{static REAL }&\c{YD}&$$&\\
\c{static REAL }&\c{hq}&$$&\\
\c{static REAL }&\c{yF}&$$&\\
\c{static REAL }&\c{yD}&$$&\\
\c{static REAL }&\c{dY}&$$&\\
\c{static REAL }&\c{dYD}&$$&\\
\c{static REAL }&\c{dyD}&$$&\\
\c{static int }&\c{i0}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{rha}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{static int }&\c{i1}&$$&\\
\c{static REAL }&\c{XF}&$$&\\
\c{static REAL }&\c{XD}&$$&\\
\c{static REAL }&\c{ha}&$$&\\
\c{static REAL }&\c{xF}&$$&\\
\c{static REAL }&\c{xD}&$$&\\
\c{static REAL }&\c{dX}&$$&\\
\c{static REAL }&\c{dXD}&$$&\\
\c{static REAL }&\c{dxD}&$$&\\
\c{static int }&\c{j00}&$$&\\
\c{static int }&\c{Np1}&$$&\\
\c{static int }&\c{j01}&$$&\\
\c{static int }&\c{j10}&$$&\\
\c{static int }&\c{j11}&$$&\\
\c{REAL }&\c{*G}&$$&\\
\c{REAL }&\c{*Gq}&$$&\\
\c{REAL }&\c{*Ga}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int esigetrzb_(R,Z,Bm,a0,q0,n)
  REAL *R;	/*  */
  REAL *Z;	/*  */
  REAL *Bm;	/*  */
  REAL *a0;	/*  */
  REAL *q0;	/*  */
  int *n;	/*  */
{
  int *n;		/*  .esigetrzb_():esiZNew.c */
  REAL *a0;		/*  .esigetrzb_():esiZNew.c */
  REAL *q0;		/*  .esigetrzb_():esiZNew.c */
  REAL *R;		/*  .esigetrzb_():esiZNew.c */
  REAL *Z;		/*  .esigetrzb_():esiZNew.c */
  REAL *Bm;		/*  .esigetrzb_():esiZNew.c */
  static int i0;	/*               :esiZNew.c */
  static REAL *sa;	/*               :esiZNew.c */
  static REAL rha;	/*               :esiZNew.c */
  static int Na;	/*               :esiZNew.c */
  static int i1;	/*               :esiZNew.c */
  static REAL XF;	/*               :esiZNew.c */
  static REAL XD;	/*               :esiZNew.c */
  static REAL ha;	/*               :esiZNew.c */
  static REAL xF;	/*               :esiZNew.c */
  static REAL xD;	/*               :esiZNew.c */
  static REAL dX;	/*               :esiZNew.c */
  static REAL dXD;	/*               :esiZNew.c */
  static REAL dxD;	/*               :esiZNew.c */
  static REAL crgq0;	/*               :esiZNew.c */
  static REAL cgq0;	/*               :esiZNew.c */
  static REAL rhq;	/*               :esiZNew.c */
  static int Np;	/*               :esiZNew.c */
  static REAL *gq;	/*               :esiZNew.c */
  static REAL YF;	/*               :esiZNew.c */
  static REAL YD;	/*               :esiZNew.c */
  static REAL hq;	/*               :esiZNew.c */
  static REAL yF;	/*               :esiZNew.c */
  static REAL yD;	/*               :esiZNew.c */
  static REAL dY;	/*               :esiZNew.c */
  static REAL dYD;	/*               :esiZNew.c */
  static REAL dyD;	/*               :esiZNew.c */
  static int j00;	/*               :esiZNew.c */
  static int Np1;	/*               :esiZNew.c */
  static int j01;	/*               :esiZNew.c */
  static int j10;	/*               :esiZNew.c */
  static int j11;	/*               :esiZNew.c */
  static REAL *sr;	/*               :esiZNew.c */
  static REAL *sra;	/*               :esiZNew.c */
  static REAL *srq;	/*               :esiZNew.c */
  static REAL *sraq;	/*               :esiZNew.c */
  static REAL *sz;	/*               :esiZNew.c */
  static REAL *sza;	/*               :esiZNew.c */
  static REAL *szq;	/*               :esiZNew.c */
  static REAL *szaq;	/*               :esiZNew.c */
  static REAL *aB;	/*               :esiZNew.c */
  static REAL *aBa;	/*               :esiZNew.c */
  static REAL *aBq;	/*               :esiZNew.c */
  static REAL *aBaq;	/*               :esiZNew.c */
/*
   */
#ifdef Doc_esigetrzb_
\Section{Skeleton of \c{esigetrzb_()}}
  \begin{verbatim}
    esigetrzb_(){
    }
  \end{verbatim}
\Section{\c{esigetrzb_()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{REAL }&\c{*R}&$$&\\
\c{REAL }&\c{*Z}&$$&\\
\c{REAL }&\c{*Bm}&$$&\\
\c{REAL }&\c{*a0}&$$&\\
\c{REAL }&\c{*q0}&$$&\\
\c{int }&\c{*n}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{int }&\c{*n}&$$&\\
\c{REAL }&\c{*a0}&$$&\\
\c{REAL }&\c{*q0}&$$&\\
\c{static int }&\c{i0}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{rha}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{static int }&\c{i1}&$$&\\
\c{static REAL }&\c{XF}&$$&\\
\c{static REAL }&\c{XD}&$$&\\
\c{static REAL }&\c{ha}&$$&\\
\c{static REAL }&\c{xF}&$$&\\
\c{static REAL }&\c{xD}&$$&\\
\c{static REAL }&\c{dX}&$$&\\
\c{static REAL }&\c{dXD}&$$&\\
\c{static REAL }&\c{dxD}&$$&\\
\c{static REAL }&\c{crgq0}&$$&\\
\c{static REAL }&\c{cgq0}&$$&\\
\c{static REAL }&\c{rhq}&$$&\\
\c{static int }&\c{Np}&$$&\\
\c{static REAL }&\c{*gq}&$$&\\
\c{static REAL }&\c{YF}&$$&\\
\c{static REAL }&\c{YD}&$$&\\
\c{static REAL }&\c{hq}&$$&\\
\c{static REAL }&\c{yF}&$$&\\
\c{static REAL }&\c{yD}&$$&\\
\c{static REAL }&\c{dY}&$$&\\
\c{static REAL }&\c{dYD}&$$&\\
\c{static REAL }&\c{dyD}&$$&\\
\c{static int }&\c{j00}&$$&\\
\c{static int }&\c{Np1}&$$&\\
\c{static int }&\c{j01}&$$&\\
\c{static int }&\c{j10}&$$&\\
\c{static int }&\c{j11}&$$&\\
\c{static REAL }&\c{*sr}&$$&\\
\c{static REAL }&\c{*sra}&$$&\\
\c{static REAL }&\c{*srq}&$$&\\
\c{static REAL }&\c{*sraq}&$$&\\
\c{REAL }&\c{*R}&$$&\\
\c{static REAL }&\c{*sz}&$$&\\
\c{static REAL }&\c{*sza}&$$&\\
\c{static REAL }&\c{*szq}&$$&\\
\c{static REAL }&\c{*szaq}&$$&\\
\c{REAL }&\c{*Z}&$$&\\
\c{static REAL }&\c{*aB}&$$&\\
\c{static REAL }&\c{*aBa}&$$&\\
\c{static REAL }&\c{*aBq}&$$&\\
\c{static REAL }&\c{*aBaq}&$$&\\
\c{REAL }&\c{*Bm}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int esiGetgH(gH0,gH0q,a0,gq0,n)
  double *gH0;	/*  */
  double *gH0q;	/*  */
  double *a0;	/*  */
  double *gq0;	/*  */
  int n;	/*  */
{
  int n;		/*  .esiGetgH():esiZNew.c */
  double *a0;		/*  .esiGetgH():esiZNew.c */
  double *gq0;		/*  .esiGetgH():esiZNew.c */
  double *gH0q;		/*  .esiGetgH():esiZNew.c */
  double *gH0;		/*  .esiGetgH():esiZNew.c */
  static REAL crgq0;	/*             :esiZNew.c */
  static REAL cgq0;	/*             :esiZNew.c */
  static REAL rhq;	/*             :esiZNew.c */
  static int Np;	/*             :esiZNew.c */
  static REAL y;	/*             :esiZNew.c */
  static REAL *gq;	/*             :esiZNew.c */
  static REAL Y;	/*             :esiZNew.c */
  static REAL YY;	/*             :esiZNew.c */
  static REAL yy;	/*             :esiZNew.c */
  static REAL YF;	/*             :esiZNew.c */
  static REAL YD;	/*             :esiZNew.c */
  static REAL hq;	/*             :esiZNew.c */
  static REAL yF;	/*             :esiZNew.c */
  static REAL yD;	/*             :esiZNew.c */
  static REAL dY;	/*             :esiZNew.c */
  static REAL dYD;	/*             :esiZNew.c */
  static REAL dyD;	/*             :esiZNew.c */
  static int i0;	/*             :esiZNew.c */
  static REAL *sa;	/*             :esiZNew.c */
  static REAL rha;	/*             :esiZNew.c */
  static int Na;	/*             :esiZNew.c */
  static int i1;	/*             :esiZNew.c */
  static REAL XF;	/*             :esiZNew.c */
  static REAL XD;	/*             :esiZNew.c */
  static REAL ha;	/*             :esiZNew.c */
  static REAL xF;	/*             :esiZNew.c */
  static REAL xD;	/*             :esiZNew.c */
  static REAL dX;	/*             :esiZNew.c */
  static REAL dXD;	/*             :esiZNew.c */
  static REAL dxD;	/*             :esiZNew.c */
  static int j00;	/*             :esiZNew.c */
  static int Np1;	/*             :esiZNew.c */
  static int j01;	/*             :esiZNew.c */
  static int j10;	/*             :esiZNew.c */
  static int j11;	/*             :esiZNew.c */
  static REAL *gHq;	/*             :esiZNew.c */
  static REAL *gHaq;	/*             :esiZNew.c */
  static REAL *gHqq;	/*             :esiZNew.c */
  static REAL *gHaqq;	/*             :esiZNew.c */
  static REAL *gH;	/*             :esiZNew.c */
  static REAL *gHa;	/*             :esiZNew.c */
  static double cr6;	/*             :esiZNew.c */
/*
   */
#ifdef Doc_esiGetgH
\Section{Skeleton of \c{esiGetgH()}}
  \begin{verbatim}
    esiGetgH(){
    }
  \end{verbatim}
\Section{\c{esiGetgH()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*gH0}&$$&\\
\c{double }&\c{*gH0q}&$$&\\
\c{double }&\c{*a0}&$$&\\
\c{double }&\c{*gq0}&$$&\\
\c{int }&\c{n}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{Undef }&\c{NULL}&$$&\\
\c{int }&\c{n}&$$&\\
\c{double }&\c{*a0}&$$&\\
\c{double }&\c{*gq0}&$$&\\
\c{static REAL }&\c{crgq0}&$$&\\
\c{static REAL }&\c{cgq0}&$$&\\
\c{static REAL }&\c{rhq}&$$&\\
\c{static int }&\c{Np}&$$&\\
\c{static REAL }&\c{y}&$$&\\
\c{static REAL }&\c{*gq}&$$&\\
\c{static REAL }&\c{Y}&$$&\\
\c{static REAL }&\c{YY}&$$&\\
\c{static REAL }&\c{yy}&$$&\\
\c{static REAL }&\c{YF}&$$&\\
\c{static REAL }&\c{YD}&$$&\\
\c{static REAL }&\c{hq}&$$&\\
\c{static REAL }&\c{yF}&$$&\\
\c{static REAL }&\c{yD}&$$&\\
\c{static REAL }&\c{dY}&$$&\\
\c{static REAL }&\c{dYD}&$$&\\
\c{static REAL }&\c{dyD}&$$&\\
\c{static int }&\c{i0}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{rha}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{static int }&\c{i1}&$$&\\
\c{static REAL }&\c{XF}&$$&\\
\c{static REAL }&\c{XD}&$$&\\
\c{static REAL }&\c{ha}&$$&\\
\c{static REAL }&\c{xF}&$$&\\
\c{static REAL }&\c{xD}&$$&\\
\c{static REAL }&\c{dX}&$$&\\
\c{static REAL }&\c{dXD}&$$&\\
\c{static REAL }&\c{dxD}&$$&\\
\c{static int }&\c{j00}&$$&\\
\c{static int }&\c{Np1}&$$&\\
\c{static int }&\c{j01}&$$&\\
\c{static int }&\c{j10}&$$&\\
\c{static int }&\c{j11}&$$&\\
\c{static REAL }&\c{*gHq}&$$&\\
\c{static REAL }&\c{*gHaq}&$$&\\
\c{static REAL }&\c{*gHqq}&$$&\\
\c{static REAL }&\c{*gHaqq}&$$&\\
\c{double }&\c{*gH0q}&$$&\\
\c{double }&\c{*gH0}&$$&\\
\c{static REAL }&\c{*gH}&$$&\\
\c{static REAL }&\c{*gHa}&$$&\\
\c{static double }&\c{cr6}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int gcmotion_(dXgr,dXa,dXgq,dXgf,Xgr,Xa,Xgq,Xgm,n)
  REAL *dXgr;	/*  */
  REAL *dXa;	/*  */
  REAL *dXgq;	/*  */
  REAL *dXgf;	/*  */
  REAL *Xgr;	/*  */
  REAL *Xa;	/*  */
  REAL *Xgq;	/*  */
  REAL *Xgm;	/*  */
  int *n;	/*  */
{
  int *n;		/*  .gcmotion_():esiZNew.c */
  REAL *Xgq;		/*  .gcmotion_():esiZNew.c */
  REAL *Xa;		/*  .gcmotion_():esiZNew.c */
  REAL *Xgr;		/*  .gcmotion_():esiZNew.c */
  REAL *Xgm;		/*  .gcmotion_():esiZNew.c */
  REAL *dXgr;		/*  .gcmotion_():esiZNew.c */
  REAL *dXgf;		/*  .gcmotion_():esiZNew.c */
  REAL *dXa;		/*  .gcmotion_():esiZNew.c */
  REAL *dXgq;		/*  .gcmotion_():esiZNew.c */
  static REAL crgq0;	/*              :esiZNew.c */
  static REAL cgq0;	/*              :esiZNew.c */
  static REAL rhq;	/*              :esiZNew.c */
  static int Np;	/*              :esiZNew.c */
  static REAL *gq;	/*              :esiZNew.c */
  static REAL hq;	/*              :esiZNew.c */
  static int i0;	/*              :esiZNew.c */
  static REAL *sa;	/*              :esiZNew.c */
  static REAL rha;	/*              :esiZNew.c */
  static int Na;	/*              :esiZNew.c */
  static int i1;	/*              :esiZNew.c */
  static REAL ha;	/*              :esiZNew.c */
  static int j00;	/*              :esiZNew.c */
  static int Np1;	/*              :esiZNew.c */
  static int j01;	/*              :esiZNew.c */
  static int j10;	/*              :esiZNew.c */
  static int j11;	/*              :esiZNew.c */
  static REAL *sr;	/*              :esiZNew.c */
  static REAL *sra;	/*              :esiZNew.c */
  static REAL *srq;	/*              :esiZNew.c */
  static REAL *sraq;	/*              :esiZNew.c */
  static REAL *sz;	/*              :esiZNew.c */
  static REAL *sza;	/*              :esiZNew.c */
  static REAL *szq;	/*              :esiZNew.c */
  static REAL *szaq;	/*              :esiZNew.c */
  static REAL *aB;	/*              :esiZNew.c */
  static REAL *aBa;	/*              :esiZNew.c */
  static REAL *aBq;	/*              :esiZNew.c */
  static REAL *aBaq;	/*              :esiZNew.c */
  static REAL *aF;	/*              :esiZNew.c */
  static REAL *daF;	/*              :esiZNew.c */
  static REAL *dgY;	/*              :esiZNew.c */
  static REAL *d2gY;	/*              :esiZNew.c */
  static REAL *aT;	/*              :esiZNew.c */
  static REAL *daT;	/*              :esiZNew.c */
  static REAL *aP;	/*              :esiZNew.c */
  static REAL *daP;	/*              :esiZNew.c */
  static int *sw;	/*              :esiZNew.c */
/*
   */
#ifdef Doc_gcmotion_
\Section{Skeleton of \c{gcmotion_()}}
  \begin{verbatim}
    gcmotion_(){
    }
  \end{verbatim}
\Section{\c{gcmotion_()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{REAL }&\c{*dXgr}&$$&\\
\c{REAL }&\c{*dXa}&$$&\\
\c{REAL }&\c{*dXgq}&$$&\\
\c{REAL }&\c{*dXgf}&$$&\\
\c{REAL }&\c{*Xgr}&$$&\\
\c{REAL }&\c{*Xa}&$$&\\
\c{REAL }&\c{*Xgq}&$$&\\
\c{REAL }&\c{*Xgm}&$$&\\
\c{int }&\c{*n}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{Undef }&\c{cos()}&$$&\\
\c{Undef }&\c{sin()}&$$&\\
\c{Undef }&\c{NULL}&$$&\\
\c{int }&\c{*n}&$$&\\
\c{REAL }&\c{*Xgq}&$$&\\
\c{static REAL }&\c{crgq0}&$$&\\
\c{static REAL }&\c{cgq0}&$$&\\
\c{static REAL }&\c{rhq}&$$&\\
\c{static int }&\c{Np}&$$&\\
\c{static REAL }&\c{*gq}&$$&\\
\c{static REAL }&\c{hq}&$$&\\
\c{REAL }&\c{*Xa}&$$&\\
\c{static int }&\c{i0}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{rha}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{static int }&\c{i1}&$$&\\
\c{static REAL }&\c{ha}&$$&\\
\c{static int }&\c{j00}&$$&\\
\c{static int }&\c{Np1}&$$&\\
\c{static int }&\c{j01}&$$&\\
\c{static int }&\c{j10}&$$&\\
\c{static int }&\c{j11}&$$&\\
\c{static REAL }&\c{*sr}&$$&\\
\c{static REAL }&\c{*sra}&$$&\\
\c{static REAL }&\c{*srq}&$$&\\
\c{static REAL }&\c{*sraq}&$$&\\
\c{static REAL }&\c{*sz}&$$&\\
\c{static REAL }&\c{*sza}&$$&\\
\c{static REAL }&\c{*szq}&$$&\\
\c{static REAL }&\c{*szaq}&$$&\\
\c{static REAL }&\c{*aB}&$$&\\
\c{static REAL }&\c{*aBa}&$$&\\
\c{static REAL }&\c{*aBq}&$$&\\
\c{static REAL }&\c{*aBaq}&$$&\\
\c{static REAL }&\c{*aF}&$$&\\
\c{static REAL }&\c{*daF}&$$&\\
\c{static REAL }&\c{*dgY}&$$&\\
\c{static REAL }&\c{*d2gY}&$$&\\
\c{static REAL }&\c{*aT}&$$&\\
\c{static REAL }&\c{*daT}&$$&\\
\c{static REAL }&\c{*aP}&$$&\\
\c{static REAL }&\c{*daP}&$$&\\
\c{REAL }&\c{*Xgr}&$$&\\
\c{REAL }&\c{*Xgm}&$$&\\
\c{REAL }&\c{*dXgr}&$$&\\
\c{REAL }&\c{*dXgf}&$$&\\
\c{REAL }&\c{*dXa}&$$&\\
\c{REAL }&\c{*dXgq}&$$&\\
\c{static int }&\c{*sw}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int MFieldLines(dXgq,dXgf,Xa,Xgq,n)
  REAL *dXgq;	/*  */
  REAL *dXgf;	/*  */
  REAL *Xa;	/*  */
  REAL *Xgq;	/*  */
  int n;	/*  */
{
  int n;		/*  .MFieldLines():esiZNew.c */
  REAL *Xa;		/*  .MFieldLines():esiZNew.c */
  REAL *Xgq;		/*  .MFieldLines():esiZNew.c */
  REAL *dXgq;		/*  .MFieldLines():esiZNew.c */
  REAL *dXgf;		/*  .MFieldLines():esiZNew.c */
  static REAL crgq0;	/*                :esiZNew.c */
  static REAL cgq0;	/*                :esiZNew.c */
  static REAL rhq;	/*                :esiZNew.c */
  static int Np;	/*                :esiZNew.c */
  static REAL *gq;	/*                :esiZNew.c */
  static REAL YF;	/*                :esiZNew.c */
  static REAL YD;	/*                :esiZNew.c */
  static REAL hq;	/*                :esiZNew.c */
  static REAL yF;	/*                :esiZNew.c */
  static REAL yD;	/*                :esiZNew.c */
  static REAL dY;	/*                :esiZNew.c */
  static REAL dYD;	/*                :esiZNew.c */
  static REAL dyD;	/*                :esiZNew.c */
  static int i0;	/*                :esiZNew.c */
  static REAL *sa;	/*                :esiZNew.c */
  static REAL rha;	/*                :esiZNew.c */
  static int Na;	/*                :esiZNew.c */
  static int i1;	/*                :esiZNew.c */
  static REAL XF;	/*                :esiZNew.c */
  static REAL XD;	/*                :esiZNew.c */
  static REAL ha;	/*                :esiZNew.c */
  static REAL xF;	/*                :esiZNew.c */
  static REAL xD;	/*                :esiZNew.c */
  static REAL dX;	/*                :esiZNew.c */
  static REAL dXD;	/*                :esiZNew.c */
  static REAL dxD;	/*                :esiZNew.c */
  static int j00;	/*                :esiZNew.c */
  static int Np1;	/*                :esiZNew.c */
  static int j01;	/*                :esiZNew.c */
  static int j10;	/*                :esiZNew.c */
  static int j11;	/*                :esiZNew.c */
  static REAL *sr;	/*                :esiZNew.c */
  static REAL *sra;	/*                :esiZNew.c */
  static REAL *srq;	/*                :esiZNew.c */
  static REAL *sraq;	/*                :esiZNew.c */
  static REAL *sz;	/*                :esiZNew.c */
  static REAL *sza;	/*                :esiZNew.c */
  static REAL *szq;	/*                :esiZNew.c */
  static REAL *szaq;	/*                :esiZNew.c */
  static REAL *aB;	/*                :esiZNew.c */
  static REAL *aBa;	/*                :esiZNew.c */
  static REAL *aBq;	/*                :esiZNew.c */
  static REAL *aBaq;	/*                :esiZNew.c */
  static REAL *dgY;	/*                :esiZNew.c */
  static REAL *d2gY;	/*                :esiZNew.c */
/*
   */
#ifdef Doc_MFieldLines
\Section{Skeleton of \c{MFieldLines()}}
  \begin{verbatim}
    MFieldLines(){
    }
  \end{verbatim}
\Section{\c{MFieldLines()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{REAL }&\c{*dXgq}&$$&\\
\c{REAL }&\c{*dXgf}&$$&\\
\c{REAL }&\c{*Xa}&$$&\\
\c{REAL }&\c{*Xgq}&$$&\\
\c{int }&\c{n}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{int }&\c{n}&$$&\\
\c{REAL }&\c{*Xa}&$$&\\
\c{REAL }&\c{*Xgq}&$$&\\
\c{static REAL }&\c{crgq0}&$$&\\
\c{static REAL }&\c{cgq0}&$$&\\
\c{static REAL }&\c{rhq}&$$&\\
\c{static int }&\c{Np}&$$&\\
\c{static REAL }&\c{*gq}&$$&\\
\c{static REAL }&\c{YF}&$$&\\
\c{static REAL }&\c{YD}&$$&\\
\c{static REAL }&\c{hq}&$$&\\
\c{static REAL }&\c{yF}&$$&\\
\c{static REAL }&\c{yD}&$$&\\
\c{static REAL }&\c{dY}&$$&\\
\c{static REAL }&\c{dYD}&$$&\\
\c{static REAL }&\c{dyD}&$$&\\
\c{static int }&\c{i0}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{rha}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{static int }&\c{i1}&$$&\\
\c{static REAL }&\c{XF}&$$&\\
\c{static REAL }&\c{XD}&$$&\\
\c{static REAL }&\c{ha}&$$&\\
\c{static REAL }&\c{xF}&$$&\\
\c{static REAL }&\c{xD}&$$&\\
\c{static REAL }&\c{dX}&$$&\\
\c{static REAL }&\c{dXD}&$$&\\
\c{static REAL }&\c{dxD}&$$&\\
\c{static int }&\c{j00}&$$&\\
\c{static int }&\c{Np1}&$$&\\
\c{static int }&\c{j01}&$$&\\
\c{static int }&\c{j10}&$$&\\
\c{static int }&\c{j11}&$$&\\
\c{static REAL }&\c{*sr}&$$&\\
\c{static REAL }&\c{*sra}&$$&\\
\c{static REAL }&\c{*srq}&$$&\\
\c{static REAL }&\c{*sraq}&$$&\\
\c{static REAL }&\c{*sz}&$$&\\
\c{static REAL }&\c{*sza}&$$&\\
\c{static REAL }&\c{*szq}&$$&\\
\c{static REAL }&\c{*szaq}&$$&\\
\c{static REAL }&\c{*aB}&$$&\\
\c{static REAL }&\c{*aBa}&$$&\\
\c{static REAL }&\c{*aBq}&$$&\\
\c{static REAL }&\c{*aBaq}&$$&\\
\c{static REAL }&\c{*dgY}&$$&\\
\c{static REAL }&\c{*d2gY}&$$&\\
\c{REAL }&\c{*dXgq}&$$&\\
\c{REAL }&\c{*dXgf}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

void esigetpressure_(p,dp,a,n)
  double *p;	/*  */
  double *dp;	/*  */
  double *a;	/*  */
  int *n;	/*  */
{
  int *n;		/*  .esigetpressure_():esiZNew.c */
  double *a;		/*  .esigetpressure_():esiZNew.c */
  double *p;		/*  .esigetpressure_():esiZNew.c */
  double *dp;		/*  .esigetpressure_():esiZNew.c */
  static REAL *aP;	/*                    :esiZNew.c */
  static REAL *dgY;	/*                    :esiZNew.c */
  static int Na1;	/*                    :esiZNew.c */
  static REAL *sa;	/*                    :esiZNew.c */
  static REAL *daP;	/*                    :esiZNew.c */
  static REAL *d2gY;	/*                    :esiZNew.c */
  static REAL ha;	/*                    :esiZNew.c */
  static int Na;	/*                    :esiZNew.c */
  static REAL rha;	/*                    :esiZNew.c */
/*
   */
#ifdef Doc_esigetpressure_
\Section{Skeleton of \c{esigetpressure_()}}
  \begin{verbatim}
    esigetpressure_(){
    }
  \end{verbatim}
\Section{\c{esigetpressure_()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*p}&$$&\\
\c{double }&\c{*dp}&$$&\\
\c{double }&\c{*a}&$$&\\
\c{int }&\c{*n}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{static REAL }&\c{*aP}&$$&\\
\c{static REAL }&\c{*dgY}&$$&\\
\c{static int }&\c{Na1}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{*daP}&$$&\\
\c{static REAL }&\c{*d2gY}&$$&\\
\c{static REAL }&\c{ha}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{int }&\c{*n}&$$&\\
\c{double }&\c{*a}&$$&\\
\c{static REAL }&\c{rha}&$$&\\
\c{double }&\c{*p}&$$&\\
\c{double }&\c{*dp}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int ESI2Geom(R,Ra,Rq,Z,Za,Zq,a0,gq0,n)
  double *R;	/*  */
  double *Ra;	/*  */
  double *Rq;	/*  */
  double *Z;	/*  */
  double *Za;	/*  */
  double *Zq;	/*  */
  double *a0;	/*  */
  double *gq0;	/*  */
  int n;	/*  */
{
  int n;		/*  .ESI2Geom():esiZNew.c */
  double *a0;		/*  .ESI2Geom():esiZNew.c */
  double *gq0;		/*  .ESI2Geom():esiZNew.c */
  double *R;		/*  .ESI2Geom():esiZNew.c */
  double *Rq;		/*  .ESI2Geom():esiZNew.c */
  double *Ra;		/*  .ESI2Geom():esiZNew.c */
  double *Z;		/*  .ESI2Geom():esiZNew.c */
  double *Zq;		/*  .ESI2Geom():esiZNew.c */
  double *Za;		/*  .ESI2Geom():esiZNew.c */
  static REAL crgq0;	/*               :esiZNew.c */
  static REAL cgq0;	/*               :esiZNew.c */
  static REAL rhq;	/*               :esiZNew.c */
  static int Np;	/*               :esiZNew.c */
  static REAL y;	/*               :esiZNew.c */
  static REAL *gq;	/*               :esiZNew.c */
  static REAL Y;	/*               :esiZNew.c */
  static REAL YY;	/*               :esiZNew.c */
  static REAL yy;	/*               :esiZNew.c */
  static REAL YF;	/*               :esiZNew.c */
  static REAL YD;	/*               :esiZNew.c */
  static REAL hq;	/*               :esiZNew.c */
  static REAL yF;	/*               :esiZNew.c */
  static REAL yD;	/*               :esiZNew.c */
  static REAL dY;	/*               :esiZNew.c */
  static REAL dYD;	/*               :esiZNew.c */
  static REAL dyD;	/*               :esiZNew.c */
  static int i0;	/*               :esiZNew.c */
  static REAL *sa;	/*               :esiZNew.c */
  static REAL rha;	/*               :esiZNew.c */
  static int Na;	/*               :esiZNew.c */
  static int i1;	/*               :esiZNew.c */
  static REAL XF;	/*               :esiZNew.c */
  static REAL XD;	/*               :esiZNew.c */
  static REAL ha;	/*               :esiZNew.c */
  static REAL xF;	/*               :esiZNew.c */
  static REAL xD;	/*               :esiZNew.c */
  static REAL dX;	/*               :esiZNew.c */
  static REAL dXD;	/*               :esiZNew.c */
  static REAL dxD;	/*               :esiZNew.c */
  static int j00;	/*               :esiZNew.c */
  static int Np1;	/*               :esiZNew.c */
  static int j01;	/*               :esiZNew.c */
  static int j10;	/*               :esiZNew.c */
  static int j11;	/*               :esiZNew.c */
  static REAL *sr;	/*               :esiZNew.c */
  static REAL *sra;	/*               :esiZNew.c */
  static REAL *srq;	/*               :esiZNew.c */
  static REAL *sraq;	/*               :esiZNew.c */
  static REAL *sz;	/*               :esiZNew.c */
  static REAL *sza;	/*               :esiZNew.c */
  static REAL *szq;	/*               :esiZNew.c */
  static REAL *szaq;	/*               :esiZNew.c */
/*
   */
#ifdef Doc_ESI2Geom
\Section{Skeleton of \c{ESI2Geom()}}
  \begin{verbatim}
    ESI2Geom(){
    }
  \end{verbatim}
\Section{\c{ESI2Geom()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*R}&$$&\\
\c{double }&\c{*Ra}&$$&\\
\c{double }&\c{*Rq}&$$&\\
\c{double }&\c{*Z}&$$&\\
\c{double }&\c{*Za}&$$&\\
\c{double }&\c{*Zq}&$$&\\
\c{double }&\c{*a0}&$$&\\
\c{double }&\c{*gq0}&$$&\\
\c{int }&\c{n}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{int }&\c{n}&$$&\\
\c{double }&\c{*a0}&$$&\\
\c{double }&\c{*gq0}&$$&\\
\c{static REAL }&\c{crgq0}&$$&\\
\c{static REAL }&\c{cgq0}&$$&\\
\c{static REAL }&\c{rhq}&$$&\\
\c{static int }&\c{Np}&$$&\\
\c{static REAL }&\c{y}&$$&\\
\c{static REAL }&\c{*gq}&$$&\\
\c{static REAL }&\c{Y}&$$&\\
\c{static REAL }&\c{YY}&$$&\\
\c{static REAL }&\c{yy}&$$&\\
\c{static REAL }&\c{YF}&$$&\\
\c{static REAL }&\c{YD}&$$&\\
\c{static REAL }&\c{hq}&$$&\\
\c{static REAL }&\c{yF}&$$&\\
\c{static REAL }&\c{yD}&$$&\\
\c{static REAL }&\c{dY}&$$&\\
\c{static REAL }&\c{dYD}&$$&\\
\c{static REAL }&\c{dyD}&$$&\\
\c{static int }&\c{i0}&$$&\\
\c{static REAL }&\c{*sa}&$$&\\
\c{static REAL }&\c{rha}&$$&\\
\c{static int }&\c{Na}&$$&\\
\c{static int }&\c{i1}&$$&\\
\c{static REAL }&\c{XF}&$$&\\
\c{static REAL }&\c{XD}&$$&\\
\c{static REAL }&\c{ha}&$$&\\
\c{static REAL }&\c{xF}&$$&\\
\c{static REAL }&\c{xD}&$$&\\
\c{static REAL }&\c{dX}&$$&\\
\c{static REAL }&\c{dXD}&$$&\\
\c{static REAL }&\c{dxD}&$$&\\
\c{static int }&\c{j00}&$$&\\
\c{static int }&\c{Np1}&$$&\\
\c{static int }&\c{j01}&$$&\\
\c{static int }&\c{j10}&$$&\\
\c{static int }&\c{j11}&$$&\\
\c{static REAL }&\c{*sr}&$$&\\
\c{static REAL }&\c{*sra}&$$&\\
\c{static REAL }&\c{*srq}&$$&\\
\c{static REAL }&\c{*sraq}&$$&\\
\c{double }&\c{*R}&$$&\\
\c{double }&\c{*Rq}&$$&\\
\c{double }&\c{*Ra}&$$&\\
\c{static REAL }&\c{*sz}&$$&\\
\c{static REAL }&\c{*sza}&$$&\\
\c{static REAL }&\c{*szq}&$$&\\
\c{static REAL }&\c{*szaq}&$$&\\
\c{double }&\c{*Z}&$$&\\
\c{double }&\c{*Zq}&$$&\\
\c{double }&\c{*Za}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}

int ESICopy(esr,esz,eaB,egH,na1,np1)
  double *esr;	/*  */
  double *esz;	/*  */
  double *eaB;	/*  */
  double *egH;	/*  */
  int na1;	/*  */
  int np1;	/*  */
{
  int np1;		/*  .ESICopy():esiZNew.c */
  int na1;		/*  .ESICopy():esiZNew.c */
  double *esr;		/*  .ESICopy():esiZNew.c */
  double *esz;		/*  .ESICopy():esiZNew.c */
  double *eaB;		/*  .ESICopy():esiZNew.c */
  double *egH;		/*  .ESICopy():esiZNew.c */
  static REAL *sr;	/*            :esiZNew.c */
  static REAL *sra;	/*            :esiZNew.c */
  static REAL *srq;	/*            :esiZNew.c */
  static REAL *sraq;	/*            :esiZNew.c */
  static REAL *sz;	/*            :esiZNew.c */
  static REAL *sza;	/*            :esiZNew.c */
  static REAL *szq;	/*            :esiZNew.c */
  static REAL *szaq;	/*            :esiZNew.c */
  static REAL *aB;	/*            :esiZNew.c */
  static REAL *aBa;	/*            :esiZNew.c */
  static REAL *aBq;	/*            :esiZNew.c */
  static REAL *aBaq;	/*            :esiZNew.c */
  static REAL *gH;	/*            :esiZNew.c */
  static REAL *gHa;	/*            :esiZNew.c */
  static REAL *gHq;	/*            :esiZNew.c */
  static REAL *gHaq;	/*            :esiZNew.c */
  static REAL *gHqq;	/*            :esiZNew.c */
  static REAL *gHaqq;	/*            :esiZNew.c */
/*
   */
#ifdef Doc_ESICopy
\Section{Skeleton of \c{ESICopy()}}
  \begin{verbatim}
    ESICopy(){
    }
  \end{verbatim}
\Section{\c{ESICopy()}}
\Subsection{Formal parameters}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{double }&\c{*esr}&$$&\\
\c{double }&\c{*esz}&$$&\\
\c{double }&\c{*eaB}&$$&\\
\c{double }&\c{*egH}&$$&\\
\c{int }&\c{na1}&$$&\\
\c{int }&\c{np1}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Return values}
{\obeylines\parindent0in
\c{0} - upon success;
\c{1} - if failed;
}
\Subsection{Description}
\Subsection{Algorithm}
\Subsection{Input}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{int }&\c{np1}&$$&\\
\c{int }&\c{na1}&$$&\\
\c{double }&\c{*esr}&$$&\\
\c{double }&\c{*esz}&$$&\\
\c{double }&\c{*eaB}&$$&\\
\c{double }&\c{*egH}&$$&\\
\c{static REAL }&\c{*sr}&$$&\\
\c{static REAL }&\c{*sra}&$$&\\
\c{static REAL }&\c{*srq}&$$&\\
\c{static REAL }&\c{*sraq}&$$&\\
\c{static REAL }&\c{*sz}&$$&\\
\c{static REAL }&\c{*sza}&$$&\\
\c{static REAL }&\c{*szq}&$$&\\
\c{static REAL }&\c{*szaq}&$$&\\
\c{static REAL }&\c{*aB}&$$&\\
\c{static REAL }&\c{*aBa}&$$&\\
\c{static REAL }&\c{*aBq}&$$&\\
\c{static REAL }&\c{*aBaq}&$$&\\
\c{static REAL }&\c{*gH}&$$&\\
\c{static REAL }&\c{*gHa}&$$&\\
\c{static REAL }&\c{*gHq}&$$&\\
\c{static REAL }&\c{*gHaq}&$$&\\
\c{static REAL }&\c{*gHqq}&$$&\\
\c{static REAL }&\c{*gHaqq}&$$&\\
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{Output}
\vskip1ex\noindent\begin{tabular}{|lllp{4in}|}\hline
\c{Type}&\c{ID}&\bf\em Math&\bf\em Comment\\\hline
\c{}&\c{}&$$&\\
\hline\end{tabular}\vskip1ex
\Subsection{ToDo}
\begin{enumerate}
\item
\end{enumerate}
\Subsection{References}
#endif
}
#endif

#undef Doc_Unused_Doc_Information /* ------------{ */
#ifdef Doc_Unused_Doc_Information
#ifndef mzl_System_esiZNew
Ignor double;
/* undefined */
#endif
#ifndef mzl_System_esiZNew
Systm NULL;
/* undefined */
#endif
#ifndef mzl_System_esiZNew
Ignor REAL;
/* undefined */
#endif
#ifndef mzl_System_esiZNew
Ignor char;
/* undefined */
#endif
#ifndef mzl_System_esiZNew
Ignor int;
/* undefined */
#endif
#endif  /* }== Remove this section ======== */

#undef Doc_Cb_Structure
#ifdef Blb_esiZNew.c
Blb_esiZNew.c{
}
#endif

#undef Doc_Table_Of_Content
#ifndef Blb_esiZNew.c
				PS p= -1 (  0);
int PHermit();			PS p=  1 (  1);
int EPPAHermit();		PS p=  2 (  1);
int ZeroOfHermit();		PS p=  3 (  2);
int InitLocalConstants();	PS p=  4 (  2);
int ESISetMem();		PS p=  4 (  2);
int esifree_();			PS p=  5 (  3);
int ESIMemAlloc();		PS p=  5 (  3);
int IsStrFound();		PS p=  6 (  3);
int ESIReadBD();		PS p=  6 (  3);
int ESIReadAD();		PS p=  7 (  4);
int ESIInit();			PS p=  9 (  5);
int esiread_();			PS p= 11 (  6);
void esilink2c_();		PS p= 12 (  6);
int ESIGetSize();		PS p= 12 (  6);
int EsiGetLink2c();		PS p= 13 (  7);
int esi2gFgY();			PS p= 13 (  7);
int esi2bsp();			PS p= 13 (  7);
int esi2EqProfiles();		PS p= 14 (  7);
int esiget2dfunctions_();	PS p= 16 (  8);
void esigetdensity_();		PS p= 19 ( 10);
void esi2baIpl();		PS p= 19 ( 10);
void esiplcurrent_();		PS p= 19 ( 10);
int esiGetgm();			PS p= 19 ( 10);
int esiget2d_();		PS p= 20 ( 10);
int esigetrzb_();		PS p= 22 ( 11);
int esiGetgH();			PS p= 23 ( 12);
int gcmotion_();		PS p= 25 ( 13);
int MFieldLines();		PS p= 28 ( 14);
void esigetpressure_();		PS p= 30 ( 15);
int ESI2Geom();		PS p= 31 ( 16);
int ESICopy();			PS p= 33 ( 17);
#endif

#ifdef H
\Large
\bqbl
\waA
=
\gl\na a
+\bgF\na\gq
+\bgY\na\gz
,\quad
L
=
\left(\waA+\gr_\parallel\waB\right)\cdot\wsv-H
.\eqbl
The term you are talking for
\bqbl
\gl\dsa
=
\R{d}{dt}\left(\int^a_0\gl da\right)
-\dgq\int^a_0\gl'_\gq da
-\dgz\int^a_0\gl'_\gz da
\eqbl
only changes the expressions for $P_\gq,P_\gz$. Elimination of $B_a$ is
sufficient for transition to a Hamiltonian form.
\vskip 3em
\Post{\helvBV You are scaring an old man with already }
\Post{\helvBV deteriorated memory !}

#endif
