#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <time.h>
#include <sys/time.h>

extern time_t time(time_t *t);
extern struct tm *localtime(const time_t *timep);
static struct tm *ltm;
static time_t stTime;
static int ID=0;
static char GrName[17][32]={
  "<ESI dimensions>",
  "<ESI [gq]>",
  "<ESI [sa]>",
  "<ESI [baF bsp gF gY]>",
  "<ESI T Ta P Pa>",
  "<ESI bgFa bgFaa bgYa bgYaa>",
  "<ESI r ra rq raq>",
  "<ESI z za zq zaq>",
  "<ESI B Ba Bq Baq>",
  "<ESI ghq ghaq ghqq ghaqq>",		/*Equilibrium \gh'_\gq*/
  "<ESI bgQq bgQaq bgQqq bgQaqq>",	/*Hamada \bgq'_\gq*/ 
  "<ESI [gz]>",    		/* only for 3D */
  "<ESI rgz ragz rqgz>",    	/* only for 3D */
  "<ESI zgz zagz zqgz>",    	/* only for 3D */
  "<ESI Bgz Bagz Bqgz>",    	/* only for 3D */
  "<ESI ghgz ghagz ghqgz>", 	/* only for 3D */
  ""
};

static char GrFormat[16][0x20]={
  "(%d x %d %d %e %e %s)",
  "%e",
  "%e",
  "(%e %e %e %e)",
  "(%e %e %e %e)",
  "(%e %e %e %e)",
  "(%e %e %e %e)",
  "(%e %e %e %e)",
  "(%e %e %e %e)",
  "(%e %e %e %e)",
  "(%e %e %e %e)",
  "(%e %e %e %e)",
  "(%e %e %e %e)",
  "(%e %e %e %e)",
  "(%e %e %e %e)",
  "(%e %e %e %e)"
};

static char GrComment[17][4][0x10]={
  "Nq1","Na1","ID","Rext",
  "","","","",
  "","","","",
  "baF","p [MPa]","gF [Vsec]","gY [Vsec]",
  "T","T'_a","P","P'_a",
  "bgF'/a","(bgF'/a)'","bgY'/a","(bgY'/a)'",
  "r","r'_a","r'_gq","r''_{a,gq}",
  "z","z'_a","z'_gq","z''_{a,gq}",
  "|B|","|B|'_a","|B|'_gq","|B|''_{a,gq}",
  "gH=gh'_gq","gH'_a","gH'_gq","gH''_{a,gq}",
  "gQ=bgq'_gq","gQ'_a","gQ'_gq","gQ''_{a,gq}",
  "","","","",
  "r'_gz","r''_{a,gz}","r''_{gq,gz}","r'''_{a,gq,gz}",
  "z'_gz","z''_{a,gz}","z''_{gq,gz}","z'''_{a,gq,gz}",
  "|B|'_gz","|B|''_{a,gz}","|B|''_{gq,gz}","|B|'''_{a,gq,gz}",
  "gH'_gz","gH''_{a,gz}","gH''_{gq,gz}","gH'''_{a,gq,gz}",
  "","","",""
};

#ifndef mzl_MLIB
extern double cr2,cr3,cr4,cr6,cr12,cgp,c2gp,cgp_4,cr2gp,cgm0,crgm0;
extern unsigned int ESmemlev;
#endif

extern int ESNa,ESNa1,ESNp,ESNp1,ESnAP,ESMp1,ES2Mp1,ESKp1,ESFp,ESFp1;
extern double *ESsa,*ESpa,*ESaR0,*ESaZ0;
extern double *ESsb,*ESsb1a,*ESsb2a;
extern double *ESgt,*EScs1,*ESsn1;
extern double *ESjp,*ESjp1a,*ESsp,*ESsp1a,*ESsp2a,*ESsq,*ESsq1a,*ESsq2a
,*ESPs,*ESPs1a,*ESPs2a;
extern double *ESjs,*ESjb;
extern double *ESgY,*ESgY2a;
extern double *ESFF,*ESFF1a,*ESFF2a,*ESaF,*ESaF1a,*ESaF2a;
extern double *ESgF,*ESdgY,*ESdgY2a,*ESaJ,*ESaJ2a;
extern double *ESPs,*ESPs1a,*ESPs2a,*ESaT,*ESaT1a,*ESaT2a;
extern double *rcT,*rcT1a,*rcT2a,*rsT,*rsT1a,*rsT2a;
extern double *ESLc,*ESLc1,*ESLc2,*ESLs,*ESLs1
,*ESVc,*ESVc1,*ESVc2,*ESVs,*ESVs1;
extern double *ESg22c,*ESg22c1,*ESg22s,*ESg22s1;
extern double *ESdgY,*ESdgY1a;
extern double ESBt,ESRBt,ESRext;

extern double *ESsr,*ESsra,*ESsrt,*ESsrat;
extern double *ESsz,*ESsza,*ESszt,*ESszat;
extern double *ESaB,*ESaBa,*ESaBt,*ESaBat;
extern double *ESgH,*ESgHa,*ESgHt,*ESgHat;
extern double *ESgQ,*ESgQa,*ESgQt,*ESgQat;
extern double ESdL[],ESdLq[];

extern int BFFl,BFNa,BFNa1,BFNp,BFNp1;
extern double *BFsr,*BFsra,*BFsrq,*BFsraq;
extern double *BFsz,*BFsza,*BFszq,*BFszaq;
extern double *BFaB,*BFaBa,*BFaBq,*BFaBaq;
extern double *BFgH,*BFgHa,*BFgHq,*BFgHaq,*BFgh,*BFgha;
static double *BFBa,*BFBaa,*BFBat,*BFBaat;

static int Spi0,Spi1,Spj0,Spj1;
static double ha,rha,hq,rhq;
static double gmP;
static double grn0,B0,Vn,Vb,pVb,Vtot;
static double Pi[0x800],Pe[0x800];
static double cmi=1.6726e-24; /* [g] - proton mass */
static double cc=2.9979e+10; /* [cm/sec] - speed of light */
static double ce=4.8032e-10; /* [CGS] - proton electric charge */
static double cEeV=1.6022e-12; /* energy of 1 eV */
static double cEkeV=1.6022e-9; /* energy of 1 keV */
double cgWc=0.9579e+8; /* [1/sec] - proton cyclotron frequency 
				 in 1 T field */
static double cV1keV;		/* velocity for the energy 1 keV */
static double cgr1keV;		/* ion larmor radius for 1 keV in 1 T */
typedef struct{
  double Pgf;	/* P_\gf canonical moment */
  double p;	/* pitch */
  double gm;	/* gm start */
  double E;	/* E energy */
  double gY;	/* \bgY poloidal flux */
  double F;	/* \baF energy */
  double t;
  double f;
  double gw;
  double a;	/* a start */	
  double q;	/* gq start */	
  double r;	/* r start */	
  double z;	/* z start */	
}TRAPPED;
TRAPPED BounceData,*lBD;
static int neq=4;
static double hcur,xout,reltol=1.0e-6,abstol=1e-6;
static double *rwork=NULL,*yh,*vatol;

static int itol=2,itask,istate=1,iopt=0,mf=10,lrw,liw=20;
static int *iwork=NULL;
static double yGC[4],dyGC[4];

static int NWRK=0,nWRK;
static double *RKgr0,*RKgr1,*RKgr2,*RKgr3,*RKgr4;
static double *RKa0,*RKa1,*RKa2,*RKa3,*RKa4;
static double *RKgq0,*RKgq1,*RKgq2,*RKgq3,*RKgq4;
static double *RKgf0,*RKgf1,*RKgf2,*RKgf3,*RKgf4;
static double *rPrt0,*rPrt1,*zPrt0,*zPrt1;

static double *Xgr,*Xa,*Xgq,*Xgf,*Xgm,*dXgr,*dXa,*dXgq,*dXgf;

int GCInitRKSolv(int n)
{
  nWRK	=(20+4+9)*n;
  if(NWRK != nWRK){
    if(NWRK) free(RKgr0);
    RKgr0	=(double*)malloc(nWRK*sizeof(double));
    if(RKgr0 == NULL){
      printf("No memory for RKgr0 in GCInitRKSolv(%d)\n",n);
      return(1);
    }
    RKgr1	=RKgr0	+n;
    RKgr2	=RKgr1	+n;
    RKgr3	=RKgr2	+n;
    RKgr4	=RKgr3	+n;

    RKa0	=RKgr4	+n;
    RKa1	=RKa0	+n;
    RKa2	=RKa1	+n;
    RKa3	=RKa2	+n;
    RKa4	=RKa3	+n;

    RKgq0	=RKa4	+n;
    RKgq1	=RKgq0	+n;
    RKgq2	=RKgq1	+n;
    RKgq3	=RKgq2	+n;
    RKgq4	=RKgq3	+n;

    RKgf0	=RKgq4	+n;
    RKgf1	=RKgf0	+n;
    RKgf2	=RKgf1	+n;
    RKgf3	=RKgf2	+n;
    RKgf4	=RKgf3	+n;

    rPrt0	=RKgf4	+n;  
    rPrt1	=rPrt0	+n;  
    zPrt0	=rPrt1	+n;  
    zPrt1	=zPrt0	+n;  
    Xgr	=zPrt1	+n;
    Xa	=Xgr	+n;
    Xgq	=Xa	+n;
    Xgf	=Xgq	+n;
    Xgm	=Xgf	+n;
    dXgr=Xgm	+n;
    dXa	=dXgr	+n;
    dXgq=dXa	+n;
    dXgf=dXgq	+n;

    NWRK	=nWRK;
  }
  return(0);
}

int GCDeInitRKSolv()
{
  if(NWRK){
    free(RKgr0);
    NWRK	=0;
  }
  return(0);
}

static int NPart=0,nPart=10;
static double XF[1024],XFa[1024],XgFa[1024],XgFaa[1024]
,XgYa[1024],XgYaa[1024],XT[1024],XTa[1024],XP[1024],XPa[1024]
,Xr[1024],Xra[1024],Xrq[1024],Xz[1024],Xza[1024],Xzq[1024]
,XB[1024],XBa[1024],XBq[1024],Xgh[1024],Xgha[1024],Xghq[1024];

int ESIget2DFunc(double *r,double *ra,double *rq
		 ,double *z,double *za,double *zq
		 ,double *B,double *Ba,double *Bq
		 ,double *gh,double *gha,double *ghq
		 ,double a,double gq)
{
  int i;
  i	=1;
#ifdef H
  esiget2dfunctions_(&a,&gq,&i);
#endif
  *r	=Xr[0];
  *ra	=Xra[0];
  *rq	=Xrq[0];
  *z	=Xz[0];
  *za	=Xza[0];
  *zq	=Xzq[0];
  *B	=XB[0];
  *Ba	=XBa[0];
  *Bq	=XBq[0];
  *gh	=Xgh[0];
  *gha	=Xgha[0];
  *ghq	=Xghq[0];
  return(0);
}    

int GCRKStep(double *X,double h)
{
  double x,x0;
  int i,j;

  x0	=*X;
  for(i=0; i < nPart; i++){
    RKgr1[i]	=h*dXgr[i];
    RKgr0[i]	=Xgr[i]+cr2*RKgr1[i];
    RKa1[i]	=h*dXa[i];
    RKa0[i]	=Xa[i]+cr2*RKa1[i];
    RKgq1[i]	=h*dXgq[i];
    RKgq0[i]	=Xgq[i]+cr2*RKgq1[i];
    RKgf1[i]	=h*dXgf[i];
    RKgf0[i]	=Xgf[i]+cr2*RKgf1[i];
  }
  x	=(*X)+cr2*h;
  gcmotion_(RKgr2,RKa2,RKgq2,RKgf2,RKgr0,RKa0,RKgq0,Xgm,&nPart);
  for(i=0; i < nPart; i++){
    RKgr2[i]	*=h;
    RKa2[i]	*=h;
    RKgq2[i]	*=h;
    RKgf2[i]	*=h;
    RKgr0[i]	=Xgr[i]+cr2*RKgr2[i];
    RKa0[i]	=Xa[i]+cr2*RKa2[i];
    RKgq0[i]	=Xgq[i]+cr2*RKgq2[i];
    RKgf0[i]	=Xgf[i]+cr2*RKgf2[i];
  }
  gcmotion_(RKgr3,RKa3,RKgq3,RKgf3,RKgr0,RKa0,RKgq0,Xgm,&nPart);
  for(i=0; i < nPart; i++){
    RKgr3[i]	*=h;
    RKa3[i]	*=h;
    RKgq3[i]	*=h;
    RKgf3[i]	*=h;
    RKgr0[i]	=Xgr[i]+RKgr3[i];
    RKa0[i]	=Xa[i]+RKa3[i];
    RKgq0[i]	=Xgq[i]+RKgq3[i];
    RKgf0[i]	=Xgf[i]+RKgf3[i];
  }
  (*X)		+=h;
  gcmotion_(RKgr4,RKa4,RKgq4,RKgf4,RKgr0,RKa0,RKgq0,Xgm,&nPart);
  for(i=0; i < nPart; i++){
    RKgr4[i]	*=h;
    RKa4[i]	*=h;
    RKgq4[i]	*=h;
    RKgf4[i]	*=h;
    Xgr[i]	+=cr6*(RKgr1[i]+RKgr4[i])+cr3*(RKgr2[i]+RKgr3[i]);
    Xa[i]	+=cr6*(RKa1[i]+RKa4[i])+cr3*(RKa2[i]+RKa3[i]);
    Xgq[i]	+=cr6*(RKgq1[i]+RKgq4[i])+cr3*(RKgq2[i]+RKgq3[i]);
    Xgf[i]	+=cr6*(RKgf1[i]+RKgf4[i])+cr3*(RKgf2[i]+RKgf3[i]);
  }
  gcmotion_(dXgr,dXa,dXgq,dXgf,Xgr,Xa,Xgq,Xgm,&nPart);
  return(0);
}

int FreeMem4BasicFunctions()
{
  if(BFNa || BFNp){
    free(BFBa);
    free(BFgH);
    free(BFaB);
    free(BFsz);
    free(BFsr);
    BFNa=0;
    BFNa1=0;
    BFNp=0;
    BFNp1=0;
  }
  return(0);
}

int Mem4BasicFunctions(int Na,int Np)
{
  char SNm[0x20]="Mem4BasicFunctions";
  int i;
  if(BFNa == Na && BFNp == Np) return(0);
  FreeMem4BasicFunctions();

  BFNa	=Na;
  BFNp	=Np;
  BFNa1	=BFNa+1;
  BFNp1	=BFNp+1;

  i	=BFNp1*BFNa1;
  BFsr	=(double*)malloc(4*i*sizeof(double));
  if(BFsr == NULL){
    printf("Failure in malloc for BFsr in %s()\n",SNm);
    return(1);
  }
  BFsra	=BFsr	+i;
  BFsrq	=BFsra	+i;
  BFsraq=BFsrq	+i;
  
  BFsz	=(double*)malloc(4*i*sizeof(double));
  if(BFsz == NULL){
    printf("Failure in malloc for BFzr in %s()\n",SNm);
    return(1);
  }
  BFsza	=BFsz	+i;
  BFszq	=BFsza	+i;
  BFszaq=BFszq	+i;
  
  BFaB	=(double*)malloc(4*i*sizeof(double));
  if(BFaB == NULL){
    printf("Failure in malloc for BFaB in %s()\n",SNm);
    return(1);
  }
  BFaBa	=BFaB	+i;
  BFaBq	=BFaBa	+i;
  BFaBaq=BFaBq	+i;

  BFgH	=(double*)malloc(6*i*sizeof(double));
  if(BFgH == NULL){
    printf("Failure in malloc for BFgH in %s()\n",SNm);
    return(1);
  }
  BFgHa	=BFgH	+i;
  BFgHq	=BFgHa	+i;
  BFgHaq=BFgHq	+i;
  BFgh	=BFgHaq	+i;
  BFgha	=BFgh	+i;

  BFBa	=(double*)malloc(4*i*sizeof(double));
  if(BFBa == NULL){
    printf("Failure in malloc for BFBa in %s()\n",SNm);
    return(1);
  }
  BFBaa	=BFBa	+i;
  BFBat	=BFBaa	+i;
  BFBaat=BFBat	+i;
  ESmemlev |=0x20000000;
  return(0);
}

int ESBasicFunctions()
{
  int i,j,ji,k,ki,kj,iK;
  double r,ra,rt,rat,raa,rtt,raat,ratt,raatt;
  double z,za,zt,zat,zaa,ztt,zaat,zatt,zaatt;
  double rc[ESFp1],rs[ESFp1];
  double rct[ESFp1],rst[ESFp1],rca[ESFp1],rsa[ESFp1],rcaa[ESFp1],rsaa[ESFp1];
  double b,ba,baa,dgY,d2gY,pF,dpF,gH,gHa,gQ,gQa;
  double D,Da,Dt,Dat,K,Ka,Kt,Kat,G,Ga,Gt,Gat;
  double cs,sn;

  Mem4BasicFunctions(ESNa,ESNp);
  pF	=ESaF[0];
  G	=1./ESaR0[0];
  b	=ESsb[0];
  ba	=ESsb1a[0];
  baa	=ESsb2a[0];
  rcaa[0]=rcT2a[0];
  rsaa[0]=rsT2a[0];
  rca[1] =2.*rcT1a[ESNa1];
  rsa[1] =2.*rsT1a[ESNa1];
  ki	=0;
  iK	=2;
  for(k=1; k < ESFp1; k++){
    ki	+=ESNa1;
    rcaa[k]	=2.*rcT2a[ki];
    rsaa[k]	=2.*rsT2a[ki];
    if(rcaa[k] != 0. || rsaa[k] != 0.) iK=k+1;
  }
  gH	=0.;
  gHa	=0.;
  gQ	=0.;
  gQa	=0.;
  r	=ESaR0[0];
  z	=ESaZ0[0];
  for(j=0; j < ESNp; j++){
    cs		=EScs1[j];
    sn		=ESsn1[j];
    ra		=rca[1]*cs+rsa[1]*sn;
    rt		=0.;
    rat		=rsa[1]*cs-rca[1]*sn;
    ratt	=-ra;
    raa		=rcaa[0]+rcaa[1]*cs+rsaa[1]*sn;
    raat	=rsaa[1]*cs-rcaa[1]*sn;
    raatt	=-rcaa[1]*cs-rsaa[1]*sn;
    za		=ba*sn;
    zt		=0.;
    zat		=ba*cs;
    zatt	=-za;
    zaa		=rsaa[0]+baa*sn;
    zaat	=baa*cs;
    zaatt	=-baa*sn;
    kj	=j < ESNp ? j : j-ESNp;
    for(k=2; k < iK; k++){
      kj	+=j;
      if(kj >= ESNp) kj -=ESNp;
      cs	=EScs1[kj];
      sn	=ESsn1[kj];
      raa	+=rcaa[k]*cs+rsaa[k]*sn;
      raat	+=k*(rsaa[k]*cs-rcaa[k]*sn);
      raatt	-=k*k*(rcaa[k]*cs+rsaa[k]*sn);
    }

    ESsra[j]	=ra;
    ESsrt[j]	=rt;
    ESsrat[j]	=rat;
    ESsza[j]	=za;
    ESszt[j]	=zt;
    ESszat[j]	=zat;

    Ga	=-ra*G*G;
    Gt	=0.;
    Gat	=-rat*G*G;

    ESaB[j]	=pF*G;
    ESaBa[j]	=pF*Ga;
    ESaBt[j]	=0.;
    ESaBat[j]	=pF*Gat;

    ESgH[j]	=0.;
    ESgHt[j]	=0.;
    ESgQ[j]	=0.;
    ESgQt[j]	=0.;

    D	=ra*zat-za*rat;
    D	=rca[1]*ba;
#ifdef H
    ra	=ra+a*raa;
    rt	=a*(rat+a*cr2*raat);

    za	=za+a*zaa;
    zt	=a*(zat+a*cr2*zaat);

    D/a=(ra+a*raa)*(zat+a*cr2*zaat)-(za+a*zaa)*(rat+a*cr2*raat);
    (D/a)=ra*zat-za*rat;
    (D/a)_a=raa*zat+ra*zaat*cr2-zaa*rat-za*raat*cr2;
    R/r=1-a*ra/R;

    L/L_0=1-a*ra/R+a(raa*zat-zaa*rat+ra*cr2*zaat-za*cr2*raat)/D0;

    D/r=1+a*(raa*zat+a*cr2*ra*zaat-a*zaa*rat-a*cr2*za*raat)/D0-a*ra*G;

    gH	=a*(raa*zat-zaa*rat+cr2*(ra*zaat-za*raat))/D0-a*ra*G;
#endif

#ifdef H
    ESgHa[j]	=(raa*zat-zaa*rat+cr2*(ra*zaat-za*raat))/D-ra*G;
    ESgHat[j]	=(raa*zatt-zaa*ratt+cr2*(ra*zaatt-rat*zaat+zat*raat-za*raatt))
      /D-rat*G;
#endif
    Da		=raa*zat-rat*zaa+cr2*(ra*zaat-raat*za);
    Dt		=raa*zat-rat*zaa+cr2*(ra*zaat-raat*za);
    ESgHa[j]	=(Da-D*ra*G)*G;
    ESgHat[j]	=(Dt-D*rat*G)*G;

    ESgQa[j]	=Da*r+D*ra;
    ESgQat[j]	=Dt*r+D*rat;

    gHa	+=ESgHa[j];
    gH	+=ESgHat[j];
    gQa	+=ESgQa[j];
    gQ	+=ESgQat[j];

    BFBa[j]	=0.;
    BFBat[j]	=0.;
    BFBaa[j]	=(ra*rat+za*zat)*ESdgY[0]*G/D;
    BFBaat[j]	=(rat*rat+ra*ratt+zat*zat+za*zatt)*ESdgY[0]*G/D;
  }
  ESsra[j]	=ESsra[0];
  ESsrt[j]	=ESsrt[0];
  ESsrat[j]	=ESsrat[0];
  ESsza[j]	=ESsza[0];
  ESszt[j]	=ESszt[0];
  ESszat[j]	=ESszat[0];

  ESaB[j]	=ESaB[0];
  ESaBa[j]	=ESaBa[0];
  ESaBt[j]	=ESaBt[0];
  ESaBat[j]	=ESaBat[0];
  gHa	/=ESNp;
  gH	/=ESNp;
  gQa	/=ESNp;
  gQ	/=ESNp;
  for(j=0; j < ESNp; j++){
    ESgHa[j]	-=gHa;
    ESgHat[j]	-=gH;
    ESgQa[j]	-=gQa;
    ESgQat[j]	-=gQ;
  }
  ESgH[j]	=ESgH[0];
  ESgHa[j]	=ESgHa[0];
  ESgHt[j]	=ESgHt[0];
  ESgHat[j]	=ESgHat[0];

  ESgQ[j]	=ESgQ[0];
  ESgQa[j]	=ESgQa[0];
  ESgQt[j]	=ESgQt[0];
  ESgQat[j]	=ESgQat[0];

  ji	=ESNp1;
  for(i=1; i < ESNa1; i++){
    dgY		=ESsa[i]*ESdgY[i];
    d2gY	=ESdgY[i]+ESsa[i]*ESdgY1a[i];
    pF		=ESFF[i];
    dpF		=ESFF1a[i];
    rc[0]	=ESaR0[0]+rcT[i];
    rs[0]	=ESaZ0[0]+rsT[i];
    rca[0]	=rcT1a[i];
    rsa[0]	=rsT1a[i];
    rcaa[0]	=rcT2a[i];
    rsaa[0]	=rsT2a[i];
    b		=ESsb[i];
    ba		=ESsb1a[i];
    baa		=ESsb2a[i];
    ki	=i;
    iK	=1;
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      rc[k]	=2.*rcT[ki];
      rct[k]	=2.*k*rsT[ki];
      rca[k]	=2.*rcT1a[ki];
      rcaa[k]	=2.*rcT2a[ki];
      rs[k]	=2.*rsT[ki];
      rst[k]	=-2.*k*rcT[ki];
      rsa[k]	=2.*rsT1a[ki];
      rsaa[k]	=2.*rsT2a[ki];
      if(rct[k] != 0. || rca[k] != 0. || rcaa[k] != 0. ||
	 rst[k] != 0. || rsa[k] != 0. || rsaa[k] != 0.){
	iK	=k+1;
      }
    }
    gH	=0.;
    gHa	=0.;
    gQ	=0.;
    gQa	=0.;

    for(j=0; j < ESNp; j++){
      r		=rc[0];
      ra	=rca[0];
      rt	=0.;
      rat	=0.;
      raa	=rcaa[0];
      rtt	=0.;
      raat	=0.;
      ratt	=0.;
      cs	=EScs1[j];
      sn	=ESsn1[j];
      za	=rsa[0]+ba*sn;
      zt	=b*cs;
      zat	=ba*cs;
      zaa	=rsaa[0]+baa*sn;
      ztt	=-b*sn;
      zaat	=baa*cs;
      zatt	=-ba*sn;
      kj	=0;
      for(k=1; k < iK; k++){
	kj	+=j;
	if(kj >= ESNp) kj -=ESNp;
	cs	=EScs1[kj];
	sn	=ESsn1[kj];
	r	+=rc[k]*cs+rs[k]*sn;
	ra	+=rca[k]*cs+rsa[k]*sn;
	rt	+=rct[k]*cs+rst[k]*sn;
	rat	+=k*(rsa[k]*cs-rca[k]*sn);
	raa	+=rcaa[k]*cs+rsaa[k]*sn;
	rtt	+=k*(rst[k]*cs-rct[k]*sn);
	raat	+=k*(rsaa[k]*cs-rcaa[k]*sn);
	ratt	-=k*k*(rca[k]*cs+rsa[k]*sn);
      }
      ESsz[ji]	=rs[0]+b*ESsn1[j];
      ESsza[ji]	=za;
      ESszt[ji]	=zt;
      ESszat[ji]=zat;
      ESsr[ji]	=r;
      ESsra[ji]	=ra;
      ESsrt[ji]	=rt;
      ESsrat[ji]=rat;

      D		=ra*zt-rt*za;
      Da	=raa*zt+ra*zat-rat*za-rt*zaa;
      Dt	=rat*zt+ra*ztt-rtt*za-rt*zat;
      Dat	=raat*zt+raa*ztt+ra*zatt-ratt*za-rtt*zaa-rt*zaat;

      K		=1./r;
      Ka	=-ra*K*K;
      Kt	=-rt*K*K;
      Kat	=(2.*rt*ra*K-rat)*K*K;
      
      ESgH[ji]	=D*K;
      ESgHa[ji]	=Da*K+D*Ka;
      ESgHt[ji]	=Dt*K+D*Kt;
      ESgHat[ji]=Dat*K+Dt*Ka+Da*Kt+D*Kat;
      
      ESgQ[ji]	=D*r;
      ESgQa[ji]	=Da*r+D*ra;
      ESgQt[ji]	=Dt*r+D*rt;
      ESgQat[ji]=Dat*r+Dt*ra+Da*rt+D*rat;

      gH	+=ESgH[ji];
      gHa	+=ESgHa[ji];
      gQ	+=ESgQ[ji];
      gQa	+=ESgQa[ji];
      
      G	=(ra*rt+za*zt)/D;
      Ga=(raa*rt+ra*rat+zaa*zt+za*zat-G*Da)/D;
      Gt=(rat*rt+ra*rtt+zat*zt+za*ztt-G*Dt)/D;
      
      Gat=(raat*rt+raa*rtt+rat*rat+ra*ratt+zaat*zt+zaa*ztt+zat*zat
	   +za*zatt-Gt*Da-G*Dat-Ga*Dt)*dgY/D+Gt*d2gY;
      Ga=Ga*dgY+G*d2gY;
      G	*=dgY;
      Gt*=dgY;

      BFBa[ji]	=G*K;
      BFBat[ji]	=Gt*K+G*Kt;
      BFBaa[ji]	=Ga*K+G*Ka;
      BFBaat[ji]=Gat*K+Ga*Kt+Gt*Ka+G*Kat;

#ifdef H 
     BFBa[ji]	=G*dgY;
      BFBat[ji]	=Gt*dgY;
      BFBaa[ji]	=Ga*dgY+G*d2gY;
      BFBaat[ji]=(raat*rt+raa*rtt+rat*rat+ra*ratt+zaat*zt+zaa*ztt+zat*zat
		  +za*zatt-Gt*Da-G*Dat-Ga*Dt)*dgY/D+Gt*d2gY;



      G	=ra*rt+za*zt;
      Ga=raa*rt+ra*rat+zaa*zt+za*zat;
      Gt=rat*rt+ra*rtt+zat*zt+za*ztt;

      BFBa[ji]	=G*dgY;
      BFBat[ji]	=Gt*dgY;
      BFBaa[ji]	=Ga*dgY+G*d2gY;
      BFBaat[ji]=(raat*rt+raa*rtt+rat*rat+ra*ratt+zaat*zt+zaa*ztt+zat*zat
		  +za*zatt)*dgY+Gt*d2gY;

      BFBa[ji]	=za;
      BFBat[ji]	=zat;
      BFBaa[ji]	=zaa;
      BFBaat[ji]=zaat;
#endif

      ESaB[ji]	=pF*K*K;
      ESaBa[ji]	=(dpF*K+2.*pF*Ka)*K;
      ESaBt[ji]	=2.*pF*Kt*K;
      ESaBat[ji]=2.*dpF*K*Kt+2.*pF*(Kat*K+Kt*Ka);

#ifdef ForFRC
      ESaB[ji]	=0.;
       ESaBa[ji]	=0.; 
       ESaBt[ji]	=0.; 
       ESaBat[ji]=0.; 
#endif  
 
       D	=1./D; 
       Da	*=D; 
       Dt	*=D; 
       Dat	=(2.*Dt*Da-Dat*D)*D; 
       Da	*=-D; 
       Dt	*=-D; 

       G		=K*D; 
       Ga	=Ka*D+K*Da; 
       Gt	=Kt*D+K*Dt; 
       Gat	=Kat*D+Kt*Da+Ka*Dt+K*Dat; 

       D		=rt*rt+zt*zt; 
       Da	=2.*(rat*rt+zat*zt); 
       Dt	=2.*(rtt*rt+ztt*zt); 
       Dat	=2.*(ratt*rt+rtt*rat+zatt*zt+ztt*zat); 

       K		=D*G*G; 
       Ka	=(Da*G+2.*D*Ga)*G; 
       Kt	=(Dt*G+2.*D*Gt)*G; 
       Kat	=(Dat*G+2.*Dt*Ga+2.*Da*Gt+2.*D*Gat)*G+2.*D*Gt*Ga; 
       ESaB[ji]	+=K*dgY*dgY; 
       ESaBa[ji]	+=(Ka*dgY+2.*K*d2gY)*dgY; 
       ESaBt[ji]	+=Kt*dgY*dgY; 
       ESaBat[ji]+=(Kat*dgY+2.*Kt*d2gY)*dgY; 

       D	=sqrt(ESaB[ji]); 
       ESaB[ji]	=D; 
       D		=cr2/D; 
       ESaBa[ji]	*=D; 
       ESaBt[ji]	*=D; 
       ESaBat[ji]=(ESaBat[ji]-2.*ESaBa[ji]*ESaBt[ji])*D; 
       ji++; 
    } 
    j	=ji-ESNp; 
    ESsra[ji]	=ESsra[j]; 
    ESsrt[ji]	=ESsrt[j]; 
    ESsrat[ji]	=ESsrat[j]; 
    ESsza[ji]	=ESsza[j]; 
    ESszt[ji]	=ESszt[j]; 
    ESszat[ji]	=ESszat[j]; 
    ESaB[ji]	=ESaB[j]; 
    ESaBa[ji]	=ESaBa[j]; 
    ESaBt[ji]	=ESaBt[j]; 
    ESaBat[ji]	=ESaBat[j]; 
    ESgH[ji]	=ESgH[j]; 
    ESgHa[ji]	=ESgHa[j]; 
    ESgHt[ji]	=ESgHt[j]; 
    ESgHat[ji]	=ESgHat[j]; 
    ESgQ[ji]	=ESgQ[j]; 
    ESgQa[ji]	=ESgQa[j]; 
    ESgQt[ji]	=ESgQt[j]; 
    ESgQat[ji]	=ESgQat[j]; 

    BFBa[ji]	=BFBa[j]; 
    BFBaa[ji]	=BFBaa[j]; 
    BFBat[ji]	=BFBat[j]; 
    BFBaat[ji]	=BFBaat[j]; 

    ji++; 
    gH	=ESNp/gH; 
    gHa/=ESNp; 
    gQ	=ESNp/gQ; 
    gQa/=ESNp; 
    while(j < ji){ 
      ESgHa[j]	=(gHa*ESgH[j]*gH-ESgHa[j])*gH; 
      ESgH[j]	=1.-ESgH[j]*gH; 
      ESgQa[j]	=(gQa*ESgQ[j]*gQ-ESgQa[j])*gQ; 
      ESgQ[j]	=1.-ESgQ[j]*gQ; 
      j++; 
    } 
  }

   ji	=0; 
   for(i=0; i < BFNa1; i++){ 
     k	=ji+BFNp1; 
     for(j=0; j < BFNp1; j++){ 
       k--; 
       BFsr[ji]	=ESsr[k]; 
       BFsra[ji]	=ESsra[k]; 
       BFsrq[ji]	=-ESsrt[k]; 
       BFsraq[ji]=-ESsrat[k]; 

       BFsz[ji]	=ESsz[k]; 
       BFsza[ji]	=ESsza[k]; 
       BFszq[ji]	=-ESszt[k]; 
       BFszaq[ji]=-ESszat[k]; 

       BFaB[ji]	=ESaB[k]; 
       BFaBa[ji]	=ESaBa[k]; 
       BFaBq[ji]	=-ESaBt[k]; 
       BFaBaq[ji]=-ESaBat[k]; 

       BFgH[ji]		=ESgH[k]; 
       BFgHa[ji]	=ESgHa[k]; 
       BFgHq[ji]	=-ESgHt[k]; 
       BFgHaq[ji]=-ESgHat[k]; 
       ji++; 
     } 
   } 

   r	=cgp/ESNp; 
   ji	=0; 
   for(i=0; i < BFNa1; i++){ 
     BFgh[ji]	=0.; 
     BFgha[ji]	=0.; 
     for(j=1; j < BFNp1; j++){ 
       kj	=ji; 
       ji++; 
       BFgh[ji]		=BFgh[kj]+(BFgH[kj]+BFgH[ji])*r; 
       BFgha[ji]	=BFgha[kj]+(BFgHa[kj]+BFgHa[ji])*r; 
     } 
     ji++; 
   } 

#ifdef H 
   r	=c2gp/ESNp; 
   ji	=0; 
   for(i=0; i < BFNa1; i++){ 
     BFgh[ji]	=0.; 
     BFgha[ji]	=0.; 
     for(j=1; j < BFNp1; j++){ 
       kj	=ji; 
       ji++; 
       BFgh[ji]	=BFgh[kj] 
 	+(BFgH[kj]+BFgH[ji]+cr6*r*(BFgHq[kj]-BFgHq[ji]))*r*0.5; 
       BFgha[ji]	=BFgha[kj] 
 	+(BFgHa[kj]+BFgHa[ji]+cr6*r*(BFgHaq[kj]-BFgHaq[ji]))*r*0.5; 
     } 
     ji++; 
   } 
#endif 
   return(0); 
} 

int ESWriteESI0()
{
  static int Fic=0;
  int i,j,ji;
  double *f=NULL,*fa,*fq,*faq,rr0;
  FILE *lf,*lfa;
  char FNm[16];

  double Te,Ti,p,dp,t;
  extern char *CbUserMessage;
  extern char ESMessage[];

  CbUserMessage	=ESMessage;
  f	=(double*)malloc(4*ESNp1*sizeof(double));
  if(f == NULL){
    sprintf(ESMessage,"No memory for f");
    return(1);
  }
  fa	=f+ESNp1;
  fq	=fa+ESNp1;
  faq	=fq+ESNp1;
  sprintf(FNm,"esiBin.00");
  lf	=fopen(FNm,"w");
  if(lf == NULL){
    sprintf(ESMessage,"%s cannot be open",FNm);
    free(f);
    return(1);
  }
  sprintf(FNm,"esiA.%2.2d",Fic);
  sprintf(FNm,"esiA.00");
  lfa	=fopen(FNm,"w");
  if(lfa == NULL){
    sprintf(ESMessage,"%s cannot be open",FNm);
    fclose(lf);
    free(f);
    return(1);
  }

  time(&stTime);
  ltm	=localtime(&stTime);
  sprintf(ESMessage,"Date: %2.2d/%2.2d/%2.2d at %2.2d:%2.2d"
	  ,ltm->tm_mon+1,ltm->tm_mday,ltm->tm_year%100,ltm->tm_hour
	  ,ltm->tm_min); 
  if(ESIWriteDensity(ESNa,Fic)){
    fclose(lfa);
    return(1);
  }

  fprintf(lfa,"!!! Do not edit this file\n");
  fprintf(lfa,"%3d x %3d - numbers of poloidal x radial data points\n"
	  ,ESNp1,ESNa1);
  fwrite(&ESNp1,sizeof(int),1,lf);
  fwrite(&ESNa1,sizeof(int),1,lf);
  fputs("gq:\n",lfa);
  for(j=0; j < ESNp1; j++){
    fprintf(lfa,"%24.16e",ESgt[j]);
    if(j%4 == 3){
      fputc('\n',lfa);
    }
  }
  if(j%4 != 0){
    fputc('\n',lfa);
  }
  fwrite(ESgt,sizeof(double),ESNp1,lf);
  fprintf(lfa,"%24s%24s%24s\n","a","F","Fa");
  fwrite(ESsa,sizeof(double),ESNa1,lf);
  fwrite(ESaF,sizeof(double),ESNa1,lf);
  fwrite(ESaF1a,sizeof(double),ESNa1,lf);
  rr0	=1./ESaR0[0];
  for(i=0; i < ESNa1; i++){
    f[i]	=ESLc[i]*ESaF[i]*rr0;
    fq[i]	=(ESLc1[i]*ESaF[i]+ESLc[i]*ESaF1a[i])*rr0;
    fprintf(lfa,"%24.16e%24.16e%24.16e\n",ESsa[i],ESaF[i],ESaF1a[i]);
  }
  fprintf(lfa,"%24s%24s%24s%24s\n","gF'/a","gF''","gY'/a","(gY'/a)'");
  fwrite(f,sizeof(double),ESNa1,lf);
  fwrite(fq,sizeof(double),ESNa1,lf);
  fwrite(ESdgY,sizeof(double),ESNa1,lf);
  fwrite(ESdgY1a,sizeof(double),ESNa1,lf);
  for(i=0; i < ESNa1; i++){
    fprintf(lfa,"%24.16e%24.16e%24.16e%24.16e\n"
	    ,f[i],fq[i],ESdgY[i],ESdgY1a[i]);
  }
  fprintf(lfa,"%24s%24s%24s%24s\n","T","Ta","P","Pa");
  fwrite(ESaT,sizeof(double),ESNa1,lf);
  splA1a(ESaT,fa,ESaT2a);
  fwrite(fa,sizeof(double),ESNa1,lf);
  fwrite(ESPs,sizeof(double),ESNa1,lf);
  fwrite(ESPs1a,sizeof(double),ESNa1,lf);
  for(i=0; i < ESNa1; i++){
    fprintf(lfa,"%24.16e%24.16e%24.16e%24.16e\n"
	    ,ESaT[i],ESaT1a[i],ESPs[i],ESPs1a[i]);
  }

  fprintf(lfa,"%24s%24s%24s%24s\n","r","r'_a","r'_gq","r''_{a,gq}");
  for(i=0; i < ESNa1; i++){
    ji	=ESNp1*i+ESNp1;
    for(j=0; j < ESNp1; j++){
      ji--;
      fprintf(lfa,"%24.16e%24.16e%24.16e%24.16e\n"
	      ,ESsr[ji],ESsra[ji],-ESsrt[ji],-ESsrat[ji]);
    }
  }

  fprintf(lfa,"%24s%24s%24s%24s\n","z","z'_a","z'_gq","z''_{a,gq}");
  for(i=0; i < ESNa1; i++){
    ji	=ESNp1*i+ESNp1;
    for(j=0; j < ESNp1; j++){
      ji--;
      fprintf(lfa,"%24.16e%24.16e%24.16e%24.16e\n"
	      ,ESsz[ji],ESsza[ji],-ESszt[ji],-ESszat[ji]);
    }
  }

  fprintf(lfa,"%24s%24s%24s%24s\n","B","B'_a","B'_gq","B''_{a,gq}");
  for(i=0; i < ESNa1; i++){
    ji	=ESNp1*i+ESNp1;
    for(j=0; j < ESNp1; j++){
      ji--;
      fprintf(lfa,"%24.16e%24.16e%24.16e%24.16e\n"
	      ,ESaB[ji],ESaBa[ji],-ESaBt[ji],-ESaBat[ji]);
    }
  }

  fprintf(lfa,"%24s%24s%24s%24s\n"
	  ,"gh'_gq","gh''_{a,gq}","gh''_{gq,gq}","gh'''_{a,gq,gq}");
  for(i=0; i < ESNa1; i++){
    ji	=ESNp1*i+ESNp1;
    for(j=0; j < ESNp1; j++){
      ji--;
      fprintf(lfa,"%24.16e%24.16e%24.16e%24.16e\n"
	      ,ESgH[ji],ESgHa[ji],-ESgHt[ji],-ESgHat[ji]);
    }
  }
  for(i=0; i < ESNa1; i++){
    ji	=ESNp1*i;
    j	=ESNp1;
    while(j > 0){
      j--;
      f[j]	=ESsr[ji];
      fa[j]	=ESsra[ji];
      fq[j]	=-ESsrt[ji];
      faq[j]	=-ESsrat[ji];
      ji++;
    }
    fwrite(f,sizeof(double),4*ESNp1,lf);
    ji	=ESNp1*i;
    j	=ESNp1;
    while(j > 0){
      j--;
      f[j]	=ESsz[ji];
      fa[j]	=ESsza[ji];
      fq[j]	=-ESszt[ji];
      faq[j]	=-ESszat[ji];
      ji++;
    }
    fwrite(f,sizeof(double),4*ESNp1,lf);
    ji	=ESNp1*i;
    j	=ESNp1;
    while(j > 0){
      j--;
      f[j]	=ESaB[ji];
      fa[j]	=ESaBa[ji];
      fq[j]	=-ESaBt[ji];
      faq[j]	=-ESaBat[ji];
      ji++;
    }
    fwrite(f,sizeof(double),4*ESNp1,lf);
    ji	=ESNp1*i;
    j	=ESNp1;
    while(j > 0){
      j--;
      f[j]	=ESgH[ji];
      fa[j]	=ESgHa[ji];
      fq[j]	=-ESgHt[ji];
      faq[j]	=-ESgHat[ji];
      ji++;
    }
    fwrite(f,sizeof(double),4*ESNp1,lf);
  }
  free(f);

  Te	=15.;
  Ti	=15.;
  t	=1./(1.60022e-2*cgm0*(Te+Ti));
  {
    double F,f0,f1;
    F	=0.;
    f1	=(ESLc[0]+ESVc[0])*ESsp[0];
    for(i=1; i < ESNa1; i++){
      f0=f1;
      f1=(ESVc[i]+ESLc[i])*ESsp[i];
      F	+=cr4*(f0+f1)*(ESpa[i]-ESpa[i-1]);
    }
    F	*=c2gp*c2gp*ESaR0[0]*t;
    fprintf(lfa,"Te=Ti=15keV N=%8.2e a%24s%24s\n",F*1e+20
	    ,"ne [10^20/m^3]","dne/da [10^20/m^3]");
  }

#ifdef H
  ji	=0;
  printf("%2s %8s %8s %12s %12s %12s %12s %12s\n","i"
	 ,"a","r [m]","dr/da","B [T]","dB/da","n[10^20/m^3]","dn/da");
  for(i=0; i < ESNa1; i++){
    fprintf(lfa,"%24.16e%24.16e%24.16e\n",ESsa[i],t*ESsp[i],t*ESsp1a[i]);
    p	=i ? 1./ESsra[ji] : 0.;
    printf("%2d %8.6f %8.6f %12.5e %12.5e %12.5e %12.5e %12.5e\n",i,ESsa[i]
	   ,ESsr[ji],ESsra[ji],ESaB[ji],ESaBa[ji]*p,t*ESsp[i],t*ESsp1a[i]*p);
    ji	+=ESNp1;
  }
#endif
  sprintf(ESMessage,"%s has been written",FNm);
  fclose(lfa);
  fclose(lf);
  Fic++;
#ifdef H
  if(esiread_(FNm)){
    sprintf(ESMessage,"%s Bad file",FNm);
  }
#endif
  return(0);
}

int ESWriteESI(char *CoordNm,double *sr,double*sz,double*aB,double*gH
	       ,int Na,int Np)
{
  static int Fic[4]={0,0,0,0},ic=0;
  int Na1,Np1;
  int i,j,ji;
  double *srt,*sra,*srat;
  double *szt,*sza,*szat;
  double *aBt,*aBa,*aBat;
  double *gHt,*gHa,*gHat;
  double rr0,a,da,t,dt;
  double F,Fa,dgY,dgYa,L,La,V,Va;

  double Te,Ti,p,dp;

  FILE *lfa;
  char FNm[16];
  extern char ESMessage[];
  extern char *CbUserMessage;
  CbUserMessage	=ESMessage;

  Na1	=Na+1;
  Np1	=Np+1;
  da	=(ESsa[ESNa]-ESsa[0])/Na;
  dt	=c2gp/Np;

  i	=Np1*Na1;
  sra	=sr	+i;
  srt	=sra	+i;
  srat	=srt	+i;
  sza	=sz	+i;
  szt	=sza	+i;
  szat	=szt	+i;
  aBa	=aB	+i;
  aBt	=aBa	+i;
  aBat	=aBt	+i;
  if(gH != NULL){
    gHa	=gH	+i;
    gHt	=gHa	+i;
    gHat=gHt	+i;
  }
  *ESMessage	=CoordNm != NULL ? *CoordNm : 'A';
  switch(*ESMessage){
  case 'P':
    ic	=1;
    break;
  case 'B':
    ic	=2;
    break;
  case 'H':
    ic	=3;
    break;
  case 'C':
    ic	=3;
    break;
  default:
    ic	=0;
    break;
  }

  sprintf(FNm,"esi%c.%2.2d",*ESMessage,Fic[ic]);
  sprintf(FNm,"esi%c.00",*ESMessage);
  *ESMessage	='\0';

  lfa	=fopen(FNm,"w");
  if(lfa == NULL){
    sprintf(ESMessage,"%s cannot be open",FNm);
    return(1);
  }

  time(&stTime);
  ltm	=localtime(&stTime);
  fprintf(lfa,"!!! Do not edit this file. "
	  "Date: %2.2d/%2.2d/%2.2d at %2.2d:%2.2d\n"
	  ,ltm->tm_mon+1,ltm->tm_mday,ltm->tm_year%100,ltm->tm_hour
	  ,ltm->tm_min); 

  if(CoordNm != NULL){
    fprintf(lfa,"%3d x %3d %s - number of poloidal x radial data points.\n"
	    ,Np1,Na1,CoordNm);
  }
  else{
    fprintf(lfa,"%3d x %3d ESC - numbers of poloidal x radial data points.\n"
	    ,Np1,Na1);
  }

  fputs("gq:\n",lfa);
  for(j=0; j < Np1; j++){
    fprintf(lfa,"%24.16e",dt*j);
    if(j%4 == 3){
      fputc('\n',lfa);
    }
  }
  if(j%4 != 0){
    fputc('\n',lfa);
  }
  fprintf(lfa,"%24s%24s%24s\n","a","F","Fa");
  rr0	=1./ESaR0[0];
  for(i=0; i < Na1; i++){
    a	=da*i;
    ESSetSplA(a);
    splRA(&F,&Fa,ESFF,ESFF2a);
    F	=sqrt(F);
    Fa	/=2.*F;
    fprintf(lfa,"%24.16e%24.16e%24.16e\n",da*i,F,Fa);
  }
  fprintf(lfa,"%24s%24s%24s%24s\n","gF'/a","gF''","gY'/a","(gY'/a)'");
  for(i=0; i < Na1; i++){
    a	=da*i;
    ESSetSplA(a);
    splRA(&F,&Fa,ESFF,ESFF2a);
    splRA(&dgY,&dgYa,ESdgY,ESdgY2a);
    F	=sqrt(F);
    Fa	/=2.*F;
    splRA(&L,&La,ESLc,ESLc2);
    fprintf(lfa,"%24.16e%24.16e%24.16e%24.16e\n"
	    ,L*F*rr0,(La*F+L*Fa)*rr0,dgY,dgYa);
  }
  fprintf(lfa,"%24s%24s%24s%24s\n","T","Ta","P","Pa");
  for(i=0; i < Na1; i++){
    a	=da*i;
    ESSetSplA(a);
    splRA(&F,&Fa,ESaT,ESaT2a);
    splRA(&dgY,&dgYa,ESPs,ESPs2a);
    fprintf(lfa,"%24.16e%24.16e%24.16e%24.16e\n",F,Fa,dgY,dgYa);
  }

  fprintf(lfa,"%24s%24s%24s%24s\n","r","r'_a","r'_gq","r''_{a,gq}");
  for(i=0; i < Na1; i++){
    ji	=Np1*i+Np1;
    for(j=0; j < Np1; j++){
      ji--;
      fprintf(lfa,"%24.16e%24.16e%24.16e%24.16e\n"
	      ,sr[ji],sra[ji],-srt[ji],-srat[ji]);
    }
  }

  fprintf(lfa,"%24s%24s%24s%24s\n","z","z'_a","z'_gq","z''_{a,gq}");
  for(i=0; i < Na1; i++){
    ji	=Np1*i+Np1;
    for(j=0; j < Np1; j++){
      ji--;
      fprintf(lfa,"%24.16e%24.16e%24.16e%24.16e\n"
	      ,sz[ji],sza[ji],-szt[ji],-szat[ji]);
    }
  }

  fprintf(lfa,"%24s%24s%24s%24s\n","B","B'_a","B'_gq","B''_{a,gq}");
  for(i=0; i < Na1; i++){
    ji	=Np1*i+Np1;
    for(j=0; j < Np1; j++){
      ji--;
      fprintf(lfa,"%24.16e%24.16e%24.16e%24.16e\n"
	      ,aB[ji],aBa[ji],-aBt[ji],-aBat[ji]);
    }
  }

  fprintf(lfa,"%24s%24s%24s%24s\n"
	  ,"gh'_gq","gh''_{a,gq}","gh''_{gq,gq}","gh'''_{a,gq,gq}");
  if(gH != NULL){
    for(i=0; i < Na1; i++){
      ji	=Np1*i+Np1;
      for(j=0; j < Np1; j++){
	ji--;
	fprintf(lfa,"%24.16e%24.16e%24.16e%24.16e\n"
		,gH[ji],gHa[ji],-gHt[ji],-gHat[ji]);
      }
    }
  }
  else{
    for(i=0; i < Na1; i++){
      for(j=0; j < Np1; j++){
	fprintf(lfa,"%24.16e%24.16e%24.16e%24.16e\n",0.,0.,0.,0.);
      }
    }
  }

  Te	=15.;
  Ti	=15.;
  t	=1./(1.60022e-2*cgm0*(Te+Ti));

  {
    double f0,f1,pa;
    F	=0.;
    f1	=(ESLc[0]+ESVc[0])*ESsp[0];
    pa	=0.;
    for(i=1; i < Na1; i++){
      a	=da*i;
      ESSetSplA(a);
      splRA(&L,&La,ESLc,ESLc2);
      splRA(&V,&Va,ESVc,ESVc2);
      splRA(&p,&dp,ESsp,ESsp2a);
      f0=f1;
      V	+=L;
      f1=V*p;
      F	+=cr4*(f0+f1)*(a*a-pa);
      pa	=a*a;
    }
    F	*=c2gp*c2gp*ESaR0[0]*t;
  }

  fprintf(lfa,"Te=Ti=15keV N=%8.2e a%24s%24s\n",F*1e+20
	  ,"ne [10^20/m^3]","dne/da [10^20/m^3]");
  for(i=0; i < Na1; i++){
    a	=da*i;
    ESSetSplA(a);
    splRA(&p,&dp,ESsp,ESsp2a);
    fprintf(lfa,"%24.16e%24.16e%24.16e\n",a,t*p,t*dp);
  }
  fclose(lfa);
  sprintf(ESMessage,"%s has been written",FNm);
#ifdef H
  if(esiread_(FNm)){
    sprintf(ESMessage,"%s Bad file",FNm);
  }
#endif
  Fic[ic]++;
  return(0);
}

int ESBasicFunctionsPEST(int Na, int Np)
{
  char *SNm="ESBasicFunctionsPEST";
  int Np1,Na1;
  int i,j,ji,k,ki,kj,iK;
  double r,ra,rt,rat,raa,rtt,rttt,raat,ratt;
  double z,za,zt,zat,zaa,ztt,zttt,zaat,zatt;
  double rc[ESFp1],rs[ESFp1];
  double rct[ESFp1],rst[ESFp1],rca[ESFp1],rsa[ESFp1],rcaa[ESFp1],rsaa[ESFp1];
  double b,ba,baa,dgY,d2gY,pF,dpF;
  double D,Da,Dt,Dat,Dtt,K,Ka,Kt,Kat,Ktt;
  double a,da,t,dt,s,s1,s2,cs,sn;
  double Q,Qa,L0,L0a,L,La,Lt,Lat,Ltt;
  double gQ[ESNp1],gQc[ESFp1],gQs[ESFp1];
  double gQa[ESNp1],gQac[ESFp1],gQas[ESFp1];

  double *sr,*sra,*srt,*srat;
  double *sz,*sza,*szt,*szat;
  double *aB,*aBa,*aBt,*aBat,B,Ba,Bt,Bat,Btt;
  double ta,tq,taq;
  double q,qa,qt,qat,qtt;

  Np1	=Np+1;
  Na1	=Na+1;
  da	=(ESsa[ESNa]-ESsa[0])/Na;
  dt	=c2gp/Np;

  i	=Np1*Na1;
  sr	=(double*)malloc(4*i*sizeof(double));
  if(sr == NULL){
    printf("Failure in malloc for sr in %s()\n",SNm);
    return(1);
  }
  sra	=sr	+i;
  srt	=sra	+i;
  srat	=srt	+i;
  
  sz	=(double*)malloc(4*i*sizeof(double));
  if(sz == NULL){
    printf("Failure in malloc for zr in %s()\n",SNm);
    return(1);
  }
  sza	=sz	+i;
  szt	=sza	+i;
  szat	=szt	+i;
  
  aB	=(double*)malloc(4*i*sizeof(double));
  if(aB == NULL){
    printf("Failure in malloc for aB in %s()\n",SNm);
    return(1);
  }
  aBa	=aB	+i;
  aBt	=aBa	+i;
  aBat	=aBt	+i;

  /* a=0 surface */
  ki	=ESNa1;
  rca[1]=2.*rcT1a[ki];
  rsa[1]=2.*rsT1a[ki];
  ba	=ESsb1a[0];
  pF	=ESaF[0];
  r	=ESaR0[0];
  K	=1./r;
  ji	=0;
  for(j=0; j < Np1; j++){
    t		=dt*j;
    cs		=cos(t);
    sn		=sin(t);

    ra		=rca[1]*cs+rsa[1]*sn;
    rt		=rsa[1]*cs-rca[1]*sn;
    rat		=rsa[1]*cs-rca[1]*sn;

    za		=ba*sn;
    zt		=ba*cs;
    zat		=ba*cs;

    sr[ji]	=r;
    sra[ji]	=ra;
    srt[ji]	=0.;
    srat[ji]	=rat;
    sz[ji]	=ESaZ0[0];
    sza[ji]	=za;
    szt[j]	=0.;
    szat[j]	=zat;
    
    Ka	=-ra*K*K;
    Kt	=0.;
    Kat	=-rat*K*K;
    
    aB[ji]	=pF*K;
    aBa[ji]	=pF*Ka;
    aBt[ji]	=0.;
    aBat[ji]	=pF*Kat;
    ji++;
  }

  for(i=1; i < Na1; i++){
    a	=da*i;
    ESSetSplA(a);
    splRA(&pF,&dpF,ESFF,ESFF2a);
    splRA(&dgY,&d2gY,ESdgY,ESdgY2a);
    d2gY	=dgY+a*d2gY;
    dgY		*=a;
    d2gY	*=2.*dgY;
    dgY		*=dgY;

    splRA2(&b,&ba,&baa,ESsb,ESsb2a);
    splRA2(rc,rca,rcaa,rcT,rcT2a);
    rc[0]	+=ESaR0[0];
    splRA2(rs,rsa,rsaa,rsT,rsT2a);
    rs[0]	+=ESaZ0[0];
    ki	=0;
    iK	=1;
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      splRA2(rc+k,rca+k,rcaa+k,rcT+ki,rcT2a+ki);
      rc[k]	*=2.;
      rst[k]	=-rc[k]*k;
      rca[k]	*=2.;
      rcaa[k]	*=2.;
      splRA2(rs+k,rsa+k,rsaa+k,rsT+ki,rsT2a+ki);
      rs[k]	*=2.;
      rct[k]	=rs[k]*k;
      rsa[k]	*=2.;
      rsaa[k]	*=2.;
      if(rct[k] != 0. || rca[k] != 0. || rcaa[k] != 0. ||
	 rst[k] != 0. || rsa[k] != 0. || rsaa[k] != 0.){
	iK	=k+1;
      }
    }
    /* PEST gq */
    Q	=0.;
    Qa	=0.;
    for(j=0; j < ESNp; j++){
      cs	=EScs1[j];
      sn	=ESsn1[j];
      r		=rc[0];
      ra	=rca[0];
      raa	=rcaa[0];
      rt	=0.;
      rat	=0.;
      z		=rs[0]+b*sn;
      za	=rsa[0]+ba*sn;
      zat	=ba*cs;
      zaa	=rsaa[0]+baa*sn;
      zt	=b*cs;
      kj	=0;
      for(k=1; k < iK; k++){
	kj	+=j;
	if(kj >= ESNp) kj -=ESNp;
	cs	=EScs1[kj];
	sn	=ESsn1[kj];
	r	+=rc[k]*cs+rs[k]*sn;
	ra	+=rca[k]*cs+rsa[k]*sn;
	rat	+=k*(rsa[k]*cs-rca[k]*sn);
	raa	+=rcaa[k]*cs+rsaa[k]*sn;
	rt	+=rct[k]*cs+rst[k]*sn;
      }
      gQ[j]	=(ra*zt-rt*za)/r;
      gQa[j]	=(raa*zt+ra*zat-rat*za-rt*zaa-ra*gQ[j])/r;
      Q		+=gQ[j];
      Qa	+=gQa[j];
    }
    Qa	/=Q;
    Q	/=ESNp;
    s	=1./Q;
    for(j=0; j < ESNp; j++){
      gQ[j]	*=s;
      gQa[j]	=gQa[j]*s-gQ[j]*Qa;
    }
    gQ[j]	=gQ[0];
    gQa[j]	=gQa[0];
    ESP2F(gQc,gQs,gQ,ESFp);
    ESP2F(gQac,gQas,gQa,ESFp);
    gQc[0]	=0.;
    gQac[0]	=0.;
    for(k=1; k < ESFp1; k++){
      gQc[k]	*=2.;
      gQs[k]	*=2.;
      gQac[k]	*=2.;
      gQas[k]	*=2.;
    }
    t		=0.;
    for(j=0; j < Np; j++){
      q		=dt*j;
      kj	=0;
      do{
	s	=t-q;
	s1	=1.;
	qa	=0.;
	for(k=1; k < ESFp1; k++){
	  cs	=cos(k*t);
	  sn	=sin(k*t);
	  s1	+=gQc[k]*cs+gQs[k]*sn;
	  s	+=(gQc[k]*sn+gQs[k]*(1.-cs))/k;
	  qa	+=(gQac[k]*sn+gQas[k]*(1.-cs))/k;
	}
	t	-=s/s1;
	kj++;
      }while(kj < 10 && fabs(s) > 1e-8);
      cs	=cos(t);
      sn	=sin(t);
      r		=rc[0];
      ra	=rca[0];
      rt	=0.;
      rat	=0.;
      raa	=rcaa[0];
      rtt	=0.;
      rttt	=0.;
      raat	=0.;
      ratt	=0.;

      sz[ji]	=rs[0]+b*sn;
      za	=rsa[0]+ba*sn;
      zt	=b*cs;
      zat	=ba*cs;
      zaa	=rsaa[0]+baa*sn;
      ztt	=-b*sn;
      zttt	=-b*cs;
      zaat	=baa*cs;
      zatt	=-ba*sn;
      for(k=1; k < iK; k++){
	s	=k*t;
	cs	=cos(s);
	sn	=sin(s);
	D	=k*k;
	s	=rc[k]*cs+rs[k]*sn;
	r	+=s;
	rtt	-=D*s;
	s	=rca[k]*cs+rsa[k]*sn;
	ra	+=s;
	ratt	-=D*s;
	s	=rct[k]*cs+rst[k]*sn;
	rt	+=s;
	rttt	-=D*s;
	rat	+=k*(rsa[k]*cs-rca[k]*sn);
	raa	+=rcaa[k]*cs+rsaa[k]*sn;
	raat	+=k*(rsaa[k]*cs-rcaa[k]*sn);
      }
      D		=ra*zt-rt*za;
      Da	=raa*zt+ra*zat-rat*za-rt*zaa;
      Dt	=rat*zt+ra*ztt-rtt*za-rt*zat;
      Dat	=raat*zt+raa*ztt+ra*zatt-ratt*za-rtt*zaa-rt*zaat;
      Dtt	=ratt*zt+ra*zttt-rttt*za+2.*(rat*ztt-rtt*zat)-rt*zatt;

      s		=1./r;
      L		=D*s;
      La	=(Da-L*ra)*s;
      Lt	=(Dt-L*rt)*s;
      Lat	=(Dat-La*rt-L*rat-Lt*ra)*s;
      Ltt	=(Dtt-L*rtt-2.*Lt*rt)*s;

      s		=1./Q;
      qt	=L*s;
      qat	=(La-L*Qa)*s;
      qtt	=Lt*s;

      tq	=1./qt;
      ta	=-qa*tq;
      taq	=-(qat+ta*qtt)*tq*tq;

      sza[ji]	=za+zt*ta;
      szt[ji]	=zt*tq;
      szat[ji]	=(zat+ztt*ta)*tq+zt*taq;

      sr[ji]	=r;
      sra[ji]	=ra+rt*ta;
      srt[ji]	=rt*tq;
      srat[ji]	=(rat+rtt*ta)*tq+rt*taq;

      L		=r*D;
      Ba	=ra*D+r*Da;
      Bt	=rt*D+r*Dt;
      Bat	=rat*D+rt*Da+ra*Dt+r*Dat;
      Btt	=rtt*D+2.*rt*Dt+r*Dtt;

      s		=1./L;
      L		=s*s;
      La	=-2.*Ba*L*s;
      Lt	=-2.*Bt*L*s;
      Lat	=-(2.*Bat*L+3.*La*Bt)*s;
      Ltt	=-(2.*Btt*L+3.*Lt*Bt)*s;

      B		=rt*rt+zt*zt;
      Ba	=2.*(rt*rat+zt*zat);
      Bt	=2.*(rt*rtt+zt*ztt);
      Bat	=2.*(rat*rtt+rt*ratt+zat*ztt+zt*zatt);
      Btt	=2.*(rtt*rtt+rt*rttt+ztt*ztt+zt*zttt);

      K		=B*L;
      Ka	=Ba*L+B*La;
      Kt	=Bt*L+B*Lt;
      Kat	=Bat*L+Ba*Lt+Bt*La+B*Lat;
      Ktt	=Btt*L+2.*Bt*Lt+B*Ltt;

      B		=K*dgY;
      Ba	=Ka*dgY+K*d2gY;
      Bt	=Kt*dgY;
      Bat	=Kat*dgY+Kt*d2gY;
      Btt	=Ktt*dgY;

      s	=1./r;
      L		=pF*s*s;
      La	=(dpF*s-2.*L*ra)*s;
      Lt	=-2.*L*rt*s;
      B		+=L;
      Ba	+=La;
      Bt	+=Lt;
      Bat	-=(2.*La*rt+2.*L*rat+Lt*ra)*s;
      Btt	-=(3.*Lt*rt+2.*L*rtt)*s;

      B		=sqrt(B);
      s		=0.5/B;
      Ba	*=s;
      Bt	*=s;
      Bat	=Bat*s-Ba*Bt/B;
      Btt	=Btt*s-Bt*Bt/B;

      aB[ji]	=B;
      aBa[ji]	=Ba+Bt*ta;
      aBt[ji]	=Bt*tq;
      aBat[ji]	=(Bat+Btt*ta)*tq+Bt*taq;
      ji++;
    }
    j	=ji-Np;
    sr[ji]	=sr[j];
    sra[ji]	=sra[j];
    srt[ji]	=srt[j];
    srat[ji]	=srat[j];

    sz[ji]	=sz[j];
    sza[ji]	=sza[j];
    szt[ji]	=szt[j];
    szat[ji]	=szat[j];

    aB[ji]	=aB[j];
    aBa[ji]	=aBa[j];
    aBt[ji]	=aBt[j];
    aBat[ji]	=aBat[j];
    ji++;
  }
  ESWriteESI("PEST",sr,sz,aB,NULL,Na,Np);
#ifdef H
  Esc2ESI("PEST",sr,sz,aB,NULL,Na,Np);
#endif
  free(aB);
  free(sz);
  free(sr);
  return(0);
}

/* Generator of PEST coordinates */
int ES2PEST(double *R, double *Z, int Na,int Np)
{
  int Na1,Np1;
  char FNm[24];

  int i,j,ji,k,kj,ki,iK;
  double a,da,t,dt;
  double s,s1,s2,cs,sn;

  double r,z,rt,zt,ra,za,B[Np+1];
  double rc[ESFp1],rs[ESFp1],rca[ESFp1],rsa[ESFp1],rct[ESFp1],rst[ESFp1],b,ba;
  double Q,q;
  double gQ[ESNp1],gQc[ESFp1],gQs[ESFp1];

  Na1	=Na+1;
  Np1	=Np+1;
  da	=(ESsa[ESNa]-ESsa[0])/Na;
  dt	=c2gp/Np;

  r	=ESaR0[0];
  z	=ESaZ0[0];
  for(ji=0; ji < Np1; ji++){
    R[ji]	=r;
    Z[ji]	=z;
  }
  for(i=1; i < Na1; i++){
    a	=da*i;
    ESSetSplA(a);
    splRA(&b,&ba,ESsb,ESsb2a);
    splRA(rc,rca,rcT,rcT2a);
    rc[0]	+=ESaR0[0];
    splRA(rs,rsa,rsT,rsT2a);
    rs[0]	+=ESaZ0[0];
    ki	=0;
    iK	=1;
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      splRA(rc+k,rca+k,rcT+ki,rcT2a+ki);
      rc[k]	*=2.;
      rst[k]	=-rc[k]*k;
      rca[k]	*=2.;
      splRA(rs+k,rsa+k,rsT+ki,rsT2a+ki);
      rs[k]	*=2.;
      rct[k]	=rs[k]*k;
      rsa[k]	*=2.;
      if(rct[k] != 0. || rca[k] != 0. || rst[k] != 0. || rsa[k] != 0.){
	iK	=k+1;
      }
    }
    Q	=0.;
    for(j=0; j < ESNp; j++){
      cs	=EScs1[j];
      sn	=ESsn1[j];
      r		=rc[0];
      ra	=rca[0];
      rt	=0.;
      z		=rs[0]+b*sn;
      za	=rsa[0]+ba*sn;
      zt	=b*cs;
      kj	=0;
      for(k=1; k < iK; k++){
	kj	+=j;
	if(kj >= ESNp) kj -=ESNp;
	cs	=EScs1[kj];
	sn	=ESsn1[kj];
	r	+=rc[k]*cs+rs[k]*sn;
	ra	+=rca[k]*cs+rsa[k]*sn;
	rt	+=rct[k]*cs+rst[k]*sn;
      }
      gQ[j]	=(ra*zt-rt*za)/r;
      Q		+=gQ[j];
    }
    Q	=ESNp/Q;
    for(j=0; j < ESNp; j++){
      gQ[j]	=gQ[j]*Q-1.;
    }
    gQ[j]	=gQ[0];
    ESP2F(gQc,gQs,gQ,ESFp);
    for(k=1; k < ESFp1; k++){
      gQc[k]	*=2.;
      gQs[k]	*=2.;
    }
    t	=0.;
    for(j=0; j < Np; j++){
      q	=dt*j;
      kj	=0;
      do{
	s	=t-q;
	s1	=1.;
	for(k=1; k < ESFp1; k++){
	  cs	=cos(k*t);
	  sn	=sin(k*t);
	  s1	+=gQc[k]*cs+gQs[k]*sn;
	  s	+=(gQc[k]*sn+gQs[k]*(1.-cs))/k;
	}
	t	-=s/s1;
	kj++;
      }while(kj < 10 && fabs(s) > 1e-8);
      if(kj == 10){
	printf("i=%2d j=%d q=%10.3e - No convergence in ES2PEST()%c\n"
	       ,i,j,q,7);
      }
      cs	=cos(t);
      sn	=sin(t);
      r		=rc[0]+rc[1]*cs+rs[1]*sn;
      z		=rs[0]+b*sn;
      for(k=2; k < ESFp1; k++){
	s	=k*t;
	cs	=cos(s);
	sn	=sin(s);
	r	+=rc[k]*cs+rs[k]*sn;
      }
      R[ji]	=r;
      Z[ji]	=z;
      ji++;
    }
    R[ji]	=R[ji-Np];
    Z[ji]	=Z[ji-Np];
    ji++;
  }
  return(0);
}

int ESBasicFunctionsHamada(int Na, int Np)
{
  char* SNm="ESBasicFunctionsHamada";
  int Np1,Na1;
  int i,j,ji,k,ki,kj,iK;
  double r,ra,rt,rat,raa,rtt,rttt,raat,ratt;
  double z,za,zt,zat,zaa,ztt,zttt,zaat,zatt;
  double rc[ESFp1],rs[ESFp1];
  double rct[ESFp1],rst[ESFp1],rca[ESFp1],rsa[ESFp1],rcaa[ESFp1],rsaa[ESFp1];
  double b,ba,baa,dgY,d2gY,pF,dpF;
  double D,Da,Dt,Dat,Dtt,K,Ka,Kt,Kat,Ktt;
  double a,da,t,dt,s,s1,cs,sn;
  double Q,Qa,L0,L0a,L,La,Lt,Lat,Ltt;
  double gQ[ESNp1],gQc[ESFp1],gQs[ESFp1];
  double gQa[ESNp1],gQac[ESFp1],gQas[ESFp1];
  double H,Ha,Ht,Hat,Htt;

  double *sr,*sra,*srt,*srat;
  double *sz,*sza,*szt,*szat;
  double *aB,*aBa,*aBt,*aBat;
  double *gH,*gHa,*gHt,*gHat;

  double B,Ba,Bt,Bat,Btt;
  double ta,tq,taq;
  double q,qa,qt,qat,qtt,qatt,qttt;

  Np1	=Np+1;
  Na1	=Na+1;
  da	=(ESsa[ESNa]-ESsa[0])/Na;
  dt	=c2gp/Np;

  i	=Np1*Na1;
  sr	=(double*)malloc(4*i*sizeof(double));
  if(sr == NULL){
    printf("Failure in malloc for sr in %s()\n",SNm);
    return(1);
  }
  sra	=sr	+i;
  srt	=sra	+i;
  srat	=srt	+i;
  
  sz	=(double*)malloc(4*i*sizeof(double));
  if(sz == NULL){
    printf("Failure in malloc for sz in %s()\n",SNm);
    return(1);
  }
  sza	=sz	+i;
  szt	=sza	+i;
  szat	=szt	+i;
  
  aB	=(double*)malloc(4*i*sizeof(double));
  if(aB == NULL){
    printf("Failure in malloc for aB in %s()\n",SNm);
    return(1);
  }
  aBa	=aB	+i;
  aBt	=aBa	+i;
  aBat	=aBt	+i;
  
  gH	=(double*)malloc(4*i*sizeof(double));
  if(gH == NULL){
    printf("Failure in malloc for gH in %s()\n",SNm);
    return(1);
  }
  gHa	=gH	+i;
  gHt	=gHa	+i;
  gHat	=gHt	+i;

  /* a=0 surface */
  ki	=ESNa1;
  rca[1]=2.*rcT1a[ki];
  rsa[1]=2.*rsT1a[ki];
  ba	=ESsb1a[0];
  pF	=ESaF[0];
  r	=ESaR0[0];
  K	=1./r;
  ji	=0;
  for(j=0; j < Np1; j++){
    t		=dt*j;
    cs		=cos(t);
    sn		=sin(t);

    ra		=rca[1]*cs+rsa[1]*sn;
    rt		=rsa[1]*cs-rca[1]*sn;
    rat		=rsa[1]*cs-rca[1]*sn;

    za		=ba*sn;
    zt		=ba*cs;
    zat		=ba*cs;

    sr[ji]	=r;
    sra[ji]	=ra;
    srt[ji]	=0.;
    srat[ji]	=rat;
    sz[ji]	=ESaZ0[0];
    sza[ji]	=za;
    szt[j]	=0.;
    szat[j]	=zat;
    
    Ka	=-ra*K*K;
    Kt	=0.;
    Kat	=-rat*K*K;
    
    aB[ji]	=pF*K;
    aBa[ji]	=pF*Ka;
    aBt[ji]	=0.;
    aBat[ji]	=pF*Kat;

    gH[ji]	=0.;
    gHa[ji]	=-2.*ra/r;
    gHt[ji]	=0.;
    gHat[ji]	=-2.*rat/r;
    ji++;
  }

  for(i=1; i < Na1; i++){
    a	=da*i;
    ESSetSplA(a);
    splRA(&L0,&L0a,ESLc,ESLc2);
    L0	=1./L0;
    L0a	*=-L0*L0;
    splRA(&pF,&dpF,ESFF,ESFF2a);
    splRA(&dgY,&d2gY,ESdgY,ESdgY2a);
    d2gY	=dgY+a*d2gY;
    dgY		*=a;
    d2gY	*=2.*dgY;
    dgY		*=dgY;

    splRA2(&b,&ba,&baa,ESsb,ESsb2a);
    splRA2(rc,rca,rcaa,rcT,rcT2a);
    rc[0]	+=ESaR0[0];
    splRA2(rs,rsa,rsaa,rsT,rsT2a);
    rs[0]	+=ESaZ0[0];
    ki	=0;
    iK	=1;
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      splRA2(rc+k,rca+k,rcaa+k,rcT+ki,rcT2a+ki);
      rc[k]	*=2.;
      rst[k]	=-rc[k]*k;
      rca[k]	*=2.;
      rcaa[k]	*=2.;
      splRA2(rs+k,rsa+k,rsaa+k,rsT+ki,rsT2a+ki);
      rs[k]	*=2.;
      rct[k]	=rs[k]*k;
      rsa[k]	*=2.;
      rsaa[k]	*=2.;
      if(rct[k] != 0. || rca[k] != 0. || rcaa[k] != 0. ||
	 rst[k] != 0. || rsa[k] != 0. || rsaa[k] != 0.){
	iK	=k+1;
      }
    }
    /* Hamada gq */
    Q	=0.;
    Qa	=0.;
    for(j=0; j < ESNp; j++){
      cs	=EScs1[j];
      sn	=ESsn1[j];
      r		=rc[0];
      ra	=rca[0];
      raa	=rcaa[0];
      rt	=0.;
      rat	=0.;
      z		=rs[0]+b*sn;
      za	=rsa[0]+ba*sn;
      zat	=ba*cs;
      zaa	=rsaa[0]+baa*sn;
      zt	=b*cs;
      kj	=0;
      for(k=1; k < iK; k++){
	kj	+=j;
	if(kj >= ESNp) kj-=ESNp;
	cs	=EScs1[kj];
	sn	=ESsn1[kj];
	r	+=rc[k]*cs+rs[k]*sn;
	ra	+=rca[k]*cs+rsa[k]*sn;
	rat	+=k*(rsa[k]*cs-rca[k]*sn);
	raa	+=rcaa[k]*cs+rsaa[k]*sn;
	rt	+=rct[k]*cs+rst[k]*sn;
      }
      D		=ra*zt-rt*za;
      Da	=raa*zt+ra*zat-rat*za-rt*zaa;
      gQ[j]	=r*D;
      gQa[j]	=ra*D+r*Da;
      Q		+=gQ[j];
      Qa	+=gQa[j];
    }
    Qa	/=Q;
    Q	/=ESNp;
    s	=1./Q;
    for(j=0; j < ESNp; j++){
      gQ[j]	*=s;
      gQa[j]	=gQa[j]*s-gQ[j]*Qa;
    }
    gQ[j]	=gQ[0];
    gQa[j]	=gQa[0];
    ESP2F(gQc,gQs,gQ,ESFp);
    ESP2F(gQac,gQas,gQa,ESFp);
    gQc[0]	=0.;
    gQac[0]	=0.;
    for(k=1; k < ESFp1; k++){
      gQc[k]	*=2.;
      gQs[k]	*=2.;
      gQac[k]	*=2.;
      gQas[k]	*=2.;
    }
    t		=0.;
    for(j=0; j < Np; j++){
      q		=dt*j;
      kj	=0;
      do{
	s	=t-q;
	s1	=1.;
	qa	=0.;
	for(k=1; k < ESFp1; k++){
	  cs	=cos(k*t);
	  sn	=sin(k*t);
	  s1	+=gQc[k]*cs+gQs[k]*sn;
	  s	+=(gQc[k]*sn+gQs[k]*(1.-cs))/k;
	  qa	+=(gQac[k]*sn+gQas[k]*(1.-cs))/k;
	}
	t	-=s/s1;
	kj++;
      }while(kj < 10 && fabs(s) > 1e-8);
      cs	=cos(t);
      sn	=sin(t);
      r		=rc[0];
      ra	=rca[0];
      rt	=0.;
      rat	=0.;
      raa	=rcaa[0];
      rtt	=0.;
      rttt	=0.;
      raat	=0.;
      ratt	=0.;

      sz[ji]	=rs[0]+b*sn;
      za	=rsa[0]+ba*sn;
      zt	=b*cs;
      zat	=ba*cs;
      zaa	=rsaa[0]+baa*sn;
      ztt	=-b*sn;
      zttt	=-b*cs;
      zaat	=baa*cs;
      zatt	=-ba*sn;
      for(k=1; k < iK; k++){
	s	=k*t;
	cs	=cos(s);
	sn	=sin(s);
	D	=k*k;
	s	=rc[k]*cs+rs[k]*sn;
	r	+=s;
	rtt	-=D*s;
	s	=rca[k]*cs+rsa[k]*sn;
	ra	+=s;
	ratt	-=D*s;
	s	=rct[k]*cs+rst[k]*sn;
	rt	+=s;
	rttt	-=D*s;
	rat	+=k*(rsa[k]*cs-rca[k]*sn);
	raa	+=rcaa[k]*cs+rsaa[k]*sn;
	raat	+=k*(rsaa[k]*cs-rcaa[k]*sn);
      }
      D		=ra*zt-rt*za;
      Da	=raa*zt+ra*zat-rat*za-rt*zaa;
      Dt	=rat*zt+ra*ztt-rtt*za-rt*zat;
      Dat	=raat*zt+raa*ztt+ra*zatt-ratt*za-rtt*zaa-rt*zaat;
      Dtt	=ratt*zt+ra*zttt-rttt*za+2.*(rat*ztt-rtt*zat)-rt*zatt;

      L		=1./(r*D);
      La	=ra*D+r*Da;
      Lt	=rt*D+r*Dt;
      Lat	=rat*D+rt*Da+ra*Dt+r*Dat;
      Ltt	=rtt*D+2.*rt*Dt+r*Dtt;

      K		=(rt*rt+zt*zt)*L;
      Ka	=(2.*(rt*rat+zt*zat)-K*La)*L;
      Kt	=(2.*(rt*rtt+zt*ztt)-K*Lt)*L; 
      Kat	=(2.*(rat*rtt+rt*ratt+zat*ztt+zt*zatt)-Ka*Lt
		  -K*(rat*D+rt*Da+ra*Dt+r*Dat)
		  -Kt*La)*L;
      Ktt	=(2.*(rtt*rtt+rt*rttt+ztt*ztt+zt*zttt-Kt*Lt)
		  -K*(rtt*D+rt*Dt+ra*Dt+r*Dtt))*L;

      B		=K*dgY*L;
      Ba	=(Ka*dgY+K*d2gY-B*La)*L;
      Bt	=(Kt*dgY-B*Lt)*L;
      Bat	=(Kat*dgY+Kt*d2gY-Ba*Lt-B*Lat-Bt*La)*L;
      Btt	=(Ktt*dgY-2.*Bt*Lt-B*Ltt)*L;

      s	=1./r;
      L		=pF*s*s;
      La	=(dpF*s-2.*L*ra)*s;
      Lt	=-2.*L*rt*s;
      B		+=L;
      Ba	+=La;
      Bt	+=Lt;
      Bat	-=(2.*La*rt+2.*L*rat+Lt*ra)*s;
      Btt	-=(3.*Lt*rt+2.*L*rtt)*s;

      s	=1./r;
      L		=D*s;
      La	=(Da-L*ra)*s;
      Lt	=(Dt-L*rt)*s;
      Lat	=(Dat-La*rt-L*rat-Lt*ra)*s;
      Ltt	=(Dtt-L*rtt-2.*Lt*rt)*s;

      s		=1./Q;
      qt	=r*D*s;
      qat	=(ra*D+r*Da)*s-qt*Qa;
      qtt	=(rt*D+r*Dt)*s;
      qatt	=(rat*D+ra*Dt+rt*Da+r*Dat)*s-qtt*Qa;
      qttt	=(rtt*D+2.*rt*Dt+r*Dtt)*s;

      tq	=1./qt;
      ta	=-qa*tq;
      taq	=-(qat+ta*qtt)*tq*tq;

      sza[ji]	=za+zt*ta;
      szt[ji]	=zt*tq;
      szat[ji]	=(zat+ztt*ta)*tq+zt*taq;

      sr[ji]	=r;
      sra[ji]	=ra+rt*ta;
      srt[ji]	=rt*tq;
      srat[ji]	=(rat+rtt*ta)*tq+rt*taq;

      B		=sqrt(B);
      s		=0.5/B;
      Ba	*=s;
      Bt	*=s;
      Bat	=Bat*s-Ba*Bt/B;
      Btt	=Btt*s-Bt*Bt/B;

      aB[ji]	=B;
      aBa[ji]	=Ba+Bt*ta;
      aBt[ji]	=Bt*tq;
      aBat[ji]	=(Bat+Btt*ta)*tq+Bt*taq;

      s	=1./qt;
      H	=L*L0*s-1.;
      Ha=(La*L0+L*L0a-H*qat)*s;
      Ht=(Lt*L0-H*qtt)*s;
      Hat=(Lat*L0+Lt*L0a-Ht*qat-H*qatt-Ha*qtt)*s;
      Htt=(Ltt*L0-2.*Ht*qtt-H*qttt)*s;

      gH[ji]	=H;
      gHa[ji]	=Ha+Ht*ta;
      gHt[ji]	=Ht*tq;
      gHat[ji]	=(Hat+Htt*ta)*tq+Ht*taq;

      ji++;
    }
    j	=ji-Np;
    sr[ji]	=sr[j];
    sra[ji]	=sra[j];
    srt[ji]	=srt[j];
    srat[ji]	=srat[j];

    sz[ji]	=sz[j];
    sza[ji]	=sza[j];
    szt[ji]	=szt[j];
    szat[ji]	=szat[j];

    aB[ji]	=aB[j];
    aBa[ji]	=aBa[j];
    aBt[ji]	=aBt[j];
    aBat[ji]	=aBat[j];
    gH[ji]	=gH[j];
    gHa[ji]	=gHa[j];
    gHt[ji]	=gHt[j];
    gHat[ji]	=gHat[j];
    ji++;
  }
#ifdef H
  ESWriteESI("Hamada",sr,sz,aB,gH,Na,Np);
#endif
  Esc2ESI("Hamada",sr,sz,aB,gH,Na,Np);
  free(gH);
  free(aB);
  free(sz);
  free(sr);
  return(0);
}

/* Generator of Hamada coordinates */
int ES2Hamada(double *R,double *Z,int Na,int Np)
{
  int Na1,Np1;
  int i,j,ji,k,kj,ki,iK;

  double r,ra,rt;
  double z,za,zt;
  double rc[ESFp1],rs[ESFp1];
  double rct[ESFp1],rst[ESFp1],rca[ESFp1],rsa[ESFp1];
  double b,ba,dgY,d2gY,pF,dpF;
  double D,K;
  double a,da,t,dt,s,s1,cs,sn;
  double Q,Qa;
  double gQ[ESNp1],gQc[ESFp1],gQs[ESFp1];
  double q;

  Na1	=Na+1;
  Np1	=Np+1;
  da	=(ESsa[ESNa]-ESsa[0])/Na;
  dt	=c2gp/Np;

  r	=ESaR0[0];
  z	=ESaZ0[0];
  ji	=0;
  for(j=0; j < Np1; j++){
    R[ji]	=r;
    Z[ji]	=z;
    ji++;
  }
  for(i=1; i < Na1; i++){
    a	=da*i;
    ESSetSplA(a);
    splRA(&b,&ba,ESsb,ESsb2a);
    splRA(rc,rca,rcT,rcT2a);
    rc[0]	+=ESaR0[0];
    splRA(rs,rsa,rsT,rsT2a);
    rs[0]	+=ESaZ0[0];
    ki	=0;
    iK	=1;
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      splRA(rc+k,rca+k,rcT+ki,rcT2a+ki);
      rc[k]	*=2.;
      rst[k]	=-rc[k]*k;
      rca[k]	*=2.;
      splRA(rs+k,rsa+k,rsT+ki,rsT2a+ki);
      rs[k]	*=2.;
      rct[k]	=rs[k]*k;
      rsa[k]	*=2.;
      if(rct[k] != 0. || rca[k] != 0. || rst[k] != 0. || rsa[k] != 0.){
	iK	=k+1;
      }
    }
    /* Hamada gq */
    Q	=0.;
    for(j=0; j < ESNp; j++){
      cs	=EScs1[j];
      sn	=ESsn1[j];
      r		=rc[0];
      ra	=rca[0];
      rt	=0.;
      z		=rs[0]+b*sn;
      za	=rsa[0]+ba*sn;
      zt	=b*cs;
      kj	=0;
      for(k=1; k < iK; k++){
	kj	+=j;
	if(kj >= ESNp) kj-=ESNp;
	cs	=EScs1[kj];
	sn	=ESsn1[kj];
	r	+=rc[k]*cs+rs[k]*sn;
	ra	+=rca[k]*cs+rsa[k]*sn;
	rt	+=rct[k]*cs+rst[k]*sn;
      }
      D		=ra*zt-rt*za;
      gQ[j]	=r*D;
      Q		+=gQ[j];
    }
    Q	/=ESNp;
    s	=1./Q;
    for(j=0; j < ESNp; j++){
      gQ[j]	*=s;
    }
    gQ[j]	=gQ[0];
    ESP2F(gQc,gQs,gQ,ESFp);
    gQc[0]	=0.;
    for(k=1; k < ESFp1; k++){
      gQc[k]	*=2.;
      gQs[k]	*=2.;
    }
    t		=0.;
    for(j=0; j < Np; j++){
      q		=dt*j;
      kj	=0;
      do{
	s	=t-q;
	s1	=1.;
	for(k=1; k < ESFp1; k++){
	  cs	=cos(k*t);
	  sn	=sin(k*t);
	  s1	+=gQc[k]*cs+gQs[k]*sn;
	  s	+=(gQc[k]*sn+gQs[k]*(1.-cs))/k;
	}
	t	-=s/s1;
	kj++;
      }while(kj < 10 && fabs(s) > 1e-8);
      cs	=cos(t);
      sn	=sin(t);
      r		=rc[0]+rc[1]*cs+rs[1]*sn;
      Z[ji]	=rs[0]+b*sn;
      for(k=2; k < iK; k++){
	s	=k*t;
	cs	=cos(s);
	sn	=sin(s);
	r	+=rc[k]*cs+rs[k]*sn;
      }
      R[ji]	=r;
      ji++;
    }
    j	=ji-Np;
    R[ji]	=R[j];
    Z[ji]	=Z[j];
    ji++;
  }
  return(0);
}

int ESBasicFunctionsBoozer(int Na, int Np)
{
  char *SNm="ESBasicFunctionsBoozer";
  int Np1,Na1;
  int i,j,ji,k,ki,kj,iK;
  double r,ra,rt,rat,raa,rtt,rttt,raat,ratt;
  double z,za,zt,zat,zaa,ztt,zttt,zaat,zatt;
  double rc[ESFp1],rs[ESFp1];
  double rct[ESFp1],rst[ESFp1],rca[ESFp1],rsa[ESFp1],rcaa[ESFp1],rsaa[ESFp1];
  double b,ba,baa,dgY,d2gY,pF,dpF;
  double D,Da,Dt,Dat,Dtt,K,Ka,Kt,Kat,Ktt;
  double a,da,t,dt,s,s1,cs,sn;
  double Q,Qa,L0,L0a,L,La,Lt,Lat,Ltt;
  double gQ[ESNp1],gQc[ESFp1],gQs[ESFp1];
  double gQa[ESNp1],gQac[ESFp1],gQas[ESFp1];
  double H,Ha,Ht,Hat,Htt;

  double *sr,*sra,*srt,*srat;
  double *sz,*sza,*szt,*szat;
  double *aB,*aBa,*aBt,*aBat;
  double *gH,*gHa,*gHt,*gHat;

  double B,Ba,Bt,Bat,Btt;
  double ta,tq,taq;
  double q,qa,qt,qat,qtt,qatt,qttt;

  Np1	=Np+1;
  Na1	=Na+1;
  da	=(ESsa[ESNa]-ESsa[0])/Na;
  dt	=c2gp/Np;

  i	=Np1*Na1;
  sr	=(double*)malloc(4*i*sizeof(double));
  if(sr == NULL){
    printf("Failure in malloc for sr in %s()\n",SNm);
    return(1);
  }
  sra	=sr	+i;
  srt	=sra	+i;
  srat	=srt	+i;
  
  sz	=(double*)malloc(4*i*sizeof(double));
  if(sz == NULL){
    printf("Failure in malloc for sz in %s()\n",SNm);
    return(1);
  }
  sza	=sz	+i;
  szt	=sza	+i;
  szat	=szt	+i;
  
  aB	=(double*)malloc(4*i*sizeof(double));
  if(aB == NULL){
    printf("Failure in malloc for aB in %s()\n",SNm);
    return(1);
  }
  aBa	=aB	+i;
  aBt	=aBa	+i;
  aBat	=aBt	+i;
  
  gH	=(double*)malloc(4*i*sizeof(double));
  if(gH == NULL){
    printf("Failure in malloc for gH in %s()\n",SNm);
    return(1);
  }
  gHa	=gH	+i;
  gHt	=gHa	+i;
  gHat	=gHt	+i;

  /* a=0 surface */
  ki	=ESNa1;
  rca[1]=2.*rcT1a[ki];
  rsa[1]=2.*rsT1a[ki];
  ba	=ESsb1a[0];
  pF	=ESaF[0];
  r	=ESaR0[0];
  K	=1./r;
  ji	=0;
  for(j=0; j < Np1; j++){
    t		=dt*j;
    cs		=cos(t);
    sn		=sin(t);

    ra		=rca[1]*cs+rsa[1]*sn;
    rt		=rsa[1]*cs-rca[1]*sn;
    rat		=rsa[1]*cs-rca[1]*sn;

    za		=ba*sn;
    zt		=ba*cs;
    zat		=ba*cs;

    sr[ji]	=r;
    sra[ji]	=ra;
    srt[ji]	=0.;
    srat[ji]	=rat;
    sz[ji]	=ESaZ0[0];
    sza[ji]	=za;
    szt[j]	=0.;
    szat[j]	=zat;
    
    Ka	=-ra*K*K;
    Kt	=0.;
    Kat	=-rat*K*K;
    
    aB[ji]	=pF*K;
    aBa[ji]	=pF*Ka;
    aBt[ji]	=0.;
    aBat[ji]	=pF*Kat;

    gH[ji]	=0.;
    gHa[ji]	=0.;
    gHt[ji]	=0.;
    gHat[ji]	=0.;
    ji++;
  }

  for(i=1; i < Na1; i++){
    a	=da*i;
    ESSetSplA(a);
    splRA(&L0,&L0a,ESLc,ESLc2);
    L0	=1./L0;
    L0a	*=-L0*L0;
    splRA(&pF,&dpF,ESFF,ESFF2a);
    splRA(&dgY,&d2gY,ESdgY,ESdgY2a);
    d2gY	=dgY+a*d2gY;
    dgY		*=a;
    d2gY	*=2.*dgY;
    dgY		*=dgY;

    splRA2(&b,&ba,&baa,ESsb,ESsb2a);
    splRA2(rc,rca,rcaa,rcT,rcT2a);
    rc[0]	+=ESaR0[0];
    splRA2(rs,rsa,rsaa,rsT,rsT2a);
    rs[0]	+=ESaZ0[0];
    ki	=0;
    iK	=1;
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      splRA2(rc+k,rca+k,rcaa+k,rcT+ki,rcT2a+ki);
      rc[k]	*=2.;
      rst[k]	=-rc[k]*k;
      rca[k]	*=2.;
      rcaa[k]	*=2.;
      splRA2(rs+k,rsa+k,rsaa+k,rsT+ki,rsT2a+ki);
      rs[k]	*=2.;
      rct[k]	=rs[k]*k;
      rsa[k]	*=2.;
      rsaa[k]	*=2.;
      if(rct[k] != 0. || rca[k] != 0. || rcaa[k] != 0. ||
	 rst[k] != 0. || rsa[k] != 0. || rsaa[k] != 0.){
	iK	=k+1;
      }
    }
    /* Boozer gq */
    Q	=0.;
    Qa	=0.;
    for(j=0; j < ESNp; j++){
      cs	=EScs1[j];
      sn	=ESsn1[j];
      r		=rc[0];
      ra	=rca[0];
      raa	=rcaa[0];
      rt	=0.;
      rat	=0.;
      z		=rs[0]+b*sn;
      za	=rsa[0]+ba*sn;
      zat	=ba*cs;
      zaa	=rsaa[0]+baa*sn;
      zt	=b*cs;
      kj	=0;
      for(k=1; k < iK; k++){
	kj	+=j;
	if(kj >= ESNp) kj-=ESNp;
	cs	=EScs1[kj];
	sn	=ESsn1[kj];
	r	+=rc[k]*cs+rs[k]*sn;
	ra	+=rca[k]*cs+rsa[k]*sn;
	rat	+=k*(rsa[k]*cs-rca[k]*sn);
	raa	+=rcaa[k]*cs+rsaa[k]*sn;
	rt	+=rct[k]*cs+rst[k]*sn;
      }
      D		=ra*zt-rt*za;
      Da	=raa*zt+ra*zat-rat*za-rt*zaa;
      s		=1./(r*D);
      K		=(rt*rt+zt*zt)*s;
      Ka	=(2.*(rt*rat+zt*zat)-K*(ra*D+r*Da))*s;
      L		=D/r;
      La	=(Da-L*ra)/r;
      gQ[j]	=K*dgY+pF*L;
      gQa[j]	=Ka*dgY+K*d2gY+pF*La+dpF*L;
      Q	+=gQ[j];
      Qa	+=gQa[j];
    }
    Qa	/=Q;
    Q	/=ESNp;
    s	=1./Q;
    for(j=0; j < ESNp; j++){
      gQ[j]	*=s;
      gQa[j]	=gQa[j]*s-gQ[j]*Qa;
    }
    gQ[j]	=gQ[0];
    gQa[j]	=gQa[0];
    ESP2F(gQc,gQs,gQ,ESFp);
    ESP2F(gQac,gQas,gQa,ESFp);
    gQc[0]	=0.;
    gQac[0]	=0.;
    for(k=1; k < ESFp1; k++){
      gQc[k]	*=2.;
      gQs[k]	*=2.;
      gQac[k]	*=2.;
      gQas[k]	*=2.;
    }
    t		=0.;
    for(j=0; j < Np; j++){
      q		=dt*j;
      kj	=0;
      do{
	s	=t-q;
	s1	=1.;
	qa	=0.;
	for(k=1; k < ESFp1; k++){
	  cs	=cos(k*t);
	  sn	=sin(k*t);
	  s1	+=gQc[k]*cs+gQs[k]*sn;
	  s	+=(gQc[k]*sn+gQs[k]*(1.-cs))/k;
	  qa	+=(gQac[k]*sn+gQas[k]*(1.-cs))/k;
	}
	t	-=s/s1;
	kj++;
      }while(kj < 10 && fabs(s) > 1e-8);
      cs	=cos(t);
      sn	=sin(t);
      r		=rc[0];
      ra	=rca[0];
      rt	=0.;
      rat	=0.;
      raa	=rcaa[0];
      rtt	=0.;
      rttt	=0.;
      raat	=0.;
      ratt	=0.;

      sz[ji]	=rs[0]+b*sn;
      za	=rsa[0]+ba*sn;
      zt	=b*cs;
      zat	=ba*cs;
      zaa	=rsaa[0]+baa*sn;
      ztt	=-b*sn;
      zttt	=-b*cs;
      zaat	=baa*cs;
      zatt	=-ba*sn;
      for(k=1; k < iK; k++){
	s	=k*t;
	cs	=cos(s);
	sn	=sin(s);
	D	=k*k;
	s	=rc[k]*cs+rs[k]*sn;
	r	+=s;
	rtt	-=D*s;
	s	=rca[k]*cs+rsa[k]*sn;
	ra	+=s;
	ratt	-=D*s;
	s	=rct[k]*cs+rst[k]*sn;
	rt	+=s;
	rttt	-=D*s;
	rat	+=k*(rsa[k]*cs-rca[k]*sn);
	raa	+=rcaa[k]*cs+rsaa[k]*sn;
	raat	+=k*(rsaa[k]*cs-rcaa[k]*sn);
      }
      D		=ra*zt-rt*za;
      Da	=raa*zt+ra*zat-rat*za-rt*zaa;
      Dt	=rat*zt+ra*ztt-rtt*za-rt*zat;
      Dat	=raat*zt+raa*ztt+ra*zatt-ratt*za-rtt*zaa-rt*zaat;
      Dtt	=ratt*zt+ra*zttt-rttt*za+2.*(rat*ztt-rtt*zat)-rt*zatt;

      L		=1./(r*D);
      La	=ra*D+r*Da;
      Lt	=rt*D+r*Dt;
      Lat	=rat*D+rt*Da+ra*Dt+r*Dat;
      Ltt	=rtt*D+2.*rt*Dt+r*Dtt;

      K		=(rt*rt+zt*zt)*L;
      Ka	=(2.*(rt*rat+zt*zat)-K*La)*L;
      Kt	=(2.*(rt*rtt+zt*ztt)-K*Lt)*L; 
      Kat	=(2.*(rat*rtt+rt*ratt+zat*ztt+zt*zatt)-Ka*Lt
		  -K*(rat*D+rt*Da+ra*Dt+r*Dat)
		  -Kt*La)*L;
      Ktt	=(2.*(rtt*rtt+rt*rttt+ztt*ztt+zt*zttt-Kt*Lt)
		  -K*(rtt*D+rt*Dt+ra*Dt+r*Dtt))*L;

      B		=K*dgY*L;
      Ba	=(Ka*dgY+K*d2gY-B*La)*L;
      Bt	=(Kt*dgY-B*Lt)*L;
      Bat	=(Kat*dgY+Kt*d2gY-Ba*Lt-B*Lat-Bt*La)*L;
      Btt	=(Ktt*dgY-2.*Bt*Lt-B*Ltt)*L;

      s	=1./r;
      L		=pF*s*s;
      La	=(dpF*s-2.*L*ra)*s;
      Lt	=-2.*L*rt*s;
      B		+=L;
      Ba	+=La;
      Bt	+=Lt;
      Bat	-=(2.*La*rt+2.*L*rat+Lt*ra)*s;
      Btt	-=(3.*Lt*rt+2.*L*rtt)*s;

      s	=1./r;
      L		=D*s;
      La	=(Da-L*ra)*s;
      Lt	=(Dt-L*rt)*s;
      Lat	=(Dat-La*rt-L*rat-Lt*ra)*s;
      Ltt	=(Dtt-L*rtt-2.*Lt*rt)*s;
      s		=1./Q;
      qt	=(K*dgY+pF*L)*s;
      qat	=(Ka*dgY+K*d2gY+pF*La+dpF*L)*s-qt*Qa;
      qtt	=(Kt*dgY+pF*Lt)*s;
      qatt	=(Kat*dgY+Kt*d2gY+pF*Lat+dpF*Lt)*s-qtt*Qa;
      qttt	=(Ktt*dgY+pF*Ltt)*s;

      tq	=1./qt;
      ta	=-qa*tq;
      taq	=-(qat+ta*qtt)*tq*tq;

      sza[ji]	=za+zt*ta;
      szt[ji]	=zt*tq;
      szat[ji]	=(zat+ztt*ta)*tq+zt*taq;

      sr[ji]	=r;
      sra[ji]	=ra+rt*ta;
      srt[ji]	=rt*tq;
      srat[ji]	=(rat+rtt*ta)*tq+rt*taq;

      B		=sqrt(B);
      s		=0.5/B;
      Ba	*=s;
      Bt	*=s;
      Bat	=Bat*s-Ba*Bt/B;
      Btt	=Btt*s-Bt*Bt/B;

      aB[ji]	=B;
      aBa[ji]	=Ba+Bt*ta;
      aBt[ji]	=Bt*tq;
      aBat[ji]	=(Bat+Btt*ta)*tq+Bt*taq;

      s	=1./qt;
      H	=L*L0*s-1.;
      Ha=(La*L0+L*L0a-H*qat)*s;
      Ht=(Lt*L0-H*qtt)*s;
      Hat=(Lat*L0+Lt*L0a-Ht*qat-H*qatt-Ha*qtt)*s;
      Htt=(Ltt*L0-2.*Ht*qtt-H*qttt)*s;

      gH[ji]	=H;
      gHa[ji]	=Ha+Ht*ta;
      gHt[ji]	=Ht*tq;
      gHat[ji]	=(Hat+Htt*ta)*tq+Ht*taq;

      ji++;
    }
    j	=ji-Np;
    sr[ji]	=sr[j];
    sra[ji]	=sra[j];
    srt[ji]	=srt[j];
    srat[ji]	=srat[j];

    sz[ji]	=sz[j];
    sza[ji]	=sza[j];
    szt[ji]	=szt[j];
    szat[ji]	=szat[j];

    aB[ji]	=aB[j];
    aBa[ji]	=aBa[j];
    aBt[ji]	=aBt[j];
    aBat[ji]	=aBat[j];
    gH[ji]	=gH[j];
    gHa[ji]	=gHa[j];
    gHt[ji]	=gHt[j];
    gHat[ji]	=gHat[j];
    ji++;
  }
#ifdef H
  ESWriteESI("Boozer",sr,sz,aB,gH,Na,Np);
#endif
  Esc2ESI("Boozer",sr,sz,aB,gH,Na,Np);
  free(gH);
  free(aB);
  free(sz);
  free(sr);
  return(0);
}

/* Generator of Boozer coordinates */
int ES2Boozer(double *R,double *Z,int Na,int Np)
{
  int Na1,Np1;
  int i,j,ji,k,kj,ki,iK;

  double r,ra,rt;
  double z,za,zt;
  double rc[ESFp1],rs[ESFp1];
  double rct[ESFp1],rst[ESFp1],rca[ESFp1],rsa[ESFp1];
  double b,ba,dgY,d2gY,pF,dpF;
  double D,K;
  double a,da,t,dt,s,s1,cs,sn;
  double Q,Qa;
  double gQ[ESNp1],gQc[ESFp1],gQs[ESFp1];
  double q;

  Na1	=Na+1;
  Np1	=Np+1;
  da	=(ESsa[ESNa]-ESsa[0])/Na;
  dt	=c2gp/Np;

  r	=ESaR0[0];
  z	=ESaZ0[0];
  ji	=0;
  for(j=0; j < Np1; j++){
    R[ji]	=r;
    Z[ji]	=z;
    ji++;
  }
  for(i=1; i < Na1; i++){
    a	=da*i;
    ESSetSplA(a);
    splRA(&pF,&dpF,ESFF,ESFF2a);
    splRA(&dgY,&d2gY,ESdgY,ESdgY2a);
    d2gY	=dgY+a*d2gY;
    dgY		*=a;
    d2gY	*=2.*dgY;
    dgY		*=dgY;

    splRA(&b,&ba,ESsb,ESsb2a);
    splRA(rc,rca,rcT,rcT2a);
    rc[0]	+=ESaR0[0];
    splRA(rs,rsa,rsT,rsT2a);
    rs[0]	+=ESaZ0[0];
    ki	=0;
    iK	=1;
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      splRA(rc+k,rca+k,rcT+ki,rcT2a+ki);
      rc[k]	*=2.;
      rst[k]	=-rc[k]*k;
      rca[k]	*=2.;
      splRA(rs+k,rsa+k,rsT+ki,rsT2a+ki);
      rs[k]	*=2.;
      rct[k]	=rs[k]*k;
      rsa[k]	*=2.;
      if(rct[k] != 0. || rca[k] != 0. || rst[k] != 0. || rsa[k] != 0.){
	iK	=k+1;
      }
    }
    /* Boozer gq */
    Q	=0.;
    for(j=0; j < ESNp; j++){
      cs	=EScs1[j];
      sn	=ESsn1[j];
      r		=rc[0];
      ra	=rca[0];
      rt	=0.;
      z		=rs[0]+b*sn;
      za	=rsa[0]+ba*sn;
      zt	=b*cs;
      kj	=0;
      for(k=1; k < iK; k++){
	kj	+=j;
	if(kj >= ESNp) kj-=ESNp;
	cs	=EScs1[kj];
	sn	=ESsn1[kj];
	r	+=rc[k]*cs+rs[k]*sn;
	ra	+=rca[k]*cs+rsa[k]*sn;
	rt	+=rct[k]*cs+rst[k]*sn;
      }
      D		=ra*zt-rt*za;
      s		=1./(r*D);
      K		=(rt*rt+zt*zt)*s;
      gQ[j]	=K*dgY+pF*D/r;
      Q	+=gQ[j];
    }
    Q	/=ESNp;
    s	=1./Q;
    for(j=0; j < ESNp; j++){
      gQ[j]	*=s;
    }
    gQ[j]	=gQ[0];
    ESP2F(gQc,gQs,gQ,ESFp);
    gQc[0]	=0.;
    for(k=1; k < ESFp1; k++){
      gQc[k]	*=2.;
      gQs[k]	*=2.;
    }
    t		=0.;
    for(j=0; j < Np; j++){
      q		=dt*j;
      kj	=0;
      do{
	s	=t-q;
	s1	=1.;
	for(k=1; k < ESFp1; k++){
	  cs	=cos(k*t);
	  sn	=sin(k*t);
	  s1	+=gQc[k]*cs+gQs[k]*sn;
	  s	+=(gQc[k]*sn+gQs[k]*(1.-cs))/k;
	}
	t	-=s/s1;
	kj++;
      }while(kj < 10 && fabs(s) > 1e-8);
      cs	=cos(t);      sn	=sin(t);
      r		=rc[0]+rc[1]*cs+rs[1]*sn;
      Z[ji]	=rs[0]+b*sn;
      for(k=2; k < iK; k++){
	s	=k*t;
	cs	=cos(s);
	sn	=sin(s);
	r	+=rc[k]*cs+rs[k]*sn;
      }
      R[ji]	=r;
      ji++;
    }
    j	=ji-Np;
    R[ji]	=R[j];
    Z[ji]	=Z[j];
    ji++;
  }
  return(0);
}

int ESBasicFunctionsHat(int Na, int Np)
{
  char *SNm="ESBasicFunctionsHat";
  int Np1,Na1;
  int i,j,ji,k,ki,kj,iK;
  double r,ra,rt,rat,raa,rtt,rttt,raat,ratt;
  double z,za,zt,zat,zaa,ztt,zttt,zaat,zatt;
  double rc[ESFp1],rs[ESFp1];
  double rct[ESFp1],rst[ESFp1],rca[ESFp1],rsa[ESFp1],rcaa[ESFp1],rsaa[ESFp1];
  double b,ba,baa,dgY,d2gY,pF,dpF;
  double D,Da,Dt,Dat,Dtt,K,Ka,Kt,Kat,Ktt;
  double a,da,t,dt,s,s1,cs,sn;
  double L0,L0a,L,La,Lt,Lat,Ltt,V,Va,Vt,Vat,Vtt;
  double aV[ESNp1],aVc[ESFp1],aVs[ESFp1];
  double aVa[ESNp1],aVac[ESFp1],aVas[ESFp1];
  double K0,K0a,V0,V0a,T,Ta,P,Pa;
  double gs,gsa;
  double H,Ha,Ht,Hat,Htt;

  double *sr,*sra,*srt,*srat;
  double *sz,*sza,*szt,*szat;
  double *aB,*aBa,*aBt,*aBat;
  double *gH,*gHa,*gHt,*gHat;

  double B,Ba,Bt,Bat,Btt;
  double ta,tq,taq;
  double q,qa,qt,qat,qtt,qatt,qttt;

  Np1	=Np+1;
  Na1	=Na+1;
  da	=(ESsa[ESNa]-ESsa[0])/Na;
  dt	=c2gp/Np;

  i	=Np1*Na1;
  sr	=(double*)malloc(4*i*sizeof(double));
  if(sr == NULL){
    printf("Failure in malloc for sr in %s()\n",SNm);
    return(1);
  }
  sra	=sr	+i;
  srt	=sra	+i;
  srat	=srt	+i;
  
  sz	=(double*)malloc(4*i*sizeof(double));
  if(sz == NULL){
    printf("Failure in malloc for sz in %s()\n",SNm);
    return(1);
  }
  sza	=sz	+i;
  szt	=sza	+i;
  szat	=szt	+i;
  
  aB	=(double*)malloc(4*i*sizeof(double));
  if(aB == NULL){
    printf("Failure in malloc for aB in %s()\n",SNm);
    return(1);
  }
  aBa	=aB	+i;
  aBt	=aBa	+i;
  aBat	=aBt	+i;
  
  gH	=(double*)malloc(4*i*sizeof(double));
  if(gH == NULL){
    printf("Failure in malloc for gH in %s()\n",SNm);
    return(1);
  }
  gHa	=gH	+i;
  gHt	=gHa	+i;
  gHat	=gHt	+i;

  /* a=0 surface */
  ki	=ESNa1;
  rca[1]=2.*rcT1a[ki];
  rsa[1]=2.*rsT1a[ki];

  ba	=ESsb1a[0];
  pF	=ESaF[0];
  r	=ESaR0[0];
  K	=1./r;

  L0	=1./ESLc[0];
  P	=ESPs[0]*r*r;
  T	=ESaT[0];

  V0	=2.*T/(L0*(T+P));
  rc[1]	=(ESLc1[ki]+ESVc1[ki])*V0;
  rs[1]	=(ESLs1[ki]+ESVs1[ki])*V0;

  ji	=0;
  for(j=0; j < Np1; j++){
    t		=dt*j;
    cs		=cos(t);
    sn		=sin(t);

    ra		=rca[1]*cs+rsa[1]*sn;
    rt		=rsa[1]*cs-rca[1]*sn;
    rat		=rsa[1]*cs-rca[1]*sn;

    za		=ba*sn;
    zt		=ba*cs;
    zat		=ba*cs;

    sr[ji]	=r;
    sra[ji]	=ra;
    srt[ji]	=0.;
    srat[ji]	=rat;
    sz[ji]	=ESaZ0[0];
    sza[ji]	=za;
    szt[j]	=0.;
    szat[j]	=zat;
    
    Ka	=-ra*K*K;
    Kt	=0.;
    Kat	=-rat*K*K;
    
    aB[ji]	=pF*K;
    aBa[ji]	=pF*Ka;
    aBt[ji]	=0.;
    aBat[ji]	=pF*Kat;

    gH[ji]	=0.;
    gHa[ji]	=0.;
    gHt[ji]	=0.;
    gHat[ji]	=0.;
    ji++;
  }

  for(i=1; i < Na1; i++){
    a	=da*i;
    ESSetSplA(a);
    splRA(&pF,&dpF,ESFF,ESFF2a);
    splRA(&dgY,&d2gY,ESdgY,ESdgY2a);
    d2gY	=dgY+a*d2gY;
    dgY		*=a;
    d2gY	*=2.*dgY;
    dgY		*=dgY;
    splRA2(&P,&Pa,&K,ESPs,ESPs2a);
    splRA2(&T,&Ta,&Ka,ESaT,ESaT2a);
    if(P == 0. && T == 0.){
      P	=Pa;
      T	=Ta;
      Pa=K;
      Ta=Ka;
    }
    gs	=dgY/pF;
    gsa	=(d2gY-gs*dpF)/pF;

    splRA2(&b,&ba,&baa,ESsb,ESsb2a);
    splRA2(rc,rca,rcaa,rcT,rcT2a);
    rc[0]	+=ESaR0[0];
    splRA2(rs,rsa,rsaa,rsT,rsT2a);
    rs[0]	+=ESaZ0[0];
    ki	=0;
    iK	=1;
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      splRA2(rc+k,rca+k,rcaa+k,rcT+ki,rcT2a+ki);
      rc[k]	*=2.;
      rst[k]	=-rc[k]*k;
      rca[k]	*=2.;
      rcaa[k]	*=2.;
      splRA2(rs+k,rsa+k,rsaa+k,rsT+ki,rsT2a+ki);
      rs[k]	*=2.;
      rct[k]	=rs[k]*k;
      rsa[k]	*=2.;
      rsaa[k]	*=2.;
      if(rct[k] != 0. || rca[k] != 0. || rcaa[k] != 0. ||
	 rst[k] != 0. || rsa[k] != 0. || rsaa[k] != 0.){
	iK	=k+1;
      }
    }

    /* Hat gqt */
    L0	=0.;
    L0a	=0.;
    V0	=0.;
    V0a	=0.;
    for(j=0; j < ESNp; j++){
      cs	=EScs1[j];
      sn	=ESsn1[j];
      r		=rc[0];
      ra	=rca[0];
      raa	=rcaa[0];
      rt	=0.;
      rat	=0.;
      z		=rs[0]+b*sn;
      za	=rsa[0]+ba*sn;
      zat	=ba*cs;
      zaa	=rsaa[0]+baa*sn;
      zt	=b*cs;
      kj	=0;
      for(k=1; k < iK; k++){
	kj	+=j;
	if(kj >= ESNp) kj-=ESNp;
	cs	=EScs1[kj];
	sn	=ESsn1[kj];
	r	+=rc[k]*cs+rs[k]*sn;
	ra	+=rca[k]*cs+rsa[k]*sn;
	rat	+=k*(rsa[k]*cs-rca[k]*sn);
	raa	+=rcaa[k]*cs+rsaa[k]*sn;
	rt	+=rct[k]*cs+rst[k]*sn;
      }
      D		=ra*zt-rt*za;
      Da	=raa*zt+ra*zat-rat*za-rt*zaa;
      V		=D*r;
      Va	=Da*r+D*ra;
      s		=1./V;
      K		=(rt*rt+zt*zt)*s;
      Ka	=(2.*(rt*rat+zt*zat)-K*(ra*D+r*Da))*s;

      L		=D/r;
      La	=(Da-L*ra)/r;
      L0	+=L;
      L0a	+=La;
      
      aV[j]	=P*V+T*(L+gs*K);
      aVa[j]	=Pa*V+P*Va+Ta*(L+gs*K)+T*(La+gsa*K+gs*Ka);
      V0	+=aV[j];
      V0a	+=aVa[j];
    }
    D	=1./ESNp;
    V0	*=D;
    V0a	*=D;
    L0	*=D;
    L0a	*=D;

    if(V0 == 0) return(1);
    s	=1./V0;
    for(j=0; j < ESNp; j++){
      aV[j]	*=s;
      aVa[j]	=(aVa[j]-aV[j]*V0a)*s;
    }
    aV[j]	=aV[0];
    aVa[j]	=aVa[0];
    ESP2F(aVc,aVs,aV,ESFp);
    ESP2F(aVac,aVas,aVa,ESFp);
    aVc[0]	=0.;
    aVac[0]	=0.;
    for(k=1; k < ESFp1; k++){
      aVc[k]	*=2.;
      aVs[k]	*=2.;
      aVac[k]	*=2.;
      aVas[k]	*=2.;
    }
    V0a	/=V0;
    t		=0.;
    for(j=0; j < Np; j++){
      q		=dt*j;
      kj	=0;
      do{
	s	=t-q;
	s1	=1.;
	qa	=0.;
	for(k=1; k < ESFp1; k++){
	  cs	=cos(k*t);
	  sn	=sin(k*t);
	  s1	+=aVc[k]*cs+aVs[k]*sn;
	  s	+=(aVc[k]*sn+aVs[k]*(1.-cs))/k;
	  qa	+=(aVac[k]*sn+aVas[k]*(1.-cs))/k;
	}
	s	/=s1;
	t	-=s;
	kj++;
      }while(kj < 10 && fabs(s) > 1e-8);
      cs	=cos(t);
      sn	=sin(t);
      r		=rc[0];
      ra	=rca[0];
      rt	=0.;
      rat	=0.;
      raa	=rcaa[0];
      rtt	=0.;
      rttt	=0.;
      raat	=0.;
      ratt	=0.;

      sz[ji]	=rs[0]+b*sn;
      za	=rsa[0]+ba*sn;
      zt	=b*cs;
      zat	=ba*cs;
      zaa	=rsaa[0]+baa*sn;
      ztt	=-b*sn;
      zttt	=-b*cs;
      zaat	=baa*cs;
      zatt	=-ba*sn;
      for(k=1; k < iK; k++){
	s	=k*t;
	cs	=cos(s);
	sn	=sin(s);
	D	=k*k;
	s	=rc[k]*cs+rs[k]*sn;
	r	+=s;
	rtt	-=D*s;
	s	=rca[k]*cs+rsa[k]*sn;
	ra	+=s;
	ratt	-=D*s;
	s	=rct[k]*cs+rst[k]*sn;
	rt	+=s;
	rttt	-=D*s;
	rat	+=k*(rsa[k]*cs-rca[k]*sn);
	raa	+=rcaa[k]*cs+rsaa[k]*sn;
	raat	+=k*(rsaa[k]*cs-rcaa[k]*sn);
      }
      D		=ra*zt-rt*za;
      Da	=raa*zt+ra*zat-rat*za-rt*zaa;
      Dt	=rat*zt+ra*ztt-rtt*za-rt*zat;
      Dat	=raat*zt+raa*ztt+ra*zatt-ratt*za-rtt*zaa-rt*zaat;
      Dtt	=ratt*zt+ra*zttt-rttt*za+2.*(rat*ztt-rtt*zat)-rt*zatt;

      V		=D*r;
      Va	=Da*r+D*ra;
      Vt	=Dt*r+D*rt;
      Vat	=Dat*r+Dt*ra+Da*rt+D*rat;
      Vtt	=Dtt*r+2.*Dt*rt+D*rtt;

      L		=1./V;
      K		=(rt*rt+zt*zt)*L;
      Ka	=(2.*(rt*rat+zt*zat)-K*Va)*L;
      Kt	=(2.*(rt*rtt+zt*ztt)-K*Vt)*L; 
      Kat	=(2.*(rat*rtt+rt*ratt+zat*ztt+zt*zatt)-Ka*Vt
		  -K*(rat*D+rt*Da+ra*Dt+r*Dat)
		  -Kt*Va)*L;
      Ktt	=(2.*(rtt*rtt+rt*rttt+ztt*ztt+zt*zttt-Kt*Vt)
		  -K*(rtt*D+rt*Dt+ra*Dt+r*Dtt))*L;

      B		=K*dgY*L;
      Ba	=(Ka*dgY+K*d2gY-B*Va)*L;
      Bt	=(Kt*dgY-B*Vt)*L;
      Bat	=(Kat*dgY+Kt*d2gY-Ba*Vt-B*Vat-Bt*Va)*L;
      Btt	=(Ktt*dgY-2.*Bt*Vt-B*Vtt)*L;

      s		=1./r;
      L		=pF*s*s;
      La	=(dpF*s-2.*L*ra)*s;
      Lt	=-2.*L*rt*s;
      B		+=L;
      Ba	+=La;
      Bt	+=Lt;
      Bat	-=(2.*La*rt+2.*L*rat+Lt*ra)*s;
      Btt	-=(3.*Lt*rt+2.*L*rtt)*s;

      s	=1./r;
      L		=D*s;
      La	=(Da-L*ra)*s;
      Lt	=(Dt-L*rt)*s;
      Lat	=(Dat-La*rt-L*rat-Lt*ra)*s;
      Ltt	=(Dtt-L*rtt-2.*Lt*rt)*s;

      s		=1./V0;
      qt	=(P*V+T*(L+gs*K))*s;
      qat	=(Pa*V+P*Va+Ta*(L+gs*K)+T*(La+gsa*K+gs*Ka))*s-qt*V0a;
      qtt	=(P*Vt+T*(Lt+gs*Kt))*s;
      qatt	=(Pa*Vt+P*Vat+Ta*(Lt+gs*Kt)+T*(Lat+gsa*Kt+gs*Kat))*s-qtt*V0a;
      qttt	=(P*Vtt+T*(Ltt+gs*Ktt))*s;

      tq	=1./qt;
      ta	=-qa*tq;
      taq	=-(qat+ta*qtt)*tq*tq;

      sza[ji]	=za+zt*ta;
      szt[ji]	=zt*tq;
      szat[ji]	=(zat+ztt*ta)*tq+zt*taq;

      sr[ji]	=r;
      sra[ji]	=ra+rt*ta;
      srt[ji]	=rt*tq;
      srat[ji]	=(rat+rtt*ta)*tq+rt*taq;

      B		=sqrt(B);
      s		=0.5/B;
      Ba	*=s;
      Bt	*=s;
      Bat	=Bat*s-Ba*Bt/B;
      Btt	=Btt*s-Bt*Bt/B;

      aB[ji]	=B;
      aBa[ji]	=Ba+Bt*ta;
      aBt[ji]	=Bt*tq;
      aBat[ji]	=(Bat+Btt*ta)*tq+Bt*taq;

      s	=1./qt;
      H	=L*L0*s-1.;
      Ha=(La*L0+L*L0a-H*qat)*s;
      Ht=(Lt*L0-H*qtt)*s;
      Hat=(Lat*L0+Lt*L0a-Ht*qat-H*qatt-Ha*qtt)*s;
      Htt=(Ltt*L0-2.*Ht*qtt-H*qttt)*s;

      gH[ji]	=H;
      gHa[ji]	=Ha+Ht*ta;
      gHt[ji]	=Ht*tq;
      gHat[ji]	=(Hat+Htt*ta)*tq+Ht*taq;

      ji++;
    }
    j	=ji-Np;
    sr[ji]	=sr[j];
    sra[ji]	=sra[j];
    srt[ji]	=srt[j];
    srat[ji]	=srat[j];

    sz[ji]	=sz[j];
    sza[ji]	=sza[j];
    szt[ji]	=szt[j];
    szat[ji]	=szat[j];

    aB[ji]	=aB[j];
    aBa[ji]	=aBa[j];
    aBt[ji]	=aBt[j];
    aBat[ji]	=aBat[j];
    gH[ji]	=gH[j];
    gHa[ji]	=gHa[j];
    gHt[ji]	=gHt[j];
    gHat[ji]	=gHat[j];
    ji++;
  }
#ifdef H
  ESWriteESI("Canonical",sr,sz,aB,gH,Na,Np);
#endif
  Esc2ESI("Boozer",sr,sz,aB,gH,Na,Np);
  free(gH);
  free(aB);
  free(sz);
  free(sr);
  return(0);
}

/* Generator of Hat coordinates */
int ES2Hat(double *R,double *Z,int Na,int Np)
{
  int Na1,Np1;
  int i,j,ji,k,kj,ki,iK;

  double r,ra,rt;
  double z,za,zt;
  double rc[ESFp1],rs[ESFp1];
  double rct[ESFp1],rst[ESFp1],rca[ESFp1],rsa[ESFp1];
  double b,ba,dgY,d2gY,pF;
  double D,K,L,V,gs;
  double a,da,t,dt,s,s1,cs,sn;
  double L0,V0,T,P;
  double aV[ESNp1],aVc[ESFp1],aVs[ESFp1];
  double q;

  Na1	=Na+1;
  Np1	=Np+1;
  da	=(ESsa[ESNa]-ESsa[0])/Na;
  dt	=c2gp/Np;

  r	=ESaR0[0];
  z	=ESaZ0[0];
  ji	=0;
  for(j=0; j < Np1; j++){
    R[ji]	=r;
    Z[ji]	=z;
    ji++;
  }
  for(i=1; i < Na1; i++){
    a	=da*i;
    ESSetSplA(a);
    splRA(&pF,NULL,ESFF,ESFF2a);
    splRA(&dgY,NULL,ESdgY,ESdgY2a);
    dgY		*=a;
    dgY		*=dgY;
    gs	=dgY/pF;
    splRA(&P,&V,ESPs,ESPs2a);
    splRA(&T,&K,ESaT,ESaT2a);
    if(P == 0. && T == 0.){
      P	=V;
      T	=K;
    }
    splRA(&b,&ba,ESsb,ESsb2a);
    splRA(rc,rca,rcT,rcT2a);
    rc[0]	+=ESaR0[0];
    splRA(rs,rsa,rsT,rsT2a);
    rs[0]	+=ESaZ0[0];
    ki	=0;
    iK	=1;
    for(k=1; k < ESFp1; k++){
      ki	+=ESNa1;
      splRA(rc+k,rca+k,rcT+ki,rcT2a+ki);
      rc[k]	*=2.;
      rst[k]	=-rc[k]*k;
      rca[k]	*=2.;
      splRA(rs+k,rsa+k,rsT+ki,rsT2a+ki);
      rs[k]	*=2.;
      rct[k]	=rs[k]*k;
      rsa[k]	*=2.;
      if(rct[k] != 0. || rca[k] != 0. || rst[k] != 0. || rsa[k] != 0.){
	iK	=k+1;
      }
    }
    /* Hat gqt */
    L0	=0.;
    V0	=0.;
    for(j=0; j < ESNp; j++){
      cs	=EScs1[j];
      sn	=ESsn1[j];
      r		=rc[0];
      ra	=rca[0];
      rt	=0.;
      z		=rs[0]+b*sn;
      za	=rsa[0]+ba*sn;
      zt	=b*cs;
      kj	=0;
      for(k=1; k < iK; k++){
	kj	+=j;
	if(kj >= ESNp) kj-=ESNp;
	cs	=EScs1[kj];
	sn	=ESsn1[kj];
	r	+=rc[k]*cs+rs[k]*sn;
	ra	+=rca[k]*cs+rsa[k]*sn;
	rt	+=rct[k]*cs+rst[k]*sn;
      }
      D		=ra*zt-rt*za;
      V		=r*D;
      K		=(rt*rt+zt*zt)/V;
      L		=D/r;
      L0	+=L;
      aV[j]	=P*V+T*(L+gs*K);
      V0	+=aV[j];
    }
    D	=1./ESNp;
    V0	*=D;
    L0	*=D;
    if(V0 == 0.) return(0);
    s	=1./V0;
    for(j=0; j < ESNp; j++) aV[j]*=s;
    aV[j]	=aV[0];
    ESP2F(aVc,aVs,aV,ESFp);
    aVc[0]	=0.;
    for(k=1; k < ESFp1; k++){
      aVc[k]	*=2.;
      aVs[k]	*=2.;
    }
    t		=0.;
    for(j=0; j < Np; j++){
      q		=dt*j;
      kj	=0;
      do{
	s	=t-q;
	s1	=1.;
	for(k=1; k < ESFp1; k++){
	  cs	=cos(k*t);
	  sn	=sin(k*t);
	  s1	+=aVc[k]*cs+aVs[k]*sn;
	  s	+=(aVc[k]*sn+aVs[k]*(1.-cs))/k;
	}
	s	/=s1;
	t	-=s;
	kj++;
      }while(kj < 10 && fabs(s) > 1e-8);
      cs	=cos(t);
      sn	=sin(t);
      r		=rc[0]+rc[1]*cs+rs[1]*sn;
      Z[ji]	=rs[0]+b*sn;
      for(k=2; k < iK; k++){
	s	=k*t;
	cs	=cos(s);
	sn	=sin(s);
	r	+=rc[k]*cs+rs[k]*sn;
      }
      R[ji]	=r;
      ji++;
    }
    j	=ji-Np;
    R[ji]	=R[j];
    Z[ji]	=Z[j];
    ji++;
  }
  return(0);
}

int ESFirstBiSpline()
{
  Spi0	=0;
  Spi1	=0;
  Spj0	=0;
  Spj1	=0;
  ha	=ESsa[1]-ESsa[0];
  rha	=1./ha;
  hq	=ESgt[1];
  rhq	=1./hq;
  return(0);
}
double YYa,YYx,YYX,YYq,YYy,YYY;
double XYa,XYx,XYX,XYq,XYy,XYY;
static double XSC1,XSC2,xSC1,xSC2;

int ESBiSplRAP(double *r,double *ra,double *rq
	       ,double *z,double *za,double *zq
	       ,double *B,double *Ba,double *Bq
	       ,double *gh,double *gha,double *ghq
	       ,double a,double gq)
{
  int j00,j01,j10,j11;
  double X,XX,x,xx,Y,YY,y,yy,YSC1,YSC2,ySC1,ySC2;
  double f0,f1,fq0,fq1;
  
  if(a < 0. && a > ESsa[ESNa]) return(1);
  Spi0	=a*rha;
  if(Spi0 >= ESNa) Spi0=ESNa-1;
  Spi1	=Spi0+1;
  x	=(a-ESsa[Spi0])*rha;
  X	=1.-x;

#ifdef H
  Spj1	=gq*cr2gp;
  if(gq > 0.) Spj1++;
  gq	=c2gp*Spj1-gq;
  Spj0	=gq*rhq;
  if(Spj0 >= ESNp){
    Spj0	-=ESNp;
    gq		-=c2gp;
  }
  if(Spj0 < 0){
    Spj0+=ESNp;
    gq	+=c2gp;
  }
  Spj1	=Spj0+1;
#endif
  Spj1	=gq*cr2gp;
  if(gq < 0.) Spj1--;
  gq	-=c2gp*Spj1;
  Spj0	=gq*rhq;
  if(Spj0 >= ESNp){
    Spj0	-=ESNp;
    gq	-=c2gp;
  }
  if(Spj0 < 0){
    Spj0	+=ESNp;
    gq	+=c2gp;
  }
  Spj1	=Spj0+1;
  Y	=(gq-ESgt[Spj0])*rhq;
  y	=1.-Y;
  Spj0	=ESNp-Spj1;
  Spj1	=Spj0+1;

  XYa	=a;
  XYx	=x;
  XYX	=X;

  XYq	=gq;
  XYy	=y;
  XYY	=Y;

  XX	=X*X;
  xx	=x*x;
  XSC1	=XX*(3.-2.*X);
  XSC2	=XX*x*ha;
  xSC1	=xx*(3.-2.*x);
  xSC2	=-xx*X*ha;

  YY	=Y*Y;
  yy	=y*y;
  YSC1	=YY*(3.-2.*Y);
  YSC2	=YY*y*hq;
  ySC1	=yy*(3.-2.*y);
  ySC2	=-yy*Y*hq;

  xx	=3.*x*X;
  X	-=xx;
  x	-=xx;
  xx	*=2.*rha;

  yy	=3.*y*Y;
  Y	-=yy;
  y	-=yy;
  yy	*=2.*rhq;

  j00	=ESNp1*Spi0+Spj0;
  j01	=j00+1;
  j10	=j00+ESNp1;
  j11	=j10+1;

  f0	=XSC1*ESsr [j00]+xSC1*ESsr [j10]+XSC2*ESsra [j00]+xSC2*ESsra [j10];
  f1	=XSC1*ESsr [j01]+xSC1*ESsr [j11]+XSC2*ESsra [j01]+xSC2*ESsra [j11];
  fq0	=XSC1*ESsrt[j00]+xSC1*ESsrt[j10]+XSC2*ESsrat[j00]+xSC2*ESsrat[j10];
  fq1	=XSC1*ESsrt[j01]+xSC1*ESsrt[j11]+XSC2*ESsrat[j01]+xSC2*ESsrat[j11];
  *r	=YSC1*f0+ySC1*f1+YSC2*fq0+ySC2*fq1;
  *rq	=-(yy*(f1-f0)+Y*fq0+y*fq1);

  f0	=XSC1*ESsz [j00]+xSC1*ESsz [j10]+XSC2*ESsza [j00]+xSC2*ESsza [j10];
  f1	=XSC1*ESsz [j01]+xSC1*ESsz [j11]+XSC2*ESsza [j01]+xSC2*ESsza [j11];
  fq0	=XSC1*ESszt[j00]+xSC1*ESszt[j10]+XSC2*ESszat[j00]+xSC2*ESszat[j10];
  fq1	=XSC1*ESszt[j01]+xSC1*ESszt[j11]+XSC2*ESszat[j01]+xSC2*ESszat[j11];
  *z	=YSC1*f0+ySC1*f1+YSC2*fq0+ySC2*fq1;
  *zq	=-(yy*(f1-f0)+Y*fq0+y*fq1);
#ifdef H
  out("?-?? IIII dd dd",j00,j01,j10,j11,ESsz[j00],ESsz[j01]
      ,ESsz[j10],ESsz[j11]);
#endif
  
  f0	=XSC1*ESaB [j00]+xSC1*ESaB [j10]+XSC2*ESaBa [j00]+xSC2*ESaBa [j10];
  f1	=XSC1*ESaB [j01]+xSC1*ESaB [j11]+XSC2*ESaBa [j01]+xSC2*ESaBa [j11];
  fq0	=XSC1*ESaBt[j00]+xSC1*ESaBt[j10]+XSC2*ESaBat[j00]+xSC2*ESaBat[j10];
  fq1	=XSC1*ESaBt[j01]+xSC1*ESaBt[j11]+XSC2*ESaBat[j01]+xSC2*ESaBat[j11];
  *B	=YSC1*f0+ySC1*f1+YSC2*fq0+ySC2*fq1;
  *Bq	=-(yy*(f1-f0)+Y*fq0+y*fq1);

  f0	=XSC1*ESgH [j00]+xSC1*ESgH [j10]+XSC2*ESgHa [j00]+xSC2*ESgHa [j10];
  f1	=XSC1*ESgH [j01]+xSC1*ESgH [j11]+XSC2*ESgHa [j01]+xSC2*ESgHa [j11];
  fq0	=XSC1*ESgHt[j00]+xSC1*ESgHt[j10]+XSC2*ESgHat[j00]+xSC2*ESgHat[j10];
  fq1	=XSC1*ESgHt[j01]+xSC1*ESgHt[j11]+XSC2*ESgHat[j01]+xSC2*ESgHat[j11];
  *gh	=YSC1*f0+ySC1*f1+YSC2*fq0+ySC2*fq1;
  *ghq	=-(yy*(f1-f0)+Y*fq0+y*fq1);

  f0	=xx*(ESsr [j10]-ESsr [j00])+X*ESsra [j00]+x*ESsra [j10];
  f1	=xx*(ESsr [j11]-ESsr [j01])+X*ESsra [j01]+x*ESsra [j11];
  fq0	=xx*(ESsrt[j10]-ESsrt[j00])+X*ESsrat[j00]+x*ESsrat[j10];
  fq1	=xx*(ESsrt[j11]-ESsrt[j01])+X*ESsrat[j01]+x*ESsrat[j11];
  *ra	=YSC1*f0+ySC1*f1+YSC2*fq0+ySC2*fq1;

  f0	=xx*(ESsz [j10]-ESsz [j00])+X*ESsza [j00]+x*ESsza [j10];
  f1	=xx*(ESsz [j11]-ESsz [j01])+X*ESsza [j01]+x*ESsza [j11];
  fq0	=xx*(ESszt[j10]-ESszt[j00])+X*ESszat[j00]+x*ESszat[j10];
  fq1	=xx*(ESszt[j11]-ESszt[j01])+X*ESszat[j01]+x*ESszat[j11];
  *za	=YSC1*f0+ySC1*f1+YSC2*fq0+ySC2*fq1;

  f0	=xx*(ESaB [j10]-ESaB [j00])+X*ESaBa [j00]+x*ESaBa [j10];
  f1	=xx*(ESaB [j11]-ESaB [j01])+X*ESaBa [j01]+x*ESaBa [j11];
  fq0	=xx*(ESaBt[j10]-ESaBt[j00])+X*ESaBat[j00]+x*ESaBat[j10];
  fq1	=xx*(ESaBt[j11]-ESaBt[j01])+X*ESaBat[j01]+x*ESaBat[j11];
  *Ba	=YSC1*f0+ySC1*f1+YSC2*fq0+ySC2*fq1;

  f0	=xx*(ESgH [j10]-ESgH [j00])+X*ESgHa [j00]+x*ESgHa [j10];
  f1	=xx*(ESgH [j11]-ESgH [j01])+X*ESgHa [j01]+x*ESgHa [j11];
  fq0	=xx*(ESgHt[j10]-ESgHt[j00])+X*ESgHat[j00]+x*ESgHat[j10];
  fq1	=xx*(ESgHt[j11]-ESgHt[j01])+X*ESgHat[j01]+x*ESgHat[j11];
  *gha	=YSC1*f0+ySC1*f1+YSC2*fq0+ySC2*fq1;
  return(0);
}

int ESBiSplRA(double *F,double *dgY,double *T,double *P)
{
  *F	=XSC1*ESaF[Spi0]+xSC1*ESaF[Spi1]+XSC2*ESaF1a[Spi0]+xSC2*ESaF1a[Spi1];
  *T	=XSC1*ESaT[Spi0]+xSC1*ESaT[Spi1]+XSC2*ESaT1a[Spi0]+xSC2*ESaT1a[Spi1];
  *P	=XSC1*ESPs[Spi0]+xSC1*ESPs[Spi1]+XSC2*ESPs1a[Spi0]+xSC2*ESPs1a[Spi1];
  *dgY=XSC1*ESdgY[Spi0]+xSC1*ESdgY[Spi1]+XSC2*ESdgY1a[Spi0]+xSC2*ESdgY1a[Spi1];
  
  return(0);
}

static int nst=0,ist=0;
static double Yr[1024],Yra[1024],Yrq[1024],Yz[1024],Yza[1024],Yzq[1024]
,YB[1024],YBa[1024],YBq[1024],Ya[1024],Yq[1024];

static double Rloss[65],Zloss[65];

void GCLagrangeEqOld(int *n,double *x,double *Y,double *dY)
{
  double r,ra,rq,z,za,zq,B,Ba,Bq,dgY,D;
  double a11,a12,a21,a22;
  double a,q;
  int i;

  a	=Y[1];
  q	=Y[2];
  if(ESBiSplRAP(&r,&ra,&rq,&z,&za,&zq,&B,&Ba,&Bq,&D,&D,&D,Y[1],Y[2])){
    istate	=3;
    return;
  }

  ESSetSplA(Y[1]);
  splRA(&dgY,NULL,ESdgY,ESdgY2a);
  dgY	*=Y[1];
  splRA(&a21,NULL,ESaF,ESaF2a);
  splRA(dY+2,NULL,ESPs,ESPs2a);
  splRA(dY+3,NULL,ESaT,ESaT2a);

  ESBiSplRA(&a21,&dgY,dY+3,dY+2);
  dgY	*=Y[1];
  dY[3]	*=Y[0];

  D	=za*rq-zq*ra;
  a12	=D*((a21+dY[3])/r+Y[0]*r*dY[2]);
  D	=-dgY/(r*D);
  a11	=(rq*rq+zq*zq)*D;
  D	*=ra*rq+za*zq;
  a22	=dgY*(1.+dY[3]/a21);
  r	=1./(a11*a22-a12*a21);
  a11	*=r;
  a12	*=r;
  a21	*=r;
  a22	*=r;
  D	*=r;

#ifdef H
  D	=0.;
???????
  r	=cgWc*Y[0]*B;
#endif
  r	=Y[0]*B;

  B	*=r;
#ifdef H
  r	=r*Y[0]+cgWc*gmP;
#endif
  r	=r*Y[0]+gmP;
  Ba	*=r;
  Bq	*=-r;

  dY[0]	=a22*Bq;
  dY[1]	=-a21*Bq;
  dY[2]	=a22*B-a21*Ba;
  dY[3]	=-a12*B+a11*Ba+D*Bq;
#ifdef H
  i	=1;
  gcmotion_(Yr,Yra,Yrq,Yz,Y,Y+1,Y+2,&gmP,&i);
  if(Yr[0] != dY[0] || Yra[0] != dY[1] || Yrq[0] !=dY[2] || Yz[0] !=dY[3]){
    out("sdddddd","dY=",Yr[0],Yr[0]-dY[0],Yra[0],Yra[0]-dY[1]
	,Yrq[0],Yrq[0]-dY[2]);
    out("sdd","dY=",Yz[0],Yz[0]-dY[3]);
  }
#endif  
  return;
}
int ESBiSpl2BaOld(double *Ba,double *Baq,double a,double gq) 
 { 
   int j00,j01,j10,j11; 
   double X,XX,x,xx,Y,YY,y,yy,YSC1,YSC2,ySC1,ySC2; 
   double f0,f1,fq0,fq1; 
  
   if(a < 0. && a > ESsa[ESNa]) return(1); 
   Spi0	=a*rha; 
   if(Spi0 >= ESNa) Spi0=ESNa-1; 
   Spi1	=Spi0+1; 
   x	=(a-ESsa[Spi0])*rha; 
   X	=1.-x; 

   Spj1	=gq*cr2gp; 
   if(gq < 0.) Spj1--; 
   gq	-=c2gp*Spj1; 
   Spj0	=gq*rhq; 
   if(Spj0 >= ESNp){ 
     Spj0	-=ESNp; 
     gq	-=c2gp; 
   } 
   if(Spj0 < 0){ 
     Spj0	+=ESNp; 
     gq	+=c2gp; 
   } 
   Spj1	=Spj0+1; 
   Y	=(gq-ESgt[Spj0])*rhq; 
   y	=1.-Y; 
   Spj0	=ESNp-Spj1; 
   Spj1	=Spj0+1; 

   XYa	=a; 
   XYx	=x; 
   XYX	=X; 

   XYq	=gq; 
   XYy	=y; 
   XYY	=Y; 

   XX	=X*X; 
   xx	=x*x; 
   XSC1	=XX*(3.-2.*X); 
   XSC2	=XX*x*ha; 
   xSC1	=xx*(3.-2.*x); 
   xSC2	=-xx*X*ha; 

   YY	=Y*Y; 
   yy	=y*y; 
   YSC1	=YY*(3.-2.*Y); 
   YSC2	=YY*y*hq; 
   ySC1	=yy*(3.-2.*y); 
   ySC2	=-yy*Y*hq; 

   xx	=3.*x*X; 
   X	-=xx; 
   x	-=xx; 
   xx	*=2.*rha; 

   yy	=3.*y*Y; 
   Y	-=yy; 
   y	-=yy; 
   yy	*=2.*rhq; 

   j00	=ESNp1*Spi0+Spj0; 
   j01	=j00+1; 
   j10	=j00+ESNp1; 
   j11	=j10+1; 

   f0	=XSC1*BFBa [j00]+xSC1*BFBa [j10]+XSC2*BFBaa [j00]+xSC2*BFBaa [j10]; 
   f1	=XSC1*BFBa [j01]+xSC1*BFBa [j11]+XSC2*BFBaa [j01]+xSC2*BFBaa [j11]; 
   fq0	=XSC1*BFBat[j00]+xSC1*BFBat[j10]+XSC2*BFBaat[j00]+xSC2*BFBaat[j10]; 
   fq1	=XSC1*BFBat[j01]+xSC1*BFBat[j11]+XSC2*BFBaat[j01]+xSC2*BFBaat[j11]; 
   *Ba	=YSC1*f0+ySC1*f1+YSC2*fq0+ySC2*fq1; 
   *Baq	=-(yy*(f1-f0)+Y*fq0+y*fq1); 
   return(0); 
 } 

int ESBiSpl2Ba(double *Ba,double *Baq,double a,double gq) 
{
  int j00,j01,j10,j11;
  double X,XX,x,xx,Y,YY,y,yy,YSC1,YSC2,ySC1,ySC2;
  double f0,f1,fq0,fq1;
  
  if(a < 0. && a > ESsa[ESNa]) return(1);
  Spi0	=a*rha;
  if(Spi0 >= ESNa) Spi0=ESNa-1;
  Spi1	=Spi0+1;
  x	=(a-ESsa[Spi0])*rha;
  X	=1.-x;

#ifdef H
  Spj1	=gq*cr2gp;
  if(gq > 0.) Spj1++;
  gq	=c2gp*Spj1-gq;
  Spj0	=gq*rhq;
  if(Spj0 >= ESNp){
    Spj0	-=ESNp;
    gq		-=c2gp;
  }
  if(Spj0 < 0){
    Spj0+=ESNp;
    gq	+=c2gp;
  }
  Spj1	=Spj0+1;
#endif
  Spj1	=gq*cr2gp;
  if(gq < 0.) Spj1--;
  gq	-=c2gp*Spj1;
  Spj0	=gq*rhq;
  if(Spj0 >= ESNp){
    Spj0	-=ESNp;
    gq	-=c2gp;
  }
  if(Spj0 < 0){
    Spj0	+=ESNp;
    gq	+=c2gp;
  }
  Spj1	=Spj0+1;
  Y	=(gq-ESgt[Spj0])*rhq;
  y	=1.-Y;
  Spj0	=ESNp-Spj1;
  Spj1	=Spj0+1;

  XYa	=a;
  XYx	=x;
  XYX	=X;

  XYq	=gq;
  XYy	=y;
  XYY	=Y;

  XX	=X*X;
  xx	=x*x;
  XSC1	=XX*(3.-2.*X);
  XSC2	=XX*x*ha;
  xSC1	=xx*(3.-2.*x);
  xSC2	=-xx*X*ha;

  YY	=Y*Y;
  yy	=y*y;
  YSC1	=YY*(3.-2.*Y);
  YSC2	=YY*y*hq;
  ySC1	=yy*(3.-2.*y);
  ySC2	=-yy*Y*hq;

  xx	=3.*x*X;
  X	-=xx;
  x	-=xx;
  xx	*=2.*rha;

  yy	=3.*y*Y;
  Y	-=yy;
  y	-=yy;
  yy	*=2.*rhq;

  j00	=ESNp1*Spi0+Spj0;
  j01	=j00+1;
  j10	=j00+ESNp1;
  j11	=j10+1;

  f0	=XSC1*BFBa [j00]+xSC1*BFBa [j10]+XSC2*BFBaa [j00]+xSC2*BFBaa [j10];
  f1	=XSC1*BFBa [j01]+xSC1*BFBa [j11]+XSC2*BFBaa [j01]+xSC2*BFBaa [j11];
  fq0	=XSC1*BFBat[j00]+xSC1*BFBat[j10]+XSC2*BFBaat[j00]+xSC2*BFBaat[j10];
  fq1	=XSC1*BFBat[j01]+xSC1*BFBat[j11]+XSC2*BFBaat[j01]+xSC2*BFBaat[j11];
  *Ba	=YSC1*f0+ySC1*f1+YSC2*fq0+ySC2*fq1;
  *Baq	=-(yy*(f1-f0)+Y*fq0+y*fq1);
#ifdef H
  out("?*?? IIII dd dd",j00,j01,j10,j11,BFBa[j00],BFBa[j01]
      ,BFBa[j10],BFBa[j11]);
#endif
  return(0);
}


int ESBiSplRAP3(double *r,double *ra,double *rq,double *raq,double *rqq 
 	       ,double *z,double *za,double *zq,double *zaq,double *zqq 
 	       ,double a,double gq) 
 { 
   int j00,j01,j10,j11; 
   double X,XX,x,xx,Y,YY,y,yy,YSC1,YSC2,ySC1,ySC2; 
   double f0,f1,fq0,fq1; 
   double dy0,dy1,dy2; 
  
   if(a < 0. && a > ESsa[ESNa]) return(1); 
   Spi0	=a*rha; 
   if(Spi0 >= ESNa) Spi0=ESNa-1; 
   Spi1	=Spi0+1; 
   x	=(a-ESsa[Spi0])*rha; 
   X	=1.-x; 

   Spj1	=gq*cr2gp; 
   if(gq < 0.) Spj1--; 
   gq	-=c2gp*Spj1; 
   Spj0	=gq*rhq; 
   if(Spj0 >= ESNp){ 
     Spj0	-=ESNp; 
     gq	-=c2gp; 
   } 
   if(Spj0 < 0){ 
     Spj0	+=ESNp; 
     gq	+=c2gp; 
   } 
   Spj1	=Spj0+1; 
   Y	=(gq-ESgt[Spj0])*rhq; 
   y	=1.-Y; 
   Spj0	=ESNp-Spj1; 
   Spj1	=Spj0+1; 

   XYa	=a; 
   XYx	=x; 
   XYX	=X; 

   XYq	=gq; 
   XYy	=y; 
   XYY	=Y; 

   XX	=X*X; 
   xx	=x*x; 
   XSC1	=XX*(3.-2.*X); 
   xSC1	=xx*(3.-2.*x); 
   XSC2	=XX*x*ha; 
   xSC2	=-xx*X*ha; 

   YY	=Y*Y; 
   yy	=y*y; 
   YSC1	=YY*(3.-2.*Y); 
   YSC2	=YY*y*hq; 
   ySC1	=yy*(3.-2.*y); 
   ySC2	=-yy*Y*hq; 


#ifdef H 
   dYSC1	=6.*Y*y*rhq; 
   dYSC2	=3.*y*Y-Y; 
   dySC2	=3.*y*Y-y; 

   d2YSC1	=6.*(y-Y)*rhq; 
   d2YSC2	=0.5*d2YSC1-1; 
   d2ySC2	= 0.5*d2YSC1+1.;
#endif

  dy0	=6.*(y-Y);
  dy1	=0.5*dy0-1.;
  dy2	=0.5*dy0+1.;
  dy0	*=rhq;

  yy	=3.*y*Y;
  Y	-=yy;
  y	-=yy;
  yy	*=2.*rhq;


  j00	=ESNp1*Spi0+Spj0;
  j01	=j00+1;
  j10	=j00+ESNp1;
  j11	=j10+1;

  f0	=XSC1*ESsr [j00]+xSC1*ESsr [j10]+XSC2*ESsra [j00]+xSC2*ESsra [j10];
  f1	=XSC1*ESsr [j01]+xSC1*ESsr [j11]+XSC2*ESsra [j01]+xSC2*ESsra [j11];
  fq0	=XSC1*ESsrt[j00]+xSC1*ESsrt[j10]+XSC2*ESsrat[j00]+xSC2*ESsrat[j10];
  fq1	=XSC1*ESsrt[j01]+xSC1*ESsrt[j11]+XSC2*ESsrat[j01]+xSC2*ESsrat[j11];
  *r	=YSC1*f0+ySC1*f1+YSC2*fq0+ySC2*fq1;
  *rq	=-(yy*(f1-f0)+Y*fq0+y*fq1);
  *rqq	=(dy0*(f0-f1)+dy1*fq0+dy2*fq1)*rhq;

  f0	=XSC1*ESsz [j00]+xSC1*ESsz [j10]+XSC2*ESsza [j00]+xSC2*ESsza [j10];
  f1	=XSC1*ESsz [j01]+xSC1*ESsz [j11]+XSC2*ESsza [j01]+xSC2*ESsza [j11];
  fq0	=XSC1*ESszt[j00]+xSC1*ESszt[j10]+XSC2*ESszat[j00]+xSC2*ESszat[j10];
  fq1	=XSC1*ESszt[j01]+xSC1*ESszt[j11]+XSC2*ESszat[j01]+xSC2*ESszat[j11];
  *z	=YSC1*f0+ySC1*f1+YSC2*fq0+ySC2*fq1;
  *zq	=-(yy*(f1-f0)+Y*fq0+y*fq1);
  *zqq	=(dy0*(f0-f1)+dy1*fq0+dy2*fq1)*rhq;

  xx	=3.*x*X;
  X	-=xx;
  x	-=xx;
  xx	*=2.*rha;

  f0	=xx*(ESsr [j10]-ESsr [j00])+X*ESsra [j00]+x*ESsra [j10];
  f1	=xx*(ESsr [j11]-ESsr [j01])+X*ESsra [j01]+x*ESsra [j11];
  fq0	=xx*(ESsrt[j10]-ESsrt[j00])+X*ESsrat[j00]+x*ESsrat[j10];
  fq1	=xx*(ESsrt[j11]-ESsrt[j01])+X*ESsrat[j01]+x*ESsrat[j11];
  *ra	=YSC1*f0+ySC1*f1+YSC2*fq0+ySC2*fq1;
  *raq	=-(yy*(f1-f0)+Y*fq0+y*fq1);

  f0	=xx*(ESsz [j10]-ESsz [j00])+X*ESsza [j00]+x*ESsza [j10];
  f1	=xx*(ESsz [j11]-ESsz [j01])+X*ESsza [j01]+x*ESsza [j11];
  fq0	=xx*(ESszt[j10]-ESszt[j00])+X*ESszat[j00]+x*ESszat[j10];
  fq1	=xx*(ESszt[j11]-ESszt[j01])+X*ESszat[j01]+x*ESszat[j11];
  *za	=YSC1*f0+ySC1*f1+YSC2*fq0+ySC2*fq1;
  *zaq	=-(yy*(f1-f0)+Y*fq0+y*fq1);
  return(0);
}

void GCLagrangeEq(int *n,double *x,double *Y,double *dY)
{
  double r,ra,rq,z,za,zq,B,Ba,Bq,dgY,F,D,rD;
  double raa,rqq,raq,zaa,zqq,zaq;
  double a11,a12,a21,a22;
  double a,q,gh,gha,ghq,ghaq,ba,baq;
  int i;

  a	=Y[1];
  q	=Y[2];
  if(ESBiSplRAP(&r,&ra,&rq,&z,&za,&zq,&B,&Ba,&Bq,&D,&D,&D,Y[1],Y[2])){
    istate	=3;
    return;
  }

  ESBiSplRAP3(&r,&ra,&rq,&raq,&rqq,&z,&za,&zq,&zaq,&zqq,a,q);

  ESSetSplA(Y[1]);
  splRA(&dgY,NULL,ESdgY,ESdgY2a);
  dgY	*=a;
  splRA(&F,NULL,ESaF,ESaF2a);
  splRA(dY+2,NULL,ESPs,ESPs2a);
  splRA(dY+3,NULL,ESaT,ESaT2a);

#ifdef H
  ESBiSplRA(&F,&dgY,dY+3,dY+2);
  dgY	*=a;
#endif
  dY[3]	*=Y[0];

  D	=za*rq-zq*ra;
  a12	=D*((F+dY[3])/r+Y[0]*r*dY[2]);

  rD	=1./(r*D);
  a11	=-(rq*rq+zq*zq)*dgY*rD;
  ba	=-(ra*rq+za*zq)*dgY*rD;
#ifdef H
  ESBiSpl2Ba(&ba,&baq,a,q);
  baq	=-((raq*rq+ra*rqq+zaq*zq+za*zqq)*dgY+
	   ba*(rq*D+r*(zaq*rq+za*rqq-zqq*ra-zq*raq)))*rD;
  a12	+=Y[0]*baq;  /*Roscoe & (ESC || SFL)*/
  ba	=0.;
#endif

  a22	=dgY*(1.+dY[3]/F);
  a21	=F;

  r	=1./(a11*a22-a12*a21);
  a11	*=r;
  a12	*=r;
  a21	*=r;
  a22	*=r;
  ba	*=r;

  r	=Y[0]*B;
  B	*=r;

#ifdef H
  r	=r*Y[0]+cgWc*gmP;
#endif
  r	=r*Y[0]+gmP;
  Ba	*=r;
  Bq	*=-r;

  dY[0]	=a22*Bq;
  dY[1]	=-a21*Bq;
  dY[2]	=a22*B-a21*Ba;
  dY[3]	=-a12*B+a11*Ba+ba*Bq;
#ifdef H
  i	=1;
  gcmotion_(Yr,Yra,Yrq,Yz,Y,Y+1,Y+2,&gmP,&i);
  if(Yr[0] != dY[0] || Yra[0] != dY[1] || Yrq[0] !=dY[2] || Yz[0] !=dY[3]){
    out("sdddddd","dY=",Yr[0],Yr[0]-dY[0],Yra[0],Yra[0]-dY[1]
	,Yrq[0],Yrq[0]-dY[2]);
    out("sdd","dY=",Yz[0],Yz[0]-dY[3]);
  }
#endif  
  return;
}

#ifdef H
??? E= 1.000e+00 dgf/E=8.08343e+02
??? E= 1.000e-01 dgf/E=7.88399e+02
??? E= 2.000e-01 dgf/E=7.92765e+02
??? E= 3.000e-01 dgf/E=7.95263e+02
??? E= 4.000e-01 dgf/E=8.00402e+02
??? E= 5.000e-01 dgf/E=8.01110e+02
??? E= 6.000e-01 dgf/E=8.03061e+02
??? E= 7.000e-01 dgf/E=8.03258e+02
??? E= 8.000e-01 dgf/E=8.05058e+02
??? E= 9.000e-01 dgf/E=8.06747e+02
??? E= 1.000e+00 dgf/E=8.08343e+02
??? E= 1.100e+00 dgf/E=8.09864e+02
??? E= 1.200e+00 dgf/E=8.11320e+02
??? E= 1.300e+00 dgf/E=8.12720e+02
??? E= 1.400e+00 dgf/E=8.14073e+02
??? E= 1.500e+00 dgf/E=8.15382e+02
??? E= 1.600e+00 dgf/E=8.16654e+02
??? E= 1.700e+00 dgf/E=8.17891e+02
??? E= 1.800e+00 dgf/E=8.19097e+02
??? E= 1.900e+00 dgf/E=8.20276e+02
??? E= 2.000e+00 dgf/E=8.21429e+02
#endif

void GCLagrangeEqA(int *n,double *x,double *Y,double *dY)
{
  double r,ra,rq,z,za,zq,B,Ba,Bq,dgY,F,D;
  double a11,a12,a21,a22;
  double a,q;
  int i;

#ifdef H
  a	=Y[1];
  q	=Y[2];
  if(ESBiSplRAP(&r,&ra,&rq,&z,&za,&zq,&B,&Ba,&Bq,&D,&D,&D,Y[1],Y[2])){
    istate	=3;
    return;
  }

  ESSetSplA(Y[1]);
  splRA(&dgY,NULL,ESdgY,ESdgY2a);
  dgY	*=Y[1];
  splRA(&a21,NULL,ESaF,ESaF2a);
  splRA(dY+2,NULL,ESPs,ESPs2a);
  splRA(dY+3,NULL,ESaT,ESaT2a);

  ESBiSplRA(&a21,&dgY,dY+3,dY+2);
  dgY	*=Y[1];
  dY[3]	*=Y[0];

  a22	=za*rq-zq*ra;
  a12	=a22*((a21+dY[3])/r+Y[0]*r*dY[2]);
  D	=-dgY/(r*a22);
  a11	=(rq*rq+zq*zq)*D;
  D	*=ra*rq+za*zq;
  a22	=dgY*(1.+dY[3]/a21);
  r	=1./(a11*a22-a12*a21);
  a11	*=r;
  a12	*=r;
  a21	*=r;
  a22	*=r;
  D	*=r;
#ifdef H
  r	=cgWc*Y[0]*B;
#endif
  r	=Y[0]*B;

  B	*=r;

#ifdef H
  r	=r*Y[0]+cgWc*gmP;
#endif
  r	=r*Y[0]+gmP;
  Ba	*=r;
  Bq	*=-r;

  dY[0]	=a22*Bq;
  dY[1]	=-a21*Bq;
  dY[2]	=a22*B-a21*Ba;
  dY[3]	=-a12*B+a11*Ba+D*Bq;
#endif  
  i	=1;
  gcmotion_(Yr,Yra,Yrq,Yz,Y,Y+1,Y+2,&gmP,&i);
  dY[0]	=Yr[0];
  dY[1]	=Yra[0];
  dY[2]	=Yrq[0];
  dY[3]	=Yz[0];
#ifdef H
  if(Yr[0] != dY[0] || Yra[0] != dY[1] || Yrq[0] !=dY[2] || Yz[0] !=dY[3]){
    out("sdddddd","dY=",Yr[0],Yr[0]-dY[0],Yra[0],Yra[0]-dY[1]
	,Yrq[0],Yrq[0]-dY[2]);
    out("sdd","dY=",Yz[0],Yz[0]-dY[3]);
  }
#endif  
  return;
}

int ES4Output(double a0,double a1,double gt0, double gt1,int na, int nt)
{

  return(0);
}

int ESInitOrbLSODE(int n)
{
  lrw	=liw+16*n;
  iwork	=(int*)malloc(liw*sizeof(int));
  if(iwork == NULL){
    printf("Failure in memory allocation for iwork in ESReInitStLSODE()\n");
    printf("Too many harmonics\n");
    return(-1);
  }
  rwork	=(double*)malloc((lrw+n)*sizeof(double));
  if(rwork == NULL){
    printf("Failure in memory allocation for rwork in ESReInitStLSODE()\n");
    printf("Too many harmonics\n");
    free(iwork);
    return(-1);
  }
  yh	=rwork+liw;
  vatol	=rwork+lrw;
  return(0);
}

int ESDeInitOrbLSODE()
{
  free(rwork);
  rwork	=NULL;
  free(iwork);
  iwork	=NULL;
  return(0);
}

int GetBounceTime(int *kk,double *t0,double *gf,double gq0,double t1)
{
  int i,k,n;
  double h,h1,t;
  double y,dy;
  double *ld;
  double s,ds,f;

  n	=iwork[13]+1;
  h	=rwork[10];
  t	=rwork[12];

  s	=0.;
  k	=0;
  do{
    ld	=yh+2;
    y	=0.;
    dy	=0.;
    f	=1.;
    for(i=0; i < n; i++){
      y	+=(*ld)*f;
      ld+=4;
      dy+=(i+1)*(*ld)*f;
      f	*=s;
    }
    ds	=(gq0-y)/dy;
    s	+=ds;
    k++;
  }while(fabs(ds) > 1e-8 && k < 10);
  if(k < 10){
    *t0=t+s*h;
    ld	=yh+3;
    y	=0.;
    f	=1.;
    for(i=0; i < n; i++){
      y	+=(*ld)*f;
      ld+=4;
      f	*=s;
    }
    *gf=y;
  }
  else{
    *t0=0.;
    *gf=0.;
  }
  *kk	=k;
  return(0);
}

int InitLsode4Orbit(double t, double va)
{
  mf		=10;
  itol	=2;	/* vector tolerance, 1 for scalar toleranc */
  itask	=5;
  iopt	=1;	/* optional input ON */
  rwork[1-1]	=t;
  rwork[2-1]	=0.;
  rwork[3-1]	=0.;
  rwork[4-1]	=0.;
  rwork[5-1]	=10;/* the step size to be attempted on the first step. */
  rwork[6-1]	=0.1*rwork[0];/* the maximum absolute step size allowed. */
  rwork[7-1]	=0.;	/* the minimum absolute step size allowed. */

  iwork[4]	=12;	/* Maximum order */
  iwork[5]	=500;
  iwork[6]	=10;

  reltol	=0.;
  vatol[0]	=va;
  vatol[1]	=abstol;
  vatol[2]	=c2gp*abstol;
  vatol[3]	=c2gp*abstol;
  istate	=1;
  return(0);
}

void GCOrbitPlot2D()
{
  double t,tout,A,R[256],Z[256],gq0,gf0,gr0;
  double t1,gq1,gf;
  int n;

  ZColor(14);
  t	=0.;
  tout	=rwork[0];
  gr0	=yGC[0];
  gq0	=yGC[2];
  gf0	=yGC[3];
  gq1	=gq0;
  t1	=t;

  ES2DMapFlx2Lab(R,Z,&A,&A,&A,&A,yGC[1],-yGC[2]);
  n	=1;
  ist	=0;
  while(t < rwork[0]){
    t1	=t;
    gq1	=yGC[2];
    lsode_(GCLagrangeEq,&neq,yGC,&t,&tout,&itol,&reltol,vatol,&itask,
	   &istate,&iopt,rwork,&lrw,iwork,&liw,NULL,&mf);
    if(yGC[0]*gr0 > 0. && (gq0-yGC[2])*(gq0-gq1) < 0.){
      int k;
      GetBounceTime(&k,&t,&gf,gq0,t1);
      lBD->t	=t/cgWc;
      lBD->f	=cgWc/t;
      lBD->gw	=c2gp*lBD->f;
#ifdef H
      printf(" %10.3e %10.3e %10.3e %10.3e %10.3e %10.3e\n"
	     ,lBD->E,lBD->t,lBD->f,lBD->gw,lBD->gY,lBD->F);
      printf(" %12.3e %10.3e %10.3e %10.3e %10.3e %10.3e\n"
	     ,lBD->r,lBD->z,lBD->a,lBD->q,gf,lBD->t != 0 ? gf/lBD->t: 0.);
#endif
      if(lBD->E <0.15) printf("??? p=%10.3e\n",lBD->p);
      printf("??? %2d E=%10.3e dgf/E=%11.5e\n",k,lBD->E,gf/(lBD->t*lBD->E));
      t	=2.*rwork[0];
    }
    
    ES2DMapFlx2Lab(R+n,Z+n,&A,&A,&A,&A,yGC[1],-yGC[2]);
    n++;
    if(n == 256){
      ZPlotPolyLine(R,Z,n);
      n--;
      R[0]	=R[n];
      Z[0]	=Z[n];
      n	=1;
    }
  }
  nst	=ist;
  if	(n > 1) ZPlotPolyLine(R,Z,n);
#ifdef H  
  out("sIIIdd","N steps=",iwork[10],iwork[11],istate
      ,(yGC[2]-gq0)*cr2gp,(yGC[3]-gf0)*cr2gp);
#endif
  return;
}

void GCOrbitPlot2DLarmor()
{
  double t,tout,A,R[256],Z[256],nr,nz,s,grn[256],B,Bt,gq0,gf0,gr0;
  double t1,gq1,r[65],z[65];
  int j,k,n;

  ZColor(4);
  t	=0.;
  tout	=rwork[0];
  gr0	=yGC[0];
  gq0	=yGC[2];
  gf0	=yGC[3];
  gq1	=gq0;
  t1	=t;
  ES2DMapFlx2Lab(R,Z,&A,&A,&A,&A,yGC[1],-yGC[2]);
  grn[0]	=grn0;
  n	=1;
  ist	=0;
  while(t < rwork[0] && yGC[1] < 1.){
    t1	=t;
    gq1	=yGC[2];
    lsode_(GCLagrangeEq,&neq,yGC,&t,&tout,&itol,&reltol,vatol,&itask,
	   &istate,&iopt,rwork,&lrw,iwork,&liw,NULL,&mf);
#ifdef H
    if(yGC[0]*gr0 > 0. && (gq0-yGC[2])*(gq0-gq1) < 0.){
      int k;
      GetBounceTime(&k,&t,gq0,t1);
      lBD->t	=t/cgWc;
      lBD->f	=cgWc/t;
      lBD->gw	=c2gp*lBD->f;
      printf(" %10.3e %10.3e %10.3e %10.3e %10.3e %10.3e\n"
	     ,lBD->E,lBD->t,lBD->f,lBD->gw,lBD->a,lBD->q);
      printf(" %32.3e %10.3e %10.3e %10.3e\n",lBD->gY,lBD->F,lBD->r,lBD->z);
      t	=2.*rwork[0];
    }
#endif
    ESBiSplRAP(R+n,&A,&nz
	       ,Z+n,&A,&nr
	       ,&B,&A,&A
	       ,&A,&A,&A
	       ,yGC[1],yGC[2]);

    if(yGC[1] > 1e-8){
      A	=1./sqrt(nr*nr+nz*nz);
      nr	*=-A;
      nz	*=A;
    }
    else{
      nr=1.;
      nz=0.;
    }
    grn[n]	=grn0*sqrt(B0/B);
    ESSetSplA(yGC[1]);
    splRA(&Bt,NULL,ESaF,ESaF2a);
    printf("%10.3e < 1.\n",Bt/(R[n]*B));

    Bt	*=grn[n]/(R[n]*B);


    if(ESIsInsidePlasma(R[n]+grn[n]*nr,Z[n]+grn[n]*nz) == 0){
      A	=grn[n];
      for(j=0; j < ESNp1; j++){
	r[j]	=R[n]+Bt*nz*EScs1[j]+A*nr*ESsn1[j];
	z[j]	=Z[n]-Bt*nr*EScs1[j]+A*nz*ESsn1[j];
      }
      ZPlotPolyLine(r,z,ESNp1);
    }
    n++;
    if(n == 256){
      ZColor(14);
      ZPlotPolyLine(R,Z,n);
#ifdef H
      for(k=0; k < n; k++){
	A	=grn[k];
	for(j=0; j < ESNp1; j++){
	  r[j]	=R[k]+A*EScs1[j];
	  z[j]	=Z[k]+A*ESsn1[j];
	}
	ZPlotPolyLine(r,z,ESNp1);
      }
#endif
      n--;
      R[0]	=R[n];
      Z[0]	=Z[n];
      n	=1;
      ZColor(4);
    }
  }
  nst	=ist;
  if(n > 1){
    ZColor(14);
    ZPlotPolyLine(R,Z,n);
#ifdef H
    for(k=0; k < n; k++){
      A	=grn[k];
      for(j=0; j < ESNp1; j++){
	r[j]	=R[k]+A*EScs1[j];
	z[j]	=Z[k]+A*ESsn1[j];
      }
      ZPlotPolyLine(r,z,ESNp1);
    }
#endif
  }
#ifdef H  
  out("sIIIdd","N steps=",iwork[10],iwork[11],istate
      ,(yGC[2]-gq0)*cr2gp,(yGC[3]-gf0)*cr2gp);
#endif
  return;
}

int FractionLosses() 
{
  int i,j,ji;
  double a,f,f1,f2,F,F1,F2,qg,psp,pi,pe;
  double Fv[0x100],fv[0x100];
  char *lD,*lN;
  char FNm[0x100];
  FILE *lF;

  lF	=fopen("ist0.dat","r");
  if(lF == NULL) return(2);
  
  ji	=0;
  for(i=0; i < ESNa1; i++){
    if(ji == 0 && fscanf(lF,"a= %lg",&a) < 1) 
      CbStr2UserMessage("[%d]-failed",i);
    if(ji && fscanf(lF,"\na= %lg",&a) < 1) puts("failed");
    for(j=0; j < ESNp1; j++){
      if(fscanf(lF,"%lg %lg",Pi+ji,Pe+ji) < 2)
	CbStr2UserMessage("[%d][%d]-Failed",i,j);
      ji++;
    }
  }
  fclose(lF);

  CbGetCurrentCbName(&lD,&lN);
  puts(lD);
  puts(lN);
  sprintf(FNm,"%s/Out/%s",lD,"gAFrac.00");
  lF	=fopen(FNm,"w");

  F	=0.;
  F2	=0.;
  f	=0.;
  f2	=0.;
  ji	=0.;
  for(i=0; i < ESNa1; i++){
    a	=ESsa[i];
    psp	=ESsp[i]*ESsp[i];
    F1	=F2;
    f1	=f2;
    F2	=0.;
    f2	=0.;
    for(j=0; j < ESNp; j++){
      qg	=ESsr[ji]*(ESsra[ji]*ESszt[ji]-ESsrt[ji]*ESsza[ji]);
      if(i == 0){
	qg=0.;
	Pi[ji]=Pi[0];
	Pe[ji]=Pe[0];
      }
      F2	+=qg;
      pi	=fabs(1.-Pi[ji]) < 0.01 ? 1. : Pi[ji];
      pe	=fabs(1.+Pe[ji]) < 0.01 ? -1. : Pe[ji];
      f2	+=qg*0.5*(2.-pi+pe);
      fprintf(lF,"%2d %2d %24.16e %24.16e %24.16e\n"
	      ,i,j,qg,0.5*(2.-pi+pe),qg*0.5*(2.-pi+pe));
      ji++;
    }
    ji++;
    F2	*=psp/ESNp;
    f2	*=psp/ESNp;
    F	+=0.5*(F1+F2)*ESsa[1];
    f	+=0.5*(f1+f2)*ESsa[1];
    f1	=F2 != 0. ? f2/F2 : 0.;
    Fv[i]	=F2;
    fv[i]	=f2;
    printf("%2d p=%10.3e F=%10.3e f=%10.3e %10.3e %10.3e %10.3e\n"
	   ,i,ESsp[i],F,f,F2,f2,f1);
  }
  fputc('\n',lF);
  for(i=0; i < ESNa1; i++) fprintf(lF,"%2d %24.16e %24.16e\n"
	      ,i,Fv[i],fv[i]);


  fprintf(lF,"%10.3e\n",f/F);
  printf("fraction=%10.3e\n",f/F);
  fclose(lF);
  printf("%s with particle losses was written\n",FNm);
  return(0);
}

int ReadFractionLosses() 
{
  int i,j,ji,k;
  double qg1[0x1000],qgFr[0x1000],Fr[0x1000];
  double vFr[0x100],vqg1[0x100];
  char *lD,*lN;
  char FNm[0x100];
  FILE *lF;

  CbGetCurrentCbName(&lD,&lN);
  puts(lD);
  puts(lN);
  sprintf(FNm,"%s/In/%s",lD,"gAFrac.00");
  lF	=fopen(FNm,"r");

  ji	=0.;
  for(i=0; i < ESNa1; i++){
    for(j=0; j < ESNp; j++){
      if(fscanf(lF,"%d %d %lg %lg %lg",&k,&k,qgFr+ji,Fr+ji,qgFr+ji) < 3){
	CbStr2UserMessage("%d %d -failed read particel loss data\n",i,j);
	return(1);
      }
      ji++;
    }
    ji++;
  }
  for(i=0; i < ESNa1; i++) fscanf(lF,"%d %lg %lg\n",&k,vqg1+i,vFr+i);
  fclose(lF);
  printf("%s with particle losses was read\n",FNm);
  return(0);
}

void GCOrbitPlot2DLoss()
{
  double p0,p1,p,dp=0.05,P1,P2;

  double t,tout,A,R[256],Z[256],nr,nz,s,grn[256],B,gq0,gf0,gr0;
  double r[65],z[65];
  double a,a1,sa[21]={
    0.01,0.05,0.1,0.15,0.2,0.25,0.3,0.35,0.4,0.45,0.5,0.55,0.6
    ,0.65,0.7,0.75,0.8,0.85,0.9,0.95,0.98
  };
  int i,j,ji,k,n,m,ii,iLoss,kL[0x40];
  FILE *lF;

  p0	=-.05;
  p1	=-.7;

  lF	=fopen("ist.dat","w");
  a	=yGC[1];
  ji	=0;
  for(i=0; i < ESNa1; i++){
    a	=sa[i];
    fprintf(lF,"a=%5.2f\n",a);
    printf("--------------------------- %5.2f -------------------------\n",a);
    for(j=0; j < ESNp1; j++){
      ZColor(4);
      fprintf(lF,"%2d",j);
      printf("%2d %10.5f ",j,ESgt[j]*cr2gp);
      for(m=0; m < 2; m++){
	if(m){
	  p	=Pe[ji];
	  P1	=p-dp;
	  P2	=p+dp;
	}
	else{
	  p	=Pi[ji];
	  P1	=p+dp;
	  P2	=p-dp;
	}

	if(P1 > 1.) P1	=1.;
	if(P1 < -1.) P1	=-1.;
	if(P2 > 1.) P2	=1.;
	if(P2 < -1.) P2	=-1.;

	ii	=6;
	while(ii){
	  ii--;
	  if(fabs(P2-P1) < 0.005) ii =0;
	  p	=0.5*(P1+P2);
	  iLoss 	=0;
	  Vb	=p*Vtot;
	  Vn	=sqrt(1.-p*p)*Vtot;
	  yGC[1]	=a;
	  yGC[2]	=c2gp-ESgt[j];
	  yGC[3]	=0.;
	  ESBiSplRAP(R,&A,&A
		     ,Z,&A,&A
		     ,&B,&A,&A
		     ,&A,&A,&A
		     ,yGC[1],yGC[2]);
	  grn0	=Vn/(cgWc*B);
	  B0	=B;
	  n	=1;
	  gmP	=cr2*Vn*Vn/(B*cgWc*cgWc);
	  yGC[0]	=Vb/(B*cgWc);

	  t		=0.;
	  tout	=rwork[0];
	  gr0	=yGC[0];
	  gq0	=yGC[2];
	  gf0	=yGC[3];
	  grn[0]	=grn0;
	  InitLsode4Orbit(rwork[0],vatol[0]);
	  while(iLoss == 0 && t < rwork[0] && yGC[1] < 1.){
	    lsode_(GCLagrangeEq,&neq,yGC,&t,&tout,&itol,&reltol,vatol,&itask,
		   &istate,&iopt,rwork,&lrw,iwork,&liw,NULL,&mf);
	    a1	=yGC[1];

	    if(a1 > 1.){
	      a1	=1.;
	      iLoss	=1;
	    }
	    ESBiSplRAP(R+n,&A,&nz
		       ,Z+n,&A,&nr
		       ,&B,&A,&A
		       ,&A,&A,&A
		       ,a1,yGC[2]);
	    if(a1 > 1e-8){
	      A	=1./sqrt(nr*nr+nz*nz);
	      nr	*=-A;
	      nz	*=A;
	    }
	    else{
	      nr=1.;
	      nz=0.;
	    }
	    grn[n]	=grn0*sqrt(B0/B);
	    if(iLoss == 0 
	       && ESIsInsidePlasma(R[n]+grn[n]*nr,Z[n]+grn[n]*nz) == 0)iLoss=1;
	    if(ii == 0 && iLoss){
	      A	=grn[n];
	      for(k=0; k < ESNp1; k++){
		r[k]	=R[n]+A*EScs1[k];
		z[k]	=Z[n]+A*ESsn1[k];
	      }
	      ZPlotPolyLine(r,z,ESNp1);
	    }
	    n++;
	    if(n == 256){
	      if(ii == 0){
		ZColor(14);
		ZPlotPolyLine(R,Z,n);
		ZColor(4);
	      }
	      n--;
	      R[0]	=R[n];
	      Z[0]	=Z[n];
	      n	=1;
	    }
	  }
	  if(ii == 0 && n > 1){
	    ZColor(14);
	    ZPlotPolyLine(R,Z,n);
	    ZColor(4);
	  }
	  if(iLoss){
	    P2	=p;
	  }
	  else{
	    P1	=p;
	  }
	}
	fprintf(lF," %10.3e",p);
	printf(" %10.3e |%10.3e|",p,fabs(P1-P2));
	if(m){
	  p1	=p;
	}
	else{
	  p0	=p;
	}
      }
      fputc('\n',lF);
      putchar('\n');
      ji++;
    }
  }
  fclose(lF);
  puts("ist.dat was written");
  return;
}

void GCOrbitPlot2DB()
{
  double t,tout,A,B,R[256],Z[256],gq0,gf0;
  int n;

  ZColor(14);
  t	=0.;
  tout	=rwork[0];
  gf0	=yGC[3];
  gq0	=yGC[2];
  ES2DMapFlx2Lab(R,Z,&A,&A,&A,&A,yGC[1],-yGC[2]);
  n	=1;
  ist	=0;
  while(t < rwork[0]){
    lsode_(GCLagrangeEqA,&neq,yGC,&t,&tout,&itol,&reltol,vatol,&itask,
	   &istate,&iopt,rwork,&lrw,iwork,&liw,NULL,&mf);
    ES2DMapFlx2Lab(R+n,Z+n,&A,&A,&A,&A,yGC[1],-yGC[2]);
    n++;
    if(n == 256){
      ZPlotPolyLine(R,Z,n);
      n--;
      R[0]	=R[n];
      Z[0]	=Z[n];
      n	=1;
    }
  }
  nst	=ist;
  if(n > 1){
    ZPlotPolyLine(R,Z,n);
  }
  out("sIIIdd","N steps=",iwork[10],iwork[11],istate
      ,(yGC[2]-gq0)*cr2gp,(yGC[3]-gf0)*cr2gp);
  return;
}

void GCOrbitPlot2DA()
{
  double t,h,A;
  int k,n,m;

  ZColor(14);
  t	=0.;
  n	=1;

  m	=1;
  for(k=0; k < nPart; k++){
    ES2DMapFlx2Lab(rPrt0+k,zPrt0+k,&A,&A,&A,&A,Xa[k],-Xgq[k]);
  }

  gcmotion_(dXgr,dXa,dXgq,dXgf,Xgr,Xa,Xgq,Xgm,&nPart);
  h	=20.*c2gp;
  ist	=0;
  while(t < rwork[0]){
    GCRKStep(&t,h);
    for(k=0; k < nPart; k++){
      if(m){
	ES2DMapFlx2Lab(rPrt1+k,zPrt1+k,&A,&A,&A,&A,Xa[k],-Xgq[k]);
      }
      else{
	ES2DMapFlx2Lab(rPrt0+k,zPrt0+k,&A,&A,&A,&A,Xa[k],-Xgq[k]);
      }
      ZPlotLine(rPrt1[k],zPrt1[k],rPrt0[k],zPrt0[k]);
    }
    m	^=1;
    ist++;
  }
  nst	=ist;
  return;
}

int ESIWriteDensity(int Na,int Fic)
{
  int i,Na1;
  double a,da,ne,Te,Ti,p,dp,f;
  FILE *lf;
  char FNm[16];
  extern char ESMessage[];

  sprintf(FNm,"esiT.%2.2d",Fic);
  lf	=fopen(FNm,"w");
  if(lf == NULL){
    sprintf(ESMessage,"%s cannot be open",FNm);
    return(1);
  }
  Na1	=Na+1;
  da	=(ESsa[ESNa]-ESsa[0])/Na;
  Te	=15.;
  Ti	=15.;
  f	=1./(1.60022e-2*cgm0*(Te+Ti));

  fprintf(lf,"!!! Do not edit this file. %s\n",ESMessage);
  fprintf(lf,"%24s%24s%24s\n","Te=Ti=15keV     a"
	  ,"ne [10^20/m^3]","dne/da [10^20/m^3]");
  for(i=0; i < Na1; i++){
    a	=da*i;
    ESSetSplA(a);
    splRA(&p,&dp,ESsp,ESsp2a);
    ne	=f*p;
    fprintf(lf,"%24.16e%24.16e%24.16e\n",a,f*p,f*dp);
  }
  fclose(lf);
  return(0);
}

int ESOrbits()
{
  double X[2],Y[2],R1,R2;
  double r,ra,rq,z,za,zq,B,Ba,Bq,gh,gha,ghq,gq;
  double V,Tc,T0;
  static double A,gt=0.,a=0.5,mi=2.,ei=1.;
  static double dt=1.,t=100.;
  static double E=1.;
  static double p=0.4,gr,grn,grb;
  static int Fl=0,Flmi=1,Next=0;
  int FlBerk=0,FlBerkS=0;
  double Pgf,F,dF,gY,dgY,gm,err;
  static int n=1000,iPl=0;

  int i,j,k,L;

  lBD	=&BounceData;
 
  j	=ESNp1*ESNa;
  i	=ESNp/2;
  X[0]	=ESsr[j+i];
  X[1]	=ESsr[j];
  i	=ESNp/4;
  Y[0]	=ESsz[j+3*i];
  Y[1]	=ESsz[j+i];

  ESBasicFunctions();
  ESFirstBiSpline();

#ifdef H
  esiread_("esiB.00");

  ESInitOrbLSODE(neq);
  esilink2c_(XF,XFa,XgFa,XgFaa,XgYa,XgYaa,XT,XTa,XP,XPa,Xr,Xra,Xrq
	     ,Xz,Xza,Xzq,XB,XBa,XBq,Xgh,Xgha,Xghq,NULL);
#endif
  L	=0;
  iwork[11-1]	=0; /* the number of steps taken for the problem so far */
  iwork[12-1]	=0; /* the number of f evaluations for the problem so far. */
  ESSetMapBox();
#ifndef Tbl_Orbits
  switch(Flmi){
  case 0:
    mi	=1.;
    ei	=1.;
    break;
  case 1:
    mi	=2.;
    ei	=1.;
    break;
  case 2:
    mi	=3.;
    ei	=1.;
    break;
  case 3:
    mi	=4.;
    ei	=2.;
    break;
  }
  cgWc		=ce*ei*1e+4/(mi*cmi*cc);
  cV1keV	=sqrt(2.*cEkeV/(mi*cmi))*1e-2;
  cgr1keV	=cV1keV/cgWc;
  T0		=c2gp/cgWc;

  if(a < ESsa[0]) a=ESsa[0];
  if(a > ESsa[ESNa]) a=ESsa[ESNa];
  while(gt >= 1.) gt	-=1.;
  while(gt < 0.)  gt	+=1.;
  switch(Fl){
  case 0:
    ES2DMapFlx2Lab(&r,&z,&gq,&gq,&gq,&gq,a,c2gp*gt);
    break;
  case 1:
    if(ES1DMapZ2gt(&gq,&R1,&R2,z) == 0 && R1 <= r && r <= R2){
      gq=c2gp*gt;
      A	=a;
      if(ES2DMapLab2Flx(&A,&gq,r,z) == 0){
	a	=A;
	gt	=cr2gp*gq;
      }
    }
    break;
  }
  gq	=1.-gt;
  while(gq >= 1.) gq	-=1.;
  while(gq < 0.)  gq	+=1.;
  gq	*=c2gp;

  ESBiSplRAP(&dgY,&ra,&rq,&dF,&za,&zq,&B,&Ba,&Bq,&gh,&gha,&ghq,a,gq);

  switch(Next){
  case 1:
    p	+=0.05;
    break;
  case 2:
    p	-=0.05;
    break;
  default:
    break;
  }

  V	=cV1keV*sqrt(E);
  Vtot	=V;

  if(fabs(p) > 1.) p=p > 0.? 1. : -1.;
  pVb	=p;
  
  if(FlBerk == 0){
    Vb	=p*V;
    Vn	=sqrt(1.-p*p)*V;
  }
  else{
    Vn	=sqrt(2.*B*gm);
    if(Vn <= V){
      if(FlBerkS == 0){
	FlBerkS	=1;
	printf("\n%s\n","<Constants of motion:>[1](7%e)");
	printf("!%10s %10s %6s %6s %10s %10s %10s\n","P_gf","v^2_|_/2B"
	       ,"R [m]","Z [m]","E [keV]","bgY [Vsec]","F [T x m]");
	printf(" %10.3e %10.3e %6.3f %6.3f %10.3e %10.3e %10.3e\n"
	       ,Pgf,gm,r,z,E,gY,F);
	printf("%s\n","<Trajectory data>[1](10%e)");
	printf("!%10s %10s %10s %10s %10s %10s\n"
	       ,"E [keV]","t_b [sec]","f_b [Hz]","gw[2gp/s]","a","gq");
	printf("!%32s %10s %10s %10s\n"
	       ,"gY [Vsec]","F [T x m]","R [m]","Z [m]");
      }
      i	=0;
      do{
	ESSetSplA(a);
	splRA(&F,&dF,ESFF,ESFF2a);
	F	=sqrt(F);
	dF	*=cr2/F;
	splRA(&gY,&dgY,ESgY,ESgY2a);
	Vn	=sqrt(2.*B*gm);
	Vb	=sqrt(V*V-Vn*Vn);
	gr	=V/(cgWc*B);
	grb	=Vb/(cgWc*B);
	grn	=Vn/(cgWc*B);
	err	=(Pgf-gY-grb*F)/(dgY+grb*dF
				 -F*(grb*grb+cr2*grn*grn)*Ba/(B*grb));
	a	+=err;
	ESBiSplRAP(&r,&ra,&rq,&z,&za,&zq,&B,&Ba,&Bq,&gh,&gha,&ghq,a,gq);
	err	=fabs(err);
	i++;
      }while(err > 1e-4 && i < 10);
      ES2DMapFlx2Lab(&r,&z,&dF,&dF,&dF,&dF,a,c2gp*gt);
      lBD->Pgf	=Pgf;
      lBD->p	=p;
      lBD->gm	=gm;
      lBD->E	=E;
      lBD->gY	=gY;
      lBD->F	=F;
      lBD->a	=a;
      lBD->q	=gq;
      lBD->r	=r;
      lBD->z	=z;
      CbStr2UserMessage("err=%10.3e Iter=%d",err,i);
      p	=Vb/V;
    }
    else{
      Vb	=p*V;
      Vn	=sqrt(1.-p*p)*V;
    }
  }

  gr	=V/(cgWc*B);
  grb	=Vb/(cgWc*B);
  grn	=Vn/(cgWc*B);
  grn0	=grn;
  B0	=B;
  if(FlBerk == 0){
    ESSetSplA(a);
    splRA(&F,NULL,ESFF,ESFF2a);
    F	=sqrt(F);
    splRA(&gY,NULL,ESgY,ESgY2a);
    Pgf	=gY+grb*F;
    gm	=cr2*Vn*Vn/B;
#ifdef H
    out("sdddd","Pgf",Pgf,gm,V,B);
#endif
    FlBerkS	=0;

    lBD->Pgf	=Pgf;
    lBD->p	=p;
    lBD->gm	=gm;
    lBD->E	=E;
    lBD->gY	=gY;
    lBD->F	=F;
    lBD->a	=a;
    lBD->q	=gq;
    lBD->r	=r;
    lBD->z	=z;
  }
  Tc	=1./B;
  n	=0;
  if(L == 0){
    CbUserPlot	=NULL;
  }
  else{
    gmP		=cr2*Vn*Vn/(B*cgWc*cgWc);
    yGC[0]	=Vb/(B*cgWc);
    yGC[1]	=a;
    yGC[2]	=gq;
    yGC[3]	=0.;

    InitLsode4Orbit(t*c2gp*ESaR0[0]/(V*T0),sqrt(grb*grb+grn*grn)*abstol);
    switch(L){
    case 1:
      CbUserPlot	=GCOrbitPlot2D;
      break;
    case 2:
      CbUserPlot	=GCOrbitPlot2DLarmor;
      break;
    case 3:
      FractionLosses(); 
      CbUserPlot	=GCOrbitPlot2DLoss;
      break;
    case 4:
      CbUserPlot	=GCOrbitPlot2DLarmor;
      FractionLosses();
      break;
    case 5:
      if(GCInitRKSolv(nPart) == 0){
	for(k=0; k < nPart; k++){
	  A	=0.05+0.9*k/nPart;
	  A	=1.-k/nPart;
	  Xa[k] =a;
	  Xgq[k]=gq;
	  Xgf[k]=0.;
	  Xgr[k]=A*gr;
	  Xgm[k]=cr2*(1.-A*A)*gr*gr*B;
	}
	CbUserPlot	=GCOrbitPlot2DA;
      }
      break;
    case 6:
      CbUserPlot	=GCOrbitPlot2DB;
      break;
    }
  }
#endif /*Tbl_Orbits*/
  FreeMem4BasicFunctions();
  GCDeInitRKSolv();
  ESDeInitOrbLSODE();
  ESFreeRZspaceEq();
  return(0);
}

int ESFractionOfTrapped(double *frac,int i)
{
  int j,k,ki,kj,iK;
  double pB[ESNp1],D[ESNp1],D0,aph,Bmx;
  double rc[ESFp1],rs[ESFp1];
  double rct[ESFp1],rst[ESFp1],rca[ESFp1],rsa[ESFp1];
  double r,ra,rt,za,zt;
  double b,ba,dgY,pF;
  double cs,sn;
  double gt[ESNp1];

  if(i == 0){
    return(0);
  }

  dgY		=ESsa[i]*ESdgY[i];
  pF		=ESFF[i];
  rc[0]		=ESaR0[0]+rcT[i];
  rs[0]		=ESaZ0[0]+rsT[i];
  rca[0]	=rcT1a[i];
  rsa[0]	=rsT1a[i];
  b		=ESsb[i];
  ba		=ESsb1a[i];
  ki	=i;
  iK	=1;
  for(k=1; k < ESFp1; k++){
    ki	+=ESNa1;
    rc[k]	=2.*rcT[ki];
    rct[k]	=2.*k*rsT[ki];
    rca[k]	=2.*rcT1a[ki];
    rs[k]	=2.*rsT[ki];
    rst[k]	=-2.*k*rcT[ki];
    rsa[k]	=2.*rsT1a[ki];
    if(rc[k] != 0. || rca[k] != 0. || rs[k] != 0. || rsa[k] != 0.){
      iK	=k+1;
    }
  }

  aph	=0.;
  Bmx	=0.;
  D0	=0.;
  for(j=0; j < ESNp; j++){
    r	=rc[0];
    ra	=rca[0];
    rt	=0.;
    cs	=EScs1[j];
    sn	=ESsn1[j];
    za	=rsa[0]+ba*sn;
    zt	=b*cs;
    kj	=0;
    for(k=1; k < iK; k++){
      kj	+=j;
      if(kj >= ESNp){
	kj	-=ESNp;
      }
      cs	=EScs1[kj];
      sn	=ESsn1[kj];
      r		+=rc[k]*cs+rs[k]*sn;
      ra	+=rca[k]*cs+rsa[k]*sn;
      rt	+=rct[k]*cs+rst[k]*sn;
    }
    D[j]	=r*(ra*zt-rt*za);
    pB[j]	=pF/(r*r)+(rt*rt+zt*zt)*dgY*dgY/(D[j]*D[j]);
    aph		+=pB[j]*D[j];
    D0		+=D[j];
    if(Bmx < pB[j]){
      Bmx	=pB[j];
    } 
  }
  D[ESNp]	=D[0];
  pB[ESNp]	=pB[0];
  aph	/=Bmx*D0;
  for(j=0; j < ESNp1; j++){
    pB[j]	=sqrt(pB[j]/Bmx);
    gt[j]	=ESgt[j]*cr2gp;
  }
  iK	=41;
  ba	=1./(iK-1);
  ra	=0.;
  pF	=0.;
  for(k=1; k < iK; k++){
    b	=ba*k;
    za	=ra;
    ra	=0.;
    for(j=0; j < ESNp; j++){
      ra	+=sqrt(1.-b*pB[j])*D[j];
    }
    ra	=b*D0/ra;
    pF	+=cr2*ba*(ra+za);
  }
  *frac	=1.-0.75*aph*pF;
  return(0);
}

int ESFractionOfTrapped1(double *frac,double a)
{
  int j,k,ki,kj,iK;
  double pB[ESNp1],D[ESNp1],D0,aph,Bmx;
  double rc[ESFp1],rs[ESFp1];
  double rct[ESFp1],rst[ESFp1],rca[ESFp1],rsa[ESFp1];
  double r,ra,rt,za,zt;
  double b,ba,dgY,pF;
  double cs,sn;
  double gt[ESNp1];

  ESSetSplA(a);
  splRA(&dgY,NULL,ESdgY,ESdgY2a);
  dgY		*=a;
  splRA(&pF,NULL,ESFF,ESFF2a);

  splRA(rc,rca,rcT,rcT2a);
  rc[0]		+=ESaR0[0];
  splRA(rs,rsa,rsT,rsT2a);
  rs[0]		+=ESaZ0[0];
  splRA(&b,&ba,ESsb,ESsb2a);
  ki	=0;
  iK	=1;
  for(k=1; k < ESFp1; k++){
    ki	+=ESNa1;
    splRA(rc+k,rca+k,rcT+ki,rcT2a+ki);
    rc[k]	*=2.;
    rca[k]	*=2.;
    rst[k]	=-k*rc[k];
    splRA(rs+k,rsa+k,rsT+ki,rsT2a+ki);
    rs[k]	*=2.;
    rsa[k]	*=2.;
    rct[k]	=k*rs[k];
    if(rc[k] != 0. || rca[k] != 0. || rs[k] != 0. || rsa[k] != 0.){
      iK	=k+1;
    }
  }
  aph	=0.;
  Bmx	=0.;
  D0	=0.;
  for(j=0; j < ESNp; j++){
    r	=rc[0];
    ra	=rca[0];
    rt	=0.;
    cs	=EScs1[j];
    sn	=ESsn1[j];
    za	=rsa[0]+ba*sn;
    zt	=b*cs;
    kj	=0;
    for(k=1; k < iK; k++){
      kj	+=j;
      if(kj >= ESNp){
	kj	-=ESNp;
      }
      cs	=EScs1[kj];
      sn	=ESsn1[kj];
      r		+=rc[k]*cs+rs[k]*sn;
      ra	+=rca[k]*cs+rsa[k]*sn;
      rt	+=rct[k]*cs+rst[k]*sn;
    }
    D[j]	=r*(ra*zt-rt*za);
    pB[j]	=pF/(r*r)+(rt*rt+zt*zt)*dgY*dgY/(D[j]*D[j]);
    aph		+=pB[j]*D[j];
    D0		+=D[j];
    if(Bmx < pB[j]){
      Bmx	=pB[j];
    } 
  }
  D[ESNp]	=D[0];
  pB[ESNp]	=pB[0];
  aph	/=Bmx*D0;
  for(j=0; j < ESNp1; j++){
    pB[j]	=sqrt(pB[j]/Bmx);
    gt[j]	=ESgt[j]*cr2gp;
  }
  iK	=41;
  ba	=1./(iK-1);
  ra	=0.;
  pF	=0.;
  for(k=1; k < iK; k++){
    b	=ba*k;
    za	=ra;
    ra	=0.;
    for(j=0; j < ESNp; j++){
      ra	+=sqrt(1.-b*pB[j])*D[j];
    }
    ra	=b*D0/ra;
    pF	+=cr2*ba*(ra+za);
  }
  *frac	=1.-0.75*aph*pF;
  return(0);
}

double ESjbs[128]={128*0.};

int ESBootstrap1()
{
  int iboot,i,j,ierr,nout;
  double zeff,rm2avx,epsx,rmajx,qx,ftrapx,
  pvale,ppvale,tempxe,tempxpe,pvali,ppvali,tempxi,tempxpi;
  double amu0=1.256637061e-6,echarge=1.6022e-19,emass=9.1095e-31
    ,pmass=1.6726e-27,xeps0=8.854e-12;
  double zave,xf,alfi,denom,anum1,anum2,tepote,tipoti,xmi,xclog,xdense,xdensi,
  xvthe,xvthi,xtaue,xtaui,xnuse,xnusi,xl31,xl32,col1,col2,alfhh;
  double a13,b13,c13,a23,b23,c23,s,ss;
  double jbs;
  int n1;
  double x[2],Y[65],YY0[65],YY[65],fr[65];
  double A[65]={
    0.0000e+00,
    1.2559e-01,
    1.7571e-01,
    2.1396e-01,
    2.4601e-01,
    2.7399e-01,
    2.9908e-01,
    3.2200e-01,
    3.4308e-01,
    3.6263e-01,
    3.8100e-01,
    3.9833e-01,
    4.1469e-01,
    4.3017e-01,
    4.4503e-01,
    4.5917e-01,
    4.7274e-01,
    4.8571e-01,
    4.9826e-01,
    5.1037e-01,
    5.2206e-01,
    5.3335e-01,
    5.4435e-01,
    5.5506e-01,
    5.6544e-01,
    5.7564e-01,
    5.8553e-01,
    5.9526e-01,
    6.0477e-01,
    6.1414e-01,
    6.2336e-01,
    6.3246e-01,
    6.4142e-01,
    6.5031e-01,
    6.5914e-01,
    6.6791e-01,
    6.7661e-01,
    6.8531e-01,
    6.9401e-01,
    7.0270e-01,
    7.1144e-01,
    7.2017e-01,
    7.2899e-01,
    7.3786e-01,
    7.4686e-01,
    7.5595e-01,
    7.6512e-01,
    7.7450e-01,
    7.8401e-01,
    7.9372e-01,
    8.0362e-01,
    8.1381e-01,
    8.2422e-01,
    8.3497e-01,
    8.4611e-01,
    8.5764e-01,
    8.6936e-01,
    8.8214e-01,
    8.9515e-01,
    9.0877e-01,
    9.2376e-01,
    9.3965e-01,
    9.5680e-01,
    9.7623e-01,
    1.0000e+00
  };
  double Ti2[65],Ti[65]={
    2.1510e+01,
    2.1600e+01,
    2.1630e+01,
    2.1640e+01,
    2.1610e+01,
    2.1560e+01,
    2.1490e+01,
    2.1400e+01,
    2.1300e+01,
    2.1170e+01,
    2.1020e+01,
    2.0860e+01,
    2.0690e+01,
    2.0500e+01,
    2.0290e+01,
    2.0070e+01,
    1.9830e+01,
    1.9590e+01,
    1.9330e+01,
    1.9050e+01,
    1.8760e+01,
    1.8460e+01,
    1.8150e+01,
    1.7830e+01,
    1.7490e+01,
    1.7150e+01,
    1.6790e+01,
    1.6420e+01,
    1.6040e+01,
    1.5650e+01,
    1.5260e+01,
    1.4850e+01,
    1.4430e+01,
    1.4000e+01,
    1.3570e+01,
    1.3120e+01,
    1.2670e+01,
    1.2220e+01,
    1.1750e+01,
    1.1280e+01,
    1.0810e+01,
    1.0330e+01,
    9.8450e+00,
    9.3580e+00,
    8.8690e+00,
    8.3790e+00,
    7.8880e+00,
    7.3970e+00,
    6.9070e+00,
    6.4190e+00,
    5.9330e+00,
    5.4500e+00,
    4.9700e+00,
    4.4940e+00,
    4.0230e+00,
    3.5560e+00,
    3.0930e+00,
    2.6350e+00,
    2.1830e+00,
    1.7380e+00,
    1.3050e+00,
    8.8930e-01,
    5.0730e-01,
    1.8700e-01,
    4.6500e-03
  };
  int n;
  double r1,r2,z,dra,dza,drp,dzp,a,gt,Lc,Vc;
  extern double ESgaP[];

  n	=64;
  n1	=65;
  z	=0.;
  f2spl(Ti,Ti2,&z,NULL,Ti,NULL,A,&n,ESgaP,ESgaP+1,ESgaP+2,ESgaP+3);
  ierr	=0;
  nout	=6;

  iboot	=2;

  xmi	=2.;
  xclog	=17.0;

  Y[0]	=0.;
  YY[0]	=0.;
  YY0[0]=0.;
  ESjbs[0]	=0.;
  i	=0;
  zeff	=1.;
  zave	=1.;
  for(i=1; i < 65; i++){
    a	=A[i];
    ESSetSplA(a);

    splRA(&Lc,NULL,ESLc,ESLc2);
    splRA(&Vc,NULL,ESVc,ESVc2);
    rm2avx	=Lc/((Lc+Vc)*ESaR0[0]*ESaR0[0]);
    gt	=0.;
    ES2DMapFlx2Lab(&r2,&z,&dra,&dza,&drp,&dzp,a,gt);
    gt	=cgp;
    ES2DMapFlx2Lab(&r1,&z,&dra,&dza,&drp,&dzp,a,gt);
    rmajx	=cr2*(r2+r1);
    epsx	=cr2*(r2-r1)/rmajx;
    splRA(&qx,NULL,ESsq,ESsq2a);
    splRA(&z,NULL,ESsp,ESsp2a);


    s	=1./zave;
    pvale	=z/(1.+s);
    pvali	=z*s/(1.+s);
    if(pvale == 0.){
      pvale	=1e-6;
    }
    if(pvali == 0.){
      pvali	=1e-6;
    }
    splRA(&z,NULL,ESPs,ESPs2a);
    ppvale	=-z/(1.+s);
    ppvali	=-z*s/(1.+s);
    tempxe	=Ti[i];
    tempxi	=Ti[i];

    tempxpe	=0.;
    tempxpi	=0.;
    splr0(&tempxe,&tempxpe,&a,Ti,Ti2,A,&n);
    splRA(&z,NULL,ESdgY,ESdgY2a);
    z	*=a;
    tempxpe	/=-z;
    tempxpi	=tempxpe;

    ESFractionOfTrapped1(&ftrapx,a);

    fr[i]	=ftrapx;
    xf	=ftrapx/(1.-ftrapx);
    alfi=1.172/(1.0+0.462*xf);

    denom=(1.414+zeff)*zeff+xf*(0.754+(2.657+2.*zeff)*zeff
				+xf*(0.348+(1.243+zeff)*zeff));
    anum1	=xf*(0.754+(2.21+zeff)*zeff+xf*(0.348+(1.24+zeff)*zeff));
    anum2	=xf*(0.884+2.07*zeff);

    tepote	=tempxpe/tempxe;
    tipoti	=tempxpi/tempxi;

    YY0[i]	=(anum1*(ESPs[i]+pvali*alfi*tipoti)+anum2*pvale*tepote)
      /(denom*rm2avx);
#ifdef H
    printf("%2d%10.3e%10.3e%10.3e%10.3e%10.3e%10.3e%10.3e\n"
	   ,i,sqrt(epsx),anum1/(xf*denom),anum1/(denom*sqrt(epsx))
	   ,1./(ESaR0[0]*rm2avx),1./(ESaR0[0]*ESaR0[0]*rm2avx)
	   ,sqrt(epsx)*ESPs[i]*crgm0*ESaR0[0],YY0[i]*crgm0/ESaR0[0]);
    printf("%2d frac=%10.3e x=%10.3e\n",i,ftrapx,xf);
    YY0[i]	=crgm0*anum1*ESPs[i]/(denom*rm2avx*ESaR0[0]);
#endif
    if(iboot == 1){
      jbs=-(anum1*(ppvale+ppvali-pvali*alfi*tipoti)-anum2*pvale*tepote)
	/(denom*rm2avx);
      return(0);
    }
#ifdef H
    zave	=pvali != 0. ? (pvale/tempxe)/(pvali/tempxi) : 0.;
    zave	=tempxi/tempxe;
#endif

    if(zeff <= 2.0){
      a13	=1.02-0.13*(zeff-1.0);
      b13	=1.07-0.45*(zeff-1.0);
      c13	=1.07-0.38*(zeff-1.0);
      a23	=0.57-0.05*(zeff-1.0);
      b23	=0.61-0.27*(zeff-1.0);
      c23	=0.61-0.23*(zeff-1.0);
    }
    else{
      a13	=0.89-0.05*(zeff-2.0);
      b13	=0.62-0.03*(zeff-2.0);
      c13	=0.69-0.09*(zeff-2.0);
      a23	=0.52-0.02*(zeff-2.0);
      b23	=0.34-0.005*(zeff-2.0);
      b23	=0.34-0.005*(zeff-2.0);
      c23	=0.38-0.05*(zeff-2.0);
    }
    xdense	=pvale/(amu0*tempxe*echarge*1.e3);
    xdensi	=pvali/(amu0*tempxi*echarge*1.e3);
    xvthe	=sqrt(2.0*echarge*1.e3*tempxe/emass);
    xvthi	=sqrt(2.0*echarge*1.e3*tempxi/(xmi*pmass));

    s	=xeps0*emass*xvthe;
    ss	=echarge*echarge;
    xtaue=(12.0*cgp*sqrt(cgp)*s*s*xvthe)/(4.0*xdense*zeff*ss*ss*xclog);
    s	=xeps0*xmi*pmass*xvthi;
    ss	=zave*echarge;
    ss	*=ss;
    xtaui=1.414*(12.0*cgp*sqrt(cgp)*s*s*xvthi)/(4.0*xdensi*ss*ss*xclog);
    s	=epsx*sqrt(epsx);
    xnuse=1.414*(rmajx*qx)/(xtaue*xvthe*s);
    xnusi=1.414*(rmajx*qx)/(xtaui*xvthi*s);
    xl31=anum1/denom;
    xl32=anum2/denom;
    col1=1./((1.+a13*sqrt(xnuse)+b13*xnuse)*(1.+c13*xnuse*s));
    col2=1./((1.+a23*sqrt(xnuse)+b23*xnuse)*(1.+c23*xnuse*s));

    s	*=s;
    ss	=s*xnusi*xnusi;
    s	=1.+ss;
    alfhh=((alfi-0.35*sqrt(xnusi))/(1.+0.7*sqrt(xnusi))-2.1*ss)
      /((1.+ss)*(1.+xnuse*xnuse*s));
    ESjbs[i]=-(xl31*col1*(ppvale+ppvali-alfhh*pvali*tipoti)
	       -pvale*(2.5*xl31*col1-(2.5*xl31-xl32)*col2)*tepote)/rm2avx;
    Y[i]	=crgm0*ESjbs[i];
    ESjbs[i]	/=ESaR0[0];
    YY0[i]	*=crgm0/ESaR0[0];
    out("sddd","Te",tempxe,tempxpe,Y[i]);
  }
  return(0);
}

int ESBootstrap()
{
  int iboot,i,j,ierr,nout;
  double zeff,rm2avx,epsx,rmajx,qx,ftrapx,
  pvale,ppvale,tempxe,tempxpe,pvali,ppvali,tempxi,tempxpi;
  double amu0=1.256637061e-6,echarge=1.6022e-19,emass=9.1095e-31
    ,pmass=1.6726e-27,xeps0=8.854e-12;
  double zave,xf,alfi,denom,anum1,anum2,tepote,tipoti,xmi,xclog,xdense,xdensi,
  xvthe,xvthi,xtaue,xtaui,xnuse,xnusi,xl31,xl32,col1,col2,alfhh;
  double a13,b13,c13,a23,b23,c23,s,ss;
  double jbs;
  double x[4],Y[ESNa1],YY0[ESNa1],YY[ESNa1],fr[ESNa1];

  ierr	=0;
  nout	=6;

  iboot	=2;

  xmi	=2.;
  xclog	=17.0;

  Y[0]	=0.;
  YY[0]	=0.;
  YY0[0]=0.;
  ESjbs[0]	=0.;
  YY[0]	=crgm0*ESjb[0];
  i	=0;
#ifdef H
  printf("%2s %11s %11s %11s %11s\n","i"
	 ,"a","jb MA/m^2","jBS MA/m^2","jBS0 MA/m^2");
  printf("%2d %11.4e %11.4e %11.4e %11.4e\n",0,0.,ESjb[0]*crgm0,0.,0.);
#endif
  x[2]	=0.;
  x[3]	=0.;
  for(i=1; i < ESNa1; i++){
    zeff	=1.2;
    zeff	=1.;
    zeff	=1.6940;
    zave	=1.21;

    zeff	=1.;
    zave	=1.;

    rm2avx	=ESLc[i]/((ESLc[i]+ESVc[i])*ESaR0[0]*ESaR0[0]);
    j		=ESNp1*i;
    rmajx	=cr2*(ESsr[j]+ESsr[j+ESNp/2]);
    epsx	=cr2*(ESsr[j]-ESsr[j+ESNp/2])/rmajx;
    qx		=ESsq[i];

    s	=1./zave;
    pvale	=ESsp[i]/(1.+s);
    pvali	=ESsp[i]*s/(1.+s);

    if(pvale == 0.){
      pvale	=1e-6;
    }
    if(pvali == 0.){
      pvali	=1e-6;
    }
    ppvale	=-ESPs[i]/(1.+s);
    ppvali	=-ESPs[i]*s/(1.+s);

    tempxe	=15.;
    tempxi	=15.;
    tempxpe	=0.;
    tempxpi	=0.;
    ESFractionOfTrapped(&ftrapx,i);
#ifdef H
    bootstrappp_(YY+i,&iboot, &j, &nout,
		 &zeff,&rm2avx,&epsx,&rmajx,&qx,&ftrapx,
		 &pvale,&ppvale,&tempxe,&tempxpe,
		 &pvali,&ppvali,&tempxi,&tempxpi);
    YY[i]	*=crgm0/ESaR0[0];
#endif

    YY[i]	=crgm0*1.46*sqrt(epsx)*ESaR0[0]*ESPs[i];

    fr[i]	=ftrapx;
    xf	=ftrapx/(1.-ftrapx);
    alfi=1.172/(1.0+0.462*xf);

    denom=(1.414+zeff)*zeff+xf*(0.754+(2.657+2.*zeff)*zeff
				+xf*(0.348+(1.243+zeff)*zeff));
    anum1	=xf*(0.754+(2.21+zeff)*zeff+xf*(0.348+(1.24+zeff)*zeff));
    anum2	=xf*(0.884+2.07*zeff);

    tepote	=tempxpe/tempxe;
    tipoti	=tempxpi/tempxi;

    YY0[i]	=(anum1*(ESPs[i]+pvali*alfi*tipoti)+anum2*pvale*tepote)
      /(denom*rm2avx);
#ifdef H
    printf("%2d%10.3e%10.3e%10.3e%10.3e%10.3e%10.3e%10.3e\n"
	   ,i,sqrt(epsx),anum1/(xf*denom),anum1/(denom*sqrt(epsx))
	   ,1./(ESaR0[0]*rm2avx),1./(ESaR0[0]*ESaR0[0]*rm2avx)
	   ,1.46*sqrt(epsx)*ESPs[i]*crgm0*ESaR0[0],YY0[i]*crgm0/ESaR0[0]);
    printf("%2d frac=%10.3e x=%10.3e\n",i,ftrapx,xf);
    YY0[i]	=crgm0*anum1*ESPs[i]/(denom*rm2avx*ESaR0[0]);
#endif
    if(iboot == 1){
      jbs=-(anum1*(ppvale+ppvali-pvali*alfi*tipoti)-anum2*pvale*tepote)
	/(denom*rm2avx);
      return(0);
    }
#ifdef H
    zave	=pvali != 0. ? (pvale/tempxe)/(pvali/tempxi) : 0.;
    zave	=tempxi/tempxe;
#endif

    if(zeff <= 2.0){
      a13	=1.02-0.13*(zeff-1.0);
      b13	=1.07-0.45*(zeff-1.0);
      c13	=1.07-0.38*(zeff-1.0);
      a23	=0.57-0.05*(zeff-1.0);
      b23	=0.61-0.27*(zeff-1.0);
      c23	=0.61-0.23*(zeff-1.0);
    }
    else{
      a13	=0.89-0.05*(zeff-2.0);
      b13	=0.62-0.03*(zeff-2.0);
      c13	=0.69-0.09*(zeff-2.0);
      a23	=0.52-0.02*(zeff-2.0);
      b23	=0.34-0.005*(zeff-2.0);
      b23	=0.34-0.005*(zeff-2.0);
      c23	=0.38-0.05*(zeff-2.0);
    }
    xdense	=pvale/(amu0*tempxe*echarge*1.e3);
    xdensi	=pvali/(amu0*tempxi*echarge*1.e3);
    xvthe	=sqrt(2.0*echarge*1.e3*tempxe/emass);
    xvthi	=sqrt(2.0*echarge*1.e3*tempxi/(xmi*pmass));
    s	=xeps0*emass*xvthe;
    ss	=echarge*echarge;
    xtaue=(12.0*cgp*sqrt(cgp)*s*s*xvthe)/(4.0*xdense*zeff*ss*ss*xclog);
    s	=xeps0*xmi*pmass*xvthi;
    ss	=zave*echarge;
    ss	*=ss;
    xtaui=1.414*(12.0*cgp*sqrt(cgp)*s*s*xvthi)/(4.0*xdensi*ss*ss*xclog);
    s	=epsx*sqrt(epsx);
    xnuse=1.414*(rmajx*qx)/(xtaue*xvthe*s);
    xnusi=1.414*(rmajx*qx)/(xtaui*xvthi*s);
    xl31=anum1/denom;
    xl32=anum2/denom;
    col1=1./((1.+a13*sqrt(xnuse)+b13*xnuse)*(1.+c13*xnuse*s));
    col2=1./((1.+a23*sqrt(xnuse)+b23*xnuse)*(1.+c23*xnuse*s));

    s	*=s;
    ss	=s*xnusi*xnusi;
    s	=1.+ss;
    alfhh=((alfi-0.35*sqrt(xnusi))/(1.+0.7*sqrt(xnusi))-2.1*ss)
      /((1.+ss)*(1.+xnuse*xnuse*s));
    ESjbs[i]=-(xl31*col1*(ppvale+ppvali-alfhh*pvali*tipoti)
	       -pvale*(2.5*xl31*col1-(2.5*xl31-xl32)*col2)*tepote)/rm2avx;
    ESjbs[i]	/=ESaR0[0];
    Y[i]	=crgm0*ESjbs[i];
    YY0[i]	*=crgm0/ESaR0[0];

    YY[i]	=crgm0*ESjb[i];

    s	=crgm0*1.46*sqrt(epsx);
    ss	=s*ESPs[i]/(rm2avx*ESaR0[0]);
    ss	=s*ESPs[i]*ESaR0[0];


    ESjbs[i]	*=0.75;
#ifdef H
    printf("%2d %11.4e %11.4e %11.4e %11.4e %11.4e %11.4e\n"
	   ,i,ESsa[i],ESjb[i]*crgm0
	   ,Y[i],YY0[i],ss,1.46*sqrt(epsx)*denom/anum1);
#endif
    if(x[3] < Y[i]) x[3]=Y[i];
    if(x[3] < YY0[i]) x[3]=YY0[i];
  }
  x[0]	=0.;
  x[1]	=1.;
  
  SetPlotName("#","j_Bootstrap [MA/m^2]","Bootstrap");
  Scale2d(2,x,x+2,2,6,2);
  Plot2d(2,ESsa,Y,ESNa1,6,0,0,0);
  Plot2d(2,ESsa,YY0,ESNa1,6,0,14,0);
  return(0);
}

#ifdef H
      subroutine bootstrap(ajavbs, iboot, ierr, nout,
     1  zeff,rm2avx,epsx,rmajx,qx,ftrapx,
     2  pvale,ppvale,tempxe,tempxpe,
     3  pvali,ppvali,tempxi,tempxpi,xnuse,xnusi)
c
c     DESCRIPTION OF RETURNED PARAMETERS:
c     ajavbs   surface averaged parallel current due to bootstrap
c              divided by surface averaged B dot grad phi
c              (divide by mu0=4*pi*1.e-7 to get amperes/meter)
c     ierr     =0 for normal calculation
c              =1 for error exit
c
c     DESCRIPTION OF INPUT PARAMETERS:
c     iboot    =1 for collisionless bootstrap calculation
c              =2 to include collisonal corrections
c     nout     logical unit for error output
c
c     zeff     effective charge
c     rm2avx   surface average of 1/R**2
c     epsx     inverse aspect ratio of this flux surface
c     rmajx    major radius of this flux surface (m)
c     qx       safety factor
c     ftrapx   trapped particle fraction
c     pvale    electron pressure times mu0
c     ppvale   derivative of electron pressure wrt psi  (PF/radian)
c     pvali    ion pressure times mu0
c     ppvali   derivative of ion pressure wrt psi       (PF/radian)
c     tempxe   electron temperature (kev)
c     tempxpe  derivative of electron pressure wrt psi  (PF/radian)
c     tempxi   ion temperature      (kev)
c     tempxpi  derivative of ion temperature wrt psi    (PF/radian)
c
c
      data amu0/1.256637061e-6/
      data pi/3.1415926535/
      data echarge/1.6022e-19/
      data emass/9.1095e-31/
      data pmass/1.6726e-27/
      data xeps0/8.854e-12/
c
      ierr = 0
      if(tempxe.le.0 .or. tempxi.le.0) go to 101
      if(ftrapx.le.0 .or. ftrapx.ge.1) go to 102
      if(pvale .le.0 .or. pvali .le.0) go to 103
      if(rm2avx .le. 0) go to 104
      if(zeff .lt. 1.0) go to 105
      if(iabs(iboot).lt.1 .or. iabs(iboot).gt.2) go to 106
      zave = (pvale/tempxe)/(pvali/tempxi)
      xf=ftrapx/(1.0-ftrapx)
      alfi = 1.172/(1.0+0.462*xf)
      denom = 1.414*zeff + zeff**2 + xf*(0.754+2.657*zeff+2.*zeff**2)
     1                          + xf**2*(0.348+1.243*zeff+   zeff**2)
      anum1 = xf*(0.754 + 2.21*zeff + zeff**2
     1      + xf*(0.348 + 1.24*zeff + zeff**2) )
      anum2 = xf*(0.884 + 2.07*zeff)
      pepope=ppvale/pvale
      pipopi=ppvali/pvali
      tepote=tempxpe/tempxe
      tipoti=tempxpi/tempxi
      if(abs(iboot) .eq. 1) then
      ajavbs=-(pvale/rm2avx)*(anum1*(pepope+(pvali/pvale)
     1          *(pipopi - alfi*tipoti) ) - anum2*tepote) / denom
      return
      endif
c
      if(zeff .le. 2.0) then
      a13=1.02-0.13*(zeff-1.0)
      b13=1.07-0.45*(zeff-1.0)
      c13=1.07-0.38*(zeff-1.0)
      a23=0.57-0.05*(zeff-1.0)
      b23=0.61-0.27*(zeff-1.0)
      c23=0.61-0.23*(zeff-1.0)
      else
      a13=0.89-0.05*(zeff-2.0)
      b13=0.62-0.03*(zeff-2.0)
      c13=0.69-0.09*(zeff-2.0)
      a23=0.52-0.02*(zeff-2.0)
      b23=0.34-0.005*(zeff-2.0)
      c23=0.38-0.05*(zeff-2.0)
      endif
      xmi=1.0
      xclog=17.0
      xdense=pvale/(amu0*tempxe*echarge*1.e3)
      xdensi=pvali/(amu0*tempxi*echarge*1.e3)
      xvthe=sqrt(2.0*echarge*1.e3*tempxe/emass)
      xvthi=sqrt(2.0*echarge*1.e3*tempxi/(xmi*pmass))
      xtaue=(12.0*pi**1.5*xeps0**2*(emass)**2*xvthe**3)/
     +(4.0*xdense*zeff*(echarge)**4*xclog)
      xtaui=1.414*(12.0*pi**1.5*xeps0**2*(xmi*pmass)**2*
     +xvthi**3)/(4.0*xdensi*(zave*echarge)**4*xclog)
      xnuse=1.414*(rmajx*qx)/(xtaue*xvthe*epsx**1.5)
      xnusi=1.414*(rmajx*qx)/(xtaui*xvthi*epsx**1.5)
      xl31=anum1/denom
      xl32=anum2/denom
      col1=(1.0/(1.0+a13*sqrt(xnuse)+b13*xnuse))*
     +(1.0/(1.0+c13*xnuse*epsx**1.5))
      col2=(1.0/(1.0+a23*sqrt(xnuse)+b23*xnuse))*
     +(1.0/(1.0+c23*xnuse*epsx**1.5))
      alfhh=((alfi-0.35*sqrt(xnusi))/(1.0+0.7*sqrt(xnusi))
     +-2.1*xnusi**2*epsx**3)*(1.0/(1.0+xnusi**2*epsx**3))*
     +(1.0/(1.0+xnuse**2*epsx**3))
      ajavbs=-(pvale/rm2avx)*(xl31*col1*(pepope+(pvali/pvale)
     +*(pipopi-alfhh*tipoti))-(2.5*xl31*col1-(2.5*xl31-xl32)*col2)
     +*tepote)
c
      return
c
c.....error exit
  101 continue
      write(nout,1101) tempxe,tempxi
 1101 format(" Error in bootstrap, tempxe,tempxi=",1p2e12.4)
      go to 200
  102 continue
      write(nout,1102) ftrapx
 1102 format(" Error in bootstrap, ftrapx=",1pe12.4)
      go to 200
  103 continue
      write(nout,1103) pvale,pvali
 1103 format(" Error in bootstrap, pvale,pvali =",1p2e12.4)
      go to 200
  104 continue
      write(nout,1104) rm2avx
 1104 format(" Error in bootstrap, rm2avx=",1pe12.4)
      go to 200
  105 continue
      write(nout,1105) zeff
 1105 format(" Error in bootstrap, zeff=",1pe12.4)
      go to 200
  106 continue
      write(nout,1106) iboot
 1106 format(" Error in bootstrap, iboot=",i5)
c
  200 ierr = 1
      return
      end
#endif

#ifdef H
       subroutine bootstrap(ajavbs, iboot, ierr, nout,
      1  zeff,rm2avx,epsx,rmajx,qx,ftrapx,
      2  pvale,ppvale,tempxe,tempxpe,
      3  pvali,ppvali,tempxi,tempxpi )

c                                             6/27/94 scj
c
c     DESCRIPTION OF RETURNED PARAMETERS:
c     ajavbs   surface averaged J dot B due to bootstrap
c              divided by surface averaged B dot grad phi
c              (divide by mu0=4*pi*1.e-7 to get amperes/meter)
c     ierr     =0 for normal calculation
c              =1 for error exit
c
c     DESCRIPTION OF INPUT PARAMETERS:
c     iboot    =1 for collisionless bootstrap calculation
c              =2 to include collisonal corrections
c     nout     logical unit for error output
c
c     zeff     effective charge
c     rm2avx   surface average of 1/R**2
c     epsx     inverse aspect ratio of this flux surface
c     rmajx    major radius of this flux surface (m)
c     qx       safety factor
c     ftrapx   trapped particle fraction
c     pvale    electron pressure times mu0
c     ppvale   derivative of electron pressure wrt psi  (PF/radian)
c     pvali    ion pressure times mu0
c     ppvali   derivative of ion pressure wrt psi       (PF/radian)
c     tempxe   electron temperature (kev)
c     tempxpe  derivative of electron pressure wrt psi  (PF/radian)
c     tempxi   ion temperature      (kev)
c     tempxpi  derivative of ion temperature wrt psi    (PF/radian)
c
c
      data amu0/1.256637061e-6/
      data pi/3.1415926535/
      data echarge/1.6022e-19/
      data emass/9.1095e-31/
      data pmass/1.6726e-27/
      data xeps0/8.854e-12/
      data igoof/0/
c
      ierr = 0
      if(tempxe.le.0 .or. tempxi.le.0) go to 101
      if(ftrapx.le.0 .or. ftrapx.ge.1) go to 102
      if(pvale .le.0 .or. pvali .le.0) go to 103
      if(rm2avx .le. 0) go to 104
      if(zeff .lt. 0.9) go to 105
      if(iboot.lt.1 .or. iboot.gt.2) go to 106
      zave = (pvale/tempxe)/(pvali/tempxi)
      xf=ftrapx/(1.0-ftrapx)

      alfi = 1.172/(1.0+0.462*xf)

      denom = 1.414*zeff + zeff**2 + xf*(0.754+2.657*zeff+2.*zeff**2)
     1                          + xf**2*(0.348+1.243*zeff+   zeff**2)
      anum1 = xf*(0.754 + 2.21*zeff + zeff**2
     1      + xf*(0.348 + 1.24*zeff + zeff**2) )
      anum2 = xf*(0.884 + 2.07*zeff)
      pepope=ppvale/pvale
      pipopi=ppvali/pvali
      tepote=tempxpe/tempxe
      tipoti=tempxpi/tempxi
      if(abs(iboot) .eq. 1) then
      ajavbs=-(pvale/rm2avx)*(anum1*(pepope+(pvali/pvale)
     1          *(pipopi - alfi*tipoti) ) - anum2*tepote) / denom
      return
      endif
c
      if(zeff .le. 2.0) then
      a13=1.02-0.13*(zeff-1.0)
      b13=1.07-0.45*(zeff-1.0)
      c13=1.07-0.38*(zeff-1.0)
      a23=0.57-0.05*(zeff-1.0)
      b23=0.61-0.27*(zeff-1.0)
      c23=0.61-0.23*(zeff-1.0)
      else
      a13=0.89-0.05*(zeff-2.0)
      b13=0.62-0.03*(zeff-2.0)
      c13=0.69-0.09*(zeff-2.0)
      a23=0.52-0.02*(zeff-2.0)
      b23=0.34-0.005*(zeff-2.0)
      b23=0.34-0.005*(zeff-2.0)
      c23=0.38-0.05*(zeff-2.0)
      endif
      xmi=1.0
      xclog=17.0
      xdense=pvale/(amu0*tempxe*echarge*1.e3)
      xdensi=pvali/(amu0*tempxi*echarge*1.e3)
      xvthe=sqrt(2.0*echarge*1.e3*tempxe/emass)
      xvthi=sqrt(2.0*echarge*1.e3*tempxi/(xmi*pmass))
      xtaue=(12.0*pi**1.5*xeps0**2*(emass)**2*xvthe**3)/
     +(4.0*xdense*zeff*(echarge)**4*xclog)
      xtaui=1.414*(12.0*pi**1.5*xeps0**2*(xmi*pmass)**2*
     +xvthi**3)/(4.0*xdensi*(zave*echarge)**4*xclog)
      xnuse=1.414*(rmajx*qx)/(xtaue*xvthe*epsx**1.5)
      xnusi=1.414*(rmajx*qx)/(xtaui*xvthi*epsx**1.5)
      xl31=anum1/denom
      xl32=anum2/denom
      col1=(1.0/(1.0+a13*sqrt(xnuse)+b13*xnuse))*
     +(1.0/(1.0+c13*xnuse*epsx**1.5))
      col2=(1.0/(1.0+a23*sqrt(xnuse)+b23*xnuse))*
     +(1.0/(1.0+c23*xnuse*epsx**1.5))
      alfhh=((alfi-0.35*sqrt(xnusi))/(1.0+0.7*sqrt(xnusi))
     +-2.1*xnusi**2*epsx**3)*(1.0/(1.0+xnusi**2*epsx**3))*
     +(1.0/(1.0+xnuse**2*epsx**3))
      ajavbs=-(pvale/rm2avx)*(xl31*col1*(pepope+(pvali/pvale)
     +*(pipopi-alfhh*tipoti))-(2.5*xl31*col1-(2.5*xl31-xl32)*col2)
     +*tepote)
c
      return
c
c.....error exit
  101 continue
      write(nout,1101) tempxe,tempxi
 1101 format(" Error in bootstrap, tempxe,tempxi=",1p2e12.4)
      go to 200
  102 continue
      write(nout,1102) ftrapx
 1102 format(" Error in bootstrap, ftrapx=",1pe12.4)
      go to 200
  103 continue
      write(nout,1103) pvale,pvali
 1103 format(" Error in bootstrap, pvale,pvali =",1p2e12.4)
      go to 200
  104 continue
      write(nout,1104) rm2avx
 1104 format(" Error in bootstrap, rm2avx=",1pe12.4)
      go to 200
  105 continue
      write(nout,1105) zeff
 1105 format(" Error in bootstrap, zeff=",1pe12.4)
      go to 200
  106 continue
      write(nout,1106) iboot
 1106 format(" Error in bootstrap, iboot=",i5)
c
  200 igoof=igoof+1
      if(igoof.gt.10) ierr=1
      return
      end
#endif

int esc2ESIn()
{
  static int Fic=0;
  int iG,Npa;
  int i,j,ji;
  double f[ESNp1],fa[ESNp1],fq[ESNp1],faq[ESNp1],rr0;
  FILE *lF;
  char FNm[16];
  char Date[32];
  int kD;
  double t;

  kD	=sizeof(double);
  ID	=2;
  Npa	=ESNp1*ESNa1;

  sprintf(FNm,"esiA.00");
  sprintf(FNm,"esiA.%2.2d",Fic);
  lF	=fopen(FNm,"w");

  time(&stTime);
  ltm	=localtime(&stTime);
  sprintf(Date,"Date: %2.2d/%2.2d/%2.2d at %2.2d:%2.2d"
	  ,ltm->tm_mon+1,ltm->tm_mday,ltm->tm_year%100,ltm->tm_hour
	  ,ltm->tm_min); 
  fprintf(lF,"!!! Do not edit this file\n");

  /* Group 0 */
  iG	=0;
  fprintf(lF,"%s[1]%s %s\n",GrName[iG],GrFormat[iG],Date);
  fprintf(lF,"!%3s x %3s %6s %5s %6s %13s | %5s %s\n"
	  ,GrComment[iG][0],GrComment[iG][1],GrComment[iG][2],GrComment[iG][3]
	  ,"RBtor","Name","Btor"," - this is a comment starting with !");
  fprintf(lF,"%4d x %3d %6.6d %5.3f %6.3f %13s   %5.3f %s\n"
	  ,ESNp1,ESNa1,ID,ESRext,ESRBt,"ESC",ESBt
	  ," - after-record comment (after Name)");

  /* Group 1 is optional if equidistant and  0 <=gq <= 2*gp */
  iG	=1;
  fprintf(lF,"\n%s[1](%d%s)\n",GrName[iG],ESNp1,GrFormat[iG]);
  for(j=0; j < ESNp1; j++){
    fprintf(lF,"%24.16e",ESgt[j]);
    if(j%4 == 3) fputc('\n',lF);
  }
  if(j%4 != 0) fputc('\n',lF);

  /* Group 2 optional if equidistant and 0 <= a <= 1 */
  iG	=2;
  fprintf(lF,"\n%s[1](%d%s)\n",GrName[iG],ESNa1,GrFormat[iG]);
  for(i=0; i < ESNa1; i++){
    fprintf(lF,"%24.16e",ESsa[i]);
    if(i%4 == 3) fputc('\n',lF);
  }
  if(i%4 != 0) fputc('\n',lF);

  /* Group 3 is ignored by ESI and dropped here */
  iG	=3;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],ESNa1,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  for(i=0; i < ESNa1; i++){
#ifdef H
    fprintf(lF,"%24.16e%24.16e%24.16e%24.16e\n"
	    ,ESaF[i],ESaF1a[i],ESsp[i],ESsp1a[i]);
#endif
    fprintf(lF,"%24.16e%24.16e%24.16e%24.16e\n"
	    ,ESaF[i],crgm0*ESsp[i],c2gp*ESgF[i],c2gp*(ESgY[i]-ESgY[ESNa]));
  }

  rr0	=1./ESaR0[0];
  iG	=4;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],ESNa1,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  for(i=0; i < ESNa1; i++){
    f[i]	=ESLc[i]*ESaF[i]*rr0;
    fq[i]	=(ESLc1[i]*ESaF[i]+ESLc[i]*ESaF1a[i])*rr0;
    fprintf(lF,"%24.16e%24.16e%24.16e%24.16e\n"
	    ,f[i],fq[i],ESdgY[i],ESdgY1a[i]);
  }

  /* Group 5 */
  iG	=5;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],ESNa1,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  splA1a(ESaT,ESaT1a,ESaT2a);
  for(i=0; i < ESNa1; i++){
    fprintf(lF,"%24.16e%24.16e%24.16e%24.16e\n"
	    ,ESaT[i],ESaT1a[i],ESPs[i],ESPs1a[i]);
  }

  /* Group 6 */
  iG	=6;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],Npa,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  for(i=0; i < ESNa1; i++){
    ji	=ESNp1*i+ESNp1;
    for(j=0; j < ESNp1; j++){
      ji--;
      fprintf(lF,"%24.16e%24.16e%24.16e%24.16e\n"
	      ,ESsr[ji],ESsra[ji],-ESsrt[ji],-ESsrat[ji]);
    }
  }

  /* Group 7 */
  iG	=7;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],Npa,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  for(i=0; i < ESNa1; i++){
    ji	=ESNp1*i+ESNp1;
    for(j=0; j < ESNp1; j++){
      ji--;
      fprintf(lF,"%24.16e%24.16e%24.16e%24.16e\n"
	      ,ESsz[ji],ESsza[ji],-ESszt[ji],-ESszat[ji]);
    }
  }

  /* Group 8 */
  iG	=8;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],Npa,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  for(i=0; i < ESNa1; i++){
    ji	=ESNp1*i+ESNp1;
    for(j=0; j < ESNp1; j++){
      ji--;
      fprintf(lF,"%24.16e%24.16e%24.16e%24.16e\n"
	      ,ESaB[ji],ESaBa[ji],-ESaBt[ji],-ESaBat[ji]);
    }
  }

  /* Group 9 */
  iG	=9;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],Npa,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  for(i=0; i < ESNa1; i++){
    ji	=ESNp1*i+ESNp1;
    for(j=0; j < ESNp1; j++){
      ji--;
      fprintf(lF,"%24.16e%24.16e%24.16e%24.16e\n"
	      ,ESgH[ji],ESgHa[ji],-ESgHt[ji],-ESgHat[ji]);
    }
  }
  CbStr2UserMessage("Output: %s\n",FNm);
  fclose(lF);
  Fic++;
  return(0);
}

int Esc2ESI(char *CoordNm,double *sr,double*sz,double*aB,double*gH
	       ,int Na,int Np)
{
  static int Fic[4]={0,0,0,0},ic=0;
  int Na1,Np1,Npa,iG;
  int i,j,ji;
  double *srt,*sra,*srat;
  double *szt,*sza,*szat;
  double *aBt,*aBa,*aBat;
  double *gHt,*gHa,*gHat;
  double rr0,a,da,t,dt;
  double F,Fa,dgF,dgFa,dgY,dgYa,L,La,T,Ta,P,Pa;

  FILE *lF;
  char FNm[16];
  char Date[32];
  int kD;

  kD	=sizeof(double);
  ID	=2;

  Na1	=Na+1;
  Np1	=Np+1;
  Npa	=Np1*Na1;
  da	=(ESsa[ESNa]-ESsa[0])/Na;
  dt	=c2gp/Np;

  i	=Np1*Na1;
  sra	=sr	+i;
  srt	=sra	+i;
  srat	=srt	+i;
  sza	=sz	+i;
  szt	=sza	+i;
  szat	=szt	+i;
  aBa	=aB	+i;
  aBt	=aBa	+i;
  aBat	=aBt	+i;
  if(gH != NULL){
    gHa	=gH	+i;
    gHt	=gHa	+i;
    gHat=gHt	+i;
  }

  *Date	=CoordNm != NULL ? *CoordNm : 'A';
  switch(*Date){
  case 'P':
    ic	=1;
    ID	=302;
    break;
  case 'B':
    ic	=2;
    ID	=402;
    break;
  case 'H':
    ic	=3;
    ID	=202;
    break;
  case 'C':
    ic	=3;
    ID	=602;
    break;
  default:
    ic	=0;
    ID	=2;
    break;
  }

  sprintf(FNm,"esi%c.%2.2d",*Date,Fic[ic]);
  sprintf(FNm,"esi%c.00",*Date);
  lF	=fopen(FNm,"w");

  time(&stTime);
  ltm	=localtime(&stTime);
  sprintf(Date,"Date: %2.2d/%2.2d/%2.2d at %2.2d:%2.2d"
	  ,ltm->tm_mon+1,ltm->tm_mday,ltm->tm_year%100,ltm->tm_hour
	  ,ltm->tm_min); 
  fprintf(lF,"!!! Do not edit this file\n");

  /* Group 0 */
  iG	=0;
  fprintf(lF,"%s[1]%s %s\n",GrName[iG],GrFormat[iG],Date);
  fprintf(lF,"!%3s x %3s %6s %5s %6s %13s | %5s %s\n"
	  ,GrComment[iG][0],GrComment[iG][1],GrComment[iG][2],GrComment[iG][3]
	  ,"RBtor","Name","Btor"," - this is a comment starting with !");
  fprintf(lF,"%4d x %3d %6.6d %5.3f %6.3f %13s   %5.3f %s\n"
	  ,Np1,Na1,ID,ESRext,ESRBt,CoordNm != NULL ? CoordNm : "ESC",ESBt
	  ," - after-record comment (after Name)");

#ifdef H
  /* Group 1 is optional if equidistant and  0 <=gq <= 2*gp */
  iG	=1;
  fprintf(lF,"\n%s[1](%d%s)\n",GrName[iG],Np1,GrFormat[iG]);
  for(j=0; j < Np1; j++){
    fprintf(lF,"%24.16e",dt*j);
    if(j%4 == 3) fputc('\n',lF);
  }
  if(j%4 != 0) fputc('\n',lF);

  /* Group 2 optional if equidistant and 0 <= a <= 1 */
  iG	=2;
  fprintf(lF,"\n%s[1](%d%s)\n",GrName[iG],ESNa1,GrFormat[iG]);
  for(i=0; i < Na1; i++){
    fprintf(lF,"%24.16e",da*i);
    if(i%4 == 3) fputc('\n',lF);
  }
  if(i%4 != 0) fputc('\n',lF);
#endif

  /* Group 3 is ignored by ESI and dropped here */
  iG	=3;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],Na1,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  for(i=0; i < Na1; i++){
    a	=da*i;
    ESSetSplA(a);
    splRA(&F,&Fa,ESFF,ESFF2a);
    splRA(&dgY,&dgYa,ESsp,ESsp2a);
    F	=sqrt(F);
    Fa	/=2.*F;
    fprintf(lF,"%24.16e%24.16e%24.16e%24.16e\n",F,Fa,dgY,dgYa);
  }

  rr0	=1./ESaR0[0];
  /* Group 4 */
  iG	=4;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],Na1,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  for(i=0; i < Na1; i++){
    a	=da*i;
    ESSetSplA(a);
    splRA(&F,&Fa,ESFF,ESFF2a);
    F	=sqrt(F);
    Fa	/=2.*F;
    splRA(&L,&La,ESLc,ESLc2);
    dgF	=rr0*L*F;
    dgFa	=rr0*(La*F+L*Fa);
    splRA(&dgY,&dgYa,ESdgY,ESdgY2a);
    fprintf(lF,"%24.16e%24.16e%24.16e%24.16e\n",dgF,dgFa,dgY,dgYa);
  }

  /* Group 5 */
  iG	=5;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],Na1,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  for(i=0; i < Na1; i++){
    a	=da*i;
    ESSetSplA(a);
    splRA(&T,&Ta,ESaT,ESaT2a);
    splRA(&P,&Pa,ESPs,ESPs2a);
    fprintf(lF,"%24.16e%24.16e%24.16e%24.16e\n",T,Ta,P,Pa);
  }

  /* Group 6 */
  iG	=6;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],Npa,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  for(i=0; i < Na1; i++){
    ji	=Np1*i+Np1;
    for(j=0; j < Np1; j++){
      ji--;
      fprintf(lF,"%24.16e%24.16e%24.16e%24.16e\n"
	      ,sr[ji],sra[ji],-srt[ji],-srat[ji]);
    }
  }

  /* Group 7 */
  iG	=7;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],Npa,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  for(i=0; i < Na1; i++){
    ji	=Np1*i+Np1;
    for(j=0; j < Np1; j++){
      ji--;
      fprintf(lF,"%24.16e%24.16e%24.16e%24.16e\n"
	      ,sz[ji],sza[ji],-szt[ji],-szat[ji]);
    }
  }

  /* Group 8 */
  iG	=8;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],Npa,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  for(i=0; i < Na1; i++){
    ji	=Np1*i+Np1;
    for(j=0; j < Np1; j++){
      ji--;
      fprintf(lF,"%24.16e%24.16e%24.16e%24.16e\n"
	      ,aB[ji],aBa[ji],-aBt[ji],-aBat[ji]);
    }
  }

  /* Group 9 */
  iG	=9;
  if(gH != NULL){
    fprintf(lF,"\n%s[%d]%s\n",GrName[iG],Npa,GrFormat[iG]);
    fprintf(lF,"!%23s%24s%24s%24s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
    for(i=0; i < Na1; i++){
      ji	=Np1*i+Np1;
      for(j=0; j < Np1; j++){
	ji--;
	fprintf(lF,"%24.16e%24.16e%24.16e%24.16e\n"
		,gH[ji],gHa[ji],-gHt[ji],-gHat[ji]);
      }
    }
  }
  else{
    fprintf(lF,"\n%s[%d]%s\n",GrName[iG],0,GrFormat[iG]);
    fprintf(lF,"!%23s%24s%24s%24s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  }
  fclose(lF);
  CbStr2UserMessage("Output: %s ",FNm);
  Fic[ic]++;
  return(0);
}

int esc2esi(FILE *lF)
{
  int iG,Npa;
  int i,j,ji;
  double f[ESNp1],fa[ESNp1],fq[ESNp1],faq[ESNp1],rr0;

  char FNm[16];
  char Date[32];

  int kD;
  double t;

  kD	=sizeof(double);
  ID	=2;
  Npa	=ESNp1*ESNa1;

  time(&stTime);
  ltm	=localtime(&stTime);
  sprintf(Date,"Date: %2.2d/%2.2d/%2.2d at %2.2d:%2.2d"
	  ,ltm->tm_mon+1,ltm->tm_mday,ltm->tm_year%100,ltm->tm_hour
	  ,ltm->tm_min); 
  fprintf(lF,"!!! Do not edit this file\n");

  /* Group 0 */
  iG	=0;
  fprintf(lF,"%s[1]%s %s\n",GrName[iG],GrFormat[iG],Date);
  fprintf(lF,"!%3s x %3s %6s %5s %6s %13s | %5s %s\n"
	  ,GrComment[iG][0],GrComment[iG][1],GrComment[iG][2],GrComment[iG][3]
	  ,"RBtor","Name","Btor"," - this is a comment starting with !");
  fprintf(lF,"%4d x %3d %6.6d %5.3f %6.3f %13s   %5.3f %s\n"
	  ,ESNp1,ESNa1,ID,ESRext,ESRBt,"ESC",ESBt
	  ," - after-record comment (after Name)");

  /* Group 1 is optional if equidistant and  0 <=gq <= 2*gp */
  iG	=1;
  fprintf(lF,"\n%s[1](%d%s)\n",GrName[iG],ESNp1,GrFormat[iG]);
  for(j=0; j < ESNp1; j++){
    fprintf(lF,"%24.16e",ESgt[j]);
    if(j%4 == 3) fputc('\n',lF);
  }
  if(j%4 != 0) fputc('\n',lF);

  /* Group 2 optional if equidistant and 0 <= a <= 1 */
  iG	=2;
  fprintf(lF,"\n%s[1](%d%s)\n",GrName[iG],ESNa1,GrFormat[iG]);
  for(i=0; i < ESNa1; i++){
    fprintf(lF,"%24.16e",ESsa[i]);
    if(i%4 == 3) fputc('\n',lF);
  }
  if(i%4 != 0) fputc('\n',lF);

  /* Group 3 is ignored by ESI */
  iG	=3;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],ESNa1,GrFormat[iG]);
  fprintf(lF,"!%22s%24s%24s%24s%4s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3],"#");
  for(i=0; i < ESNa1; i++){
    fprintf(lF,"%23.16e%24.16e%24.16e%24.16e%4d\n"
	    ,ESaF[i],crgm0*ESsp[i],c2gp*ESgF[i],c2gp*(ESgY[i]-ESgY[ESNa]),i);
  }

  /* Group 4 */
  iG	=4;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],ESNa1,GrFormat[iG]);
  fprintf(lF,"!%22s%24s%24s%24s%4s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3],"#");
  splA1a(ESaT,ESaT1a,ESaT2a);
  for(i=0; i < ESNa1; i++){
    fprintf(lF,"%23.16e%24.16e%24.16e%24.16e%4d\n"
	    ,ESaT[i],ESaT1a[i],ESPs[i],ESPs1a[i],i);
  }

  /* Group 5 */
  rr0	=1./ESaR0[0];
  iG	=5;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],ESNa1,GrFormat[iG]);
  fprintf(lF,"!%22s%24s%24s%24s%4s\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3],"#");
  for(i=0; i < ESNa1; i++){
    f[i]	=ESLc[i]*ESaF[i]*rr0;
    fq[i]	=(ESLc1[i]*ESaF[i]+ESLc[i]*ESaF1a[i])*rr0;
    fprintf(lF,"%23.16e%24.16e%24.16e%24.16e%4d\n"
	    ,f[i],fq[i],ESdgY[i],ESdgY1a[i],i);
  }

  /* Group 6 */
  iG	=6;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],Npa,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s  ia  jq\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  for(i=0; i < ESNa1; i++){
    ji	=ESNp1*i+ESNp1;
    for(j=0; j < ESNp1; j++){
      ji--;
      fprintf(lF,"%24.16e%24.16e%24.16e%24.16e %3d %3d\n"
	      ,ESsr[ji],ESsra[ji],-ESsrt[ji],-ESsrat[ji],i,j);
    }
  }

  /* Group 7 */
  iG	=7;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],Npa,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s  ia  jq\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  for(i=0; i < ESNa1; i++){
    ji	=ESNp1*i+ESNp1;
    for(j=0; j < ESNp1; j++){
      ji--;
      fprintf(lF,"%24.16e%24.16e%24.16e%24.16e %3d %3d\n"
	      ,ESsz[ji],ESsza[ji],-ESszt[ji],-ESszat[ji],i,j);
    }
  }

  /* Group 8 */
  iG	=8;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],Npa,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s  ia  jq\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  for(i=0; i < ESNa1; i++){
    ji	=ESNp1*i+ESNp1;
    for(j=0; j < ESNp1; j++){
      ji--;
      fprintf(lF,"%24.16e%24.16e%24.16e%24.16e %3d %3d\n"
	      ,ESaB[ji],ESaBa[ji],-ESaBt[ji],-ESaBat[ji],i,j);
    }
  }

  /* Group 9 */
  iG	=9;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],Npa,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s  ia  jq\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  for(i=0; i < ESNa1; i++){
    ji	=ESNp1*i+ESNp1;
    for(j=0; j < ESNp1; j++){
      ji--;
      fprintf(lF,"%24.16e%24.16e%24.16e%24.16e %3d %3d\n"
	      ,ESgH[ji],ESgHa[ji],-ESgHt[ji],-ESgHat[ji],i,j);
    }
  }

  /* Group 10 */
  iG	=10;
  fprintf(lF,"\n%s[%d]%s\n",GrName[iG],Npa,GrFormat[iG]);
  fprintf(lF,"!%23s%24s%24s%24s  ia  jq\n",GrComment[iG][0],GrComment[iG][1]
	  ,GrComment[iG][2],GrComment[iG][3]);
  for(i=0; i < ESNa1; i++){
    ji	=ESNp1*i+ESNp1;
    for(j=0; j < ESNp1; j++){
      ji--;
      fprintf(lF,"%24.16e%24.16e%24.16e%24.16e %3d %3d\n"
	      ,ESgQ[ji],ESgQa[ji],-ESgQt[ji],-ESgQat[ji],i,j);
    }
  }
  return(0);
}

int esc2ESI()
{
  static int Fic=0;
  FILE *lF;
  char FNm[0x40];

  sprintf(FNm,"esiA.00");
  sprintf(FNm,"esiA.%2.2d",Fic);

  lF	=fopen(FNm,"w");
  esc2esi(lF);
  fclose(lF);

  CbStr2UserMessage("Output: %s\n",FNm);
  return(0);
  Fic++;
  return(0);
}
