/* Equilibrium reconstruction routines with Eddy Currents */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <math.h>

#include "pfc.h"
extern char ShotNm[];

static struct dirent*direntp;
static struct stat statbuf;
static DIR *dirp;

#define NPFC 0x20
#define NMFP 0x100
#define KRSP 0x40

static RAWDATA *Raw,*RawY;

extern double cr2,cr3,cr4,cr6,cr12,cgp,c2gp,cgp_4,cr2gp,cgm0,crgm0;
extern int kCol[];

#ifdef H
typedef struct{
  int iA;
  int K;
  int K1;
  double err;
  double dT;
  double tk[0x80];
  double r0[0x80];
  double r2[0x80];
  char Nm[32];
}RESPFUNC;
#endif

extern unsigned int ESmemlevP;
extern int nPFC,nPFC1,nPEC;
extern PFCOIL *PFC;
extern int nMFP;
extern MFPROBE *MFP;
extern int kTimeHistory;
extern double **gYiC;
extern int iSignIp;

static double A0=0.;

static RESPFUNC RF[NMFP][NPFC];

int PFGiveRawAddress(RAWDATA *lR)
{
  Raw	=lR;
  RawY	=lR+nPFC+1;
#ifdef H
  CheckSensorC();
#endif
  return(0);
}

int CheckSensorC()
{
  int i,j,k;
  double *ld;
  double s,ss;
  RAWDATA *lR,*lS;

  j	=nMFP+nPFC1;
  lS	=Raw;
  j	=0;

  for(i=0; i < j; i++,lS++) out("???? i dd<s>",i,lS->C,lS->G,lS->Nm);

  lS	=Raw+nPFC+1+2;
  for(i=2; i < 6; i++){
    ld	=gYiC[i];
    if(lS->kA){
      s	=lS->C/lS->G;
      lR	=Raw+1;
      for(j=1; j < nPFC; j++){
	k	=RF[i][j].K;
	ss	=RF[i][j].r0[k]*s*lR->G/lR->C;
	out("???? ii dd",i,j,ld[j],ss);
	lR++;
      }
    }
    lS++;
  }
  return(0);
}

int SigLayOutBI(FILE *lF)
{
  static double SensorC[NPFC+NMFP],SensorG[NPFC+NMFP];
  int nI,nS;
  int i,j,k,n,L,Ld,Li;
  char Nm[0x20];
  int kS[NPFC];

  Li	=sizeof(int);
  Ld	=sizeof(double);
  L	=0x20;

  fread(&nI,Li,1,lF);
  if(nI != nPFC+1){
    CbStr2UserMessage("SigLayOut.bi: nI=%d != %d=nPFC+1 \n",nI,nPFC+1);
    return(1);
  }

  memset(kS,'\0',nI*Li);
  k	=0;
  j	=0;
  n	=0;
  while(k < nI){
    for(i=0; i < nI; i++) if(PFC[i].kG == j) kS[n+PFC[i].kD]=k++;
    n	=k;
    j++;
  }
  for(i=0; i < nI; i++){
    memset(Nm,'\0',L);
    fread(Nm,L,1,lF);
    if(strcmp(Nm,PFC[kS[i]].Nm)){
      CbStr2UserMessage("SigLayOut.bi: %d <%s> != <%s>\n",i,Nm,PFC[kS[i]].Nm);
      return(1);
    }
  }
  fread(&nS,Li,1,lF);
  if(nS != nMFP){
    CbStr2UserMessage("SigLayOut.bi: nS=%d != %d=nMFP \n",nS,nMFP);
    return(1);
  }

  for(i=0,j=nI; i < nS; i++,j++){
    memset(Nm,'\0',L);
    fread(Nm,L,1,lF);
    if(strcmp(Nm,MFP[i].Nm)){
      CbStr2UserMessage("SigLayOut.bi: %d <%s> != <%s>\n",i,Nm,MFP[i].Nm);
      return(1);
    }
  }
  k	=nI+nS;
  fread(SensorC,Ld,k,lF);
  fread(SensorG,Ld,k,lF);
#ifdef H
  for(j=0; j < k; j++) out("????*** idd<s>",j,SensorC[j],SensorG[j]
			   ,j < nI ? PFC[j].Nm:MFP[j-nI].Nm);
#endif
  for(i=0; i < nS; i++){
    for(k=0; k < nI; k++){
      j	=kS[k];
      fread(&RF[i][j].iA,Li,1,lF);
      if(RF[i][j].iA){
	fread(&RF[i][j].K,Li,1,lF);
	fread(&RF[i][j].err,Ld,1,lF);
	n	=RF[i][j].K+1;
	fread(RF[i][j].tk,Ld,n,lF);
	fread(RF[i][j].r0,Ld,n,lF);
	fread(RF[i][j].r2,Ld,n,lF);
      }
    }
  }
  if(0){
    int jj;
    double *ld,s,ss;

    for(j=1; j < nPFC; j++) out("??***?? id<s>",j,gYiC[2][j],PFC[j].Nm);

    for(i=2; i < 3; i++){
      if(RF[i][2].iA){
	ld	=gYiC[i];
	s	=SensorC[nI+i]/SensorG[nI+i];
	for(j=1; j < nPFC; j++){
	  jj	=j+1;
	  k	=RF[i][j].K;
	  ss	=RF[i][j].r0[k]*s*SensorG[jj]/SensorC[jj];
	}
      }
    }
  }
  SetSensorC(SensorC,SensorG);
  kTimeHistory	=1;
  return(0);
}

int GetRespFunction(double **r0, double **r2, double **tk,int *K1,int i,int j)
{
  *r0	=RF[i][j].r0;
  *r2	=RF[i][j].r2;
  *tk	=RF[i][j].tk;
  *K1	=RF[i][j].K+1;
  return(0);
}

int ISignal2Splines(double *V0, double *V2, double *V, double *ti, int I1
		  ,double ga3)
{
  int I;
  
  I	=I1-1;
  f2spl(V0,V2,NULL,NULL,V,NULL,ti,&I,&A0,&A0,&A0,&ga3);
  return(0);
}

int HistoryDeltaV2S(double *dS, int j
		    ,double *s0, double *s2, double *tk,int K1
		    ,double *ti,int I1)
{
  int I,K;
  int i,ii,i1,k,kk;
  double d,dd,dd6;
  double f0,f1;
  double h,rh,q0,q1,q2;
  double Q0[KRSP],Q1[KRSP],Q2[KRSP];
  double Y;
  double u0,u1,u2;
  double t,t0,t1,t2,t3;
  double Tk,Ti,Tik;


  I	=I1-1;
  K	=K1-1;
  t0	=ti[0];
  Tk	=tk[K];
  Ti	=ti[I];
  Tik	=Ti+Tk;
  for(kk=0,k=1; k < K1; kk++,k++){
    h	=tk[k]-tk[kk];
    rh	=1./h;
    Q0[k]	=(s0[k]-s0[kk])*rh-(cr6*s2[k]+cr3*s2[kk])*h;
    Q1[k]	=s2[kk];
    Q2[k]	=0.5*(s2[k]-s2[kk])*rh;
  }

  h	=Tk/K;
  rh	=1./h;

  t	=ti[j];
  t1	=t-t0;
  *dS	=0.;
  /*Convolution integrals*/
  ii	=0;
  if(t < Ti){
    i	=ii+1;
    while(i < I && ti[i] < t) i++;
    *dS	+=s0[0];
  }
  else{
    i	=I;
    if(t >= Tik) return(0);
  }
  ii	=i-1;
  t2	=t;
  kk	=0;
  k	=kk+1;
  q0	=Q0[k];
  q1	=Q1[k];
  q2	=Q2[k];
  t3	=t2;
  t2	=tk[k] < t1 ? t-tk[k] : t0;

  if(t2 < Ti){
    if(Ti <= t3){
      h		=tk[k]-tk[kk];
      rh	=1./h;
      f1	=(t-Ti-tk[kk])*rh;
      f0	=1.-f1;
      *dS	+=(f0*s0[kk]+f1*s0[k]
		   -cr6*h*h*f0*f1*((1.+f0)*s2[kk]+(1.+f1)*s2[k]));
    }
    else{
      i1	=i;
      while(i1 && ti[i1] >= t3) i1--;
      i	=i1+1;
      d	=ti[i]-ti[i1];
      dd	=d*d;
      dd6	=cr6*dd;
      if(ti[i] > t3){
	f0	=t3-ti[i1];
	f1	=f0*f0;
	Y	=f0/d;
	u0  =f0*Y*cr2;
	u1	=f1*cr3*Y;
	u2	=f0*f1*0.25*Y;
      }
      else{
	u0  =d*cr2;
	u1  =dd*cr3;
	u2  =d*dd*0.25;
      }
      if(t2 > ti[i1]){
	f0	=t2-ti[i1];
	f1	=f0*f0;
	Y	=f0/d;
	u0  -=f0*cr2*Y;
	u1	-=f1*Y*cr3;
	u2	-=f0*f1*Y*0.25;
      }
      f0	=t-(ti[i1]+tk[kk]);
      *dS	+=q0*u0+q1*(f0*u0-u1)+q2*(f0*(f0*u0-2.*u1)+u2);
    }
  }
  return(0);
}

int HistoryMatR(double *dS,int iMFP,int jPFC)
{
  RESPFUNC *lR;
  RAWDATA *lS;

  lR	=RF[iMFP]+jPFC;
  lS	=Raw+jPFC;
  HistoryDeltaV2S(dS,Raw[0].i,lR->r0,lR->r2,lR->tk,lR->K1,lS->ti,lS->I1);
  *dS	-=lR->r0[lR->K1-1];

  return(0);
}

int HistoryV2S(double *S, int j
	       ,double *s0, double *s2, double *tk,int K1
	       ,double *V0, double *ti,int I1)
{
  int I,K;
  int i,ii,i1,k,kk;
  double d,dd,dd6;
  double f0,f1,p0,p1;
  double h,rh,q0,q1,q2;
  double Q0[KRSP],Q1[KRSP],Q2[KRSP];
  double Y;
  double u0,u1,u2;
  double t,t0,t1,t2,t3;
  double Tk,Ti,Tik;


  I	=I1-1;
  K	=K1-1;
  t0	=ti[0];
  Tk	=tk[K];
  Ti	=ti[I];
  Tik	=Ti+Tk;
  for(kk=0,k=1; k < K1; kk++,k++){
    h	=tk[k]-tk[kk];
    rh	=1./h;
    Q0[k]	=(s0[k]-s0[kk])*rh-(cr6*s2[k]+cr3*s2[kk])*h;
    Q1[k]	=s2[kk];
    Q2[k]	=0.5*(s2[k]-s2[kk])*rh;
  }

  h	=Tk/K;
  rh	=1./h;

  t	=ti[j];
  t1	=t-t0;
  *S	=0.;
  /*Convolution integrals*/
  ii	=0;
  if(t < Ti){
    i	=ii+1;
    while(i < I && ti[i] < t) i++;
    *S	+=s0[0]*V0[i];
  }
  else{
    i	=I;
    if(t >= Tik) return(0);
  }
  ii	=i-1;
  t2	=t;
  for(kk=0; kk < K && tk[kk] < t1; kk++){
    k	=kk+1;
    q0	=Q0[k];
    q1	=Q1[k];
    q2	=Q2[k];
    t3	=t2;
    t2	=tk[k] < t1 ? t-tk[k] : t0;
    if(t2 < Ti){
      if(Ti <= t3){
	h	=tk[k]-tk[kk];
	rh	=1./h;
	f1	=(t-Ti-tk[kk])*rh;
	f0	=1.-f0;
	*S	+=(f0*s0[kk]+f1*s0[k]
		   -cr6*h*h*f0*f1*((1.+f0)*s2[kk]+(1.+f1)*s2[k]))*V0[I];
      }
      else{
	i1	=i;
	while(i1 && ti[i1] >= t3) i1--;
	i	=i1+1;
	while(i && ti[i] > t2){
	  d	=ti[i]-ti[i1];
	  dd	=d*d;
	  dd6	=cr6*dd;
	  p0	=V0[i1];
	  p1	=V0[i]-p0;
	  if(ti[i] > t3){
	    f0	=t3-ti[i1];
	    f1	=f0*f0;
	    Y	=f0/d;
	    u0  =f0*(p0+Y*cr2*p1);
	    u1	=f1*(cr2*p0+cr3*Y*p1);
	    u2	=f0*f1*(cr3*p0+0.25*Y*p1);
	  }
	  else{
	    u0  =d*(p0+cr2*p1);
	    u1  =dd*(cr2*p0+cr3*p1);
	    u2  =d*dd*(cr3*p0+0.25*p1);
	  }
	  if(t2 > ti[i1]){
	    f0	=t2-ti[i1];
	    f1	=f0*f0;
	    Y	=f0/d;
	    u0  -=f0*(p0+cr2*Y*p1);
	    u1	-=f1*(cr2*p0+Y*cr3*p1);
	    u2	-=f0*f1*(cr3*p0+Y*0.25*p1);
	  }
	  f0	=t-(ti[i1]+tk[kk]);
	  *S	+=q0*u0+q1*(f0*u0-u1)+q2*(f0*(f0*u0-2.*u1)+u2);
	  i--;
	  i1--;
	}
      }
    }
  }
  return(0);
}

int HistoryEddyIp2PhysicalSignal(double *S,int iMFP)
{
  int i,j;
  double s;
  RESPFUNC *lR;
  RAWDATA *lS;
  
  i	=Raw[0].i;
  lR	=RF[iMFP]+1;
  lS	=Raw+1;
  s	=0.;
  for(j=1; j < nPFC; j++){
    HistoryV2S(S,Raw[0].i,lR->r0,lR->r2,lR->tk,lR->K1,lS->V,lS->ti,lS->I1);
    s	=(*S)-lR->r0[lR->K1-1]*lS->V[i];
    lR++;
    lS++;
  }
  if(iSignIp) s=-s;
  lS	=RawY+iMFP;
  *S	=s*lS->C/lS->G;
  return(0);
}

int MakeSignalF(double *tj, double *S, int J1
	       ,double *s0, double *s2, double *tk,int K1
	       ,double *V0, double *V2, double *ti,int I1)
{
  int I,K;
  int i,ii,i1,j,k,kk;
  double d,dd,dd6;
  double f0,f1,p0,p1,p2,p3;
  double h,rh,q0,q1,q2;
  double Q0[KRSP],Q1[KRSP],Q2[KRSP];
  double Y;
  double u0,u1,u2;
  double t,t0,t1,t2,t3;
  double Tk,Ti,Tik;


  I	=I1-1;
  K	=K1-1;
  t0	=ti[0];
  Tk	=tk[K];
  Ti	=ti[I];
  Tik	=Ti+Tk;

  for(kk=0,k=1; k < K1; kk++,k++){
    h	=tk[k]-tk[kk];
    rh	=1./h;
    Q0[k]	=(s0[k]-s0[kk])*rh-(cr6*s2[k]+cr3*s2[kk])*h;
    Q1[k]	=s2[kk];
    Q2[k]	=0.5*(s2[k]-s2[kk])*rh;
  }

  h	=Tk/K;
  rh	=1./h;

  S[0]	=0.;
  j	=0;
  while(j < J1 && tj[j] < t0) S[j++]=0.;

  ii	=0;
  for(; j < J1; j++){
    t	=tj[j];
    t1	=t-t0;
#ifdef H
    S[j]=B*(t-tt0);
#endif

    /*Convolution integrals*/
    i	=ii+1;
    if(t < Ti){
      i	=ii+1;
      while(i < I && ti[i] < t) i++;
      ii=i-1;
      d	=ti[i]-ti[ii];
      dd=d*d;
      dd6=cr6*dd;
      f0	=V2[ii]*dd6;
      f1	=V2[i]*dd6;
      p0	=V0[ii];
      p1	=V0[i]-p0-2.*f0-f1;
      p2	=3.*f0;
      p3	=f1-f0;
      Y		=(t-ti[ii])/d;
      S[j]	+=s0[0]*(p0+Y*(p1+Y*(p2+Y*p3)));
    }
    else{
      i	=I;
      if(0 && Tik <= t) S[j] +=s0[K]*V0[I];
    }
    t2	=t;
    for(kk=0; kk < K && tk[kk] < t1; kk++){
      k		=kk+1;
      q0	=Q0[k];
      q1	=Q1[k];
      q2	=Q2[k];
      t3	=t2;
      t2	=tk[k] < t1 ? t-tk[k] : t0;
      if(t2 < Ti){
	if(Ti <= t3){
	  h	=tk[k]-tk[kk];
	  rh	=1./h;
	  f1	=(t-Ti-tk[kk])*rh;
	  f0	=1.-f0;
	  S[j]	+=(f0*s0[kk]+f1*s0[k]
		   -cr6*h*h*f0*f1*((1.+f0)*s2[kk]+(1.+f1)*s2[k]))*V0[I];
	}
	else{
	  i1	=i;
	  while(i1 && ti[i1] >= t3) i1--;
	  i	=i1+1;
	  while(i && ti[i] > t2){
	    d	=ti[i]-ti[i1];
	    dd	=d*d;
	    dd6	=cr6*dd;
	    f0	=V2[i1]*dd6;
	    f1	=V2[i]*dd6;
	    p0	=V0[i1];
	    p1	=V0[i]-p0-2.*f0-f1;
	    p2	=3.*f0;
	    p3	=f1-f0;
	    if(ti[i] > t3){
	      f0	=t3-ti[i1];
	      f1	=f0*f0;
	      Y	=f0/d;
	      u0  =f0*(p0+Y*(cr2*p1+Y*(cr3*p2+0.25*Y*p3)));
	      u1	=f1*(cr2*p0+Y*(cr3*p1+Y*(0.25*p2+0.2*Y*p3)));
	      u2	=f0*f1*(cr3*p0+Y*(0.25*p1+Y*(0.2*p2+cr6*Y*p3)));
	    }
	    else{
	      u0  =d*(p0+cr2*p1+cr3*p2+0.25*p3);
	      u1  =dd*(cr2*p0+cr3*p1+0.25*p2+0.2*p3);
	      u2  =d*dd*(cr3*p0+0.25*p1+0.2*p2+cr6*p3);
	    }
	    if(t2 > ti[i1]){
	      f0	=t2-ti[i1];
	      f1	=f0*f0;
	      Y	=f0/d;
	      u0  -=f0*(p0+Y*(cr2*p1+Y*(cr3*p2+0.25*Y*p3)));
	      u1	-=f1*(cr2*p0+Y*(cr3*p1+Y*(0.25*p2+0.2*Y*p3)));
	      u2	-=f0*f1*(cr3*p0+Y*(0.25*p1+Y*(0.2*p2+cr6*Y*p3)));
	    }
	    f0	=t-(ti[i1]+tk[kk]);
	    S[j]	+=q0*u0+q1*(f0*u0-u1)+q2*(f0*(f0*u0-2.*u1)+u2);
	    i--;
	    i1--;
	  }
	}
      }
    }
  }
  return(0);
}

static double EigenMax,EigenMin; 
static int mSVD,nSVD;
static double *aSVD[0x100],*vSVD[NPFC],wSVD[NPFC],r2[NPFC],EigenSVD[NPFC];

int DeInitSVD()
{
  if(nSVD){
    free(aSVD[0]);
    free(vSVD[0]);
  }
  return(0);
}

int SVD4Wall(double *gYWC[],int m,int n)
{
  static int Start=1;
  int i,j;
  double d;

  if(Start){
    vSVD[0]=(double*)malloc(NPFC*n*sizeof(double));
    for(i=1; i < NPFC; i++) vSVD[i]=vSVD[i-1]+n;
    aSVD[0]=(double*)malloc(0x100*n*sizeof(double));
    for(i=1; i < 0x100; i++) aSVD[i]=aSVD[i-1]+n;
    if(vSVD[0] == NULL || aSVD[0] == NULL){
      CbStr2UserWarning("SVD4Wall(): %d x %d - no memory",0x100,n);
      return(0);
    }
    else{
      ESmemlevP	|=0x00200000;
      Start	=0;
    }
  }
  mSVD	=m;
  nSVD	=n;
  j	=n*sizeof(double);
  for(i=0; i < mSVD; i++) memcpy(aSVD[i],gYWC[i],j);
  SVDcmp(mSVD,nSVD,aSVD,vSVD,wSVD,EigenSVD);
  EigenMax	=0.;
  for(i=0; i < nSVD; i++){
    EigenSVD[i]	=wSVD[i];
    if(EigenMax < wSVD[i]) EigenMax=wSVD[i];
  }
  if(EigenMax == 0.){
    puts("!!! Wrong matrix SVD EigenMax=0.");
    return(1);
  }
  SVDsortR(nSVD,EigenSVD);
  d	=1./EigenMax;
  for(i=0; i < nSVD; i++) EigenSVD[i] *=d;
  return(0);
}

int GetSVDExpansion4Wall(double *I,double *gY,double Emn)
{
  int i,j,n;
  double s,w;
  double *ld;

  n	=0;
  EigenMin	=Emn;
  w	=Emn*EigenMax;
  for(j=0; j < nSVD; j++){
    s	=0.;
    if(wSVD[j] > w){
      for(i=0; i < mSVD; i++) s+=aSVD[i][j]*gY[i];
      s	=s/wSVD[j];
      n++;
    }
    r2[j]=s;
  }
  for(j=0; j < nSVD; j++){
    ld	=vSVD[j];
    s	=0.;
    for(i=0; i < nSVD; i++) s+=(*ld++)*r2[i];
    I[j]	=s;
  }
  return(n);
}

int HistoryPhysicalMatRV(double *dS, double *gY,int iMFP,double Emn)
{
  int i,j;
  double s,w;
  double *ld;
  double V[NPFC];
  RAWDATA *lS;

  EigenMin	=Emn;
  w	=Emn*EigenMax;
  for(j=0; j < nSVD; j++){
    s	=0.;
    if(wSVD[j] > w){
      for(i=0; i < mSVD; i++) s+=aSVD[i][j]*gY[i];
      s	=s/wSVD[j];
    }
    r2[j]=s;
  }
  lS	=Raw+1;
  for(j=0; j < nSVD; j++){
    ld	=vSVD[j];
    s	=0.;
    for(i=0; i < nSVD; i++) s+=(*ld++)*r2[i];
    V[j]	=s*lS->G/lS->G;
    lS++;
  }
  s	=0.;
  for(j=0; j < nSVD; j++){
    HistoryMatR(&w,iMFP,j+1);
    s	+=w*V[j];
  }
  lS	=RawY+iMFP;
  *dS	=s*lS->C/lS->G;
  return(0);
}


int PlotDiscrWall(double *gYWC[],double *Iim,double *gY)
{
  int i,j,iW=4;
  double X[0x100],Y[0x100],x[2],y[2],s;
  
  SetPlotName("i","gY","gY at Wall");

  y[0]	=1e20;
  y[1]	=-1e20;
  for(i=0; i < mSVD; i++){
    X[i]=i+1.;
    s	=0.;
    for(j=0; j < nSVD; j++) s+=gYWC[i][j]*Iim[j];
    if(y[0] > s) y[0]=s;
    if(y[1] < s) y[1]=s;
    Y[i]	=s;
    if(y[0] > gY[i]) y[0]=gY[i];
    if(y[1] < gY[i]) y[1]=gY[i];
  }
  x[0]	=0.;
  x[1]	=mSVD+1.;
  Scale2d(iW,x,y,2,6,2);
  Plot2d(iW,X,gY,mSVD,6,0,0,0);
  Plot2d(iW,X,gY,mSVD,6,0x0008,0,0);
  Plot2d(iW,X,Y,mSVD,6,0,4,0);
  return(0);
}

int PlotEigenSVD()
{
  int k;
  int i,iW=2;
  double eL[NPFC],eLmn;
  double x[NPFC],y[2],s,d,d2;
  char ln[NPFC];
  double v;
  
  d2	=EigenMin*EigenMin;
  s	=0.;
  v	=0.;
  for(i=0; i < nSVD; i++){
    d	=EigenSVD[i]*EigenSVD[i];
    if(d >= d2)  v	+=d;
    s	+=d;
  }
  v	*=100./s;
  
  sprintf(ln,"Variance=%10.3e",v);
  SetPlotName("k","log10 of Eigenvalues",ln);

  y[0]	=100;
  y[1]	=-100;
  for(i=0; i < nSVD; i++){
    eL[i]	=EigenSVD[i] > 1e-10 ? log10(EigenSVD[i]) : -10.;
    if(y[0] > eL[i]) y[0]=eL[i];
    if(y[1] < eL[i]) y[1]=eL[i];
  }
  eLmn	=log10(EigenMin);
  if(y[0] > eLmn) y[0]=eLmn;
  x[0]	=0.;
  x[1]	=nSVD+1.;

  y[0]	=-10.;
  y[1]	=0.;
  Scale2d(iW,x,y,2,6,2);
  i	=0;
  k	=0;
  while(i < nSVD && eL[i] > eLmn){
    x[0]=i+1;
    x[1]=x[0];
    Plot2d(iW,x,y,2,6,0,kCol[k%16],0);
    i++;
  }
  for(i=0; i < nSVD; i++) x[i]=i+1;
  Plot2d(iW,x,eL,nSVD,6,0,4,0);
  y[0]	=eLmn;
  y[1]	=y[0];
  x[0]	=1.;
  x[1]	=nSVD;
  Plot2d(iW,x,y,2,6,0,14,0);
  i	=0;
  while(i < nSVD && eL[i] < eLmn){
    x[0]=i+1;
    x[1]=x[0];
    i++;
  }
  return(0);
}

#ifdef H

char Mess[128];
#ifndef mzl_C2E

char DirNm[32]="../dCDX/2004/Dat/";
static int kTName;

static int nData=0,mData=0;
double *lTime,*lData[256];
int nD[4]={0,0,0,0},i0D=0,i1D=0;
int iD[4][3]={0,0,1,0,0,1,0,0,1,0,0,1};
double *lT[4],*lD[4],*lD2[4];
static double hSS2[KRSP],rSS2[KRSP],tSS2[KRSP];
static double SS2[0x4000],SS2T[0x4000];


static double aLSQ[KRSP][0x4000];
static double dLSQ[0x4000];
static double gALSQ[0x4000],ALSQ[0x4000],gAALSQ[0x4000];

typedef struct{
  char Nm[16];
  int kA;
  int kG;
  int kxy;
  int kspl;
  double x;
  double y;
  double dS;
  double C;
  double G;
  double A;
  double B;
  double dC;
  double dG;
  double dA;
  double dB;
  double ga;
  double *t;
  double *D;
  double *s0;
  double *s2;
  double *S; /* reproduced signal*/
  double *tk;
  double *r0;
  double *r2;
  int K1;
  int i0;
  int i;
  int i1;
  int n;
  int c;
}SIGNAL;

static double A0=0.;
int nSig=0,iSig=0,kSig=0,NSig=64;
int nPFC=0,nY=0,nS=0,nYS=0,nB=0,nMFP;
double *U0=NULL,*U1,*U2;
int iGr[8]={0,0,0,0,0,0,0,0},kGr[8]={0,0,0,0,0,0,0,0};

typedef struct{
  int k;	/*activity index*/
  int kI;	/*index of the calibrating current*/
  int iD[4][2]; /*index of start and end times for I,Y,B*/
  double tW[4][2]; /*start and end times for I,Y,B*/
  int K[4];
  int iA[NSNM];		/*status of the signal:
			  0 - unprocessed;
			  1 - splined;
			  2 - response function is calculated;
			*/
  int kS[NSNM];		/*activity index for each signal*/
  double C[NSNM];	/*calibration coefficient*/; 
  double G[NSNM];	/*gain factor*/; 
  double gaS[NSNM];	/*regularization parameters for splining the signal*/;
  double gaR[NSNM];	/*regularization parameters for response function*/;
  double tol[NSNM];	/*cutoff of SVD reconstruction of response function*/;
  double B[NSNM];	/*slope of the amplifier drift*/; 
  double dT[NSNM];	/*time intervals for response function*/; 
  double errS[NSNM];	/*Error in splining*/; 
  double errR[NSNM];	/*Average Error in reconstruction the signal*/; 
  double errA[NSNM];	/*Maximum Error in reconstruction the signal*/; 
  double tk[NSNM][KRSP];
  double r0[NSNM][KRSP];
  double r2[NSNM][KRSP];
  char Nm[16];
  char NmS[NSNM][16];
}CDX;

typedef struct{
  int K;
  int iA;
  int c;
  double x;
  double y;
  double G;
  double Cx;
  double C;
  double err;
  double rA;
  double dT;
  double *tk;
  double *r0;
  double *r2;
  char *Nm;
}RESPFUNC;
double gYiC[NMFP][NPFC];
RESPFUNC RF[NMFP][NPFC],*lRF;

static int nCdxNm,mCdxNm,NCdxNm=32;
static char CdxNm[32][16];
static int kCdxNm[32];
static int iCdxNm[32];
static int nCdx=0,iCdx=0;
static CDX Cdx[32];

SIGNAL Sig[NSNM];
SIGNAL *lS0[4]={NULL,NULL,NULL,NULL},*llS[256];
double SensorCdx[NSNM],SensorC[NSNM],SensorX[NSNM];
int nSNm=0;
char SNm[NSNM][32];
int kSNm[NSNM];
static int IsRFready=0;

double tWF=0.;
double ga3[4]={1e-13,1e-13,1e-13,1e-13};
double ga3rf[4]={1e-13,1e-13,1e-13,1e-13};
double pX[4]={-10.,-10.,-10.,-10.},pY[4]={0.,0.,0.,0.};
double *px,*py;
char Msk[4][16]={"","","",""};
char *lNmShot;

int TridiagInv(double *A, double *AT, int n1, int Case)
{
  int i,j,n;
  double s,d;
  double V[n1];
  double *ld,*ld1;
  double *r,*h;

  r	=rSS2;
  h	=hSS2;

  n	=n1-1;
  d	=1./n;
  tSS2[0]	=0.;
  hSS2[0]	=0.;
  rSS2[0]	=0.;
  for(i=1; i < n1; i++){
    tSS2[i]	=i*i*d*d;
    hSS2[i]	=(2.*i-1.)*d;

    tSS2[i]	=i*d;
    hSS2[i]	=1.;

    rSS2[i]	=1./hSS2[i];
  }

  memset(A,'\0',n1*n1*LDBL);
  V[0]	=r[2]+r[1];
  s	=1./(h[1]+2.*h[2]);
  V[1]	=(h[1]-h[2])*s;
  s	/=h[1]+h[2];
  ld1	=A+n1;
  ld1[0]=h[2]*r[1]*s;
  ld1[1]=-h[2]*(r[1]+r[2])*s;
  ld1[2]=s;

  for(i=2; i < n; i++){
    j	=i+1;
    s	=1./(2.*(h[i]+h[j])+h[i]*V[i-1]);
    V[i]=-h[j]*s;
    ld	=ld1;
    ld1	+=n1;
    while(j){
      j--;
      ld1[j]	=-h[i]*ld[j]*s;
    }
    j	=i+1;
    ld1[i-1]	+=r[i]*s;
    ld1[i]	-=(r[i]+r[j])*s;
    ld1[j]	=r[j]*s;
  }
  ld	=ld1;
  ld1	+=n1;
  switch(Case){
  case 0:
    s	=1./(2.+V[n-1]);
    for(j=0; j < n1; j++) ld1[j]=-ld[j]*s;
    s	*=r[n]*r[n];
    ld1[n]	-=s;
    ld1[n-1]	+=s;
    break;
  case 1:
    d	=r[n]+r[n-1]-r[n-1]*V[n-2];
    s	=1./(r[n]-d*V[n-1]);
    d	*=s;
    for(j=0; j < n1; j++) ld1[j]=(d*ld[j]-r[n-1]*ld[j-n1]*s);
    break;
  }
  while(i){
    i--;
    s	=V[i];
    for(j=0; j < n1; j++) ld[j]	+=s*ld1[j];
    ld1	=ld;
    ld	-=n1;
  }
  ld	=A+2*n1;
  for(j=0; j < n1; j++) ld1[j]	=(ld1[j]-r[2]*ld[j])*h[1];

  /* Transposition*/
  ld	=A;
  for(i=0; i < n1; i++){
    for(j=0; j < n1; j++)	AT[i+n1*j]=*ld++;
  }
  return(0);
}

int PFSetShotNm(char *ShotNm)
{
  char *lD,*lN;

  CbGetCurrentCbName(&lD,&lN);
  lN	=ShotNm;
  while(*lD != '\0'){
    if(*lD == '/'){
      lN	=ShotNm;
      lD++;
    }
    else{
      *lN++	=*lD++;
    }
  }
  *lN	='\0';
  return(0);
}

int MaxV(double *d,double *y, int n)
{
  double s;
  *d=0.;  /* maximum value */
  while(n){
    n--;
    s	=fabs(y[n]);
    if(*d < s) *d=s;
  }
  return(0);
}

int MaxDiff(double *d,double *a,double *y, double *y1, int n)
{
  int i;
  double s;
  *d=0.;  /* maximum difference */
  *a=0.;  /* averaged difference */
  i	=n;
  while(i){
    i--;
    s	=fabs(y[i]-y1[i]);
    *a +=s*s;
    if(*d < s) *d=s;
  }
  *a	=sqrt(*a/n);
  return(0);
}

int FreeSig()
{
  int j;

  j	=8;
  while(j){
    j--;
    if(kGr[j]){
      iSig	=iGr[j];
      kSig	=kGr[j];
      nD[j]	=0;
      if(Sig[iSig].n){
	while(kSig){
	  kSig--;
	  Sig[iSig+kSig].n =0;
	  free(Sig[iSig+kSig].D);
	}
	free(Sig[iSig].t);
      }
    }
  }
  memset(Sig,'\0',nSig*sizeof(SIGNAL));
  return(0);
}

int SetInterval(int iShot,int *nxt,int iM)
{
  int k,j,j0,j1,n;
  double *t,s;
  double *ld;
  SIGNAL *lS;
  int *jD;
  
  lS	=Sig+iGr[iM];
  t	=lS->t;
  n	=lS->n-1;
  for(k=0; k < 3; k++){
    switch(k){
    case 0:
      ld	=Cdx[iShot].tW[iM];
      jD	=Cdx[iShot].iD[iM];
      break;
    case 1:
      ld=&tWF;
      jD	=&j;
      break;
    case 2:
      ld	=Cdx[iShot].tW[iM]+1;
      jD	=Cdx[iShot].iD[iM]+1;
      break;
    }
    s	=*ld;
    j0	=0;
    j1	=n;
    j	=(j1-j0)/2;
    while(j >j0){
      if(s >= t[j]){
	j0=j;
      }
      else{
	j1=j;
      }
      j	=(j1+j0)/2;
    }
    switch(nxt[k]){
    case 1:
      *ld=t[j];
      break;
    case 2:
      if(j < n-2) j++;
      *ld=t[j];
      nxt[k]=1;
      break;
    case 3:
      if(j) j--;
      *ld=t[j];
      nxt[k]=1;
      break;
    }
    *jD=k < 2 ? j : j+1;
  }
  return(0);
}

int SigSlope(SIGNAL *lS,double te)
{
  int i;
  int i0,n;
  double a,s,d,t0;
  double *t,*S;

  i0=lS->i0+1;
  t	=lS->t;
  S	=lS->D;
  n	=lS->n;
  t0	=t[0];
  i	=0;
  d	=0.;
  s	=0.;
  for(i=0; i < i0; i++){
    a	=t[i]-t0;
    s+=a*S[i];
    d+=a*a;
  }
  for(i=lS->i1; i < n && t[i] <= te; i++){
    a	=t[i]-t0;
    s	+=a*S[i];
    d	+=a*a;
  }
  lS->B=s/d;
  return(0);
}

int BasicIntegrals(double *u0, double *u1, double *u2
		   ,double *V0, double *V2, double *ti,int I1)
{
  int i,i1;
  double d,dd,dd6;
  double f0,f1,p0,p1,p2,p3;

  u0[0]	=0.;
  u1[0]	=0.;
  u2[0]	=0.;
  for(i=1,i1=0; i < I1; i++,i1++){
    d	=ti[i]-ti[i1];
    dd	=d*d;
    dd6	=cr6*dd;
    f0	=V2[i1]*dd6;
    f1	=V2[i]*dd6;
    p0	=V0[i1];
    p1	=V0[i]-p0-2.*f0-f1;
    p2	=3.*f0;
    p3	=f1-f0;
    u0[i]	=d*(p0+cr2*p1+cr3*p2+0.25*p3);
    u1[i]	=dd*(cr2*p0+cr3*p1+0.25*p2+0.2*p3);
    u2[i]	=d*dd*(cr3*p0+0.25*p1+0.2*p2+cr6*p3);
  }
  return(0);
}

int SetMatrixLSQ(double ga)
{
  int i,j;
  double *lA,*la,*ld;

  /*regularization part*/
  lA	=gALSQ;
  la	=ALSQ;
  ld	=gAALSQ;
  for(i=0; i < nSS2; i++){
    lA	+=i;
    la	+=i;
    ld	+=i;
    for(j=i; j < nSS2; j++){
      *ld++=(*la++)+ga*(*lA++);
    }
    *ld++=*la++;
  }
  la	+=i;
  ld	+=i;
  *ld	=*la;
  if(CholDc(gAALSQ,nSVD) < 0){
    CbStr2UserWarning("Cholesly decomposition failed in SetMatrixLSQ()");
    return(1);
  }
  return(0);
}

int Signal2RespFunc(double *r0,double *r2,double *tk,double *S)
{
  int i,j;
  double rh,d;
  double *ld;

  for(i=0; i < nSVD; i++){
    ld	=aLSQ[i];
    d	=0.;
    for(j=0; j < mSVD; j++) d+=(*ld++)*dLSQ[j]*S[j];
    r0[i]	=d;
  }
  CholBksb(gAALSQ,nSVD,r0);

  i	=nSS2-1;
  rh	=i/tk[i];
  rh	*=6.*rh;
  ld	=SS2;
  for(i=0; i < nSS2; i++){
    d	=0.;
    for(j=0; j < nSS2; j++) d+=(*ld++)*r0[j];
    r2[i]	=d*rh;
  }
  return(0);
}

int Spline2RespF(int iBB, double tt0,double *B,double *r0,double *r2
		 ,double *tk,int K1
		 ,double ga,double *S, double *tj, int J1
		 ,double *V0, double *V2, double *ti,int I1)
{
  int I,J,K;
  int i,ii,i1,j,jj,k,kk;
  double d,dd,dd6;
  double f0,f1,p0,p1,p2,p3;
  double h,rh;
  double Y;
  double u0,u1,u2;
  double Tk,Ti,Tj;
  double t,t0,t1,t2,t3;
  int m,mm,n,iB;

  double A[(3*K1+1)*(3*K1+1)];
  double w[2][K1];
  double x[3*K1+1];
  int iA[3*K1+1];
  
  iB	=iBB == 0 ? 1 : 0;

  I	=I1-1;
  J	=J1-1;
  K	=K1-1;
  t0	=ti[0];
  Tk	=tk[K];
  h	=Tk/K;
  rh	=1./h;
  Ti	=ti[I];
  Tj	=Ti+Tk;
  n	=3*K1+1;
  memset(A,'\0',n*n*sizeof(double));
  memset(x,'\0',n*sizeof(double));


  j	=0;
  while(tj[j] < t0) j++;

  ii	=0;
  i	=ii+1;
  for(; j < J1 && tj[j] < Tj; j++){
    /* Variations of signal representations */
    memset(w[0],'\0',K1*LDBL);
    memset(w[1],'\0',K1*LDBL);
    t	=tj[j];
    t1	=t-t0;
    /* Basic integral related to the source*/
    if(t < Ti){
      i	=ii+1;
      while(i < I && ti[i] < t) i++;
      ii=i-1;
      d	=ti[i]-ti[ii];
      dd=d*d;
      dd6=cr6*dd;
      f0	=V2[ii]*dd6;
      f1	=V2[i]*dd6;
      p0	=V0[ii];
      p1	=V0[i]-p0-2.*f0-f1;
      p2	=3.*f0;
      p3	=f1-f0;
      Y		=(t-ti[ii])/d;
      w[0][0]	=p0+Y*(p1+Y*(p2+Y*p3));
    }
    t2	=t;
    for(kk=0; kk < K && tk[kk] < t1; kk++){
      k		=kk+1;
      h		=tk[k]-tk[kk];
      rh	=1./h;
      t3	=t2;
      t2	=tk[k] < t1 ? t-tk[k] : t-t1;
      if(t2 < Ti){
	if(Ti <= t3){
	  f1	=(t-Ti-tk[kk])*rh;
	  f0	=1.-f0;
	  p0	=cr6*h*h*f0*f1*V0[I];
	  w[0][kk]	+=f0*V0[I];
	  w[0][k]	+=f1*V0[I];
	  w[1][kk]	-=p0*(1.+f0);
	  w[1][k]	-=p0*(1.+f1);
	}
	else{
	  i1	=i;
	  while(i1 && ti[i1] >= t3) i1--;
	  i	=i1+1;
	  while(i && ti[i] > t2){
	    d	=ti[i]-ti[i1];
	    dd	=d*d;
	    dd6	=cr6*dd;
	    f0	=V2[i1]*dd6;
	    f1	=V2[i]*dd6;
	    p0	=V0[i1];
	    p1	=V0[i]-p0-2.*f0-f1;
	    p2	=3.*f0;
	    p3	=f1-f0;
	    if(ti[i] > t3){
	      f0	=t3-ti[i1];
	      f1	=f0*f0;
	      Y	=f0/d;
	      u0  =f0*(p0+Y*(cr2*p1+Y*(cr3*p2+0.25*Y*p3)));
	      u1	=f1*(cr2*p0+Y*(cr3*p1+Y*(0.25*p2+0.2*Y*p3)));
	      u2	=f0*f1*(cr3*p0+Y*(0.25*p1+Y*(0.2*p2+cr6*Y*p3)));
	    }
	    else{
	      u0	=U0[i];
	      u1	=U1[i];
	      u2	=U2[i];
	    }
	    if(t2 > ti[i1]){
	      f0	=t2-ti[i1];
	      f1	=f0*f0;
	      Y	=f0/d;
	      u0  -=f0*(p0+Y*(cr2*p1+Y*(cr3*p2+0.25*Y*p3)));
	      u1	-=f1*(cr2*p0+Y*(cr3*p1+Y*(0.25*p2+0.2*Y*p3)));
	      u2	-=f0*f1*(cr3*p0+Y*(0.25*p1+Y*(0.2*p2+cr6*Y*p3)));
	    }
	    f0	=t-(ti[i1]+tk[kk]);
	    p0	=rh*u0;
	    p1	=h*u0;
	    p2	=0.5*rh*(f0*(f0*u0-2.*u1)+u2);
	    w[0][kk]	-=p0;
	    w[0][k]	+=p0;
	    w[1][kk]	+=f0*u0-u1-p2-cr3*p1;
	    w[1][k]	+=p2-cr6*p1;
	    i--;
	    i1--;
	  }
	}
      }
    }

    /* Matrix of the linear system */
    for(m=0; m < K1; m++){
      kk	=n*m+m;
      u0	=w[0][m];
      u1	=w[1][m];
      for(k=m; k < K1; k++){
	A[kk]	+=u0*w[0][k];
	kk++;
      }
      for(k=0; k < K1; k++){
	A[kk]	+=u0*w[1][k];
	kk++;
      }
      if(iB) A[kk]	+=u0*(t-tt0);
      kk	=n*(m+K1)+K1+m;
      for(k=m; k < K1; k++){
	A[kk]	+=u1*w[1][k];
	kk++;
      }
      if(iB){
	A[kk]+=u1*t;
	x[m]	+=S[j]*u0;
	x[m+K1]	+=S[j]*u1;
      }
      else{
	x[m]	+=(S[j]-*B*t1)*u0;
	x[m+K1]	+=(S[j]-*B*t1)*u1;
      }
    }
    m	=2*K1;
    if(iB) x[m]	+=S[j]*(t-tt0);
    A[n*m+m]	+=(t-tt0)*(t-tt0);
    j++;
  }

  /*regularization*/
  for(m=0; m < K1; m++){
    k	=n*(m+K1)+K1+m;
    if(m){
      A[k]	+=ga*h;
      A[k-1]	-=ga*h;
    }
    if(m < K){
      A[k]	+=ga*h;
      A[k+1]	-=ga*h;
    }
    k	=n*m+2*K1+1;
    kk	=k+n*K1;
    if(m == 0) A[kk]	+=1.;
    if(m == 1) A[kk]	-=2.;
    if(m == 2) A[kk]	+=1.;
    mm	=m+1;
    if(mm < K){
      A[kk+mm]+=cr6*h;
      A[k+mm]	-=rh;
    }
    if(m && m < K){
      A[kk+m]	+=cr3*(h+h);
      A[k+m]	+=rh+rh;
    }
    mm	=m-1;
    if(mm > 0){
      A[kk+mm]	+=cr6*h;
      A[k+mm]	-=rh;
    }
#ifdef H
    if(m == K-2)A[kk+K]	+=1.;
    if(m == K-1)A[kk+K]	-=2.;
    if(m == K)	A[kk+K]	+=1.;
#endif
  }
  m	=K1;
  k	=n*m-1;
  kk	=k+n*K1;
  A[k]	=rh;
  A[kk]	=cr3*h;
  A[k-n]-=rh;
  A[kk-n]+=cr6*h;

  for(m=0; m < n; m++){
    for(k=0; k < m; k++) A[n*m+k]=A[n*k+m];
  }
  LUdcmp(A,n,iA,&d);
  LUbksb(A,n,iA,x);
  for(i=0; i < K1; i++){
    r0[i]	=x[i];
    r2[i]	=x[i+K1];
  }

  if(iB) *B=x[2*K1];

  return(0);
}

int GetSigNm()
{
  FILE *lF;
  int nTNames=8;
  char TNames[8][16]={
    "CDX"
    ,"nstx"
    ,"JET"
    ,"NSTX"
    ,"DIII-D"
    ,"MAST"
    ,"ASDEX"
    ,""
  };

  char TNm[16]="CDX";
  char FNm[32]="../dB0/CDX/In/";
  char FNmI[2][32]={
    "ElecPar.ai",
    "MFProbe.ai"
  };
  char NmR[3][32]={
    "<Flux Loops>",
    "<Saddle Coils>",
    "<Magnetic Probes>"
  };
  int iD[NSNM];
  int iG[NSNM];
  double s;
  int i,n,j,k,L,LNm;
  char str[64];
  char *lc;
  char *lm;
  char *lv[16];
  int dl[16];
  char Nm[32];

#ifndef Tbl_c2e
  for(i=0; i < nTNames; i++){
    if(strcmp(TNames[i],TNm) == 0) break;
  }
  if(i == nTNames){
    CbStr2UserWarning("%s name is not in the list.\n Choose from:\n",TNm);
    for(i=0; i < nTNames; i++) CbStr2UserWarning("%2d %s\n",i,TNames[i]);
    iGr[0]	=0;
    for(i=0; i < 4; i++){
      strcpy(SNm[i],"Plasma");
      iGr[i+1]	=i+1;
    }
  }
  else{
    kTName	=i;
    strcpy(FNm,"../dB0/");
    lm	=FNm+7;
    lc	=TNm;
    while(*lc != '\0') *lm++=*lc++;
    strcpy(lm,"/In/");

    L	=sizeof(SNm[0]);
    LNm	=strlen(FNm);
    lm	=FNmI[0];
    strcpy(FNm+LNm,lm);
    lF	=fopen(FNm,"r");
    nSNm	=0;
    for(k=0; k < 2; k++){
      lc	=k ? "<TF Coil>" : "<Electric Parameters>";
      if(IsKeyWordFound(lF,lm,lc,str,&n)){
	lv[0]	=(char*)&j;
	lv[1]	=(char*)(kSNm+nSNm);
	lv[2]	=(char*)(iD+nSNm);
	lv[3]	=(char*)(iG+nSNm);
	lv[4]	=Nm;
	lv[5]	=SNm[nSNm];
	dl[0]	=0;
	dl[1]	=sizeof(int);
	dl[2]	=sizeof(int);
	dl[3]	=sizeof(int);
	dl[4]	=0;
	dl[5]	=L;
	ReadAI(lF,lc,lm,str,n,lv,dl,6);
      }
      else{
	CbStr2UserMessage("%s: NO %s section.\n",lm,lc);
	fclose(lF);
	return(1);
      }
      nSNm	+=n;
    }	
    CbStr2UserMessage("Input: %s\n",FNm);
    fclose(lF);

    /*compression of the list*/
    j	=0;
    while(j < nSNm){
      i	=j+1;
      while(i < nSNm){
	if(iG[i] <= iG[j]){
	  strcpy(SNm[nSNm],SNm[j]);
	  strcpy(SNm[j],SNm[i]);
	  strcpy(SNm[i],SNm[nSNm]);
	  n	=kSNm[j];
	  kSNm[j]	=kSNm[i];
	  kSNm[i]	=n;
	  n	=iD[j];
	  iD[j]	=iD[i];
	  iD[i]	=n;
	  n	=iG[j];
	  iG[j]	=iG[i];
	  iG[i]	=n;
	}
	i++;
      }
      i	=j+1;
      while(i < nSNm){
	if(iD[i] < iD[j] && iG[i] == iG[j]){
	  strcpy(SNm[nSNm],SNm[j]);
	  strcpy(SNm[j],SNm[i]);
	  strcpy(SNm[i],SNm[nSNm]);
	  n	=iD[j];
	  iD[j]	=iD[i];
	  iD[i]	=n;
	  n	=kSNm[j];
	  kSNm[j]	=kSNm[i];
	  kSNm[i]	=n;
	}
	i++;
      }
      j++;
    }
    k	=0;
    j	=1;
    while(j < nSNm){
      while(j < nSNm && iD[j] == iD[k] && iG[j] == iG[k]) j++;
      if(j < nSNm){
	k++;
	strcpy(SNm[k],SNm[j]);
	kSNm[k]	=kSNm[j];
	iD[k]	=iD[j];
      }
    }
    nPFC	=k;
    nSNm	=k+1;
    lm	=FNmI[1];
    strcpy(FNm+LNm,lm);
    lF	=fopen(FNm,"r");
    lc	="<Flux Loops>";
    if(IsKeyWordFound(lF,lm,lc,str,&n)){
      nY		=n;
      nYS		=nY;
      lv[0]	=(char*)&j;
      lv[1]	=(char*)&s;
      lv[2]	=(char*)&s;
      lv[3]	=(char*)&s;
      lv[4]	=(char*)&s;
      lv[5]	=(char*)&s;
      lv[6]	=(char*)(iG+nSNm);
      lv[7]	=SNm[nSNm];
      dl[0]	=0;
      dl[1]	=0;
      dl[2]	=0;
      dl[3]	=0;
      dl[4]	=0;
      dl[5]	=0;
      dl[6]	=sizeof(int);
      dl[7]	=L;
      ReadAI(lF,lc,lm,str,n,lv,dl,8);
      nSNm	+=n;
    }
    else{
      CbStr2UserMessage("%s: NO %s section.\n",lm,lc);
      fclose(lF);
      return(1);
    }
    lc	="<Saddle Coils>";
    if(IsKeyWordFound(lF,lm,lc,str,&n)){
      nYS		=nY+n;
      lv[0]	=(char*)&j;
      lv[1]	=(char*)&s;
      lv[2]	=(char*)&s;
      lv[3]	=(char*)&s;
      lv[4]	=(char*)&s;
      lv[5]	=(char*)&s;
      lv[6]	=(char*)&s;
      lv[7]	=(char*)&s;
      lv[8]	=(char*)(iG+nSNm);
      lv[9]	=SNm[nSNm];
      dl[0]	=0;
      dl[1]	=0;
      dl[2]	=0;
      dl[3]	=0;
      dl[4]	=0;
      dl[5]	=0;
      dl[6]	=0;
      dl[7]	=0;
      dl[8]	=sizeof(int);
      dl[9]	=L;
      ReadAI(lF,lc,Mess,str,n,lv,dl,10);
      nSNm	+=n;
    }
    else{
      CbStr2UserMessage("%s: NO %s section.\n",lm,lc);
      fclose(lF);
      return(1);
    }
    lc	="<Magnetic Probes>";
    if(IsKeyWordFound(lF,lm,lc,str,&n)){
      nB		=n;
      lv[0]	=(char*)&j;
      lv[1]	=(char*)&s;
      lv[2]	=(char*)&s;
      lv[3]	=(char*)&s;
      lv[4]	=(char*)&s;
      lv[5]	=(char*)&s;
      lv[6]	=(char*)&s;
      lv[7]	=(char*)(iG+nSNm);
      lv[8]	=SNm[nSNm];;
      dl[0]	=0;
      dl[1]	=0;
      dl[2]	=0;
      dl[3]	=0;
      dl[4]	=0;
      dl[5]	=0;
      dl[6]	=0;
      dl[7]	=sizeof(int);
      dl[8]	=L;
      ReadAI(lF,lc,Mess,str,n,lv,dl,9);
      nSNm	+=n;
    }
    else{
      CbStr2UserMessage("%s: NO %s section.\n",lm,lc);
      fclose(lF);
      return(1);
    }
    CbStr2UserMessage("Input: %s\n",FNm);
    
    for(k=0; k < 8; k++) kGr[k]=0;
    for(i=0; i < nSNm; i++) kGr[iG[i]]++;
    iGr[0]	=0;
    for(k=0; k < 8; k++) iGr[k]=iGr[k-1]+kGr[k-1];
    nMFP	=nYS+nB;
    j	=nPFC+1;
    for(i=0; i < nMFP; i++) kSNm[j]=j;
  }
#endif /*Tbl_c2e*/
  return(0);
}

int c2eAI(FILE *lF)
{
  int i,j,k,n,m,L,Ld;
  char Nm[0x20],NmC[NPFC][0x20];
  char str[0x40],str1[0x40];
  char *lc,*lm;
  char *lv[NPFC];
  int dl[NPFC];
  double *ld;
  double GF[NMFP][NPFC];

  lm="c2e.ai";
  
  L	=sizeof(NmC[0]);
  Ld	=sizeof(double);
  
  lc	="<PFC Names>";
  m	=kGr[0]+kGr[1];
  if(IsKeyWordFound(lF,lm,lc,str,&n)){
    for(i=0; i < m; i++){
      lv[i]=NmC[i]; 
      dl[i]=L;
    }
    ReadAI(lF,lc,lm,str,n,lv,dl,m);
  }
  else{
    CbStr2UserMessage("%s: %s record is absent.\n",lm,lc);
  }

  for(i=0; i < m; i++){
    for(k=0; k < m; k++) if(strcmp(SNm[k],NmC[i]) == 0) break;
    if(k == m){
      CbStr2UserWarning("%s: <%s> does not match signal names",lm,NmC[i]);
      return(1);
    }
  }

  L	=sizeof(GF[0]);
  lc="<MFProbe-PFC Green Functions>";
  if(IsKeyWordFound(lF,lm,lc,str,&n)){
    if(nMFP != n){
      CbStr2UserMessage("%s: %s %d != %d inconsistent data\n",lm,lc,nMFP,n);
    }
    lv[0]	=(char*)Nm;
    dl[0]	=0;
    ld	=GF[0]+1;
    for(i=1; i < m; i++){
      lv[i]=(char*)ld++; 
      dl[i]=L;
    }
    ReadAI(lF,lc,lm,str,n,lv,dl,m-1);
  }
  else{
    CbStr2UserMessage("%s: %s record is absent.\n",lm,lc);
  }
  for(j=0; j < nMFP; j++) gYiC[j][0]=0.;
  for(i=1; i < m; i++){
    for(k=1; k < m; k++) if(strcmp(SNm[k],NmC[i]) == 0) break;
    for(j=0; j < nMFP; j++) gYiC[j][k]=GF[j][i];
  }
  return(0);
}

int FindSig(int *iS,double *x,double *y,int M)
{
  double *t,s,d;
  int j,j0,j1;

  t	=llS[0]->t;
  j0	=0;
  j1	=llS[0]->n-1;
  j	=(j1-j0)/2;
  while(j >j0){
    if(*x >= t[j]){
      j0=j;
    }
    else{
      j1=j;
    }
    j	=(j1+j0)/2;
  }
  *x	=t[j];

  for(j1=0; j1 < M; j1++) if(llS[j1]->c) break;
  if(j1 == M) return(0);
  s	=fabs(*y-llS[j1]->D[j]);
  for(j0=j1+1; j0 < M; j0++){
    if(llS[j0]->c){
      d	=fabs(*y-llS[j0]->D[j]);
      if(s > d){
	s	=d;
	j1	=j0;
      }
    }
  }
  *iS	=j1;
  *y	=llS[j1]->D[j];
  return(0);
}

int ShotListAO(FILE* lF)
{
  int i,n;
  char ln[32],str[32];

  strcpy(ln,"%s%c");
  strcpy(str,ln);
  FormatWrite2Read(str);
  fprintf(lF,"<DirName>[%d](%s)\n",1,str);
  fprintf(lF,"%s\n\n",DirNm);
  n	=0;
  for(i=0; i < nCdxNm; i++) if(kCdxNm[i]) n++;
  strcpy(ln,"%2d %s%c");
  strcpy(str,ln);
  FormatWrite2Read(str);
  fprintf(lF,"<ShotList>[%d](%s)\n",n,str);
  for(i=0; i < nCdxNm; i++){
    if(kCdxNm[i])fprintf(lF,ln,i,CdxNm[i],'\n');
  }
  return(0);
}

int ShotListAI(FILE* lF)
{
  int i,n;
  char str[64];
  char *lc;
  char *lm;
  char *lv[4];
  int dl[4];

  lm	="ShotList.ai";
  lc	="<DirName>";
  if(IsKeyWordFound(lF,lm,lc,str,&n)){
    lv[0]	=DirNm;
    dl[0]	=0;
    ReadAI(lF,lc,lm,str,n,lv,dl,1);
  }
  else{
    CbStr2UserMessage("%s: %s record is absent.\n",lm,lc);
  }

  lc	="<ShotList>";
  if(IsKeyWordFound(lF,lm,lc,str,&n)){
    nCdxNm	=n;
    lv[0]	=(char*)&i;
    lv[1]	=CdxNm[0];
    dl[0]	=0;
    dl[1]	=sizeof(CdxNm[0]);
    ReadAI(lF,lc,lm,str,n,lv,dl,2);
    for(i=0; i < nCdxNm; i++) kCdxNm[i]=1;
  }
  else{
    CbStr2UserMessage("%s: %s record is absent.\n",lm,lc);
  }
  return(0);
}

int FileList(char *Mask)
{
  int iUpdate=1;
  int i,k,L,L1;
  char*p,*lc,*lt;
  char Nm[16];	
  char Nm1[256];	
  int nNm;
  char FakeNm[16]="GiveMeMask";

#ifndef AscI_ShotList
#undef AscI_ShotList
#endif/*AscI_ShotList*/

  if(nCdxNm) iUpdate=0;
#ifndef Tbl_ShotList
  lc	=DirNm+strlen(DirNm);
  if(*(lc-1) != '/'){
    *lc++	='/';
    *lc	='\0';
  }

  strcpy(Nm1,DirNm);
  L1	=strlen(Nm1);

  L	=strlen(Mask);
  if(iUpdate && (dirp=opendir(DirNm)) != NULL){
    iUpdate	=0;
    nCdxNm	=0;
    while(nCdxNm < NCdxNm && (direntp=readdir(dirp)) != NULL){
      p	=direntp->d_name;
      lc	=Nm;
      lt	=Mask;
      while(*p != '.' && *p != '\0'){
	if(*lt != '\0'){
	  if(*lt != *p) break;
	  lt++;
	}
	*lc++	=*p++;
      }
      *lc	='\0';
      if(*lt == '\0' && strcmp(p,".PSI") == 0){
	lc	=CdxNm[nCdxNm]; 
	/* Sorting */
	if(nCdxNm){
	  i	=nCdxNm;
	  while(i > 0){
	    i--;
	    if(strcmp(Nm,CdxNm[i]) > 0) break;
	    strcpy(lc,CdxNm[i]);
	    lc	=CdxNm[i]; 
	  }
	}
	strcpy(Nm1+L1,Nm);
	strcpy(Nm1+L1+strlen(Nm),".CCT");
	if(stat(Nm1,&statbuf) == 0){
	  strcpy(Nm1+L1+strlen(Nm),".MAG");
	  if(stat(Nm1,&statbuf) == 0){
	    strcpy(lc,Nm);
	    kCdxNm[i]=0;
	    nCdxNm++;
	    if(nCdxNm == NCdxNm){
#ifdef XWIN
	      CbStr2UserWarning("%d Number of data file names"
				"reached its limit\n",nCdxNm);
#endif
	    }
	  }
	}
      }
    }
    closedir(dirp);
  }
  nNm	=nCdxNm;
  if(nNm == 0){
    strcpy(CdxNm[0],FakeNm);
    CbStr2UserWarning("Empty Shot list");
    nNm	=1;
  }
#endif /*Tbl_ShotList*/

#ifndef AscO_ShotList
#undef AscO_ShotList
#endif/*AscO_ShotList*/

  k	=sizeof(CDX);
  for(i=0; i < nCdxNm; i++) memset(Cdx+i,'\0',k);
  k	=sizeof(RESPFUNC);
  for(i=0; i < nSNm; i++) memset(RF[i],'\0',k);

  nCdx	=0;
  for(i=0; i < nCdxNm; i++){
    if(kCdxNm[i]){
      strcpy(Cdx[nCdx].Nm,CdxNm[i]);
      for(k=0; k < 4; k++){
	Cdx[nCdx].K[k]	=nSS2-1;
      }
      for(k=0; k < nSNm; k++){
	strcpy(Cdx[nCdx].NmS[k],SNm[k]);
	Cdx[nCdx].kS[k]	=1;
	Cdx[nCdx].gaS[k]=1e-14;
	Cdx[nCdx].gaR[k]=1e-15;
	Cdx[nCdx].tol[k]=1e-3;
	Cdx[nCdx].errS[k]=1.;
	Cdx[nCdx].errR[k]=1.;
      }
      nCdx++;
    }
  }
  return(0);
}

int SigSetIp(FILE *lf,char *FNmI,char *FNmO,int Mk)
{
  char ch,ch1;
  char Nm[16];
  char ln[0x0400],*lc;
  SIGNAL *lS;
  FILE *lF;
  double s,ss;
  int i,j,k,n,L;
  long int lFile;

  i	=0;
  while((*ln=getc(lf)) == 'c' || *ln == 'C'){
    while(feof(lf) == 0 && (ch=getc(lf)) != '\n');
    i++;
  }
  lc	=ln+1;
  while(feof(lf) == 0 && (*lc=getc(lf)) != '\n') lc++;
  *lc	='\0';
  if(sscanf(ln,"%lg",&s) < 1){
    printf("%s:%d wrong line with a number scale factor\n",FNmI,i);
    puts(ln);
    return(1);
  } 
  lS	=Sig+iGr[0];
  strcpy(lS->Nm,SNm[0]);
  lS->kA=1;
  lS->kG=0;
  lS->dS=0.;
  lS->C	=16.9*0.001;
  lS->C	=s*0.001;
  lS->dC=0.02;
  lS->G	=1.;
  lS->dG=0.;
  lS->dG	=0.;
  lS->A	=0.;
  lS->dA=0.;
  lS->B	=0.;
  lS->dB=0.;
  lS->ga=1e-14;
  lS->i0=0;
  lS->i=0;
  lS->i1=0;
  lS->n=0;
  lS->c=0;
  lS->kxy	=0;
  lS->kspl	=0;

  while(isspace(*ln=getc(lf)));
  lc	=ln+1;
  while(feof(lf) == 0 && (*lc=getc(lf)) != '\n') lc++;
  *lc	='\0';
  i++;
  if(sscanf(ln,"%lg%lg",&s,&ss) < 2){
    printf("%s:%d wrong line with a number of data lines\n",FNmI,i);
    puts(ln);
    return(1);
  }
  if((Mk&0x01) == 0) return(0); 
  n	=s;
  k	=ss;
  lFile	=ftell(lf);

  lS->n	=n;
  lTime	=(double *)malloc(4*n*sizeof(double));
  lT[0]	=lTime;
  U0	=lTime+n;
  U1	=U0+n;
  U2	=U1+n;
  lS->D	=(double *)malloc(3*n*sizeof(double));
  lS->s0	=lS->D+n;
  lS->s2	=lS->s0+n;
  lS->tk	=NULL;
  lS->r0	=NULL;
  lS->r2	=NULL;
  lS->t		=lTime;
  lS->K1	=0;
  lS->n		=n;
  lS->i1	=n;
  lS->c		=1;
  lData[0]	=lS->D;

  lD[0]	=lS->s0;
  lD2[0]=lS->s2;
  for(i=0; i < n; i++){
    if(fscanf(lf,"%lg",lTime+i) < 1){
      printf("%d data line is wrong for time value\n",i);
      return(1);
    }
    if(fscanf(lf,"%lg%lg",lData[0]+i,&s) < 2){
      printf("%d data line is wrong for signal <%s>\n",i,Sig[iSig].Nm);
      return(1);
    }
  }
  if((Mk&0x02) == 0) return(0); 

  fseek(lf,lFile,SEEK_SET);
  lF	=fopen(FNmO,"w");
  k--;
  fprintf(lF,"<WaveFormIp>[%d](t=%d%%e)\n",n,k);
  putc('!',lF);
  fprintf(lF,"%9s%10s\n","t",lS->Nm);
  for(i=0; i < n; i++){
    fputs("t=",lF);
    L	=2;
    for(j=0; j < k; j++){
      while(feof(lf) == 0 && isspace(ch=getc(lf)));
      if(j){
	putc(' ',lF);
	L++;
	if(isdigit(ch)){
	  putc(' ',lF);
	  L++;
	}
      }
      putc(ch,lF);
      L++;
      while(feof(lf) == 0 && !isspace((ch=getc(lf)))){
	putc(ch,lF);
	L++;
      }	
      if(L > 72){
	putc('\n',lF);
	L	=0;
      }
      lS++;
    }
    if(L) putc('\n',lF);
    while(feof(lf) == 0 && (ch=getc(lf))!= '\n');
  }
  fclose(lF);
  printf("Output: %s\n",FNmO);
  return(0);
}

int SigSetI(FILE *lf,char *FNmI,char *FNmO,int Mk)
{
  char ch,ch1;
  char *lNm;
  char ln[1024],*lc;
  SIGNAL *lS;
  FILE *lF;
  double s,ss;
  int i,j,k,m,n,L;
  long int lFile;

  n	=kGr[1];
  m	=iGr[2];
  for(i=0; i < 5; i++) while(feof(lf) == 0 && (ch=getc(lf)) != '\n') ;
  lc	=ln;
  while(feof(lf) == 0 && (*lc=getc(lf)) != '\n') lc++;
  *lc	='\0';
  lS	=Sig+iGr[1];
  lc	=ln+1;
  for(i=0; i < n; i++){
    lNm	=lS->Nm;
    while(isspace(*lc)) lc++;
    if(sscanf(lc,"%s",lNm) < 1){
      printf("%s : wrong line 6 with signal names\n",FNmI);
      puts(ln);
      return(1);
    }
    for(j=iGr[1]; j < m && strcmp(SNm[j],lNm); j++);
    if(j == m){
      printf("%s : names in line 6 do not match names in ElecPar.ai\n",FNmI);
      puts(ln);
      return(1);
    }
    lc	+=strlen(lNm);
    lS++;
  }
  lc	=ln;
  while(feof(lf) == 0 && (*lc=getc(lf)) != '\n') lc++;
  *lc	='\0';
  lc	=ln;
  lS	=Sig+iGr[1];
  for(i=0; i < n; i++){
    while(isspace(*lc)) lc++;
    lS->C	=1.;
    if(sscanf(lc,"%lg",&lS->C) < 1){
      printf("%s : wrong line 7 with Conversion factors\n",FNmI);
      puts(ln);
      return(1);
    }
    lS->kA	=1;
    lS->kG	=1;
    lS->dS	=0.;
    lS->C	*=0.001;
    lS->dC	=0.;
    lS->G	=1.;
    lS->dG	=0.02;
    lS->A	=0.;
    lS->dA	=0.;
    lS->B	=0.;
    lS->dB	=0.;
    lS->ga	=1e-14;
    lS->i0	=0;
    lS->i	=0;
    lS->i1	=0;
    lS->n	=0;
    lS->c	=0;
    lS->kxy	=0;
    lS->kspl	=0;
    while(!isspace(*lc) && *lc != '\0') lc++;
    lS++;
  }
  if((Mk&0x01) == 0) return(0); 

  while(feof(lf) == 0 && (ch=getc(lf)) != '\n') ;
  lc	=ln;
  while(feof(lf) == 0 && isspace(*ln=getc(lf)));
  lc++;
  while(feof(lf) == 0 && (*lc=getc(lf)) != '\n') lc++;
  *lc	='\0';
  if(sscanf(ln,"%lg%lg",&s,&ss) < 2){
    printf("%s : wrong line 9 with a number of data lines\n",FNmI);
    puts(ln);
    return(1);
  }
  n	=s;
  k	=ss;
  if(k-1 !=  kGr[1]){
    printf("%s:9 - number of %d != %d PFCoils in ElecPar.ai\n"
	   ,FNmI,k,kGr[1]);
    return(1);
  }


  lFile	=ftell(lf);

  lS=Sig+iGr[1];
  lS->n	=n;
  lTime	=(double *)malloc(4*n*sizeof(double));
  lT[1]	=lTime;
  U0	=lTime+n;
  U1	=U0+n;
  U2	=U1+n;
  for(j=0; j < kSig; j++){
    lS->D	=(double *)malloc(3*n*sizeof(double));
    lS->s0	=lS->D+n;
    lS->s2	=lS->s0+n;
    lS->tk	=NULL;
    lS->r0	=NULL;
    lS->r2	=NULL;
    lS->t	=lTime;
    lS->K1	=0;
    lS->n	=n;
    lS->i1	=n;
    lS->c	=1;
    lData[j]	=lS->D;
    lS++;
  }
  lD[1]	=Sig[iGr[1]].s0;
  lD2[1]=Sig[iGr[1]].s2;
  for(i=0; i < n; i++){
    if(fscanf(lf,"%lg",lTime+i) < 1){
      printf("%d data line is wrong for time value\n",i);
      return(1);
    }
    for(j=0; j < kSig; j++){
      if(fscanf(lf,"%lg",lData[j]+i) < 1){
	printf("%d data line is wrong for signal <%s>\n",i,Sig[iSig+j].Nm);
	return(1);
      }
    }
  }
  if((Mk&0x02) == 0) return(0); 

  fseek(lf,lFile,SEEK_SET);
  lF	=fopen(FNmO,"w");
  fprintf(lF,"<WaveFormI>[%d](t=%d%%e)\n",n,k);
  L=fprintf(lF,"!%9s","t");
  lS=Sig+iGr[1];
  for(j=0; j < k-1; j++){
    L+=fprintf(lF,L ? "%10s" : "%9s",lS->Nm);
    if(L > 69){
      fputs("\n!",lF);
      L	=0;
    }
    lS++;
  }
  if(L)putc('\n',lF);
  for(i=0; i < n; i++){
    fputs("t=",lF);
    L	=2;
    for(j=0; j < k; j++){
      while(feof(lf) == 0 && isspace(ch=getc(lf)));
      if(j){
	putc(' ',lF);
	L++;
	if(isdigit(ch)){
	  putc(' ',lF);
	  L++;
	}
      }
      putc(ch,lF);
      L++;
      while(feof(lf) == 0 && !isspace((ch=getc(lf)))){
	putc(ch,lF);
	L++;
      }	
      if(L > 69){
	putc('\n',lF);
	L	=0;
      }
    }
    if(L) putc('\n',lF);
  }
  fclose(lF);
  printf("Output: %s\n",FNmO);
  return(0);
}

int SigSetY(FILE *lf,char *FNmI,char *FNmO,int Mk)
{
  char ch,ch1;
  char Nm[16];
  char ln[1024],*lc;
  SIGNAL *lS;
  FILE *lF;
  double s,ss;
  int i,j,k,n,m,L;
  long int lFile;

  m	=kGr[2];
  for(i=0; i < 6; i++){
    while(feof(lf) == 0 && (ch=getc(lf)) != '\n') ;
  }
  lc	=ln;
  while(feof(lf) == 0 && (*lc=getc(lf)) != '\n') lc++;
  *lc	='\0';
  if(sscanf(ln,"%lg%lg",&s,&ss) < 2){
    printf("%s : wrong line 9 with a number of data lines\n",FNmI);
    puts(ln);
    return(1);
  }
  n	=s;
  k	=ss;
  if(k !=  kGr[2]){
    printf("%s:7 - number of data %d != %d of the flux loops in MFProbe.ai\n"
	   ,FNmI,k,kGr[2]);
    return(1);
  }

  for(i=0; i < 4; i++){
    while(feof(lf) == 0 && (ch=getc(lf)) != '\n') ;
    n--;
  }
  for(i=0; i < n; i++){
    lc	=ln;
    while(feof(lf) == 0 && (*lc=getc(lf)) != '\n') lc++;
    *lc	='\0';
    lc	=ln;
    lS	=Sig+iGr[2];
    for(j=0; j < m; j++){
      while(isspace(*lc)) lc++;
      if(sscanf(lc,"%lg",&s) < 1){
	printf("%s : wrong line %d with Conversion factors\n",FNmI,i+12);
	puts(ln);
	return(1);
      }
      switch(i){
      case 0:
	lS->G=s;
	break;
      case 1:
	lS->dS=s;
	break;
      case 2:
	lS->C=s;
	break;
      case 3:
	lS->dC=s;
	break;
      case 4:
	lS->A=s;
	break;
      case 5:
	lS->B=s;
	break;
      case 6:
	lS->kA=s;
	break;
      case 7:
	lS->C *=s;
	break;
      }
      while(!isspace(*lc) && *lc != '\0') lc++;
      lS++;
    }
  }
  lS	=Sig+iGr[2];
  n	=iGr[3];
  for(j=iGr[2]; j < n; j++){
    strcpy(lS->Nm,SNm[j]);
    lS->kG	=2;
    lS->dG	=0.;
    lS->dA	=0.;
    lS->dB	=0.;
    lS->i0	=0;
    lS->i	=0;
    lS->i1	=0;
    lS->n	=0;
    lS->c	=lS->kA ? 1 : 0;
    lS->kxy	=0;
    lS->kspl	=0;
    lS++;
  }
  if((Mk&0x01) == 0) return(0); 
  while(feof(lf) == 0 && isspace(*ln=getc(lf)));
  lc	=ln+1;
  while(feof(lf) == 0 && (*lc=getc(lf)) != '\n') lc++;
  *lc	='\0';
  if(sscanf(ln,"%lg%lg",&s,&ss) < 2){
    printf("%s : wrong line 9 with a number of data lines\n",FNmI);
    puts(ln);
    return(1);
  }
  n	=s;
  k	=ss;
  lFile	=ftell(lf);

  lS=Sig+iGr[2];
  lS->n	=n;

  lTime	=(double *)malloc(n*sizeof(double));
  lT[2]	=lTime;
  for(j=0; j < m; j++){
    lS->D	=(double *)malloc(4*n*sizeof(double));
    lS->s0	=lS->D+n;
    lS->s2	=lS->s0+n;
    lS->S	=lS->s2+n;
    lS->tk	=NULL;
    lS->r0	=NULL;
    lS->r2	=NULL;
    lS->t	=lTime;
    lS->K1	=0;
    lS->n	=n;
    lS->i1	=n;
    lS->c	=1;
    lData[j]	=lS->D;
    lS++;
  }
  j	=iGr[2];
  lD[2]	=Sig[j].s0;
  lD2[2]=Sig[j].s2;
  for(i=0; i < n; i++){
    if(fscanf(lf,"%lg",lTime+i) < 1){
      printf("%d data line is wrong for time value\n",i);
      return(0);
    }
    for(j=0; j < m; j++){
      if(fscanf(lf,"%lg",lData[j]+i) < 1){
	CbStr2UserWarning("%d data line is wrong for signal <%s>\n"
			  ,i,Sig[iGr[2]+j].Nm);
	return(0);
      }
    }
  }
  if((Mk&0x2) == 0) return(0); 

  fseek(lf,lFile,SEEK_SET);
  lF	=fopen(FNmO,"w");
  fprintf(lF,"<WaveFormY>[%d](t=%d%%e)\n",n,k);
  L=fprintf(lF,"!%9s","t");
  lS	=Sig+iGr[2];
  for(j=0; j < m; j++){
    L+=fprintf(lF,L ? "%10s" : "%9s",lS->Nm);
    if(L > 69){
      fputs("\n!",lF);
      L	=0;
    }
    lS++;
  }
  if(L) putc('\n',lF);

  for(i=0; i < n; i++){
    fputs("t=",lF);
    L	=2;
    lS	=Sig+iGr[2];
    for(j=0; j < k; j++){
      while(feof(lf) == 0 && isspace(ch=getc(lf)));
      if(j){
	putc(' ',lF);
	L++;
	if(isdigit(ch)){
	  putc(' ',lF);
	  L++;
	}
      }
      putc(ch,lF);
      L++;
      while(feof(lf) == 0 && !isspace((ch=getc(lf)))){
	putc(ch,lF);
	L++;
      }
      if(L > 69){
	putc('\n',lF);
	L	=0;
      }
      lS++;
    }
    if(L) putc('\n',lF);
  }
  fclose(lF);
  printf("Output: %s\n",FNmO);
  return(0);
}

int SigSetB(FILE *lf,char *FNmI,char *FNmO,int Mk)
{
  char ch,ch1;
  char Nm[16];
  char ln[1024],*lc;
  SIGNAL *lS;
  FILE *lF;
  double s,ss;
  int i,j,k,n,m,L;
  long int lFile;

  m	=kGr[3];
  for(i=0; i < 6; i++){
    while(feof(lf) == 0 && (ch=getc(lf)) != '\n') ;
  }
  lc	=ln;
  while(feof(lf) == 0 && (*lc=getc(lf)) != '\n') lc++;
  *lc	='\0';
  if(sscanf(ln,"%lg%lg",&s,&ss) < 2){
    printf("%s : wrong line 9 with a number of data lines\n",FNmI);
    puts(ln);
    return(1);
  }
  n	=s;
  k	=ss;
  if(k != kGr[3]){
    printf("%s:7 - number of data %d != %d number of probe names "
	   "in MFProbe.ai\n",FNmI,k,kGr[3]);
    return(1);
  }
  for(i=0; i < 4; i++){
    while(feof(lf) == 0 && (ch=getc(lf)) != '\n') ;
    n--;
  }
  for(i=0; i < n; i++){
    lc	=ln;
    while(feof(lf) == 0 && (*lc=getc(lf)) != '\n') lc++;
    *lc	='\0';
    lc	=ln;
    lS	=Sig+iGr[3];
    for(j=0; j < m; j++){
      while(isspace(*lc)) lc++;
      if(sscanf(lc,"%lg",&s) < 1){
	printf("%s : wrong line %d with Conversion factors\n",FNmI,i+12);
	puts(ln);
	return(1);
      }
      switch(i){
      case 0:
	lS->G=s;
	break;
      case 1:
	lS->dS=s;
	break;
      case 2:
	lS->C=s;
	break;
      case 3:
	lS->dC=s;
	break;
      case 4:
	lS->A=s;
	break;
      case 5:
	lS->B=s;
	break;
      case 6:
	lS->kA=s;
	break;
      case 7:
	lS->C *=s;
	break;
      }
      while(!isspace(*lc) && *lc != '\0') lc++;
      lS++;
    }
  }
  lS	=Sig+iGr[3];
  n	=iGr[4];
  for(j=iGr[3]; j < n; j++){
    strcpy(lS->Nm,SNm[j]);
    lS->kG	=3;
    lS->dG	=0.;
    lS->dA	=0.;
    lS->dB	=0.;
    lS->ga	=1e-13;
    lS->i0	=0;
    lS->i	=0;
    lS->i1	=0;
    lS->n	=0;
    lS->c	=0;
    lS->kxy	=0;
    lS->kspl	=0;
    lS++;
  }
  if((Mk&0x01) == 0) return(0); 

  while(feof(lf) == 0 && isspace(*ln=getc(lf)));
  lc	=ln+1;
  while(feof(lf) == 0 && (*lc=getc(lf)) != '\n') lc++;
  *lc	='\0';
  if(sscanf(ln,"%lg%lg",&s,&ss) < 2){
    printf("%s : wrong line 9 with a number of data lines\n",FNmI);
    puts(ln);
    return(1);
  }
  n	=s;
  k	=ss;
  lFile	=ftell(lf);

  lS=Sig+iSig;
  lS->n	=n;
  lTime	=(double *)malloc(n*sizeof(double));
  lT[3]	=lTime;
  for(j=0; j < m; j++){
    lS->D	=(double *)malloc(4*n*sizeof(double));
    lS->s0	=lS->D+n;
    lS->s2	=lS->s0+n;
    lS->S	=lS->s2+n;
    lS->tk	=NULL;
    lS->r0	=NULL;
    lS->r2	=NULL;
    lS->t	=lTime;
    lS->K1	=0;
    lS->n	=n;
    lS->i1	=n;
    lS->c	=1;
    lData[j]	=lS->D;
    lS++;
  }
  lD[3]	=Sig[iGr[3]].s0;
  lD2[3]=Sig[iGr[3]].s2;
  for(i=0; i < n; i++){
    if(fscanf(lf,"%lg",lTime+i) < 1){
      printf("%d data line is wrong for time value\n",i);
      return(0);
    }
    for(j=0; j < kSig; j++){
      if(fscanf(lf,"%lg",lData[j]+i) < 1){
	printf("%d data line is wrong for signal <%s>\n",i,Sig[iSig+j].Nm);
	return(0);
      }
    }
  }
  if((Mk&0x2) == 0) return(0); 

  fseek(lf,lFile,SEEK_SET);
  lF	=fopen(FNmO,"w");
  fprintf(lF,"<WaveFormB>[%d](t=%d%%e)\n",n,k);
  L=fprintf(lF,"!%9s","t");
  lS	=Sig+iGr[3];
  for(j=0; j < m; j++){
    L+fprintf(lF,L ? "%10s" : "%9s",lS->Nm);
    if(L > 69){
      fputs("\n!",lF);
      L	=0;
    }
    lS++;
  }
  if(L) putc('\n',lF);
  for(i=0; i < n; i++){
    fputs("t=",lF);
    L	=2;
    lS	=Sig+iGr[3];
    for(j=0; j < k; j++){
      while(feof(lf) == 0 && isspace(ch=getc(lf)));
      if(j){
	putc(' ',lF);
	L++;
	if(isdigit(ch)){
	  putc(' ',lF);
	  L++;
	}
      }
      putc(ch,lF);
      L++;
      while(feof(lf) == 0 && !isspace((ch=getc(lf)))){
	putc(ch,lF);
	L++;
      }
      if(L > 69){
	putc('\n',lF);
	L	=0;
      }
      lS++;
    }
    if(L) putc('\n',lF);
  }
  fclose(lF);
  printf("Output: %s\n",FNmO);
  return(0);
}

int ReadCdxWF(char *NmShot,int Mk)
{
  int i,j,Fl;
  int M,LI,LO;
  char Ext[8][8]={".IPN",".CCT",".PSI",".MAG",".MDT","","",""};
  FILE *lf;
  char FNmI[128]="../dCDX/2004/Dat/";
  char FNmO[128];
  char str[512]; 
  double *ld;
  char *lD,*lN;
  SIGNAL *lS;

  strcpy(FNmI,DirNm);
  LI	=strlen(FNmI);
  strcpy(FNmI+LI,NmShot);
  M	=LI+strlen(FNmI+LI);
  Fl	=1;
  for(j=0; j < 4; j++){
    strcpy(FNmI+M,Ext[j]);
    lf	=fopen(FNmI,"r");
    if(lf == NULL){
      printf("%s: No such file, shot data are ignored\n",FNmI);
      Fl=0;
    }
    else{
      fclose(lf);
    }
  }

  CbGetCurrentCbName(&lD,&lN);
  sprintf(FNmO,"%s/Out/",lD);
  LO	=strlen(FNmO);

  for(j=0; j < 4; j++){
    strcpy(FNmI+M,Ext[j]);
    lf	=fopen(FNmI,"r");
    *Msk[j-1]	='\0';
    iSig	=iGr[j];
    kSig	=kGr[j];
    switch(j){
    case 0:
      strcpy(FNmO+LO,"WFormIp.ai");
      if(SigSetIp(lf,FNmI,FNmO,Mk)){
	Fl=0;
	kSig	=0;
      }
      break;
    case 1:
      strcpy(FNmO+LO,"WFormI.ai");
      if(SigSetI(lf,FNmI,FNmO,Mk)){
	Fl=0;
	kSig	=0;
      }
      break;
    case 2:
      strcpy(FNmO+LO,"WFormY.ai");
      if(SigSetY(lf,FNmI,FNmO,Mk)){
	Fl=0;
	kSig	=0;
	break;
      }
      lS	=Sig+iSig;
      for(i=0; i < nY; i++){
	if(1 || lS->kA){
	  lS->kA=1;
	  lS->c=1;
	}
	lS++;
      }
      while(i < nYS){
	if(1 || lS->kA){
	  lS->kA=2;
	  lS->c=1;
	}
	lS++;
	i++;
      }
      break;
    case 3:
      strcpy(FNmO+LO,"WFormB.ai");
      if(SigSetB(lf,FNmI,FNmO,Mk)){
	Fl=0;
	kSig	=0;
	break;
      }
      lS	=Sig+iSig;
      for(i=iSig; i < nSig; i++){
	if(1 || lS->kA){
	  lS->kA=3;
	  lS->c=1;
	}
	lS++;
      }
      break;
    }
    fclose(lf);
  }
  return(0);
}

int WriteCdxLayOut(char *NmShot)
{
  int i,j,k,n,nI;
  double err,V;
  char str[128];
  char FNmO[128];
  char *lD,*lN;
  CDX *lC;
  FILE *lF;

  if(nCdx == 0){
    CbStr2UserMessage("No data for SigLayOut.ai");
    return(1);
  }
  k	=0;
  for(i=0; i < nSig; i++){
    j=strlen(SNm[i]);
    if(k < j) k=j;
  }
  CbGetCurrentCbName(&lD,&lN);
  sprintf(FNmO,"%s/Out/SigLayOut.ai",lD);
  lF	=fopen(FNmO,"w");
  fputs("! Signal=C/G(RawS+A*t+B), kG - group number,"
	" kA=0/1 - activity\n",lF);
  fprintf(lF,"<Layout of Signals>[%d](%%s 3%%d 9%%e)\n",nSig);
  *str='!';
  *(str+1)='%';
  sprintf(str+2,"%d",k-1);
  strcpy(str+strlen(str)
	 ,"s%3s%6s%4s%11s%11s%11s%11s%11s\n!%30s%11s%11s%11s\n");
  fprintf(lF,str,"Name","kA","Group","kS","C","Gain","A","B"
	  ,"Error dS","Error dC","Error dG","Error dA","Error dB");
  *str='%';
  sprintf(str+1,"%d",k);
  strcpy(str+strlen(str),"s%3d%6d%4d%11.3e%11.3e%11.3e%11.3e%11.3e\n"
	 "%31.3e %10.3e %10.3e %10.3e\n");
  nI	=iGr[2];

  for(i=0; i < nI; i++){
    for(j=0; j < nCdx; j++) if(Cdx[j].kI == i) break;
    if(j < nCdx){
      k	=j < nCdx ? 1 : 0;
      lC=Cdx+j;
    }
    else{
      k	=0;
      lC=Cdx;
    }

    /********************????*****************/
    if(strcmp(SNm[i],"TF")) k=1;

    fprintf(lF,str,SNm[i],k,i < iGr[1] ? 0 : 1,kSNm[i],lC->C[i],lC->G[i]
	    ,0.,0.,0.,0.,0.,0.,0.);
  }
  while(i < nSig){
    err	=0.;
    n	=0;
    for(j=0; j < nCdx; j++){
      if((Cdx[j].k&0x02) && (Cdx[j].kS[i]&0x0f)){
	k	=Cdx[j].kI;
	err	+=Cdx[j].errR[i]/Cdx[j].errR[k];
	if(n == 0) lC=Cdx;
	n++;
      }
    }
    k	=n ? (i < iGr[3] ? 1 : 3) : 0; 
    err	=n ? err/n : 0.; 
    fprintf(lF,str,SNm[i],k,i < iGr[3] ? 2 : 3,i,lC->C[i],lC->G[i]
	    ,0.,0.,err,0.,0.,0.,0.);
    i++;
  }
  fclose(lF);
  printf("Output for ESC: %s\n",FNmO);
  return(0);
}

void PlotFullRawWF()
{
  int j,n,k[5]={0,0,14,11,4};
  SIGNAL *lS;

  if(nData == 0) return;

  lS	=Sig+iSig;
  n	=lS->n;
  lS	=Sig+iSig;
  ZColor(0);
  ZPlotUTopMString(lNmShot,strlen(lNmShot));
  for(j=0; j < kSig; j++){
    if((lS->c&0x100) == 0 && lS->c){
      ZColor(k[lS->c]);
      ZPlotPolyLine(lTime,lS->D,n);
      if(lS->kxy) ZPlotLUString(lS->x,lS->y,lS->Nm,strlen(lS->Nm));
    }
    lS++;
  }
}

void PlotFullRawWFandSlope()
{
  int j,n,k[5]={0,0,14,11,4};
  SIGNAL *lS;
  double x[2],y[2];

  if(nData == 0) return;

  lS	=Sig+iSig;
  n	=lS->n;
  lS	=Sig+iSig;
  x[0]=lTime[0];
  x[1]=lTime[n-1];
  y[0]=0;
  ZColor(0);
  ZPlotUTopMString(lNmShot,strlen(lNmShot));
  for(j=0; j < kSig; j++){
    if((lS->c&0x100) == 0 && lS->c){
      ZColor(k[lS->c]);
      ZPlotPolyLine(lTime,lS->D,n);
      if(lS->kxy) ZPlotLUString(lS->x,lS->y,lS->Nm,strlen(lS->Nm));
      ZColor(4);
      y[1]=lS->B*(x[1]-x[0]);
      ZPlotPolyLine(x,y,2);
    }
    lS++;
  }
}

void PlotRawWF()
{
  int j,n,k[5]={0,0,14,11,4};
  SIGNAL *lS;

  if(nData == 0) return;

  lS	=Sig+iSig;
  n	=i1D-i0D;
  lS	=Sig+iSig;
  ZColor(0);
  ZPlotUTopMString(lNmShot,strlen(lNmShot));
  for(j=0; j < kSig; j++){
    if((lS->c&0x100) == 0 && lS->c){
      ZColor(k[lS->c]);
      ZPlotPolyLine(lTime+i0D,lS->D+i0D,n);
      if(lS->kxy) ZPlotLUString(lS->x,lS->y,lS->Nm,strlen(lS->Nm));
    }
    lS++;
  }
}

static int iRF,jRF;

void PlotRFLevel()
{
  double x[4],y[4];

  ZColor(11);
  x[0]=RF[iRF][jRF].tk[0];
  x[1]=RF[iRF][jRF].tk[RF[iRF][jRF].K];
  y[0]=RF[iRF][jRF].rA;
  y[1]=y[0];
  ZPlotPolyLine(x,y,2);
  ZColor(4);
  y[0]=RF[iRF][jRF].G*SensorC[jRF]/RF[iRF][jRF].C;
  y[1]=y[0];
  ZPlotPolyLine(x,y,2);
}

int PlotRespFunc(int W,int Fl,int iM)
{
  int kPS;
  int i,j,jj,J,n,M,K,k[8]={0,0,14,11,4,0,0,0};
  int m[NMFP];
  double x[2],y[2],X[3*KRSP],Y[3*KRSP];
  double *t,*S,h;
  char str[128];
  SIGNAL *lS;

  kPS	=Fl/256;
  if(kPS == 3) kPS=4;
  K	=kGr[iM];
  lS	=Sig+iGr[iM];
  for(i=0,M=0; i < K; i++){
    if((Fl&0x01) && lS->c == 1  || (Fl&0x02) && lS->c == 2
       || (Fl&0x04) && lS->c == 3 || (Fl&0x08) && lS->c == 4){
      n	=lS->K1;
      t	=lS->tk;
      S	=lS->r0;
      if(M == 0){
	x[0]	=t[0];
	x[1]	=t[n-1];
	y[0]	=*S++;
	y[1]	=y[0];
      }
      if(x[0] > t[0]) x[0]=t[0];
      if(x[1] < t[n-1]) x[1]=t[n-1];
      for(j=1; j < n; j++){
	if(y[0] > *S) y[0]=*S;
	if(y[1] < *S) y[1]=*S;
	S++;
      }
      m[M]	=iGr[iM]+i;
      M++;
    }
    lS++;
  }
  if(M == 0) return(0);

  sprintf(str,"%s: Response Function",lNmShot);
  SetPlotName("gt",lS0[iM]->Nm,str);
  Scale2d(W,x,y,2,6,2);
  if(kPS){
    PSFileOpen(kPS,kPS);
    PSNewFrame();
  }
  n	=3*(nSS2-1);
  for(i=0; i < M; i++){
    lS	=Sig+m[i];
    J	=lS->K1-1;
    t	=lS->tk;
    h	=t[J]/n;
    S	=lS->r0;
    for(j=0; j < n; j++){
      X[j]	=h*j;
      jj	=j/3;
      splr0(Y+j,NULL,X+j,lS->r0,lS->r2,t,&J);
    }
    X[j]=h*j;
    Y[j]=lS->r0[J];
    
    Plot2d(W,X,Y,n+1,6,0,k[lS->c],0);
    if(kPS) PSPlot2d(X,Y,n+1,6,0,k[lS->c],0);
  }
  if(kPS) PSFileClose();
  return(0);
}

int PlotSplineSig(int W,int Mk,int Fl,int iM)
{
  int kPS;
  int i,j,n,n1,K,M,k[8]={0,11,14,11,4,0,0,0};
  int m[kSig];
  double x[2],y[2],t0;
  double *t,*t1,*S;
  char str[128];
  SIGNAL *lS;
  
  kPS	=Mk/256;
  if(kPS == 3) kPS=4;
  K	=kGr[iM];
  lS	=Sig+iGr[iM];
  n1	=i1D-i0D;
  t1	=lS->t+i0D;
  t0=lS->t[0];
  if(Fl){
    n	=lS->n;
    t	=lS->t;
  }
  else{
    n	=n1;
    t	=t1;
  }
  x[0]	=t[0];
  x[1]	=t[n-1];
  for(i=0,M=0; i < K; i++){
    if((Mk&0x01) && lS->c == 1  || (Mk&0x02) && lS->c == 2
       || (Mk&0x04) && lS->c == 3 || (Mk&0x08) && lS->c == 4){
      S	=Fl ? lS->D : lS->D+i0D;
      if(M == 0){
	y[0]	=*S++;
	y[1]	=y[0];
      }
      for(j=1; j < n; j++){
	S++;
	if(y[0] > *S) y[0]=*S;
	if(y[1] < *S) y[1]=*S;
      }
      m[M]	=iGr[iM]+i;
      M++;
    }
    lS++;
  }
  if(M == 0) return(0);

  sprintf(str,"%s: Spline approximation",lNmShot);
  SetPlotName("t",lS0[iM]->Nm,str);
  Scale2d(W,x,y,2,6,2);
  if(kPS){
    PSFileOpen(kPS,kPS);
    PSNewFrame();
  }
  for(i=0; i < M; i++){
    lS	=Sig+m[i];
    S	=lS->s0;
    Plot2d(W,t1,S,n1,6,0,0,0);
    if(kPS) PSPlot2d(t1,S,n1,6,0,0,0);

    if((Mk&0x10) == 0){
      S	=Fl ? lS->D : lS->D+i0D;
      Plot2d(W,t,S,n,6,0,k[lS->c],0);
      if(kPS) PSPlot2d(t,S,n,6,0,k[lS->c],0);
    }
    if((Mk&0x20) && lS->K1){
      Plot2d(W,t1,lS->S,n1,6,0,4,0);
      if(kPS) PSPlot2d(t1,lS->S,n1,6,0,4,0);
    }
    if((Mk&0x40) == 0 && lS->B != 0.){
      if(iSVD == 1){
	y[0]=lS->B*(x[0]-t0);
	y[1]=lS->B*(x[1]-t0);
      }
      else{
	y[0]	=0.;
	y[1]	=lS->B*(x[1]-x[0]);
      }
      Plot2d(W,x,y,2,6,0,4,0);
      if(kPS) PSPlot2d(x,y,2,6,0,4,0);
    }	
  }
  if(kPS) PSFileClose();
  return(0);
}

int PlotError(int W,int RorS,int Fl,int iM)
{
  int kPS;
  int i,j,K,M,k[8]={0,0,14,11,4,0,0,0};
  int m[NMFP];
  double x[4],y[4];
  double s;
  char str[128];
  SIGNAL *lS;
  
  kPS	=Fl/256;
  if(kPS == 3) kPS=4;
  K	=kGr[iM];
  lS	=Sig+iGr[iM];
  x[0]	=0.;
  x[1]	=kSig+1;
  y[0]	=0.;
  y[1]	=0.;
  for(i=0,M=0; i < K; i++){
    if((Fl&0x01) && lS->c == 1  || (Fl&0x02) && lS->c == 2
       || (Fl&0x04) && lS->c == 3 || (Fl&0x08) && lS->c == 4){
      m[M]	=iGr[iM]+i;
      j		=m[M];
      s	=RorS ? Cdx[iCdx].errR[j] : Cdx[iCdx].errS[j];
      if(y[1] < s) y[1]=s;
      M++;
    }
    lS++;
  }
  if(M == 0) return(0);
  sprintf(str,"%s: Error in %s",lNmShot,RorS ? "reconstruction":"splining");
  SetPlotName("t",lS0[iM]->Nm,str);
  Scale2d(W,x,y,2,6,2);
  if(kPS){
    PSFileOpen(kPS,kPS);
    PSNewFrame();
  }
  for(i=0; i < M; i++){
    lS	=Sig+m[i];
    x[0]	=m[i]-iGr[iM]+1;
    x[1]	=x[0];
    j		=m[i];
    y[1]	=RorS ? Cdx[iCdx].errR[j] : Cdx[iCdx].errS[j];
    Plot2d(W,x,y,2,6,0,k[lS->c],0);
    if(kPS) PSPlot2d(x,y,2,6,0,k[lS->c],0);
    x[0]	-=0.1;
    x[1]	+=0.1;
    y[2]	=y[1];
    Plot2d(W,x,y+1,2,6,0,k[lS->c],0);
    if(kPS) PSPlot2d(x,y+1,2,6,0,k[lS->c],0);
  }
  if(kPS) PSFileClose();
  return(0);
}

int RawDataAO(FILE* lF)
{
  int i,j,k,m,n;
  char ln1[64],ln[64],str[64];
  int L;
  int LNm[nSNm];
  char Nm[nSNm][16];
  int kk[NPFC];
  char Nm0[16];
  CDX *lC;

  strcpy(ln,"%s%c");
  strcpy(str,ln);
  FormatWrite2Read(str);
  fprintf(lF,"<DirName>[%d](%s)\n",1,str);
  fprintf(lF,"%s\n\n",DirNm);

  L	=0;
  n	=iGr[2];
  for(i=iGr[1]; i < n; i++){
    memset(Nm[i],' ',15);
    LNm[i]=strlen(SNm[i]);
    if(L < LNm[i]) L=LNm[i];
  }
  for(i=iGr[1]; i < n; i++) strcpy(Nm[i]+L-LNm[i],SNm[i]);

  for(j=0; j < L; j++){
    for(i=iGr[1]; i < n; i++){
      putc(' ',lF);
      putc(' ',lF);
      putc(Nm[i][j],lF);
    }
    putc('\n',lF);
  }
  for(i=iGr[1]; i < n; i++) fputs("---",lF);
  putc('\n',lF);

  for(i=iGr[1]; i < n; i++) kk[i]=nCdx;
  for(i=0; i < nCdx; i++){
    for(j=iGr[1]; j < n; j++) if(Cdx[i].kS[j] == 2) break;
    if(j < n) kk[j]	=i;
  }

  m	=iGr[5];
  for(j=iGr[2]; j < m; j++){
    for(i=iGr[1]; i < n; i++){
      putc(' ',lF);
      putc(' ',lF);
      putc(kk[i] < nCdx ? (Cdx[kk[i]].kS[j] ? 'x' : '-') : ' ',lF);
    }
    putc(' ',lF);
    putc(' ',lF);
    fputs(SNm[j],lF);
    putc('\n',lF);
  }

  for(i=0; i < nSNm; i++){
    LNm[i]=strlen(SNm[i]);
    if(L < LNm[i]) L=LNm[i];
  }
  for(i=0; i < nSNm; i++){
    memset(Nm[i],' ',15);
    strcpy(Nm[i]+L-LNm[i],SNm[i]);
  }
  memset(Nm0,' ',15);
  strcpy(Nm0+L-4,"Name");

  n	=0;
  for(j=0; j < nCdx; j++) if(Cdx[j].k == 2) n++;
  strcpy(ln1,"%2d %2d %6d %6d %6d %6d %6d %6d %3d %3d %s%c");
  strcpy(str,ln1);
  FormatWrite2Read(str);
  fprintf(lF,"<ShotList>[%d](%s)\n",n,str);
  strcpy(ln,"%2d %s %2d %10.3e %9.3e %8.2e %8.2e %8.2e %8.2e %8.2e %9.2e%c");
  strcpy(str,ln);
  FormatWrite2Read(str);
  for(j=0; j < nCdx; j++){
    if(Cdx[j].k == 2){
      lC	=Cdx+j;
      fprintf(lF,ln1,j,lC->kI,lC->iD[1][0],lC->iD[1][1]
	      ,lC->iD[2][0],lC->iD[2][1],lC->iD[3][0]
	      ,lC->iD[3][1],lC->K[2],lC->K[3],lC->Nm,'\n');
      fprintf(lF,"  <ShotList-%d>[%d](%s)\n",j,nSNm,str);
      fprintf(lF,"!# %s %2s %10s %9s %8s %8s %8s %8s %8s %9s\n"
	      ,Nm0," c","C","G","dT","gaS","gaR","ErrS","ErrR","Slope");
      for(i=0; i < nSNm; i++){
	fprintf(lF,ln,i,Nm[i],lC->kS[i],lC->C[i],lC->G[i]
		,lC->dT[i],lC->gaS[i]
		,lC->gaR[i],lC->errS[i],lC->errR[i],Cdx[j].B[i],'\n');
      }
      fprintf(lF,"!# %s %2s %10s %9s %8s %8s %8s %8s %8s %9s\n"
	      ,Nm0," c","C","G","dT","gaS","gaR","ErrS","ErrR","Slope");
    }
  }
  return(0);
}

int RawDataAI(FILE *lF,char *lm)
{
  int i,j,k,n,m,L,Ln;
  char ln[32]="<ShotList>";
  char str[64],str1[64];
  char *lc;
  char *lv[24];
  int dl[24];

  Ln	=strlen(ln)-1;
  L	=sizeof(CDX);

  if(IsKeyWordFound(lF,lm,ln,str,&n)){
    if(nCdx < n) nCdx=n;
    for(k=0; k < nCdx; k++) Cdx[k].k=2;
    lv[0]	=(char*)&k;
    lv[1]	=(char*)&Cdx[0].kI;
    lv[2]	=(char*)Cdx[0].iD[1];
    lv[3]	=(char*)(Cdx[0].iD[1]+1);
    lv[4]	=(char*)Cdx[0].iD[2];
    lv[5]	=(char*)(Cdx[0].iD[2]+1);
    lv[6]	=(char*)Cdx[0].iD[3];
    lv[7]	=(char*)(Cdx[0].iD[3]+1);
    lv[8]	=(char*)(Cdx[0].K+2);
    lv[9]	=(char*)(Cdx[0].K+3);
    lv[10]	=Cdx[0].Nm;
    dl[0]	=0;
    dl[1]	=L;
    dl[2]	=L;
    dl[3]	=L;
    dl[4]	=L;
    dl[5]	=L;
    dl[6]	=L;
    dl[7]	=L;
    dl[8]	=L;
    dl[9]	=L;
    dl[10]	=L;
    for(i=0; i < n; i++){
      strcpy(ln+Ln,">");
      ReadAI(lF,ln,lm,str,1,lv,dl,11);
      sprintf(ln+Ln,"-%d>",i);
      if(IsKeyWordFound(lF,lm,ln,str1,&m)){
	if(m != nSNm){
	  CbStr2UserMessage("%s:%s: %d != %d number of signals.\n"
			    ,lm,ln,n,nSNm);
	  return(1);
	}
	lv[11]	=(char*)&k;
	lv[12]	=Cdx[i].NmS[0];
	lv[13]	=(char*)Cdx[i].kS;
	lv[14]	=(char*)Cdx[i].C;
	lv[15]	=(char*)Cdx[i].G;
	lv[16]	=(char*)Cdx[i].dT;
	lv[17]	=(char*)Cdx[i].gaS;
	lv[18]	=(char*)Cdx[i].gaR;
	lv[19]	=(char*)Cdx[i].errS;
	lv[20]	=(char*)Cdx[i].errR;
	lv[21]	=(char*)Cdx[i].B;
	dl[11]	=0;
	dl[12]	=sizeof(Cdx[i].NmS[0]);
	dl[13]	=sizeof(int);
	dl[14]	=sizeof(double);
	dl[15]	=sizeof(double);
	dl[16]	=sizeof(double);
	dl[17]	=sizeof(double);
	dl[18]	=sizeof(double);
	dl[19]	=sizeof(double);
	dl[20]	=sizeof(double);
	dl[21]	=sizeof(double);
	ReadAI(lF,ln,lm,str1,m,lv+11,dl+11,11);
      }
      else{
	CbStr2UserMessage("%s: %s record is absent.\n",lm,ln);
      }
    }
  }
  else{
    CbStr2UserMessage("%s: %s record is absent.\n",lm,lc);
  }
  return(0);
}

int SetCdxShot(int iM,int Mk)
{
  int i;
  FreeSig();
  lNmShot	=Cdx[iCdx].Nm;
  switch(kTName){ /*Tokamaks*/
  case 0:
    i=ReadCdxWF(lNmShot,Mk);
    break;
  case 1:
    break;
  case 2:
    break;
  case 3:
    break;
  case 4:
    break;
  case 5:
    break;
  case 6:
    break;
  case 7:
    break;
  }
  if(i) return(1);
  if((Cdx[iCdx].k&0x02)){
    for(i=0; i < nSig; i++){
      Sig[i].c=Cdx[iCdx].kS[i];
      Sig[i].B=Cdx[iCdx].B[i];
    }
  }
  else{
    for(i=1; i < 4; i++){
      iSig	=iGr[i];
      nD[i]	=Sig[iSig].n;
      iD[i][0]=0;
      iD[i][1]=nD[i];
      iD[i][2]=nD[i];
      Cdx[iCdx].iD[i][0]=0;
      Cdx[iCdx].iD[i][1]=Sig[iSig].n;
      Cdx[iCdx].tW[i][0]=Sig[iSig].t[iD[i][0]];
      Cdx[iCdx].tW[i][1]=Sig[iSig].t[iD[i][2]-1];
    }
    for(i=0; i < nSig; i++) Cdx[iCdx].kS[i]=Sig[i].c;
  }
  i	=0;
  iSig	=iGr[i];
  nD[i]	=Sig[iSig].n;
  iD[i][0]	=0;
  iD[i][1]	=nD[i];
  iD[i][2]	=nD[i];
  Cdx[iCdx].iD[i][0]=0;
  Cdx[iCdx].iD[i][1]=Sig[iSig].n;
  Cdx[iCdx].tW[i][0]=Sig[iSig].t[iD[i][0]];
  Cdx[iCdx].tW[i][1]=Sig[iSig].t[iD[i][2]-1];

  for(i=0; i < 4; i++){
    iSig	=iGr[i];
    iD[i][0]	=Cdx[iCdx].iD[i][0];
    iD[i][2]	=Cdx[iCdx].iD[i][1];
    nD[i]	=iD[i][2]-iD[i][0];

    Cdx[iCdx].tW[i][0]=Sig[iSig].t[iD[i][0]];
    Cdx[iCdx].tW[i][1]=Sig[iSig].t[iD[i][2]-1];
    lS0[i]	=NULL;
    lT[i]	=Sig[iSig].t;
  }
  for(i=0; i < nSig; i++){
    Cdx[iCdx].iA[i]	=0;
    Cdx[iCdx].C[i]	=Sig[i].C;
    Cdx[iCdx].G[i]	=Sig[i].G;
    Sig[i].kxy	=0;
    Sig[i].kspl	=0;
  }
  Cdx[iCdx].k	=2;
  iSig	=iGr[iM];
  kSig	=kGr[iM];
  return(0);
}

int RawData()
{
  int iP=0,Fl=0,iExt=1;
  int n=0,iM=1,M=0,Mk=0;
  int i,j,k,m,LO,iL,I;
  int colOld[4]={1,1,1,1};
  char Mask[16]	="";
  char str[128];
  char *lc,*lt;
  char NmShot[16]="0428042057";
  char *MaskNm;

  FILE *lf,*lF; 
  SIGNAL *lS;
  char *Nm[256],Nm0[16]="GetData";
  double *ld,Y[8]={0.,0.,1.,0.,0.,0.,0.,0.};
  double X[8]={0.,1.,0.,0.,0.,0.,0.,0.};
  int kc[256];
  int nDt;
  int Nxt[8]={0,0,0,0,0,0,0,0};
  double *lTm,*lDt,*ltm;
  double B=0.,s;

#ifndef AscI_RawData
#undef AscI_RawData
#endif /*AscI_RawData*/
  nSig	=nSNm;
  iSig	=iGr[1];
  kSig	=0;
  lNmShot=Cdx[iCdx].Nm;
 L1:
#ifndef Tbl_RawData
  for(i=0; i < nCdx; i++) if((Cdx[i].k&0x01)) break;
  if(i < nCdx){
    iCdx	=i;
    if(SetCdxShot(iM,Fl < 4 ? 1 : 3))
      CbStr2UserMessage("%s - failure reading",Cdx[iCdx].Nm);
  }
  iSig	=iGr[iM];
  kSig	=kGr[iM];
  px	=pX+iM;
  py	=pY+iM;
  MaskNm=Msk[iM];
  lS	=Sig+iSig;
  nData	=lS->n;
  lTime	=lS->t;
  M	=0;
  if(nData == 0){
    ltm	=X;
    iSig	=0;
    kSig	=1;
    Nm[0]	=Nm0;
    kc[0]	=0;
    M	=1;
    X[0]=ltm[0];
    X[1]=ltm[1];
  }
  if(nData){
    ltm	=Cdx[iCdx].tW[iM];
    if(tWF < ltm[0]) tWF=ltm[0];
    if(tWF > ltm[1]) tWF=ltm[1];
    SetInterval(iCdx,Nxt,iM);
    ltm	=Cdx[iCdx].tW[iM];
    if(iExt){
      X[0]=lTime[0];
      X[2]=lTime[nData-1];
      if(X[1] < ltm[1] || X[1] > X[2]) X[1]=X[2];
    }
    else{
      X[0]=ltm[0];
      X[1]=ltm[1];
      X[2]=ltm[1];
    }
    if(*px < -1.){
      *px=cr2*(ltm[0]+ltm[1]);
      *py=cr2*(Y[0]+Y[2]);
    }
    if(*px < ltm[0]) *px	=ltm[0];
    if(*px > ltm[1]) *px	=ltm[1];
    if(*py < Y[0]) *py	=Y[0];
    if(*py > Y[2]) *py	=Y[2];

    i0D	=Cdx[iCdx].iD[iM][0];
    i1D	=Cdx[iCdx].iD[iM][1];
    iD[iM][0]	=i0D;
    iD[iM][2]	=i1D;
    j	=strlen(MaskNm);
    M	=0;
    k	=iD[iM][1];
    for(i=0; i < kSig; i++){
      lS->i0	=i0D;
      lS->i	=k;
      lS->i1	=i1D;
      lS->c	&=0xff;
      if(strncmp(MaskNm,lS->Nm,j) == 0){
	kc[M]	=i;
	Nm[M]	=lS->Nm;
	llS[M]	=lS;
	M++;
      }
      else{
	lS->c	|=0x100;
      }
      lS++;
    }
    Y[0]	=0.;
    Y[1]	=0.;
    Y[2]	=0.;
    for(i=0; i < M; i++){
      if(llS[i]->c){
	ld	=llS[i]->D;
	for(j=0; j < nData; j++){
	  if(Y[0] > *ld) Y[0]	=*ld;
	  if(Y[2] < *ld) Y[2]	=*ld;
	  ld++;
	}
      }
    }
    Y[3]=0.;
    Y[4]=0.;
    switch(iP){
    case 1:
      for(i=0; i < M; i++) if(llS[i]->c == 2) llS[i]->c =0;
      break;
    case 2:
      for(i=0; i < M; i++) if(llS[i]->c == 1) llS[i]->c =0;
      break;
    case 3:
      for(i=0; i < M; i++) if(llS[i]->c == 3) llS[i]->c =0;
      break;
    case 4:
      for(i=0; i < M; i++) if(llS[i]->c == 4) llS[i]->c =0;
      break;
    case 5:
      for(i=0,j=iSig; i < kSig; i++,j++) Sig[j].c=1;
      *MaskNm='\0';
      break;
    case 6:
      for(i=0; i < M; i++) if(llS[i]->c && llS[i]->c != 2) llS[i]->c =1;
      break; 
    case 7:
      for(i=0; i < M; i++) if(llS[i]->c && llS[i]->c != 2) llS[i]->c =3;
      break; 
    case 8:
      for(i=0; i < M; i++) if(llS[i]->c && llS[i]->c != 2) llS[i]->c =4;
      break; 
    }
    iP	=0;
    if(Fl == 1 || Fl > 1 && lS->kxy== 0){
      FindSig(&j,px,py,M);
      k	=colOld[iM];
      for(i=0; i < M; i++) if(llS[i]->c == 2) llS[i]->c=k != 2 ? k : 1;
      lS	=llS[j];
      lS0[iM]	=lS;
      colOld[iM]=lS->c;
      lS->c	=2;
      lS->kxy	=1;
      lS->x	=*px;
      lS->y	=*py;
      CbStr2UserMessage(lS->Nm);
      B=lS->B;
    }
    ld	=Cdx[iCdx].tW[iM];
    i	=cr3*(ld[1]-ld[0])*1000.;
    s	=0.001*i;
    for(j=0,i=iSig; j < kSig; j++,i++){
      Cdx[iCdx].kS[i]=Sig[i].c;
      if(Sig[i].c == 2)	Cdx[iCdx].gaS[i]=ga3[iM];
      Cdx[iCdx].dT[i]=s;
    }
    Mk	&=0x071f;
    nD[iM]	=i1D-i0D;
    lT[iM]	=lTime+i0D;
    lS	=lS0[iM];
    if(lS0[iM] != NULL && iM < 2) MaxV(Cdx[iCdx].errR+m,lS->D+i0D,nD[iM]);
    if(lS0[iM] != NULL && Fl > 1){
      double d,a;
      lS	=Sig+iSig;
      m=iSig;
      k=(Mk&0x0f) ? Mk : Mk+2;
      for(i=0; i < kSig; i++){
	lS->i0	=i0D;
	lS->i1	=i1D;
	if(lS->c == 2) B=lS->B;
	if(lS == lS0[iM] || (k&0x01) && lS->c == 1 || (k&0x02) && lS->c == 2
	   || (k&0x04) && lS->c == 3 || (k&0x08) && lS->c == 4){
	  if(Fl == 2){
	    memcpy(lS->s0,lS->D+i0D,nD[iM]*sizeof(double));
	    if(iM < 2){
	      lS->s0[0]=0.;
	      j	=i1D-i0D-1;
	      lS->s0[j]=0.;	
	      f2spl(lS->s0,lS->s2,NULL,NULL,lS->s0,NULL,lS->t+i0D
		    ,&j,&A0,&A0,&A0,Cdx[iCdx].gaS+m);
	    }
	    else{
	      Data2Spline(lS->s0,lS->s2,lS->s0,lS->t+i0D,i1D-i0D
			  ,0.,0.,0.,Cdx[iCdx].gaS[m]);
	    }
	    lS->kspl	|=1;
	    MaxDiff(&d,&a,lS->s0,lS->D+i0D,nD[iM]);
	    Cdx[iCdx].iA[m]	=1;
	    Cdx[iCdx].errS[m]	=a;
	    CbStr2UserMessage("%8s: %8.2e <%8.2e>\n",lS->Nm,d,a);
	  }
	  if(iM > 1 && Fl == 3){
	    SigSlope(lS,X[1]);
	    if(lS->c == 2) B=lS->B;
	  }
	}
	m++;
	lS++;
      }
      lS	=lS0[iM];
      PlotSplineSig(3,k,iExt,iM);
      PlotError(4,0,k,iM);
      Mk&=0x001f;

      lD[iM]	=lS->s0;
      lD2[iM]	=lS->s2;
      lDt	=lS->s0;
      lTm	=lT[iM];
    }
    k	=0;
    for(i=0; i < M; i++){
      if(llS[i]->c == 2){
	if(k)  llS[i]->c=1;
	k	=1;
      }
    }
  }
  lS	=Sig+iSig;
  nDt	=Fl != 2 || (Mk&0x10) ? 0 : nD[iM];
#ifdef H
  if(Fl == 4) WriteCdxLayOut(lNmShot);
  CbUserPlot=iExt && Fl == 3 ? PlotFullRawWFandSlope : PlotFullRawWF;
#endif

  CbUserPlot=iExt ? (Fl == 3 ? PlotFullRawWFandSlope : PlotFullRawWF)
    : PlotRawWF;
#endif /*Tbl_RawData*/
  for(i=0; i < nCdx; i++) if((Cdx[i].k&0x02)) break;
  if(i == nCdx){
    CbStr2UserWarning("No Source specified for calibration");
    goto L1;
  }

  k	=sizeof(CDX);
  for(i=0,j=0; i < nCdx; i++){
    if((Cdx[i].k&0x02)){
      for(m=iGr[1]; m < iGr[2]; m++){
	if((Cdx[i].kS[m]&0xff) == 2) break;
      }
      if(m < iGr[2]){
	Cdx[i].kI	=m;
	if(j < i){
	  memcpy(Cdx+j,Cdx+i,k);
	  if(iCdx == i) iCdx =j;
	}
	j++;
      }
    }
  }

  nCdx	=j;
  for(j=0; j < nCdx; j++){
    for(i=0; i < nSNm; i++) Cdx[j].kS[i]&=0xff;
  }
#ifndef AscO_RawData
#undef AscO_RawData
#endif /*AscO_RawData*/
  return(0);
}

int MakeSignal(double *tj, double *S, int J1
	       ,double B,double *s0, double *s2, double *tk,int K1
	       ,double *V0, double *V2, double *ti,int I1)
{
  int I,J,K;
  int i,ii,i1,j,k,kk;
  double d,dd,dd6;
  double f0,f1,p0,p1,p2,p3;
  double h,rh,q0,q1,q2;
  double Q0[KRSP],Q1[KRSP],Q2[KRSP];
  double Y;
  double u0,u1,u2;
  double t,t0,t1,t2,t3;
  double Tk,Ti,Tik;

  I	=I1-1;
  J	=J1-1;
  K	=K1-1;
  t0	=ti[0];
  Tk	=tk[K];
  Ti	=ti[I];
  Tik	=Ti+Tk;

  for(kk=0,k=1; k < K1; kk++,k++){
    h	=tk[k]-tk[kk];
    rh	=1./h;
    Q0[k]	=(s0[k]-s0[kk])*rh-(cr6*s2[k]+cr3*s2[kk])*h;
    Q1[k]	=s2[kk];
    Q2[k]	=0.5*(s2[k]-s2[kk])*rh;
  }

  h	=Tk/K;
  rh	=1./h;

  S[0]	=0.;
  j	=0;
  while(j < J1 && tj[j] < t0) S[j++]=0.;

  ii	=0;
  for(; j < J1; j++){
    t	=tj[j];
    t1	=t-t0;
    S[j]=B*t1;

    /*Convolution integrals*/
    i	=ii+1;
    if(t < Ti){
      i	=ii+1;
      while(i < I && ti[i] < t) i++;
      ii=i-1;
      d	=ti[i]-ti[ii];
      dd=d*d;
      dd6=cr6*dd;
      f0	=V2[ii]*dd6;
      f1	=V2[i]*dd6;
      p0	=V0[ii];
      p1	=V0[i]-p0-2.*f0-f1;
      p2	=3.*f0;
      p3	=f1-f0;
      Y		=(t-ti[ii])/d;
      S[j]	+=s0[0]*(p0+Y*(p1+Y*(p2+Y*p3)));
    }
    else{
      i	=I;
      if(Tik <= t) S[j] +=s0[K]*V0[I];
    }
    t2	=t;
    for(kk=0; kk < K && tk[kk] < t1; kk++){
      k		=kk+1;
      q0	=Q0[k];
      q1	=Q1[k];
      q2	=Q2[k];
      t3	=t2;
      t2	=tk[k] < t1 ? t-tk[k] : t0;
      if(t2 < Ti && Ti <= t3){
	h	=tk[k]-tk[kk];
	rh	=1./h;
	f1	=(t-Ti-tk[kk])*rh;
	f0	=1.-f0;
	S[j]	+=(f0*s0[kk]+f1*s0[k]
		   -cr6*h*h*f0*f1*((1.+f0)*s2[kk]+(1.+f1)*s2[k]))*V0[I];
	f0	=(f0*s0[kk]+f1*s0[k]
		   -cr6*h*h*f0*f1*((1.+f0)*s2[kk]+(1.+f1)*s2[k]))*V0[I];
      }
      if(t2 < Ti){
	i1	=i;
	while(i1 && ti[i1] >= t3) i1--;
	i	=i1+1;
	while(i && ti[i] > t2){
	  d	=ti[i]-ti[i1];
	  dd	=d*d;
	  dd6	=cr6*dd;
	  f0	=V2[i1]*dd6;
	  f1	=V2[i]*dd6;
	  p0	=V0[i1];
	  p1	=V0[i]-p0-2.*f0-f1;
	  p2	=3.*f0;
	  p3	=f1-f0;
	  if(ti[i] > t3){
 	    f0	=t3-ti[i1];
	    f1	=f0*f0;
	    Y	=f0/d;
	    u0  =f0*(p0+Y*(cr2*p1+Y*(cr3*p2+0.25*Y*p3)));
	    u1	=f1*(cr2*p0+Y*(cr3*p1+Y*(0.25*p2+0.2*Y*p3)));
	    u2	=f0*f1*(cr3*p0+Y*(0.25*p1+Y*(0.2*p2+cr6*Y*p3)));
	  }
	  else{
	    u0  =d*(p0+cr2*p1+cr3*p2+0.25*p3);
	    u1	=dd*(cr2*p0+cr3*p1+0.25*p2+0.2*p3);
	    u2	=d*dd*(cr3*p0+0.25*p1+0.2*p2+cr6*p3);
	  }
	  if(t2 > ti[i1]){
	    f0	=t2-ti[i1];
	    f1	=f0*f0;
	    Y	=f0/d;
	    u0  -=f0*(p0+Y*(cr2*p1+Y*(cr3*p2+0.25*Y*p3)));
	    u1	-=f1*(cr2*p0+Y*(cr3*p1+Y*(0.25*p2+0.2*Y*p3)));
	    u2	-=f0*f1*(cr3*p0+Y*(0.25*p1+Y*(0.2*p2+cr6*Y*p3)));
	  }
	  f0	=t-(ti[i1]+tk[kk]);
	  S[j]	+=q0*u0+q1*(f0*u0-u1)+q2*(f0*(f0*u0-2.*u1)+u2);
	  f0	=q0*u0+q1*(f0*u0-u1)+q2*(f0*(f0*u0-2.*u1)+u2);
	  i--;
	  i1--;
	}
      }
    } 
  }
  return(0);
}

int MakeSignalF(double tt0,double *tj, double *S, int J1
	       ,double B,double *s0, double *s2, double *tk,int K1
	       ,double *V0, double *V2, double *ti,int I1)
{
  int I,J,K;
  int i,ii,i1,j,k,kk;
  double d,dd,dd6;
  double f0,f1,p0,p1,p2,p3;
  double h,rh,q0,q1,q2;
  double Q0[KRSP],Q1[KRSP],Q2[KRSP];
  double Y;
  double u0,u1,u2;
  double t,t0,t1,t2,t3;
  double Tk,Ti,Tik;

  I	=I1-1;
  J	=J1-1;
  K	=K1-1;
  t0	=ti[0];
  Tk	=tk[K];
  Ti	=ti[I];
  Tik	=Ti+Tk;

  for(kk=0,k=1; k < K1; kk++,k++){
    h	=tk[k]-tk[kk];
    rh	=1./h;
    Q0[k]	=(s0[k]-s0[kk])*rh-(cr6*s2[k]+cr3*s2[kk])*h;
    Q1[k]	=s2[kk];
    Q2[k]	=0.5*(s2[k]-s2[kk])*rh;
  }

  h	=Tk/K;
  rh	=1./h;

  S[0]	=0.;
  j	=0;
  while(j < J1 && tj[j] < t0) S[j++]=0.;

  ii	=0;
  for(; j < J1; j++){
    t	=tj[j];
    t1	=t-t0;
    S[j]=B*(t-tt0);
    /*Convolution integrals*/
    i	=ii+1;
    if(t < Ti){
      i	=ii+1;
      while(i < I && ti[i] < t) i++;
      ii=i-1;
      d	=ti[i]-ti[ii];
      dd=d*d;
      dd6=cr6*dd;
      f0	=V2[ii]*dd6;
      f1	=V2[i]*dd6;
      p0	=V0[ii];
      p1	=V0[i]-p0-2.*f0-f1;
      p2	=3.*f0;
      p3	=f1-f0;
      Y		=(t-ti[ii])/d;
      S[j]	+=s0[0]*(p0+Y*(p1+Y*(p2+Y*p3)));
    }
    else{
      i	=I;
      if(0 && Tik <= t) S[j] +=s0[K]*V0[I];
    }
    t2	=t;
    for(kk=0; kk < K && tk[kk] < t1; kk++){
      k		=kk+1;
      q0	=Q0[k];
      q1	=Q1[k];
      q2	=Q2[k];
      t3	=t2;
      t2	=tk[k] < t1 ? t-tk[k] : t0;
      if(t2 < Ti){
	if(Ti <= t3){
	  h	=tk[k]-tk[kk];
	  rh	=1./h;
	  f1	=(t-Ti-tk[kk])*rh;
	  f0	=1.-f0;
	  S[j]	+=(f0*s0[kk]+f1*s0[k]
		   -cr6*h*h*f0*f1*((1.+f0)*s2[kk]+(1.+f1)*s2[k]))*V0[I];
	}
	else{
	  i1	=i;
	  while(i1 && ti[i1] >= t3) i1--;
	  i	=i1+1;
	  while(i && ti[i] > t2){
	    d	=ti[i]-ti[i1];
	    dd	=d*d;
	    dd6	=cr6*dd;
	    f0	=V2[i1]*dd6;
	    f1	=V2[i]*dd6;
	    p0	=V0[i1];
	    p1	=V0[i]-p0-2.*f0-f1;
	    p2	=3.*f0;
	    p3	=f1-f0;
	    if(ti[i] > t3){
	      f0	=t3-ti[i1];
	      f1	=f0*f0;
	      Y	=f0/d;
	      u0  =f0*(p0+Y*(cr2*p1+Y*(cr3*p2+0.25*Y*p3)));
	      u1	=f1*(cr2*p0+Y*(cr3*p1+Y*(0.25*p2+0.2*Y*p3)));
	      u2	=f0*f1*(cr3*p0+Y*(0.25*p1+Y*(0.2*p2+cr6*Y*p3)));
	    }
	    else{
	      u0  =d*(p0+cr2*p1+cr3*p2+0.25*p3);
	      u1  =dd*(cr2*p0+cr3*p1+0.25*p2+0.2*p3);
	      u2  =d*dd*(cr3*p0+0.25*p1+0.2*p2+cr6*p3);
	    }
	    if(t2 > ti[i1]){
	      f0	=t2-ti[i1];
	      f1	=f0*f0;
	      Y	=f0/d;
	      u0  -=f0*(p0+Y*(cr2*p1+Y*(cr3*p2+0.25*Y*p3)));
	      u1	-=f1*(cr2*p0+Y*(cr3*p1+Y*(0.25*p2+0.2*Y*p3)));
	      u2	-=f0*f1*(cr3*p0+Y*(0.25*p1+Y*(0.2*p2+cr6*Y*p3)));
	    }
	    f0	=t-(ti[i1]+tk[kk]);
	    S[j]	+=q0*u0+q1*(f0*u0-u1)+q2*(f0*(f0*u0-2.*u1)+u2);
	    i--;
	    i1--;
	  }
	}
      }
    }
  }
  return(0);
}

int RespFuncBO(FILE *lF)
{
  int i,j,k,j1,L,Ld,Li;

  Li	=sizeof(int);
  Ld	=sizeof(double);
  L	=sizeof(Cdx[0].Nm);
  fwrite(&nCdx,Li,1,lF);
  for(i=0; i < nCdx; i++){
    fwrite(Cdx[i].Nm,L,1,lF);
    fwrite(Cdx[i].K,Li,4,lF);
  }
  for(i=0; i < nCdx; i++){
    for(k=2; k < 4; k++){
      L		=Cdx[i].K[k]+1;
      j1	=iGr[k+1];
      for(j=iGr[k]; j < j1; j++){
	fwrite(Cdx[i].tk[j],Ld,L,lF);
	fwrite(Cdx[i].r0[j],Ld,L,lF);
	fwrite(Cdx[i].r2[j],Ld,L,lF);
      }
    }
  }
  return(0);
}

int RespFuncBI(FILE *lF)
{
  int i,j,k,j1,L,Ld,Li;
  double dT;
  char Nm[32];
  int K[4];

  Li	=sizeof(int);
  Ld	=sizeof(double);
  L	=sizeof(Cdx[0].Nm);
  fread(&k,Li,1,lF);
  if(k != nCdx) return(1);
  for(i=0; i < nCdx; i++){
    fread(Nm,L,1,lF);
    if(strcmp(Cdx[i].Nm,Nm)) return(1);
    strcpy(Cdx[i].Nm,Nm);
    fread(K,Li,4,lF);
    if(memcmp(Cdx[i].K+2,K+2,2*Li)) return(1);
    memcpy(Cdx[i].K+2,K+2,2*Li);
  }
  for(i=0; i < nCdx; i++){
    for(k=2; k < 4; k++){
      L	=Cdx[i].K[k]+1;
      j1	=iGr[k+1];
      for(j=iGr[k]; j < j1; j++){
	fread(Cdx[i].tk[j],Ld,L,lF);
	fread(Cdx[i].r0[j],Ld,L,lF);
	fread(Cdx[i].r2[j],Ld,L,lF);
	dT	=Cdx[i].tk[j][L-1];
	if(dT != 0.) Cdx[i].dT[j]=dT;
      }
    }
  }
  return(0);
}

int MakeAllRespFunctions()
{
  int i,j,j1,m;

  for(i=0; i < nCdx; i++){
    iSig=iGr[1];
    kSig=kGr[1];
    j1	=iGr[2];
    m	=0;
    for(j=iGr[1]; j < j1; j++) if(Cdx[i].kS[j] == 2) break; 

    if(j == j1){
      CbStr2UserWarning("MakeAllRespFunctions():%s No Coil Current\n"
			,Cdx[i].Nm);
      return(1);
    }
    if((Cdx[i].iA[j]&0x01) == 0){
      m =1;
    }
    else{
      while(j < nSig){
	if(Cdx[i].kS[j] && (Cdx[i].iA[j]&0x02) == 0) break;
	j++;
      }
      if(j < nSig) m=1;
    }
    if(m){
      FreeSig();
      ReadCdxWF(Cdx[i].Nm,1);
#ifdef H
      MakeRespFunctions();
#endif
    }
  }
  return(0);
}

int TestRespFunc(double *r0,double *r2,double h)
{
  int i,n;
  double d0,d1;

  h	*=cr6*h;
  n	=nSS2-1;
  d0	=r2[0]-2.*r2[1]+r2[2];
  d1	=0.;
  out("siddd","r''",0,d0,d1,d0-d1);
  for(i=1; i < n; i++){
    d0=r0[i]-r0[i-1]+h*(2.*r2[i]+r2[i-1]);
    d1=r0[i+1]-r0[i]-h*(r2[i+1]+2.*r2[i]);
    out("siddd"," r'",0,d0,d1,d0-d1);
  }
  d0=r0[i]-r0[i-1]+h*(2.*r2[i]+r2[i-1]);
  d1	=0.;
  out("siddd"," r'",0,d0,d1,d0-d1);
  return(0);
}

int GetIndexRF(int *iS,int *iC,int jj,int iCdx)
{
#ifdef H
  int k,n;
  n=iGr[2];
  for(k=iGr[1]; k < n; k++) if(Cdx[iCdx].kS[k] == 2) break;
  if(k == n) return(1);
  k	=Cdx[iCdx].kI;
#endif
  *iC=Cdx[iCdx].kI;
  *iS=jj-iGr[2];
  return(0);
}

int RespFunc()
{
  int Fl,iSh;
  int Next;
  static int K,K1,iB=0,iExt=1;
  int iM,iMM=0,M=0,Mk=0,iP=0;
  int i,j,k,m;
  int colOld[4]={1,1,1,};
  char *MaskNm;
  SIGNAL *lS;
  char *Nm[256];
  double *ld,Y[8]={0.,0.,1.,0.,0.,0.,0.,0.},X[8]={0.,0.,0.,0.,0.,0.,0.,0.};
  int kc[256];
  int nDt,nRt;
  double dt,*lTm,*lDt,*tk,*tj;
  double B=0.,a,d;
  double *lgaR,*ltol,*lgaS,*ldT;
  int kSVD[8],nEigen=0;
  int nxt[4]={1,1,1,1};
  double FlSVD,xSVD[8];
  int nDR,iDR;
  double *ldX,*ldY,*tS,*yS,*ySS;
  double tR[0x80],r0R[0x80],r2R[0x80];

#ifndef AscI_RespFunc
#undef AscI_RespFunc
#endif /*AscI_RespFunc*/
  i	=0;
#ifndef BnrI_RespFunc
#undef BnrI_RespFunc
#endif /*BnrI_RespFunc*/
  if(i) CbStr2UserWarning("Failure to read binary file");
  K1	=nSS2;
  K	=K1-1;
  for(i=1; i < nSig; i++){
    if(Sig[i].n == 0){
      iCdx	=0;
      Cdx[0].k	|=1;
      break;
    }
  }
  lS	=Sig+iGr[2];
  if(lS->n){
    for(k=2; k < 4; k++){
      m	=iGr[k];
      kSig	=kGr[k];
      Cdx[iCdx].K[k]	=K;
      for(i=0; i < kSig; i++){
	if(Cdx[iCdx].tk[m][K] != 0.){
	  lS->kspl	|=2;
	  lS->K1	=K1;
	  lS->tk	=Cdx[iCdx].tk[m];
	  lS->r0	=Cdx[iCdx].r0[m];
	  lS->r2	=Cdx[iCdx].r2[m];
	}
	m++;
	lS++;
      }
    }
  }
  Memory4SVD();
  lNmShot=Cdx[iCdx].Nm;
  Next	=0;
  Fl	=0;
  FlSVD	=0;
  iM	=iMM+1;
  X[0]	=Cdx[iCdx].tW[iM][0];
  X[1]	=tWF;
  X[2]	=Cdx[iCdx].tW[iM][1];
  iDR	=0;
  iSh	=0;
#ifndef Tbl_RespFunc
  if(iM != 5) iSh=0;
  Cdx[iCdx].tW[iM][1]	=X[2];
  iM	=iMM+1;
  for(i=0; i < nCdx; i++) if((Cdx[i].k&0x01)) break;
  if(i < nCdx){
    iCdx	=i;
    if(SetCdxShot(iM,1))CbStr2UserMessage("%s - failure reading",Cdx[iCdx].Nm);
    lS	=Sig+iGr[2];
    j	=iGr[2];
    for(k=2; k < 4; k++){
      m	=kGr[k];
      i0D	=Cdx[iCdx].iD[k][0];
      i1D	=Cdx[iCdx].iD[k][1];
      Cdx[iCdx].K[k]	=K;
      for(i=0; i < m; i++){
	if(Cdx[iCdx].tk[j][K] != 0.){
	  lS->B		=Cdx[iCdx].B[j];
	  lS->kspl	|=2;
	  lS->K1	=K1;
	  lS->tk	=Cdx[iCdx].tk[j];
	  lS->r0	=Cdx[iCdx].r0[j];
	  lS->r2	=Cdx[iCdx].r2[j];
	}
	j++;
	lS++;
      }
    }
    Fl	=0;
    X[2]=Cdx[iCdx].tW[iM][1];
  }
  iSig	=iGr[iM];
  kSig	=kGr[iM];
  if(Fl == 0){
    int im;
    im	=1;
    m	=Cdx[iCdx].kI;
    if(lS0[im] == NULL){
      lS	=Sig+m;
      llS[0]	=lS;
      FindSig(&j,pX+im,pY+im,1);
      k	=colOld[im];
      for(i=0; i < M; i++) if(llS[i]->c == 2) llS[i]->c=k != 2 ? k : 1;
      lS0[im]	=lS;
      colOld[im]=lS->c;
      lS->c	=2;
      lS->kxy=1;
      lS->x=pX[im];
      lS->y=pY[im];
      CbStr2UserMessage(lS->Nm);
    }
    lS	=lS0[im];
    i0D	=Cdx[iCdx].iD[im][0];
    i1D	=Cdx[iCdx].iD[im][1];
    lS->i0=i0D;
    lS->i1=i1D;
    nD[im]	=i1D-i0D;
    lT[im]	=lS->t+i0D;
    lD[im]	=lS->s0;
    lD2[im]	=lS->s2;
    memcpy(lS->s0,lS->D+i0D,nD[im]*sizeof(double));
    lS->s0[0]=0.;
    j	=nD[im]-1;
    lS->s0[j]=0.;	
    f2spl(lS->s0,lS->s2,NULL,NULL,lS->s0,NULL,lS->t+i0D,&j
	  ,&A0,&A0,&A0,Cdx[iCdx].gaS+m);
    BasicIntegrals(U0,U1,U2,lS->s0,lS->s2,lS->t+lS->i0,lS->i1-lS->i0);
    Cdx[iCdx].iA[m]	=1;
    lS->kspl	=1;
    Fl	=1;
    FlSVD	=0;
  }

  MaskNm	=Msk[iM];
  j	=strlen(MaskNm);
  M	=0;
  lS	=Sig+iSig;
  for(i=0; i < kSig; i++){
    lS->c	&=0xff;
    if(strncmp(MaskNm,lS->Nm,j) == 0){
      kc[M]	=i;
      Nm[M]	=lS->Nm;
      llS[M]	=lS;
      M++;
    }
    else{
      lS->c	|=0x100;
    }
    lS++;
  }
  if(M == 0){
    M	=1;
    llS[0]	=Sig+iSig;
    llS[0]->c	&=0x100;
  }
  switch(Next){
  case 0:
    break;
  case 1:
    m	=iGr[iM+1];
    if(lS0[iM] == NULL){
      i	=iGr[iM];
      j	=i;
    }
    else{
      i	=lS0[iM]-Sig;
      j	=i+1;
    }
    lS	=Sig+j;
    while(j < m && (lS->c&0xff) == 0){
      j++;
      lS++;
    }
    if(j == m){
      CbStr2UserMessage("No next active signals\n");
      j	=i;
    }
    break;
  case 2:
    m	=iGr[iM];
    if(lS0[iM] == NULL){
      i	=iGr[iM+1]-1;
      j	=i;
    }
    else{
      i	=lS0[iM]-Sig;
      j	=i-1;
    }
    lS	=Sig+j;
    while(j >= m && (lS->c&0xff) == 0){
      j--;
      lS--;
    }
    if(j < m){
      CbStr2UserMessage("No next active signals\n");
      j	=i;
    }
    break;
  }
  if(Next){
    lS0[iM]	=Sig+j;
    lS	=Sig+iGr[iM];
    k	=colOld[iM];
    m	=iGr[iM+1];
    for(i=iGr[iM]; i < m; i++,lS++) if(lS->c == 2) lS->c=k != 2 ? k : 1;
    colOld[iM]=Cdx[iCdx].kS[j];
    Cdx[iCdx].kS[j]=2;
    lS	=lS0[iM];
    lS->c=2;
    if(lS->kxy == 0){
      i		=(Cdx[iCdx].iD[iM][0]+Cdx[iCdx].iD[iM][1])/2;
      lS->x	=lS->t[i];
      lS->y	=lS->D[i];
      lS->kxy=1;
    }
    pX[iM]=lS->x;
    pY[iM]=lS->y;
  }

  Cdx[iCdx].K[iM]=K;
  px	=pX+iM;
  py	=pY+iM;

  if(iP == 0) Mk=0;
  if(iP){
    FindSig(&j,px,py,M);
    k	=colOld[iM];
    for(i=0; i < M; i++) if(llS[i]->c == 2) llS[i]->c=k != 2 ? k : 1;
    lS	=llS[j];
    lS0[iM]	=lS;
    colOld[iM]=lS->c;
    lS->c=2;
    lS->kxy=1;
    lS->x=*px;
    lS->y=*py;
    if(iB == 1){
      lS->B=B;
    }
    else{
      B=lS->B;
    }
    if(iP == 1) CbStr2UserMessage(lS->Nm);
  }
  /* Searching for red signal */
  i	=0;
  while(i < M && llS[i]->c == 0) i++;
  m	=i;
  while(i < M && llS[i]->c != 2) i++;
  if(i == M) i=m;
  lS	=llS[i];
  if(lS->kxy == 0){
    lS->kxy=1;
    j	=(lS->i0+lS->i1)/2;
    lS->x	=Sig[iSig].t[j];
    lS->y	=lS->D[j];
  }
  lS0[iM]	=lS;
  if(lS->c != 2){
    colOld[iM]	=lS->c;
    lS->c	=2;
  }
  pX[iM]	=lS->x;
  pY[iM]	=lS->y;
  for(i++; i < M; i++) if(llS[i]->c == 2) llS[i]->c=1;

  for(j=0,i=iSig; j < kSig; j++,i++){
    Cdx[iCdx].kS[i]=Sig[i].c != 2 ? Sig[i].c&0xff : colOld[iM];
  }

  m	=lS-Sig;
  ldT	=Cdx[iCdx].dT+m;
  lgaR	=Cdx[iCdx].gaR+m;
  ltol	=Cdx[iCdx].tol+m;
  lgaS=Cdx[iCdx].gaS+m;
  ga3[iM]=*lgaS;
  ga3rf[iM]=*lgaR;
  lS	=Sig+iSig;
  nData	=lS->n;
  lTime	=lS->t;
  tWF	=Cdx[iCdx].tW[iM][0]+*ldT;
  SetInterval(iCdx,nxt,iM);

  tWF	=Cdx[iCdx].tW[iM][0]+*ldT;
  i0D	=Cdx[iCdx].iD[iM][0];

  i1D=Cdx[iCdx].iD[iM][1];
  iD[iM][0]	=i0D;
  iD[iM][2]	=i1D;
  X[0]	=Cdx[iCdx].tW[iM][0];
  X[1]	=tWF;
  X[2]	=Cdx[iCdx].tW[iM][1];
  if(iExt){
    X[3]=lTime[0];
    X[5]=lTime[nData-1];
    if(X[4] < X[2] || X[4] > X[5]) X[4]=X[5];
  }
  else{
    X[3]=X[0];
    X[4]=X[2];
    X[5]=X[2];
  }
  Y[0]	=0.;
  Y[1]	=0.;
  Y[2]	=0.;
  for(i=0; i < M; i++){
    if(llS[i]->c){
      ld	=llS[i]->D;
      for(j=0; j < nData; j++){
	if(Y[0] > *ld) Y[0]	=*ld;
	if(Y[2] < *ld) Y[2]	=*ld;
	ld++;
      }
    }
  }
  if(*px < -1.){
    *px=cr2*(X[0]+X[2]);
    *py=cr2*(Y[0]+Y[2]);
  }
  if(*px < X[0]) *px	=X[0];
  if(*px > X[2]) *px	=X[2];
  if(*py < Y[0]) *py	=Y[0];
  if(*py > Y[2]) *py	=Y[2];
  
  nD[iM]	=i1D-i0D;
  lT[iM]	=lTime+i0D;
  nDt	=0;
  nRt	=0;
  if(iP > 1 && iP < 5){
    if((Mk&0x10) == 0) nDt=nD[iM];
    lS	=Sig+iSig;
    m	=iSig;
    k	=(Mk&0x0f) ? Mk : Mk+2;
    for(i=0; i < kSig; i++){
      lS->i0	=i0D;
      lS->i1	=i1D;
      if(lS == lS0[iM] || (k&0x01) && lS->c == 1 || (k&0x02) && lS->c == 2
	 || (k&0x04) && lS->c == 3 || (k&0x08) && lS->c == 4){
	memcpy(lS->s0,lS->D+i0D,nD[iM]*sizeof(double));
	if(iM < 2){
	  lS->s0[0]=0.;
	  j	=nD[iM]-1;
	  lS->s0[j]=0.;	
	  f2spl(lS->s0,lS->s2,NULL,NULL,lS->s0,NULL,lS->t+i0D
		,&j,&A0,&A0,&A0,Cdx[iCdx].gaS+m);
	}
	else{
	  Data2Spline(lS->s0,lS->s2,lS->s0,lS->t+i0D,nD[iM]
		      ,0.,0.,0.,Cdx[iCdx].gaS[m]);
	}
	MaxDiff(&d,&a,lS->s0,lS->D+i0D,nD[iM]);
	lS->kspl	|=0x01;
	Cdx[iCdx].iA[m]	=1;
	Cdx[iCdx].errS[m]	=a;
	if(iB == 2 && iP == 2){
	  SigSlope(lS,X[4]);
	  if(lS->c == 2) B=lS->B;
	}
	if(iP==2)CbStr2UserMessage("%8s: <%8.2e>\n",lS->Nm,Cdx[iCdx].errS[m]);
      }
      m++;
      lS++;
    }
    if(iM == 1) BasicIntegrals(U0,U1,U2,lS0[iM]->s0,lS0[iM]->s2
			       ,lS0[iM]->t+i0D,nD[iM]);
  }
  if(iP == 2){
    PlotSplineSig(3,k,iExt,iM);
    PlotError(4,0,k,iM);
  }
  lS	=lS0[iM];

  if(iB == 1) lS->B=iM > 1 ? B : 0.;

  lD[iM]	=lS->s0;
  lD2[iM]	=lS->s2;

  if(iM > 1 && iP > 2){
    tj	=lT[iM];
    lS	=Sig+iSig;
    m	=iSig;
    k	=(Mk&0x0f) ? Mk : Mk+2;
    for(i=0; i < kSig; i++){
      if(lS == lS0[iM] || (k&0x01) && lS->c == 1 || (k&0x02) && lS->c == 2
	 || (k&0x04) && lS->c == 3 || (k&0x08) && lS->c == 4){
	if(1 || lS == lS0[iM] || lS->K1 != K1 || (lS->kspl&0x02) == 0){
	  lS->kspl	&=0x01;
	  lS->K1=K1;
	  tk	=lS->tk;
	  dt	=Cdx[iCdx].dT[m];
	  for(j=0; j < K1; j++) tk[j]=dt*tSS2[j];
	  if(iP == 3){
	    lS->ga	=Cdx[iCdx].gaR[m];
	    if(FlSVD){
	      if(iCdx != kSVD[0] || kSVD[1] != K1 || kSVD[2] != nD[iM]
		 || kSVD[3] != nD[0] || kSVD[4] != iB
		 || xSVD[0] != Cdx[iCdx].dT[m]
		 || xSVD[1] != lT[0][0]) FlSVD=0;
	    }
	    if(FlSVD == 0){
	      SetMatrixSVD(tk,K1,lT[iM],nD[iM],lD[1],lD2[1],lT[1],nD[1]);
	      FlSVD	=1;
	    }
	    EigenMin	=*ltol;
	    if(FlSVD < 2 && iSVD == 2){
	      MakeSVD4RespF(iB);
	      FlSVD	=2;
	      kSVD[0]	=iCdx;
	      kSVD[1]	=K1;
	      kSVD[2]	=nD[iM];
	      kSVD[3]	=nD[1];
	      kSVD[4]	=iB;
	      xSVD[0]	=Cdx[iCdx].dT[m];
	      xSVD[1]	=lT[1][0];
	      ScalePlotEigenSVD();
	      PlotEigenSVD();
	    }
	    j	=0;
	    while(lT[iM][j] < lT[1][0]) j++;

	    if(iB == 0) lS->B=B;
	    lS->r0[K1]=lS->B;

	    if(FlSVD == 2 && iSVD == 2){
	      GetSVDSolution(lS->r0,lS->r2,tk,EigenMin,lS->s0+j);
	      ScalePlotEigenSVD();
	      PlotEigenSVD();
	      j	=1;
	    }
	    else{
	      if(iSVD == 1){
		Spline2RespF(iB,lTime[0],lS->r0+K1,lS->r0,lS->r2,tk,K1,
			     Cdx[iCdx].gaR[m]
			     ,lS->s0+j,lT[iM],nD[iM],lD[1],lD2[1],lT[1],nD[1]);
		j	=1;
	      }
	      else{
		ZMessage(Cdx[iCdx].NmS[m]);
		if(SetMatrixLSQ(Cdx[iCdx].gaR[m])){
		  CbStr2UserWarning("%8s: Failure\n",lS->Nm);
		  j	=0;
		}
		else{
		  Signal2RespFunc(lS->r0,lS->r2,tk,lS->s0+j);
		  j	=1;
		}
	      }
	    }
	    if(j){
	      if(iB == 0) lS->B=lS->r0[K1];
	      j	=K1*sizeof(double);
	      Cdx[iCdx].B[m]=lS->B;
	      lS->kspl	|=2;
	      Cdx[iCdx].iA[m]	|=2;
	      if(iSVD == 1){
		MakeSignalF(lTime[0],lT[iM],lS->S,nD[iM],lS->B,lS->r0,lS->r2
			    ,tk,K1,lD[1],lD2[1],lT[1],nD[1]);
	      }
	      else{
		MakeSignal(lT[iM],lS->S,nD[iM],lS->B,lS->r0,lS->r2,tk,K1
			   ,lD[1],lD2[1],lT[1],nD[1]);
	      }
	      MaxDiff(&d,&a,lS->s0,lS->S,nD[iM]);
	      Cdx[iCdx].errA[m]	=d;
	      Cdx[iCdx].errR[m]	=a;
	      CbStr2UserMessage("%8s: %8.2e <%8.2e>\n"
				,lS->Nm,Cdx[iCdx].errA[m],Cdx[iCdx].errR[m]);
	    }
	  }
	  if(iP == 4){
	    if (iSVD == 1){
	      MakeSignalF(lTime[0],lT[iM],lS->S,nD[iM],lS->B,lS->r0,lS->r2
			  ,tk,K1,lD[1],lD2[1],lT[1],nD[1]);
	    }
	    else{
	      MakeSignal(lT[iM],lS->S,nD[iM],lS->B,lS->r0,lS->r2,tk,K1
			 ,lD[1],lD2[1],lT[1],nD[1]);
	    }
	    MaxDiff(&d,&a,lS->s0,lS->S,nD[iM]);
	    Cdx[iCdx].errA[m]	=d;
	    Cdx[iCdx].errR[m]	=a;
	    CbStr2UserMessage("%8s: %8.2e <%8.2e>\n"
			      ,lS->Nm,Cdx[iCdx].errA[m],Cdx[iCdx].errR[m]);
	    
	  }
	}
      }
      m++;
      lS++;
    }
    lS	=lS0[iM];
    B	=iM > 1 ? lS->B : 0.;
  }
  Mk	&=0xff;
  lTm	=lT[iM];
  lDt	=lD[iM];
  iDR	=iP > 4 &&  iM > 1 ? 1 : 0;
  switch(iDR){
  case 0:
    nDR	=6;
    ldX	=X;
    ldY	=Y;
    tS	=lTm;
    yS	=lDt;
    ySS =lS->S;
    CbUserPlot=iExt ? PlotFullRawWF : PlotRawWF;
    break;
  case 1:
    lS	=lS0[iM];
    m	=lS-Sig;
    nDR	=lS->K1;
    j	=K1*sizeof(double);
    memcpy(tR,lS->tk,j);
    if(iP == 5){
      memcpy(r0R,lS->r0,j);
      memcpy(r2R,lS->r2,j);
    }
    j	=nDR-1;
    if(iP == 6) {
      f2spl(r0R,r2R,NULL,NULL,r0R,NULL,lS->tk,&j,tR,tR,tR,Cdx[iCdx].gaR+m);
    }
    if(iSVD == 1){
      MakeSignalF(lTime[0],lT[iM],lS->S,nD[iM],lS->B,r0R,r2R,tR,K1
		  ,lD[1],lD2[1],lT[1],nD[1]);
    }
    else{
      MakeSignal(lT[iM],lS->S,nD[iM],lS->B,r0R,r2R,tR,K1
		 ,lD[1],lD2[1],lT[1],nD[1]);
    }
    MaxDiff(&a,&d,lS->s0,lS->S,nD[iM]);
    CbStr2UserMessage("%8s: %8.2e <%8.2e>\n",lS->Nm,a,d);
    ldY=r0R;
    ldX=tR;
    px	=ldX;
    py	=ldY;
    tS	=tR;
    yS	=r0R;
    ySS =lS->r0;
    nRt =K1;
    nDt	=nDR;
    if(IsRFready){
      GetIndexRF(&iRF,&jRF,m,iCdx);
      CbUserPlot=PlotRFLevel;
      r0R[K1]=1.1*RF[iRF][jRF].G*SensorC[jRF]/SensorC[iGr[2]+iRF];
      py=r0R+K1;
    }
    if(iP == 7){
      Cdx[iCdx].errA[m]	=d;
      Cdx[iCdx].errR[m]	=a;
      j	=K1*sizeof(double);
      memcpy(Cdx[iCdx].r0[m],r0R,j);
      memcpy(Cdx[iCdx].r2[m],r2R,j);
      if(IsRFready) RF[iRF][jRF].rA=r0R[K];
      iP=5;
    }
    break;
  }
  if(iM > 1 && iP > 2){  
    k	=(Mk&0x0f) ? Mk : Mk+2;
    k^=0x20;
    if((k&0x20) && lS->K1) nRt=nD[iM]; 
    PlotSplineSig(3,k,iExt,iM);
    PlotError(4,0,k,iM);
    PlotRespFunc(5,k,iM);
    PlotError(2,1,k,iM);
  }
  lS	=Sig+iSig;
#endif /*Tbl_RespFunc*/

#ifdef H
  MakeAllRespFunctions();
#endif



  for(j=0; j < nCdx; j++){
    for(i=0; i < nSNm; i++) Cdx[j].kS[i]&=0xff;
  }

  FreeMemory4SVD();

#ifndef AscO_RespFunc
#undef AscO_RespFunc
#endif /*AscO_RespFunc*/
#ifndef BnrO_RespFunc
#undef BnrO_RespFunc
#endif /*BnrO_RespFunc*/
  return(0);
}

int FindRF(double *x,double *y)
{
  RESPFUNC *lR;
  double *t,s,d;
  int i,i0,j,j0,m,n;

  s	=1e+20;
  m	=iGr[2];
  for(j=iGr[0],lR=lRF; j < m; j++,lR++){
    if((lR->c&0x100) == 0 && lR->c){
      n	=lR->K;
      i	=n*(*x)/lR->dT;
      if(i > n) i=n;
      d	=fabs(*y-lR->r0[i]);
      if(s > d){
	s	=d; 
	j0	=j;
	i0	=i;
      }
    }
  }
  lR	=lRF+j0;
  if(s < 1e+20){
    *x	=lR->tk[i0];
    *y	=lR->r0[i0];
    lR->x=*x;
    lR->y=*y;
  }
  return(0);
}

int InitCalibr()
{
  int i,j,jj,k,K;
  int nI,nS;

  nI	=iGr[2];
  nS	=nSig-nI;
  for(k=0; k < nSig; k++){
    SensorCdx[k]=Cdx[0].C[k];
    SensorX[k]	=SensorCdx[k];
    SensorC[k]	=SensorCdx[k];
  }
  for(i=0; i < nCdx; i++){
    k	=Cdx[i].kI;
    SensorCdx[k]=Cdx[i].C[k];
    SensorX[k]	=SensorCdx[k];
    SensorC[k]	=SensorCdx[k];
    
    for(j=0,jj=nI; jj < nSig; j++,jj++){
      RF[j][k].Nm=Cdx[i].NmS[jj];
      RF[j][k].iA	=Cdx[i].kS[jj]&0x0f;
      if(RF[j][k].iA){
	if(RF[j][k].c == 0) RF[j][k].c=3;
	K	=jj < iGr[3] ? Cdx[i].K[2] : Cdx[i].K[3];
	RF[j][k].K	=K;
	RF[j][k].err	=Cdx[i].errR[jj];	
	RF[j][k].tk	=Cdx[i].tk[jj];
	RF[j][k].r0	=Cdx[i].r0[jj];
	RF[j][k].r2	=Cdx[i].r2[jj];
	RF[j][k].dT	=Cdx[i].tk[jj][K];
	RF[j][k].G	=gYiC[j][k]*Cdx[i].G[jj]/Cdx[i].G[k];
	RF[j][k].Cx	=Cdx[i].C[jj];
	RF[j][k].rA	=RF[j][k].r0[K];
	RF[j][k].C 	=RF[j][k].rA == 0 ? Cdx[i].C[jj]
	  : RF[j][k].G*Cdx[i].C[k]/RF[j][k].rA;
	RF[j][k].x	=RF[j][k].tk[K/2];
	RF[j][k].y	=RF[j][k].r0[K/2];
	if(RF[j][k].c != 3){
	  SensorCdx[jj]	=Cdx[i].C[jj];
	  SensorX[jj]	=SensorCdx[jj];
	  SensorC[jj]	=SensorCdx[jj];
	}
      }
      else{
	RF[j][k].c	=0;
      }
    }
  }
  for(i=0; i < nS; i++) CalibrSensor(i);
  return(0);
}

int ColorRF2Cdx()
{
  int i,j,jj,k;
  int nI,nS;

  nI	=iGr[2];
  nS	=nSig-nI;
  for(i=0; i < nCdx; i++){
    k	=Cdx[i].kI;
    for(j=0,jj=nI; jj < nS; j++,jj++){
      if(RF[j][k].iA) Cdx[i].kS[jj]=RF[j][k].c;
    }
  }
  return(0);
}

int CalibrAO(FILE *lF)
{
  int i,j,jj,k,m;
  char ln[64],str[64];
  int LI,LM;
  RESPFUNC *lR;
  char *lc;

  LI	=0;
  k	=iGr[2];
  for(i=iGr[0];  i < k; i++) if(LI < (j=strlen(Sig[i].Nm))) LI=j;
  LM	=0;
  for(i=iGr[2];  i < nSig; i++) if(LM < (j=strlen(Sig[i].Nm))) LM=j;

  sprintf(ln,"%s %ds %s%ds %s"
	  ,"%2d %2d %",LI,"%",LM,"%2d %2d %11.4e %11.4e %11.4e %11.4e\n");

  strcpy(str,ln);
  FormatWrite2Read(str);
  fprintf(lF,"<Calibration>[%d](%s)\n",nCdx*nMFP,str);
  sprintf(str,"%s %ds %s%ds %s"
	  ,"%2s %2s %",LI,"%",LM,"%2s %2s %11s %11s %11s %11s\n");
  fprintf(lF,str,"!S","#C","Coil","Signal","A","c","r asymptot"
	  ,"C Cdx","C factor","C factor/C Cdx");
  jj	=iGr[2];
  for(j=0; j < nMFP; j++,jj++){
    for(k=0; k < iGr[2]; k++){
      for(i=0; i < nCdx; i++) if(Cdx[i].kI == k) break;
      if(i < nCdx){
	lc	=Cdx[i].NmS[k];
	lRF[k].c&=0xff;
	if(RF[j][k].iA){
	  fprintf(lF,ln,j,k,lc,Cdx[i].NmS[jj],RF[j][k].iA,RF[j][k].c
		  ,RF[j][k].rA,Cdx[i].C[jj],RF[j][k].C
		  ,RF[j][k].C/Cdx[i].C[jj]);
	}
	else{
	  fprintf(lF,ln,j,k,lc,Cdx[i].NmS[jj],RF[j][k].iA,RF[j][k].c,0.
		  ,Cdx[i].C[jj],0.,0.);
	}
      }
    }
  }	
  fprintf(lF,str,"!S","#C","Coil","Signal","A","c","r asymptot"
	  ,"C Cdx","C factor","C factor/C Cdx");
  fprintf(lF,"\n\n<Another order>\n");
  fprintf(lF,str,"!S","#C","Coil","Signal","A","c","r asymptot"
	  ,"C Cdx","C factor","C/C Cdx");
  for(k=1; k < nPFC; k++){
    for(i=0; i < nCdx; i++) if(Cdx[i].kI == k) break;
    if(i < nCdx){
      lc	=Cdx[i].NmS[k];
      jj	=iGr[2];
      for(j=0; j < nMFP; j++,jj++){
	lRF[k].c&=0xff;
	if(RF[j][k].iA){
	  fprintf(lF,ln,j,k,lc,Cdx[i].NmS[jj],RF[j][k].iA,lRF[k].c,RF[j][k].rA
		  ,Cdx[i].C[jj],RF[j][k].C
		  ,RF[j][k].C/Cdx[i].C[jj]);
	}
	else{
	  fprintf(lF,ln,j,k,lc,Cdx[i].NmS[jj],RF[j][k].iA,lRF[k].c,0.
		  ,Cdx[i].C[jj],0.,0.);
	}
      }
    }
  }
  fprintf(lF,str,"!S","#C","Coil","Signal","A","c","r asymptot"
	  ,"C Cdx","C factor","C/C Cdx");

  fputs("Calibration factors\n",lF);
  fprintf(lF,"%2s %10s %10s %10s %10s\n","#","Coil","C factor"	
	  ,"C Cdx","C/C Cdx");
  for(i=0; i < iGr[2]; i++){
    fprintf(lF,"%2d %10s %10.3e %10.3e %10.3e\n",i,SNm[i],SensorC[i]
	    ,SensorCdx[i],SensorC[i]/SensorCdx[i]);
  }
  fprintf(lF,"\n%2s %10s %10s %10s %10s\n","#","Sensor","C factor","C Cdx"
	  ,"C/C Cdx");
  j	=iGr[2];
  for(i=0; i < nMFP; i++,j++) fprintf(lF,"%2d %10s %10.3e %10.3e %10.3e\n"
				      ,i,SNm[j],SensorC[j]
				      ,SensorCdx[j],SensorC[j]/SensorCdx[j]);
  return(0);
}

int CalibrAI(FILE *lF)
{
  int i,m,n;
  char ln[32]="<Calibration>";
  char NmC[16],NmS[16],str[32];
  char *lv[16];
  int dl[16];
  double r,s,C;
  char *lm;
  int j,k,iA,ic;

  lm	="Calibr.ai";
  if(IsKeyWordFound(lF,lm,ln,str,&n)){
    while(n){
      n--;
      lv[0]	=(char*)&j;
      lv[1]	=(char*)&k;
      lv[2]	=NmC;
      lv[3]	=NmS;
      lv[4]	=(char*)&iA;
      lv[5]	=(char*)&ic;
      lv[6]	=(char*)&r;
      lv[7]	=(char*)&s;
      lv[8]	=(char*)&C;
      dl[0]	=0;
      dl[1]	=0;
      dl[2]	=0;
      dl[3]	=0;
      dl[4]	=0;
      dl[5]	=0;
      dl[6]	=0;
      dl[7]	=0;
      dl[8]	=0;
      ReadAI(lF,ln,lm,str,1,lv,dl,9);
      RF[j][k].rA	=r;
      RF[j][k].iA	=iA;
      RF[j][k].c	=ic;
      RF[j][k].C	=C;
    }
  }
  else{
    CbStr2UserMessage("%s: %s record is absent.\n",lm,ln);
    return(1);
  }
  return(0);
}

void PlotRF()
{
  double x[2],y[2];
  int j,n,K,k[8]={0,0,14,11,4,0,0,0};
  RESPFUNC *lR;
  int kc[NPFC]={0,0,4,4 ,0,0,11,14 ,14,0,0,0 ,0,0,0,0};
  ZColor(0);
  n	=iGr[2];
  j	=(lRF-RF[0])/NPFC+n;
  ZPlotUTopMString(SNm[j],strlen(SNm[j]));
  for(j=0,lR=lRF; j < n; j++,lR++){
    if((lR->c&0x100) == 0 && lR->c && lR->c < 5){
      K		=lR->K;
      x[0]	=lR->tk[0];
      x[1]	=lR->tk[K];
      y[0]	=lR->rA;
      y[1]	=y[0];
      K++;
      ZColor(k[lR->c]);
      ZPlotPolyLine(lR->tk,lR->r0,K);
      ZPlotPolyLine(x,y,2);
      ZPlotLUString(lR->x,lR->y,SNm[j],strlen(SNm[j]));
    }
  }
}

int PlotScRF(int W,int Fl,int iS)
{
  int kPS;
  int j,M,n,k[8]={0,0,14,11,4,0,0,0};
  double x[2],y[4];
  int m[NPFC],c[NPFC];
  double Sc[NPFC],Sc0[NPFC];
  char str[64];
  RESPFUNC *lR;
  int kc[NPFC]={0,0,4,4 ,0,0,11,14 ,14,0,0,0 ,0,0,0,0};
  
  kPS	=Fl < 3 ? Fl : 4;

  n	=iGr[2];
  lR	=lRF;

  for(j=0,lR=lRF; j < n; j++,lR++) if(lR->iA) break;
  y[0]	=0.;
  y[1]	=0.;
  y[2]	=lR->C;
  y[3]	=lR->Cx;

  M	=0;
  while(j < n){
    if((lR->c&0x100) == 0 && lR->c && lR->c < 5){
      c[M]	=kc[j];
      c[M]	=k[lR->c];
      m[M]	=j;
      if(lR->rA != 0.){
	Sc[M]	=lR->G*SensorC[j]/lR->rA;
	Sc[M]	=lR->C;
	Sc0[M]	=lR->G*SensorX[j]/lR->rA;
      }
      else{
	Sc[M]	=lR->C;
	Sc0[M]	=lR->C;
      }
      if(y[0] > Sc[M]) y[0]=Sc[M];
      if(y[1] < Sc[M]) y[1]=Sc[M];
      if(y[0] > Sc0[M]) y[0]=Sc0[M];
      if(y[1] < Sc0[M]) y[1]=Sc0[M];
      M++;
    }
    j++;
    lR++;
  }
  if(M == 0) return(0);
  Sc[M]	=y[2];
  Sc0[M]=SensorX[n+iS];
  if(y[0] > Sc[M]) y[0]=Sc[M];
  if(y[1] < Sc[M]) y[1]=Sc[M];
  if(y[0] > Sc0[M]) y[0]=Sc0[M];
  if(y[1] < Sc0[M]) y[1]=Sc0[M];
  if(y[0] > y[3]) y[0]=y[3];
  if(y[1] < y[3]) y[1]=y[3];

  x[0]	=0.1*(y[1]-y[0]);
  y[0]	-=x[0];
  y[1]	+=x[0];

  x[0]	=0.;
  x[1]	=n;
  sprintf(str,"%s: ScaleFactors",lNmShot);
  SetPlotName("gt",SNm[n+iS],str);
  Scale2d(W,x,y,2,6,2);
  if(kPS){
    PSFileOpen(kPS,kPS);
    PSNewFrame();
  }
  y[0]	=y[3];
  y[1]	=y[3];
  Plot2d(W,x,y,2,6,0,12,0);
  if(kPS) PSPlot2d(x,y,2,6,0,12,0);

  y[0]	=Sc[M];
  y[1]	=Sc[M];
  Plot2d(W,x,y,2,6,0,14,0);
  if(kPS) PSPlot2d(x,y,2,6,0,14,0);

  y[0]	=Sc0[M];
  y[1]	=Sc0[M];
  Plot2d(W,x,y,2,6,0,4,0);
  if(kPS) PSPlot2d(x,y,2,6,0,4,0);

  y[0]	=0.;
  for(j=0; j < M; j++){
    x[0]	=m[j];
    x[1]	=x[0];
    y[1]	=Sc[j];
    Plot2d(W,x,y,2,6,0,c[j],0);
    if(kPS) PSPlot2d(x,y,2,6,0,c[j],0);
    x[0]	-=0.1;
    x[1]	+=0.1;
    y[2]	=y[1];
    Plot2d(W,x,y+1,2,6,0,c[j],0);
    if(kPS) PSPlot2d(x,y+1,2,6,0,c[j],0);
  }
  for(j=0; j < M; j++){
    x[0]	=m[j]-0.3;
    x[1]	=x[0];
    y[1]	=Sc0[j];
    Plot2d(W,x,y,2,6,0,c[j],0);
    if(kPS) PSPlot2d(x,y,2,6,0,c[j],0);
    x[0]	-=0.1;
    x[1]	+=0.1;
    y[2]	=y[1];
    Plot2d(W,x,y+1,2,6,0,c[j],0);
    if(kPS) PSPlot2d(x,y+1,2,6,0,c[j],0);
  }
  if(kPS) PSFileClose();
  return(0);
}

int CalibrSensor(int i)
{
  int j,n;
  double s,ss,w;
  s	=0.;
  ss	=0.;
  n	=iGr[2];
  for(j=0; j < n; j++){
    if(RF[i][j].iA && RF[i][j].c != 3){
      w	=RF[i][j].rA/RF[i][j].err;
      s	+=w*RF[i][j].G*SensorC[j]/RF[i][j].err;
      ss+=w*w;
    }
  }
  if(ss != 0.) s/=ss;
  SensorC[n+i]	=s;
  return(0);
}

int CalibrAllSensors()
{
  int i,j,k,m,n,nI,nS;
  double s,ss,w;
  int iA[nSig];
  double A[nSig*nSig];

  n	=nSig;
  nI	=iGr[2];
  nS	=n-nI;
  memset(A,'\0',n*n*sizeof(double));
  memset(SensorX,'\0',n*sizeof(double));
  m	=0;
  for(i=0; i < nI; i++){
    m	=n*i+i;
    j	=i;
    s	=0.;
    for(k=0; k < nS; k++){
      if(RF[k][j].iA && RF[k][j].c != 3){
	w	=RF[k][j].G/RF[k][j].err;
	s	+=w*w;
      }
    }
    A[m]	=s != 0. ? s : 1.;
    m	=n*i+nI;
    for(k=0; k < nS; k++){
      if(RF[k][j].iA && RF[k][j].c != 3){
	s	=RF[k][j].err;
	A[m]=-RF[k][j].rA*RF[k][j].G/(s*s);
      }
      m++;
    }
  } 
  for(i=0; i < nS; i++){
    m	=n*(nI+i);
    for(j=0; j < nI; j++){
      if(RF[i][j].iA && RF[i][j].c != 3){
	s	=RF[i][j].err;
	A[m]	=RF[i][j].rA*RF[i][j].G/(s*s);
      }
      m++;
    }
    m	+=i;
    s	=0.;
    for(j=0; j < nI; j++){
      if(RF[i][j].iA && RF[i][j].c != 3){
	w	=RF[i][j].rA/RF[i][j].err;
	s	+=w*w;
      }
    }
    A[m]	=s != 0. ? -s : 1.;
  }
  for(j=0; j < nI; j++) if(strcmp(SNm[j],"OH") == 0) break;
  if(j < nI){
    for(k=0; k < nS; k++) if(RF[k][j].iA && RF[k][j].c != 3) break;
    if(k == nS) j = nI;
  }
  if(j == nI){
    for(j=0; j < nI; j++){
      for(k=0; k < nS; k++) if(RF[k][j].iA && RF[k][j].c != 3) break;
      if(k < nS) break;
    }
  }
  m	=n*j;
  memset(A+m,'\0',n*sizeof(double));
  SensorX[j]	=SensorCdx[j];
  A[m+j]=1.;
  LUdcmp(A,n,iA,&s);
  LUbksb(A,n,iA,SensorX);
  return(0);
}

int Calibr()
{
  static int Fl=1;
  int nI,nS,PS=0,Nxt=0;
  int i,j,k,L,M,N,iS;
  char *NmC[NPFC];
  int kc[NPFC];
  char *NmS[NMFP];
  int ks[NMFP];
  int kS[NMFP],kk[NMFP];
  char ImC[16]="";
  char ImS[16]="";
  double px[2],py[2],s;
  RESPFUNC *lR;
  double x[NPFC];

  nI	=iGr[2];
  nS	=nSig-nI;

  InitCalibr();
  if(Fl){
#ifndef AscI_Calibr
#undef AscI_Calibr
#endif /*AscI_Calibr*/
    Fl	=0;
  }
  for(i=0; i < nS; i++){
    kk[i]=0;
    kS[i]	=0;
  }
  j	=nI;
  for(i=0; i < nS; i++){
    for(j=0; j < nI; j++) if(RF[i][j].iA) break;
    if(j < nI) break;
  }
  kS[i]	=1;
  px[0]	=RF[i][j].x;
  py[0]	=RF[i][j].y;

  iS	=i;
  PS	=0;
  Nxt	=0;
#ifndef Tbl_Calibr
  if(Fl) IsRFready=1;
  for(i=0; i < nS; i++){
    if(kS[i] == 1 && kS[i] != kk[i]) iS =i;
    kk[i]	=0;
    kS[i]	=0;
  }
  CalibrAllSensors();
  CalibrSensor(iS);
  i	=iS;
  for(j=0; j < nI; j++){
    if(RF[i][j].iA){
      if(RF[i][j].rA == 0.) RF[i][j].rA=RF[i][j].G*SensorC[j]/SensorC[i+nI];
      RF[i][j].C=RF[i][j].G*SensorC[j]/RF[i][j].rA;
    }
  }

  switch(Fl){
  case 0:
    break;
  case 1:
    i	=iS;
    for(j=0; j < nI; j++){
      if(RF[i][j].iA){
	RF[i][j].C=SensorC[i+nI];
	RF[i][j].rA=RF[i][j].G*SensorC[j]/SensorC[i+nI];
      }
    }
    break;
  case 2:
    for(i=0; i < nS; i++){
      for(j=0; j < nI; j++) if(RF[i][j].iA && RF[i][j].c != 3) break;
      if(j < nI){
	CalibrSensor(i);
	for(j=0; j < nI; j++){
	  if(RF[i][j].iA){
	    RF[i][j].C=SensorC[i+nI];
	    RF[i][j].rA=RF[i][j].G*SensorC[j]/SensorC[i+nI];
	  }
	}
      }
    }
    break;
  case 3:
    for(i=0; i < nS; i++){
      for(j=0; j < nI; j++){
	if(RF[i][j].iA){
	  RF[i][j].C	=SensorX[i+nI];
	  RF[i][j].rA	=RF[i][j].G*SensorC[j]/RF[i][j].C;
	}
      }
    }
    for(j=0; j < nI; j++) SensorC[j]	=SensorX[j];

    for(i=0; i < nS; i++){
      for(j=0; j < nI; j++) if(RF[i][j].iA && RF[i][j].c != 3) break;
      if(j < nI){
	CalibrSensor(i);
	for(j=0; j < nI; j++){
	  if(RF[i][j].iA){
	    RF[i][j].C=SensorC[i+nI];
	    RF[i][j].rA=RF[i][j].G*SensorC[j]/SensorC[i+nI];
	  }
	}
      }
    }
    break;
  }

  switch(Nxt){
  case 0:
    break;
  case 1:
    i	=1;
    while(i){
      if(iS < nS-1){
	iS++;
	for(j=0; j < nI; j++) if(RF[iS][j].iA) break;
	i	=j < nI ? 0 : 1;
      }
      if(i && iS == nS-1) iS=0;
    }
    break;
  case 2:
    i	=1;
    while(i){
      if(iS){
	iS--;
	for(j=0; j < nI; j++) if(RF[iS][j].iA) break;
	i	=j < nI ? 0 : 1;
      }
      if(i && iS == 0) iS=nS;
    }
    break;
  }
  lRF	=RF[iS];
  kk[iS]	=1;
  kS[iS]	=1;

  L	=strlen(ImS);
  N	=0;
  for(i=0,j=nI; i < nS; i++,j++){
    if(*ImS != '!'){
      k	=strncmp(ImS,SNm[j],L);
    }
    else{
      k	=strncmp(ImS+1,SNm[j],L) ? 0 : 1;
    }
    if(k == 0){
      ks[N]	=i;
      NmS[N]	=SNm[j];
      N++;
    }
  }
  FindRF(px,py);
  px[1]	=0.;
  py[1]	=py[0];
  M	=0;
  L	=strlen(ImC);
  while(M == 0){
    for(j=0; j < nI; j++){
      if(lRF[j].iA){
	lRF[j].c	&=0xff;
	if(lRF[j].c == 0) lRF[j].c=1;
	if(*ImC != '!'){
	  i	=strncmp(ImC,SNm[j],L);
	}
	else{
	  i	=strncmp(ImC+1,SNm[j],L) ? 0 : 1;
	}
	if(i == 0){
	  kc[M]	=j;
	  NmC[M]	=SNm[j];
	  x[M]	=lRF[j].dT;
	  M++;
	}
	else{
	  lRF[j].c	|=0x100;
	}
      }
      else{
	lRF[j].c=0;
      }
    }
    if(M == 0){
      CbStr2UserMessage("%s - wrong image",ImC);
      *ImC='\0';
      L=0;
   }
  }
  CbUserPlot	=PlotRF;
  PlotScRF(3,PS,iS);
  Fl	=0;
  Nxt	=0;
  PS	=0;
  ColorRF2Cdx();
#endif /*Tbl_Calibr*/
#ifndef AscO_Calibr
#undef AscO_Calibr
#endif /*AscO_Calibr*/
  if(IsRFready == 0) CbStr2UserMessage("Calibration is not ready yet\n");
  return(0);
}

int BasicU(double *V,double *u0,double *u1,double *u2,double t,double a)
{
  double e;
  double ra;
  ra	=1./a;
  e	=2.*exp(-a*t);
  *V	=2.-e;
  *u0	=2.*t-(*V)*ra;
  *u1	=t*t+(t*e-(*V)*ra)*ra;
  *u2	=2.*cr3*t*t*t+((t+2.*ra)*t*e-2.*(*V)*ra*ra)*ra;
  return(0);
}

int DefSignal(double *S,double a,double b,double Tk,double Ti,double t)
{
  double ea,eb;
  double ra;

  ra	=b/(a-b);
  ea	=exp(-a*t);
  if(t < Tk){
    eb	=exp(-b*t);
    *S	=2.*(1.-eb+ra*(ea-eb));
    return(0);
  }
  if(t < Ti){
    eb	=exp((a-b)*Tk);
    *S	=2.*(1.-exp(-b*Tk)+ra*ea*(1.-eb));
    return(0);
  }
  if(t < Ti+Tk){
    *S=2.*(exp(-b*(t-Ti))-exp(-b*Tk)+ra*ea*(exp((a-b)*(t-Ti))-exp((a-b)*Tk)));
    return(0);
  }
  *S	=0.;
  return(0);
}

int SigTest()
{
  int i,It,Kt;
  int ii,I1,J1,K1;
  static int Fl=1;
  static int I=1000,J=2500,K=10;
  static double y[32];
  static double Y[32],Y0[32],Y2[32];
  static double x[32],xV[32];
  static double ti[1024],tj[4092],tk[256];
  static double V0[1024],V2[1024];
  double s0[256],s2[256],S[4092],S1[4092];
  double r0[256],r2[256];
  static double t2=1.,t0=0.,Ts=0.25,TV=2.,TS=3.;
  double t,dt;
  double B;
  double v;

  if(Fl){
    Fl	=0;
    K1	=K+1;
    dt	=Ts/K;
    for(i=0; i < K1; i++){
      tk[i]	=dt*i;


      x[i]	=tk[i];
      xV[i]	=tk[i];
      y[i]	=1.-exp(-4.*x[i]/Ts);
      Y[i]	=2.-2.*exp(-8.*x[i]/TV);
    }
    dt	=TV/K;
    for(i=0; i < K1; i++) xV[i]=dt*i;
    dt	=TV/J;
    J1	=J+1;
    for(i=0; i < J1; i++) tj[i]=dt*i;
  }
#ifndef Tbl_SigTest
  I1	=I+1;
  J1	=J+1;
  K1	=K+1;
  Kt	=K;
  It	=I;
  dt	=Ts/K;
  for(i=0; i < K1; i++){
    tk[i]=dt*i;
    x[i]=tk[i];
    s0[i]	=y[i];
  }
  dt	=TV/K;
  for(i=0; i < K1; i++) xV[i]=dt*i;

  dt	=4./Ts;
  t	=dt*exp(-4.*tk[K]/Ts);
#ifdef H
  f2spl(s0,s2,NULL,NULL,y,NULL,tk,&K,ga0+2,ga1+2,ga2+2,ga3+2);
#endif
  spl(s0,s2,NULL,NULL,tk,&K);

  f2spl(Y0,Y2,NULL,NULL,Y,NULL,xV,&K,tk,tk,tk,ga3+2);
  dt	=TV/I;
  for(i=0; i < I1; i++){
    ti[i]	=dt*i;
    ii	=(int)(ti[i]*K/TV);
    if(ii == K) ii =K-1;
    splr1(V0+i,NULL,ti+i,Y0,Y2,xV,&K,&ii);
  }
  f2spl(V0,V2,NULL,ti,V0,NULL,ti,&I,tk,tk,tk,ga3);
  dt	=TS/J;
  for(i=0; i < J1; i++) tj[i]=dt*i;

  if(U0 == NULL){
    i	=1024;
    U0	=(double*)malloc(3*i*sizeof(double));
    U1	=U0+i;
    U2	=U1+i;
  }
  
  BasicIntegrals(U0,U1,U2,V0,V2,ti,I1);
  B	=0.2/tj[J];
  MakeSignal(tj,S,J1,B,s0,s2,tk,K1,V0,V2,ti,I1);
  Spline2RespF(0,0.,&B,r0,r2,tk,K1,ga3rf[0],S,tj,J1,V0,V2,ti,I1);
  MakeSignal(tj,S1,J1,B,r0,r2,tk,K1,V0,V2,ti,I1);
  for(i=0; i < J1; i++) S1[i] -=0.02;

  for(i=0; i < K1; i++)out("ridd",i,tk[i],r0[i]);

#endif /*Tbl_SigTest*/
  if(U0 != NULL){
    free(U0);
    U0	=NULL;
  }
  return(0);
}

int main(int argc, char**argv)
{
  int i;
  char Mask[16]	="";
#include <fenv.h>
  feenableexcept(FE_INVALID|FE_OVERFLOW|FE_UNDERFLOW|FE_DIVBYZERO);

  InitConstants();
  TridiagInv(SS2,SS2T,nSS2,0);

  for(i=0; i < NCdxNm; i++) kCdxNm[i]=0;
  memset(Sig,'\0',NSig*sizeof(SIGNAL));
  for(i=0; i < NSig; i++) Sig[i].c=1;

  if(GetSigNm()) return(1);

#ifndef AscI_c2e
#undef AscI_c2e
#endif /*AscI_c2e*/

#ifndef mzl_C2E
#ifndef blb_Year
#ifndef mzl_C2E

#ifndef blb_ShotList
  PFSetShotNm(Mask);
  FileList(Mask);
  FreeSig();

#ifndef mzl_C2E
#ifndef stg_RawData
  RawData();
#endif /*stg_RawData*/

#ifndef stg_RespFunc
  RespFunc();
#endif /*stg_RespFunc*/

#ifndef stg_Calibr
  Calibr();
#endif /*stg_Calibr*/

#ifndef stg_SigTest
  SigTest();
#endif /*stg_SigTest*/

#endif/*mzl_C2E*/
#endif/*blb_ShotList*/
#endif/*mzl_C2E*/
#endif/*blb_Year*/
#endif/*mzl_C2E*/
  return(0);
}

int SplineSVD2RespF(int iB,double *r0,double *r2,double *tk,int K1
		 ,double ga,double *S, double *tj, int J1
		 ,double *V0, double *V2, double *ti,int I1)
{
  int I,J,K;
  int i,ii,i1,j,jj,k,kk;
  double d,dd,dd6;
  double f0,f1,p0,p1,p2,p3;
  double h,rh,rhh;
  double Y;
  double u0,u1,u2;
  double Tk,Ti,Tj;
  double t,t0,t1,t2,t3;
  double *ld,*ls;

  I	=I1-1;
  J	=J1-1;
  K	=K1-1;
  t0	=ti[0];
  Tk	=tk[K];
  h	=Tk/K;
  rh	=1./h;
  rhh	=6./(h*h);
  Ti	=ti[I];
  Tj	=Ti+Tk;

  nSVD	=iB ? K1+1 : K1;
  j	=0;
  while(tj[j] < t0) j++;

  ii	=0;
  i	=ii+1;
  mSVD	=0;
  for(; j < J1 && tj[j] < Tj; j++){
    ld	=aSVD[mSVD];
    memset(ld,'\0',nSVD*LDBL);
    memset(r2,'\0',K1*LDBL);
    t	=tj[j];
    t1	=t-t0;
    /* Basic integral related to the source*/
    if(t < Ti){
      i	=ii+1;
      while(i < I && ti[i] < t) i++;
      ii=i-1;
      d	=ti[i]-ti[ii];
      dd=d*d;
      dd6=cr6*dd;
      f0	=V2[ii]*dd6;
      f1	=V2[i]*dd6;
      p0	=V0[ii];
      p1	=V0[i]-p0-2.*f0-f1;
      p2	=3.*f0;
      p3	=f1-f0;
      Y		=(t-ti[ii])/d;
      ld[0]	=p0+Y*(p1+Y*(p2+Y*p3));
    }
    for(kk=0; kk < K && tk[kk] < t1; kk++){
      k		=kk+1;
      h		=tk[k]-tk[kk];
      rh	=1./h;
      t3	=t2;
      t2	=tk[k] < t1 ? t-tk[k] : t-t1;
      if(t2 < Ti){
	i1	=i;
	while(i1 && ti[i1] >= t3) i1--;
	i	=i1+1;
	while(i && ti[i] > t2){
	  d	=ti[i]-ti[i1];
	  dd	=d*d;
	  dd6	=cr6*dd;
	  f0	=V2[i1]*dd6;
	  f1	=V2[i]*dd6;
	  p0	=V0[i1];
	  p1	=V0[i]-p0-2.*f0-f1;
	  p2	=3.*f0;
	  p3	=f1-f0;
	  if(ti[i] > t3){
 	    f0	=t3-ti[i1];
	    f1	=f0*f0;
	    Y	=f0/d;
	    u0  =f0*(p0+Y*(cr2*p1+Y*(cr3*p2+0.25*Y*p3)));
	    u1	=f1*(cr2*p0+Y*(cr3*p1+Y*(0.25*p2+0.2*Y*p3)));
	    u2	=f0*f1*(cr3*p0+Y*(0.25*p1+Y*(0.2*p2+cr6*Y*p3)));
	  }
	  else{
	    u0  =d*(p0+cr2*p1+cr3*p2+0.25*p3);
	    u1	=dd*(cr2*p0+cr3*p1+0.25*p2+0.2*p3);
	    u2	=d*dd*(cr3*p0+0.25*p1+0.2*p2+cr6*p3);
	  }
	  if(t2 > ti[i1]){
	    f0	=t2-ti[i1];
	    f1	=f0*f0;
	    Y	=f0/d;
	    u0  -=f0*(p0+Y*(cr2*p1+Y*(cr3*p2+0.25*Y*p3)));
	    u1	-=f1*(cr2*p0+Y*(cr3*p1+Y*(0.25*p2+0.2*Y*p3)));
	    u2	-=f0*f1*(cr3*p0+Y*(0.25*p1+Y*(0.2*p2+cr6*Y*p3)));
	  }
	  f0	=t-(ti[i1]+tk[kk]);
	  p0	=rh*u0;
	  p1	=h*u0;
	  p2	=0.5*rh*(f0*(f0*u0-2.*u1)+u2);
	  ld[kk]-=p0;
	  ld[k]	+=p0;
	  r2[kk]+=f0*u0-u1-p2-cr3*p1;
	  r2[k]	+=p2-cr6*p1;
	  i--;
	  i1--;
	}
      }
    }
    ls	=SS2;
    for(k=0; k < K1; k++){
      d	=0.;
      for(kk=0; kk < K1; kk++) 	d+=r2[kk]*(*ls++);
      ld[k]	+=d*rhh;
    }
    if(iB) ld[K1]	=t1;
    mSVD++;
    j++;
  }
  SVDcmp(mSVD,nSVD,aSVD,vSVD,wSVD,EigenSVD);
  EigenMax	=0.;
  for(i=0; i < nSVD; i++){
    EigenSVD[i]	=wSVD[i];
    if(EigenMax < wSVD[i]) EigenMax=wSVD[i];
  }
  if(EigenMax == 0.){
    puts("!!! Wrong matrix SVD EigenMax=0.");
    return(1);
  }
  SVDsortR(nSVD,EigenSVD);
  i	=0;
  j	=nSVD-1;
  while(i < j){
    d	=EigenSVD[i];
    EigenSVD[i]	=EigenSVD[j];
    EigenSVD[j]	=d;
    i++;
    j--;
  }
  d	=1./EigenMax;
  for(i=0; i < nSVD; i++) EigenSVD[i] *=d;
  return(0);
}
#endif

int GetSVDExpansion4Wall(double *I,double *gY,double Emn)
{
  int i,j,n;
  double s,w,h;
  double *ld;

  n	=0;
  EigenMin	=Emn;
  w	=Emn*EigenMax;
  for(j=0; j < nSVD; j++){
    s	=0.;
    if(wSVD[j] > w){
      for(i=0; i < mSVD; i++) s+=aSVD[i][j]*gY[i];
      s	=s/wSVD[j];
      n++;
    }
    r2[j]=s;
  }
  for(j=0; j < nSVD; j++){
    ld	=vSVD[j];
    s	=0.;
    for(i=0; i < nSVD; i++) s+=(*ld++)*r2[i];
    I[j]	=s;
  }
  return(n);
}
#endif

int PlotSVDEigen(int iW,int N,double *E,double Emin,double *Y,int FlP)
{
  int i,k;
  double eL[N],eLmn;
  double x[N],y[2],s,d,d2;
  char ln[N];
  double v;
  
  d2	=Emin*Emin;
  s	=0.;
  v	=0.;
  for(i=0; i < N; i++){
    d	=E[i]*E[i];
    if(d >= d2)  v+=d;
    s	+=d;
  }
  v	*=100./s;
  if(FlP){
    static int inx=1,iny=1;
    static int kPS=1;
    static char PSFileNm[16];
    if(FlP > 1){
      if(FlP != 4){
	inx	=FlP-1;
	iny	=FlP-1;
      }
      else{
	inx	=4;
	iny	=4;
      }
    }
    if(kPS && *ShotNm != '\0'){
      sprintf(PSFileNm,"%s.ps",ShotNm);
      PSSetFileName(PSFileNm);
      kPS	=0;
    }
    PSFileOpen(inx,iny);
  }
  
  sprintf(ln,"Variance=%10.3e",v);
  SetPlotName("k","log10 of Eigenvalues",ln);

#ifdef H
  y[0]	=100;
  y[1]	=-100;
  for(i=0; i < N; i++){
    eL[i]	=E[i] > 1e-10 ? log10(E[i]) : -10.;
    if(y[0] > eL[i]) y[0]=eL[i];
    if(y[1] < eL[i]) y[1]=eL[i];
  }
  eLmn	=log10(Emin)+y[1];
  if(y[0] > eLmn) y[0]=eLmn;

  y[1]	=4.;
  y[0]	=y[1]-10.;
  y[0]	=-6.;
#endif
  eLmn	=log10(Emin);
  x[0]	=0.;
  x[1]	=N+1.;
  for(i=0; i < N; i++){
    eL[i]=log10(E[i]);
    if(eL[i] < Y[0]) eL[i]=Y[0];
  }

  Scale2d(iW,x,Y,2,6,2);
  if(FlP) PSNewFrame();
  i	=0;
  k	=0;
  y[0]	=Y[0]-1.;
  y[1]	=Y[1];
  /*
  while(i < N (&& eL[i] > eLmn)){
  */
  while(i < N){
    x[0]=i+1;
    x[1]=x[0];
    Plot2d(iW,x,y,2,6,0,kCol[k%16],0);
    if(FlP) PSPlot2d(x,y,2,6,0,kCol[k%16],0);
    k++;
    i++;
  }
  for(i=0; i < N; i++) x[i]=i+1;
  Plot2d(iW,x,eL,N,6,0,4,0);
  if(FlP) PSPlot2d(x,eL,N,6,0,4,0);
  y[0]	=eLmn;
  y[1]	=y[0];
  x[0]	=1.;
  x[1]	=N;
  Plot2d(iW,x,y,2,6,0,14,0);
  if(FlP) PSPlot2d(x,y,2,6,0,14,0);
  if(FlP) PSFileClose();
  return(0);
}
